"""
Molecular integrals over Gaussian basis functions generated by sympleints.
See https://github.com/eljost/sympleints for more information.

sympleints version: 0.1.dev79+g63f1ef8.d20230515
symppy version: 1.10.1

sympleints was executed with the following arguments:
	lmax = 4
	lauxmax = 6
	write = False
	out_dir = devel_ints
	keys = ['~2c2e', '~3c2e_sph']
	sph = False
	opt_basic = True
	normalize = cgto
"""

"""

        Quadrupole integrals contain the upper triangular part of the symmetric
        3x3 quadrupole matrix.

        / xx xy xz \\
        |    yy yz |
        \       zz /
        
"""

import numpy


def quadrupole3d_00(ax, da, A, bx, db, B, R):
    """Cartesian 3D (ss) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 1, 1), dtype=float)

    x0 = (ax + bx) ** (-1.0)
    x1 = -x0 * (ax * A[0] + bx * B[0]) + R[0]
    x2 = 0.5 * x0
    x3 = ax * bx * x0
    x4 = (
        5.568327996831708
        * da
        * db
        * x0**1.5
        * numpy.exp(-x3 * ((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2 + (A[2] - B[2]) ** 2))
    )
    x5 = -x0 * (ax * A[1] + bx * B[1]) + R[1]
    x6 = x1 * x4
    x7 = -x0 * (ax * A[2] + bx * B[2]) + R[2]

    # 6 item(s)
    result[0, 0, 0] = numpy.sum(x4 * (x1**2 + x2))
    result[1, 0, 0] = numpy.sum(x5 * x6)
    result[2, 0, 0] = numpy.sum(x6 * x7)
    result[3, 0, 0] = numpy.sum(x4 * (x2 + x5**2))
    result[4, 0, 0] = numpy.sum(x4 * x5 * x7)
    result[5, 0, 0] = numpy.sum(x4 * (x2 + x7**2))
    return result


def quadrupole3d_01(ax, da, A, bx, db, B, R):
    """Cartesian 3D (sp) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 1, 3), dtype=float)

    x0 = (ax + bx) ** (-1.0)
    x1 = x0 * (ax * A[0] + bx * B[0])
    x2 = -x1
    x3 = x2 + R[0]
    x4 = x2 + B[0]
    x5 = x3 * x4
    x6 = 0.5 * x0
    x7 = ax * bx * x0
    x8 = (
        5.568327996831708
        * da
        * db
        * numpy.sqrt(x0)
        * numpy.exp(-x7 * ((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2 + (A[2] - B[2]) ** 2))
    )
    x9 = x6 * x8
    x10 = x0 * (ax * A[1] + bx * B[1])
    x11 = -x10
    x12 = x11 + B[1]
    x13 = x0 * x8
    x14 = x13 * (x3**2 + x6)
    x15 = x0 * (ax * A[2] + bx * B[2])
    x16 = -x15
    x17 = x16 + B[2]
    x18 = x11 + R[1]
    x19 = x13 * (x5 + x6)
    x20 = x12 * x18
    x21 = x20 + x6
    x22 = x13 * x3
    x23 = x16 + R[2]
    x24 = x17 * x23
    x25 = x24 + x6
    x26 = x13 * (x18**2 + x6)
    x27 = x13 * x23
    x28 = x13 * (x23**2 + x6)

    # 18 item(s)
    result[0, 0, 0] = numpy.sum(
        -x9 * (x0 * (-2.0 * x1 + B[0] + R[0]) + x3 * (x0 + 2.0 * x5))
    )
    result[0, 0, 1] = numpy.sum(-x12 * x14)
    result[0, 0, 2] = numpy.sum(-x14 * x17)
    result[1, 0, 0] = numpy.sum(-x18 * x19)
    result[1, 0, 1] = numpy.sum(-x21 * x22)
    result[1, 0, 2] = numpy.sum(-x17 * x18 * x22)
    result[2, 0, 0] = numpy.sum(-x19 * x23)
    result[2, 0, 1] = numpy.sum(-x12 * x22 * x23)
    result[2, 0, 2] = numpy.sum(-x22 * x25)
    result[3, 0, 0] = numpy.sum(-x26 * x4)
    result[3, 0, 1] = numpy.sum(
        -x9 * (x0 * (-2.0 * x10 + B[1] + R[1]) + x18 * (x0 + 2.0 * x20))
    )
    result[3, 0, 2] = numpy.sum(-x17 * x26)
    result[4, 0, 0] = numpy.sum(-x18 * x27 * x4)
    result[4, 0, 1] = numpy.sum(-x21 * x27)
    result[4, 0, 2] = numpy.sum(-x13 * x18 * x25)
    result[5, 0, 0] = numpy.sum(-x28 * x4)
    result[5, 0, 1] = numpy.sum(-x12 * x28)
    result[5, 0, 2] = numpy.sum(
        -x9 * (x0 * (-2.0 * x15 + B[2] + R[2]) + x23 * (x0 + 2.0 * x24))
    )
    return result


def quadrupole3d_02(ax, da, A, bx, db, B, R):
    """Cartesian 3D (sd) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 1, 6), dtype=float)

    x0 = (ax + bx) ** (-1.0)
    x1 = x0 * (ax * A[0] + bx * B[0])
    x2 = -x1
    x3 = x2 + R[0]
    x4 = x3**2
    x5 = 3.0 * x0
    x6 = x2 + B[0]
    x7 = x3 * x6
    x8 = x0 * (-2.0 * x1 + B[0] + R[0])
    x9 = x0 + 2.0 * x7
    x10 = x3 * x9 + x8
    x11 = 1.732050807568877
    x12 = ax * bx * x0
    x13 = (
        5.568327996831708
        * da
        * db
        * numpy.exp(-x12 * ((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2 + (A[2] - B[2]) ** 2))
    )
    x14 = numpy.sqrt(x0) * x13
    x15 = x0 * x14
    x16 = x11 * x15
    x17 = 0.08333333333333333 * x16
    x18 = x0 * (ax * A[1] + bx * B[1])
    x19 = -x18
    x20 = x19 + B[1]
    x21 = 0.5 * x0
    x22 = x14 * x21
    x23 = x10 * x22
    x24 = x0 * (ax * A[2] + bx * B[2])
    x25 = -x24
    x26 = x25 + B[2]
    x27 = 0.3333333333333333 * x20**2 + 0.3333333333333333 * x21
    x28 = x21 + x4
    x29 = x0**1.5 * x13
    x30 = x11 * x29
    x31 = x28 * x30
    x32 = x15 * x26
    x33 = 0.3333333333333333 * x21 + 0.3333333333333333 * x26**2
    x34 = x19 + R[1]
    x35 = 0.1666666666666667 * x16
    x36 = x35 * (x6 * x9 + x8)
    x37 = x20 * x34
    x38 = x21 + x37
    x39 = x21 + x7
    x40 = x29 * x39
    x41 = x0 * (-2.0 * x18 + B[1] + R[1])
    x42 = x0 + 2.0 * x37
    x43 = x20 * x42 + x41
    x44 = x3 * x35
    x45 = x16 * x3
    x46 = x25 + R[2]
    x47 = x15 * x20
    x48 = x26 * x46
    x49 = x21 + x48
    x50 = x0 * (-2.0 * x24 + B[2] + R[2])
    x51 = x0 + 2.0 * x48
    x52 = x26 * x51 + x50
    x53 = x34**2
    x54 = x21 + x53
    x55 = x30 * x54
    x56 = 0.3333333333333333 * x21 + 0.3333333333333333 * x6**2
    x57 = x34 * x42 + x41
    x58 = x22 * x57
    x59 = x15 * x6
    x60 = x46**2
    x61 = x21 + x60
    x62 = x30 * x61
    x63 = x46 * x51 + x50
    x64 = x22 * x63

    # 36 item(s)
    result[0, 0, 0] = numpy.sum(x17 * (x0 * (2.0 * x4 + x5 + 4.0 * x7) + 2.0 * x10 * x6))
    result[0, 0, 1] = numpy.sum(x20 * x23)
    result[0, 0, 2] = numpy.sum(x23 * x26)
    result[0, 0, 3] = numpy.sum(x27 * x31)
    result[0, 0, 4] = numpy.sum(x20 * x28 * x32)
    result[0, 0, 5] = numpy.sum(x31 * x33)
    result[1, 0, 0] = numpy.sum(x34 * x36)
    result[1, 0, 1] = numpy.sum(x38 * x40)
    result[1, 0, 2] = numpy.sum(x32 * x34 * x39)
    result[1, 0, 3] = numpy.sum(x43 * x44)
    result[1, 0, 4] = numpy.sum(x3 * x32 * x38)
    result[1, 0, 5] = numpy.sum(x33 * x34 * x45)
    result[2, 0, 0] = numpy.sum(x36 * x46)
    result[2, 0, 1] = numpy.sum(x39 * x46 * x47)
    result[2, 0, 2] = numpy.sum(x40 * x49)
    result[2, 0, 3] = numpy.sum(x27 * x45 * x46)
    result[2, 0, 4] = numpy.sum(x3 * x47 * x49)
    result[2, 0, 5] = numpy.sum(x44 * x52)
    result[3, 0, 0] = numpy.sum(x55 * x56)
    result[3, 0, 1] = numpy.sum(x58 * x6)
    result[3, 0, 2] = numpy.sum(x32 * x54 * x6)
    result[3, 0, 3] = numpy.sum(
        x17 * (x0 * (4.0 * x37 + x5 + 2.0 * x53) + 2.0 * x20 * x57)
    )
    result[3, 0, 4] = numpy.sum(x26 * x58)
    result[3, 0, 5] = numpy.sum(x33 * x55)
    result[4, 0, 0] = numpy.sum(x16 * x34 * x46 * x56)
    result[4, 0, 1] = numpy.sum(x38 * x46 * x59)
    result[4, 0, 2] = numpy.sum(x34 * x49 * x59)
    result[4, 0, 3] = numpy.sum(x35 * x43 * x46)
    result[4, 0, 4] = numpy.sum(x29 * x38 * x49)
    result[4, 0, 5] = numpy.sum(x34 * x35 * x52)
    result[5, 0, 0] = numpy.sum(x56 * x62)
    result[5, 0, 1] = numpy.sum(x47 * x6 * x61)
    result[5, 0, 2] = numpy.sum(x6 * x64)
    result[5, 0, 3] = numpy.sum(x27 * x62)
    result[5, 0, 4] = numpy.sum(x20 * x64)
    result[5, 0, 5] = numpy.sum(
        x17 * (x0 * (4.0 * x48 + x5 + 2.0 * x60) + 2.0 * x26 * x63)
    )
    return result


def quadrupole3d_03(ax, da, A, bx, db, B, R):
    """Cartesian 3D (sf) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 1, 10), dtype=float)

    x0 = (ax + bx) ** (-1.0)
    x1 = x0 * (ax * A[0] + bx * B[0])
    x2 = -x1
    x3 = x2 + B[0]
    x4 = x2 + R[0]
    x5 = x4**2
    x6 = 3.0 * x0
    x7 = x3 * x4
    x8 = x6 + 4.0 * x7
    x9 = x0 * (-2.0 * x1 + B[0] + R[0])
    x10 = x0 + 2.0 * x7
    x11 = x10 * x4
    x12 = x11 + x9
    x13 = 2.0 * x3
    x14 = x0 * (2.0 * x5 + x8) + x12 * x13
    x15 = x10 * x3
    x16 = 2.0 * x0
    x17 = 3.872983346207417
    x18 = ax * bx * x0
    x19 = (
        5.568327996831708
        * da
        * db
        * numpy.exp(-x18 * ((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2 + (A[2] - B[2]) ** 2))
    )
    x20 = numpy.sqrt(x0) * x19
    x21 = x0 * x20
    x22 = x17 * x21
    x23 = 0.01666666666666667 * x22
    x24 = x0 * (ax * A[1] + bx * B[1])
    x25 = -x24
    x26 = x25 + B[1]
    x27 = 1.732050807568877
    x28 = x26 * x27
    x29 = 0.08333333333333333 * x21
    x30 = x14 * x29
    x31 = x0 * (ax * A[2] + bx * B[2])
    x32 = -x31
    x33 = x32 + B[2]
    x34 = x27 * x33
    x35 = x26**2
    x36 = 0.5 * x0
    x37 = x35 + x36
    x38 = x0**1.5 * x19
    x39 = 0.1666666666666667 * x27
    x40 = x38 * x39
    x41 = x12 * x40
    x42 = x20 * x33 * x36
    x43 = x33**2
    x44 = x36 + x43
    x45 = 1.5 * x0
    x46 = x26 * (x35 + x45)
    x47 = x38 * (x36 + x5)
    x48 = 0.06666666666666667 * x17
    x49 = x47 * x48
    x50 = 0.3333333333333333 * x47
    x51 = x33 * (x43 + x45)
    x52 = x25 + R[1]
    x53 = x3**2
    x54 = x15 + x9
    x55 = x23 * (x0 * (2.0 * x53 + x8) + x13 * x54)
    x56 = x26 * x52
    x57 = x36 + x56
    x58 = x38 * x57
    x59 = x39 * x58
    x60 = x21 * x39
    x61 = x33 * x60
    x62 = x0 * (-2.0 * x24 + B[1] + R[1])
    x63 = x0 + 2.0 * x56
    x64 = x26 * x63
    x65 = x62 + x64
    x66 = x38 * (x36 + x7)
    x67 = x39 * x66
    x68 = 0.3333333333333333 * x27
    x69 = x44 * x68
    x70 = 4.0 * x56 + x6
    x71 = 2.0 * x26
    x72 = x0 * (2.0 * x35 + x70) + x65 * x71
    x73 = x23 * x4
    x74 = 0.06666666666666667 * x22 * x4
    x75 = x32 + R[2]
    x76 = x26 * x60
    x77 = x33 * x75
    x78 = x36 + x77
    x79 = x40 * x78
    x80 = x37 * x68
    x81 = x0 * (-2.0 * x31 + B[2] + R[2])
    x82 = x0 + 2.0 * x77
    x83 = x33 * x82
    x84 = x81 + x83
    x85 = x38 * x78
    x86 = x6 + 4.0 * x77
    x87 = 2.0 * x33
    x88 = x0 * (2.0 * x43 + x86) + x84 * x87
    x89 = x3 * (x45 + x53)
    x90 = x52**2
    x91 = x38 * (x36 + x90)
    x92 = x48 * x91
    x93 = x36 + x53
    x94 = x52 * x63
    x95 = x62 + x94
    x96 = x40 * x95
    x97 = 0.3333333333333333 * x93
    x98 = x0 * (x70 + 2.0 * x90) + x71 * x95
    x99 = x29 * x98
    x100 = x27 * x3
    x101 = x68 * x93
    x102 = x3 * x60
    x103 = x75**2
    x104 = x38 * (x103 + x36)
    x105 = x104 * x48
    x106 = x75 * x82
    x107 = x106 + x81
    x108 = x107 * x40
    x109 = x0 * (2.0 * x103 + x86) + x107 * x87
    x110 = x109 * x29

    # 60 item(s)
    result[0, 0, 0] = numpy.sum(-x23 * (x14 * x3 + x16 * (x11 + x15 + 2.0 * x9)))
    result[0, 0, 1] = numpy.sum(-x28 * x30)
    result[0, 0, 2] = numpy.sum(-x30 * x34)
    result[0, 0, 3] = numpy.sum(-x37 * x41)
    result[0, 0, 4] = numpy.sum(-x12 * x26 * x42)
    result[0, 0, 5] = numpy.sum(-x41 * x44)
    result[0, 0, 6] = numpy.sum(-x46 * x49)
    result[0, 0, 7] = numpy.sum(-x34 * x37 * x50)
    result[0, 0, 8] = numpy.sum(-x28 * x44 * x50)
    result[0, 0, 9] = numpy.sum(-x49 * x51)
    result[1, 0, 0] = numpy.sum(-x52 * x55)
    result[1, 0, 1] = numpy.sum(-x54 * x59)
    result[1, 0, 2] = numpy.sum(-x52 * x54 * x61)
    result[1, 0, 3] = numpy.sum(-x65 * x67)
    result[1, 0, 4] = numpy.sum(-x33 * x57 * x66)
    result[1, 0, 5] = numpy.sum(-x52 * x66 * x69)
    result[1, 0, 6] = numpy.sum(-x72 * x73)
    result[1, 0, 7] = numpy.sum(-x4 * x61 * x65)
    result[1, 0, 8] = numpy.sum(-x4 * x58 * x69)
    result[1, 0, 9] = numpy.sum(-x51 * x52 * x74)
    result[2, 0, 0] = numpy.sum(-x55 * x75)
    result[2, 0, 1] = numpy.sum(-x54 * x75 * x76)
    result[2, 0, 2] = numpy.sum(-x54 * x79)
    result[2, 0, 3] = numpy.sum(-x66 * x75 * x80)
    result[2, 0, 4] = numpy.sum(-x26 * x66 * x78)
    result[2, 0, 5] = numpy.sum(-x67 * x84)
    result[2, 0, 6] = numpy.sum(-x46 * x74 * x75)
    result[2, 0, 7] = numpy.sum(-x4 * x80 * x85)
    result[2, 0, 8] = numpy.sum(-x4 * x76 * x84)
    result[2, 0, 9] = numpy.sum(-x73 * x88)
    result[3, 0, 0] = numpy.sum(-x89 * x92)
    result[3, 0, 1] = numpy.sum(-x93 * x96)
    result[3, 0, 2] = numpy.sum(-x34 * x91 * x97)
    result[3, 0, 3] = numpy.sum(-x100 * x99)
    result[3, 0, 4] = numpy.sum(-x3 * x42 * x95)
    result[3, 0, 5] = numpy.sum(-x3 * x69 * x91)
    result[3, 0, 6] = numpy.sum(-x23 * (x16 * (2.0 * x62 + x64 + x94) + x26 * x98))
    result[3, 0, 7] = numpy.sum(-x34 * x99)
    result[3, 0, 8] = numpy.sum(-x44 * x96)
    result[3, 0, 9] = numpy.sum(-x51 * x92)
    result[4, 0, 0] = numpy.sum(-0.06666666666666667 * x22 * x52 * x75 * x89)
    result[4, 0, 1] = numpy.sum(-x101 * x58 * x75)
    result[4, 0, 2] = numpy.sum(-x101 * x52 * x85)
    result[4, 0, 3] = numpy.sum(-x102 * x65 * x75)
    result[4, 0, 4] = numpy.sum(-x3 * x58 * x78)
    result[4, 0, 5] = numpy.sum(-x102 * x52 * x84)
    result[4, 0, 6] = numpy.sum(-x23 * x72 * x75)
    result[4, 0, 7] = numpy.sum(-x65 * x79)
    result[4, 0, 8] = numpy.sum(-x59 * x84)
    result[4, 0, 9] = numpy.sum(-x23 * x52 * x88)
    result[5, 0, 0] = numpy.sum(-x105 * x89)
    result[5, 0, 1] = numpy.sum(-x104 * x28 * x97)
    result[5, 0, 2] = numpy.sum(-x108 * x93)
    result[5, 0, 3] = numpy.sum(-x104 * x3 * x80)
    result[5, 0, 4] = numpy.sum(-x107 * x20 * x26 * x3 * x36)
    result[5, 0, 5] = numpy.sum(-x100 * x110)
    result[5, 0, 6] = numpy.sum(-x105 * x46)
    result[5, 0, 7] = numpy.sum(-x108 * x37)
    result[5, 0, 8] = numpy.sum(-x110 * x28)
    result[5, 0, 9] = numpy.sum(-x23 * (x109 * x33 + x16 * (x106 + 2.0 * x81 + x83)))
    return result


def quadrupole3d_04(ax, da, A, bx, db, B, R):
    """Cartesian 3D (sg) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 1, 15), dtype=float)

    x0 = 0.5 / (ax + bx)
    x1 = (ax + bx) ** (-1.0)
    x2 = -x1 * (ax * A[0] + bx * B[0])
    x3 = -x2 - B[0]
    x4 = ax * bx * x1
    x5 = numpy.exp(-x4 * (A[0] - B[0]) ** 2)
    x6 = 1.772453850905516 * numpy.sqrt(x1)
    x7 = x5 * x6
    x8 = x3**2 * x7
    x9 = x0 * x7
    x10 = 3.0 * x9
    x11 = -x2 - R[0]
    x12 = x3 * x7
    x13 = x11 * x12
    x14 = x10 + 2.0 * x13
    x15 = x0 * (x14 + x8)
    x16 = x11 * x7
    x17 = x0 * (x12 + x16)
    x18 = x13 + x9
    x19 = x18 * x3
    x20 = x17 + x19
    x21 = x20 * x3
    x22 = x11**2 * x7
    x23 = x0 * (x14 + x22)
    x24 = x11 * x18
    x25 = x17 + x24
    x26 = x25 * x3
    x27 = x23 + x26
    x28 = 2.0 * x0 * (2.0 * x17 + x19 + x24) + x27 * x3
    x29 = da * db
    x30 = 0.09759000729485332 * x29
    x31 = numpy.exp(-x4 * (A[1] - B[1]) ** 2)
    x32 = numpy.exp(-x4 * (A[2] - B[2]) ** 2)
    x33 = 3.141592653589793 * x1 * x32
    x34 = x31 * x33
    x35 = x30 * x34
    x36 = -x1 * (ax * A[1] + bx * B[1])
    x37 = -x36 - B[1]
    x38 = 0.2581988897471611 * x29
    x39 = x37 * x38
    x40 = x28 * x34
    x41 = -x1 * (ax * A[2] + bx * B[2])
    x42 = -x41 - B[2]
    x43 = x38 * x42
    x44 = x32 * x6
    x45 = x31 * x6
    x46 = x37**2 * x45
    x47 = x0 * x45
    x48 = x46 + x47
    x49 = 0.3333333333333333 * x29
    x50 = x48 * x49
    x51 = 1.732050807568877
    x52 = x42 * x49 * x51
    x53 = x42**2 * x44
    x54 = x0 * x44
    x55 = x53 + x54
    x56 = x49 * x55
    x57 = x37 * (2.0 * x47 + x48)
    x58 = x25 * x38
    x59 = x42 * x44
    x60 = x25 * x51
    x61 = x37 * x45
    x62 = x42 * (2.0 * x54 + x55)
    x63 = 3.0 * x47
    x64 = x0 * (3.0 * x46 + x63) + x37 * x57
    x65 = x22 + x9
    x66 = x30 * x65
    x67 = x38 * x65
    x68 = 3.0 * x54
    x69 = x0 * (3.0 * x53 + x68) + x42 * x62
    x70 = -x36 - R[1]
    x71 = x8 + x9
    x72 = 2.0 * x0 * x12 + x3 * x71
    x73 = x15 + x21
    x74 = x35 * (x0 * (3.0 * x17 + 3.0 * x19 + x72) + x3 * x73)
    x75 = x61 * x70
    x76 = x47 + x75
    x77 = x38 * x44
    x78 = x34 * x73
    x79 = x45 * x70
    x80 = x0 * (x61 + x79)
    x81 = x37 * x76
    x82 = x80 + x81
    x83 = x20 * x49
    x84 = x51 * x76
    x85 = x63 + 2.0 * x75
    x86 = x0 * (x46 + x85)
    x87 = x37 * x82
    x88 = x86 + x87
    x89 = x18 * x51
    x90 = x49 * x82
    x91 = x38 * x62
    x92 = x30 * x5
    x93 = x33 * x92
    x94 = x93 * (x0 * (x57 + 3.0 * x80 + 3.0 * x81) + x37 * x88)
    x95 = x33 * x5
    x96 = x88 * x95
    x97 = 3.141592653589793 * x1 * x31
    x98 = x92 * x97
    x99 = -x41 - R[2]
    x100 = x59 * x99
    x101 = x100 + x54
    x102 = x38 * x45
    x103 = x44 * x99
    x104 = x101 * x51
    x105 = x0 * (x103 + x59)
    x106 = x101 * x42
    x107 = x105 + x106
    x108 = x38 * x57
    x109 = x107 * x49
    x110 = 2.0 * x100 + x68
    x111 = x0 * (x110 + x53)
    x112 = x107 * x42
    x113 = x111 + x112
    x114 = x5 * x97
    x115 = x113 * x114
    x116 = x98 * (x0 * (3.0 * x105 + 3.0 * x106 + x62) + x113 * x42)
    x117 = x0 * (x10 + 3.0 * x8) + x3 * x72
    x118 = x45 * x70**2
    x119 = x118 + x47
    x120 = x119 * x30
    x121 = x70 * x76
    x122 = x121 + x80
    x123 = x38 * x72
    x124 = x0 * (x118 + x85)
    x125 = x122 * x37
    x126 = x124 + x125
    x127 = x49 * x71
    x128 = x122 * x51
    x129 = 2.0 * x0 * (x121 + 2.0 * x80 + x81) + x126 * x37
    x130 = x129 * x95
    x131 = x3 * x38
    x132 = x38 * x7
    x133 = x44 * x99**2
    x134 = x133 + x54
    x135 = x134 * x30
    x136 = x101 * x99
    x137 = x105 + x136
    x138 = x137 * x51
    x139 = x0 * (x110 + x133)
    x140 = x137 * x42
    x141 = x139 + x140
    x142 = 2.0 * x0 * (2.0 * x105 + x106 + x136) + x141 * x42
    x143 = x114 * x142

    # 90 item(s)
    result[0, 0, 0] = numpy.sum(
        x35 * (x0 * (2.0 * x15 + 2.0 * x21 + 3.0 * x23 + 3.0 * x26) + x28 * x3)
    )
    result[0, 0, 1] = numpy.sum(x39 * x40)
    result[0, 0, 2] = numpy.sum(x40 * x43)
    result[0, 0, 3] = numpy.sum(x27 * x44 * x50)
    result[0, 0, 4] = numpy.sum(x27 * x34 * x37 * x52)
    result[0, 0, 5] = numpy.sum(x27 * x45 * x56)
    result[0, 0, 6] = numpy.sum(x44 * x57 * x58)
    result[0, 0, 7] = numpy.sum(x50 * x59 * x60)
    result[0, 0, 8] = numpy.sum(x56 * x60 * x61)
    result[0, 0, 9] = numpy.sum(x45 * x58 * x62)
    result[0, 0, 10] = numpy.sum(x44 * x64 * x66)
    result[0, 0, 11] = numpy.sum(x57 * x59 * x67)
    result[0, 0, 12] = numpy.sum(x48 * x56 * x65)
    result[0, 0, 13] = numpy.sum(x61 * x62 * x67)
    result[0, 0, 14] = numpy.sum(x45 * x66 * x69)
    result[1, 0, 0] = numpy.sum(x70 * x74)
    result[1, 0, 1] = numpy.sum(x73 * x76 * x77)
    result[1, 0, 2] = numpy.sum(x43 * x70 * x78)
    result[1, 0, 3] = numpy.sum(x44 * x82 * x83)
    result[1, 0, 4] = numpy.sum(x59 * x83 * x84)
    result[1, 0, 5] = numpy.sum(x20 * x56 * x79)
    result[1, 0, 6] = numpy.sum(x18 * x77 * x88)
    result[1, 0, 7] = numpy.sum(x59 * x89 * x90)
    result[1, 0, 8] = numpy.sum(x56 * x76 * x89)
    result[1, 0, 9] = numpy.sum(x18 * x79 * x91)
    result[1, 0, 10] = numpy.sum(x11 * x94)
    result[1, 0, 11] = numpy.sum(x11 * x43 * x96)
    result[1, 0, 12] = numpy.sum(x16 * x56 * x82)
    result[1, 0, 13] = numpy.sum(x16 * x76 * x91)
    result[1, 0, 14] = numpy.sum(x11 * x69 * x70 * x98)
    result[2, 0, 0] = numpy.sum(x74 * x99)
    result[2, 0, 1] = numpy.sum(x39 * x78 * x99)
    result[2, 0, 2] = numpy.sum(x101 * x102 * x73)
    result[2, 0, 3] = numpy.sum(x103 * x20 * x50)
    result[2, 0, 4] = numpy.sum(x104 * x61 * x83)
    result[2, 0, 5] = numpy.sum(x107 * x45 * x83)
    result[2, 0, 6] = numpy.sum(x103 * x108 * x18)
    result[2, 0, 7] = numpy.sum(x101 * x50 * x89)
    result[2, 0, 8] = numpy.sum(x109 * x61 * x89)
    result[2, 0, 9] = numpy.sum(x102 * x113 * x18)
    result[2, 0, 10] = numpy.sum(x11 * x64 * x93 * x99)
    result[2, 0, 11] = numpy.sum(x101 * x108 * x16)
    result[2, 0, 12] = numpy.sum(x107 * x16 * x50)
    result[2, 0, 13] = numpy.sum(x11 * x115 * x39)
    result[2, 0, 14] = numpy.sum(x11 * x116)
    result[3, 0, 0] = numpy.sum(x117 * x120 * x44)
    result[3, 0, 1] = numpy.sum(x122 * x72 * x77)
    result[3, 0, 2] = numpy.sum(x119 * x123 * x59)
    result[3, 0, 3] = numpy.sum(x126 * x127 * x44)
    result[3, 0, 4] = numpy.sum(x127 * x128 * x59)
    result[3, 0, 5] = numpy.sum(x119 * x56 * x71)
    result[3, 0, 6] = numpy.sum(x130 * x131)
    result[3, 0, 7] = numpy.sum(x126 * x3 * x52 * x95)
    result[3, 0, 8] = numpy.sum(x12 * x128 * x56)
    result[3, 0, 9] = numpy.sum(x119 * x12 * x91)
    result[3, 0, 10] = numpy.sum(
        x93 * (x0 * (3.0 * x124 + 3.0 * x125 + 2.0 * x86 + 2.0 * x87) + x129 * x37)
    )
    result[3, 0, 11] = numpy.sum(x130 * x43)
    result[3, 0, 12] = numpy.sum(x126 * x56 * x7)
    result[3, 0, 13] = numpy.sum(x122 * x132 * x62)
    result[3, 0, 14] = numpy.sum(x120 * x69 * x7)
    result[4, 0, 0] = numpy.sum(x117 * x35 * x70 * x99)
    result[4, 0, 1] = numpy.sum(x103 * x123 * x76)
    result[4, 0, 2] = numpy.sum(x101 * x123 * x79)
    result[4, 0, 3] = numpy.sum(x103 * x127 * x82)
    result[4, 0, 4] = numpy.sum(x101 * x127 * x84)
    result[4, 0, 5] = numpy.sum(x107 * x127 * x79)
    result[4, 0, 6] = numpy.sum(x131 * x96 * x99)
    result[4, 0, 7] = numpy.sum(x104 * x12 * x90)
    result[4, 0, 8] = numpy.sum(x109 * x12 * x84)
    result[4, 0, 9] = numpy.sum(x115 * x131 * x70)
    result[4, 0, 10] = numpy.sum(x94 * x99)
    result[4, 0, 11] = numpy.sum(x101 * x132 * x88)
    result[4, 0, 12] = numpy.sum(x107 * x7 * x90)
    result[4, 0, 13] = numpy.sum(x113 * x132 * x76)
    result[4, 0, 14] = numpy.sum(x116 * x70)
    result[5, 0, 0] = numpy.sum(x117 * x135 * x45)
    result[5, 0, 1] = numpy.sum(x123 * x134 * x61)
    result[5, 0, 2] = numpy.sum(x102 * x137 * x72)
    result[5, 0, 3] = numpy.sum(x134 * x50 * x71)
    result[5, 0, 4] = numpy.sum(x127 * x138 * x61)
    result[5, 0, 5] = numpy.sum(x127 * x141 * x45)
    result[5, 0, 6] = numpy.sum(x108 * x12 * x134)
    result[5, 0, 7] = numpy.sum(x12 * x138 * x50)
    result[5, 0, 8] = numpy.sum(x114 * x141 * x3 * x37 * x49 * x51)
    result[5, 0, 9] = numpy.sum(x131 * x143)
    result[5, 0, 10] = numpy.sum(x135 * x64 * x7)
    result[5, 0, 11] = numpy.sum(x132 * x137 * x57)
    result[5, 0, 12] = numpy.sum(x141 * x50 * x7)
    result[5, 0, 13] = numpy.sum(x143 * x39)
    result[5, 0, 14] = numpy.sum(
        x98 * (x0 * (2.0 * x111 + 2.0 * x112 + 3.0 * x139 + 3.0 * x140) + x142 * x42)
    )
    return result


def quadrupole3d_10(ax, da, A, bx, db, B, R):
    """Cartesian 3D (ps) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 3, 1), dtype=float)

    x0 = (ax + bx) ** (-1.0)
    x1 = x0 * (ax * A[0] + bx * B[0])
    x2 = -x1
    x3 = x2 + R[0]
    x4 = x2 + A[0]
    x5 = x3 * x4
    x6 = 0.5 * x0
    x7 = ax * bx * x0
    x8 = (
        5.568327996831708
        * da
        * db
        * numpy.sqrt(x0)
        * numpy.exp(-x7 * ((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2 + (A[2] - B[2]) ** 2))
    )
    x9 = x6 * x8
    x10 = x0 * (ax * A[1] + bx * B[1])
    x11 = -x10
    x12 = x11 + A[1]
    x13 = x0 * x8
    x14 = x13 * (x3**2 + x6)
    x15 = x0 * (ax * A[2] + bx * B[2])
    x16 = -x15
    x17 = x16 + A[2]
    x18 = x11 + R[1]
    x19 = x13 * (x5 + x6)
    x20 = x12 * x18
    x21 = x20 + x6
    x22 = x13 * x3
    x23 = x16 + R[2]
    x24 = x17 * x23
    x25 = x24 + x6
    x26 = x13 * (x18**2 + x6)
    x27 = x13 * x23
    x28 = x13 * (x23**2 + x6)

    # 18 item(s)
    result[0, 0, 0] = numpy.sum(
        -x9 * (x0 * (-2.0 * x1 + A[0] + R[0]) + x3 * (x0 + 2.0 * x5))
    )
    result[0, 1, 0] = numpy.sum(-x12 * x14)
    result[0, 2, 0] = numpy.sum(-x14 * x17)
    result[1, 0, 0] = numpy.sum(-x18 * x19)
    result[1, 1, 0] = numpy.sum(-x21 * x22)
    result[1, 2, 0] = numpy.sum(-x17 * x18 * x22)
    result[2, 0, 0] = numpy.sum(-x19 * x23)
    result[2, 1, 0] = numpy.sum(-x12 * x22 * x23)
    result[2, 2, 0] = numpy.sum(-x22 * x25)
    result[3, 0, 0] = numpy.sum(-x26 * x4)
    result[3, 1, 0] = numpy.sum(
        -x9 * (x0 * (-2.0 * x10 + A[1] + R[1]) + x18 * (x0 + 2.0 * x20))
    )
    result[3, 2, 0] = numpy.sum(-x17 * x26)
    result[4, 0, 0] = numpy.sum(-x18 * x27 * x4)
    result[4, 1, 0] = numpy.sum(-x21 * x27)
    result[4, 2, 0] = numpy.sum(-x13 * x18 * x25)
    result[5, 0, 0] = numpy.sum(-x28 * x4)
    result[5, 1, 0] = numpy.sum(-x12 * x28)
    result[5, 2, 0] = numpy.sum(
        -x9 * (x0 * (-2.0 * x15 + A[2] + R[2]) + x23 * (x0 + 2.0 * x24))
    )
    return result


def quadrupole3d_11(ax, da, A, bx, db, B, R):
    """Cartesian 3D (pp) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 3, 3), dtype=float)

    x0 = (ax + bx) ** (-1.0)
    x1 = 3.0 * x0
    x2 = x0 * (ax * A[0] + bx * B[0])
    x3 = -x2
    x4 = x3 + A[0]
    x5 = x3 + B[0]
    x6 = x4 * x5
    x7 = x3 + R[0]
    x8 = x4 * x7
    x9 = 2.0 * x8
    x10 = x5 * x7
    x11 = 2.0 * x10
    x12 = -2.0 * x2 + R[0]
    x13 = x0 * (x12 + B[0])
    x14 = x0 + x11
    x15 = x13 + x14 * x4
    x16 = ax * bx * x0
    x17 = (
        5.568327996831708
        * da
        * db
        * numpy.exp(-x16 * ((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2 + (A[2] - B[2]) ** 2))
    )
    x18 = numpy.sqrt(x0) * x17
    x19 = x0 * x18
    x20 = 0.25 * x19
    x21 = x0 * (ax * A[1] + bx * B[1])
    x22 = -x21
    x23 = x22 + B[1]
    x24 = 0.5 * x0
    x25 = x18 * x24
    x26 = x25 * (x0 * (x12 + A[0]) + x7 * (x0 + x9))
    x27 = x0 * (ax * A[2] + bx * B[2])
    x28 = -x27
    x29 = x28 + B[2]
    x30 = x22 + A[1]
    x31 = x25 * (x13 + x14 * x7)
    x32 = x23 * x30
    x33 = x24 + x32
    x34 = x24 + x7**2
    x35 = x0**1.5 * x17
    x36 = x34 * x35
    x37 = x19 * x34
    x38 = x28 + A[2]
    x39 = x29 * x38
    x40 = x24 + x39
    x41 = x22 + R[1]
    x42 = x15 * x25
    x43 = x23 * x41
    x44 = x24 + x43
    x45 = x24 + x8
    x46 = x35 * x45
    x47 = x19 * x29
    x48 = x30 * x41
    x49 = x24 + x48
    x50 = x10 + x24
    x51 = x35 * x50
    x52 = -2.0 * x21 + R[1]
    x53 = x0 * (x52 + B[1])
    x54 = 2.0 * x43
    x55 = x0 + x54
    x56 = x30 * x55 + x53
    x57 = x25 * x7
    x58 = x19 * x38
    x59 = x19 * x7
    x60 = x28 + R[2]
    x61 = x19 * x60
    x62 = x29 * x60
    x63 = x24 + x62
    x64 = x38 * x60
    x65 = x24 + x64
    x66 = -2.0 * x27 + R[2]
    x67 = x0 * (x66 + B[2])
    x68 = 2.0 * x62
    x69 = x0 + x68
    x70 = x38 * x69 + x67
    x71 = x24 + x6
    x72 = x24 + x41**2
    x73 = x35 * x72
    x74 = x25 * (x41 * x55 + x53)
    x75 = 2.0 * x48
    x76 = x25 * (x0 * (x52 + A[1]) + x41 * (x0 + x75))
    x77 = x19 * x41
    x78 = x24 + x60**2
    x79 = x35 * x78
    x80 = x19 * x78
    x81 = x25 * (x60 * x69 + x67)
    x82 = 2.0 * x64
    x83 = x25 * (x0 * (x66 + A[2]) + x60 * (x0 + x82))

    # 54 item(s)
    result[0, 0, 0] = numpy.sum(x20 * (x0 * (x1 + x11 + 2.0 * x6 + x9) + 2.0 * x15 * x7))
    result[0, 0, 1] = numpy.sum(x23 * x26)
    result[0, 0, 2] = numpy.sum(x26 * x29)
    result[0, 1, 0] = numpy.sum(x30 * x31)
    result[0, 1, 1] = numpy.sum(x33 * x36)
    result[0, 1, 2] = numpy.sum(x29 * x30 * x37)
    result[0, 2, 0] = numpy.sum(x31 * x38)
    result[0, 2, 1] = numpy.sum(x23 * x37 * x38)
    result[0, 2, 2] = numpy.sum(x36 * x40)
    result[1, 0, 0] = numpy.sum(x41 * x42)
    result[1, 0, 1] = numpy.sum(x44 * x46)
    result[1, 0, 2] = numpy.sum(x41 * x45 * x47)
    result[1, 1, 0] = numpy.sum(x49 * x51)
    result[1, 1, 1] = numpy.sum(x56 * x57)
    result[1, 1, 2] = numpy.sum(x47 * x49 * x7)
    result[1, 2, 0] = numpy.sum(x41 * x50 * x58)
    result[1, 2, 1] = numpy.sum(x44 * x58 * x7)
    result[1, 2, 2] = numpy.sum(x40 * x41 * x59)
    result[2, 0, 0] = numpy.sum(x42 * x60)
    result[2, 0, 1] = numpy.sum(x23 * x45 * x61)
    result[2, 0, 2] = numpy.sum(x46 * x63)
    result[2, 1, 0] = numpy.sum(x30 * x50 * x61)
    result[2, 1, 1] = numpy.sum(x33 * x59 * x60)
    result[2, 1, 2] = numpy.sum(x30 * x59 * x63)
    result[2, 2, 0] = numpy.sum(x51 * x65)
    result[2, 2, 1] = numpy.sum(x23 * x59 * x65)
    result[2, 2, 2] = numpy.sum(x57 * x70)
    result[3, 0, 0] = numpy.sum(x71 * x73)
    result[3, 0, 1] = numpy.sum(x4 * x74)
    result[3, 0, 2] = numpy.sum(x4 * x47 * x72)
    result[3, 1, 0] = numpy.sum(x5 * x76)
    result[3, 1, 1] = numpy.sum(
        x20 * (x0 * (x1 + 2.0 * x32 + x54 + x75) + 2.0 * x41 * x56)
    )
    result[3, 1, 2] = numpy.sum(x29 * x76)
    result[3, 2, 0] = numpy.sum(x5 * x58 * x72)
    result[3, 2, 1] = numpy.sum(x38 * x74)
    result[3, 2, 2] = numpy.sum(x40 * x73)
    result[4, 0, 0] = numpy.sum(x41 * x61 * x71)
    result[4, 0, 1] = numpy.sum(x4 * x44 * x61)
    result[4, 0, 2] = numpy.sum(x4 * x63 * x77)
    result[4, 1, 0] = numpy.sum(x49 * x5 * x61)
    result[4, 1, 1] = numpy.sum(x25 * x56 * x60)
    result[4, 1, 2] = numpy.sum(x35 * x49 * x63)
    result[4, 2, 0] = numpy.sum(x5 * x65 * x77)
    result[4, 2, 1] = numpy.sum(x35 * x44 * x65)
    result[4, 2, 2] = numpy.sum(x25 * x41 * x70)
    result[5, 0, 0] = numpy.sum(x71 * x79)
    result[5, 0, 1] = numpy.sum(x23 * x4 * x80)
    result[5, 0, 2] = numpy.sum(x4 * x81)
    result[5, 1, 0] = numpy.sum(x30 * x5 * x80)
    result[5, 1, 1] = numpy.sum(x33 * x79)
    result[5, 1, 2] = numpy.sum(x30 * x81)
    result[5, 2, 0] = numpy.sum(x5 * x83)
    result[5, 2, 1] = numpy.sum(x23 * x83)
    result[5, 2, 2] = numpy.sum(
        x20 * (x0 * (x1 + 2.0 * x39 + x68 + x82) + 2.0 * x60 * x70)
    )
    return result


def quadrupole3d_12(ax, da, A, bx, db, B, R):
    """Cartesian 3D (pd) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 3, 6), dtype=float)

    x0 = (ax + bx) ** (-1.0)
    x1 = x0 * (ax * A[0] + bx * B[0])
    x2 = -x1
    x3 = x2 + R[0]
    x4 = x2 + B[0]
    x5 = x3 * x4
    x6 = 2.0 * x5
    x7 = x0 + x6
    x8 = x3 * x7
    x9 = x2 + A[0]
    x10 = x7 * x9
    x11 = -2.0 * x1
    x12 = x11 + R[0]
    x13 = x12 + B[0]
    x14 = 3.0 * x0
    x15 = x3 * x9
    x16 = 2.0 * x15
    x17 = x0 * (x12 + A[0]) + x3 * (x0 + x16)
    x18 = x4 * x9
    x19 = 2.0 * x18
    x20 = x0 * (x14 + x16 + x19 + x6)
    x21 = x0 * x13
    x22 = x10 + x21
    x23 = 2.0 * x22
    x24 = x20 + x23 * x3
    x25 = 1.732050807568877
    x26 = ax * bx * x0
    x27 = (
        5.568327996831708
        * da
        * db
        * numpy.exp(-x26 * ((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2 + (A[2] - B[2]) ** 2))
    )
    x28 = numpy.sqrt(x0) * x27
    x29 = x0 * x28
    x30 = x25 * x29
    x31 = 0.08333333333333333 * x30
    x32 = x0 * (ax * A[1] + bx * B[1])
    x33 = -x32
    x34 = x33 + B[1]
    x35 = 0.25 * x29
    x36 = x24 * x35
    x37 = x0 * (ax * A[2] + bx * B[2])
    x38 = -x37
    x39 = x38 + B[2]
    x40 = 0.5 * x0
    x41 = x34**2 + x40
    x42 = x0**1.5 * x27
    x43 = x25 * x42
    x44 = 0.1666666666666667 * x43
    x45 = x17 * x44
    x46 = x28 * x40
    x47 = x39 * x46
    x48 = x39**2 + x40
    x49 = x33 + A[1]
    x50 = x3**2
    x51 = x21 + x8
    x52 = x31 * (x0 * (x14 + 4.0 * x5 + 2.0 * x50) + 2.0 * x4 * x51)
    x53 = x34 * x49
    x54 = x42 * (x40 + x53)
    x55 = 0.5 * x51
    x56 = -2.0 * x32
    x57 = x56 + B[1]
    x58 = 2.0 * x53
    x59 = x0 * (x57 + A[1]) + x34 * (x0 + x58)
    x60 = x40 + x50
    x61 = x44 * x60
    x62 = 0.3333333333333333 * x43
    x63 = x60 * x62
    x64 = x38 + A[2]
    x65 = x34 * x46
    x66 = x39 * x64
    x67 = x42 * (x40 + x66)
    x68 = -2.0 * x37
    x69 = x68 + B[2]
    x70 = 2.0 * x66
    x71 = x0 * (x69 + A[2]) + x39 * (x0 + x70)
    x72 = x33 + R[1]
    x73 = x31 * (x20 + x23 * x4)
    x74 = x34 * x72
    x75 = x40 + x74
    x76 = x42 * x75
    x77 = 0.5 * x22
    x78 = x57 + R[1]
    x79 = x0 * x78
    x80 = 2.0 * x74
    x81 = x0 + x80
    x82 = x34 * x81 + x79
    x83 = x15 + x40
    x84 = x44 * x83
    x85 = x48 * x62
    x86 = x49 * x72
    x87 = x40 + x86
    x88 = x21 + x4 * x7
    x89 = x44 * x88
    x90 = x49 * x81
    x91 = x79 + x90
    x92 = x40 + x5
    x93 = x42 * x92
    x94 = 0.5 * x93
    x95 = 2.0 * x86
    x96 = x0 * (x14 + x58 + x80 + x95)
    x97 = 2.0 * x34
    x98 = x91 * x97 + x96
    x99 = x3 * x31
    x100 = 0.1666666666666667 * x30
    x101 = x100 * x64
    x102 = x100 * x3
    x103 = x38 + R[2]
    x104 = x103 * x39
    x105 = x104 + x40
    x106 = x105 * x42
    x107 = x41 * x62
    x108 = x69 + R[2]
    x109 = x0 * x108
    x110 = 2.0 * x104
    x111 = x0 + x110
    x112 = x109 + x111 * x39
    x113 = x100 * x103
    x114 = x103 * x64
    x115 = x114 + x40
    x116 = x111 * x64
    x117 = x109 + x116
    x118 = 2.0 * x114
    x119 = x0 * (x110 + x118 + x14 + x70)
    x120 = 2.0 * x39
    x121 = x117 * x120 + x119
    x122 = x0 * (x11 + A[0] + B[0]) + x4 * (x0 + x19)
    x123 = x72**2
    x124 = x123 + x40
    x125 = x124 * x44
    x126 = x18 + x40
    x127 = x126 * x42
    x128 = x72 * x81
    x129 = x128 + x79
    x130 = 0.5 * x129
    x131 = x31 * (x0 * (2.0 * x123 + x14 + 4.0 * x74) + x129 * x97)
    x132 = x4**2 + x40
    x133 = x0 * (x56 + A[1] + R[1]) + x72 * (x0 + x95)
    x134 = x133 * x44
    x135 = 2.0 * x72 * x91 + x96
    x136 = x135 * x35
    x137 = x132 * x62
    x138 = x4 * x46
    x139 = x103**2
    x140 = x139 + x40
    x141 = x140 * x44
    x142 = x103 * x111
    x143 = x109 + x142
    x144 = 0.5 * x143
    x145 = x31 * (x0 * (4.0 * x104 + 2.0 * x139 + x14) + x120 * x143)
    x146 = x0 * (x68 + A[2] + R[2]) + x103 * (x0 + x118)
    x147 = x146 * x44
    x148 = 2.0 * x103 * x117 + x119
    x149 = x148 * x35

    # 108 item(s)
    result[0, 0, 0] = numpy.sum(
        -x31 * (x0 * (2.0 * x10 + x13 * x14 + x17 + x8) + x24 * x4)
    )
    result[0, 0, 1] = numpy.sum(-x34 * x36)
    result[0, 0, 2] = numpy.sum(-x36 * x39)
    result[0, 0, 3] = numpy.sum(-x41 * x45)
    result[0, 0, 4] = numpy.sum(-x17 * x34 * x47)
    result[0, 0, 5] = numpy.sum(-x45 * x48)
    result[0, 1, 0] = numpy.sum(-x49 * x52)
    result[0, 1, 1] = numpy.sum(-x54 * x55)
    result[0, 1, 2] = numpy.sum(-x47 * x49 * x51)
    result[0, 1, 3] = numpy.sum(-x59 * x61)
    result[0, 1, 4] = numpy.sum(-x39 * x54 * x60)
    result[0, 1, 5] = numpy.sum(-x48 * x49 * x63)
    result[0, 2, 0] = numpy.sum(-x52 * x64)
    result[0, 2, 1] = numpy.sum(-x51 * x64 * x65)
    result[0, 2, 2] = numpy.sum(-x55 * x67)
    result[0, 2, 3] = numpy.sum(-x41 * x63 * x64)
    result[0, 2, 4] = numpy.sum(-x34 * x60 * x67)
    result[0, 2, 5] = numpy.sum(-x61 * x71)
    result[1, 0, 0] = numpy.sum(-x72 * x73)
    result[1, 0, 1] = numpy.sum(-x76 * x77)
    result[1, 0, 2] = numpy.sum(-x22 * x47 * x72)
    result[1, 0, 3] = numpy.sum(-x82 * x84)
    result[1, 0, 4] = numpy.sum(-x39 * x76 * x83)
    result[1, 0, 5] = numpy.sum(-x72 * x83 * x85)
    result[1, 1, 0] = numpy.sum(-x87 * x89)
    result[1, 1, 1] = numpy.sum(-x91 * x94)
    result[1, 1, 2] = numpy.sum(-x39 * x87 * x93)
    result[1, 1, 3] = numpy.sum(-x98 * x99)
    result[1, 1, 4] = numpy.sum(-x3 * x47 * x91)
    result[1, 1, 5] = numpy.sum(-x3 * x85 * x87)
    result[1, 2, 0] = numpy.sum(-x101 * x72 * x88)
    result[1, 2, 1] = numpy.sum(-x64 * x76 * x92)
    result[1, 2, 2] = numpy.sum(-x67 * x72 * x92)
    result[1, 2, 3] = numpy.sum(-x101 * x3 * x82)
    result[1, 2, 4] = numpy.sum(-x3 * x67 * x75)
    result[1, 2, 5] = numpy.sum(-x102 * x71 * x72)
    result[2, 0, 0] = numpy.sum(-x103 * x73)
    result[2, 0, 1] = numpy.sum(-x103 * x22 * x65)
    result[2, 0, 2] = numpy.sum(-x106 * x77)
    result[2, 0, 3] = numpy.sum(-x103 * x107 * x83)
    result[2, 0, 4] = numpy.sum(-x106 * x34 * x83)
    result[2, 0, 5] = numpy.sum(-x112 * x84)
    result[2, 1, 0] = numpy.sum(-x113 * x49 * x88)
    result[2, 1, 1] = numpy.sum(-x103 * x54 * x92)
    result[2, 1, 2] = numpy.sum(-x105 * x49 * x93)
    result[2, 1, 3] = numpy.sum(-x102 * x103 * x59)
    result[2, 1, 4] = numpy.sum(-x105 * x3 * x54)
    result[2, 1, 5] = numpy.sum(-x102 * x112 * x49)
    result[2, 2, 0] = numpy.sum(-x115 * x89)
    result[2, 2, 1] = numpy.sum(-x115 * x34 * x93)
    result[2, 2, 2] = numpy.sum(-x117 * x94)
    result[2, 2, 3] = numpy.sum(-x107 * x115 * x3)
    result[2, 2, 4] = numpy.sum(-x117 * x3 * x65)
    result[2, 2, 5] = numpy.sum(-x121 * x99)
    result[3, 0, 0] = numpy.sum(-x122 * x125)
    result[3, 0, 1] = numpy.sum(-x127 * x130)
    result[3, 0, 2] = numpy.sum(-x124 * x127 * x39)
    result[3, 0, 3] = numpy.sum(-x131 * x9)
    result[3, 0, 4] = numpy.sum(-x129 * x47 * x9)
    result[3, 0, 5] = numpy.sum(-x124 * x85 * x9)
    result[3, 1, 0] = numpy.sum(-x132 * x134)
    result[3, 1, 1] = numpy.sum(-x136 * x4)
    result[3, 1, 2] = numpy.sum(-x133 * x4 * x47)
    result[3, 1, 3] = numpy.sum(
        -x31 * (x0 * (x128 + x133 + x14 * x78 + 2.0 * x90) + x135 * x34)
    )
    result[3, 1, 4] = numpy.sum(-x136 * x39)
    result[3, 1, 5] = numpy.sum(-x134 * x48)
    result[3, 2, 0] = numpy.sum(-x124 * x137 * x64)
    result[3, 2, 1] = numpy.sum(-x129 * x138 * x64)
    result[3, 2, 2] = numpy.sum(-x124 * x4 * x67)
    result[3, 2, 3] = numpy.sum(-x131 * x64)
    result[3, 2, 4] = numpy.sum(-x130 * x67)
    result[3, 2, 5] = numpy.sum(-x125 * x71)
    result[4, 0, 0] = numpy.sum(-x113 * x122 * x72)
    result[4, 0, 1] = numpy.sum(-x103 * x126 * x76)
    result[4, 0, 2] = numpy.sum(-x106 * x126 * x72)
    result[4, 0, 3] = numpy.sum(-x113 * x82 * x9)
    result[4, 0, 4] = numpy.sum(-x105 * x76 * x9)
    result[4, 0, 5] = numpy.sum(-x100 * x112 * x72 * x9)
    result[4, 1, 0] = numpy.sum(-x103 * x137 * x87)
    result[4, 1, 1] = numpy.sum(-x103 * x138 * x91)
    result[4, 1, 2] = numpy.sum(-x106 * x4 * x87)
    result[4, 1, 3] = numpy.sum(-x103 * x31 * x98)
    result[4, 1, 4] = numpy.sum(-0.5 * x106 * x91)
    result[4, 1, 5] = numpy.sum(-x112 * x44 * x87)
    result[4, 2, 0] = numpy.sum(-x115 * x137 * x72)
    result[4, 2, 1] = numpy.sum(-x115 * x4 * x76)
    result[4, 2, 2] = numpy.sum(-x117 * x138 * x72)
    result[4, 2, 3] = numpy.sum(-x115 * x44 * x82)
    result[4, 2, 4] = numpy.sum(-0.5 * x117 * x76)
    result[4, 2, 5] = numpy.sum(-x121 * x31 * x72)
    result[5, 0, 0] = numpy.sum(-x122 * x141)
    result[5, 0, 1] = numpy.sum(-x127 * x140 * x34)
    result[5, 0, 2] = numpy.sum(-x127 * x144)
    result[5, 0, 3] = numpy.sum(-x107 * x140 * x9)
    result[5, 0, 4] = numpy.sum(-x143 * x65 * x9)
    result[5, 0, 5] = numpy.sum(-x145 * x9)
    result[5, 1, 0] = numpy.sum(-x137 * x140 * x49)
    result[5, 1, 1] = numpy.sum(-x140 * x4 * x54)
    result[5, 1, 2] = numpy.sum(-x138 * x143 * x49)
    result[5, 1, 3] = numpy.sum(-x141 * x59)
    result[5, 1, 4] = numpy.sum(-x144 * x54)
    result[5, 1, 5] = numpy.sum(-x145 * x49)
    result[5, 2, 0] = numpy.sum(-x132 * x147)
    result[5, 2, 1] = numpy.sum(-x146 * x4 * x65)
    result[5, 2, 2] = numpy.sum(-x149 * x4)
    result[5, 2, 3] = numpy.sum(-x147 * x41)
    result[5, 2, 4] = numpy.sum(-x149 * x34)
    result[5, 2, 5] = numpy.sum(
        -x31 * (x0 * (x108 * x14 + 2.0 * x116 + x142 + x146) + x148 * x39)
    )
    return result


def quadrupole3d_13(ax, da, A, bx, db, B, R):
    """Cartesian 3D (pf) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 3, 10), dtype=float)

    x0 = (ax + bx) ** (-1.0)
    x1 = 3.0 * x0
    x2 = x0 * (ax * A[0] + bx * B[0])
    x3 = -x2
    x4 = x3 + A[0]
    x5 = x3 + B[0]
    x6 = x4 * x5
    x7 = 2.0 * x6
    x8 = x3 + R[0]
    x9 = x4 * x8
    x10 = 2.0 * x9
    x11 = x5 * x8
    x12 = 2.0 * x11
    x13 = x0 * (x1 + x10 + x12 + x7)
    x14 = -2.0 * x2
    x15 = x14 + R[0]
    x16 = x15 + B[0]
    x17 = x0 * x16
    x18 = x0 + x12
    x19 = x18 * x4
    x20 = x17 + x19
    x21 = 4.0 * x20
    x22 = x8**2
    x23 = x1 + 4.0 * x11
    x24 = x18 * x8
    x25 = x17 + x24
    x26 = 2.0 * x5
    x27 = x0 * (2.0 * x22 + x23) + x25 * x26
    x28 = x0 * (x15 + A[0]) + x8 * (x0 + x10)
    x29 = x1 * x16 + 2.0 * x19
    x30 = x13 + 2.0 * x20 * x8
    x31 = x0 * (x24 + x28 + x29) + x30 * x5
    x32 = 3.872983346207417
    x33 = ax * bx * x0
    x34 = (
        5.568327996831708
        * da
        * db
        * numpy.exp(-x33 * ((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2 + (A[2] - B[2]) ** 2))
    )
    x35 = x0**1.5 * x34
    x36 = x32 * x35
    x37 = 0.008333333333333333 * x36
    x38 = x0 * (ax * A[1] + bx * B[1])
    x39 = -x38
    x40 = x39 + B[1]
    x41 = x35 * x40
    x42 = 1.732050807568877
    x43 = 0.08333333333333333 * x42
    x44 = x31 * x43
    x45 = x0 * (ax * A[2] + bx * B[2])
    x46 = -x45
    x47 = x46 + B[2]
    x48 = x35 * x47
    x49 = x40**2
    x50 = 0.5 * x0
    x51 = x49 + x50
    x52 = x0**1.5 * x34
    x53 = x43 * x52
    x54 = x30 * x53
    x55 = 0.25 * x41
    x56 = x47**2
    x57 = x50 + x56
    x58 = x28 * x40
    x59 = 1.5 * x0
    x60 = x49 + x59
    x61 = x32 * x52
    x62 = 0.03333333333333333 * x61
    x63 = x60 * x62
    x64 = x28 * x47
    x65 = 0.1666666666666667 * x42
    x66 = x52 * x65
    x67 = x51 * x66
    x68 = x57 * x66
    x69 = x56 + x59
    x70 = x62 * x69
    x71 = x39 + A[1]
    x72 = x18 * x5
    x73 = 2.0 * x0
    x74 = 0.01666666666666667 * x36
    x75 = x74 * (x27 * x5 + x73 * (2.0 * x17 + x24 + x72))
    x76 = x40 * x71
    x77 = x50 + x76
    x78 = x27 * x53
    x79 = x27 * x43
    x80 = -2.0 * x38
    x81 = x80 + B[1]
    x82 = 2.0 * x76
    x83 = x0 * (x81 + A[1]) + x40 * (x0 + x82)
    x84 = x25 * x53
    x85 = 0.5 * x47
    x86 = x25 * x52
    x87 = x65 * x86
    x88 = x1 + 2.0 * x49
    x89 = 2.0 * x40
    x90 = 0.01666666666666667 * x0 * (4.0 * x76 + x88) + 0.01666666666666667 * x83 * x89
    x91 = x22 + x50
    x92 = x61 * x91
    x93 = x66 * x91
    x94 = 0.3333333333333333 * x42
    x95 = x57 * x94
    x96 = x52 * x91
    x97 = 0.06666666666666667 * x92
    x98 = x47 * x69
    x99 = x46 + A[2]
    x100 = x47 * x99
    x101 = x100 + x50
    x102 = 0.5 * x40
    x103 = -2.0 * x45
    x104 = x103 + B[2]
    x105 = 2.0 * x100
    x106 = x0 * (x104 + A[2]) + x47 * (x0 + x105)
    x107 = x40 * x60
    x108 = x51 * x94
    x109 = x1 + 2.0 * x56
    x110 = 2.0 * x47
    x111 = (
        0.01666666666666667 * x0 * (4.0 * x100 + x109) + 0.01666666666666667 * x106 * x110
    )
    x112 = x39 + R[1]
    x113 = x0 * (x14 + A[0] + B[0]) + x5 * (x0 + x7)
    x114 = x13 + x20 * x26
    x115 = x74 * (x0 * (x113 + x29 + x72) + x114 * x5)
    x116 = x112 * x40
    x117 = x116 + x50
    x118 = x117 * x52
    x119 = x118 * x43
    x120 = x43 * x48
    x121 = x81 + R[1]
    x122 = x0 * x121
    x123 = 2.0 * x116
    x124 = x0 + x123
    x125 = x124 * x40
    x126 = x122 + x125
    x127 = x20 * x53
    x128 = 4.0 * x116
    x129 = x0 * (x128 + x88) + x126 * x89
    x130 = x50 + x9
    x131 = 0.01666666666666667 * x61
    x132 = x130 * x131
    x133 = x47 * x66
    x134 = 0.06666666666666667 * x61
    x135 = x134 * x98
    x136 = x112 * x71
    x137 = x136 + x50
    x138 = x5**2
    x139 = 2.0 * x138
    x140 = x17 + x72
    x141 = x0 * (x139 + x23) + x140 * x26
    x142 = x131 * x141
    x143 = x124 * x71
    x144 = x122 + x143
    x145 = x140 * x53
    x146 = 2.0 * x136
    x147 = x0 * (x1 + x123 + x146 + x82)
    x148 = x144 * x89 + x147
    x149 = x52 * (x11 + x50)
    x150 = x149 * x43
    x151 = x1 * x121 + 2.0 * x143
    x152 = x0 * (x125 + x151 + x83) + x148 * x40
    x153 = x74 * x8
    x154 = x112 * x74
    x155 = x65 * x99
    x156 = x101 * x66
    x157 = x106 * x65
    x158 = x36 * x8
    x159 = x46 + R[2]
    x160 = x41 * x43
    x161 = x159 * x47
    x162 = x161 + x50
    x163 = x162 * x53
    x164 = x162 * x52
    x165 = x104 + R[2]
    x166 = x0 * x165
    x167 = 2.0 * x161
    x168 = x0 + x167
    x169 = x168 * x47
    x170 = x166 + x169
    x171 = x130 * x40
    x172 = x134 * x60
    x173 = x170 * x66
    x174 = 4.0 * x161
    x175 = x0 * (x109 + x174) + x110 * x170
    x176 = x159 * x74
    x177 = x140 * x66
    x178 = x65 * x71
    x179 = x65 * x83
    x180 = x159 * x99
    x181 = x180 + x50
    x182 = x181 * x40
    x183 = x168 * x99
    x184 = x166 + x183
    x185 = 2.0 * x180
    x186 = x0 * (x1 + x105 + x167 + x185)
    x187 = x110 * x184 + x186
    x188 = x1 * x165 + 2.0 * x183
    x189 = x0 * (x106 + x169 + x188) + x187 * x47
    x190 = x112**2
    x191 = x190 + x50
    x192 = x0 * (x1 + x139 + 4.0 * x6) + x113 * x26
    x193 = x131 * x192
    x194 = x112 * x124
    x195 = x122 + x194
    x196 = x195 * x53
    x197 = x50 + x6
    x198 = x0 * (x1 + x128 + 2.0 * x190) + x195 * x89
    x199 = x198 * x53
    x200 = x195 * x52
    x201 = x191 * x52
    x202 = x74 * (x198 * x40 + x73 * (2.0 * x122 + x125 + x194))
    x203 = x4 * x65
    x204 = x191 * x61
    x205 = 0.06666666666666667 * x204
    x206 = x0 * (x80 + A[1] + R[1]) + x112 * (x0 + x146)
    x207 = x206 * x5
    x208 = x138 + x59
    x209 = x208 * x62
    x210 = x138 + x50
    x211 = x112 * x144
    x212 = x147 + 2.0 * x211
    x213 = x212 * x53
    x214 = x0 * (x151 + x194 + x206) + x212 * x40
    x215 = x214 * x43
    x216 = x35 * x5
    x217 = x208 * x5
    x218 = x210 * x94
    x219 = x216 * x43
    x220 = 0.5 * x5
    x221 = x5 * x66
    x222 = x113 * x65
    x223 = x159 * x66
    x224 = x137 * x5
    x225 = x210 * x66
    x226 = x159**2
    x227 = x226 + x50
    x228 = x159 * x168
    x229 = x166 + x228
    x230 = x229 * x53
    x231 = x227 * x52
    x232 = x229 * x52
    x233 = x0 * (x1 + x174 + 2.0 * x226) + x110 * x229
    x234 = x233 * x53
    x235 = x227 * x61
    x236 = 0.06666666666666667 * x235
    x237 = x74 * (x233 * x47 + x73 * (2.0 * x166 + x169 + x228))
    x238 = x0 * (x103 + A[2] + R[2]) + x159 * (x0 + x185)
    x239 = x238 * x5
    x240 = x238 * x40
    x241 = x159 * x184
    x242 = x186 + 2.0 * x241
    x243 = x242 * x53
    x244 = x0 * (x188 + x228 + x238) + x242 * x47
    x245 = x244 * x43

    # 180 item(s)
    result[0, 0, 0] = numpy.sum(
        x37 * (x0 * (4.0 * x13 + x21 * x5 + x21 * x8 + x27) + x26 * x31)
    )
    result[0, 0, 1] = numpy.sum(x41 * x44)
    result[0, 0, 2] = numpy.sum(x44 * x48)
    result[0, 0, 3] = numpy.sum(x51 * x54)
    result[0, 0, 4] = numpy.sum(x30 * x47 * x55)
    result[0, 0, 5] = numpy.sum(x54 * x57)
    result[0, 0, 6] = numpy.sum(x58 * x63)
    result[0, 0, 7] = numpy.sum(x64 * x67)
    result[0, 0, 8] = numpy.sum(x58 * x68)
    result[0, 0, 9] = numpy.sum(x64 * x70)
    result[0, 1, 0] = numpy.sum(x71 * x75)
    result[0, 1, 1] = numpy.sum(x77 * x78)
    result[0, 1, 2] = numpy.sum(x48 * x71 * x79)
    result[0, 1, 3] = numpy.sum(x83 * x84)
    result[0, 1, 4] = numpy.sum(x77 * x85 * x86)
    result[0, 1, 5] = numpy.sum(x57 * x71 * x87)
    result[0, 1, 6] = numpy.sum(x90 * x92)
    result[0, 1, 7] = numpy.sum(x47 * x83 * x93)
    result[0, 1, 8] = numpy.sum(x77 * x95 * x96)
    result[0, 1, 9] = numpy.sum(x71 * x97 * x98)
    result[0, 2, 0] = numpy.sum(x75 * x99)
    result[0, 2, 1] = numpy.sum(x41 * x79 * x99)
    result[0, 2, 2] = numpy.sum(x101 * x78)
    result[0, 2, 3] = numpy.sum(x51 * x87 * x99)
    result[0, 2, 4] = numpy.sum(x101 * x102 * x86)
    result[0, 2, 5] = numpy.sum(x106 * x84)
    result[0, 2, 6] = numpy.sum(x107 * x97 * x99)
    result[0, 2, 7] = numpy.sum(x101 * x108 * x96)
    result[0, 2, 8] = numpy.sum(x106 * x40 * x93)
    result[0, 2, 9] = numpy.sum(x111 * x92)
    result[1, 0, 0] = numpy.sum(x112 * x115)
    result[1, 0, 1] = numpy.sum(x114 * x119)
    result[1, 0, 2] = numpy.sum(x112 * x114 * x120)
    result[1, 0, 3] = numpy.sum(x126 * x127)
    result[1, 0, 4] = numpy.sum(x118 * x20 * x85)
    result[1, 0, 5] = numpy.sum(x112 * x20 * x68)
    result[1, 0, 6] = numpy.sum(x129 * x132)
    result[1, 0, 7] = numpy.sum(x126 * x130 * x133)
    result[1, 0, 8] = numpy.sum(x118 * x130 * x95)
    result[1, 0, 9] = numpy.sum(x112 * x130 * x135)
    result[1, 1, 0] = numpy.sum(x137 * x142)
    result[1, 1, 1] = numpy.sum(x144 * x145)
    result[1, 1, 2] = numpy.sum(x133 * x137 * x140)
    result[1, 1, 3] = numpy.sum(x148 * x150)
    result[1, 1, 4] = numpy.sum(x144 * x149 * x85)
    result[1, 1, 5] = numpy.sum(x137 * x149 * x95)
    result[1, 1, 6] = numpy.sum(x152 * x153)
    result[1, 1, 7] = numpy.sum(x120 * x148 * x8)
    result[1, 1, 8] = numpy.sum(x144 * x68 * x8)
    result[1, 1, 9] = numpy.sum(x135 * x137 * x8)
    result[1, 2, 0] = numpy.sum(x141 * x154 * x99)
    result[1, 2, 1] = numpy.sum(x118 * x140 * x155)
    result[1, 2, 2] = numpy.sum(x112 * x140 * x156)
    result[1, 2, 3] = numpy.sum(x126 * x149 * x155)
    result[1, 2, 4] = numpy.sum(x101 * x117 * x149)
    result[1, 2, 5] = numpy.sum(x112 * x149 * x157)
    result[1, 2, 6] = numpy.sum(x129 * x153 * x99)
    result[1, 2, 7] = numpy.sum(x126 * x156 * x8)
    result[1, 2, 8] = numpy.sum(x118 * x157 * x8)
    result[1, 2, 9] = numpy.sum(x111 * x112 * x158)
    result[2, 0, 0] = numpy.sum(x115 * x159)
    result[2, 0, 1] = numpy.sum(x114 * x159 * x160)
    result[2, 0, 2] = numpy.sum(x114 * x163)
    result[2, 0, 3] = numpy.sum(x159 * x20 * x67)
    result[2, 0, 4] = numpy.sum(x102 * x164 * x20)
    result[2, 0, 5] = numpy.sum(x127 * x170)
    result[2, 0, 6] = numpy.sum(x159 * x171 * x172)
    result[2, 0, 7] = numpy.sum(x108 * x130 * x164)
    result[2, 0, 8] = numpy.sum(x171 * x173)
    result[2, 0, 9] = numpy.sum(x132 * x175)
    result[2, 1, 0] = numpy.sum(x141 * x176 * x71)
    result[2, 1, 1] = numpy.sum(x159 * x177 * x77)
    result[2, 1, 2] = numpy.sum(x140 * x164 * x178)
    result[2, 1, 3] = numpy.sum(x149 * x159 * x179)
    result[2, 1, 4] = numpy.sum(x149 * x162 * x77)
    result[2, 1, 5] = numpy.sum(x149 * x170 * x178)
    result[2, 1, 6] = numpy.sum(x158 * x159 * x90)
    result[2, 1, 7] = numpy.sum(x164 * x179 * x8)
    result[2, 1, 8] = numpy.sum(x173 * x77 * x8)
    result[2, 1, 9] = numpy.sum(x153 * x175 * x71)
    result[2, 2, 0] = numpy.sum(x142 * x181)
    result[2, 2, 1] = numpy.sum(x177 * x182)
    result[2, 2, 2] = numpy.sum(x145 * x184)
    result[2, 2, 3] = numpy.sum(x108 * x149 * x181)
    result[2, 2, 4] = numpy.sum(x102 * x149 * x184)
    result[2, 2, 5] = numpy.sum(x150 * x187)
    result[2, 2, 6] = numpy.sum(x172 * x182 * x8)
    result[2, 2, 7] = numpy.sum(x184 * x67 * x8)
    result[2, 2, 8] = numpy.sum(x160 * x187 * x8)
    result[2, 2, 9] = numpy.sum(x153 * x189)
    result[3, 0, 0] = numpy.sum(x191 * x193)
    result[3, 0, 1] = numpy.sum(x113 * x196)
    result[3, 0, 2] = numpy.sum(x113 * x133 * x191)
    result[3, 0, 3] = numpy.sum(x197 * x199)
    result[3, 0, 4] = numpy.sum(x197 * x200 * x85)
    result[3, 0, 5] = numpy.sum(x197 * x201 * x95)
    result[3, 0, 6] = numpy.sum(x202 * x4)
    result[3, 0, 7] = numpy.sum(x120 * x198 * x4)
    result[3, 0, 8] = numpy.sum(x200 * x203 * x57)
    result[3, 0, 9] = numpy.sum(x205 * x4 * x98)
    result[3, 1, 0] = numpy.sum(x207 * x209)
    result[3, 1, 1] = numpy.sum(x210 * x213)
    result[3, 1, 2] = numpy.sum(x133 * x206 * x210)
    result[3, 1, 3] = numpy.sum(x215 * x216)
    result[3, 1, 4] = numpy.sum(0.25 * x212 * x48 * x5)
    result[3, 1, 5] = numpy.sum(x207 * x68)
    result[3, 1, 6] = numpy.sum(
        x37 * (x0 * (4.0 * x144 * x40 + 4.0 * x147 + x198 + 4.0 * x211) + x214 * x89)
    )
    result[3, 1, 7] = numpy.sum(x215 * x48)
    result[3, 1, 8] = numpy.sum(x213 * x57)
    result[3, 1, 9] = numpy.sum(x206 * x47 * x70)
    result[3, 2, 0] = numpy.sum(x205 * x217 * x99)
    result[3, 2, 1] = numpy.sum(x155 * x200 * x210)
    result[3, 2, 2] = numpy.sum(x101 * x201 * x218)
    result[3, 2, 3] = numpy.sum(x198 * x219 * x99)
    result[3, 2, 4] = numpy.sum(x101 * x200 * x220)
    result[3, 2, 5] = numpy.sum(x106 * x191 * x221)
    result[3, 2, 6] = numpy.sum(x202 * x99)
    result[3, 2, 7] = numpy.sum(x101 * x199)
    result[3, 2, 8] = numpy.sum(x106 * x196)
    result[3, 2, 9] = numpy.sum(x111 * x204)
    result[4, 0, 0] = numpy.sum(x112 * x176 * x192)
    result[4, 0, 1] = numpy.sum(x118 * x159 * x222)
    result[4, 0, 2] = numpy.sum(x112 * x164 * x222)
    result[4, 0, 3] = numpy.sum(x126 * x197 * x223)
    result[4, 0, 4] = numpy.sum(x118 * x162 * x197)
    result[4, 0, 5] = numpy.sum(x112 * x173 * x197)
    result[4, 0, 6] = numpy.sum(x129 * x176 * x4)
    result[4, 0, 7] = numpy.sum(x126 * x164 * x203)
    result[4, 0, 8] = numpy.sum(x118 * x170 * x203)
    result[4, 0, 9] = numpy.sum(x154 * x175 * x4)
    result[4, 1, 0] = numpy.sum(x134 * x159 * x208 * x224)
    result[4, 1, 1] = numpy.sum(x144 * x210 * x223)
    result[4, 1, 2] = numpy.sum(x137 * x164 * x218)
    result[4, 1, 3] = numpy.sum(x148 * x159 * x219)
    result[4, 1, 4] = numpy.sum(x144 * x164 * x220)
    result[4, 1, 5] = numpy.sum(x173 * x224)
    result[4, 1, 6] = numpy.sum(x152 * x176)
    result[4, 1, 7] = numpy.sum(x148 * x163)
    result[4, 1, 8] = numpy.sum(x144 * x170 * x53)
    result[4, 1, 9] = numpy.sum(x131 * x137 * x175)
    result[4, 2, 0] = numpy.sum(x112 * x134 * x181 * x217)
    result[4, 2, 1] = numpy.sum(x118 * x181 * x218)
    result[4, 2, 2] = numpy.sum(x112 * x184 * x225)
    result[4, 2, 3] = numpy.sum(x126 * x181 * x221)
    result[4, 2, 4] = numpy.sum(x118 * x184 * x220)
    result[4, 2, 5] = numpy.sum(x112 * x187 * x219)
    result[4, 2, 6] = numpy.sum(x129 * x131 * x181)
    result[4, 2, 7] = numpy.sum(x126 * x184 * x53)
    result[4, 2, 8] = numpy.sum(x119 * x187)
    result[4, 2, 9] = numpy.sum(x154 * x189)
    result[5, 0, 0] = numpy.sum(x193 * x227)
    result[5, 0, 1] = numpy.sum(x113 * x227 * x40 * x66)
    result[5, 0, 2] = numpy.sum(x113 * x230)
    result[5, 0, 3] = numpy.sum(x108 * x197 * x231)
    result[5, 0, 4] = numpy.sum(x102 * x197 * x232)
    result[5, 0, 5] = numpy.sum(x197 * x234)
    result[5, 0, 6] = numpy.sum(x107 * x236 * x4)
    result[5, 0, 7] = numpy.sum(x203 * x232 * x51)
    result[5, 0, 8] = numpy.sum(x160 * x233 * x4)
    result[5, 0, 9] = numpy.sum(x237 * x4)
    result[5, 1, 0] = numpy.sum(x217 * x236 * x71)
    result[5, 1, 1] = numpy.sum(x218 * x231 * x77)
    result[5, 1, 2] = numpy.sum(x178 * x210 * x232)
    result[5, 1, 3] = numpy.sum(x221 * x227 * x83)
    result[5, 1, 4] = numpy.sum(x220 * x232 * x77)
    result[5, 1, 5] = numpy.sum(x219 * x233 * x71)
    result[5, 1, 6] = numpy.sum(x235 * x90)
    result[5, 1, 7] = numpy.sum(x230 * x83)
    result[5, 1, 8] = numpy.sum(x234 * x77)
    result[5, 1, 9] = numpy.sum(x237 * x71)
    result[5, 2, 0] = numpy.sum(x209 * x239)
    result[5, 2, 1] = numpy.sum(x225 * x240)
    result[5, 2, 2] = numpy.sum(x210 * x243)
    result[5, 2, 3] = numpy.sum(x239 * x67)
    result[5, 2, 4] = numpy.sum(x242 * x5 * x55)
    result[5, 2, 5] = numpy.sum(x216 * x245)
    result[5, 2, 6] = numpy.sum(x240 * x63)
    result[5, 2, 7] = numpy.sum(x243 * x51)
    result[5, 2, 8] = numpy.sum(x245 * x41)
    result[5, 2, 9] = numpy.sum(
        x37 * (x0 * (4.0 * x184 * x47 + 4.0 * x186 + x233 + 4.0 * x241) + x110 * x244)
    )
    return result


def quadrupole3d_14(ax, da, A, bx, db, B, R):
    """Cartesian 3D (pg) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 3, 15), dtype=float)

    x0 = 0.5 / (ax + bx)
    x1 = (ax + bx) ** (-1.0)
    x2 = -x1 * (ax * A[0] + bx * B[0])
    x3 = -x2 - B[0]
    x4 = ax * bx * x1
    x5 = numpy.exp(-x4 * (A[0] - B[0]) ** 2)
    x6 = 1.772453850905516 * numpy.sqrt(x1)
    x7 = x5 * x6
    x8 = x0 * x7
    x9 = -x2 - R[0]
    x10 = x3 * x7
    x11 = x10 * x9
    x12 = x11 + x8
    x13 = x12 * x3
    x14 = -x2 - A[0]
    x15 = x14 * x7
    x16 = x0 * (x10 + x15)
    x17 = x10 * x14
    x18 = x17 + x8
    x19 = x18 * x3
    x20 = x16 + x19
    x21 = x7 * x9
    x22 = x0 * (x10 + x21)
    x23 = 3.0 * x22
    x24 = x12 * x14
    x25 = x23 + 2.0 * x24
    x26 = x0 * (x13 + x20 + x25)
    x27 = 3.0 * x8
    x28 = x14 * x21
    x29 = x0 * (x11 + x17 + x27 + x28)
    x30 = x22 + x24
    x31 = x3 * x30
    x32 = x29 + x31
    x33 = x3 * x32
    x34 = x12 * x9
    x35 = x28 + x8
    x36 = x0 * (x15 + x21) + x35 * x9
    x37 = x0 * (x25 + x34 + x36)
    x38 = x30 * x9
    x39 = x29 + x38
    x40 = x3 * x39
    x41 = x7 * x9**2
    x42 = 2.0 * x11 + x27
    x43 = x0 * (x41 + x42)
    x44 = x22 + x34
    x45 = x3 * x44
    x46 = x43 + x45
    x47 = 2.0 * x0 * (x13 + 2.0 * x22 + x34) + x3 * x46
    x48 = x37 + x40
    x49 = x0 * (4.0 * x29 + 2.0 * x31 + 2.0 * x38 + x46) + x3 * x48
    x50 = da * db
    x51 = 0.09759000729485332
    x52 = x50 * x51
    x53 = numpy.exp(-x4 * (A[1] - B[1]) ** 2)
    x54 = numpy.exp(-x4 * (A[2] - B[2]) ** 2)
    x55 = 3.141592653589793 * x1 * x54
    x56 = x53 * x55
    x57 = x52 * x56
    x58 = -x1 * (ax * A[1] + bx * B[1])
    x59 = -x58 - B[1]
    x60 = 0.2581988897471611
    x61 = x50 * x60
    x62 = x59 * x61
    x63 = x49 * x56
    x64 = -x1 * (ax * A[2] + bx * B[2])
    x65 = -x64 - B[2]
    x66 = x61 * x65
    x67 = x54 * x6
    x68 = x53 * x6
    x69 = x59**2 * x68
    x70 = x0 * x68
    x71 = x69 + x70
    x72 = 0.3333333333333333 * x50
    x73 = x71 * x72
    x74 = 1.732050807568877
    x75 = x72 * x74
    x76 = x65 * x75
    x77 = x65**2 * x67
    x78 = x0 * x67
    x79 = x77 + x78
    x80 = x72 * x79
    x81 = x59 * (2.0 * x70 + x71)
    x82 = x61 * x81
    x83 = x65 * x67
    x84 = x73 * x74
    x85 = x59 * x68
    x86 = x74 * x80
    x87 = x65 * (2.0 * x78 + x79)
    x88 = x61 * x87
    x89 = 3.0 * x70
    x90 = x0 * (3.0 * x69 + x89) + x59 * x81
    x91 = x36 * x52
    x92 = 3.0 * x78
    x93 = x0 * (3.0 * x77 + x92) + x65 * x87
    x94 = -x58 - A[1]
    x95 = x3**2 * x7
    x96 = x0 * (x42 + x95)
    x97 = x13 + x22
    x98 = x3 * x97
    x99 = x57 * (x0 * (3.0 * x43 + 3.0 * x45 + 2.0 * x96 + 2.0 * x98) + x3 * x47)
    x100 = x68 * x94
    x101 = x100 * x59
    x102 = x101 + x70
    x103 = x61 * x67
    x104 = x47 * x56
    x105 = x0 * (x100 + x85)
    x106 = x102 * x59
    x107 = x105 + x106
    x108 = x46 * x72
    x109 = x75 * x83
    x110 = x69 + x89
    x111 = x0 * (2.0 * x101 + x110) + x107 * x59
    x112 = x44 * x74
    x113 = x112 * x72
    x114 = x0 * (3.0 * x105 + 3.0 * x106 + x81) + x111 * x59
    x115 = x41 + x8
    x116 = x115 * x50
    x117 = x116 * x51
    x118 = x116 * x60
    x119 = -x64 - A[2]
    x120 = x119 * x67
    x121 = x120 * x65
    x122 = x121 + x78
    x123 = x61 * x68
    x124 = x122 * x75
    x125 = x0 * (x120 + x83)
    x126 = x122 * x65
    x127 = x125 + x126
    x128 = x77 + x92
    x129 = x0 * (2.0 * x121 + x128) + x127 * x65
    x130 = x0 * (3.0 * x125 + 3.0 * x126 + x87) + x129 * x65
    x131 = -x58 - R[1]
    x132 = x96 + x98
    x133 = x0 * (2.0 * x17 + x27 + x95) + x20 * x3
    x134 = x26 + x33
    x135 = x57 * (x0 * (x132 + x133 + 3.0 * x29 + 3.0 * x31) + x134 * x3)
    x136 = x131 * x85
    x137 = x136 + x70
    x138 = x134 * x56
    x139 = x131 * x68
    x140 = x0 * (x139 + x85)
    x141 = x137 * x59
    x142 = x140 + x141
    x143 = x67 * x72
    x144 = x137 * x75
    x145 = 2.0 * x136
    x146 = x0 * (x110 + x145)
    x147 = x142 * x59
    x148 = x146 + x147
    x149 = 3.0 * x140
    x150 = x0 * (3.0 * x141 + x149 + x81) + x148 * x59
    x151 = x52 * x67
    x152 = x61 * x83
    x153 = x52 * x93
    x154 = x100 * x131
    x155 = x154 + x70
    x156 = x8 + x95
    x157 = x3 * (x156 + 2.0 * x8)
    x158 = x0 * (3.0 * x13 + x157 + x23) + x132 * x3
    x159 = x137 * x94
    x160 = x140 + x159
    x161 = x0 * (x101 + x136 + x154 + x89)
    x162 = x160 * x59
    x163 = x161 + x162
    x164 = x149 + 2.0 * x159
    x165 = x0 * (x107 + x141 + x164)
    x166 = x163 * x59
    x167 = x165 + x166
    x168 = x12 * x75
    x169 = x5 * x52
    x170 = x169 * x55
    x171 = x170 * (x0 * (x111 + x148 + 3.0 * x161 + 3.0 * x162) + x167 * x59)
    x172 = x5 * x55
    x173 = x172 * x66
    x174 = x158 * x57
    x175 = x132 * x61
    x176 = x72 * x97
    x177 = x12 * x61
    x178 = x170 * x9
    x179 = x21 * x61
    x180 = x21 * x72
    x181 = 3.141592653589793 * x1 * x53
    x182 = x169 * x181
    x183 = x182 * x9
    x184 = -x64 - R[2]
    x185 = x184 * x83
    x186 = x185 + x78
    x187 = x184 * x67
    x188 = x186 * x75
    x189 = x0 * (x187 + x83)
    x190 = x186 * x65
    x191 = x189 + x190
    x192 = x68 * x72
    x193 = x75 * x85
    x194 = 2.0 * x185
    x195 = x0 * (x128 + x194)
    x196 = x191 * x65
    x197 = x195 + x196
    x198 = x52 * x90
    x199 = x197 * x61
    x200 = 3.0 * x189
    x201 = x0 * (3.0 * x190 + x200 + x87) + x197 * x65
    x202 = x52 * x68
    x203 = x120 * x184
    x204 = x203 + x78
    x205 = x119 * x186
    x206 = x189 + x205
    x207 = x0 * (x121 + x185 + x203 + x92)
    x208 = x206 * x65
    x209 = x207 + x208
    x210 = x200 + 2.0 * x205
    x211 = x0 * (x127 + x190 + x210)
    x212 = x209 * x65
    x213 = x211 + x212
    x214 = x181 * x5
    x215 = x213 * x214
    x216 = x182 * (x0 * (x129 + x197 + 3.0 * x207 + 3.0 * x208) + x213 * x65)
    x217 = x131**2 * x68
    x218 = x217 + x70
    x219 = x0 * (x157 + 3.0 * x16 + 3.0 * x19) + x133 * x3
    x220 = x131 * x137
    x221 = x140 + x220
    x222 = x0 * (x145 + x217 + x89)
    x223 = x221 * x59
    x224 = x222 + x223
    x225 = 2.0 * x0 * (2.0 * x140 + x141 + x220) + x224 * x59
    x226 = x170 * (x0 * (2.0 * x146 + 2.0 * x147 + 3.0 * x222 + 3.0 * x223) + x225 * x59)
    x227 = x0 * (x100 + x139) + x131 * x155
    x228 = x0 * (x27 + 3.0 * x95) + x157 * x3
    x229 = x131 * x160
    x230 = x161 + x229
    x231 = x157 * x61
    x232 = x0 * (x164 + x220 + x227)
    x233 = x230 * x59
    x234 = x232 + x233
    x235 = x156 * x72
    x236 = x235 * x74
    x237 = x0 * (4.0 * x161 + 2.0 * x162 + x224 + 2.0 * x229) + x234 * x59
    x238 = x172 * x237
    x239 = x3 * x61
    x240 = x52 * x7
    x241 = x228 * x52
    x242 = x172 * x239
    x243 = x10 * x75
    x244 = x10 * x61
    x245 = x61 * x7
    x246 = x7 * x72
    x247 = x133 * x61
    x248 = x20 * x72
    x249 = x148 * x61
    x250 = x184**2 * x67
    x251 = x250 + x78
    x252 = x184 * x186
    x253 = x189 + x252
    x254 = x0 * (x194 + x250 + x92)
    x255 = x253 * x65
    x256 = x254 + x255
    x257 = 2.0 * x0 * (2.0 * x189 + x190 + x252) + x256 * x65
    x258 = x214 * x257
    x259 = x182 * (x0 * (2.0 * x195 + 2.0 * x196 + 3.0 * x254 + 3.0 * x255) + x257 * x65)
    x260 = x0 * (x120 + x187) + x184 * x204
    x261 = x184 * x206
    x262 = x207 + x261
    x263 = x0 * (x210 + x252 + x260)
    x264 = x262 * x65
    x265 = x263 + x264
    x266 = x0 * (4.0 * x207 + 2.0 * x208 + x256 + 2.0 * x261) + x265 * x65
    x267 = x214 * x266

    # 270 item(s)
    result[0, 0, 0] = numpy.sum(
        x57 * (x0 * (2.0 * x26 + 2.0 * x33 + 3.0 * x37 + 3.0 * x40 + x47) + x3 * x49)
    )
    result[0, 0, 1] = numpy.sum(x62 * x63)
    result[0, 0, 2] = numpy.sum(x63 * x66)
    result[0, 0, 3] = numpy.sum(x48 * x67 * x73)
    result[0, 0, 4] = numpy.sum(x48 * x56 * x59 * x76)
    result[0, 0, 5] = numpy.sum(x48 * x68 * x80)
    result[0, 0, 6] = numpy.sum(x39 * x67 * x82)
    result[0, 0, 7] = numpy.sum(x39 * x83 * x84)
    result[0, 0, 8] = numpy.sum(x39 * x85 * x86)
    result[0, 0, 9] = numpy.sum(x39 * x68 * x88)
    result[0, 0, 10] = numpy.sum(x67 * x90 * x91)
    result[0, 0, 11] = numpy.sum(x36 * x82 * x83)
    result[0, 0, 12] = numpy.sum(x36 * x71 * x80)
    result[0, 0, 13] = numpy.sum(x36 * x85 * x88)
    result[0, 0, 14] = numpy.sum(x68 * x91 * x93)
    result[0, 1, 0] = numpy.sum(x94 * x99)
    result[0, 1, 1] = numpy.sum(x102 * x103 * x47)
    result[0, 1, 2] = numpy.sum(x104 * x66 * x94)
    result[0, 1, 3] = numpy.sum(x107 * x108 * x67)
    result[0, 1, 4] = numpy.sum(x102 * x109 * x46)
    result[0, 1, 5] = numpy.sum(x100 * x46 * x80)
    result[0, 1, 6] = numpy.sum(x103 * x111 * x44)
    result[0, 1, 7] = numpy.sum(x107 * x113 * x83)
    result[0, 1, 8] = numpy.sum(x102 * x112 * x80)
    result[0, 1, 9] = numpy.sum(x100 * x44 * x88)
    result[0, 1, 10] = numpy.sum(x114 * x117 * x67)
    result[0, 1, 11] = numpy.sum(x111 * x118 * x83)
    result[0, 1, 12] = numpy.sum(x107 * x115 * x80)
    result[0, 1, 13] = numpy.sum(x102 * x118 * x87)
    result[0, 1, 14] = numpy.sum(x100 * x117 * x93)
    result[0, 2, 0] = numpy.sum(x119 * x99)
    result[0, 2, 1] = numpy.sum(x104 * x119 * x62)
    result[0, 2, 2] = numpy.sum(x122 * x123 * x47)
    result[0, 2, 3] = numpy.sum(x120 * x46 * x73)
    result[0, 2, 4] = numpy.sum(x124 * x46 * x85)
    result[0, 2, 5] = numpy.sum(x108 * x127 * x68)
    result[0, 2, 6] = numpy.sum(x120 * x44 * x82)
    result[0, 2, 7] = numpy.sum(x112 * x122 * x73)
    result[0, 2, 8] = numpy.sum(x113 * x127 * x85)
    result[0, 2, 9] = numpy.sum(x123 * x129 * x44)
    result[0, 2, 10] = numpy.sum(x117 * x120 * x90)
    result[0, 2, 11] = numpy.sum(x118 * x122 * x81)
    result[0, 2, 12] = numpy.sum(x115 * x127 * x73)
    result[0, 2, 13] = numpy.sum(x118 * x129 * x85)
    result[0, 2, 14] = numpy.sum(x117 * x130 * x68)
    result[1, 0, 0] = numpy.sum(x131 * x135)
    result[1, 0, 1] = numpy.sum(x103 * x134 * x137)
    result[1, 0, 2] = numpy.sum(x131 * x138 * x66)
    result[1, 0, 3] = numpy.sum(x142 * x143 * x32)
    result[1, 0, 4] = numpy.sum(x144 * x32 * x83)
    result[1, 0, 5] = numpy.sum(x139 * x32 * x80)
    result[1, 0, 6] = numpy.sum(x103 * x148 * x30)
    result[1, 0, 7] = numpy.sum(x109 * x142 * x30)
    result[1, 0, 8] = numpy.sum(x137 * x30 * x86)
    result[1, 0, 9] = numpy.sum(x139 * x30 * x88)
    result[1, 0, 10] = numpy.sum(x150 * x151 * x35)
    result[1, 0, 11] = numpy.sum(x148 * x152 * x35)
    result[1, 0, 12] = numpy.sum(x142 * x35 * x80)
    result[1, 0, 13] = numpy.sum(x137 * x35 * x88)
    result[1, 0, 14] = numpy.sum(x139 * x153 * x35)
    result[1, 1, 0] = numpy.sum(x151 * x155 * x158)
    result[1, 1, 1] = numpy.sum(x103 * x132 * x160)
    result[1, 1, 2] = numpy.sum(x132 * x152 * x155)
    result[1, 1, 3] = numpy.sum(x143 * x163 * x97)
    result[1, 1, 4] = numpy.sum(x109 * x160 * x97)
    result[1, 1, 5] = numpy.sum(x155 * x80 * x97)
    result[1, 1, 6] = numpy.sum(x103 * x12 * x167)
    result[1, 1, 7] = numpy.sum(x163 * x168 * x83)
    result[1, 1, 8] = numpy.sum(x12 * x160 * x86)
    result[1, 1, 9] = numpy.sum(x12 * x155 * x88)
    result[1, 1, 10] = numpy.sum(x171 * x9)
    result[1, 1, 11] = numpy.sum(x167 * x173 * x9)
    result[1, 1, 12] = numpy.sum(x163 * x21 * x80)
    result[1, 1, 13] = numpy.sum(x160 * x21 * x88)
    result[1, 1, 14] = numpy.sum(x153 * x155 * x21)
    result[1, 2, 0] = numpy.sum(x119 * x131 * x174)
    result[1, 2, 1] = numpy.sum(x120 * x137 * x175)
    result[1, 2, 2] = numpy.sum(x122 * x139 * x175)
    result[1, 2, 3] = numpy.sum(x120 * x142 * x176)
    result[1, 2, 4] = numpy.sum(x124 * x137 * x97)
    result[1, 2, 5] = numpy.sum(x127 * x139 * x176)
    result[1, 2, 6] = numpy.sum(x120 * x148 * x177)
    result[1, 2, 7] = numpy.sum(x12 * x124 * x142)
    result[1, 2, 8] = numpy.sum(x127 * x137 * x168)
    result[1, 2, 9] = numpy.sum(x129 * x139 * x177)
    result[1, 2, 10] = numpy.sum(x119 * x150 * x178)
    result[1, 2, 11] = numpy.sum(x122 * x148 * x179)
    result[1, 2, 12] = numpy.sum(x127 * x142 * x180)
    result[1, 2, 13] = numpy.sum(x129 * x137 * x179)
    result[1, 2, 14] = numpy.sum(x130 * x131 * x183)
    result[2, 0, 0] = numpy.sum(x135 * x184)
    result[2, 0, 1] = numpy.sum(x138 * x184 * x62)
    result[2, 0, 2] = numpy.sum(x123 * x134 * x186)
    result[2, 0, 3] = numpy.sum(x187 * x32 * x73)
    result[2, 0, 4] = numpy.sum(x188 * x32 * x85)
    result[2, 0, 5] = numpy.sum(x191 * x192 * x32)
    result[2, 0, 6] = numpy.sum(x187 * x30 * x82)
    result[2, 0, 7] = numpy.sum(x186 * x30 * x84)
    result[2, 0, 8] = numpy.sum(x191 * x193 * x30)
    result[2, 0, 9] = numpy.sum(x123 * x197 * x30)
    result[2, 0, 10] = numpy.sum(x187 * x198 * x35)
    result[2, 0, 11] = numpy.sum(x186 * x35 * x82)
    result[2, 0, 12] = numpy.sum(x191 * x35 * x73)
    result[2, 0, 13] = numpy.sum(x199 * x35 * x85)
    result[2, 0, 14] = numpy.sum(x201 * x202 * x35)
    result[2, 1, 0] = numpy.sum(x174 * x184 * x94)
    result[2, 1, 1] = numpy.sum(x102 * x175 * x187)
    result[2, 1, 2] = numpy.sum(x100 * x175 * x186)
    result[2, 1, 3] = numpy.sum(x107 * x176 * x187)
    result[2, 1, 4] = numpy.sum(x102 * x188 * x97)
    result[2, 1, 5] = numpy.sum(x100 * x176 * x191)
    result[2, 1, 6] = numpy.sum(x111 * x177 * x187)
    result[2, 1, 7] = numpy.sum(x107 * x168 * x186)
    result[2, 1, 8] = numpy.sum(x102 * x168 * x191)
    result[2, 1, 9] = numpy.sum(x100 * x177 * x197)
    result[2, 1, 10] = numpy.sum(x114 * x178 * x184)
    result[2, 1, 11] = numpy.sum(x111 * x179 * x186)
    result[2, 1, 12] = numpy.sum(x107 * x180 * x191)
    result[2, 1, 13] = numpy.sum(x102 * x179 * x197)
    result[2, 1, 14] = numpy.sum(x183 * x201 * x94)
    result[2, 2, 0] = numpy.sum(x158 * x202 * x204)
    result[2, 2, 1] = numpy.sum(x175 * x204 * x85)
    result[2, 2, 2] = numpy.sum(x123 * x132 * x206)
    result[2, 2, 3] = numpy.sum(x204 * x73 * x97)
    result[2, 2, 4] = numpy.sum(x193 * x206 * x97)
    result[2, 2, 5] = numpy.sum(x192 * x209 * x97)
    result[2, 2, 6] = numpy.sum(x12 * x204 * x82)
    result[2, 2, 7] = numpy.sum(x12 * x206 * x84)
    result[2, 2, 8] = numpy.sum(x168 * x209 * x85)
    result[2, 2, 9] = numpy.sum(x12 * x123 * x213)
    result[2, 2, 10] = numpy.sum(x198 * x204 * x21)
    result[2, 2, 11] = numpy.sum(x206 * x21 * x82)
    result[2, 2, 12] = numpy.sum(x209 * x21 * x73)
    result[2, 2, 13] = numpy.sum(x215 * x62 * x9)
    result[2, 2, 14] = numpy.sum(x216 * x9)
    result[3, 0, 0] = numpy.sum(x151 * x218 * x219)
    result[3, 0, 1] = numpy.sum(x103 * x133 * x221)
    result[3, 0, 2] = numpy.sum(x133 * x152 * x218)
    result[3, 0, 3] = numpy.sum(x143 * x20 * x224)
    result[3, 0, 4] = numpy.sum(x109 * x20 * x221)
    result[3, 0, 5] = numpy.sum(x20 * x218 * x80)
    result[3, 0, 6] = numpy.sum(x103 * x18 * x225)
    result[3, 0, 7] = numpy.sum(x109 * x18 * x224)
    result[3, 0, 8] = numpy.sum(x18 * x221 * x86)
    result[3, 0, 9] = numpy.sum(x18 * x218 * x88)
    result[3, 0, 10] = numpy.sum(x14 * x226)
    result[3, 0, 11] = numpy.sum(x14 * x173 * x225)
    result[3, 0, 12] = numpy.sum(x15 * x224 * x80)
    result[3, 0, 13] = numpy.sum(x15 * x221 * x88)
    result[3, 0, 14] = numpy.sum(x15 * x153 * x218)
    result[3, 1, 0] = numpy.sum(x151 * x227 * x228)
    result[3, 1, 1] = numpy.sum(x230 * x231 * x67)
    result[3, 1, 2] = numpy.sum(x227 * x231 * x83)
    result[3, 1, 3] = numpy.sum(x234 * x235 * x67)
    result[3, 1, 4] = numpy.sum(x230 * x236 * x83)
    result[3, 1, 5] = numpy.sum(x156 * x227 * x80)
    result[3, 1, 6] = numpy.sum(x238 * x239)
    result[3, 1, 7] = numpy.sum(x172 * x234 * x3 * x76)
    result[3, 1, 8] = numpy.sum(x10 * x230 * x86)
    result[3, 1, 9] = numpy.sum(x10 * x227 * x88)
    result[3, 1, 10] = numpy.sum(
        x170
        * (x0 * (2.0 * x165 + 2.0 * x166 + x225 + 3.0 * x232 + 3.0 * x233) + x237 * x59)
    )
    result[3, 1, 11] = numpy.sum(x238 * x66)
    result[3, 1, 12] = numpy.sum(x234 * x7 * x80)
    result[3, 1, 13] = numpy.sum(x230 * x7 * x88)
    result[3, 1, 14] = numpy.sum(x227 * x240 * x93)
    result[3, 2, 0] = numpy.sum(x120 * x218 * x241)
    result[3, 2, 1] = numpy.sum(x120 * x221 * x231)
    result[3, 2, 2] = numpy.sum(x122 * x218 * x231)
    result[3, 2, 3] = numpy.sum(x120 * x224 * x235)
    result[3, 2, 4] = numpy.sum(x122 * x221 * x236)
    result[3, 2, 5] = numpy.sum(x127 * x218 * x235)
    result[3, 2, 6] = numpy.sum(x119 * x225 * x242)
    result[3, 2, 7] = numpy.sum(x10 * x124 * x224)
    result[3, 2, 8] = numpy.sum(x127 * x221 * x243)
    result[3, 2, 9] = numpy.sum(x129 * x218 * x244)
    result[3, 2, 10] = numpy.sum(x119 * x226)
    result[3, 2, 11] = numpy.sum(x122 * x225 * x245)
    result[3, 2, 12] = numpy.sum(x127 * x224 * x246)
    result[3, 2, 13] = numpy.sum(x129 * x221 * x245)
    result[3, 2, 14] = numpy.sum(x130 * x218 * x240)
    result[4, 0, 0] = numpy.sum(x131 * x184 * x219 * x57)
    result[4, 0, 1] = numpy.sum(x137 * x187 * x247)
    result[4, 0, 2] = numpy.sum(x139 * x186 * x247)
    result[4, 0, 3] = numpy.sum(x142 * x187 * x248)
    result[4, 0, 4] = numpy.sum(x137 * x188 * x20)
    result[4, 0, 5] = numpy.sum(x139 * x191 * x248)
    result[4, 0, 6] = numpy.sum(x18 * x187 * x249)
    result[4, 0, 7] = numpy.sum(x142 * x18 * x188)
    result[4, 0, 8] = numpy.sum(x144 * x18 * x191)
    result[4, 0, 9] = numpy.sum(x139 * x18 * x199)
    result[4, 0, 10] = numpy.sum(x14 * x150 * x170 * x184)
    result[4, 0, 11] = numpy.sum(x15 * x186 * x249)
    result[4, 0, 12] = numpy.sum(x142 * x15 * x191 * x72)
    result[4, 0, 13] = numpy.sum(x137 * x15 * x199)
    result[4, 0, 14] = numpy.sum(x131 * x14 * x182 * x201)
    result[4, 1, 0] = numpy.sum(x155 * x187 * x241)
    result[4, 1, 1] = numpy.sum(x160 * x187 * x231)
    result[4, 1, 2] = numpy.sum(x155 * x186 * x231)
    result[4, 1, 3] = numpy.sum(x163 * x187 * x235)
    result[4, 1, 4] = numpy.sum(x160 * x186 * x236)
    result[4, 1, 5] = numpy.sum(x155 * x191 * x235)
    result[4, 1, 6] = numpy.sum(x167 * x184 * x242)
    result[4, 1, 7] = numpy.sum(x10 * x163 * x188)
    result[4, 1, 8] = numpy.sum(x160 * x191 * x243)
    result[4, 1, 9] = numpy.sum(x10 * x155 * x199)
    result[4, 1, 10] = numpy.sum(x171 * x184)
    result[4, 1, 11] = numpy.sum(x167 * x186 * x245)
    result[4, 1, 12] = numpy.sum(x163 * x191 * x246)
    result[4, 1, 13] = numpy.sum(x160 * x197 * x245)
    result[4, 1, 14] = numpy.sum(x155 * x201 * x240)
    result[4, 2, 0] = numpy.sum(x139 * x204 * x241)
    result[4, 2, 1] = numpy.sum(x137 * x204 * x231)
    result[4, 2, 2] = numpy.sum(x139 * x206 * x231)
    result[4, 2, 3] = numpy.sum(x142 * x204 * x235)
    result[4, 2, 4] = numpy.sum(x137 * x206 * x236)
    result[4, 2, 5] = numpy.sum(x139 * x209 * x235)
    result[4, 2, 6] = numpy.sum(x148 * x204 * x244)
    result[4, 2, 7] = numpy.sum(x142 * x206 * x243)
    result[4, 2, 8] = numpy.sum(x10 * x144 * x209)
    result[4, 2, 9] = numpy.sum(x131 * x215 * x239)
    result[4, 2, 10] = numpy.sum(x150 * x204 * x240)
    result[4, 2, 11] = numpy.sum(x148 * x206 * x245)
    result[4, 2, 12] = numpy.sum(x142 * x209 * x246)
    result[4, 2, 13] = numpy.sum(x137 * x213 * x245)
    result[4, 2, 14] = numpy.sum(x131 * x216)
    result[5, 0, 0] = numpy.sum(x202 * x219 * x251)
    result[5, 0, 1] = numpy.sum(x247 * x251 * x85)
    result[5, 0, 2] = numpy.sum(x123 * x133 * x253)
    result[5, 0, 3] = numpy.sum(x20 * x251 * x73)
    result[5, 0, 4] = numpy.sum(x193 * x20 * x253)
    result[5, 0, 5] = numpy.sum(x192 * x20 * x256)
    result[5, 0, 6] = numpy.sum(x18 * x251 * x82)
    result[5, 0, 7] = numpy.sum(x18 * x253 * x84)
    result[5, 0, 8] = numpy.sum(x18 * x193 * x256)
    result[5, 0, 9] = numpy.sum(x123 * x18 * x257)
    result[5, 0, 10] = numpy.sum(x15 * x198 * x251)
    result[5, 0, 11] = numpy.sum(x15 * x253 * x82)
    result[5, 0, 12] = numpy.sum(x15 * x256 * x73)
    result[5, 0, 13] = numpy.sum(x14 * x258 * x62)
    result[5, 0, 14] = numpy.sum(x14 * x259)
    result[5, 1, 0] = numpy.sum(x100 * x241 * x251)
    result[5, 1, 1] = numpy.sum(x102 * x231 * x251)
    result[5, 1, 2] = numpy.sum(x100 * x231 * x253)
    result[5, 1, 3] = numpy.sum(x107 * x235 * x251)
    result[5, 1, 4] = numpy.sum(x102 * x236 * x253)
    result[5, 1, 5] = numpy.sum(x100 * x235 * x256)
    result[5, 1, 6] = numpy.sum(x111 * x244 * x251)
    result[5, 1, 7] = numpy.sum(x107 * x243 * x253)
    result[5, 1, 8] = numpy.sum(x102 * x243 * x256)
    result[5, 1, 9] = numpy.sum(x239 * x258 * x94)
    result[5, 1, 10] = numpy.sum(x114 * x240 * x251)
    result[5, 1, 11] = numpy.sum(x111 * x245 * x253)
    result[5, 1, 12] = numpy.sum(x107 * x246 * x256)
    result[5, 1, 13] = numpy.sum(x102 * x245 * x257)
    result[5, 1, 14] = numpy.sum(x259 * x94)
    result[5, 2, 0] = numpy.sum(x202 * x228 * x260)
    result[5, 2, 1] = numpy.sum(x231 * x260 * x85)
    result[5, 2, 2] = numpy.sum(x231 * x262 * x68)
    result[5, 2, 3] = numpy.sum(x156 * x260 * x73)
    result[5, 2, 4] = numpy.sum(x236 * x262 * x85)
    result[5, 2, 5] = numpy.sum(x235 * x265 * x68)
    result[5, 2, 6] = numpy.sum(x10 * x260 * x82)
    result[5, 2, 7] = numpy.sum(x10 * x262 * x84)
    result[5, 2, 8] = numpy.sum(x214 * x265 * x3 * x59 * x75)
    result[5, 2, 9] = numpy.sum(x239 * x267)
    result[5, 2, 10] = numpy.sum(x240 * x260 * x90)
    result[5, 2, 11] = numpy.sum(x262 * x7 * x82)
    result[5, 2, 12] = numpy.sum(x265 * x7 * x73)
    result[5, 2, 13] = numpy.sum(x267 * x62)
    result[5, 2, 14] = numpy.sum(
        x182
        * (x0 * (2.0 * x211 + 2.0 * x212 + x257 + 3.0 * x263 + 3.0 * x264) + x266 * x65)
    )
    return result


def quadrupole3d_20(ax, da, A, bx, db, B, R):
    """Cartesian 3D (ds) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 6, 1), dtype=float)

    x0 = (ax + bx) ** (-1.0)
    x1 = x0 * (ax * A[0] + bx * B[0])
    x2 = -x1
    x3 = x2 + R[0]
    x4 = x3**2
    x5 = 3.0 * x0
    x6 = x2 + A[0]
    x7 = x3 * x6
    x8 = x0 * (-2.0 * x1 + A[0] + R[0])
    x9 = x0 + 2.0 * x7
    x10 = x3 * x9 + x8
    x11 = 1.732050807568877
    x12 = ax * bx * x0
    x13 = (
        5.568327996831708
        * da
        * db
        * numpy.exp(-x12 * ((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2 + (A[2] - B[2]) ** 2))
    )
    x14 = numpy.sqrt(x0) * x13
    x15 = x0 * x14
    x16 = x11 * x15
    x17 = 0.08333333333333333 * x16
    x18 = x0 * (ax * A[1] + bx * B[1])
    x19 = -x18
    x20 = x19 + A[1]
    x21 = 0.5 * x0
    x22 = x14 * x21
    x23 = x10 * x22
    x24 = x0 * (ax * A[2] + bx * B[2])
    x25 = -x24
    x26 = x25 + A[2]
    x27 = 0.3333333333333333 * x20**2 + 0.3333333333333333 * x21
    x28 = x21 + x4
    x29 = x0**1.5 * x13
    x30 = x11 * x29
    x31 = x28 * x30
    x32 = x15 * x26
    x33 = 0.3333333333333333 * x21 + 0.3333333333333333 * x26**2
    x34 = x19 + R[1]
    x35 = 0.1666666666666667 * x16
    x36 = x35 * (x6 * x9 + x8)
    x37 = x20 * x34
    x38 = x21 + x37
    x39 = x21 + x7
    x40 = x29 * x39
    x41 = x0 * (-2.0 * x18 + A[1] + R[1])
    x42 = x0 + 2.0 * x37
    x43 = x20 * x42 + x41
    x44 = x3 * x35
    x45 = x16 * x3
    x46 = x25 + R[2]
    x47 = x15 * x20
    x48 = x26 * x46
    x49 = x21 + x48
    x50 = x0 * (-2.0 * x24 + A[2] + R[2])
    x51 = x0 + 2.0 * x48
    x52 = x26 * x51 + x50
    x53 = x34**2
    x54 = x21 + x53
    x55 = x30 * x54
    x56 = 0.3333333333333333 * x21 + 0.3333333333333333 * x6**2
    x57 = x34 * x42 + x41
    x58 = x22 * x57
    x59 = x15 * x6
    x60 = x46**2
    x61 = x21 + x60
    x62 = x30 * x61
    x63 = x46 * x51 + x50
    x64 = x22 * x63

    # 36 item(s)
    result[0, 0, 0] = numpy.sum(x17 * (x0 * (2.0 * x4 + x5 + 4.0 * x7) + 2.0 * x10 * x6))
    result[0, 1, 0] = numpy.sum(x20 * x23)
    result[0, 2, 0] = numpy.sum(x23 * x26)
    result[0, 3, 0] = numpy.sum(x27 * x31)
    result[0, 4, 0] = numpy.sum(x20 * x28 * x32)
    result[0, 5, 0] = numpy.sum(x31 * x33)
    result[1, 0, 0] = numpy.sum(x34 * x36)
    result[1, 1, 0] = numpy.sum(x38 * x40)
    result[1, 2, 0] = numpy.sum(x32 * x34 * x39)
    result[1, 3, 0] = numpy.sum(x43 * x44)
    result[1, 4, 0] = numpy.sum(x3 * x32 * x38)
    result[1, 5, 0] = numpy.sum(x33 * x34 * x45)
    result[2, 0, 0] = numpy.sum(x36 * x46)
    result[2, 1, 0] = numpy.sum(x39 * x46 * x47)
    result[2, 2, 0] = numpy.sum(x40 * x49)
    result[2, 3, 0] = numpy.sum(x27 * x45 * x46)
    result[2, 4, 0] = numpy.sum(x3 * x47 * x49)
    result[2, 5, 0] = numpy.sum(x44 * x52)
    result[3, 0, 0] = numpy.sum(x55 * x56)
    result[3, 1, 0] = numpy.sum(x58 * x6)
    result[3, 2, 0] = numpy.sum(x32 * x54 * x6)
    result[3, 3, 0] = numpy.sum(
        x17 * (x0 * (4.0 * x37 + x5 + 2.0 * x53) + 2.0 * x20 * x57)
    )
    result[3, 4, 0] = numpy.sum(x26 * x58)
    result[3, 5, 0] = numpy.sum(x33 * x55)
    result[4, 0, 0] = numpy.sum(x16 * x34 * x46 * x56)
    result[4, 1, 0] = numpy.sum(x38 * x46 * x59)
    result[4, 2, 0] = numpy.sum(x34 * x49 * x59)
    result[4, 3, 0] = numpy.sum(x35 * x43 * x46)
    result[4, 4, 0] = numpy.sum(x29 * x38 * x49)
    result[4, 5, 0] = numpy.sum(x34 * x35 * x52)
    result[5, 0, 0] = numpy.sum(x56 * x62)
    result[5, 1, 0] = numpy.sum(x47 * x6 * x61)
    result[5, 2, 0] = numpy.sum(x6 * x64)
    result[5, 3, 0] = numpy.sum(x27 * x62)
    result[5, 4, 0] = numpy.sum(x20 * x64)
    result[5, 5, 0] = numpy.sum(
        x17 * (x0 * (4.0 * x48 + x5 + 2.0 * x60) + 2.0 * x26 * x63)
    )
    return result


def quadrupole3d_21(ax, da, A, bx, db, B, R):
    """Cartesian 3D (dp) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 6, 3), dtype=float)

    x0 = (ax + bx) ** (-1.0)
    x1 = x0 * (ax * A[0] + bx * B[0])
    x2 = -x1
    x3 = x2 + R[0]
    x4 = x2 + B[0]
    x5 = x3 * x4
    x6 = 2.0 * x5
    x7 = x0 + x6
    x8 = x3 * x7
    x9 = x2 + A[0]
    x10 = x7 * x9
    x11 = -2.0 * x1
    x12 = x11 + R[0]
    x13 = x12 + B[0]
    x14 = 3.0 * x0
    x15 = x0 * (x12 + A[0])
    x16 = x3 * x9
    x17 = 2.0 * x16
    x18 = x0 + x17
    x19 = x15 + x18 * x3
    x20 = x4 * x9
    x21 = 2.0 * x20
    x22 = x0 * (x14 + x17 + x21 + x6)
    x23 = x0 * x13
    x24 = x10 + x23
    x25 = 2.0 * x24
    x26 = x22 + x25 * x3
    x27 = 1.732050807568877
    x28 = ax * bx * x0
    x29 = (
        5.568327996831708
        * da
        * db
        * numpy.exp(-x28 * ((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2 + (A[2] - B[2]) ** 2))
    )
    x30 = numpy.sqrt(x0) * x29
    x31 = x0 * x30
    x32 = x27 * x31
    x33 = 0.08333333333333333 * x32
    x34 = x0 * (ax * A[1] + bx * B[1])
    x35 = -x34
    x36 = x35 + B[1]
    x37 = x3**2
    x38 = x33 * (x0 * (x14 + 4.0 * x16 + 2.0 * x37) + 2.0 * x19 * x9)
    x39 = x0 * (ax * A[2] + bx * B[2])
    x40 = -x39
    x41 = x40 + B[2]
    x42 = x35 + A[1]
    x43 = 0.25 * x31
    x44 = x26 * x43
    x45 = 0.5 * x0
    x46 = x36 * x42
    x47 = x0**1.5 * x29
    x48 = x47 * (x45 + x46)
    x49 = 0.5 * x19
    x50 = x30 * x45
    x51 = x19 * x50
    x52 = x40 + A[2]
    x53 = x41 * x52
    x54 = x47 * (x45 + x53)
    x55 = x42**2 + x45
    x56 = x23 + x8
    x57 = 0.1666666666666667 * x27
    x58 = x47 * x57
    x59 = x56 * x58
    x60 = -2.0 * x34
    x61 = x60 + B[1]
    x62 = 2.0 * x46
    x63 = x0 * (x61 + A[1]) + x42 * (x0 + x62)
    x64 = x37 + x45
    x65 = x58 * x64
    x66 = 0.3333333333333333 * x27
    x67 = x47 * x66
    x68 = x64 * x67
    x69 = x50 * x52
    x70 = x45 + x52**2
    x71 = -2.0 * x39
    x72 = x71 + B[2]
    x73 = 2.0 * x53
    x74 = x0 * (x72 + A[2]) + x52 * (x0 + x73)
    x75 = x35 + R[1]
    x76 = x33 * (x22 + x25 * x9)
    x77 = x36 * x75
    x78 = x45 + x77
    x79 = x15 + x18 * x9
    x80 = x58 * x79
    x81 = 0.1666666666666667 * x32
    x82 = x41 * x81
    x83 = x42 * x75
    x84 = x45 + x83
    x85 = 0.5 * x47
    x86 = x24 * x85
    x87 = x61 + R[1]
    x88 = x0 * x87
    x89 = 2.0 * x77
    x90 = x0 + x89
    x91 = x42 * x90
    x92 = x88 + x91
    x93 = x16 + x45
    x94 = x85 * x93
    x95 = x47 * x93
    x96 = x0 * (x60 + A[1] + R[1])
    x97 = 2.0 * x83
    x98 = x0 + x97
    x99 = x42 * x98 + x96
    x100 = x47 * (x45 + x5)
    x101 = x100 * x57
    x102 = x0 * (x14 + x62 + x89 + x97)
    x103 = 2.0 * x92
    x104 = x102 + x103 * x42
    x105 = x3 * x33
    x106 = x100 * x66
    x107 = x3 * x67
    x108 = x3 * x81
    x109 = x40 + R[2]
    x110 = x109 * x81
    x111 = x109 * x41
    x112 = x111 + x45
    x113 = x42 * x50
    x114 = x109 * x52
    x115 = x114 + x45
    x116 = x72 + R[2]
    x117 = x0 * x116
    x118 = 2.0 * x111
    x119 = x0 + x118
    x120 = x119 * x52
    x121 = x117 + x120
    x122 = x0 * (x71 + A[2] + R[2])
    x123 = 2.0 * x114
    x124 = x0 + x123
    x125 = x122 + x124 * x52
    x126 = x0 * (x118 + x123 + x14 + x73)
    x127 = 2.0 * x121
    x128 = x126 + x127 * x52
    x129 = x0 * (x11 + A[0] + B[0]) + x9 * (x0 + x21)
    x130 = x75**2
    x131 = x130 + x45
    x132 = x131 * x58
    x133 = x45 + x9**2
    x134 = x75 * x90
    x135 = x134 + x88
    x136 = x135 * x58
    x137 = x133 * x67
    x138 = x75 * x98 + x96
    x139 = x20 + x45
    x140 = x139 * x85
    x141 = x102 + x103 * x75
    x142 = x141 * x43
    x143 = x50 * x9
    x144 = x139 * x47
    x145 = x33 * (x0 * (2.0 * x130 + x14 + 4.0 * x83) + 2.0 * x138 * x42)
    x146 = x4 * x47
    x147 = x146 * x66
    x148 = x47 * x9
    x149 = x109**2
    x150 = x149 + x45
    x151 = x150 * x58
    x152 = x109 * x119
    x153 = x117 + x152
    x154 = x153 * x58
    x155 = x109 * x124 + x122
    x156 = x109 * x127 + x126
    x157 = x156 * x43
    x158 = x33 * (x0 * (4.0 * x114 + x14 + 2.0 * x149) + 2.0 * x155 * x52)

    # 108 item(s)
    result[0, 0, 0] = numpy.sum(
        -x33 * (x0 * (2.0 * x10 + x13 * x14 + x19 + x8) + x26 * x9)
    )
    result[0, 0, 1] = numpy.sum(-x36 * x38)
    result[0, 0, 2] = numpy.sum(-x38 * x41)
    result[0, 1, 0] = numpy.sum(-x42 * x44)
    result[0, 1, 1] = numpy.sum(-x48 * x49)
    result[0, 1, 2] = numpy.sum(-x41 * x42 * x51)
    result[0, 2, 0] = numpy.sum(-x44 * x52)
    result[0, 2, 1] = numpy.sum(-x36 * x51 * x52)
    result[0, 2, 2] = numpy.sum(-x49 * x54)
    result[0, 3, 0] = numpy.sum(-x55 * x59)
    result[0, 3, 1] = numpy.sum(-x63 * x65)
    result[0, 3, 2] = numpy.sum(-x41 * x55 * x68)
    result[0, 4, 0] = numpy.sum(-x42 * x56 * x69)
    result[0, 4, 1] = numpy.sum(-x48 * x52 * x64)
    result[0, 4, 2] = numpy.sum(-x42 * x54 * x64)
    result[0, 5, 0] = numpy.sum(-x59 * x70)
    result[0, 5, 1] = numpy.sum(-x36 * x68 * x70)
    result[0, 5, 2] = numpy.sum(-x65 * x74)
    result[1, 0, 0] = numpy.sum(-x75 * x76)
    result[1, 0, 1] = numpy.sum(-x78 * x80)
    result[1, 0, 2] = numpy.sum(-x75 * x79 * x82)
    result[1, 1, 0] = numpy.sum(-x84 * x86)
    result[1, 1, 1] = numpy.sum(-x92 * x94)
    result[1, 1, 2] = numpy.sum(-x41 * x84 * x95)
    result[1, 2, 0] = numpy.sum(-x24 * x69 * x75)
    result[1, 2, 1] = numpy.sum(-x52 * x78 * x95)
    result[1, 2, 2] = numpy.sum(-x54 * x75 * x93)
    result[1, 3, 0] = numpy.sum(-x101 * x99)
    result[1, 3, 1] = numpy.sum(-x104 * x105)
    result[1, 3, 2] = numpy.sum(-x3 * x82 * x99)
    result[1, 4, 0] = numpy.sum(-x100 * x52 * x84)
    result[1, 4, 1] = numpy.sum(-x3 * x69 * x92)
    result[1, 4, 2] = numpy.sum(-x3 * x54 * x84)
    result[1, 5, 0] = numpy.sum(-x106 * x70 * x75)
    result[1, 5, 1] = numpy.sum(-x107 * x70 * x78)
    result[1, 5, 2] = numpy.sum(-x108 * x74 * x75)
    result[2, 0, 0] = numpy.sum(-x109 * x76)
    result[2, 0, 1] = numpy.sum(-x110 * x36 * x79)
    result[2, 0, 2] = numpy.sum(-x112 * x80)
    result[2, 1, 0] = numpy.sum(-x109 * x113 * x24)
    result[2, 1, 1] = numpy.sum(-x109 * x48 * x93)
    result[2, 1, 2] = numpy.sum(-x112 * x42 * x95)
    result[2, 2, 0] = numpy.sum(-x115 * x86)
    result[2, 2, 1] = numpy.sum(-x115 * x36 * x95)
    result[2, 2, 2] = numpy.sum(-x121 * x94)
    result[2, 3, 0] = numpy.sum(-x106 * x109 * x55)
    result[2, 3, 1] = numpy.sum(-x108 * x109 * x63)
    result[2, 3, 2] = numpy.sum(-x107 * x112 * x55)
    result[2, 4, 0] = numpy.sum(-x100 * x115 * x42)
    result[2, 4, 1] = numpy.sum(-x115 * x3 * x48)
    result[2, 4, 2] = numpy.sum(-x113 * x121 * x3)
    result[2, 5, 0] = numpy.sum(-x101 * x125)
    result[2, 5, 1] = numpy.sum(-x108 * x125 * x36)
    result[2, 5, 2] = numpy.sum(-x105 * x128)
    result[3, 0, 0] = numpy.sum(-x129 * x132)
    result[3, 0, 1] = numpy.sum(-x133 * x136)
    result[3, 0, 2] = numpy.sum(-x131 * x137 * x41)
    result[3, 1, 0] = numpy.sum(-x138 * x140)
    result[3, 1, 1] = numpy.sum(-x142 * x9)
    result[3, 1, 2] = numpy.sum(-x138 * x143 * x41)
    result[3, 2, 0] = numpy.sum(-x131 * x144 * x52)
    result[3, 2, 1] = numpy.sum(-x135 * x69 * x9)
    result[3, 2, 2] = numpy.sum(-x131 * x54 * x9)
    result[3, 3, 0] = numpy.sum(-x145 * x4)
    result[3, 3, 1] = numpy.sum(
        -x33 * (x0 * (x134 + x138 + x14 * x87 + 2.0 * x91) + x141 * x42)
    )
    result[3, 3, 2] = numpy.sum(-x145 * x41)
    result[3, 4, 0] = numpy.sum(-x138 * x4 * x69)
    result[3, 4, 1] = numpy.sum(-x142 * x52)
    result[3, 4, 2] = numpy.sum(-0.5 * x138 * x54)
    result[3, 5, 0] = numpy.sum(-x131 * x147 * x70)
    result[3, 5, 1] = numpy.sum(-x136 * x70)
    result[3, 5, 2] = numpy.sum(-x132 * x74)
    result[4, 0, 0] = numpy.sum(-x110 * x129 * x75)
    result[4, 0, 1] = numpy.sum(-x109 * x137 * x78)
    result[4, 0, 2] = numpy.sum(-x112 * x137 * x75)
    result[4, 1, 0] = numpy.sum(-x109 * x144 * x84)
    result[4, 1, 1] = numpy.sum(-x109 * x143 * x92)
    result[4, 1, 2] = numpy.sum(-x112 * x148 * x84)
    result[4, 2, 0] = numpy.sum(-x115 * x144 * x75)
    result[4, 2, 1] = numpy.sum(-x115 * x148 * x78)
    result[4, 2, 2] = numpy.sum(-x121 * x143 * x75)
    result[4, 3, 0] = numpy.sum(-x110 * x4 * x99)
    result[4, 3, 1] = numpy.sum(-x104 * x109 * x33)
    result[4, 3, 2] = numpy.sum(-x112 * x58 * x99)
    result[4, 4, 0] = numpy.sum(-x115 * x146 * x84)
    result[4, 4, 1] = numpy.sum(-x115 * x85 * x92)
    result[4, 4, 2] = numpy.sum(-x121 * x84 * x85)
    result[4, 5, 0] = numpy.sum(-x125 * x4 * x75 * x81)
    result[4, 5, 1] = numpy.sum(-x125 * x58 * x78)
    result[4, 5, 2] = numpy.sum(-x128 * x33 * x75)
    result[5, 0, 0] = numpy.sum(-x129 * x151)
    result[5, 0, 1] = numpy.sum(-x137 * x150 * x36)
    result[5, 0, 2] = numpy.sum(-x133 * x154)
    result[5, 1, 0] = numpy.sum(-x144 * x150 * x42)
    result[5, 1, 1] = numpy.sum(-x150 * x48 * x9)
    result[5, 1, 2] = numpy.sum(-x113 * x153 * x9)
    result[5, 2, 0] = numpy.sum(-x140 * x155)
    result[5, 2, 1] = numpy.sum(-x143 * x155 * x36)
    result[5, 2, 2] = numpy.sum(-x157 * x9)
    result[5, 3, 0] = numpy.sum(-x147 * x150 * x55)
    result[5, 3, 1] = numpy.sum(-x151 * x63)
    result[5, 3, 2] = numpy.sum(-x154 * x55)
    result[5, 4, 0] = numpy.sum(-x113 * x155 * x4)
    result[5, 4, 1] = numpy.sum(-0.5 * x155 * x48)
    result[5, 4, 2] = numpy.sum(-x157 * x42)
    result[5, 5, 0] = numpy.sum(-x158 * x4)
    result[5, 5, 1] = numpy.sum(-x158 * x36)
    result[5, 5, 2] = numpy.sum(
        -x33 * (x0 * (x116 * x14 + 2.0 * x120 + x152 + x155) + x156 * x52)
    )
    return result


def quadrupole3d_22(ax, da, A, bx, db, B, R):
    """Cartesian 3D (dd) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 6, 6), dtype=float)

    x0 = (ax + bx) ** (-1.0)
    x1 = 3.0 * x0
    x2 = x0 * (ax * A[0] + bx * B[0])
    x3 = -x2
    x4 = x3 + A[0]
    x5 = x3 + B[0]
    x6 = x4 * x5
    x7 = 2.0 * x6
    x8 = x3 + R[0]
    x9 = x4 * x8
    x10 = 2.0 * x9
    x11 = x5 * x8
    x12 = 2.0 * x11
    x13 = x0 * (x1 + x10 + x12 + x7)
    x14 = -2.0 * x2
    x15 = x14 + R[0]
    x16 = x15 + B[0]
    x17 = x0 * x16
    x18 = x0 + x12
    x19 = x18 * x4
    x20 = x17 + x19
    x21 = 4.0 * x20
    x22 = x8**2
    x23 = x1 + 2.0 * x22
    x24 = x18 * x8
    x25 = x17 + x24
    x26 = 2.0 * x5
    x27 = x0 * (4.0 * x11 + x23) + x25 * x26
    x28 = x0 * (x15 + A[0])
    x29 = x0 + x10
    x30 = x28 + x29 * x8
    x31 = x1 * x16 + 2.0 * x19
    x32 = x0 * (x24 + x30 + x31)
    x33 = 2.0 * x20
    x34 = x13 + x33 * x8
    x35 = x32 + x34 * x5
    x36 = 2.0 * x4
    x37 = ax * bx * x0
    x38 = (
        5.568327996831708
        * da
        * db
        * numpy.exp(-x37 * ((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2 + (A[2] - B[2]) ** 2))
    )
    x39 = x0**1.5 * x38
    x40 = 0.04166666666666667 * x39
    x41 = x0 * (ax * A[1] + bx * B[1])
    x42 = -x41
    x43 = x42 + B[1]
    x44 = 1.732050807568877
    x45 = 0.08333333333333333 * x39
    x46 = x44 * x45
    x47 = x46 * (x32 + x34 * x4)
    x48 = x0 * (ax * A[2] + bx * B[2])
    x49 = -x48
    x50 = x49 + B[2]
    x51 = 0.08333333333333333 * x0 * (x23 + 4.0 * x9) + 0.08333333333333333 * x30 * x36
    x52 = x43**2
    x53 = 0.5 * x0
    x54 = x0**1.5 * x38
    x55 = x54 * (x52 + x53)
    x56 = x44 * x50
    x57 = x39 * x56
    x58 = x50**2
    x59 = x54 * (x53 + x58)
    x60 = x42 + A[1]
    x61 = x35 * x46
    x62 = x43 * x60
    x63 = x54 * (x53 + x62)
    x64 = 0.25 * x34
    x65 = x39 * x64
    x66 = -2.0 * x41
    x67 = x66 + B[1]
    x68 = x0 * (x67 + A[1])
    x69 = 2.0 * x62
    x70 = x0 + x69
    x71 = x43 * x70 + x68
    x72 = 0.08333333333333333 * x54
    x73 = x44 * x72
    x74 = x30 * x73
    x75 = 0.5 * x30
    x76 = 0.1666666666666667 * x59
    x77 = x30 * x44
    x78 = x49 + A[2]
    x79 = x50 * x78
    x80 = x53 + x79
    x81 = x54 * x80
    x82 = 0.1666666666666667 * x55
    x83 = -2.0 * x48
    x84 = x83 + B[2]
    x85 = x0 * (x84 + A[2])
    x86 = 2.0 * x79
    x87 = x0 + x86
    x88 = x50 * x87 + x85
    x89 = x53 + x60**2
    x90 = x27 * x72
    x91 = x60 * x70 + x68
    x92 = x25 * x73
    x93 = x50 * x54
    x94 = 0.1666666666666667 * x44
    x95 = x93 * x94
    x96 = 2.0 * x60
    x97 = x0 * (x1 + 2.0 * x52 + 4.0 * x62) + x71 * x96
    x98 = x22 + x53
    x99 = x72 * x98
    x100 = 0.3333333333333333 * x98
    x101 = x46 * x78
    x102 = 0.5 * x25
    x103 = x54 * x78
    x104 = x103 * x94
    x105 = 0.1666666666666667 * x54
    x106 = x105 * x44
    x107 = x106 * x98
    x108 = x53 + x78**2
    x109 = x105 * x108
    x110 = x109 * x44
    x111 = x78 * x87 + x85
    x112 = 2.0 * x78
    x113 = x0 * (x1 + 2.0 * x58 + 4.0 * x79) + x112 * x88
    x114 = x42 + R[1]
    x115 = x18 * x5
    x116 = x0 * (x14 + A[0] + B[0])
    x117 = x0 + x7
    x118 = x116 + x117 * x5
    x119 = x13 + x20 * x26
    x120 = x45 * (x0 * (x115 + x118 + x31) + x119 * x4)
    x121 = x114 * x43
    x122 = x121 + x53
    x123 = x13 + x33 * x4
    x124 = x123 * x73
    x125 = x114 * x45
    x126 = x67 + R[1]
    x127 = x0 * x126
    x128 = 2.0 * x121
    x129 = x0 + x128
    x130 = x129 * x43
    x131 = x127 + x130
    x132 = x28 + x29 * x4
    x133 = x132 * x72
    x134 = x114 * x60
    x135 = x134 + x53
    x136 = x119 * x73
    x137 = x129 * x60
    x138 = x127 + x137
    x139 = 0.25 * x54
    x140 = x139 * x20
    x141 = 0.5 * x20
    x142 = 2.0 * x134
    x143 = x0 * (x1 + x128 + x142 + x69)
    x144 = 2.0 * x43
    x145 = x138 * x144 + x143
    x146 = x53 + x9
    x147 = x146 * x73
    x148 = 0.5 * x138
    x149 = 0.3333333333333333 * x44
    x150 = x146 * x149
    x151 = x44 * x78
    x152 = x106 * x88
    x153 = x0 * (x66 + A[1] + R[1])
    x154 = x0 + x142
    x155 = x153 + x154 * x60
    x156 = x115 + x17
    x157 = x156 * x72
    x158 = x138 * x96 + x143
    x159 = x11 + x53
    x160 = x159 * x73
    x161 = x1 * x126 + 2.0 * x137
    x162 = x0 * (x130 + x161 + x71) + x145 * x60
    x163 = x45 * x8
    x164 = x149 * x159 * x54
    x165 = x106 * x111
    x166 = x49 + R[2]
    x167 = x166 * x45
    x168 = x43 * x44
    x169 = x166 * x50
    x170 = x169 + x53
    x171 = x106 * x170
    x172 = x84 + R[2]
    x173 = x0 * x172
    x174 = 2.0 * x169
    x175 = x0 + x174
    x176 = x175 * x50
    x177 = x173 + x176
    x178 = x44 * x60
    x179 = x141 * x54
    x180 = x106 * x146
    x181 = x166 * x78
    x182 = x181 + x53
    x183 = x175 * x78
    x184 = x173 + x183
    x185 = 0.5 * x184
    x186 = x185 * x54
    x187 = 2.0 * x181
    x188 = x0 * (x1 + x174 + x187 + x86)
    x189 = 2.0 * x50
    x190 = x184 * x189 + x188
    x191 = x105 * x89
    x192 = x106 * x159
    x193 = x106 * x182
    x194 = x0 * (x83 + A[2] + R[2])
    x195 = x0 + x187
    x196 = x194 + x195 * x78
    x197 = x112 * x184 + x188
    x198 = x1 * x172 + 2.0 * x183
    x199 = x0 * (x176 + x198 + x88) + x190 * x78
    x200 = x5**2
    x201 = x0 * (x1 + 2.0 * x200 + 4.0 * x6) + x118 * x36
    x202 = x114**2
    x203 = x202 + x53
    x204 = x203 * x72
    x205 = x116 + x117 * x4
    x206 = x114 * x129
    x207 = x127 + x206
    x208 = x207 * x73
    x209 = x4**2 + x53
    x210 = x1 + 2.0 * x202
    x211 = x0 * (4.0 * x121 + x210) + x144 * x207
    x212 = x211 * x72
    x213 = 0.3333333333333333 * x203
    x214 = x114 * x154 + x153
    x215 = x214 * x73
    x216 = x114 * x138
    x217 = x143 + 2.0 * x216
    x218 = x53 + x6
    x219 = x139 * x218
    x220 = x218 * x54
    x221 = 0.5 * x220
    x222 = x0 * (x161 + x206 + x214)
    x223 = x217 * x43 + x222
    x224 = x223 * x46
    x225 = 0.25 * x217
    x226 = x225 * x39
    x227 = x4 * x44
    x228 = 0.5 * x81
    x229 = x0 * (4.0 * x134 + x210) + x214 * x96
    x230 = x200 + x53
    x231 = x230 * x72
    x232 = x46 * (x217 * x60 + x222)
    x233 = 0.08333333333333333 * x229
    x234 = x230 * x54
    x235 = x106 * x166
    x236 = x105 * x209
    x237 = x148 * x54
    x238 = x106 * x4
    x239 = x105 * x230
    x240 = x44 * x5
    x241 = x106 * x5
    x242 = x166**2
    x243 = x242 + x53
    x244 = x243 * x72
    x245 = x106 * x243
    x246 = x166 * x175
    x247 = x173 + x246
    x248 = x247 * x73
    x249 = 0.3333333333333333 * x243
    x250 = x247 * x44
    x251 = x1 + 2.0 * x242
    x252 = x0 * (4.0 * x169 + x251) + x189 * x247
    x253 = x252 * x72
    x254 = 0.5 * x63
    x255 = x166 * x195 + x194
    x256 = x255 * x73
    x257 = x166 * x184
    x258 = x188 + 2.0 * x257
    x259 = 0.25 * x258
    x260 = x259 * x39
    x261 = x0 * (x198 + x246 + x255)
    x262 = x258 * x50 + x261
    x263 = x262 * x46
    x264 = x0 * (4.0 * x181 + x251) + x112 * x255
    x265 = 0.08333333333333333 * x264
    x266 = x46 * (x258 * x78 + x261)

    # 216 item(s)
    result[0, 0, 0] = numpy.sum(
        x40 * (x0 * (4.0 * x13 + x21 * x5 + x21 * x8 + x27) + x35 * x36)
    )
    result[0, 0, 1] = numpy.sum(x43 * x47)
    result[0, 0, 2] = numpy.sum(x47 * x50)
    result[0, 0, 3] = numpy.sum(x51 * x55)
    result[0, 0, 4] = numpy.sum(x43 * x51 * x57)
    result[0, 0, 5] = numpy.sum(x51 * x59)
    result[0, 1, 0] = numpy.sum(x60 * x61)
    result[0, 1, 1] = numpy.sum(x63 * x64)
    result[0, 1, 2] = numpy.sum(x50 * x60 * x65)
    result[0, 1, 3] = numpy.sum(x71 * x74)
    result[0, 1, 4] = numpy.sum(x50 * x63 * x75)
    result[0, 1, 5] = numpy.sum(x60 * x76 * x77)
    result[0, 2, 0] = numpy.sum(x61 * x78)
    result[0, 2, 1] = numpy.sum(x43 * x65 * x78)
    result[0, 2, 2] = numpy.sum(x64 * x81)
    result[0, 2, 3] = numpy.sum(x77 * x78 * x82)
    result[0, 2, 4] = numpy.sum(x43 * x75 * x81)
    result[0, 2, 5] = numpy.sum(x74 * x88)
    result[0, 3, 0] = numpy.sum(x89 * x90)
    result[0, 3, 1] = numpy.sum(x91 * x92)
    result[0, 3, 2] = numpy.sum(x25 * x89 * x95)
    result[0, 3, 3] = numpy.sum(x97 * x99)
    result[0, 3, 4] = numpy.sum(x91 * x95 * x98)
    result[0, 3, 5] = numpy.sum(x100 * x59 * x89)
    result[0, 4, 0] = numpy.sum(x101 * x27 * x60)
    result[0, 4, 1] = numpy.sum(x102 * x63 * x78)
    result[0, 4, 2] = numpy.sum(x102 * x60 * x81)
    result[0, 4, 3] = numpy.sum(x104 * x71 * x98)
    result[0, 4, 4] = numpy.sum(x63 * x80 * x98)
    result[0, 4, 5] = numpy.sum(x107 * x60 * x88)
    result[0, 5, 0] = numpy.sum(x108 * x90)
    result[0, 5, 1] = numpy.sum(x110 * x25 * x43)
    result[0, 5, 2] = numpy.sum(x111 * x92)
    result[0, 5, 3] = numpy.sum(x100 * x108 * x55)
    result[0, 5, 4] = numpy.sum(x107 * x111 * x43)
    result[0, 5, 5] = numpy.sum(x113 * x99)
    result[1, 0, 0] = numpy.sum(x114 * x120)
    result[1, 0, 1] = numpy.sum(x122 * x124)
    result[1, 0, 2] = numpy.sum(x123 * x125 * x56)
    result[1, 0, 3] = numpy.sum(x131 * x133)
    result[1, 0, 4] = numpy.sum(x122 * x132 * x95)
    result[1, 0, 5] = numpy.sum(x114 * x132 * x76)
    result[1, 1, 0] = numpy.sum(x135 * x136)
    result[1, 1, 1] = numpy.sum(x138 * x140)
    result[1, 1, 2] = numpy.sum(x135 * x141 * x93)
    result[1, 1, 3] = numpy.sum(x145 * x147)
    result[1, 1, 4] = numpy.sum(x146 * x148 * x93)
    result[1, 1, 5] = numpy.sum(x135 * x150 * x59)
    result[1, 2, 0] = numpy.sum(x119 * x125 * x151)
    result[1, 2, 1] = numpy.sum(x103 * x122 * x141)
    result[1, 2, 2] = numpy.sum(x114 * x141 * x81)
    result[1, 2, 3] = numpy.sum(x104 * x131 * x146)
    result[1, 2, 4] = numpy.sum(x122 * x146 * x81)
    result[1, 2, 5] = numpy.sum(x114 * x146 * x152)
    result[1, 3, 0] = numpy.sum(x155 * x157)
    result[1, 3, 1] = numpy.sum(x158 * x160)
    result[1, 3, 2] = numpy.sum(x155 * x159 * x95)
    result[1, 3, 3] = numpy.sum(x162 * x163)
    result[1, 3, 4] = numpy.sum(x158 * x163 * x56)
    result[1, 3, 5] = numpy.sum(x155 * x76 * x8)
    result[1, 4, 0] = numpy.sum(x104 * x135 * x156)
    result[1, 4, 1] = numpy.sum(x103 * x148 * x159)
    result[1, 4, 2] = numpy.sum(x135 * x159 * x81)
    result[1, 4, 3] = numpy.sum(x145 * x151 * x163)
    result[1, 4, 4] = numpy.sum(x148 * x8 * x81)
    result[1, 4, 5] = numpy.sum(x135 * x152 * x8)
    result[1, 5, 0] = numpy.sum(x109 * x114 * x156)
    result[1, 5, 1] = numpy.sum(x108 * x122 * x164)
    result[1, 5, 2] = numpy.sum(x114 * x159 * x165)
    result[1, 5, 3] = numpy.sum(x109 * x131 * x8)
    result[1, 5, 4] = numpy.sum(x122 * x165 * x8)
    result[1, 5, 5] = numpy.sum(x113 * x114 * x163)
    result[2, 0, 0] = numpy.sum(x120 * x166)
    result[2, 0, 1] = numpy.sum(x123 * x167 * x168)
    result[2, 0, 2] = numpy.sum(x124 * x170)
    result[2, 0, 3] = numpy.sum(x132 * x166 * x82)
    result[2, 0, 4] = numpy.sum(x132 * x171 * x43)
    result[2, 0, 5] = numpy.sum(x133 * x177)
    result[2, 1, 0] = numpy.sum(x119 * x167 * x178)
    result[2, 1, 1] = numpy.sum(x141 * x166 * x63)
    result[2, 1, 2] = numpy.sum(x170 * x179 * x60)
    result[2, 1, 3] = numpy.sum(x166 * x180 * x71)
    result[2, 1, 4] = numpy.sum(x146 * x170 * x63)
    result[2, 1, 5] = numpy.sum(x177 * x180 * x60)
    result[2, 2, 0] = numpy.sum(x136 * x182)
    result[2, 2, 1] = numpy.sum(x179 * x182 * x43)
    result[2, 2, 2] = numpy.sum(x140 * x184)
    result[2, 2, 3] = numpy.sum(x150 * x182 * x55)
    result[2, 2, 4] = numpy.sum(x146 * x186 * x43)
    result[2, 2, 5] = numpy.sum(x147 * x190)
    result[2, 3, 0] = numpy.sum(x156 * x166 * x191)
    result[2, 3, 1] = numpy.sum(x166 * x192 * x91)
    result[2, 3, 2] = numpy.sum(x164 * x170 * x89)
    result[2, 3, 3] = numpy.sum(x163 * x166 * x97)
    result[2, 3, 4] = numpy.sum(x171 * x8 * x91)
    result[2, 3, 5] = numpy.sum(x177 * x191 * x8)
    result[2, 4, 0] = numpy.sum(x156 * x193 * x60)
    result[2, 4, 1] = numpy.sum(x159 * x182 * x63)
    result[2, 4, 2] = numpy.sum(x159 * x186 * x60)
    result[2, 4, 3] = numpy.sum(x193 * x71 * x8)
    result[2, 4, 4] = numpy.sum(x185 * x63 * x8)
    result[2, 4, 5] = numpy.sum(x163 * x178 * x190)
    result[2, 5, 0] = numpy.sum(x157 * x196)
    result[2, 5, 1] = numpy.sum(x192 * x196 * x43)
    result[2, 5, 2] = numpy.sum(x160 * x197)
    result[2, 5, 3] = numpy.sum(x196 * x8 * x82)
    result[2, 5, 4] = numpy.sum(x163 * x168 * x197)
    result[2, 5, 5] = numpy.sum(x163 * x199)
    result[3, 0, 0] = numpy.sum(x201 * x204)
    result[3, 0, 1] = numpy.sum(x205 * x208)
    result[3, 0, 2] = numpy.sum(x203 * x205 * x95)
    result[3, 0, 3] = numpy.sum(x209 * x212)
    result[3, 0, 4] = numpy.sum(x207 * x209 * x95)
    result[3, 0, 5] = numpy.sum(x209 * x213 * x59)
    result[3, 1, 0] = numpy.sum(x118 * x215)
    result[3, 1, 1] = numpy.sum(x217 * x219)
    result[3, 1, 2] = numpy.sum(x214 * x221 * x50)
    result[3, 1, 3] = numpy.sum(x224 * x4)
    result[3, 1, 4] = numpy.sum(x226 * x4 * x50)
    result[3, 1, 5] = numpy.sum(x214 * x227 * x76)
    result[3, 2, 0] = numpy.sum(x104 * x118 * x203)
    result[3, 2, 1] = numpy.sum(x207 * x221 * x78)
    result[3, 2, 2] = numpy.sum(x203 * x218 * x81)
    result[3, 2, 3] = numpy.sum(x101 * x211 * x4)
    result[3, 2, 4] = numpy.sum(x207 * x228 * x4)
    result[3, 2, 5] = numpy.sum(x152 * x203 * x4)
    result[3, 3, 0] = numpy.sum(x229 * x231)
    result[3, 3, 1] = numpy.sum(x232 * x5)
    result[3, 3, 2] = numpy.sum(x233 * x5 * x57)
    result[3, 3, 3] = numpy.sum(
        x40 * (x0 * (4.0 * x138 * x43 + 4.0 * x143 + x211 + 4.0 * x216) + x223 * x96)
    )
    result[3, 3, 4] = numpy.sum(x232 * x50)
    result[3, 3, 5] = numpy.sum(x233 * x59)
    result[3, 4, 0] = numpy.sum(x104 * x214 * x230)
    result[3, 4, 1] = numpy.sum(x226 * x5 * x78)
    result[3, 4, 2] = numpy.sum(x214 * x228 * x5)
    result[3, 4, 3] = numpy.sum(x224 * x78)
    result[3, 4, 4] = numpy.sum(x225 * x81)
    result[3, 4, 5] = numpy.sum(x215 * x88)
    result[3, 5, 0] = numpy.sum(x108 * x213 * x234)
    result[3, 5, 1] = numpy.sum(x110 * x207 * x5)
    result[3, 5, 2] = numpy.sum(x165 * x203 * x5)
    result[3, 5, 3] = numpy.sum(x108 * x212)
    result[3, 5, 4] = numpy.sum(x111 * x208)
    result[3, 5, 5] = numpy.sum(x113 * x204)
    result[4, 0, 0] = numpy.sum(x114 * x167 * x201)
    result[4, 0, 1] = numpy.sum(x122 * x205 * x235)
    result[4, 0, 2] = numpy.sum(x114 * x171 * x205)
    result[4, 0, 3] = numpy.sum(x131 * x166 * x236)
    result[4, 0, 4] = numpy.sum(x122 * x149 * x170 * x209 * x54)
    result[4, 0, 5] = numpy.sum(x114 * x177 * x236)
    result[4, 1, 0] = numpy.sum(x118 * x135 * x235)
    result[4, 1, 1] = numpy.sum(x148 * x166 * x220)
    result[4, 1, 2] = numpy.sum(x135 * x170 * x220)
    result[4, 1, 3] = numpy.sum(x145 * x167 * x227)
    result[4, 1, 4] = numpy.sum(x170 * x237 * x4)
    result[4, 1, 5] = numpy.sum(x135 * x177 * x238)
    result[4, 2, 0] = numpy.sum(x114 * x118 * x193)
    result[4, 2, 1] = numpy.sum(x122 * x182 * x220)
    result[4, 2, 2] = numpy.sum(x114 * x185 * x220)
    result[4, 2, 3] = numpy.sum(x131 * x193 * x4)
    result[4, 2, 4] = numpy.sum(x122 * x186 * x4)
    result[4, 2, 5] = numpy.sum(x125 * x190 * x227)
    result[4, 3, 0] = numpy.sum(x155 * x166 * x239)
    result[4, 3, 1] = numpy.sum(x158 * x167 * x240)
    result[4, 3, 2] = numpy.sum(x155 * x171 * x5)
    result[4, 3, 3] = numpy.sum(x162 * x167)
    result[4, 3, 4] = numpy.sum(x158 * x170 * x73)
    result[4, 3, 5] = numpy.sum(x155 * x177 * x72)
    result[4, 4, 0] = numpy.sum(x135 * x149 * x182 * x234)
    result[4, 4, 1] = numpy.sum(x182 * x237 * x5)
    result[4, 4, 2] = numpy.sum(x135 * x186 * x5)
    result[4, 4, 3] = numpy.sum(x145 * x182 * x73)
    result[4, 4, 4] = numpy.sum(x138 * x139 * x184)
    result[4, 4, 5] = numpy.sum(x135 * x190 * x73)
    result[4, 5, 0] = numpy.sum(x114 * x196 * x239)
    result[4, 5, 1] = numpy.sum(x122 * x196 * x241)
    result[4, 5, 2] = numpy.sum(x125 * x197 * x240)
    result[4, 5, 3] = numpy.sum(x131 * x196 * x72)
    result[4, 5, 4] = numpy.sum(x122 * x197 * x73)
    result[4, 5, 5] = numpy.sum(x125 * x199)
    result[5, 0, 0] = numpy.sum(x201 * x244)
    result[5, 0, 1] = numpy.sum(x205 * x245 * x43)
    result[5, 0, 2] = numpy.sum(x205 * x248)
    result[5, 0, 3] = numpy.sum(x209 * x249 * x55)
    result[5, 0, 4] = numpy.sum(x236 * x250 * x43)
    result[5, 0, 5] = numpy.sum(x209 * x253)
    result[5, 1, 0] = numpy.sum(x118 * x245 * x60)
    result[5, 1, 1] = numpy.sum(x218 * x243 * x63)
    result[5, 1, 2] = numpy.sum(x221 * x247 * x60)
    result[5, 1, 3] = numpy.sum(x238 * x243 * x71)
    result[5, 1, 4] = numpy.sum(x247 * x254 * x4)
    result[5, 1, 5] = numpy.sum(x252 * x4 * x46 * x60)
    result[5, 2, 0] = numpy.sum(x118 * x256)
    result[5, 2, 1] = numpy.sum(x221 * x255 * x43)
    result[5, 2, 2] = numpy.sum(x219 * x258)
    result[5, 2, 3] = numpy.sum(x227 * x255 * x82)
    result[5, 2, 4] = numpy.sum(x260 * x4 * x43)
    result[5, 2, 5] = numpy.sum(x263 * x4)
    result[5, 3, 0] = numpy.sum(x234 * x249 * x89)
    result[5, 3, 1] = numpy.sum(x241 * x243 * x91)
    result[5, 3, 2] = numpy.sum(x191 * x250 * x5)
    result[5, 3, 3] = numpy.sum(x244 * x97)
    result[5, 3, 4] = numpy.sum(x248 * x91)
    result[5, 3, 5] = numpy.sum(x253 * x89)
    result[5, 4, 0] = numpy.sum(x178 * x239 * x255)
    result[5, 4, 1] = numpy.sum(x254 * x255 * x5)
    result[5, 4, 2] = numpy.sum(x260 * x5 * x60)
    result[5, 4, 3] = numpy.sum(x256 * x71)
    result[5, 4, 4] = numpy.sum(x259 * x63)
    result[5, 4, 5] = numpy.sum(x263 * x60)
    result[5, 5, 0] = numpy.sum(x231 * x264)
    result[5, 5, 1] = numpy.sum(x168 * x265 * x39 * x5)
    result[5, 5, 2] = numpy.sum(x266 * x5)
    result[5, 5, 3] = numpy.sum(x265 * x55)
    result[5, 5, 4] = numpy.sum(x266 * x43)
    result[5, 5, 5] = numpy.sum(
        x40 * (x0 * (4.0 * x184 * x50 + 4.0 * x188 + x252 + 4.0 * x257) + x112 * x262)
    )
    return result


def quadrupole3d_23(ax, da, A, bx, db, B, R):
    """Cartesian 3D (df) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 6, 10), dtype=float)

    x0 = (ax + bx) ** (-1.0)
    x1 = x0 * (ax * A[0] + bx * B[0])
    x2 = -x1
    x3 = x2 + B[0]
    x4 = x2 + R[0]
    x5 = -2.0 * x1
    x6 = x5 + R[0]
    x7 = x6 + B[0]
    x8 = x0 * x7
    x9 = x2 + A[0]
    x10 = x3 * x4
    x11 = 2.0 * x10
    x12 = x0 + x11
    x13 = x12 * x9
    x14 = x13 + x8
    x15 = 4.0 * x14
    x16 = 4.0 * x10
    x17 = x4**2
    x18 = 3.0 * x0
    x19 = 2.0 * x17 + x18
    x20 = x12 * x4
    x21 = x20 + x8
    x22 = 2.0 * x3
    x23 = x0 * (x16 + x19) + x21 * x22
    x24 = x3 * x9
    x25 = 2.0 * x24
    x26 = x4 * x9
    x27 = 2.0 * x26
    x28 = x0 * (x11 + x18 + x25 + x27)
    x29 = x15 * x3 + 4.0 * x28
    x30 = x0 * (x15 * x4 + x23 + x29)
    x31 = x0 * (x6 + A[0])
    x32 = x0 + x27
    x33 = x31 + x32 * x4
    x34 = 2.0 * x13 + x18 * x7
    x35 = x0 * (x20 + x33 + x34)
    x36 = 2.0 * x14
    x37 = x28 + x36 * x4
    x38 = x3 * x37
    x39 = x35 + x38
    x40 = 2.0 * x9
    x41 = x30 + x39 * x40
    x42 = x37 * x9
    x43 = x12 * x3
    x44 = x0 * (x5 + A[0] + B[0])
    x45 = x0 + x25
    x46 = x3 * x45
    x47 = x44 + x46
    x48 = x0 * (x34 + x43 + x47)
    x49 = x14 * x22 + x28
    x50 = x48 + x49 * x9
    x51 = 2.0 * x0
    x52 = 2.23606797749979
    x53 = 0.008333333333333333 * x52
    x54 = ax * bx * x0
    x55 = (
        5.568327996831708
        * da
        * db
        * numpy.exp(-x54 * ((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2 + (A[2] - B[2]) ** 2))
    )
    x56 = x0**1.5 * x55
    x57 = x53 * x56
    x58 = x0 * (ax * A[1] + bx * B[1])
    x59 = -x58
    x60 = x59 + B[1]
    x61 = 0.04166666666666667 * x56
    x62 = x41 * x61
    x63 = x0 * (ax * A[2] + bx * B[2])
    x64 = -x63
    x65 = x64 + B[2]
    x66 = x60**2
    x67 = 0.5 * x0
    x68 = x66 + x67
    x69 = x35 + x42
    x70 = x0**1.5 * x55
    x71 = 0.08333333333333333 * x70
    x72 = x69 * x71
    x73 = 0.08333333333333333 * x56
    x74 = 1.732050807568877
    x75 = x65 * x74
    x76 = x73 * x75
    x77 = x65**2
    x78 = x67 + x77
    x79 = x0 * (x19 + 4.0 * x26) + x33 * x40
    x80 = 0.01666666666666667 * x52
    x81 = x79 * x80
    x82 = 1.5 * x0
    x83 = x66 + x82
    x84 = x60 * x70
    x85 = x83 * x84
    x86 = x71 * x79
    x87 = x77 + x82
    x88 = x65 * x70
    x89 = x87 * x88
    x90 = x59 + A[1]
    x91 = x56 * x90
    x92 = 3.872983346207417
    x93 = 0.008333333333333333 * x92
    x94 = x93 * (x22 * x39 + x30)
    x95 = x60 * x90
    x96 = x67 + x95
    x97 = x71 * x74
    x98 = x39 * x97
    x99 = 0.08333333333333333 * x39
    x100 = -2.0 * x58
    x101 = x100 + B[1]
    x102 = x0 * (x101 + A[1])
    x103 = 2.0 * x95
    x104 = x0 + x103
    x105 = x104 * x60
    x106 = x102 + x105
    x107 = 0.04166666666666667 * x70
    x108 = x107 * x74
    x109 = x108 * x37
    x110 = x70 * x96
    x111 = 0.25 * x37
    x112 = x37 * x97
    x113 = x18 + 2.0 * x66
    x114 = x0 * (x113 + 4.0 * x95)
    x115 = 2.0 * x106
    x116 = x114 + x115 * x60
    x117 = x70 * x92
    x118 = 0.008333333333333333 * x117
    x119 = x118 * x33
    x120 = x33 * x97
    x121 = 0.1666666666666667 * x78
    x122 = x33 * x74
    x123 = x65 * x87
    x124 = 0.03333333333333333 * x117
    x125 = x124 * x33
    x126 = x64 + A[2]
    x127 = x126 * x56
    x128 = x127 * x74
    x129 = x126 * x65
    x130 = x129 + x67
    x131 = x130 * x70
    x132 = -2.0 * x63
    x133 = x132 + B[2]
    x134 = x0 * (x133 + A[2])
    x135 = 2.0 * x129
    x136 = x0 + x135
    x137 = x136 * x65
    x138 = x134 + x137
    x139 = x60 * x83
    x140 = 0.1666666666666667 * x68
    x141 = x18 + 2.0 * x77
    x142 = x0 * (4.0 * x129 + x141)
    x143 = 2.0 * x138
    x144 = x142 + x143 * x65
    x145 = 0.01666666666666667 * x23 * x3 + 0.01666666666666667 * x51 * (
        x20 + x43 + 2.0 * x8
    )
    x146 = x67 + x90**2
    x147 = x146 * x70
    x148 = x147 * x52
    x149 = x104 * x90
    x150 = x102 + x149
    x151 = x107 * x23
    x152 = x65 * x71
    x153 = x114 + x115 * x90
    x154 = x107 * x21
    x155 = x21 * x97
    x156 = x17 + x67
    x157 = x156 * x70
    x158 = x80 * (x153 * x60 + x51 * (2.0 * x102 + x105 + x149))
    x159 = 0.06666666666666667 * x123
    x160 = x91 * x92
    x161 = x23 * x97
    x162 = 0.5 * x110
    x163 = 0.01666666666666667 * x117
    x164 = x156 * x163
    x165 = 0.1666666666666667 * x74
    x166 = x156 * x165
    x167 = x126**2 + x67
    x168 = x167 * x70
    x169 = x168 * x52
    x170 = x60 * x71
    x171 = x126 * x136
    x172 = x134 + x171
    x173 = x126 * x143 + x142
    x174 = 0.06666666666666667 * x139
    x175 = x174 * x52
    x176 = x80 * (x173 * x65 + x51 * (2.0 * x134 + x137 + x171))
    x177 = x59 + R[1]
    x178 = x3**2
    x179 = 2.0 * x178 + x18
    x180 = x0 * (x179 + 4.0 * x24)
    x181 = x180 + x40 * x47
    x182 = x57 * (x0 * (x15 * x9 + x181 + x29) + x22 * x50)
    x183 = x177 * x60
    x184 = x183 + x67
    x185 = x50 * x71
    x186 = x177 * x65
    x187 = x50 * x73
    x188 = x101 + R[1]
    x189 = x0 * x188
    x190 = 2.0 * x183
    x191 = x0 + x190
    x192 = x191 * x60
    x193 = x189 + x192
    x194 = x28 + x36 * x9
    x195 = x107 * x194
    x196 = x184 * x71
    x197 = x196 * x74
    x198 = x71 * x78
    x199 = 4.0 * x183
    x200 = 2.0 * x60
    x201 = x0 * (x113 + x199) + x193 * x200
    x202 = x31 + x32 * x9
    x203 = x202 * x70
    x204 = x203 * x53
    x205 = 0.03333333333333333 * x52
    x206 = x203 * x205
    x207 = x177 * x90
    x208 = x207 + x67
    x209 = x3 * x49 + x48
    x210 = x163 * x209
    x211 = x191 * x90
    x212 = x189 + x211
    x213 = x108 * x49
    x214 = x65 * x97
    x215 = 2.0 * x207
    x216 = x0 * (x103 + x18 + x190 + x215)
    x217 = x200 * x212 + x216
    x218 = x108 * x14
    x219 = x212 * x70
    x220 = 0.25 * x14
    x221 = x121 * x74
    x222 = x208 * x70
    x223 = x18 * x188 + 2.0 * x211
    x224 = x0 * (x106 + x192 + x223)
    x225 = x217 * x60 + x224
    x226 = x26 + x67
    x227 = x163 * x226
    x228 = x117 * x226
    x229 = 0.01666666666666667 * x209
    x230 = x127 * x92
    x231 = x177 * x97
    x232 = x126 * x97
    x233 = 0.5 * x131
    x234 = x165 * x226
    x235 = x184 * x70
    x236 = x0 * (x100 + A[1] + R[1])
    x237 = x0 + x215
    x238 = x236 + x237 * x90
    x239 = x238 * x70
    x240 = x43 + x8
    x241 = x0 * (x16 + x179) + x22 * x240
    x242 = x241 * x53
    x243 = x212 * x90
    x244 = x216 + 2.0 * x243
    x245 = x107 * x240
    x246 = x217 * x90 + x224
    x247 = x10 + x67
    x248 = x247 * x71
    x249 = x248 * x74
    x250 = 4.0 * x212
    x251 = 4.0 * x216 + x250 * x60
    x252 = x0 * (x153 + 4.0 * x243 + x251) + x200 * x246
    x253 = x4 * x57
    x254 = x4 * x73
    x255 = x205 * x4
    x256 = x163 * x208
    x257 = x165 * x240
    x258 = x165 * x247
    x259 = 0.01666666666666667 * x4
    x260 = x4 * x97
    x261 = x168 * x80
    x262 = 0.1666666666666667 * x168
    x263 = x172 * x71
    x264 = x4 * x56
    x265 = x64 + R[2]
    x266 = x265 * x60
    x267 = x265 * x65
    x268 = x267 + x67
    x269 = x265 * x71
    x270 = x268 * x71
    x271 = x270 * x74
    x272 = x133 + R[2]
    x273 = x0 * x272
    x274 = 2.0 * x267
    x275 = x0 + x274
    x276 = x275 * x65
    x277 = x273 + x276
    x278 = 4.0 * x267
    x279 = 2.0 * x65
    x280 = x0 * (x141 + x278) + x277 * x279
    x281 = x265 * x97
    x282 = x90 * x97
    x283 = x268 * x70
    x284 = x126 * x265
    x285 = x284 + x67
    x286 = x60 * x97
    x287 = x126 * x275
    x288 = x273 + x287
    x289 = x140 * x74
    x290 = x285 * x70
    x291 = x288 * x70
    x292 = 2.0 * x284
    x293 = x0 * (x135 + x18 + x274 + x292)
    x294 = x279 * x288 + x293
    x295 = x18 * x272 + 2.0 * x287
    x296 = x0 * (x138 + x276 + x295)
    x297 = x294 * x65 + x296
    x298 = 0.01666666666666667 * x148
    x299 = 0.1666666666666667 * x147
    x300 = x4 * x71
    x301 = x163 * x285
    x302 = x0 * (x132 + A[2] + R[2])
    x303 = x0 + x292
    x304 = x126 * x303 + x302
    x305 = x304 * x70
    x306 = x126 * x288
    x307 = x293 + 2.0 * x306
    x308 = x126 * x294 + x296
    x309 = 4.0 * x288
    x310 = 4.0 * x293 + x309 * x65
    x311 = x0 * (x173 + 4.0 * x306 + x310) + x279 * x308
    x312 = x177**2
    x313 = x312 + x67
    x314 = x313 * x70
    x315 = x45 * x9
    x316 = x80 * (x181 * x3 + x51 * (x315 + 2.0 * x44 + x46))
    x317 = x177 * x191
    x318 = x189 + x317
    x319 = x107 * x318
    x320 = x315 + x44
    x321 = x18 + 2.0 * x312
    x322 = x0 * (x199 + x321) + x200 * x318
    x323 = x107 * x322
    x324 = x67 + x9**2
    x325 = x324 * x70
    x326 = x322 * x60 + x51 * (2.0 * x189 + x192 + x317)
    x327 = x326 * x80
    x328 = x180 + x22 * x47
    x329 = x177 * x237 + x236
    x330 = x118 * x329
    x331 = 2.0 * x177 * x212 + x216
    x332 = x108 * x331
    x333 = x24 + x67
    x334 = x0 * (x223 + x317 + x329)
    x335 = x331 * x60
    x336 = x334 + x335
    x337 = x336 * x97
    x338 = 0.25 * x331
    x339 = x333 * x70
    x340 = x0 * (x177 * x250 + x251 + x322)
    x341 = x93 * (x200 * x336 + x340)
    x342 = x56 * x9
    x343 = x9 * x97
    x344 = x124 * x329
    x345 = x163 * x313
    x346 = x165 * x47
    x347 = x165 * x333
    x348 = 0.01666666666666667 * x9
    x349 = 2.0 * x90
    x350 = x0 * (4.0 * x207 + x321) + x329 * x349
    x351 = x3 * x350
    x352 = x178 + x82
    x353 = x352 * x70 * x80
    x354 = x178 + x67
    x355 = x331 * x90
    x356 = x334 + x355
    x357 = x356 * x71
    x358 = x336 * x349 + x340
    x359 = x358 * x61
    x360 = x3 * x73
    x361 = x3 * x352
    x362 = 0.1666666666666667 * x354
    x363 = x362 * x74
    x364 = 0.08333333333333333 * x3
    x365 = x3 * x97
    x366 = 0.06666666666666667 * x361
    x367 = x3 * x71
    x368 = x177 * x71
    x369 = x325 * x80
    x370 = 0.1666666666666667 * x325
    x371 = 0.5 * x333
    x372 = 0.01666666666666667 * x342 * x92
    x373 = x205 * x361
    x374 = 0.25 * x3
    x375 = x265**2
    x376 = x375 + x67
    x377 = x376 * x70
    x378 = x265 * x275
    x379 = x273 + x378
    x380 = x107 * x379
    x381 = x18 + 2.0 * x375
    x382 = x0 * (x278 + x381) + x279 * x379
    x383 = x107 * x382
    x384 = x382 * x65 + x51 * (2.0 * x273 + x276 + x378)
    x385 = x163 * x376
    x386 = x265 * x303 + x302
    x387 = x118 * x386
    x388 = 2.0 * x265 * x288 + x293
    x389 = x108 * x388
    x390 = x0 * (x295 + x378 + x386)
    x391 = x388 * x65
    x392 = x390 + x391
    x393 = x392 * x97
    x394 = x124 * x386
    x395 = x392 * x74
    x396 = x0 * (x265 * x309 + x310 + x382)
    x397 = x93 * (x279 * x392 + x396)
    x398 = 2.0 * x126
    x399 = x0 * (4.0 * x284 + x381) + x386 * x398
    x400 = x126 * x388
    x401 = x390 + x400
    x402 = x401 * x71
    x403 = x392 * x398 + x396
    x404 = x403 * x61

    # 360 item(s)
    result[0, 0, 0] = numpy.sum(-x57 * (x3 * x41 + x51 * (2.0 * x35 + x38 + x42 + x50)))
    result[0, 0, 1] = numpy.sum(-x60 * x62)
    result[0, 0, 2] = numpy.sum(-x62 * x65)
    result[0, 0, 3] = numpy.sum(-x68 * x72)
    result[0, 0, 4] = numpy.sum(-x60 * x69 * x76)
    result[0, 0, 5] = numpy.sum(-x72 * x78)
    result[0, 0, 6] = numpy.sum(-x81 * x85)
    result[0, 0, 7] = numpy.sum(-x65 * x68 * x86)
    result[0, 0, 8] = numpy.sum(-x60 * x78 * x86)
    result[0, 0, 9] = numpy.sum(-x81 * x89)
    result[0, 1, 0] = numpy.sum(-x91 * x94)
    result[0, 1, 1] = numpy.sum(-x96 * x98)
    result[0, 1, 2] = numpy.sum(-x75 * x91 * x99)
    result[0, 1, 3] = numpy.sum(-x106 * x109)
    result[0, 1, 4] = numpy.sum(-x110 * x111 * x65)
    result[0, 1, 5] = numpy.sum(-x112 * x78 * x90)
    result[0, 1, 6] = numpy.sum(-x116 * x119)
    result[0, 1, 7] = numpy.sum(-x106 * x120 * x65)
    result[0, 1, 8] = numpy.sum(-x110 * x121 * x122)
    result[0, 1, 9] = numpy.sum(-x123 * x125 * x90)
    result[0, 2, 0] = numpy.sum(-x127 * x94)
    result[0, 2, 1] = numpy.sum(-x128 * x60 * x99)
    result[0, 2, 2] = numpy.sum(-x130 * x98)
    result[0, 2, 3] = numpy.sum(-x112 * x126 * x68)
    result[0, 2, 4] = numpy.sum(-x111 * x131 * x60)
    result[0, 2, 5] = numpy.sum(-x109 * x138)
    result[0, 2, 6] = numpy.sum(-x125 * x126 * x139)
    result[0, 2, 7] = numpy.sum(-x122 * x131 * x140)
    result[0, 2, 8] = numpy.sum(-x120 * x138 * x60)
    result[0, 2, 9] = numpy.sum(-x119 * x144)
    result[0, 3, 0] = numpy.sum(-x145 * x148)
    result[0, 3, 1] = numpy.sum(-x150 * x151)
    result[0, 3, 2] = numpy.sum(-x146 * x152 * x23)
    result[0, 3, 3] = numpy.sum(-x153 * x154)
    result[0, 3, 4] = numpy.sum(-x150 * x155 * x65)
    result[0, 3, 5] = numpy.sum(-x121 * x147 * x21)
    result[0, 3, 6] = numpy.sum(-x157 * x158)
    result[0, 3, 7] = numpy.sum(-x152 * x153 * x156)
    result[0, 3, 8] = numpy.sum(-x121 * x150 * x157)
    result[0, 3, 9] = numpy.sum(-x148 * x156 * x159)
    result[0, 4, 0] = numpy.sum(-x126 * x145 * x160)
    result[0, 4, 1] = numpy.sum(-x126 * x161 * x96)
    result[0, 4, 2] = numpy.sum(-x130 * x161 * x90)
    result[0, 4, 3] = numpy.sum(-x106 * x126 * x155)
    result[0, 4, 4] = numpy.sum(-x130 * x162 * x21)
    result[0, 4, 5] = numpy.sum(-x138 * x155 * x90)
    result[0, 4, 6] = numpy.sum(-x116 * x126 * x164)
    result[0, 4, 7] = numpy.sum(-x106 * x131 * x166)
    result[0, 4, 8] = numpy.sum(-x110 * x138 * x166)
    result[0, 4, 9] = numpy.sum(-x144 * x164 * x90)
    result[0, 5, 0] = numpy.sum(-x145 * x169)
    result[0, 5, 1] = numpy.sum(-x167 * x170 * x23)
    result[0, 5, 2] = numpy.sum(-x151 * x172)
    result[0, 5, 3] = numpy.sum(-x140 * x168 * x21)
    result[0, 5, 4] = numpy.sum(-x155 * x172 * x60)
    result[0, 5, 5] = numpy.sum(-x154 * x173)
    result[0, 5, 6] = numpy.sum(-x157 * x167 * x175)
    result[0, 5, 7] = numpy.sum(-x140 * x157 * x172)
    result[0, 5, 8] = numpy.sum(-x156 * x170 * x173)
    result[0, 5, 9] = numpy.sum(-x157 * x176)
    result[1, 0, 0] = numpy.sum(-x177 * x182)
    result[1, 0, 1] = numpy.sum(-x184 * x185)
    result[1, 0, 2] = numpy.sum(-x186 * x187)
    result[1, 0, 3] = numpy.sum(-x193 * x195)
    result[1, 0, 4] = numpy.sum(-x194 * x197 * x65)
    result[1, 0, 5] = numpy.sum(-x177 * x194 * x198)
    result[1, 0, 6] = numpy.sum(-x201 * x204)
    result[1, 0, 7] = numpy.sum(-x152 * x193 * x202)
    result[1, 0, 8] = numpy.sum(-x121 * x184 * x203)
    result[1, 0, 9] = numpy.sum(-x186 * x206 * x87)
    result[1, 1, 0] = numpy.sum(-x208 * x210)
    result[1, 1, 1] = numpy.sum(-x212 * x213)
    result[1, 1, 2] = numpy.sum(-x208 * x214 * x49)
    result[1, 1, 3] = numpy.sum(-x217 * x218)
    result[1, 1, 4] = numpy.sum(-x219 * x220 * x65)
    result[1, 1, 5] = numpy.sum(-x14 * x221 * x222)
    result[1, 1, 6] = numpy.sum(-x225 * x227)
    result[1, 1, 7] = numpy.sum(-x214 * x217 * x226)
    result[1, 1, 8] = numpy.sum(-x219 * x221 * x226)
    result[1, 1, 9] = numpy.sum(-x159 * x208 * x228)
    result[1, 2, 0] = numpy.sum(-x177 * x229 * x230)
    result[1, 2, 1] = numpy.sum(-x126 * x197 * x49)
    result[1, 2, 2] = numpy.sum(-x130 * x231 * x49)
    result[1, 2, 3] = numpy.sum(-x14 * x193 * x232)
    result[1, 2, 4] = numpy.sum(-x14 * x184 * x233)
    result[1, 2, 5] = numpy.sum(-x138 * x14 * x231)
    result[1, 2, 6] = numpy.sum(-x126 * x201 * x227)
    result[1, 2, 7] = numpy.sum(-x131 * x193 * x234)
    result[1, 2, 8] = numpy.sum(-x138 * x234 * x235)
    result[1, 2, 9] = numpy.sum(-x144 * x177 * x227)
    result[1, 3, 0] = numpy.sum(-x239 * x242)
    result[1, 3, 1] = numpy.sum(-x244 * x245)
    result[1, 3, 2] = numpy.sum(-x152 * x238 * x240)
    result[1, 3, 3] = numpy.sum(-x246 * x248)
    result[1, 3, 4] = numpy.sum(-x244 * x249 * x65)
    result[1, 3, 5] = numpy.sum(-x121 * x239 * x247)
    result[1, 3, 6] = numpy.sum(-x252 * x253)
    result[1, 3, 7] = numpy.sum(-x246 * x254 * x65)
    result[1, 3, 8] = numpy.sum(-x198 * x244 * x4)
    result[1, 3, 9] = numpy.sum(-x123 * x239 * x255)
    result[1, 4, 0] = numpy.sum(-x126 * x241 * x256)
    result[1, 4, 1] = numpy.sum(-x212 * x232 * x240)
    result[1, 4, 2] = numpy.sum(-x131 * x208 * x257)
    result[1, 4, 3] = numpy.sum(-x126 * x217 * x249)
    result[1, 4, 4] = numpy.sum(-x212 * x233 * x247)
    result[1, 4, 5] = numpy.sum(-x138 * x222 * x258)
    result[1, 4, 6] = numpy.sum(-x225 * x230 * x259)
    result[1, 4, 7] = numpy.sum(-x130 * x217 * x260)
    result[1, 4, 8] = numpy.sum(-x138 * x212 * x260)
    result[1, 4, 9] = numpy.sum(-x144 * x256 * x4)
    result[1, 5, 0] = numpy.sum(-x177 * x241 * x261)
    result[1, 5, 1] = numpy.sum(-x184 * x240 * x262)
    result[1, 5, 2] = numpy.sum(-x177 * x240 * x263)
    result[1, 5, 3] = numpy.sum(-x193 * x247 * x262)
    result[1, 5, 4] = numpy.sum(-x172 * x235 * x258)
    result[1, 5, 5] = numpy.sum(-x173 * x177 * x248)
    result[1, 5, 6] = numpy.sum(-x201 * x261 * x4)
    result[1, 5, 7] = numpy.sum(-x193 * x263 * x4)
    result[1, 5, 8] = numpy.sum(-x173 * x196 * x4)
    result[1, 5, 9] = numpy.sum(-x176 * x177 * x264)
    result[2, 0, 0] = numpy.sum(-x182 * x265)
    result[2, 0, 1] = numpy.sum(-x187 * x266)
    result[2, 0, 2] = numpy.sum(-x185 * x268)
    result[2, 0, 3] = numpy.sum(-x194 * x269 * x68)
    result[2, 0, 4] = numpy.sum(-x194 * x271 * x60)
    result[2, 0, 5] = numpy.sum(-x195 * x277)
    result[2, 0, 6] = numpy.sum(-x206 * x266 * x83)
    result[2, 0, 7] = numpy.sum(-x140 * x203 * x268)
    result[2, 0, 8] = numpy.sum(-x170 * x202 * x277)
    result[2, 0, 9] = numpy.sum(-x204 * x280)
    result[2, 1, 0] = numpy.sum(-x160 * x229 * x265)
    result[2, 1, 1] = numpy.sum(-x281 * x49 * x96)
    result[2, 1, 2] = numpy.sum(-x271 * x49 * x90)
    result[2, 1, 3] = numpy.sum(-x106 * x14 * x281)
    result[2, 1, 4] = numpy.sum(-x14 * x162 * x268)
    result[2, 1, 5] = numpy.sum(-x14 * x277 * x282)
    result[2, 1, 6] = numpy.sum(-x116 * x227 * x265)
    result[2, 1, 7] = numpy.sum(-x106 * x234 * x283)
    result[2, 1, 8] = numpy.sum(-x110 * x234 * x277)
    result[2, 1, 9] = numpy.sum(-x227 * x280 * x90)
    result[2, 2, 0] = numpy.sum(-x210 * x285)
    result[2, 2, 1] = numpy.sum(-x285 * x286 * x49)
    result[2, 2, 2] = numpy.sum(-x213 * x288)
    result[2, 2, 3] = numpy.sum(-x14 * x289 * x290)
    result[2, 2, 4] = numpy.sum(-x220 * x291 * x60)
    result[2, 2, 5] = numpy.sum(-x218 * x294)
    result[2, 2, 6] = numpy.sum(-x174 * x228 * x285)
    result[2, 2, 7] = numpy.sum(-x226 * x289 * x291)
    result[2, 2, 8] = numpy.sum(-x226 * x286 * x294)
    result[2, 2, 9] = numpy.sum(-x227 * x297)
    result[2, 3, 0] = numpy.sum(-x241 * x265 * x298)
    result[2, 3, 1] = numpy.sum(-x150 * x240 * x269)
    result[2, 3, 2] = numpy.sum(-x240 * x268 * x299)
    result[2, 3, 3] = numpy.sum(-x153 * x248 * x265)
    result[2, 3, 4] = numpy.sum(-x150 * x258 * x283)
    result[2, 3, 5] = numpy.sum(-x247 * x277 * x299)
    result[2, 3, 6] = numpy.sum(-x158 * x264 * x265)
    result[2, 3, 7] = numpy.sum(-x153 * x270 * x4)
    result[2, 3, 8] = numpy.sum(-x150 * x277 * x300)
    result[2, 3, 9] = numpy.sum(-x280 * x298 * x4)
    result[2, 4, 0] = numpy.sum(-x241 * x301 * x90)
    result[2, 4, 1] = numpy.sum(-x110 * x257 * x285)
    result[2, 4, 2] = numpy.sum(-x240 * x282 * x288)
    result[2, 4, 3] = numpy.sum(-x106 * x258 * x290)
    result[2, 4, 4] = numpy.sum(-x162 * x247 * x288)
    result[2, 4, 5] = numpy.sum(-x249 * x294 * x90)
    result[2, 4, 6] = numpy.sum(-x116 * x301 * x4)
    result[2, 4, 7] = numpy.sum(-x106 * x260 * x288)
    result[2, 4, 8] = numpy.sum(-x260 * x294 * x96)
    result[2, 4, 9] = numpy.sum(-x160 * x259 * x297)
    result[2, 5, 0] = numpy.sum(-x242 * x305)
    result[2, 5, 1] = numpy.sum(-x170 * x240 * x304)
    result[2, 5, 2] = numpy.sum(-x245 * x307)
    result[2, 5, 3] = numpy.sum(-x140 * x247 * x305)
    result[2, 5, 4] = numpy.sum(-x249 * x307 * x60)
    result[2, 5, 5] = numpy.sum(-x248 * x308)
    result[2, 5, 6] = numpy.sum(-x139 * x255 * x305)
    result[2, 5, 7] = numpy.sum(-x300 * x307 * x68)
    result[2, 5, 8] = numpy.sum(-x254 * x308 * x60)
    result[2, 5, 9] = numpy.sum(-x253 * x311)
    result[3, 0, 0] = numpy.sum(-x314 * x316)
    result[3, 0, 1] = numpy.sum(-x181 * x319)
    result[3, 0, 2] = numpy.sum(-x152 * x181 * x313)
    result[3, 0, 3] = numpy.sum(-x320 * x323)
    result[3, 0, 4] = numpy.sum(-x214 * x318 * x320)
    result[3, 0, 5] = numpy.sum(-x121 * x314 * x320)
    result[3, 0, 6] = numpy.sum(-x325 * x327)
    result[3, 0, 7] = numpy.sum(-x152 * x322 * x324)
    result[3, 0, 8] = numpy.sum(-x121 * x318 * x325)
    result[3, 0, 9] = numpy.sum(-x159 * x314 * x324 * x52)
    result[3, 1, 0] = numpy.sum(-x328 * x330)
    result[3, 1, 1] = numpy.sum(-x332 * x47)
    result[3, 1, 2] = numpy.sum(-x214 * x329 * x47)
    result[3, 1, 3] = numpy.sum(-x333 * x337)
    result[3, 1, 4] = numpy.sum(-x333 * x338 * x88)
    result[3, 1, 5] = numpy.sum(-x221 * x329 * x339)
    result[3, 1, 6] = numpy.sum(-x341 * x342)
    result[3, 1, 7] = numpy.sum(-x336 * x76 * x9)
    result[3, 1, 8] = numpy.sum(-x331 * x343 * x78)
    result[3, 1, 9] = numpy.sum(-x123 * x344 * x9)
    result[3, 2, 0] = numpy.sum(-x126 * x328 * x345)
    result[3, 2, 1] = numpy.sum(-x232 * x318 * x47)
    result[3, 2, 2] = numpy.sum(-x131 * x313 * x346)
    result[3, 2, 3] = numpy.sum(-x232 * x322 * x333)
    result[3, 2, 4] = numpy.sum(-x233 * x318 * x333)
    result[3, 2, 5] = numpy.sum(-x138 * x314 * x347)
    result[3, 2, 6] = numpy.sum(-x230 * x326 * x348)
    result[3, 2, 7] = numpy.sum(-x130 * x322 * x343)
    result[3, 2, 8] = numpy.sum(-x138 * x318 * x343)
    result[3, 2, 9] = numpy.sum(-x144 * x345 * x9)
    result[3, 3, 0] = numpy.sum(-x351 * x353)
    result[3, 3, 1] = numpy.sum(-x354 * x357)
    result[3, 3, 2] = numpy.sum(-x152 * x350 * x354)
    result[3, 3, 3] = numpy.sum(-x3 * x359)
    result[3, 3, 4] = numpy.sum(-x356 * x360 * x75)
    result[3, 3, 5] = numpy.sum(-x198 * x351)
    result[3, 3, 6] = numpy.sum(
        -x57 * (x358 * x60 + x51 * (x246 + 2.0 * x334 + x335 + x355))
    )
    result[3, 3, 7] = numpy.sum(-x359 * x65)
    result[3, 3, 8] = numpy.sum(-x357 * x78)
    result[3, 3, 9] = numpy.sum(-x350 * x80 * x89)
    result[3, 4, 0] = numpy.sum(-x126 * x344 * x361)
    result[3, 4, 1] = numpy.sum(-x232 * x331 * x354)
    result[3, 4, 2] = numpy.sum(-x131 * x329 * x363)
    result[3, 4, 3] = numpy.sum(-x128 * x336 * x364)
    result[3, 4, 4] = numpy.sum(-x131 * x3 * x338)
    result[3, 4, 5] = numpy.sum(-x138 * x329 * x365)
    result[3, 4, 6] = numpy.sum(-x127 * x341)
    result[3, 4, 7] = numpy.sum(-x130 * x337)
    result[3, 4, 8] = numpy.sum(-x138 * x332)
    result[3, 4, 9] = numpy.sum(-x144 * x330)
    result[3, 5, 0] = numpy.sum(-x169 * x313 * x366)
    result[3, 5, 1] = numpy.sum(-x262 * x318 * x354)
    result[3, 5, 2] = numpy.sum(-x172 * x314 * x362)
    result[3, 5, 3] = numpy.sum(-x167 * x322 * x367)
    result[3, 5, 4] = numpy.sum(-x172 * x318 * x365)
    result[3, 5, 5] = numpy.sum(-x173 * x313 * x367)
    result[3, 5, 6] = numpy.sum(-x168 * x327)
    result[3, 5, 7] = numpy.sum(-x172 * x323)
    result[3, 5, 8] = numpy.sum(-x173 * x319)
    result[3, 5, 9] = numpy.sum(-x176 * x314)
    result[4, 0, 0] = numpy.sum(-x177 * x265 * x316 * x56)
    result[4, 0, 1] = numpy.sum(-x181 * x196 * x265)
    result[4, 0, 2] = numpy.sum(-x177 * x181 * x270)
    result[4, 0, 3] = numpy.sum(-x193 * x269 * x320)
    result[4, 0, 4] = numpy.sum(-x165 * x235 * x268 * x320)
    result[4, 0, 5] = numpy.sum(-x277 * x320 * x368)
    result[4, 0, 6] = numpy.sum(-x201 * x265 * x369)
    result[4, 0, 7] = numpy.sum(-x193 * x268 * x370)
    result[4, 0, 8] = numpy.sum(-x184 * x277 * x370)
    result[4, 0, 9] = numpy.sum(-x177 * x280 * x369)
    result[4, 1, 0] = numpy.sum(-x256 * x265 * x328)
    result[4, 1, 1] = numpy.sum(-x212 * x281 * x47)
    result[4, 1, 2] = numpy.sum(-x222 * x268 * x346)
    result[4, 1, 3] = numpy.sum(-x217 * x281 * x333)
    result[4, 1, 4] = numpy.sum(-x219 * x268 * x371)
    result[4, 1, 5] = numpy.sum(-x222 * x277 * x347)
    result[4, 1, 6] = numpy.sum(-x225 * x265 * x372)
    result[4, 1, 7] = numpy.sum(-x217 * x271 * x9)
    result[4, 1, 8] = numpy.sum(-x212 * x277 * x343)
    result[4, 1, 9] = numpy.sum(-x256 * x280 * x9)
    result[4, 2, 0] = numpy.sum(-x177 * x301 * x328)
    result[4, 2, 1] = numpy.sum(-x235 * x285 * x346)
    result[4, 2, 2] = numpy.sum(-x231 * x288 * x47)
    result[4, 2, 3] = numpy.sum(-x193 * x290 * x347)
    result[4, 2, 4] = numpy.sum(-x184 * x291 * x371)
    result[4, 2, 5] = numpy.sum(-x231 * x294 * x333)
    result[4, 2, 6] = numpy.sum(-x201 * x301 * x9)
    result[4, 2, 7] = numpy.sum(-x193 * x288 * x343)
    result[4, 2, 8] = numpy.sum(-x197 * x294 * x9)
    result[4, 2, 9] = numpy.sum(-x177 * x297 * x372)
    result[4, 3, 0] = numpy.sum(-x239 * x265 * x373)
    result[4, 3, 1] = numpy.sum(-x244 * x269 * x354)
    result[4, 3, 2] = numpy.sum(-x239 * x268 * x362)
    result[4, 3, 3] = numpy.sum(-x246 * x265 * x360)
    result[4, 3, 4] = numpy.sum(-x244 * x271 * x3)
    result[4, 3, 5] = numpy.sum(-x238 * x277 * x367)
    result[4, 3, 6] = numpy.sum(-x252 * x265 * x57)
    result[4, 3, 7] = numpy.sum(-x246 * x270)
    result[4, 3, 8] = numpy.sum(-x107 * x244 * x277)
    result[4, 3, 9] = numpy.sum(-x239 * x280 * x53)
    result[4, 4, 0] = numpy.sum(-x117 * x208 * x285 * x366)
    result[4, 4, 1] = numpy.sum(-x219 * x285 * x363)
    result[4, 4, 2] = numpy.sum(-x208 * x291 * x363)
    result[4, 4, 3] = numpy.sum(-x217 * x285 * x365)
    result[4, 4, 4] = numpy.sum(-x219 * x288 * x374)
    result[4, 4, 5] = numpy.sum(-x208 * x294 * x365)
    result[4, 4, 6] = numpy.sum(-x225 * x301)
    result[4, 4, 7] = numpy.sum(-x108 * x217 * x288)
    result[4, 4, 8] = numpy.sum(-x108 * x212 * x294)
    result[4, 4, 9] = numpy.sum(-x256 * x297)
    result[4, 5, 0] = numpy.sum(-x177 * x305 * x373)
    result[4, 5, 1] = numpy.sum(-x184 * x305 * x362)
    result[4, 5, 2] = numpy.sum(-x307 * x354 * x368)
    result[4, 5, 3] = numpy.sum(-x193 * x304 * x367)
    result[4, 5, 4] = numpy.sum(-x197 * x3 * x307)
    result[4, 5, 5] = numpy.sum(-x177 * x308 * x360)
    result[4, 5, 6] = numpy.sum(-x201 * x305 * x53)
    result[4, 5, 7] = numpy.sum(-x107 * x193 * x307)
    result[4, 5, 8] = numpy.sum(-x196 * x308)
    result[4, 5, 9] = numpy.sum(-x177 * x311 * x57)
    result[5, 0, 0] = numpy.sum(-x316 * x377)
    result[5, 0, 1] = numpy.sum(-x170 * x181 * x376)
    result[5, 0, 2] = numpy.sum(-x181 * x380)
    result[5, 0, 3] = numpy.sum(-x140 * x320 * x377)
    result[5, 0, 4] = numpy.sum(-x286 * x320 * x379)
    result[5, 0, 5] = numpy.sum(-x320 * x383)
    result[5, 0, 6] = numpy.sum(-x175 * x325 * x376)
    result[5, 0, 7] = numpy.sum(-x140 * x325 * x379)
    result[5, 0, 8] = numpy.sum(-x170 * x324 * x382)
    result[5, 0, 9] = numpy.sum(-x369 * x384)
    result[5, 1, 0] = numpy.sum(-x328 * x385 * x90)
    result[5, 1, 1] = numpy.sum(-x110 * x346 * x376)
    result[5, 1, 2] = numpy.sum(-x282 * x379 * x47)
    result[5, 1, 3] = numpy.sum(-x106 * x347 * x377)
    result[5, 1, 4] = numpy.sum(-x162 * x333 * x379)
    result[5, 1, 5] = numpy.sum(-x282 * x333 * x382)
    result[5, 1, 6] = numpy.sum(-x116 * x385 * x9)
    result[5, 1, 7] = numpy.sum(-x106 * x343 * x379)
    result[5, 1, 8] = numpy.sum(-x343 * x382 * x96)
    result[5, 1, 9] = numpy.sum(-x160 * x348 * x384)
    result[5, 2, 0] = numpy.sum(-x328 * x387)
    result[5, 2, 1] = numpy.sum(-x286 * x386 * x47)
    result[5, 2, 2] = numpy.sum(-x389 * x47)
    result[5, 2, 3] = numpy.sum(-x289 * x339 * x386)
    result[5, 2, 4] = numpy.sum(-0.25 * x333 * x388 * x84)
    result[5, 2, 5] = numpy.sum(-x333 * x393)
    result[5, 2, 6] = numpy.sum(-x139 * x394 * x9)
    result[5, 2, 7] = numpy.sum(-x343 * x388 * x68)
    result[5, 2, 8] = numpy.sum(-x395 * x60 * x73 * x9)
    result[5, 2, 9] = numpy.sum(-x342 * x397)
    result[5, 3, 0] = numpy.sum(-x148 * x366 * x376)
    result[5, 3, 1] = numpy.sum(-x150 * x362 * x377)
    result[5, 3, 2] = numpy.sum(-x299 * x354 * x379)
    result[5, 3, 3] = numpy.sum(-x153 * x367 * x376)
    result[5, 3, 4] = numpy.sum(-x150 * x365 * x379)
    result[5, 3, 5] = numpy.sum(-x146 * x367 * x382)
    result[5, 3, 6] = numpy.sum(-x158 * x377)
    result[5, 3, 7] = numpy.sum(-x153 * x380)
    result[5, 3, 8] = numpy.sum(-x150 * x383)
    result[5, 3, 9] = numpy.sum(-x298 * x384)
    result[5, 4, 0] = numpy.sum(-x361 * x394 * x90)
    result[5, 4, 1] = numpy.sum(-x110 * x363 * x386)
    result[5, 4, 2] = numpy.sum(-x282 * x354 * x388)
    result[5, 4, 3] = numpy.sum(-x106 * x365 * x386)
    result[5, 4, 4] = numpy.sum(-x110 * x374 * x388)
    result[5, 4, 5] = numpy.sum(-x364 * x395 * x91)
    result[5, 4, 6] = numpy.sum(-x116 * x387)
    result[5, 4, 7] = numpy.sum(-x106 * x389)
    result[5, 4, 8] = numpy.sum(-x393 * x96)
    result[5, 4, 9] = numpy.sum(-x397 * x91)
    result[5, 5, 0] = numpy.sum(-x3 * x353 * x399)
    result[5, 5, 1] = numpy.sum(-x170 * x354 * x399)
    result[5, 5, 2] = numpy.sum(-x354 * x402)
    result[5, 5, 3] = numpy.sum(-x367 * x399 * x68)
    result[5, 5, 4] = numpy.sum(-x360 * x401 * x60 * x74)
    result[5, 5, 5] = numpy.sum(-x3 * x404)
    result[5, 5, 6] = numpy.sum(-x399 * x80 * x85)
    result[5, 5, 7] = numpy.sum(-x402 * x68)
    result[5, 5, 8] = numpy.sum(-x404 * x60)
    result[5, 5, 9] = numpy.sum(
        -x57 * (x403 * x65 + x51 * (x308 + 2.0 * x390 + x391 + x400))
    )
    return result


def quadrupole3d_24(ax, da, A, bx, db, B, R):
    """Cartesian 3D (dg) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 6, 15), dtype=float)

    x0 = 0.5 / (ax + bx)
    x1 = (ax + bx) ** (-1.0)
    x2 = -x1 * (ax * A[0] + bx * B[0])
    x3 = -x2 - A[0]
    x4 = -x2 - B[0]
    x5 = ax * bx * x1
    x6 = numpy.exp(-x5 * (A[0] - B[0]) ** 2)
    x7 = 1.772453850905516 * numpy.sqrt(x1)
    x8 = x6 * x7
    x9 = x4 * x8
    x10 = -x2 - R[0]
    x11 = x10 * x8
    x12 = x0 * (x11 + x9)
    x13 = x0 * x8
    x14 = x10 * x9
    x15 = x13 + x14
    x16 = x15 * x3
    x17 = x12 + x16
    x18 = x17 * x3
    x19 = x3 * x8
    x20 = x19 * x4
    x21 = x4**2 * x8
    x22 = 3.0 * x13
    x23 = x21 + x22
    x24 = x0 * (2.0 * x20 + x23)
    x25 = x0 * (x19 + x9)
    x26 = x13 + x20
    x27 = x26 * x4
    x28 = x25 + x27
    x29 = x28 * x3
    x30 = x24 + x29
    x31 = x17 * x4
    x32 = x10 * x19
    x33 = x0 * (x14 + x20 + x22 + x32)
    x34 = 2.0 * x31 + 4.0 * x33
    x35 = x0 * (2.0 * x18 + x30 + x34)
    x36 = x15 * x4
    x37 = 3.0 * x12
    x38 = 2.0 * x16 + x37
    x39 = x0 * (x28 + x36 + x38)
    x40 = x31 + x33
    x41 = x3 * x40
    x42 = x39 + x41
    x43 = x4 * x42
    x44 = x10 * x15
    x45 = x0 * (x11 + x19)
    x46 = x13 + x32
    x47 = x10 * x46 + x45
    x48 = x0 * (x38 + x44 + x47)
    x49 = x10 * x17
    x50 = x33 + x49
    x51 = x4 * x50
    x52 = x48 + x51
    x53 = x4 * x52
    x54 = x3 * x52
    x55 = 2.0 * x14
    x56 = x10**2 * x8
    x57 = x22 + x56
    x58 = x0 * (x55 + x57)
    x59 = x12 + x44
    x60 = x4 * x59
    x61 = x58 + x60
    x62 = x0 * (x34 + 2.0 * x49 + x61)
    x63 = 2.0 * x39
    x64 = x3 * x50
    x65 = x54 + x62
    x66 = x0 * (2.0 * x41 + 4.0 * x48 + 2.0 * x51 + x63 + 2.0 * x64) + x4 * x65
    x67 = da * db
    x68 = 0.0563436169819011
    x69 = x67 * x68
    x70 = numpy.exp(-x5 * (A[1] - B[1]) ** 2)
    x71 = numpy.exp(-x5 * (A[2] - B[2]) ** 2)
    x72 = 3.141592653589793 * x1 * x71
    x73 = x70 * x72
    x74 = x69 * x73
    x75 = -x1 * (ax * A[1] + bx * B[1])
    x76 = -x75 - B[1]
    x77 = x73 * x76
    x78 = 2.23606797749979
    x79 = 0.06666666666666667 * x67
    x80 = x78 * x79
    x81 = x66 * x80
    x82 = -x1 * (ax * A[2] + bx * B[2])
    x83 = -x82 - B[2]
    x84 = x73 * x83
    x85 = x7 * x71
    x86 = x7 * x70
    x87 = x76**2 * x86
    x88 = x0 * x86
    x89 = x87 + x88
    x90 = 1.732050807568877
    x91 = 0.1111111111111111 * x90
    x92 = x89 * x91
    x93 = x67 * x92
    x94 = 0.3333333333333333 * x67
    x95 = x83 * x94
    x96 = x83**2 * x85
    x97 = x0 * x85
    x98 = x96 + x97
    x99 = x91 * x98
    x100 = x67 * x99
    x101 = x48 + x64
    x102 = x76 * (2.0 * x88 + x89)
    x103 = x102 * x79
    x104 = x103 * x78
    x105 = x83 * x85
    x106 = x89 * x94
    x107 = x76 * x86
    x108 = x94 * x98
    x109 = x83 * (2.0 * x97 + x98)
    x110 = x109 * x79
    x111 = x110 * x78
    x112 = x67 * (x0 * (2.0 * x32 + x57) + x3 * x47)
    x113 = 3.0 * x88
    x114 = x0 * (x113 + 3.0 * x87) + x102 * x76
    x115 = x114 * x68
    x116 = 0.06666666666666667 * x112 * x78
    x117 = 3.0 * x97
    x118 = x0 * (x117 + 3.0 * x96) + x109 * x83
    x119 = x118 * x68
    x120 = -x75 - A[1]
    x121 = 0.09759000729485332
    x122 = x121 * x67
    x123 = x120 * x122
    x124 = x4 * x40
    x125 = 2.0 * x124
    x126 = 2.0 * x0 * (2.0 * x12 + x36 + x44) + x4 * x61
    x127 = x53 + x62
    x128 = x73 * (x0 * (x125 + x126 + 3.0 * x48 + 3.0 * x51 + x63) + x127 * x4)
    x129 = x120 * x86
    x130 = x129 * x76
    x131 = x130 + x88
    x132 = x131 * x79
    x133 = 3.872983346207417
    x134 = x133 * x85
    x135 = x133 * x79
    x136 = x127 * x135
    x137 = x0 * (x107 + x129)
    x138 = x131 * x76
    x139 = x137 + x138
    x140 = x139 * x94
    x141 = x131 * x90
    x142 = x141 * x94
    x143 = x113 + x87
    x144 = x0 * (2.0 * x130 + x143)
    x145 = x139 * x76
    x146 = x144 + x145
    x147 = x146 * x79
    x148 = x50 * x90
    x149 = x110 * x133
    x150 = x0 * (x102 + 3.0 * x137 + 3.0 * x138) + x146 * x76
    x151 = x122 * x47
    x152 = x133 * x47
    x153 = x47 * x94
    x154 = -x82 - A[2]
    x155 = x122 * x154
    x156 = x154 * x85
    x157 = x156 * x83
    x158 = x157 + x97
    x159 = x158 * x79
    x160 = x133 * x86
    x161 = x158 * x90
    x162 = x161 * x94
    x163 = x0 * (x105 + x156)
    x164 = x158 * x83
    x165 = x163 + x164
    x166 = x165 * x94
    x167 = x133 * x156
    x168 = x117 + x96
    x169 = x0 * (2.0 * x157 + x168)
    x170 = x165 * x83
    x171 = x169 + x170
    x172 = x171 * x79
    x173 = x0 * (x109 + 3.0 * x163 + 3.0 * x164) + x171 * x83
    x174 = x120**2 * x86 + x88
    x175 = x0 * (x23 + x55)
    x176 = x12 + x36
    x177 = x176 * x4
    x178 = x0 * (2.0 * x175 + 2.0 * x177 + 3.0 * x58 + 3.0 * x60) + x126 * x4
    x179 = x69 * x85
    x180 = x120 * x131
    x181 = x137 + x180
    x182 = x80 * x85
    x183 = x174 * x80
    x184 = x67 * x85
    x185 = x120 * x139
    x186 = x144 + x185
    x187 = x186 * x91
    x188 = x61 * x94
    x189 = 2.0 * x0 * (2.0 * x137 + x138 + x180) + x186 * x76
    x190 = x59 * x94
    x191 = x0 * (5.0 * x144 + 2.0 * x145 + 3.0 * x185) + x189 * x76
    x192 = x13 + x56
    x193 = x192 * x67
    x194 = x193 * x68
    x195 = x192 * x80
    x196 = x123 * x73
    x197 = x129 * x133
    x198 = x121 * x193
    x199 = x133 * x192
    x200 = x154**2 * x85 + x97
    x201 = x200 * x67
    x202 = x68 * x86
    x203 = x200 * x80
    x204 = x154 * x158
    x205 = x163 + x204
    x206 = x80 * x86
    x207 = x205 * x94
    x208 = x86 * x91
    x209 = x154 * x165
    x210 = x169 + x209
    x211 = x210 * x67
    x212 = 2.0 * x0 * (2.0 * x163 + x164 + x204) + x210 * x83
    x213 = x0 * (5.0 * x169 + 2.0 * x170 + 3.0 * x209) + x212 * x83
    x214 = -x75 - R[1]
    x215 = x26 * x3
    x216 = 2.0 * x0 * (x215 + 2.0 * x25 + x27) + x30 * x4
    x217 = x35 + x43
    x218 = x74 * (x0 * (x125 + x216 + 5.0 * x39 + 3.0 * x41) + x217 * x4)
    x219 = x107 * x214
    x220 = x219 + x88
    x221 = x217 * x80
    x222 = x214 * x86
    x223 = x0 * (x107 + x222)
    x224 = x220 * x76
    x225 = x223 + x224
    x226 = x225 * x91
    x227 = x105 * x94
    x228 = 2.0 * x219
    x229 = x0 * (x143 + x228)
    x230 = x225 * x76
    x231 = x229 + x230
    x232 = x18 + x33
    x233 = x3 * x46 + x45
    x234 = 3.0 * x223
    x235 = x0 * (x102 + 3.0 * x224 + x234) + x231 * x76
    x236 = x105 * x80
    x237 = x119 * x67
    x238 = x175 + x177
    x239 = x28 * x4
    x240 = x239 + x24
    x241 = x124 + x39
    x242 = x0 * (x238 + x240 + 3.0 * x31 + 3.0 * x33) + x241 * x4
    x243 = x129 * x214
    x244 = x243 + x88
    x245 = x122 * x244
    x246 = x120 * x220
    x247 = x223 + x246
    x248 = x134 * x79
    x249 = x133 * x244
    x250 = x249 * x79
    x251 = x0 * (x113 + x130 + x219 + x243)
    x252 = x247 * x76
    x253 = x251 + x252
    x254 = x85 * x94
    x255 = x40 * x90
    x256 = x234 + 2.0 * x246
    x257 = x0 * (x139 + x224 + x256)
    x258 = x253 * x76
    x259 = x257 + x258
    x260 = x17 * x90
    x261 = x0 * (x146 + x231 + 3.0 * x251 + 3.0 * x252) + x259 * x76
    x262 = x122 * x46
    x263 = x133 * x46
    x264 = x263 * x79
    x265 = x167 * x79
    x266 = x133 * x222
    x267 = x156 * x94
    x268 = x0 * (x129 + x222)
    x269 = x120 * x244 + x268
    x270 = x13 + x21
    x271 = 2.0 * x0 * x9 + x270 * x4
    x272 = x0 * (x271 + 3.0 * x36 + x37) + x238 * x4
    x273 = x120 * x247
    x274 = x251 + x273
    x275 = x120 * x253
    x276 = x257 + x275
    x277 = x176 * x91
    x278 = 4.0 * x251 + 2.0 * x252
    x279 = x0 * (x186 + 2.0 * x273 + x278)
    x280 = x276 * x76
    x281 = x279 + x280
    x282 = x15 * x94
    x283 = 2.0 * x258
    x284 = x6 * x69
    x285 = x284 * x72
    x286 = x285 * (x0 * (x189 + 5.0 * x257 + 3.0 * x275 + x283) + x281 * x76)
    x287 = x6 * x72
    x288 = x287 * x83
    x289 = x10 * x80
    x290 = x166 * x90
    x291 = x155 * x287
    x292 = x11 * x133
    x293 = x201 * x68
    x294 = x222 * x80
    x295 = x11 * x80
    x296 = 3.141592653589793 * x1 * x70
    x297 = x284 * x296
    x298 = -x82 - R[2]
    x299 = x105 * x298
    x300 = x299 + x97
    x301 = x298 * x85
    x302 = x300 * x94
    x303 = x0 * (x105 + x301)
    x304 = x300 * x83
    x305 = x303 + x304
    x306 = x305 * x67
    x307 = x305 * x94
    x308 = 2.0 * x299
    x309 = x0 * (x168 + x308)
    x310 = x305 * x83
    x311 = x309 + x310
    x312 = x301 * x67
    x313 = x107 * x80
    x314 = 3.0 * x303
    x315 = x0 * (x109 + 3.0 * x304 + x314) + x311 * x83
    x316 = x315 * x69
    x317 = x133 * x301
    x318 = x197 * x79
    x319 = x156 * x298
    x320 = x319 + x97
    x321 = x122 * x320
    x322 = x133 * x320
    x323 = x322 * x79
    x324 = x154 * x300
    x325 = x303 + x324
    x326 = x160 * x79
    x327 = x325 * x94
    x328 = x0 * (x117 + x157 + x299 + x319)
    x329 = x325 * x83
    x330 = x328 + x329
    x331 = x330 * x94
    x332 = x314 + 2.0 * x324
    x333 = x0 * (x165 + x304 + x332)
    x334 = x330 * x83
    x335 = x333 + x334
    x336 = x0 * (x171 + x311 + 3.0 * x328 + 3.0 * x329) + x335 * x83
    x337 = x301 * x69
    x338 = x301 * x80
    x339 = x325 * x90
    x340 = x296 * x6
    x341 = x336 * x340
    x342 = x0 * (x156 + x301)
    x343 = x154 * x320 + x342
    x344 = x343 * x67
    x345 = x154 * x325
    x346 = x328 + x345
    x347 = x346 * x94
    x348 = x154 * x330
    x349 = x333 + x348
    x350 = x349 * x67
    x351 = 4.0 * x328 + 2.0 * x329
    x352 = x0 * (x210 + 2.0 * x345 + x351)
    x353 = x349 * x83
    x354 = x352 + x353
    x355 = x340 * x76
    x356 = 2.0 * x334
    x357 = x297 * (x0 * (x212 + 5.0 * x333 + 3.0 * x348 + x356) + x354 * x83)
    x358 = x0 * (2.0 * x239 + 5.0 * x24 + 3.0 * x29) + x216 * x4
    x359 = x214**2 * x86
    x360 = x359 + x88
    x361 = x360 * x67
    x362 = x361 * x68
    x363 = x214 * x220
    x364 = x223 + x363
    x365 = x113 + x359
    x366 = x0 * (x228 + x365)
    x367 = x364 * x76
    x368 = x366 + x367
    x369 = x368 * x91
    x370 = 2.0 * x0 * (2.0 * x223 + x224 + x363) + x368 * x76
    x371 = x215 + x25
    x372 = x13 + x3**2 * x8
    x373 = x0 * (2.0 * x229 + 2.0 * x230 + 3.0 * x366 + 3.0 * x367) + x370 * x76
    x374 = x372 * x80
    x375 = x0 * (3.0 * x25 + 3.0 * x27 + x271) + x240 * x4
    x376 = x214 * x244 + x268
    x377 = x122 * x376
    x378 = x214 * x247
    x379 = x251 + x378
    x380 = x133 * x376
    x381 = x240 * x79
    x382 = x0 * (x256 + x363 + x376)
    x383 = x379 * x76
    x384 = x382 + x383
    x385 = x28 * x90
    x386 = x0 * (x278 + x368 + 2.0 * x378)
    x387 = x384 * x76
    x388 = x386 + x387
    x389 = x133 * x26
    x390 = x389 * x79
    x391 = x26 * x90
    x392 = 2.0 * x257
    x393 = x287 * (x0 * (x283 + x370 + 3.0 * x382 + 3.0 * x383 + x392) + x388 * x76)
    x394 = x122 * x3
    x395 = x135 * x388
    x396 = x122 * x360
    x397 = x133 * x360
    x398 = x28 * x94
    x399 = x133 * x19
    x400 = x0 * (2.0 * x243 + x365) + x120 * x376
    x401 = x0 * (3.0 * x21 + x22) + x271 * x4
    x402 = x401 * x68
    x403 = x120 * x379
    x404 = x382 + x403
    x405 = x271 * x80
    x406 = x120 * x384
    x407 = x386 + x406
    x408 = x270 * x91
    x409 = x270 * x94
    x410 = x287 * x4
    x411 = x0 * (2.0 * x275 + 4.0 * x382 + 2.0 * x383 + x392 + 2.0 * x403) + x407 * x76
    x412 = x411 * x80
    x413 = x67 * x8
    x414 = x133 * x271
    x415 = x414 * x79
    x416 = x133 * x8
    x417 = x9 * x94
    x418 = x80 * x9
    x419 = x68 * x8
    x420 = x8 * x80
    x421 = x306 * x91
    x422 = x399 * x79
    x423 = x320 * x94
    x424 = x9 * x90
    x425 = x416 * x79
    x426 = x340 * x4
    x427 = x298**2 * x85
    x428 = x427 + x97
    x429 = x428 * x67
    x430 = x298 * x300
    x431 = x303 + x430
    x432 = x107 * x94
    x433 = x117 + x427
    x434 = x0 * (x308 + x433)
    x435 = x431 * x83
    x436 = x434 + x435
    x437 = x436 * x67
    x438 = 2.0 * x0 * (2.0 * x303 + x304 + x430) + x436 * x83
    x439 = x0 * (2.0 * x309 + 2.0 * x310 + 3.0 * x434 + 3.0 * x435) + x438 * x83
    x440 = x439 * x69
    x441 = x122 * x428
    x442 = x133 * x428
    x443 = x298 * x320 + x342
    x444 = x122 * x443
    x445 = x133 * x443
    x446 = x298 * x325
    x447 = x328 + x446
    x448 = x0 * (x332 + x430 + x443)
    x449 = x447 * x83
    x450 = x448 + x449
    x451 = x0 * (x351 + x436 + 2.0 * x446)
    x452 = x450 * x83
    x453 = x451 + x452
    x454 = x135 * x453
    x455 = 2.0 * x333
    x456 = x340 * (x0 * (x356 + x438 + 3.0 * x448 + 3.0 * x449 + x455) + x453 * x83)
    x457 = x0 * (2.0 * x319 + x433) + x154 * x443
    x458 = x67 * x86
    x459 = x154 * x447
    x460 = x448 + x459
    x461 = x154 * x450
    x462 = x451 + x461
    x463 = x0 * (2.0 * x348 + 4.0 * x448 + 2.0 * x449 + x455 + 2.0 * x459) + x462 * x83
    x464 = x463 * x80

    # 540 item(s)
    result[0, 0, 0] = numpy.sum(
        x74
        * (x0 * (2.0 * x35 + 2.0 * x43 + 2.0 * x53 + 3.0 * x54 + 5.0 * x62) + x4 * x66)
    )
    result[0, 0, 1] = numpy.sum(x77 * x81)
    result[0, 0, 2] = numpy.sum(x81 * x84)
    result[0, 0, 3] = numpy.sum(x65 * x85 * x93)
    result[0, 0, 4] = numpy.sum(x65 * x77 * x95)
    result[0, 0, 5] = numpy.sum(x100 * x65 * x86)
    result[0, 0, 6] = numpy.sum(x101 * x104 * x85)
    result[0, 0, 7] = numpy.sum(x101 * x105 * x106)
    result[0, 0, 8] = numpy.sum(x101 * x107 * x108)
    result[0, 0, 9] = numpy.sum(x101 * x111 * x86)
    result[0, 0, 10] = numpy.sum(x112 * x115 * x85)
    result[0, 0, 11] = numpy.sum(x102 * x105 * x116)
    result[0, 0, 12] = numpy.sum(x112 * x89 * x99)
    result[0, 0, 13] = numpy.sum(x107 * x109 * x116)
    result[0, 0, 14] = numpy.sum(x112 * x119 * x86)
    result[0, 1, 0] = numpy.sum(x123 * x128)
    result[0, 1, 1] = numpy.sum(x127 * x132 * x134)
    result[0, 1, 2] = numpy.sum(x120 * x136 * x84)
    result[0, 1, 3] = numpy.sum(x140 * x52 * x85)
    result[0, 1, 4] = numpy.sum(x105 * x142 * x52)
    result[0, 1, 5] = numpy.sum(x108 * x129 * x52)
    result[0, 1, 6] = numpy.sum(x134 * x147 * x50)
    result[0, 1, 7] = numpy.sum(x105 * x140 * x148)
    result[0, 1, 8] = numpy.sum(x108 * x131 * x148)
    result[0, 1, 9] = numpy.sum(x129 * x149 * x50)
    result[0, 1, 10] = numpy.sum(x150 * x151 * x85)
    result[0, 1, 11] = numpy.sum(x105 * x147 * x152)
    result[0, 1, 12] = numpy.sum(x139 * x153 * x98)
    result[0, 1, 13] = numpy.sum(x110 * x131 * x152)
    result[0, 1, 14] = numpy.sum(x118 * x129 * x151)
    result[0, 2, 0] = numpy.sum(x128 * x155)
    result[0, 2, 1] = numpy.sum(x136 * x154 * x77)
    result[0, 2, 2] = numpy.sum(x127 * x159 * x160)
    result[0, 2, 3] = numpy.sum(x106 * x156 * x52)
    result[0, 2, 4] = numpy.sum(x107 * x162 * x52)
    result[0, 2, 5] = numpy.sum(x166 * x52 * x86)
    result[0, 2, 6] = numpy.sum(x103 * x167 * x50)
    result[0, 2, 7] = numpy.sum(x106 * x148 * x158)
    result[0, 2, 8] = numpy.sum(x107 * x148 * x166)
    result[0, 2, 9] = numpy.sum(x160 * x172 * x50)
    result[0, 2, 10] = numpy.sum(x114 * x151 * x156)
    result[0, 2, 11] = numpy.sum(x102 * x152 * x159)
    result[0, 2, 12] = numpy.sum(x153 * x165 * x89)
    result[0, 2, 13] = numpy.sum(x107 * x152 * x172)
    result[0, 2, 14] = numpy.sum(x151 * x173 * x86)
    result[0, 3, 0] = numpy.sum(x174 * x178 * x179)
    result[0, 3, 1] = numpy.sum(x126 * x181 * x182)
    result[0, 3, 2] = numpy.sum(x105 * x126 * x183)
    result[0, 3, 3] = numpy.sum(x184 * x187 * x61)
    result[0, 3, 4] = numpy.sum(x105 * x181 * x188)
    result[0, 3, 5] = numpy.sum(x100 * x174 * x61)
    result[0, 3, 6] = numpy.sum(x182 * x189 * x59)
    result[0, 3, 7] = numpy.sum(x105 * x186 * x190)
    result[0, 3, 8] = numpy.sum(x181 * x190 * x98)
    result[0, 3, 9] = numpy.sum(x111 * x174 * x59)
    result[0, 3, 10] = numpy.sum(x191 * x194 * x85)
    result[0, 3, 11] = numpy.sum(x105 * x189 * x195)
    result[0, 3, 12] = numpy.sum(x100 * x186 * x192)
    result[0, 3, 13] = numpy.sum(x111 * x181 * x192)
    result[0, 3, 14] = numpy.sum(x119 * x174 * x193)
    result[0, 4, 0] = numpy.sum(x154 * x178 * x196)
    result[0, 4, 1] = numpy.sum(x126 * x132 * x167)
    result[0, 4, 2] = numpy.sum(x126 * x159 * x197)
    result[0, 4, 3] = numpy.sum(x140 * x156 * x61)
    result[0, 4, 4] = numpy.sum(x131 * x161 * x188)
    result[0, 4, 5] = numpy.sum(x129 * x166 * x61)
    result[0, 4, 6] = numpy.sum(x147 * x167 * x59)
    result[0, 4, 7] = numpy.sum(x139 * x161 * x190)
    result[0, 4, 8] = numpy.sum(x141 * x165 * x190)
    result[0, 4, 9] = numpy.sum(x172 * x197 * x59)
    result[0, 4, 10] = numpy.sum(x150 * x156 * x198)
    result[0, 4, 11] = numpy.sum(x146 * x159 * x199)
    result[0, 4, 12] = numpy.sum(x139 * x166 * x192)
    result[0, 4, 13] = numpy.sum(x131 * x172 * x199)
    result[0, 4, 14] = numpy.sum(x129 * x173 * x198)
    result[0, 5, 0] = numpy.sum(x178 * x201 * x202)
    result[0, 5, 1] = numpy.sum(x107 * x126 * x203)
    result[0, 5, 2] = numpy.sum(x126 * x205 * x206)
    result[0, 5, 3] = numpy.sum(x201 * x61 * x92)
    result[0, 5, 4] = numpy.sum(x107 * x207 * x61)
    result[0, 5, 5] = numpy.sum(x208 * x211 * x61)
    result[0, 5, 6] = numpy.sum(x104 * x200 * x59)
    result[0, 5, 7] = numpy.sum(x190 * x205 * x89)
    result[0, 5, 8] = numpy.sum(x107 * x190 * x210)
    result[0, 5, 9] = numpy.sum(x206 * x212 * x59)
    result[0, 5, 10] = numpy.sum(x115 * x193 * x200)
    result[0, 5, 11] = numpy.sum(x104 * x192 * x205)
    result[0, 5, 12] = numpy.sum(x193 * x210 * x92)
    result[0, 5, 13] = numpy.sum(x107 * x195 * x212)
    result[0, 5, 14] = numpy.sum(x194 * x213 * x86)
    result[1, 0, 0] = numpy.sum(x214 * x218)
    result[1, 0, 1] = numpy.sum(x182 * x217 * x220)
    result[1, 0, 2] = numpy.sum(x214 * x221 * x84)
    result[1, 0, 3] = numpy.sum(x184 * x226 * x42)
    result[1, 0, 4] = numpy.sum(x220 * x227 * x42)
    result[1, 0, 5] = numpy.sum(x100 * x222 * x42)
    result[1, 0, 6] = numpy.sum(x182 * x231 * x232)
    result[1, 0, 7] = numpy.sum(x225 * x227 * x232)
    result[1, 0, 8] = numpy.sum(x108 * x220 * x232)
    result[1, 0, 9] = numpy.sum(x111 * x222 * x232)
    result[1, 0, 10] = numpy.sum(x179 * x233 * x235)
    result[1, 0, 11] = numpy.sum(x231 * x233 * x236)
    result[1, 0, 12] = numpy.sum(x100 * x225 * x233)
    result[1, 0, 13] = numpy.sum(x111 * x220 * x233)
    result[1, 0, 14] = numpy.sum(x222 * x233 * x237)
    result[1, 1, 0] = numpy.sum(x242 * x245 * x85)
    result[1, 1, 1] = numpy.sum(x241 * x247 * x248)
    result[1, 1, 2] = numpy.sum(x105 * x241 * x250)
    result[1, 1, 3] = numpy.sum(x253 * x254 * x40)
    result[1, 1, 4] = numpy.sum(x227 * x247 * x255)
    result[1, 1, 5] = numpy.sum(x108 * x244 * x40)
    result[1, 1, 6] = numpy.sum(x17 * x248 * x259)
    result[1, 1, 7] = numpy.sum(x227 * x253 * x260)
    result[1, 1, 8] = numpy.sum(x108 * x247 * x260)
    result[1, 1, 9] = numpy.sum(x149 * x17 * x244)
    result[1, 1, 10] = numpy.sum(x261 * x262 * x85)
    result[1, 1, 11] = numpy.sum(x105 * x259 * x264)
    result[1, 1, 12] = numpy.sum(x108 * x253 * x46)
    result[1, 1, 13] = numpy.sum(x149 * x247 * x46)
    result[1, 1, 14] = numpy.sum(x118 * x244 * x262)
    result[1, 2, 0] = numpy.sum(x155 * x214 * x242 * x73)
    result[1, 2, 1] = numpy.sum(x220 * x241 * x265)
    result[1, 2, 2] = numpy.sum(x159 * x241 * x266)
    result[1, 2, 3] = numpy.sum(x225 * x267 * x40)
    result[1, 2, 4] = numpy.sum(x162 * x220 * x40)
    result[1, 2, 5] = numpy.sum(x166 * x222 * x40)
    result[1, 2, 6] = numpy.sum(x17 * x231 * x265)
    result[1, 2, 7] = numpy.sum(x162 * x17 * x225)
    result[1, 2, 8] = numpy.sum(x166 * x220 * x260)
    result[1, 2, 9] = numpy.sum(x17 * x172 * x266)
    result[1, 2, 10] = numpy.sum(x156 * x235 * x262)
    result[1, 2, 11] = numpy.sum(x159 * x231 * x263)
    result[1, 2, 12] = numpy.sum(x166 * x225 * x46)
    result[1, 2, 13] = numpy.sum(x172 * x220 * x263)
    result[1, 2, 14] = numpy.sum(x173 * x222 * x262)
    result[1, 3, 0] = numpy.sum(x179 * x269 * x272)
    result[1, 3, 1] = numpy.sum(x182 * x238 * x274)
    result[1, 3, 2] = numpy.sum(x236 * x238 * x269)
    result[1, 3, 3] = numpy.sum(x184 * x276 * x277)
    result[1, 3, 4] = numpy.sum(x176 * x227 * x274)
    result[1, 3, 5] = numpy.sum(x100 * x176 * x269)
    result[1, 3, 6] = numpy.sum(x15 * x182 * x281)
    result[1, 3, 7] = numpy.sum(x105 * x276 * x282)
    result[1, 3, 8] = numpy.sum(x108 * x15 * x274)
    result[1, 3, 9] = numpy.sum(x111 * x15 * x269)
    result[1, 3, 10] = numpy.sum(x10 * x286)
    result[1, 3, 11] = numpy.sum(x281 * x288 * x289)
    result[1, 3, 12] = numpy.sum(x100 * x11 * x276)
    result[1, 3, 13] = numpy.sum(x11 * x111 * x274)
    result[1, 3, 14] = numpy.sum(x11 * x237 * x269)
    result[1, 4, 0] = numpy.sum(x156 * x245 * x272)
    result[1, 4, 1] = numpy.sum(x238 * x247 * x265)
    result[1, 4, 2] = numpy.sum(x159 * x238 * x249)
    result[1, 4, 3] = numpy.sum(x176 * x253 * x267)
    result[1, 4, 4] = numpy.sum(x162 * x176 * x247)
    result[1, 4, 5] = numpy.sum(x166 * x176 * x244)
    result[1, 4, 6] = numpy.sum(x15 * x259 * x265)
    result[1, 4, 7] = numpy.sum(x161 * x253 * x282)
    result[1, 4, 8] = numpy.sum(x15 * x247 * x290)
    result[1, 4, 9] = numpy.sum(x15 * x172 * x249)
    result[1, 4, 10] = numpy.sum(x10 * x261 * x291)
    result[1, 4, 11] = numpy.sum(x159 * x259 * x292)
    result[1, 4, 12] = numpy.sum(x11 * x166 * x253)
    result[1, 4, 13] = numpy.sum(x172 * x247 * x292)
    result[1, 4, 14] = numpy.sum(x11 * x173 * x245)
    result[1, 5, 0] = numpy.sum(x222 * x272 * x293)
    result[1, 5, 1] = numpy.sum(x203 * x220 * x238)
    result[1, 5, 2] = numpy.sum(x205 * x238 * x294)
    result[1, 5, 3] = numpy.sum(x201 * x225 * x277)
    result[1, 5, 4] = numpy.sum(x176 * x207 * x220)
    result[1, 5, 5] = numpy.sum(x211 * x222 * x277)
    result[1, 5, 6] = numpy.sum(x15 * x203 * x231)
    result[1, 5, 7] = numpy.sum(x15 * x207 * x225)
    result[1, 5, 8] = numpy.sum(x210 * x220 * x282)
    result[1, 5, 9] = numpy.sum(x15 * x212 * x294)
    result[1, 5, 10] = numpy.sum(x11 * x235 * x293)
    result[1, 5, 11] = numpy.sum(x205 * x231 * x295)
    result[1, 5, 12] = numpy.sum(x11 * x211 * x226)
    result[1, 5, 13] = numpy.sum(x212 * x220 * x295)
    result[1, 5, 14] = numpy.sum(x10 * x213 * x214 * x297)
    result[2, 0, 0] = numpy.sum(x218 * x298)
    result[2, 0, 1] = numpy.sum(x221 * x298 * x77)
    result[2, 0, 2] = numpy.sum(x206 * x217 * x300)
    result[2, 0, 3] = numpy.sum(x301 * x42 * x93)
    result[2, 0, 4] = numpy.sum(x107 * x302 * x42)
    result[2, 0, 5] = numpy.sum(x208 * x306 * x42)
    result[2, 0, 6] = numpy.sum(x104 * x232 * x301)
    result[2, 0, 7] = numpy.sum(x106 * x232 * x300)
    result[2, 0, 8] = numpy.sum(x107 * x232 * x307)
    result[2, 0, 9] = numpy.sum(x206 * x232 * x311)
    result[2, 0, 10] = numpy.sum(x115 * x233 * x312)
    result[2, 0, 11] = numpy.sum(x104 * x233 * x300)
    result[2, 0, 12] = numpy.sum(x233 * x306 * x92)
    result[2, 0, 13] = numpy.sum(x233 * x311 * x313)
    result[2, 0, 14] = numpy.sum(x233 * x316 * x86)
    result[2, 1, 0] = numpy.sum(x196 * x242 * x298)
    result[2, 1, 1] = numpy.sum(x132 * x241 * x317)
    result[2, 1, 2] = numpy.sum(x241 * x300 * x318)
    result[2, 1, 3] = numpy.sum(x140 * x301 * x40)
    result[2, 1, 4] = numpy.sum(x141 * x302 * x40)
    result[2, 1, 5] = numpy.sum(x129 * x307 * x40)
    result[2, 1, 6] = numpy.sum(x147 * x17 * x317)
    result[2, 1, 7] = numpy.sum(x140 * x260 * x300)
    result[2, 1, 8] = numpy.sum(x141 * x17 * x307)
    result[2, 1, 9] = numpy.sum(x17 * x311 * x318)
    result[2, 1, 10] = numpy.sum(x150 * x262 * x301)
    result[2, 1, 11] = numpy.sum(x146 * x264 * x300)
    result[2, 1, 12] = numpy.sum(x140 * x305 * x46)
    result[2, 1, 13] = numpy.sum(x131 * x264 * x311)
    result[2, 1, 14] = numpy.sum(x129 * x262 * x315)
    result[2, 2, 0] = numpy.sum(x242 * x321 * x86)
    result[2, 2, 1] = numpy.sum(x107 * x241 * x323)
    result[2, 2, 2] = numpy.sum(x241 * x325 * x326)
    result[2, 2, 3] = numpy.sum(x106 * x320 * x40)
    result[2, 2, 4] = numpy.sum(x107 * x255 * x327)
    result[2, 2, 5] = numpy.sum(x331 * x40 * x86)
    result[2, 2, 6] = numpy.sum(x103 * x17 * x322)
    result[2, 2, 7] = numpy.sum(x106 * x260 * x325)
    result[2, 2, 8] = numpy.sum(x107 * x260 * x331)
    result[2, 2, 9] = numpy.sum(x17 * x326 * x335)
    result[2, 2, 10] = numpy.sum(x114 * x262 * x320)
    result[2, 2, 11] = numpy.sum(x103 * x263 * x325)
    result[2, 2, 12] = numpy.sum(x106 * x330 * x46)
    result[2, 2, 13] = numpy.sum(x107 * x264 * x335)
    result[2, 2, 14] = numpy.sum(x262 * x336 * x86)
    result[2, 3, 0] = numpy.sum(x174 * x272 * x337)
    result[2, 3, 1] = numpy.sum(x181 * x238 * x338)
    result[2, 3, 2] = numpy.sum(x183 * x238 * x300)
    result[2, 3, 3] = numpy.sum(x186 * x277 * x312)
    result[2, 3, 4] = numpy.sum(x176 * x181 * x302)
    result[2, 3, 5] = numpy.sum(x174 * x277 * x306)
    result[2, 3, 6] = numpy.sum(x15 * x189 * x338)
    result[2, 3, 7] = numpy.sum(x186 * x282 * x300)
    result[2, 3, 8] = numpy.sum(x181 * x282 * x305)
    result[2, 3, 9] = numpy.sum(x15 * x183 * x311)
    result[2, 3, 10] = numpy.sum(x10 * x191 * x285 * x298)
    result[2, 3, 11] = numpy.sum(x189 * x295 * x300)
    result[2, 3, 12] = numpy.sum(x11 * x187 * x306)
    result[2, 3, 13] = numpy.sum(x181 * x295 * x311)
    result[2, 3, 14] = numpy.sum(x11 * x174 * x316)
    result[2, 4, 0] = numpy.sum(x129 * x272 * x321)
    result[2, 4, 1] = numpy.sum(x131 * x238 * x323)
    result[2, 4, 2] = numpy.sum(x238 * x318 * x325)
    result[2, 4, 3] = numpy.sum(x140 * x176 * x320)
    result[2, 4, 4] = numpy.sum(x141 * x176 * x327)
    result[2, 4, 5] = numpy.sum(x129 * x176 * x331)
    result[2, 4, 6] = numpy.sum(x146 * x15 * x323)
    result[2, 4, 7] = numpy.sum(x139 * x282 * x339)
    result[2, 4, 8] = numpy.sum(x141 * x282 * x330)
    result[2, 4, 9] = numpy.sum(x15 * x318 * x335)
    result[2, 4, 10] = numpy.sum(x11 * x150 * x321)
    result[2, 4, 11] = numpy.sum(x147 * x292 * x325)
    result[2, 4, 12] = numpy.sum(x11 * x140 * x330)
    result[2, 4, 13] = numpy.sum(x132 * x292 * x335)
    result[2, 4, 14] = numpy.sum(x10 * x123 * x341)
    result[2, 5, 0] = numpy.sum(x202 * x272 * x344)
    result[2, 5, 1] = numpy.sum(x238 * x313 * x343)
    result[2, 5, 2] = numpy.sum(x206 * x238 * x346)
    result[2, 5, 3] = numpy.sum(x176 * x344 * x92)
    result[2, 5, 4] = numpy.sum(x107 * x176 * x347)
    result[2, 5, 5] = numpy.sum(x277 * x350 * x86)
    result[2, 5, 6] = numpy.sum(x104 * x15 * x343)
    result[2, 5, 7] = numpy.sum(x282 * x346 * x89)
    result[2, 5, 8] = numpy.sum(x107 * x282 * x349)
    result[2, 5, 9] = numpy.sum(x15 * x206 * x354)
    result[2, 5, 10] = numpy.sum(x11 * x115 * x344)
    result[2, 5, 11] = numpy.sum(x104 * x11 * x346)
    result[2, 5, 12] = numpy.sum(x11 * x349 * x93)
    result[2, 5, 13] = numpy.sum(x289 * x354 * x355)
    result[2, 5, 14] = numpy.sum(x10 * x357)
    result[3, 0, 0] = numpy.sum(x358 * x362 * x85)
    result[3, 0, 1] = numpy.sum(x182 * x216 * x364)
    result[3, 0, 2] = numpy.sum(x216 * x236 * x360)
    result[3, 0, 3] = numpy.sum(x184 * x30 * x369)
    result[3, 0, 4] = numpy.sum(x227 * x30 * x364)
    result[3, 0, 5] = numpy.sum(x100 * x30 * x360)
    result[3, 0, 6] = numpy.sum(x182 * x370 * x371)
    result[3, 0, 7] = numpy.sum(x227 * x368 * x371)
    result[3, 0, 8] = numpy.sum(x108 * x364 * x371)
    result[3, 0, 9] = numpy.sum(x111 * x360 * x371)
    result[3, 0, 10] = numpy.sum(x179 * x372 * x373)
    result[3, 0, 11] = numpy.sum(x105 * x370 * x374)
    result[3, 0, 12] = numpy.sum(x100 * x368 * x372)
    result[3, 0, 13] = numpy.sum(x111 * x364 * x372)
    result[3, 0, 14] = numpy.sum(x119 * x361 * x372)
    result[3, 1, 0] = numpy.sum(x375 * x377 * x85)
    result[3, 1, 1] = numpy.sum(x240 * x248 * x379)
    result[3, 1, 2] = numpy.sum(x105 * x380 * x381)
    result[3, 1, 3] = numpy.sum(x254 * x28 * x384)
    result[3, 1, 4] = numpy.sum(x227 * x379 * x385)
    result[3, 1, 5] = numpy.sum(x108 * x28 * x376)
    result[3, 1, 6] = numpy.sum(x388 * x390 * x85)
    result[3, 1, 7] = numpy.sum(x227 * x384 * x391)
    result[3, 1, 8] = numpy.sum(x108 * x379 * x391)
    result[3, 1, 9] = numpy.sum(x149 * x26 * x376)
    result[3, 1, 10] = numpy.sum(x393 * x394)
    result[3, 1, 11] = numpy.sum(x288 * x3 * x395)
    result[3, 1, 12] = numpy.sum(x108 * x19 * x384)
    result[3, 1, 13] = numpy.sum(x149 * x19 * x379)
    result[3, 1, 14] = numpy.sum(x118 * x19 * x377)
    result[3, 2, 0] = numpy.sum(x156 * x375 * x396)
    result[3, 2, 1] = numpy.sum(x240 * x265 * x364)
    result[3, 2, 2] = numpy.sum(x159 * x240 * x397)
    result[3, 2, 3] = numpy.sum(x156 * x368 * x398)
    result[3, 2, 4] = numpy.sum(x162 * x28 * x364)
    result[3, 2, 5] = numpy.sum(x166 * x28 * x360)
    result[3, 2, 6] = numpy.sum(x156 * x370 * x390)
    result[3, 2, 7] = numpy.sum(x162 * x26 * x368)
    result[3, 2, 8] = numpy.sum(x26 * x290 * x364)
    result[3, 2, 9] = numpy.sum(x172 * x26 * x397)
    result[3, 2, 10] = numpy.sum(x291 * x3 * x373)
    result[3, 2, 11] = numpy.sum(x159 * x370 * x399)
    result[3, 2, 12] = numpy.sum(x166 * x19 * x368)
    result[3, 2, 13] = numpy.sum(x172 * x364 * x399)
    result[3, 2, 14] = numpy.sum(x173 * x19 * x396)
    result[3, 3, 0] = numpy.sum(x184 * x400 * x402)
    result[3, 3, 1] = numpy.sum(x404 * x405 * x85)
    result[3, 3, 2] = numpy.sum(x105 * x400 * x405)
    result[3, 3, 3] = numpy.sum(x184 * x407 * x408)
    result[3, 3, 4] = numpy.sum(x105 * x404 * x409)
    result[3, 3, 5] = numpy.sum(x100 * x270 * x400)
    result[3, 3, 6] = numpy.sum(x410 * x412)
    result[3, 3, 7] = numpy.sum(x407 * x410 * x95)
    result[3, 3, 8] = numpy.sum(x108 * x404 * x9)
    result[3, 3, 9] = numpy.sum(x111 * x400 * x9)
    result[3, 3, 10] = numpy.sum(
        x285
        * (
            x0 * (2.0 * x279 + 2.0 * x280 + 5.0 * x386 + 2.0 * x387 + 3.0 * x406)
            + x411 * x76
        )
    )
    result[3, 3, 11] = numpy.sum(x288 * x412)
    result[3, 3, 12] = numpy.sum(x100 * x407 * x8)
    result[3, 3, 13] = numpy.sum(x111 * x404 * x8)
    result[3, 3, 14] = numpy.sum(x119 * x400 * x413)
    result[3, 4, 0] = numpy.sum(x156 * x377 * x401)
    result[3, 4, 1] = numpy.sum(x156 * x379 * x415)
    result[3, 4, 2] = numpy.sum(x159 * x376 * x414)
    result[3, 4, 3] = numpy.sum(x156 * x384 * x409)
    result[3, 4, 4] = numpy.sum(x161 * x379 * x409)
    result[3, 4, 5] = numpy.sum(x166 * x270 * x376)
    result[3, 4, 6] = numpy.sum(x154 * x395 * x410)
    result[3, 4, 7] = numpy.sum(x162 * x384 * x9)
    result[3, 4, 8] = numpy.sum(x290 * x379 * x9)
    result[3, 4, 9] = numpy.sum(x172 * x380 * x9)
    result[3, 4, 10] = numpy.sum(x155 * x393)
    result[3, 4, 11] = numpy.sum(x159 * x388 * x416)
    result[3, 4, 12] = numpy.sum(x166 * x384 * x8)
    result[3, 4, 13] = numpy.sum(x172 * x379 * x416)
    result[3, 4, 14] = numpy.sum(x173 * x377 * x8)
    result[3, 5, 0] = numpy.sum(x201 * x360 * x402)
    result[3, 5, 1] = numpy.sum(x203 * x271 * x364)
    result[3, 5, 2] = numpy.sum(x205 * x360 * x405)
    result[3, 5, 3] = numpy.sum(x201 * x368 * x408)
    result[3, 5, 4] = numpy.sum(x207 * x270 * x364)
    result[3, 5, 5] = numpy.sum(x210 * x361 * x408)
    result[3, 5, 6] = numpy.sum(x203 * x370 * x9)
    result[3, 5, 7] = numpy.sum(x207 * x368 * x9)
    result[3, 5, 8] = numpy.sum(x210 * x364 * x417)
    result[3, 5, 9] = numpy.sum(x212 * x360 * x418)
    result[3, 5, 10] = numpy.sum(x201 * x373 * x419)
    result[3, 5, 11] = numpy.sum(x205 * x370 * x420)
    result[3, 5, 12] = numpy.sum(x211 * x369 * x8)
    result[3, 5, 13] = numpy.sum(x212 * x364 * x420)
    result[3, 5, 14] = numpy.sum(x213 * x362 * x8)
    result[4, 0, 0] = numpy.sum(x214 * x298 * x358 * x74)
    result[4, 0, 1] = numpy.sum(x216 * x220 * x338)
    result[4, 0, 2] = numpy.sum(x216 * x294 * x300)
    result[4, 0, 3] = numpy.sum(x226 * x30 * x312)
    result[4, 0, 4] = numpy.sum(x220 * x30 * x302)
    result[4, 0, 5] = numpy.sum(x222 * x30 * x421)
    result[4, 0, 6] = numpy.sum(x231 * x338 * x371)
    result[4, 0, 7] = numpy.sum(x225 * x302 * x371)
    result[4, 0, 8] = numpy.sum(x220 * x307 * x371)
    result[4, 0, 9] = numpy.sum(x294 * x311 * x371)
    result[4, 0, 10] = numpy.sum(x235 * x337 * x372)
    result[4, 0, 11] = numpy.sum(x231 * x300 * x374)
    result[4, 0, 12] = numpy.sum(x226 * x306 * x372)
    result[4, 0, 13] = numpy.sum(x220 * x311 * x374)
    result[4, 0, 14] = numpy.sum(x222 * x316 * x372)
    result[4, 1, 0] = numpy.sum(x245 * x301 * x375)
    result[4, 1, 1] = numpy.sum(x247 * x317 * x381)
    result[4, 1, 2] = numpy.sum(x240 * x250 * x300)
    result[4, 1, 3] = numpy.sum(x253 * x301 * x398)
    result[4, 1, 4] = numpy.sum(x247 * x302 * x385)
    result[4, 1, 5] = numpy.sum(x244 * x28 * x307)
    result[4, 1, 6] = numpy.sum(x259 * x301 * x390)
    result[4, 1, 7] = numpy.sum(x253 * x302 * x391)
    result[4, 1, 8] = numpy.sum(x247 * x307 * x391)
    result[4, 1, 9] = numpy.sum(x250 * x26 * x311)
    result[4, 1, 10] = numpy.sum(x261 * x287 * x298 * x394)
    result[4, 1, 11] = numpy.sum(x259 * x300 * x422)
    result[4, 1, 12] = numpy.sum(x19 * x253 * x307)
    result[4, 1, 13] = numpy.sum(x247 * x311 * x422)
    result[4, 1, 14] = numpy.sum(x19 * x245 * x315)
    result[4, 2, 0] = numpy.sum(x222 * x321 * x375)
    result[4, 2, 1] = numpy.sum(x220 * x240 * x323)
    result[4, 2, 2] = numpy.sum(x266 * x325 * x381)
    result[4, 2, 3] = numpy.sum(x225 * x28 * x423)
    result[4, 2, 4] = numpy.sum(x220 * x327 * x385)
    result[4, 2, 5] = numpy.sum(x222 * x28 * x331)
    result[4, 2, 6] = numpy.sum(x231 * x26 * x323)
    result[4, 2, 7] = numpy.sum(x225 * x327 * x391)
    result[4, 2, 8] = numpy.sum(x220 * x331 * x391)
    result[4, 2, 9] = numpy.sum(x222 * x335 * x390)
    result[4, 2, 10] = numpy.sum(x19 * x235 * x321)
    result[4, 2, 11] = numpy.sum(x231 * x325 * x422)
    result[4, 2, 12] = numpy.sum(x19 * x225 * x331)
    result[4, 2, 13] = numpy.sum(x220 * x335 * x422)
    result[4, 2, 14] = numpy.sum(x214 * x341 * x394)
    result[4, 3, 0] = numpy.sum(x269 * x312 * x402)
    result[4, 3, 1] = numpy.sum(x274 * x301 * x405)
    result[4, 3, 2] = numpy.sum(x269 * x300 * x405)
    result[4, 3, 3] = numpy.sum(x276 * x312 * x408)
    result[4, 3, 4] = numpy.sum(x270 * x274 * x302)
    result[4, 3, 5] = numpy.sum(x269 * x306 * x408)
    result[4, 3, 6] = numpy.sum(x281 * x298 * x410 * x80)
    result[4, 3, 7] = numpy.sum(x276 * x302 * x9)
    result[4, 3, 8] = numpy.sum(x274 * x307 * x9)
    result[4, 3, 9] = numpy.sum(x269 * x311 * x418)
    result[4, 3, 10] = numpy.sum(x286 * x298)
    result[4, 3, 11] = numpy.sum(x281 * x300 * x420)
    result[4, 3, 12] = numpy.sum(x276 * x421 * x8)
    result[4, 3, 13] = numpy.sum(x274 * x311 * x420)
    result[4, 3, 14] = numpy.sum(x269 * x316 * x8)
    result[4, 4, 0] = numpy.sum(x245 * x320 * x401)
    result[4, 4, 1] = numpy.sum(x247 * x271 * x323)
    result[4, 4, 2] = numpy.sum(x250 * x271 * x325)
    result[4, 4, 3] = numpy.sum(x253 * x270 * x423)
    result[4, 4, 4] = numpy.sum(x247 * x339 * x409)
    result[4, 4, 5] = numpy.sum(x244 * x330 * x409)
    result[4, 4, 6] = numpy.sum(x259 * x323 * x9)
    result[4, 4, 7] = numpy.sum(x253 * x327 * x424)
    result[4, 4, 8] = numpy.sum(x247 * x331 * x424)
    result[4, 4, 9] = numpy.sum(x250 * x335 * x9)
    result[4, 4, 10] = numpy.sum(x261 * x321 * x8)
    result[4, 4, 11] = numpy.sum(x259 * x325 * x425)
    result[4, 4, 12] = numpy.sum(x253 * x331 * x8)
    result[4, 4, 13] = numpy.sum(x247 * x335 * x425)
    result[4, 4, 14] = numpy.sum(x245 * x336 * x8)
    result[4, 5, 0] = numpy.sum(x222 * x344 * x402)
    result[4, 5, 1] = numpy.sum(x220 * x343 * x405)
    result[4, 5, 2] = numpy.sum(x222 * x346 * x405)
    result[4, 5, 3] = numpy.sum(x225 * x344 * x408)
    result[4, 5, 4] = numpy.sum(x220 * x346 * x409)
    result[4, 5, 5] = numpy.sum(x222 * x350 * x408)
    result[4, 5, 6] = numpy.sum(x231 * x343 * x418)
    result[4, 5, 7] = numpy.sum(x225 * x347 * x9)
    result[4, 5, 8] = numpy.sum(x220 * x349 * x417)
    result[4, 5, 9] = numpy.sum(x214 * x354 * x426 * x80)
    result[4, 5, 10] = numpy.sum(x235 * x344 * x419)
    result[4, 5, 11] = numpy.sum(x231 * x346 * x420)
    result[4, 5, 12] = numpy.sum(x226 * x350 * x8)
    result[4, 5, 13] = numpy.sum(x220 * x354 * x420)
    result[4, 5, 14] = numpy.sum(x214 * x357)
    result[5, 0, 0] = numpy.sum(x202 * x358 * x429)
    result[5, 0, 1] = numpy.sum(x216 * x313 * x428)
    result[5, 0, 2] = numpy.sum(x206 * x216 * x431)
    result[5, 0, 3] = numpy.sum(x30 * x429 * x92)
    result[5, 0, 4] = numpy.sum(x30 * x431 * x432)
    result[5, 0, 5] = numpy.sum(x208 * x30 * x437)
    result[5, 0, 6] = numpy.sum(x104 * x371 * x428)
    result[5, 0, 7] = numpy.sum(x106 * x371 * x431)
    result[5, 0, 8] = numpy.sum(x371 * x432 * x436)
    result[5, 0, 9] = numpy.sum(x206 * x371 * x438)
    result[5, 0, 10] = numpy.sum(x115 * x372 * x429)
    result[5, 0, 11] = numpy.sum(x104 * x372 * x431)
    result[5, 0, 12] = numpy.sum(x372 * x437 * x92)
    result[5, 0, 13] = numpy.sum(x107 * x374 * x438)
    result[5, 0, 14] = numpy.sum(x372 * x440 * x86)
    result[5, 1, 0] = numpy.sum(x129 * x375 * x441)
    result[5, 1, 1] = numpy.sum(x132 * x240 * x442)
    result[5, 1, 2] = numpy.sum(x240 * x318 * x431)
    result[5, 1, 3] = numpy.sum(x140 * x28 * x428)
    result[5, 1, 4] = numpy.sum(x142 * x28 * x431)
    result[5, 1, 5] = numpy.sum(x129 * x398 * x436)
    result[5, 1, 6] = numpy.sum(x147 * x26 * x442)
    result[5, 1, 7] = numpy.sum(x140 * x391 * x431)
    result[5, 1, 8] = numpy.sum(x142 * x26 * x436)
    result[5, 1, 9] = numpy.sum(x129 * x390 * x438)
    result[5, 1, 10] = numpy.sum(x150 * x19 * x441)
    result[5, 1, 11] = numpy.sum(x147 * x399 * x431)
    result[5, 1, 12] = numpy.sum(x140 * x19 * x436)
    result[5, 1, 13] = numpy.sum(x132 * x399 * x438)
    result[5, 1, 14] = numpy.sum(x123 * x3 * x340 * x439)
    result[5, 2, 0] = numpy.sum(x375 * x444 * x86)
    result[5, 2, 1] = numpy.sum(x107 * x381 * x445)
    result[5, 2, 2] = numpy.sum(x240 * x326 * x447)
    result[5, 2, 3] = numpy.sum(x106 * x28 * x443)
    result[5, 2, 4] = numpy.sum(x385 * x432 * x447)
    result[5, 2, 5] = numpy.sum(x398 * x450 * x86)
    result[5, 2, 6] = numpy.sum(x103 * x389 * x443)
    result[5, 2, 7] = numpy.sum(x106 * x391 * x447)
    result[5, 2, 8] = numpy.sum(x391 * x432 * x450)
    result[5, 2, 9] = numpy.sum(x390 * x453 * x86)
    result[5, 2, 10] = numpy.sum(x114 * x19 * x444)
    result[5, 2, 11] = numpy.sum(x103 * x399 * x447)
    result[5, 2, 12] = numpy.sum(x106 * x19 * x450)
    result[5, 2, 13] = numpy.sum(x3 * x355 * x454)
    result[5, 2, 14] = numpy.sum(x394 * x456)
    result[5, 3, 0] = numpy.sum(x174 * x402 * x429)
    result[5, 3, 1] = numpy.sum(x181 * x405 * x428)
    result[5, 3, 2] = numpy.sum(x183 * x271 * x431)
    result[5, 3, 3] = numpy.sum(x186 * x408 * x429)
    result[5, 3, 4] = numpy.sum(x181 * x409 * x431)
    result[5, 3, 5] = numpy.sum(x174 * x408 * x437)
    result[5, 3, 6] = numpy.sum(x189 * x418 * x428)
    result[5, 3, 7] = numpy.sum(x186 * x417 * x431)
    result[5, 3, 8] = numpy.sum(x181 * x417 * x436)
    result[5, 3, 9] = numpy.sum(x183 * x438 * x9)
    result[5, 3, 10] = numpy.sum(x191 * x419 * x429)
    result[5, 3, 11] = numpy.sum(x189 * x420 * x431)
    result[5, 3, 12] = numpy.sum(x187 * x437 * x8)
    result[5, 3, 13] = numpy.sum(x181 * x420 * x438)
    result[5, 3, 14] = numpy.sum(x174 * x440 * x8)
    result[5, 4, 0] = numpy.sum(x129 * x401 * x444)
    result[5, 4, 1] = numpy.sum(x132 * x414 * x443)
    result[5, 4, 2] = numpy.sum(x129 * x415 * x447)
    result[5, 4, 3] = numpy.sum(x140 * x270 * x443)
    result[5, 4, 4] = numpy.sum(x141 * x409 * x447)
    result[5, 4, 5] = numpy.sum(x129 * x409 * x450)
    result[5, 4, 6] = numpy.sum(x147 * x445 * x9)
    result[5, 4, 7] = numpy.sum(x140 * x424 * x447)
    result[5, 4, 8] = numpy.sum(x142 * x450 * x9)
    result[5, 4, 9] = numpy.sum(x120 * x426 * x454)
    result[5, 4, 10] = numpy.sum(x150 * x444 * x8)
    result[5, 4, 11] = numpy.sum(x147 * x416 * x447)
    result[5, 4, 12] = numpy.sum(x140 * x450 * x8)
    result[5, 4, 13] = numpy.sum(x132 * x416 * x453)
    result[5, 4, 14] = numpy.sum(x123 * x456)
    result[5, 5, 0] = numpy.sum(x402 * x457 * x458)
    result[5, 5, 1] = numpy.sum(x107 * x405 * x457)
    result[5, 5, 2] = numpy.sum(x405 * x460 * x86)
    result[5, 5, 3] = numpy.sum(x270 * x457 * x93)
    result[5, 5, 4] = numpy.sum(x107 * x409 * x460)
    result[5, 5, 5] = numpy.sum(x408 * x458 * x462)
    result[5, 5, 6] = numpy.sum(x104 * x457 * x9)
    result[5, 5, 7] = numpy.sum(x106 * x460 * x9)
    result[5, 5, 8] = numpy.sum(x426 * x462 * x76 * x94)
    result[5, 5, 9] = numpy.sum(x426 * x464)
    result[5, 5, 10] = numpy.sum(x115 * x413 * x457)
    result[5, 5, 11] = numpy.sum(x104 * x460 * x8)
    result[5, 5, 12] = numpy.sum(x462 * x8 * x93)
    result[5, 5, 13] = numpy.sum(x355 * x464)
    result[5, 5, 14] = numpy.sum(
        x297
        * (
            x0 * (2.0 * x352 + 2.0 * x353 + 5.0 * x451 + 2.0 * x452 + 3.0 * x461)
            + x463 * x83
        )
    )
    return result


def quadrupole3d_30(ax, da, A, bx, db, B, R):
    """Cartesian 3D (fs) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 10, 1), dtype=float)

    x0 = (ax + bx) ** (-1.0)
    x1 = x0 * (ax * A[0] + bx * B[0])
    x2 = -x1
    x3 = x2 + A[0]
    x4 = x2 + R[0]
    x5 = x4**2
    x6 = 3.0 * x0
    x7 = x3 * x4
    x8 = x6 + 4.0 * x7
    x9 = x0 * (-2.0 * x1 + A[0] + R[0])
    x10 = x0 + 2.0 * x7
    x11 = x10 * x4
    x12 = x11 + x9
    x13 = 2.0 * x3
    x14 = x0 * (2.0 * x5 + x8) + x12 * x13
    x15 = x10 * x3
    x16 = 2.0 * x0
    x17 = 3.872983346207417
    x18 = ax * bx * x0
    x19 = (
        5.568327996831708
        * da
        * db
        * numpy.exp(-x18 * ((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2 + (A[2] - B[2]) ** 2))
    )
    x20 = numpy.sqrt(x0) * x19
    x21 = x0 * x20
    x22 = x17 * x21
    x23 = 0.01666666666666667 * x22
    x24 = x0 * (ax * A[1] + bx * B[1])
    x25 = -x24
    x26 = x25 + A[1]
    x27 = 1.732050807568877
    x28 = x26 * x27
    x29 = 0.08333333333333333 * x21
    x30 = x14 * x29
    x31 = x0 * (ax * A[2] + bx * B[2])
    x32 = -x31
    x33 = x32 + A[2]
    x34 = x27 * x33
    x35 = x26**2
    x36 = 0.5 * x0
    x37 = x35 + x36
    x38 = x0**1.5 * x19
    x39 = 0.1666666666666667 * x27
    x40 = x38 * x39
    x41 = x12 * x40
    x42 = x20 * x33 * x36
    x43 = x33**2
    x44 = x36 + x43
    x45 = 1.5 * x0
    x46 = x26 * (x35 + x45)
    x47 = x38 * (x36 + x5)
    x48 = 0.06666666666666667 * x17
    x49 = x47 * x48
    x50 = 0.3333333333333333 * x47
    x51 = x33 * (x43 + x45)
    x52 = x25 + R[1]
    x53 = x3**2
    x54 = x15 + x9
    x55 = x23 * (x0 * (2.0 * x53 + x8) + x13 * x54)
    x56 = x26 * x52
    x57 = x36 + x56
    x58 = x38 * x57
    x59 = x39 * x58
    x60 = x21 * x39
    x61 = x33 * x60
    x62 = x0 * (-2.0 * x24 + A[1] + R[1])
    x63 = x0 + 2.0 * x56
    x64 = x26 * x63
    x65 = x62 + x64
    x66 = x38 * (x36 + x7)
    x67 = x39 * x66
    x68 = 0.3333333333333333 * x27
    x69 = x44 * x68
    x70 = 4.0 * x56 + x6
    x71 = 2.0 * x26
    x72 = x0 * (2.0 * x35 + x70) + x65 * x71
    x73 = x23 * x4
    x74 = 0.06666666666666667 * x22 * x4
    x75 = x32 + R[2]
    x76 = x26 * x60
    x77 = x33 * x75
    x78 = x36 + x77
    x79 = x40 * x78
    x80 = x37 * x68
    x81 = x0 * (-2.0 * x31 + A[2] + R[2])
    x82 = x0 + 2.0 * x77
    x83 = x33 * x82
    x84 = x81 + x83
    x85 = x38 * x78
    x86 = x6 + 4.0 * x77
    x87 = 2.0 * x33
    x88 = x0 * (2.0 * x43 + x86) + x84 * x87
    x89 = x3 * (x45 + x53)
    x90 = x52**2
    x91 = x38 * (x36 + x90)
    x92 = x48 * x91
    x93 = x36 + x53
    x94 = x52 * x63
    x95 = x62 + x94
    x96 = x40 * x95
    x97 = 0.3333333333333333 * x93
    x98 = x0 * (x70 + 2.0 * x90) + x71 * x95
    x99 = x29 * x98
    x100 = x27 * x3
    x101 = x68 * x93
    x102 = x3 * x60
    x103 = x75**2
    x104 = x38 * (x103 + x36)
    x105 = x104 * x48
    x106 = x75 * x82
    x107 = x106 + x81
    x108 = x107 * x40
    x109 = x0 * (2.0 * x103 + x86) + x107 * x87
    x110 = x109 * x29

    # 60 item(s)
    result[0, 0, 0] = numpy.sum(-x23 * (x14 * x3 + x16 * (x11 + x15 + 2.0 * x9)))
    result[0, 1, 0] = numpy.sum(-x28 * x30)
    result[0, 2, 0] = numpy.sum(-x30 * x34)
    result[0, 3, 0] = numpy.sum(-x37 * x41)
    result[0, 4, 0] = numpy.sum(-x12 * x26 * x42)
    result[0, 5, 0] = numpy.sum(-x41 * x44)
    result[0, 6, 0] = numpy.sum(-x46 * x49)
    result[0, 7, 0] = numpy.sum(-x34 * x37 * x50)
    result[0, 8, 0] = numpy.sum(-x28 * x44 * x50)
    result[0, 9, 0] = numpy.sum(-x49 * x51)
    result[1, 0, 0] = numpy.sum(-x52 * x55)
    result[1, 1, 0] = numpy.sum(-x54 * x59)
    result[1, 2, 0] = numpy.sum(-x52 * x54 * x61)
    result[1, 3, 0] = numpy.sum(-x65 * x67)
    result[1, 4, 0] = numpy.sum(-x33 * x57 * x66)
    result[1, 5, 0] = numpy.sum(-x52 * x66 * x69)
    result[1, 6, 0] = numpy.sum(-x72 * x73)
    result[1, 7, 0] = numpy.sum(-x4 * x61 * x65)
    result[1, 8, 0] = numpy.sum(-x4 * x58 * x69)
    result[1, 9, 0] = numpy.sum(-x51 * x52 * x74)
    result[2, 0, 0] = numpy.sum(-x55 * x75)
    result[2, 1, 0] = numpy.sum(-x54 * x75 * x76)
    result[2, 2, 0] = numpy.sum(-x54 * x79)
    result[2, 3, 0] = numpy.sum(-x66 * x75 * x80)
    result[2, 4, 0] = numpy.sum(-x26 * x66 * x78)
    result[2, 5, 0] = numpy.sum(-x67 * x84)
    result[2, 6, 0] = numpy.sum(-x46 * x74 * x75)
    result[2, 7, 0] = numpy.sum(-x4 * x80 * x85)
    result[2, 8, 0] = numpy.sum(-x4 * x76 * x84)
    result[2, 9, 0] = numpy.sum(-x73 * x88)
    result[3, 0, 0] = numpy.sum(-x89 * x92)
    result[3, 1, 0] = numpy.sum(-x93 * x96)
    result[3, 2, 0] = numpy.sum(-x34 * x91 * x97)
    result[3, 3, 0] = numpy.sum(-x100 * x99)
    result[3, 4, 0] = numpy.sum(-x3 * x42 * x95)
    result[3, 5, 0] = numpy.sum(-x3 * x69 * x91)
    result[3, 6, 0] = numpy.sum(-x23 * (x16 * (2.0 * x62 + x64 + x94) + x26 * x98))
    result[3, 7, 0] = numpy.sum(-x34 * x99)
    result[3, 8, 0] = numpy.sum(-x44 * x96)
    result[3, 9, 0] = numpy.sum(-x51 * x92)
    result[4, 0, 0] = numpy.sum(-0.06666666666666667 * x22 * x52 * x75 * x89)
    result[4, 1, 0] = numpy.sum(-x101 * x58 * x75)
    result[4, 2, 0] = numpy.sum(-x101 * x52 * x85)
    result[4, 3, 0] = numpy.sum(-x102 * x65 * x75)
    result[4, 4, 0] = numpy.sum(-x3 * x58 * x78)
    result[4, 5, 0] = numpy.sum(-x102 * x52 * x84)
    result[4, 6, 0] = numpy.sum(-x23 * x72 * x75)
    result[4, 7, 0] = numpy.sum(-x65 * x79)
    result[4, 8, 0] = numpy.sum(-x59 * x84)
    result[4, 9, 0] = numpy.sum(-x23 * x52 * x88)
    result[5, 0, 0] = numpy.sum(-x105 * x89)
    result[5, 1, 0] = numpy.sum(-x104 * x28 * x97)
    result[5, 2, 0] = numpy.sum(-x108 * x93)
    result[5, 3, 0] = numpy.sum(-x104 * x3 * x80)
    result[5, 4, 0] = numpy.sum(-x107 * x20 * x26 * x3 * x36)
    result[5, 5, 0] = numpy.sum(-x100 * x110)
    result[5, 6, 0] = numpy.sum(-x105 * x46)
    result[5, 7, 0] = numpy.sum(-x108 * x37)
    result[5, 8, 0] = numpy.sum(-x110 * x28)
    result[5, 9, 0] = numpy.sum(-x23 * (x109 * x33 + x16 * (x106 + 2.0 * x81 + x83)))
    return result


def quadrupole3d_31(ax, da, A, bx, db, B, R):
    """Cartesian 3D (fp) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 10, 3), dtype=float)

    x0 = (ax + bx) ** (-1.0)
    x1 = 3.0 * x0
    x2 = x0 * (ax * A[0] + bx * B[0])
    x3 = -x2
    x4 = x3 + A[0]
    x5 = x3 + B[0]
    x6 = x4 * x5
    x7 = 2.0 * x6
    x8 = x3 + R[0]
    x9 = x4 * x8
    x10 = 2.0 * x9
    x11 = x5 * x8
    x12 = 2.0 * x11
    x13 = x0 * (x1 + x10 + x12 + x7)
    x14 = -2.0 * x2
    x15 = x14 + R[0]
    x16 = x15 + B[0]
    x17 = x0 * x16
    x18 = x0 + x12
    x19 = x18 * x4
    x20 = x17 + x19
    x21 = 4.0 * x20
    x22 = x8**2
    x23 = x1 + 4.0 * x9
    x24 = x0 * (x15 + A[0])
    x25 = x0 + x10
    x26 = x25 * x8
    x27 = x24 + x26
    x28 = 2.0 * x4
    x29 = x0 * (2.0 * x22 + x23) + x27 * x28
    x30 = x18 * x8
    x31 = 2.0 * x19
    x32 = x13 + 2.0 * x20 * x8
    x33 = x0 * (x1 * x16 + x27 + x30 + x31) + x32 * x4
    x34 = 3.872983346207417
    x35 = ax * bx * x0
    x36 = (
        5.568327996831708
        * da
        * db
        * numpy.exp(-x35 * ((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2 + (A[2] - B[2]) ** 2))
    )
    x37 = x0**1.5 * x36
    x38 = x34 * x37
    x39 = 0.008333333333333333 * x38
    x40 = x0 * (ax * A[1] + bx * B[1])
    x41 = -x40
    x42 = x41 + B[1]
    x43 = x25 * x4
    x44 = 2.0 * x0
    x45 = 0.01666666666666667 * x38
    x46 = x45 * (x29 * x4 + x44 * (2.0 * x24 + x26 + x43))
    x47 = x0 * (ax * A[2] + bx * B[2])
    x48 = -x47
    x49 = x48 + B[2]
    x50 = x41 + A[1]
    x51 = x37 * x50
    x52 = 1.732050807568877
    x53 = 0.08333333333333333 * x52
    x54 = x33 * x53
    x55 = 0.5 * x0
    x56 = x42 * x50
    x57 = x55 + x56
    x58 = x0**1.5 * x36
    x59 = x57 * x58
    x60 = x29 * x53
    x61 = x48 + A[2]
    x62 = x37 * x61
    x63 = x49 * x61
    x64 = x55 + x63
    x65 = x58 * x64
    x66 = x50**2
    x67 = x55 + x66
    x68 = x53 * x58
    x69 = x32 * x68
    x70 = -2.0 * x40
    x71 = x70 + B[1]
    x72 = 2.0 * x56
    x73 = x0 * (x71 + A[1]) + x50 * (x0 + x72)
    x74 = x27 * x68
    x75 = x52 * x67
    x76 = 0.1666666666666667 * x58
    x77 = x27 * x76
    x78 = 0.25 * x51
    x79 = 0.5 * x27
    x80 = x61**2
    x81 = x55 + x80
    x82 = x52 * x81
    x83 = -2.0 * x47
    x84 = x83 + B[2]
    x85 = 2.0 * x63
    x86 = x0 * (x84 + A[2]) + x61 * (x0 + x85)
    x87 = x17 + x30
    x88 = x50 * x87
    x89 = 1.5 * x0
    x90 = x66 + x89
    x91 = x34 * x58
    x92 = 0.03333333333333333 * x91
    x93 = x90 * x92
    x94 = x1 + 2.0 * x66
    x95 = 2.0 * x50
    x96 = 0.01666666666666667 * x0 * (4.0 * x56 + x94) + 0.01666666666666667 * x73 * x95
    x97 = x22 + x55
    x98 = x91 * x97
    x99 = 0.06666666666666667 * x98
    x100 = x50 * x90
    x101 = x61 * x76
    x102 = x52 * x97
    x103 = 0.3333333333333333 * x97
    x104 = x76 * x82
    x105 = x50 * x76
    x106 = x61 * (x80 + x89)
    x107 = x106 * x92
    x108 = x1 + 2.0 * x80
    x109 = 2.0 * x61
    x110 = (
        0.01666666666666667 * x0 * (x108 + 4.0 * x63) + 0.01666666666666667 * x109 * x86
    )
    x111 = x41 + R[1]
    x112 = x24 + x43
    x113 = x0 * (x14 + A[0] + B[0]) + x4 * (x0 + x7)
    x114 = x13 + x20 * x28
    x115 = x45 * (x0 * (x112 + x113 + 2.0 * x17 + x31) + x114 * x4)
    x116 = x111 * x42
    x117 = x116 + x55
    x118 = x4**2
    x119 = 2.0 * x118
    x120 = x0 * (x119 + x23) + x112 * x28
    x121 = 0.01666666666666667 * x91
    x122 = x120 * x121
    x123 = x111 * x45
    x124 = x111 * x50
    x125 = x124 + x55
    x126 = x125 * x58
    x127 = x126 * x53
    x128 = x71 + R[1]
    x129 = x0 * x128
    x130 = 2.0 * x116
    x131 = x0 + x130
    x132 = x131 * x50
    x133 = x129 + x132
    x134 = x112 * x68
    x135 = x112 * x52
    x136 = 0.1666666666666667 * x135
    x137 = x53 * x62
    x138 = x0 * (x70 + A[1] + R[1])
    x139 = 2.0 * x124
    x140 = x0 + x139
    x141 = x140 * x50
    x142 = x138 + x141
    x143 = x20 * x68
    x144 = x0 * (x1 + x130 + x139 + x72)
    x145 = x133 * x95 + x144
    x146 = x58 * (x55 + x9)
    x147 = x146 * x53
    x148 = x142 * x52
    x149 = 0.1666666666666667 * x146
    x150 = 0.5 * x61
    x151 = x20 * x58
    x152 = 0.1666666666666667 * x82
    x153 = 0.3333333333333333 * x82
    x154 = x52 * x86
    x155 = 4.0 * x124
    x156 = x0 * (x155 + x94) + x142 * x95
    x157 = x11 + x55
    x158 = x121 * x157
    x159 = 2.0 * x132
    x160 = x0 * (2.0 * x129 + x142 + x159 + x73) + x145 * x50
    x161 = x45 * x8
    x162 = 0.1666666666666667 * x8
    x163 = x133 * x58
    x164 = 0.06666666666666667 * x91
    x165 = x106 * x164
    x166 = x38 * x8
    x167 = x48 + R[2]
    x168 = x167 * x45
    x169 = x167 * x49
    x170 = x169 + x55
    x171 = x51 * x53
    x172 = x167 * x61
    x173 = x172 + x55
    x174 = x173 * x68
    x175 = x173 * x76
    x176 = x84 + R[2]
    x177 = x0 * x176
    x178 = 2.0 * x169
    x179 = x0 + x178
    x180 = x179 * x61
    x181 = x177 + x180
    x182 = 0.1666666666666667 * x167
    x183 = x149 * x52
    x184 = 0.3333333333333333 * x75
    x185 = 0.5 * x50
    x186 = x0 * (x83 + A[2] + R[2])
    x187 = 2.0 * x172
    x188 = x0 + x187
    x189 = x188 * x61
    x190 = x186 + x189
    x191 = x0 * (x1 + x178 + x187 + x85)
    x192 = x109 * x181 + x191
    x193 = x100 * x164
    x194 = x173 * x58
    x195 = x175 * x52
    x196 = x181 * x76
    x197 = x190 * x52
    x198 = 4.0 * x172
    x199 = x0 * (x108 + x198) + x109 * x190
    x200 = 2.0 * x180
    x201 = x0 * (2.0 * x177 + x190 + x200 + x86) + x192 * x61
    x202 = x111**2
    x203 = x202 + x55
    x204 = x0 * (x1 + x119 + 4.0 * x6) + x113 * x28
    x205 = x121 * x204
    x206 = x111 * x131
    x207 = x129 + x206
    x208 = x207 * x4
    x209 = x118 + x89
    x210 = x209 * x92
    x211 = x203 * x91
    x212 = 0.06666666666666667 * x211
    x213 = x209 * x4
    x214 = x111 * x140
    x215 = x138 + x214
    x216 = x215 * x68
    x217 = x118 + x55
    x218 = x111 * x133
    x219 = x144 + 2.0 * x218
    x220 = x219 * x68
    x221 = x217 * x52
    x222 = x221 * x76
    x223 = x113 * x52
    x224 = 0.3333333333333333 * x221
    x225 = x0 * (x1 + x155 + 2.0 * x202) + x215 * x95
    x226 = x55 + x6
    x227 = x226 * x68
    x228 = x0 * (x1 * x128 + x159 + x206 + x215) + x219 * x50
    x229 = x228 * x53
    x230 = x37 * x4
    x231 = x225 * x53
    x232 = x226 * x58
    x233 = 0.5 * x4
    x234 = x4 * x76
    x235 = x45 * (x225 * x50 + x44 * (2.0 * x138 + x141 + x214))
    x236 = x164 * x213
    x237 = x230 * x53
    x238 = 0.1666666666666667 * x197
    x239 = x167**2
    x240 = x239 + x55
    x241 = x240 * x91
    x242 = 0.06666666666666667 * x241
    x243 = x167 * x179
    x244 = x177 + x243
    x245 = x167 * x188
    x246 = x186 + x245
    x247 = x246 * x68
    x248 = x167 * x181
    x249 = x191 + 2.0 * x248
    x250 = x249 * x68
    x251 = x0 * (x1 + x198 + 2.0 * x239) + x109 * x246
    x252 = x251 * x53
    x253 = x0 * (x1 * x176 + x200 + x243 + x246) + x249 * x61
    x254 = x253 * x53
    x255 = x45 * (x251 * x61 + x44 * (2.0 * x186 + x189 + x245))

    # 180 item(s)
    result[0, 0, 0] = numpy.sum(
        x39 * (x0 * (4.0 * x13 + x21 * x4 + x21 * x8 + x29) + x28 * x33)
    )
    result[0, 0, 1] = numpy.sum(x42 * x46)
    result[0, 0, 2] = numpy.sum(x46 * x49)
    result[0, 1, 0] = numpy.sum(x51 * x54)
    result[0, 1, 1] = numpy.sum(x59 * x60)
    result[0, 1, 2] = numpy.sum(x49 * x51 * x60)
    result[0, 2, 0] = numpy.sum(x54 * x62)
    result[0, 2, 1] = numpy.sum(x42 * x60 * x62)
    result[0, 2, 2] = numpy.sum(x60 * x65)
    result[0, 3, 0] = numpy.sum(x67 * x69)
    result[0, 3, 1] = numpy.sum(x73 * x74)
    result[0, 3, 2] = numpy.sum(x49 * x75 * x77)
    result[0, 4, 0] = numpy.sum(x32 * x61 * x78)
    result[0, 4, 1] = numpy.sum(x59 * x61 * x79)
    result[0, 4, 2] = numpy.sum(x50 * x65 * x79)
    result[0, 5, 0] = numpy.sum(x69 * x81)
    result[0, 5, 1] = numpy.sum(x42 * x77 * x82)
    result[0, 5, 2] = numpy.sum(x74 * x86)
    result[0, 6, 0] = numpy.sum(x88 * x93)
    result[0, 6, 1] = numpy.sum(x96 * x98)
    result[0, 6, 2] = numpy.sum(x100 * x49 * x99)
    result[0, 7, 0] = numpy.sum(x101 * x75 * x87)
    result[0, 7, 1] = numpy.sum(x101 * x102 * x73)
    result[0, 7, 2] = numpy.sum(x103 * x65 * x75)
    result[0, 8, 0] = numpy.sum(x104 * x88)
    result[0, 8, 1] = numpy.sum(x103 * x59 * x82)
    result[0, 8, 2] = numpy.sum(x102 * x105 * x86)
    result[0, 9, 0] = numpy.sum(x107 * x87)
    result[0, 9, 1] = numpy.sum(x106 * x42 * x99)
    result[0, 9, 2] = numpy.sum(x110 * x98)
    result[1, 0, 0] = numpy.sum(x111 * x115)
    result[1, 0, 1] = numpy.sum(x117 * x122)
    result[1, 0, 2] = numpy.sum(x120 * x123 * x49)
    result[1, 1, 0] = numpy.sum(x114 * x127)
    result[1, 1, 1] = numpy.sum(x133 * x134)
    result[1, 1, 2] = numpy.sum(x126 * x136 * x49)
    result[1, 2, 0] = numpy.sum(x111 * x114 * x137)
    result[1, 2, 1] = numpy.sum(x101 * x117 * x135)
    result[1, 2, 2] = numpy.sum(x111 * x136 * x65)
    result[1, 3, 0] = numpy.sum(x142 * x143)
    result[1, 3, 1] = numpy.sum(x145 * x147)
    result[1, 3, 2] = numpy.sum(x148 * x149 * x49)
    result[1, 4, 0] = numpy.sum(x126 * x150 * x20)
    result[1, 4, 1] = numpy.sum(x133 * x146 * x150)
    result[1, 4, 2] = numpy.sum(x125 * x146 * x64)
    result[1, 5, 0] = numpy.sum(x111 * x151 * x152)
    result[1, 5, 1] = numpy.sum(x117 * x146 * x153)
    result[1, 5, 2] = numpy.sum(x111 * x149 * x154)
    result[1, 6, 0] = numpy.sum(x156 * x158)
    result[1, 6, 1] = numpy.sum(x160 * x161)
    result[1, 6, 2] = numpy.sum(x156 * x161 * x49)
    result[1, 7, 0] = numpy.sum(x101 * x148 * x157)
    result[1, 7, 1] = numpy.sum(x137 * x145 * x8)
    result[1, 7, 2] = numpy.sum(x148 * x162 * x65)
    result[1, 8, 0] = numpy.sum(x126 * x153 * x157)
    result[1, 8, 1] = numpy.sum(x152 * x163 * x8)
    result[1, 8, 2] = numpy.sum(x126 * x154 * x162)
    result[1, 9, 0] = numpy.sum(x111 * x157 * x165)
    result[1, 9, 1] = numpy.sum(x117 * x165 * x8)
    result[1, 9, 2] = numpy.sum(x110 * x111 * x166)
    result[2, 0, 0] = numpy.sum(x115 * x167)
    result[2, 0, 1] = numpy.sum(x120 * x168 * x42)
    result[2, 0, 2] = numpy.sum(x122 * x170)
    result[2, 1, 0] = numpy.sum(x114 * x167 * x171)
    result[2, 1, 1] = numpy.sum(x136 * x167 * x59)
    result[2, 1, 2] = numpy.sum(x105 * x135 * x170)
    result[2, 2, 0] = numpy.sum(x114 * x174)
    result[2, 2, 1] = numpy.sum(x135 * x175 * x42)
    result[2, 2, 2] = numpy.sum(x134 * x181)
    result[2, 3, 0] = numpy.sum(x151 * x182 * x75)
    result[2, 3, 1] = numpy.sum(x167 * x183 * x73)
    result[2, 3, 2] = numpy.sum(x146 * x170 * x184)
    result[2, 4, 0] = numpy.sum(x151 * x173 * x185)
    result[2, 4, 1] = numpy.sum(x146 * x173 * x57)
    result[2, 4, 2] = numpy.sum(x146 * x181 * x185)
    result[2, 5, 0] = numpy.sum(x143 * x190)
    result[2, 5, 1] = numpy.sum(x183 * x190 * x42)
    result[2, 5, 2] = numpy.sum(x147 * x192)
    result[2, 6, 0] = numpy.sum(x157 * x167 * x193)
    result[2, 6, 1] = numpy.sum(x166 * x167 * x96)
    result[2, 6, 2] = numpy.sum(x170 * x193 * x8)
    result[2, 7, 0] = numpy.sum(x157 * x184 * x194)
    result[2, 7, 1] = numpy.sum(x195 * x73 * x8)
    result[2, 7, 2] = numpy.sum(x196 * x75 * x8)
    result[2, 8, 0] = numpy.sum(x105 * x157 * x197)
    result[2, 8, 1] = numpy.sum(x162 * x197 * x59)
    result[2, 8, 2] = numpy.sum(x171 * x192 * x8)
    result[2, 9, 0] = numpy.sum(x158 * x199)
    result[2, 9, 1] = numpy.sum(x161 * x199 * x42)
    result[2, 9, 2] = numpy.sum(x161 * x201)
    result[3, 0, 0] = numpy.sum(x203 * x205)
    result[3, 0, 1] = numpy.sum(x208 * x210)
    result[3, 0, 2] = numpy.sum(x212 * x213 * x49)
    result[3, 1, 0] = numpy.sum(x113 * x216)
    result[3, 1, 1] = numpy.sum(x217 * x220)
    result[3, 1, 2] = numpy.sum(x215 * x222 * x49)
    result[3, 2, 0] = numpy.sum(x101 * x203 * x223)
    result[3, 2, 1] = numpy.sum(x101 * x207 * x221)
    result[3, 2, 2] = numpy.sum(x203 * x224 * x65)
    result[3, 3, 0] = numpy.sum(x225 * x227)
    result[3, 3, 1] = numpy.sum(x229 * x230)
    result[3, 3, 2] = numpy.sum(x230 * x231 * x49)
    result[3, 4, 0] = numpy.sum(x150 * x215 * x232)
    result[3, 4, 1] = numpy.sum(0.25 * x219 * x4 * x62)
    result[3, 4, 2] = numpy.sum(x215 * x233 * x65)
    result[3, 5, 0] = numpy.sum(x153 * x203 * x232)
    result[3, 5, 1] = numpy.sum(x104 * x208)
    result[3, 5, 2] = numpy.sum(x154 * x203 * x234)
    result[3, 6, 0] = numpy.sum(x235 * x5)
    result[3, 6, 1] = numpy.sum(
        x39 * (x0 * (4.0 * x133 * x50 + 4.0 * x144 + 4.0 * x218 + x225) + x228 * x95)
    )
    result[3, 6, 2] = numpy.sum(x235 * x49)
    result[3, 7, 0] = numpy.sum(x231 * x5 * x62)
    result[3, 7, 1] = numpy.sum(x229 * x62)
    result[3, 7, 2] = numpy.sum(x231 * x65)
    result[3, 8, 0] = numpy.sum(x104 * x215 * x5)
    result[3, 8, 1] = numpy.sum(x220 * x81)
    result[3, 8, 2] = numpy.sum(x216 * x86)
    result[3, 9, 0] = numpy.sum(x106 * x212 * x5)
    result[3, 9, 1] = numpy.sum(x107 * x207)
    result[3, 9, 2] = numpy.sum(x110 * x211)
    result[4, 0, 0] = numpy.sum(x111 * x168 * x204)
    result[4, 0, 1] = numpy.sum(x117 * x167 * x236)
    result[4, 0, 2] = numpy.sum(x111 * x170 * x236)
    result[4, 1, 0] = numpy.sum(x126 * x182 * x223)
    result[4, 1, 1] = numpy.sum(x163 * x182 * x221)
    result[4, 1, 2] = numpy.sum(x126 * x170 * x224)
    result[4, 2, 0] = numpy.sum(x111 * x113 * x195)
    result[4, 2, 1] = numpy.sum(x117 * x194 * x224)
    result[4, 2, 2] = numpy.sum(x111 * x196 * x221)
    result[4, 3, 0] = numpy.sum(x148 * x182 * x232)
    result[4, 3, 1] = numpy.sum(x145 * x167 * x237)
    result[4, 3, 2] = numpy.sum(x148 * x170 * x234)
    result[4, 4, 0] = numpy.sum(x126 * x173 * x226)
    result[4, 4, 1] = numpy.sum(x163 * x173 * x233)
    result[4, 4, 2] = numpy.sum(x126 * x181 * x233)
    result[4, 5, 0] = numpy.sum(x111 * x232 * x238)
    result[4, 5, 1] = numpy.sum(x117 * x197 * x234)
    result[4, 5, 2] = numpy.sum(x111 * x192 * x237)
    result[4, 6, 0] = numpy.sum(x156 * x168 * x5)
    result[4, 6, 1] = numpy.sum(x160 * x168)
    result[4, 6, 2] = numpy.sum(x121 * x156 * x170)
    result[4, 7, 0] = numpy.sum(x148 * x175 * x5)
    result[4, 7, 1] = numpy.sum(x145 * x174)
    result[4, 7, 2] = numpy.sum(x142 * x181 * x68)
    result[4, 8, 0] = numpy.sum(x126 * x238 * x5)
    result[4, 8, 1] = numpy.sum(x133 * x190 * x68)
    result[4, 8, 2] = numpy.sum(x127 * x192)
    result[4, 9, 0] = numpy.sum(x123 * x199 * x5)
    result[4, 9, 1] = numpy.sum(x117 * x121 * x199)
    result[4, 9, 2] = numpy.sum(x123 * x201)
    result[5, 0, 0] = numpy.sum(x205 * x240)
    result[5, 0, 1] = numpy.sum(x213 * x242 * x42)
    result[5, 0, 2] = numpy.sum(x210 * x244 * x4)
    result[5, 1, 0] = numpy.sum(x105 * x223 * x240)
    result[5, 1, 1] = numpy.sum(x224 * x240 * x59)
    result[5, 1, 2] = numpy.sum(x105 * x221 * x244)
    result[5, 2, 0] = numpy.sum(x113 * x247)
    result[5, 2, 1] = numpy.sum(x222 * x246 * x42)
    result[5, 2, 2] = numpy.sum(x217 * x250)
    result[5, 3, 0] = numpy.sum(x184 * x232 * x240)
    result[5, 3, 1] = numpy.sum(x234 * x240 * x52 * x73)
    result[5, 3, 2] = numpy.sum(x234 * x244 * x75)
    result[5, 4, 0] = numpy.sum(x185 * x232 * x246)
    result[5, 4, 1] = numpy.sum(x233 * x246 * x59)
    result[5, 4, 2] = numpy.sum(x249 * x4 * x78)
    result[5, 5, 0] = numpy.sum(x227 * x251)
    result[5, 5, 1] = numpy.sum(x230 * x252 * x42)
    result[5, 5, 2] = numpy.sum(x230 * x254)
    result[5, 6, 0] = numpy.sum(x100 * x242 * x5)
    result[5, 6, 1] = numpy.sum(x241 * x96)
    result[5, 6, 2] = numpy.sum(x244 * x50 * x93)
    result[5, 7, 0] = numpy.sum(x246 * x5 * x75 * x76)
    result[5, 7, 1] = numpy.sum(x247 * x73)
    result[5, 7, 2] = numpy.sum(x250 * x67)
    result[5, 8, 0] = numpy.sum(x252 * x5 * x51)
    result[5, 8, 1] = numpy.sum(x252 * x59)
    result[5, 8, 2] = numpy.sum(x254 * x51)
    result[5, 9, 0] = numpy.sum(x255 * x5)
    result[5, 9, 1] = numpy.sum(x255 * x42)
    result[5, 9, 2] = numpy.sum(
        x39 * (x0 * (4.0 * x181 * x61 + 4.0 * x191 + 4.0 * x248 + x251) + x109 * x253)
    )
    return result


def quadrupole3d_32(ax, da, A, bx, db, B, R):
    """Cartesian 3D (fd) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 10, 6), dtype=float)

    x0 = (ax + bx) ** (-1.0)
    x1 = x0 * (ax * A[0] + bx * B[0])
    x2 = -x1
    x3 = x2 + A[0]
    x4 = x2 + B[0]
    x5 = -2.0 * x1
    x6 = x5 + R[0]
    x7 = x6 + B[0]
    x8 = x0 * x7
    x9 = x2 + R[0]
    x10 = x4 * x9
    x11 = 2.0 * x10
    x12 = x0 + x11
    x13 = x12 * x3
    x14 = x13 + x8
    x15 = 4.0 * x14
    x16 = x15 * x4
    x17 = x9**2
    x18 = 3.0 * x0
    x19 = 2.0 * x17 + x18
    x20 = x12 * x9
    x21 = x20 + x8
    x22 = 2.0 * x4
    x23 = x0 * (4.0 * x10 + x19) + x21 * x22
    x24 = x3 * x4
    x25 = 2.0 * x24
    x26 = x3 * x9
    x27 = 2.0 * x26
    x28 = x0 * (x11 + x18 + x25 + x27)
    x29 = 4.0 * x28
    x30 = x15 * x9 + x29
    x31 = x0 * (x6 + A[0])
    x32 = x0 + x27
    x33 = x32 * x9
    x34 = x31 + x33
    x35 = 2.0 * x13
    x36 = x18 * x7 + x35
    x37 = x0 * (x20 + x34 + x36)
    x38 = 2.0 * x14
    x39 = x28 + x38 * x9
    x40 = x39 * x4
    x41 = x37 + x40
    x42 = 2.0 * x3
    x43 = x0 * (x16 + x23 + x30) + x41 * x42
    x44 = x3 * x39
    x45 = x12 * x4
    x46 = x0 * (x5 + A[0] + B[0])
    x47 = x0 + x25
    x48 = x4 * x47
    x49 = x46 + x48
    x50 = x14 * x22 + x28
    x51 = x0 * (x36 + x45 + x49) + x3 * x50
    x52 = 2.0 * x0
    x53 = 2.23606797749979
    x54 = 0.008333333333333333 * x53
    x55 = ax * bx * x0
    x56 = (
        5.568327996831708
        * da
        * db
        * numpy.exp(-x55 * ((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2 + (A[2] - B[2]) ** 2))
    )
    x57 = x0**1.5 * x56
    x58 = x54 * x57
    x59 = x0 * (ax * A[1] + bx * B[1])
    x60 = -x59
    x61 = x60 + B[1]
    x62 = x57 * x61
    x63 = x15 * x3
    x64 = 4.0 * x26
    x65 = x0 * (x19 + x64) + x34 * x42
    x66 = x37 + x44
    x67 = 3.872983346207417
    x68 = 0.008333333333333333 * x67
    x69 = x68 * (x0 * (x30 + x63 + x65) + x42 * x66)
    x70 = x0 * (ax * A[2] + bx * B[2])
    x71 = -x70
    x72 = x71 + B[2]
    x73 = x57 * x72
    x74 = x3 * x32
    x75 = 0.01666666666666667 * x3 * x65 + 0.01666666666666667 * x52 * (
        2.0 * x31 + x33 + x74
    )
    x76 = x61**2
    x77 = 0.5 * x0
    x78 = x76 + x77
    x79 = x0**1.5 * x56
    x80 = x78 * x79
    x81 = x53 * x80
    x82 = x62 * x67
    x83 = x72**2
    x84 = x77 + x83
    x85 = x79 * x84
    x86 = x53 * x85
    x87 = x60 + A[1]
    x88 = 0.04166666666666667 * x57
    x89 = x43 * x88
    x90 = x61 * x87
    x91 = x77 + x90
    x92 = 1.732050807568877
    x93 = 0.08333333333333333 * x79
    x94 = x92 * x93
    x95 = x66 * x94
    x96 = 0.08333333333333333 * x66
    x97 = x87 * x92
    x98 = -2.0 * x59
    x99 = x98 + B[1]
    x100 = x0 * (x99 + A[1])
    x101 = 2.0 * x90
    x102 = x0 + x101
    x103 = x102 * x61
    x104 = x100 + x103
    x105 = 0.04166666666666667 * x79
    x106 = x105 * x65
    x107 = x65 * x94
    x108 = x65 * x93
    x109 = x71 + A[2]
    x110 = x109 * x92
    x111 = x109 * x72
    x112 = x111 + x77
    x113 = -2.0 * x70
    x114 = x113 + B[2]
    x115 = x0 * (x114 + A[2])
    x116 = 2.0 * x111
    x117 = x0 + x116
    x118 = x117 * x72
    x119 = x115 + x118
    x120 = x87**2
    x121 = x120 + x77
    x122 = x41 * x93
    x123 = x102 * x87
    x124 = x100 + x123
    x125 = x105 * x92
    x126 = x125 * x39
    x127 = x72 * x94
    x128 = x18 + 4.0 * x90
    x129 = 2.0 * x87
    x130 = x0 * (x128 + 2.0 * x76) + x104 * x129
    x131 = x105 * x34
    x132 = 0.1666666666666667 * x34
    x133 = 0.08333333333333333 * x57
    x134 = x110 * x133
    x135 = x79 * x91
    x136 = 0.25 * x39
    x137 = x112 * x79
    x138 = x34 * x94
    x139 = 0.5 * x135
    x140 = x109**2
    x141 = x140 + x77
    x142 = x61 * x94
    x143 = x109 * x117
    x144 = x115 + x143
    x145 = 4.0 * x111 + x18
    x146 = 2.0 * x109
    x147 = x0 * (x145 + 2.0 * x83) + x119 * x146
    x148 = 0.01666666666666667 * x53
    x149 = x148 * x23
    x150 = 1.5 * x0
    x151 = x120 + x150
    x152 = x79 * x87
    x153 = x151 * x152
    x154 = 2.0 * x120
    x155 = x0 * (x128 + x154) + x124 * x129
    x156 = x67 * x79
    x157 = 0.008333333333333333 * x156
    x158 = x157 * x21
    x159 = x151 * x87
    x160 = 0.03333333333333333 * x156
    x161 = x160 * x21
    x162 = x17 + x77
    x163 = x162 * x79
    x164 = x148 * (x130 * x87 + x52 * (2.0 * x100 + x103 + x123))
    x165 = 0.01666666666666667 * x156
    x166 = x162 * x165
    x167 = 0.06666666666666667 * x162
    x168 = x109 * x93
    x169 = x21 * x94
    x170 = 0.1666666666666667 * x121
    x171 = x137 * x92
    x172 = 0.1666666666666667 * x162
    x173 = 0.1666666666666667 * x163
    x174 = x87 * x93
    x175 = 0.1666666666666667 * x141
    x176 = x135 * x92
    x177 = x140 + x150
    x178 = x109 * x79
    x179 = x177 * x178
    x180 = x109 * x177
    x181 = 2.0 * x140
    x182 = x0 * (x145 + x181) + x144 * x146
    x183 = x148 * (x109 * x147 + x52 * (2.0 * x115 + x118 + x143))
    x184 = x60 + R[1]
    x185 = x4**2
    x186 = x18 + 4.0 * x24
    x187 = x0 * (2.0 * x185 + x186) + x42 * x49
    x188 = x58 * (x0 * (x16 + x187 + x29 + x63) + x42 * x51)
    x189 = x184 * x61
    x190 = x189 + x77
    x191 = x31 + x74
    x192 = x3 * x47
    x193 = x192 + x46
    x194 = x28 + x3 * x38
    x195 = x0 * (x191 + x193 + x35 + 2.0 * x8) + x194 * x3
    x196 = x165 * x195
    x197 = 0.01666666666666667 * x195
    x198 = x67 * x73
    x199 = x99 + R[1]
    x200 = x0 * x199
    x201 = 2.0 * x189
    x202 = x0 + x201
    x203 = x202 * x61
    x204 = x200 + x203
    x205 = x204 * x79
    x206 = x3**2
    x207 = 2.0 * x206
    x208 = x0 * (x18 + x207 + x64) + x191 * x42
    x209 = x208 * x54
    x210 = x165 * x190
    x211 = 0.01666666666666667 * x86
    x212 = x184 * x87
    x213 = x212 + x77
    x214 = x51 * x93
    x215 = x202 * x87
    x216 = x200 + x215
    x217 = x125 * x194
    x218 = x213 * x93
    x219 = x72 * x92
    x220 = 2.0 * x212
    x221 = x0 * (x101 + x18 + x201 + x220)
    x222 = x216 * x61
    x223 = x221 + 2.0 * x222
    x224 = x105 * x191
    x225 = 0.1666666666666667 * x85
    x226 = x109 * x184
    x227 = x133 * x51
    x228 = x194 * x94
    x229 = x109 * x190
    x230 = 0.1666666666666667 * x190
    x231 = x184 * x93
    x232 = x0 * (x98 + A[1] + R[1])
    x233 = x0 + x220
    x234 = x233 * x87
    x235 = x232 + x234
    x236 = x105 * x50
    x237 = x129 * x216 + x221
    x238 = x125 * x14
    x239 = 2.0 * x215
    x240 = x18 * x199 + x239
    x241 = x0 * (x104 + x203 + x240) + x223 * x87
    x242 = x26 + x77
    x243 = x242 * x93
    x244 = x216 * x79
    x245 = 0.25 * x14
    x246 = 0.5 * x137
    x247 = x79 * x92
    x248 = x242 * x247
    x249 = 0.1666666666666667 * x213
    x250 = x14 * x247
    x251 = x144 * x94
    x252 = x18 + 4.0 * x212
    x253 = x0 * (x154 + x252) + x129 * x235
    x254 = x45 + x8
    x255 = x254 * x79
    x256 = x255 * x54
    x257 = x0 * (x124 + 2.0 * x200 + x235 + x239) + x237 * x87
    x258 = x10 + x77
    x259 = x165 * x258
    x260 = 4.0 * x216
    x261 = x260 * x87
    x262 = 4.0 * x221
    x263 = 4.0 * x222 + x262
    x264 = x0 * (x130 + x261 + x263) + x129 * x241
    x265 = x58 * x9
    x266 = 0.01666666666666667 * x9
    x267 = x237 * x94
    x268 = 0.1666666666666667 * x258
    x269 = x133 * x9
    x270 = x9 * x93
    x271 = x258 * x92
    x272 = x247 * x249
    x273 = 0.03333333333333333 * x53
    x274 = x255 * x273
    x275 = 0.06666666666666667 * x156
    x276 = x258 * x275
    x277 = x273 * x9
    x278 = x57 * x9
    x279 = x71 + R[2]
    x280 = x279 * x72
    x281 = x280 + x77
    x282 = 0.01666666666666667 * x81
    x283 = x165 * x281
    x284 = x114 + R[2]
    x285 = x0 * x284
    x286 = 2.0 * x280
    x287 = x0 + x286
    x288 = x287 * x72
    x289 = x285 + x288
    x290 = x289 * x79
    x291 = x279 * x87
    x292 = x281 * x87
    x293 = x279 * x93
    x294 = 0.1666666666666667 * x281
    x295 = x109 * x279
    x296 = x295 + x77
    x297 = x296 * x93
    x298 = x297 * x92
    x299 = x109 * x287
    x300 = x285 + x299
    x301 = 0.1666666666666667 * x80
    x302 = 2.0 * x295
    x303 = x0 * (x116 + x18 + x286 + x302)
    x304 = x300 * x72
    x305 = x303 + 2.0 * x304
    x306 = x124 * x94
    x307 = x300 * x79
    x308 = x243 * x92
    x309 = x0 * (x113 + A[2] + R[2])
    x310 = x0 + x302
    x311 = x109 * x310
    x312 = x309 + x311
    x313 = x146 * x300 + x303
    x314 = 2.0 * x299
    x315 = x18 * x284 + x314
    x316 = x0 * (x119 + x288 + x315) + x109 * x305
    x317 = x247 * x296
    x318 = x313 * x94
    x319 = x18 + 4.0 * x295
    x320 = x0 * (x181 + x319) + x146 * x312
    x321 = x0 * (x144 + 2.0 * x285 + x312 + x314) + x109 * x313
    x322 = 4.0 * x300
    x323 = x109 * x322
    x324 = 4.0 * x303
    x325 = 4.0 * x304 + x324
    x326 = x0 * (x147 + x323 + x325) + x146 * x316
    x327 = x184**2
    x328 = x327 + x77
    x329 = x328 * x79
    x330 = x148 * (x187 * x3 + x52 * (x192 + 2.0 * x46 + x48))
    x331 = x0 * (x186 + x207) + x193 * x42
    x332 = x184 * x202
    x333 = x200 + x332
    x334 = x157 * x333
    x335 = x165 * x328
    x336 = 2.0 * x327
    x337 = x0 * (x18 + 4.0 * x189 + x336) + 2.0 * x333 * x61
    x338 = x148 * x337
    x339 = x3 * (x150 + x206)
    x340 = x339 * x79
    x341 = x160 * x333
    x342 = 0.06666666666666667 * x339
    x343 = x184 * x233
    x344 = x232 + x343
    x345 = x105 * x344
    x346 = 2.0 * x184 * x216 + x221
    x347 = x125 * x346
    x348 = x206 + x77
    x349 = x0 * (x240 + x332 + x344)
    x350 = x346 * x61
    x351 = x349 + x350
    x352 = x351 * x93
    x353 = x109 * x94
    x354 = 0.1666666666666667 * x193
    x355 = 0.1666666666666667 * x348
    x356 = x0 * (x252 + x336) + x129 * x344
    x357 = x105 * x356
    x358 = x24 + x77
    x359 = x346 * x87
    x360 = x349 + x359
    x361 = x360 * x94
    x362 = x184 * x260
    x363 = x0 * (x263 + x337 + x362) + x129 * x351
    x364 = x363 * x88
    x365 = 0.08333333333333333 * x3 * x92
    x366 = x3 * x93
    x367 = 0.25 * x346
    x368 = x133 * x3
    x369 = x3 * x94
    x370 = x247 * x358
    x371 = 0.1666666666666667 * x358 * x92
    x372 = x356 * x87 + x52 * (2.0 * x232 + x234 + x343)
    x373 = x185 + x77
    x374 = x373 * x79
    x375 = x148 * x374
    x376 = x68 * (x0 * (x261 + x262 + x356 + x362) + x129 * x360)
    x377 = x4 * x57
    x378 = 0.01666666666666667 * x4
    x379 = x4 * x94
    x380 = 0.06666666666666667 * x53
    x381 = x273 * x339
    x382 = x193 * x94
    x383 = x355 * x92
    x384 = 0.5 * x358
    x385 = 0.25 * x3
    x386 = x218 * x92
    x387 = 0.01666666666666667 * x377 * x67
    x388 = 0.1666666666666667 * x374
    x389 = x279**2
    x390 = x389 + x77
    x391 = x390 * x79
    x392 = x165 * x390
    x393 = x279 * x287
    x394 = x285 + x393
    x395 = x157 * x394
    x396 = 2.0 * x389
    x397 = x0 * (x18 + 4.0 * x280 + x396) + 2.0 * x394 * x72
    x398 = x148 * x397
    x399 = x394 * x87
    x400 = x279 * x310
    x401 = x309 + x400
    x402 = x105 * x401
    x403 = 2.0 * x279 * x300 + x303
    x404 = x125 * x403
    x405 = x0 * (x315 + x393 + x401)
    x406 = x403 * x72
    x407 = x405 + x406
    x408 = x407 * x93
    x409 = x0 * (x319 + x396) + x146 * x401
    x410 = x105 * x409
    x411 = x109 * x403
    x412 = x405 + x411
    x413 = x412 * x94
    x414 = x279 * x322
    x415 = x0 * (x325 + x397 + x414) + x146 * x407
    x416 = x415 * x88
    x417 = x109 * x409 + x52 * (2.0 * x309 + x311 + x400)
    x418 = x68 * (x0 * (x323 + x324 + x409 + x414) + x146 * x412)

    # 360 item(s)
    result[0, 0, 0] = numpy.sum(-x58 * (x3 * x43 + x52 * (2.0 * x37 + x40 + x44 + x51)))
    result[0, 0, 1] = numpy.sum(-x62 * x69)
    result[0, 0, 2] = numpy.sum(-x69 * x73)
    result[0, 0, 3] = numpy.sum(-x75 * x81)
    result[0, 0, 4] = numpy.sum(-x72 * x75 * x82)
    result[0, 0, 5] = numpy.sum(-x75 * x86)
    result[0, 1, 0] = numpy.sum(-x87 * x89)
    result[0, 1, 1] = numpy.sum(-x91 * x95)
    result[0, 1, 2] = numpy.sum(-x73 * x96 * x97)
    result[0, 1, 3] = numpy.sum(-x104 * x106)
    result[0, 1, 4] = numpy.sum(-x107 * x72 * x91)
    result[0, 1, 5] = numpy.sum(-x108 * x84 * x87)
    result[0, 2, 0] = numpy.sum(-x109 * x89)
    result[0, 2, 1] = numpy.sum(-x110 * x62 * x96)
    result[0, 2, 2] = numpy.sum(-x112 * x95)
    result[0, 2, 3] = numpy.sum(-x108 * x109 * x78)
    result[0, 2, 4] = numpy.sum(-x107 * x112 * x61)
    result[0, 2, 5] = numpy.sum(-x106 * x119)
    result[0, 3, 0] = numpy.sum(-x121 * x122)
    result[0, 3, 1] = numpy.sum(-x124 * x126)
    result[0, 3, 2] = numpy.sum(-x121 * x127 * x39)
    result[0, 3, 3] = numpy.sum(-x130 * x131)
    result[0, 3, 4] = numpy.sum(-x124 * x127 * x34)
    result[0, 3, 5] = numpy.sum(-x121 * x132 * x85)
    result[0, 4, 0] = numpy.sum(-x134 * x41 * x87)
    result[0, 4, 1] = numpy.sum(-x109 * x135 * x136)
    result[0, 4, 2] = numpy.sum(-x136 * x137 * x87)
    result[0, 4, 3] = numpy.sum(-x104 * x109 * x138)
    result[0, 4, 4] = numpy.sum(-x112 * x139 * x34)
    result[0, 4, 5] = numpy.sum(-x119 * x138 * x87)
    result[0, 5, 0] = numpy.sum(-x122 * x141)
    result[0, 5, 1] = numpy.sum(-x141 * x142 * x39)
    result[0, 5, 2] = numpy.sum(-x126 * x144)
    result[0, 5, 3] = numpy.sum(-x132 * x141 * x80)
    result[0, 5, 4] = numpy.sum(-x138 * x144 * x61)
    result[0, 5, 5] = numpy.sum(-x131 * x147)
    result[0, 6, 0] = numpy.sum(-x149 * x153)
    result[0, 6, 1] = numpy.sum(-x155 * x158)
    result[0, 6, 2] = numpy.sum(-x159 * x161 * x72)
    result[0, 6, 3] = numpy.sum(-x163 * x164)
    result[0, 6, 4] = numpy.sum(-x155 * x166 * x72)
    result[0, 6, 5] = numpy.sum(-x159 * x167 * x86)
    result[0, 7, 0] = numpy.sum(-x121 * x168 * x23)
    result[0, 7, 1] = numpy.sum(-x109 * x124 * x169)
    result[0, 7, 2] = numpy.sum(-x170 * x171 * x21)
    result[0, 7, 3] = numpy.sum(-x130 * x162 * x168)
    result[0, 7, 4] = numpy.sum(-x124 * x171 * x172)
    result[0, 7, 5] = numpy.sum(-x119 * x121 * x173)
    result[0, 8, 0] = numpy.sum(-x141 * x174 * x23)
    result[0, 8, 1] = numpy.sum(-x175 * x176 * x21)
    result[0, 8, 2] = numpy.sum(-x144 * x169 * x87)
    result[0, 8, 3] = numpy.sum(-x104 * x141 * x173)
    result[0, 8, 4] = numpy.sum(-x144 * x172 * x176)
    result[0, 8, 5] = numpy.sum(-x147 * x162 * x174)
    result[0, 9, 0] = numpy.sum(-x149 * x179)
    result[0, 9, 1] = numpy.sum(-x161 * x180 * x61)
    result[0, 9, 2] = numpy.sum(-x158 * x182)
    result[0, 9, 3] = numpy.sum(-x167 * x180 * x81)
    result[0, 9, 4] = numpy.sum(-x166 * x182 * x61)
    result[0, 9, 5] = numpy.sum(-x163 * x183)
    result[1, 0, 0] = numpy.sum(-x184 * x188)
    result[1, 0, 1] = numpy.sum(-x190 * x196)
    result[1, 0, 2] = numpy.sum(-x184 * x197 * x198)
    result[1, 0, 3] = numpy.sum(-x205 * x209)
    result[1, 0, 4] = numpy.sum(-x208 * x210 * x72)
    result[1, 0, 5] = numpy.sum(-x184 * x208 * x211)
    result[1, 1, 0] = numpy.sum(-x213 * x214)
    result[1, 1, 1] = numpy.sum(-x216 * x217)
    result[1, 1, 2] = numpy.sum(-x194 * x218 * x219)
    result[1, 1, 3] = numpy.sum(-x223 * x224)
    result[1, 1, 4] = numpy.sum(-x127 * x191 * x216)
    result[1, 1, 5] = numpy.sum(-x191 * x213 * x225)
    result[1, 2, 0] = numpy.sum(-x226 * x227)
    result[1, 2, 1] = numpy.sum(-x228 * x229)
    result[1, 2, 2] = numpy.sum(-x112 * x184 * x228)
    result[1, 2, 3] = numpy.sum(-x168 * x191 * x204)
    result[1, 2, 4] = numpy.sum(-x171 * x191 * x230)
    result[1, 2, 5] = numpy.sum(-x119 * x191 * x231)
    result[1, 3, 0] = numpy.sum(-x235 * x236)
    result[1, 3, 1] = numpy.sum(-x237 * x238)
    result[1, 3, 2] = numpy.sum(-x127 * x14 * x235)
    result[1, 3, 3] = numpy.sum(-x241 * x243)
    result[1, 3, 4] = numpy.sum(-x219 * x237 * x243)
    result[1, 3, 5] = numpy.sum(-x225 * x235 * x242)
    result[1, 4, 0] = numpy.sum(-x110 * x218 * x50)
    result[1, 4, 1] = numpy.sum(-x109 * x244 * x245)
    result[1, 4, 2] = numpy.sum(-x14 * x213 * x246)
    result[1, 4, 3] = numpy.sum(-x110 * x223 * x243)
    result[1, 4, 4] = numpy.sum(-x216 * x242 * x246)
    result[1, 4, 5] = numpy.sum(-x119 * x248 * x249)
    result[1, 5, 0] = numpy.sum(-x141 * x231 * x50)
    result[1, 5, 1] = numpy.sum(-x175 * x190 * x250)
    result[1, 5, 2] = numpy.sum(-x14 * x184 * x251)
    result[1, 5, 3] = numpy.sum(-x175 * x205 * x242)
    result[1, 5, 4] = numpy.sum(-x144 * x230 * x248)
    result[1, 5, 5] = numpy.sum(-x147 * x184 * x243)
    result[1, 6, 0] = numpy.sum(-x253 * x256)
    result[1, 6, 1] = numpy.sum(-x257 * x259)
    result[1, 6, 2] = numpy.sum(-x253 * x259 * x72)
    result[1, 6, 3] = numpy.sum(-x264 * x265)
    result[1, 6, 4] = numpy.sum(-x198 * x257 * x266)
    result[1, 6, 5] = numpy.sum(-x211 * x253 * x9)
    result[1, 7, 0] = numpy.sum(-x168 * x235 * x254)
    result[1, 7, 1] = numpy.sum(-x109 * x258 * x267)
    result[1, 7, 2] = numpy.sum(-x171 * x235 * x268)
    result[1, 7, 3] = numpy.sum(-x109 * x241 * x269)
    result[1, 7, 4] = numpy.sum(-x112 * x267 * x9)
    result[1, 7, 5] = numpy.sum(-x119 * x235 * x270)
    result[1, 8, 0] = numpy.sum(-x175 * x213 * x255)
    result[1, 8, 1] = numpy.sum(-x175 * x244 * x271)
    result[1, 8, 2] = numpy.sum(-x144 * x258 * x272)
    result[1, 8, 3] = numpy.sum(-x141 * x223 * x270)
    result[1, 8, 4] = numpy.sum(-x216 * x251 * x9)
    result[1, 8, 5] = numpy.sum(-x147 * x218 * x9)
    result[1, 9, 0] = numpy.sum(-x177 * x226 * x274)
    result[1, 9, 1] = numpy.sum(-x177 * x229 * x276)
    result[1, 9, 2] = numpy.sum(-x182 * x184 * x259)
    result[1, 9, 3] = numpy.sum(-x180 * x205 * x277)
    result[1, 9, 4] = numpy.sum(-x182 * x210 * x9)
    result[1, 9, 5] = numpy.sum(-x183 * x184 * x278)
    result[2, 0, 0] = numpy.sum(-x188 * x279)
    result[2, 0, 1] = numpy.sum(-x197 * x279 * x82)
    result[2, 0, 2] = numpy.sum(-x196 * x281)
    result[2, 0, 3] = numpy.sum(-x208 * x279 * x282)
    result[2, 0, 4] = numpy.sum(-x208 * x283 * x61)
    result[2, 0, 5] = numpy.sum(-x209 * x290)
    result[2, 1, 0] = numpy.sum(-x227 * x291)
    result[2, 1, 1] = numpy.sum(-x228 * x279 * x91)
    result[2, 1, 2] = numpy.sum(-x228 * x292)
    result[2, 1, 3] = numpy.sum(-x104 * x191 * x293)
    result[2, 1, 4] = numpy.sum(-x176 * x191 * x294)
    result[2, 1, 5] = numpy.sum(-x174 * x191 * x289)
    result[2, 2, 0] = numpy.sum(-x214 * x296)
    result[2, 2, 1] = numpy.sum(-x194 * x298 * x61)
    result[2, 2, 2] = numpy.sum(-x217 * x300)
    result[2, 2, 3] = numpy.sum(-x191 * x296 * x301)
    result[2, 2, 4] = numpy.sum(-x142 * x191 * x300)
    result[2, 2, 5] = numpy.sum(-x224 * x305)
    result[2, 3, 0] = numpy.sum(-x121 * x293 * x50)
    result[2, 3, 1] = numpy.sum(-x14 * x279 * x306)
    result[2, 3, 2] = numpy.sum(-x170 * x250 * x281)
    result[2, 3, 3] = numpy.sum(-x130 * x243 * x279)
    result[2, 3, 4] = numpy.sum(-x124 * x248 * x294)
    result[2, 3, 5] = numpy.sum(-x170 * x242 * x290)
    result[2, 4, 0] = numpy.sum(-x298 * x50 * x87)
    result[2, 4, 1] = numpy.sum(-x139 * x14 * x296)
    result[2, 4, 2] = numpy.sum(-x245 * x307 * x87)
    result[2, 4, 3] = numpy.sum(-0.1666666666666667 * x104 * x248 * x296)
    result[2, 4, 4] = numpy.sum(-x139 * x242 * x300)
    result[2, 4, 5] = numpy.sum(-x305 * x308 * x87)
    result[2, 5, 0] = numpy.sum(-x236 * x312)
    result[2, 5, 1] = numpy.sum(-x14 * x142 * x312)
    result[2, 5, 2] = numpy.sum(-x238 * x313)
    result[2, 5, 3] = numpy.sum(-x242 * x301 * x312)
    result[2, 5, 4] = numpy.sum(-x308 * x313 * x61)
    result[2, 5, 5] = numpy.sum(-x243 * x316)
    result[2, 6, 0] = numpy.sum(-x151 * x274 * x291)
    result[2, 6, 1] = numpy.sum(-x155 * x259 * x279)
    result[2, 6, 2] = numpy.sum(-x151 * x276 * x292)
    result[2, 6, 3] = numpy.sum(-x164 * x278 * x279)
    result[2, 6, 4] = numpy.sum(-x155 * x283 * x9)
    result[2, 6, 5] = numpy.sum(-x159 * x277 * x290)
    result[2, 7, 0] = numpy.sum(-x170 * x255 * x296)
    result[2, 7, 1] = numpy.sum(-x124 * x268 * x317)
    result[2, 7, 2] = numpy.sum(-x170 * x271 * x307)
    result[2, 7, 3] = numpy.sum(-x130 * x297 * x9)
    result[2, 7, 4] = numpy.sum(-x300 * x306 * x9)
    result[2, 7, 5] = numpy.sum(-x121 * x270 * x305)
    result[2, 8, 0] = numpy.sum(-x174 * x254 * x312)
    result[2, 8, 1] = numpy.sum(-x176 * x268 * x312)
    result[2, 8, 2] = numpy.sum(-x258 * x318 * x87)
    result[2, 8, 3] = numpy.sum(-x104 * x270 * x312)
    result[2, 8, 4] = numpy.sum(-x318 * x9 * x91)
    result[2, 8, 5] = numpy.sum(-x269 * x316 * x87)
    result[2, 9, 0] = numpy.sum(-x256 * x320)
    result[2, 9, 1] = numpy.sum(-x259 * x320 * x61)
    result[2, 9, 2] = numpy.sum(-x259 * x321)
    result[2, 9, 3] = numpy.sum(-x282 * x320 * x9)
    result[2, 9, 4] = numpy.sum(-x266 * x321 * x82)
    result[2, 9, 5] = numpy.sum(-x265 * x326)
    result[3, 0, 0] = numpy.sum(-x329 * x330)
    result[3, 0, 1] = numpy.sum(-x331 * x334)
    result[3, 0, 2] = numpy.sum(-x331 * x335 * x72)
    result[3, 0, 3] = numpy.sum(-x338 * x340)
    result[3, 0, 4] = numpy.sum(-x339 * x341 * x72)
    result[3, 0, 5] = numpy.sum(-x328 * x342 * x86)
    result[3, 1, 0] = numpy.sum(-x187 * x345)
    result[3, 1, 1] = numpy.sum(-x193 * x347)
    result[3, 1, 2] = numpy.sum(-x127 * x193 * x344)
    result[3, 1, 3] = numpy.sum(-x348 * x352)
    result[3, 1, 4] = numpy.sum(-x127 * x346 * x348)
    result[3, 1, 5] = numpy.sum(-x225 * x344 * x348)
    result[3, 2, 0] = numpy.sum(-x168 * x187 * x328)
    result[3, 2, 1] = numpy.sum(-x193 * x333 * x353)
    result[3, 2, 2] = numpy.sum(-x171 * x328 * x354)
    result[3, 2, 3] = numpy.sum(-x168 * x337 * x348)
    result[3, 2, 4] = numpy.sum(-x171 * x333 * x355)
    result[3, 2, 5] = numpy.sum(-x119 * x329 * x355)
    result[3, 3, 0] = numpy.sum(-x357 * x49)
    result[3, 3, 1] = numpy.sum(-x358 * x361)
    result[3, 3, 2] = numpy.sum(-x127 * x356 * x358)
    result[3, 3, 3] = numpy.sum(-x3 * x364)
    result[3, 3, 4] = numpy.sum(-x360 * x365 * x73)
    result[3, 3, 5] = numpy.sum(-x356 * x366 * x84)
    result[3, 4, 0] = numpy.sum(-x344 * x353 * x49)
    result[3, 4, 1] = numpy.sum(-x178 * x358 * x367)
    result[3, 4, 2] = numpy.sum(-x246 * x344 * x358)
    result[3, 4, 3] = numpy.sum(-x110 * x351 * x368)
    result[3, 4, 4] = numpy.sum(-x137 * x3 * x367)
    result[3, 4, 5] = numpy.sum(-x119 * x344 * x369)
    result[3, 5, 0] = numpy.sum(-x175 * x329 * x49)
    result[3, 5, 1] = numpy.sum(-x175 * x333 * x370)
    result[3, 5, 2] = numpy.sum(-x144 * x329 * x371)
    result[3, 5, 3] = numpy.sum(-x141 * x337 * x366)
    result[3, 5, 4] = numpy.sum(-x251 * x3 * x333)
    result[3, 5, 5] = numpy.sum(-x147 * x328 * x366)
    result[3, 6, 0] = numpy.sum(-x372 * x375)
    result[3, 6, 1] = numpy.sum(-x376 * x377)
    result[3, 6, 2] = numpy.sum(-x198 * x372 * x378)
    result[3, 6, 3] = numpy.sum(
        -x58 * (x363 * x87 + x52 * (x241 + 2.0 * x349 + x350 + x359))
    )
    result[3, 6, 4] = numpy.sum(-x376 * x73)
    result[3, 6, 5] = numpy.sum(-x211 * x372)
    result[3, 7, 0] = numpy.sum(-x168 * x356 * x373)
    result[3, 7, 1] = numpy.sum(-x134 * x360 * x4)
    result[3, 7, 2] = numpy.sum(-x112 * x356 * x379)
    result[3, 7, 3] = numpy.sum(-x109 * x364)
    result[3, 7, 4] = numpy.sum(-x112 * x361)
    result[3, 7, 5] = numpy.sum(-x119 * x357)
    result[3, 8, 0] = numpy.sum(-x175 * x344 * x374)
    result[3, 8, 1] = numpy.sum(-x141 * x346 * x379)
    result[3, 8, 2] = numpy.sum(-x251 * x344 * x4)
    result[3, 8, 3] = numpy.sum(-x141 * x352)
    result[3, 8, 4] = numpy.sum(-x144 * x347)
    result[3, 8, 5] = numpy.sum(-x147 * x345)
    result[3, 9, 0] = numpy.sum(-x180 * x329 * x373 * x380)
    result[3, 9, 1] = numpy.sum(-x180 * x341 * x4)
    result[3, 9, 2] = numpy.sum(-x182 * x335 * x4)
    result[3, 9, 3] = numpy.sum(-x179 * x338)
    result[3, 9, 4] = numpy.sum(-x182 * x334)
    result[3, 9, 5] = numpy.sum(-x183 * x329)
    result[4, 0, 0] = numpy.sum(-x184 * x279 * x330 * x57)
    result[4, 0, 1] = numpy.sum(-x210 * x279 * x331)
    result[4, 0, 2] = numpy.sum(-x184 * x283 * x331)
    result[4, 0, 3] = numpy.sum(-x205 * x279 * x381)
    result[4, 0, 4] = numpy.sum(-x190 * x275 * x281 * x339)
    result[4, 0, 5] = numpy.sum(-x184 * x290 * x381)
    result[4, 1, 0] = numpy.sum(-x187 * x218 * x279)
    result[4, 1, 1] = numpy.sum(-x216 * x279 * x382)
    result[4, 1, 2] = numpy.sum(-x193 * x272 * x281)
    result[4, 1, 3] = numpy.sum(-x223 * x293 * x348)
    result[4, 1, 4] = numpy.sum(-x244 * x281 * x383)
    result[4, 1, 5] = numpy.sum(-x213 * x290 * x355)
    result[4, 2, 0] = numpy.sum(-x184 * x187 * x297)
    result[4, 2, 1] = numpy.sum(-x193 * x230 * x317)
    result[4, 2, 2] = numpy.sum(-x184 * x300 * x382)
    result[4, 2, 3] = numpy.sum(-x205 * x296 * x355)
    result[4, 2, 4] = numpy.sum(-x190 * x307 * x383)
    result[4, 2, 5] = numpy.sum(-x231 * x305 * x348)
    result[4, 3, 0] = numpy.sum(-x235 * x293 * x49)
    result[4, 3, 1] = numpy.sum(-x267 * x279 * x358)
    result[4, 3, 2] = numpy.sum(-x235 * x294 * x370)
    result[4, 3, 3] = numpy.sum(-x241 * x279 * x368)
    result[4, 3, 4] = numpy.sum(-x267 * x281 * x3)
    result[4, 3, 5] = numpy.sum(-x235 * x289 * x366)
    result[4, 4, 0] = numpy.sum(-x272 * x296 * x49)
    result[4, 4, 1] = numpy.sum(-x244 * x296 * x384)
    result[4, 4, 2] = numpy.sum(-x213 * x307 * x384)
    result[4, 4, 3] = numpy.sum(-x223 * x298 * x3)
    result[4, 4, 4] = numpy.sum(-x244 * x300 * x385)
    result[4, 4, 5] = numpy.sum(-x3 * x305 * x386)
    result[4, 5, 0] = numpy.sum(-x231 * x312 * x49)
    result[4, 5, 1] = numpy.sum(-x230 * x312 * x370)
    result[4, 5, 2] = numpy.sum(-x184 * x318 * x358)
    result[4, 5, 3] = numpy.sum(-x204 * x312 * x366)
    result[4, 5, 4] = numpy.sum(-x190 * x3 * x318)
    result[4, 5, 5] = numpy.sum(-x184 * x316 * x368)
    result[4, 6, 0] = numpy.sum(-x253 * x279 * x375)
    result[4, 6, 1] = numpy.sum(-x257 * x279 * x387)
    result[4, 6, 2] = numpy.sum(-x253 * x283 * x4)
    result[4, 6, 3] = numpy.sum(-x264 * x279 * x58)
    result[4, 6, 4] = numpy.sum(-x257 * x283)
    result[4, 6, 5] = numpy.sum(-x253 * x290 * x54)
    result[4, 7, 0] = numpy.sum(-x235 * x296 * x388)
    result[4, 7, 1] = numpy.sum(-x237 * x298 * x4)
    result[4, 7, 2] = numpy.sum(-x235 * x300 * x379)
    result[4, 7, 3] = numpy.sum(-x241 * x297)
    result[4, 7, 4] = numpy.sum(-x125 * x237 * x300)
    result[4, 7, 5] = numpy.sum(-x105 * x235 * x305)
    result[4, 8, 0] = numpy.sum(-x213 * x312 * x388)
    result[4, 8, 1] = numpy.sum(-x216 * x312 * x379)
    result[4, 8, 2] = numpy.sum(-x313 * x386 * x4)
    result[4, 8, 3] = numpy.sum(-x105 * x223 * x312)
    result[4, 8, 4] = numpy.sum(-x125 * x216 * x313)
    result[4, 8, 5] = numpy.sum(-x218 * x316)
    result[4, 9, 0] = numpy.sum(-x184 * x320 * x375)
    result[4, 9, 1] = numpy.sum(-x210 * x320 * x4)
    result[4, 9, 2] = numpy.sum(-x184 * x321 * x387)
    result[4, 9, 3] = numpy.sum(-x205 * x320 * x54)
    result[4, 9, 4] = numpy.sum(-x210 * x321)
    result[4, 9, 5] = numpy.sum(-x184 * x326 * x58)
    result[5, 0, 0] = numpy.sum(-x330 * x391)
    result[5, 0, 1] = numpy.sum(-x331 * x392 * x61)
    result[5, 0, 2] = numpy.sum(-x331 * x395)
    result[5, 0, 3] = numpy.sum(-x342 * x390 * x81)
    result[5, 0, 4] = numpy.sum(-x160 * x339 * x394 * x61)
    result[5, 0, 5] = numpy.sum(-x340 * x398)
    result[5, 1, 0] = numpy.sum(-x174 * x187 * x390)
    result[5, 1, 1] = numpy.sum(-x176 * x354 * x390)
    result[5, 1, 2] = numpy.sum(-x382 * x399)
    result[5, 1, 3] = numpy.sum(-x104 * x355 * x391)
    result[5, 1, 4] = numpy.sum(-x176 * x355 * x394)
    result[5, 1, 5] = numpy.sum(-x174 * x348 * x397)
    result[5, 2, 0] = numpy.sum(-x187 * x402)
    result[5, 2, 1] = numpy.sum(-x142 * x193 * x401)
    result[5, 2, 2] = numpy.sum(-x193 * x404)
    result[5, 2, 3] = numpy.sum(-x301 * x348 * x401)
    result[5, 2, 4] = numpy.sum(-x142 * x348 * x403)
    result[5, 2, 5] = numpy.sum(-x348 * x408)
    result[5, 3, 0] = numpy.sum(-x170 * x391 * x49)
    result[5, 3, 1] = numpy.sum(-x124 * x371 * x391)
    result[5, 3, 2] = numpy.sum(-x170 * x370 * x394)
    result[5, 3, 3] = numpy.sum(-x130 * x366 * x390)
    result[5, 3, 4] = numpy.sum(-x3 * x306 * x394)
    result[5, 3, 5] = numpy.sum(-x121 * x366 * x397)
    result[5, 4, 0] = numpy.sum(-x401 * x49 * x87 * x94)
    result[5, 4, 1] = numpy.sum(-x139 * x358 * x401)
    result[5, 4, 2] = numpy.sum(-0.25 * x152 * x358 * x403)
    result[5, 4, 3] = numpy.sum(-x104 * x369 * x401)
    result[5, 4, 4] = numpy.sum(-x135 * x385 * x403)
    result[5, 4, 5] = numpy.sum(-x368 * x407 * x97)
    result[5, 5, 0] = numpy.sum(-x410 * x49)
    result[5, 5, 1] = numpy.sum(-x142 * x358 * x409)
    result[5, 5, 2] = numpy.sum(-x358 * x413)
    result[5, 5, 3] = numpy.sum(-x366 * x409 * x78)
    result[5, 5, 4] = numpy.sum(-x365 * x412 * x62)
    result[5, 5, 5] = numpy.sum(-x3 * x416)
    result[5, 6, 0] = numpy.sum(-x159 * x374 * x380 * x390)
    result[5, 6, 1] = numpy.sum(-x155 * x392 * x4)
    result[5, 6, 2] = numpy.sum(-x151 * x160 * x399 * x4)
    result[5, 6, 3] = numpy.sum(-x164 * x391)
    result[5, 6, 4] = numpy.sum(-x155 * x395)
    result[5, 6, 5] = numpy.sum(-x153 * x398)
    result[5, 7, 0] = numpy.sum(-x170 * x374 * x401)
    result[5, 7, 1] = numpy.sum(-x306 * x4 * x401)
    result[5, 7, 2] = numpy.sum(-x121 * x379 * x403)
    result[5, 7, 3] = numpy.sum(-x130 * x402)
    result[5, 7, 4] = numpy.sum(-x124 * x404)
    result[5, 7, 5] = numpy.sum(-x121 * x408)
    result[5, 8, 0] = numpy.sum(-x174 * x373 * x409)
    result[5, 8, 1] = numpy.sum(-x379 * x409 * x91)
    result[5, 8, 2] = numpy.sum(-x133 * x4 * x412 * x97)
    result[5, 8, 3] = numpy.sum(-x104 * x410)
    result[5, 8, 4] = numpy.sum(-x413 * x91)
    result[5, 8, 5] = numpy.sum(-x416 * x87)
    result[5, 9, 0] = numpy.sum(-x375 * x417)
    result[5, 9, 1] = numpy.sum(-x378 * x417 * x82)
    result[5, 9, 2] = numpy.sum(-x377 * x418)
    result[5, 9, 3] = numpy.sum(-x282 * x417)
    result[5, 9, 4] = numpy.sum(-x418 * x62)
    result[5, 9, 5] = numpy.sum(
        -x58 * (x109 * x415 + x52 * (x316 + 2.0 * x405 + x406 + x411))
    )
    return result


def quadrupole3d_33(ax, da, A, bx, db, B, R):
    """Cartesian 3D (ff) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 10, 10), dtype=float)

    x0 = (ax + bx) ** (-1.0)
    x1 = x0 * (ax * A[0] + bx * B[0])
    x2 = -x1
    x3 = x2 + A[0]
    x4 = -2.0 * x1
    x5 = x4 + B[0]
    x6 = x5 + R[0]
    x7 = x0 * x6
    x8 = x2 + B[0]
    x9 = x2 + R[0]
    x10 = x8 * x9
    x11 = 2.0 * x10
    x12 = x0 + x11
    x13 = x12 * x3
    x14 = x13 + x7
    x15 = 4.0 * x14
    x16 = x15 * x3
    x17 = x3 * x8
    x18 = 4.0 * x17
    x19 = x8**2
    x20 = 3.0 * x0
    x21 = 2.0 * x19 + x20
    x22 = x0 * (x18 + x21)
    x23 = x0 * (x5 + A[0])
    x24 = 2.0 * x17
    x25 = x0 + x24
    x26 = x25 * x8
    x27 = x23 + x26
    x28 = 2.0 * x3
    x29 = x22 + x27 * x28
    x30 = x3 * x9
    x31 = 2.0 * x30
    x32 = x0 * (x11 + x20 + x24 + x31)
    x33 = 4.0 * x32
    x34 = x15 * x8 + x33
    x35 = x0 * (x16 + x29 + x34)
    x36 = x12 * x8
    x37 = 2.0 * x13
    x38 = x20 * x6 + x37
    x39 = x0 * (x27 + x36 + x38)
    x40 = 2.0 * x14
    x41 = x32 + x40 * x8
    x42 = x3 * x41
    x43 = x39 + x42
    x44 = 4.0 * x8
    x45 = x12 * x9
    x46 = x0 * (x4 + A[0] + R[0])
    x47 = x0 + x31
    x48 = x47 * x9
    x49 = x46 + x48
    x50 = x0 * (x38 + x45 + x49)
    x51 = x32 + x40 * x9
    x52 = x51 * x8
    x53 = x50 + x52
    x54 = x15 * x9
    x55 = 4.0 * x10
    x56 = x9**2
    x57 = x20 + 2.0 * x56
    x58 = x45 + x7
    x59 = 2.0 * x8
    x60 = x0 * (x55 + x57) + x58 * x59
    x61 = x0 * (x34 + x54 + x60)
    x62 = 6.0 * x3
    x63 = x28 * x53 + x61
    x64 = x3 * x51
    x65 = 2.0 * x0
    x66 = x65 * (x43 + 2.0 * x50 + x52 + x64)
    x67 = x63 * x8 + x66
    x68 = ax * bx * x0
    x69 = (
        5.568327996831708
        * da
        * db
        * numpy.exp(-x68 * ((A[0] - B[0]) ** 2 + (A[1] - B[1]) ** 2 + (A[2] - B[2]) ** 2))
    )
    x70 = 0.004166666666666667 * x69
    x71 = x0**1.5
    x72 = x70 * x71
    x73 = x0 * (ax * A[1] + bx * B[1])
    x74 = -x73
    x75 = x74 + B[1]
    x76 = 2.23606797749979
    x77 = x69 * x71
    x78 = 0.008333333333333333 * x77
    x79 = x76 * x78
    x80 = x79 * (x3 * x63 + x66)
    x81 = x0 * (ax * A[2] + bx * B[2])
    x82 = -x81
    x83 = x82 + B[2]
    x84 = x75**2
    x85 = 0.5 * x0
    x86 = x84 + x85
    x87 = 4.0 * x30
    x88 = x0 * (x57 + x87) + x28 * x49
    x89 = x50 + x64
    x90 = x0 * (x16 + x33 + x54 + x88) + x28 * x89
    x91 = x0**1.5
    x92 = x69 * x91
    x93 = 0.008333333333333333 * x92
    x94 = x76 * x93
    x95 = x90 * x94
    x96 = 3.872983346207417
    x97 = x78 * x96
    x98 = x83 * x97
    x99 = x83**2
    x100 = x85 + x99
    x101 = x3 * x47
    x102 = 0.01666666666666667 * x3 * x88 + 0.01666666666666667 * x65 * (
        x101 + 2.0 * x46 + x48
    )
    x103 = 1.5 * x0
    x104 = x75 * x92
    x105 = x104 * (x103 + x84)
    x106 = x83 * x92
    x107 = x102 * x76
    x108 = x106 * (x103 + x99)
    x109 = x74 + A[1]
    x110 = x67 * x79
    x111 = x109 * x75
    x112 = x111 + x85
    x113 = 0.04166666666666667 * x92
    x114 = x113 * x63
    x115 = 0.04166666666666667 * x77
    x116 = x115 * x63
    x117 = -2.0 * x73
    x118 = x117 + B[1]
    x119 = x0 * (x118 + A[1])
    x120 = 2.0 * x111
    x121 = x0 + x120
    x122 = x121 * x75
    x123 = x119 + x122
    x124 = x113 * x89
    x125 = 0.08333333333333333 * x106
    x126 = 1.732050807568877
    x127 = x126 * x89
    x128 = 0.08333333333333333 * x92
    x129 = x100 * x128
    x130 = 2.0 * x84
    x131 = 4.0 * x111 + x20
    x132 = x0 * (x130 + x131)
    x133 = 2.0 * x123
    x134 = x132 + x133 * x75
    x135 = x70 * x91
    x136 = x135 * x76
    x137 = x136 * x88
    x138 = x113 * x88
    x139 = 0.01666666666666667 * x108
    x140 = x76 * x88
    x141 = x82 + A[2]
    x142 = x141 * x83
    x143 = x142 + x85
    x144 = x128 * x86
    x145 = 0.08333333333333333 * x104
    x146 = -2.0 * x81
    x147 = x146 + B[2]
    x148 = x0 * (x147 + A[2])
    x149 = 2.0 * x142
    x150 = x0 + x149
    x151 = x150 * x83
    x152 = x148 + x151
    x153 = 0.01666666666666667 * x105
    x154 = 2.0 * x99
    x155 = 4.0 * x142 + x20
    x156 = x0 * (x154 + x155)
    x157 = 2.0 * x152
    x158 = x156 + x157 * x83
    x159 = x109**2
    x160 = x159 + x85
    x161 = x53 * x59 + x61
    x162 = x161 * x94
    x163 = x109 * x121
    x164 = x119 + x163
    x165 = x113 * x53
    x166 = 0.08333333333333333 * x53
    x167 = x109 * x133 + x132
    x168 = 0.02083333333333333 * x92
    x169 = x168 * x51
    x170 = x113 * x83
    x171 = x126 * x51
    x172 = x65 * (2.0 * x119 + x122 + x163)
    x173 = x167 * x75 + x172
    x174 = x49 * x94
    x175 = x113 * x49
    x176 = 0.03333333333333333 * x76
    x177 = x176 * x49
    x178 = x141 * x97
    x179 = x126 * x128
    x180 = x179 * x53
    x181 = x113 * x141
    x182 = 0.25 * x92
    x183 = x143 * x182
    x184 = x109 * x113
    x185 = x93 * x96
    x186 = x185 * x49
    x187 = x179 * x49
    x188 = x141**2
    x189 = x188 + x85
    x190 = x141 * x150
    x191 = x148 + x190
    x192 = x113 * x75
    x193 = x141 * x157 + x156
    x194 = x65 * (2.0 * x148 + x151 + x190)
    x195 = x193 * x83 + x194
    x196 = x109 * (x103 + x159)
    x197 = 2.0 * x7
    x198 = 0.01666666666666667 * x92
    x199 = x198 * (x60 * x8 + x65 * (x197 + x36 + x45))
    x200 = 2.0 * x159
    x201 = 2.0 * x109
    x202 = x0 * (x131 + x200) + x164 * x201
    x203 = x136 * x60
    x204 = 0.01666666666666667 * x106
    x205 = x204 * x76
    x206 = x109 * x167 + x172
    x207 = x58 * x94
    x208 = x185 * x58
    x209 = x176 * x92
    x210 = x209 * x58
    x211 = 4.0 * x75
    x212 = 6.0 * x109
    x213 = x0 * (x123 * x211 + x123 * x212 + 5.0 * x132) + x173 * x201
    x214 = x56 + x85
    x215 = x214 * x93
    x216 = x214 * x76
    x217 = x198 * x216
    x218 = 0.06666666666666667 * x214
    x219 = x199 * x76
    x220 = x128 * x160
    x221 = x179 * x58
    x222 = x128 * x214
    x223 = x128 * x189
    x224 = x141 * (x103 + x188)
    x225 = 0.01666666666666667 * x104
    x226 = x225 * x76
    x227 = 2.0 * x188
    x228 = 2.0 * x141
    x229 = x0 * (x155 + x227) + x191 * x228
    x230 = x141 * x193 + x194
    x231 = 4.0 * x83
    x232 = 6.0 * x141
    x233 = x0 * (x152 * x231 + x152 * x232 + 5.0 * x156) + x195 * x228
    x234 = x74 + R[1]
    x235 = x41 * x8
    x236 = x25 * x3
    x237 = x65 * (2.0 * x23 + x236 + x26)
    x238 = x237 + x29 * x8
    x239 = x35 + x43 * x59
    x240 = x78 * (x0 * (2.0 * x235 + x238 + 5.0 * x39 + 3.0 * x42) + x239 * x3)
    x241 = x234 * x75
    x242 = x241 + x85
    x243 = x28 * x43 + x35
    x244 = x243 * x94
    x245 = x234 * x78
    x246 = x245 * x76
    x247 = x118 + R[1]
    x248 = x0 * x247
    x249 = 2.0 * x241
    x250 = x0 + x249
    x251 = x250 * x75
    x252 = x248 + x251
    x253 = x101 + x46
    x254 = x23 + x236
    x255 = x14 * x28 + x32
    x256 = x0 * (x197 + x253 + x254 + x37) + x255 * x3
    x257 = x256 * x94
    x258 = x204 * x96
    x259 = x198 * x76
    x260 = x100 * x259
    x261 = x20 + 4.0 * x241
    x262 = 2.0 * x75
    x263 = x0 * (x130 + x261) + x252 * x262
    x264 = x3**2
    x265 = x20 + 2.0 * x264
    x266 = x0 * (x265 + x87) + x253 * x28
    x267 = x135 * x266
    x268 = x252 * x94
    x269 = x109 * x234
    x270 = x269 + x85
    x271 = x239 * x94
    x272 = x109 * x250
    x273 = x248 + x272
    x274 = x113 * x43
    x275 = 2.0 * x269
    x276 = x0 * (x120 + x20 + x249 + x275)
    x277 = x262 * x273 + x276
    x278 = x168 * x255
    x279 = x113 * x273
    x280 = x126 * x83
    x281 = 2.0 * x272
    x282 = x20 * x247 + x281
    x283 = x0 * (x123 + x251 + x282)
    x284 = x277 * x75
    x285 = x283 + x284
    x286 = x253 * x94
    x287 = x108 * x176
    x288 = x128 * x43
    x289 = x179 * x242
    x290 = x113 * x234
    x291 = x128 * x253
    x292 = x0 * (x117 + A[1] + R[1])
    x293 = x0 + x275
    x294 = x109 * x293
    x295 = x292 + x294
    x296 = x235 + x39
    x297 = x296 * x94
    x298 = x201 * x273 + x276
    x299 = x168 * x41
    x300 = x109 * x277
    x301 = x283 + x300
    x302 = x113 * x14
    x303 = 4.0 * x273
    x304 = x109 * x303
    x305 = 4.0 * x276
    x306 = x211 * x273 + x305
    x307 = x0 * (x167 + x304 + x306)
    x308 = x262 * x301 + x307
    x309 = x30 + x85
    x310 = x309 * x94
    x311 = x198 * x96
    x312 = x141 * x311
    x313 = x126 * x41
    x314 = x128 * x270
    x315 = x126 * x277
    x316 = x126 * x152
    x317 = x128 * x309
    x318 = x309 * x311
    x319 = x189 * x259
    x320 = x195 * x259
    x321 = x20 + 4.0 * x269
    x322 = x0 * (x200 + x321) + x201 * x295
    x323 = x36 + x7
    x324 = x0 * (x21 + x55) + x323 * x59
    x325 = x135 * x324
    x326 = 2.0 * x248
    x327 = x0 * (x164 + x281 + x295 + x326) + x109 * x298
    x328 = x323 * x94
    x329 = x201 * x301 + x307
    x330 = x10 + x85
    x331 = x330 * x94
    x332 = x0 * (x173 + 5.0 * x283 + 2.0 * x284 + 3.0 * x300) + x109 * x308
    x333 = x78 * x9
    x334 = x333 * x76
    x335 = x295 * x94
    x336 = x128 * x295
    x337 = x128 * x301
    x338 = x179 * x330
    x339 = x113 * x9
    x340 = x198 * x224
    x341 = x209 * x224
    x342 = x311 * x330
    x343 = x230 * x259
    x344 = x82 + R[2]
    x345 = x344 * x78
    x346 = x345 * x76
    x347 = x344 * x83
    x348 = x347 + x85
    x349 = x259 * x86
    x350 = x225 * x96
    x351 = x147 + R[2]
    x352 = x0 * x351
    x353 = 2.0 * x347
    x354 = x0 + x353
    x355 = x354 * x83
    x356 = x352 + x355
    x357 = x356 * x94
    x358 = x20 + 4.0 * x347
    x359 = 2.0 * x83
    x360 = x0 * (x154 + x358) + x356 * x359
    x361 = x113 * x344
    x362 = x179 * x348
    x363 = x141 * x344
    x364 = x363 + x85
    x365 = x141 * x354
    x366 = x352 + x365
    x367 = x113 * x366
    x368 = x126 * x75
    x369 = 2.0 * x363
    x370 = x0 * (x149 + x20 + x353 + x369)
    x371 = x359 * x366 + x370
    x372 = x105 * x176
    x373 = 2.0 * x365
    x374 = x20 * x351 + x373
    x375 = x0 * (x152 + x355 + x374)
    x376 = x371 * x83
    x377 = x375 + x376
    x378 = x259 * x344
    x379 = x160 * x259
    x380 = x311 * x364
    x381 = x128 * x364
    x382 = x123 * x126
    x383 = x182 * x366
    x384 = x126 * x371
    x385 = x0 * (x146 + A[2] + R[2])
    x386 = x0 + x369
    x387 = x141 * x386
    x388 = x385 + x387
    x389 = x228 * x366 + x370
    x390 = x141 * x371
    x391 = x375 + x390
    x392 = 4.0 * x366
    x393 = x141 * x392
    x394 = 4.0 * x370
    x395 = x231 * x366 + x394
    x396 = x0 * (x193 + x393 + x395)
    x397 = x359 * x391 + x396
    x398 = x196 * x198
    x399 = x196 * x209
    x400 = x259 * x9
    x401 = x388 * x94
    x402 = x128 * x388
    x403 = x128 * x391
    x404 = x20 + 4.0 * x363
    x405 = x0 * (x227 + x404) + x228 * x388
    x406 = 2.0 * x352
    x407 = x0 * (x191 + x373 + x388 + x406) + x141 * x389
    x408 = x228 * x391 + x396
    x409 = x0 * (x195 + 5.0 * x375 + 2.0 * x376 + 3.0 * x390) + x141 * x397
    x410 = x0 * (5.0 * x22 + x27 * x44 + x27 * x62) + x238 * x28
    x411 = x234**2
    x412 = x411 + x85
    x413 = x412 * x93
    x414 = x237 + x29 * x3
    x415 = x234 * x250
    x416 = x248 + x415
    x417 = x416 * x94
    x418 = x0 * (x18 + x265) + x254 * x28
    x419 = 2.0 * x411
    x420 = x0 * (x261 + x419) + x262 * x416
    x421 = x136 * x420
    x422 = x185 * x416
    x423 = x198 * (x420 * x75 + x65 * (x251 + x326 + x415))
    x424 = x3 * (x103 + x264)
    x425 = x209 * x424
    x426 = 0.06666666666666667 * x412
    x427 = x234 * x293
    x428 = x292 + x427
    x429 = x428 * x94
    x430 = 2.0 * x234 * x273 + x276
    x431 = x168 * x430
    x432 = x0 * (x282 + x415 + x428)
    x433 = x430 * x75
    x434 = x432 + x433
    x435 = x113 * x434
    x436 = x126 * x430
    x437 = x264 + x85
    x438 = x234 * x303
    x439 = x0 * (x306 + x420 + x438)
    x440 = x262 * x434 + x439
    x441 = x440 * x94
    x442 = x259 * x412
    x443 = x128 * x412
    x444 = x143 * x179
    x445 = x423 * x76
    x446 = x128 * x437
    x447 = x22 + x27 * x59
    x448 = x0 * (x321 + x419) + x201 * x428
    x449 = x136 * x448
    x450 = x109 * x430
    x451 = x432 + x450
    x452 = x113 * x451
    x453 = x17 + x85
    x454 = x201 * x434 + x439
    x455 = x113 * x454
    x456 = x126 * x453
    x457 = x65 * (x301 + 2.0 * x432 + x433 + x450)
    x458 = x454 * x75 + x457
    x459 = x458 * x79
    x460 = x115 * x454
    x461 = x448 * x76
    x462 = x185 * x428
    x463 = x179 * x453
    x464 = x113 * x3
    x465 = x109 * x448 + x65 * (2.0 * x292 + x294 + x427)
    x466 = x103 + x19
    x467 = x198 * x8
    x468 = x466 * x467
    x469 = x19 + x85
    x470 = x0 * (x304 + x305 + x438 + x448) + x201 * x451
    x471 = x470 * x94
    x472 = x79 * (x109 * x454 + x457)
    x473 = x467 * x76
    x474 = x128 * x469
    x475 = x113 * x8
    x476 = x466 * x8
    x477 = x209 * x476
    x478 = x476 * x92
    x479 = x234 * x259
    x480 = x198 * x424
    x481 = x259 * x437
    x482 = x126 * x27
    x483 = x259 * x469
    x484 = x467 * x96
    x485 = x364 * x8
    x486 = x126 * x8
    x487 = x344**2
    x488 = x487 + x85
    x489 = x488 * x93
    x490 = x344 * x354
    x491 = x352 + x490
    x492 = x491 * x94
    x493 = x185 * x491
    x494 = 2.0 * x487
    x495 = x0 * (x358 + x494) + x359 * x491
    x496 = x136 * x495
    x497 = 0.06666666666666667 * x488
    x498 = x198 * (x495 * x83 + x65 * (x355 + x406 + x490))
    x499 = x259 * x488
    x500 = x128 * x488
    x501 = x112 * x179
    x502 = x498 * x76
    x503 = x344 * x386
    x504 = x385 + x503
    x505 = x504 * x94
    x506 = 2.0 * x344 * x366 + x370
    x507 = x168 * x506
    x508 = x126 * x506
    x509 = x0 * (x374 + x490 + x504)
    x510 = x506 * x83
    x511 = x509 + x510
    x512 = x113 * x511
    x513 = x344 * x392
    x514 = x0 * (x395 + x495 + x513)
    x515 = x359 * x511 + x514
    x516 = x515 * x94
    x517 = x185 * x504
    x518 = x0 * (x404 + x494) + x228 * x504
    x519 = x136 * x518
    x520 = x141 * x506
    x521 = x509 + x520
    x522 = x113 * x521
    x523 = x228 * x511 + x514
    x524 = x113 * x523
    x525 = x518 * x76
    x526 = x115 * x523
    x527 = x65 * (x391 + 2.0 * x509 + x510 + x520)
    x528 = x523 * x83 + x527
    x529 = x528 * x79
    x530 = x141 * x518 + x65 * (2.0 * x385 + x387 + x503)
    x531 = x0 * (x393 + x394 + x513 + x518) + x228 * x521
    x532 = x531 * x94
    x533 = x79 * (x141 * x523 + x527)

    # 600 item(s)
    result[0, 0, 0] = numpy.sum(
        x72
        * (x0 * (2.0 * x35 + x43 * x44 + x44 * x53 + x53 * x62 + 5.0 * x61) + x28 * x67)
    )
    result[0, 0, 1] = numpy.sum(x75 * x80)
    result[0, 0, 2] = numpy.sum(x80 * x83)
    result[0, 0, 3] = numpy.sum(x86 * x95)
    result[0, 0, 4] = numpy.sum(x75 * x90 * x98)
    result[0, 0, 5] = numpy.sum(x100 * x95)
    result[0, 0, 6] = numpy.sum(x102 * x105)
    result[0, 0, 7] = numpy.sum(x106 * x107 * x86)
    result[0, 0, 8] = numpy.sum(x100 * x104 * x107)
    result[0, 0, 9] = numpy.sum(x102 * x108)
    result[0, 1, 0] = numpy.sum(x109 * x110)
    result[0, 1, 1] = numpy.sum(x112 * x114)
    result[0, 1, 2] = numpy.sum(x109 * x116 * x83)
    result[0, 1, 3] = numpy.sum(x123 * x124)
    result[0, 1, 4] = numpy.sum(x112 * x125 * x127)
    result[0, 1, 5] = numpy.sum(x109 * x129 * x89)
    result[0, 1, 6] = numpy.sum(x134 * x137)
    result[0, 1, 7] = numpy.sum(x123 * x138 * x83)
    result[0, 1, 8] = numpy.sum(x112 * x129 * x88)
    result[0, 1, 9] = numpy.sum(x109 * x139 * x140)
    result[0, 2, 0] = numpy.sum(x110 * x141)
    result[0, 2, 1] = numpy.sum(x116 * x141 * x75)
    result[0, 2, 2] = numpy.sum(x114 * x143)
    result[0, 2, 3] = numpy.sum(x141 * x144 * x89)
    result[0, 2, 4] = numpy.sum(x127 * x143 * x145)
    result[0, 2, 5] = numpy.sum(x124 * x152)
    result[0, 2, 6] = numpy.sum(x140 * x141 * x153)
    result[0, 2, 7] = numpy.sum(x143 * x144 * x88)
    result[0, 2, 8] = numpy.sum(x138 * x152 * x75)
    result[0, 2, 9] = numpy.sum(x137 * x158)
    result[0, 3, 0] = numpy.sum(x160 * x162)
    result[0, 3, 1] = numpy.sum(x164 * x165)
    result[0, 3, 2] = numpy.sum(x106 * x160 * x166)
    result[0, 3, 3] = numpy.sum(x167 * x169)
    result[0, 3, 4] = numpy.sum(x164 * x170 * x171)
    result[0, 3, 5] = numpy.sum(x129 * x160 * x51)
    result[0, 3, 6] = numpy.sum(x173 * x174)
    result[0, 3, 7] = numpy.sum(x167 * x175 * x83)
    result[0, 3, 8] = numpy.sum(x129 * x164 * x49)
    result[0, 3, 9] = numpy.sum(x108 * x160 * x177)
    result[0, 4, 0] = numpy.sum(x109 * x161 * x178)
    result[0, 4, 1] = numpy.sum(x112 * x141 * x180)
    result[0, 4, 2] = numpy.sum(x109 * x143 * x180)
    result[0, 4, 3] = numpy.sum(x123 * x171 * x181)
    result[0, 4, 4] = numpy.sum(x112 * x183 * x51)
    result[0, 4, 5] = numpy.sum(x152 * x171 * x184)
    result[0, 4, 6] = numpy.sum(x134 * x141 * x186)
    result[0, 4, 7] = numpy.sum(x123 * x143 * x187)
    result[0, 4, 8] = numpy.sum(x112 * x152 * x187)
    result[0, 4, 9] = numpy.sum(x109 * x158 * x186)
    result[0, 5, 0] = numpy.sum(x162 * x189)
    result[0, 5, 1] = numpy.sum(x104 * x166 * x189)
    result[0, 5, 2] = numpy.sum(x165 * x191)
    result[0, 5, 3] = numpy.sum(x144 * x189 * x51)
    result[0, 5, 4] = numpy.sum(x171 * x191 * x192)
    result[0, 5, 5] = numpy.sum(x169 * x193)
    result[0, 5, 6] = numpy.sum(x105 * x177 * x189)
    result[0, 5, 7] = numpy.sum(x144 * x191 * x49)
    result[0, 5, 8] = numpy.sum(x175 * x193 * x75)
    result[0, 5, 9] = numpy.sum(x174 * x195)
    result[0, 6, 0] = numpy.sum(x196 * x199)
    result[0, 6, 1] = numpy.sum(x202 * x203)
    result[0, 6, 2] = numpy.sum(x196 * x205 * x60)
    result[0, 6, 3] = numpy.sum(x206 * x207)
    result[0, 6, 4] = numpy.sum(x202 * x208 * x83)
    result[0, 6, 5] = numpy.sum(x100 * x196 * x210)
    result[0, 6, 6] = numpy.sum(x213 * x215)
    result[0, 6, 7] = numpy.sum(x204 * x206 * x216)
    result[0, 6, 8] = numpy.sum(x100 * x202 * x217)
    result[0, 6, 9] = numpy.sum(x108 * x196 * x218)
    result[0, 7, 0] = numpy.sum(x141 * x160 * x219)
    result[0, 7, 1] = numpy.sum(x164 * x181 * x60)
    result[0, 7, 2] = numpy.sum(x143 * x220 * x60)
    result[0, 7, 3] = numpy.sum(x167 * x181 * x58)
    result[0, 7, 4] = numpy.sum(x143 * x164 * x221)
    result[0, 7, 5] = numpy.sum(x152 * x220 * x58)
    result[0, 7, 6] = numpy.sum(x141 * x173 * x217)
    result[0, 7, 7] = numpy.sum(x143 * x167 * x222)
    result[0, 7, 8] = numpy.sum(x152 * x164 * x222)
    result[0, 7, 9] = numpy.sum(x158 * x160 * x217)
    result[0, 8, 0] = numpy.sum(x109 * x189 * x219)
    result[0, 8, 1] = numpy.sum(x112 * x223 * x60)
    result[0, 8, 2] = numpy.sum(x184 * x191 * x60)
    result[0, 8, 3] = numpy.sum(x123 * x223 * x58)
    result[0, 8, 4] = numpy.sum(x112 * x191 * x221)
    result[0, 8, 5] = numpy.sum(x184 * x193 * x58)
    result[0, 8, 6] = numpy.sum(x134 * x189 * x217)
    result[0, 8, 7] = numpy.sum(x123 * x191 * x222)
    result[0, 8, 8] = numpy.sum(x112 * x193 * x222)
    result[0, 8, 9] = numpy.sum(x109 * x195 * x217)
    result[0, 9, 0] = numpy.sum(x199 * x224)
    result[0, 9, 1] = numpy.sum(x224 * x226 * x60)
    result[0, 9, 2] = numpy.sum(x203 * x229)
    result[0, 9, 3] = numpy.sum(x210 * x224 * x86)
    result[0, 9, 4] = numpy.sum(x208 * x229 * x75)
    result[0, 9, 5] = numpy.sum(x207 * x230)
    result[0, 9, 6] = numpy.sum(x105 * x218 * x224)
    result[0, 9, 7] = numpy.sum(x217 * x229 * x86)
    result[0, 9, 8] = numpy.sum(x216 * x225 * x230)
    result[0, 9, 9] = numpy.sum(x215 * x233)
    result[1, 0, 0] = numpy.sum(x234 * x240)
    result[1, 0, 1] = numpy.sum(x242 * x244)
    result[1, 0, 2] = numpy.sum(x243 * x246 * x83)
    result[1, 0, 3] = numpy.sum(x252 * x257)
    result[1, 0, 4] = numpy.sum(x242 * x256 * x258)
    result[1, 0, 5] = numpy.sum(x234 * x256 * x260)
    result[1, 0, 6] = numpy.sum(x263 * x267)
    result[1, 0, 7] = numpy.sum(x266 * x268 * x83)
    result[1, 0, 8] = numpy.sum(x242 * x260 * x266)
    result[1, 0, 9] = numpy.sum(x139 * x234 * x266)
    result[1, 1, 0] = numpy.sum(x270 * x271)
    result[1, 1, 1] = numpy.sum(x273 * x274)
    result[1, 1, 2] = numpy.sum(x125 * x270 * x43)
    result[1, 1, 3] = numpy.sum(x277 * x278)
    result[1, 1, 4] = numpy.sum(x255 * x279 * x280)
    result[1, 1, 5] = numpy.sum(x129 * x255 * x270)
    result[1, 1, 6] = numpy.sum(x285 * x286)
    result[1, 1, 7] = numpy.sum(x170 * x253 * x277)
    result[1, 1, 8] = numpy.sum(x129 * x253 * x273)
    result[1, 1, 9] = numpy.sum(x253 * x270 * x287)
    result[1, 2, 0] = numpy.sum(x141 * x239 * x246)
    result[1, 2, 1] = numpy.sum(x141 * x242 * x288)
    result[1, 2, 2] = numpy.sum(x143 * x234 * x288)
    result[1, 2, 3] = numpy.sum(x181 * x252 * x255)
    result[1, 2, 4] = numpy.sum(x143 * x255 * x289)
    result[1, 2, 5] = numpy.sum(x152 * x255 * x290)
    result[1, 2, 6] = numpy.sum(x141 * x263 * x286)
    result[1, 2, 7] = numpy.sum(x143 * x252 * x291)
    result[1, 2, 8] = numpy.sum(x152 * x242 * x291)
    result[1, 2, 9] = numpy.sum(x158 * x234 * x286)
    result[1, 3, 0] = numpy.sum(x295 * x297)
    result[1, 3, 1] = numpy.sum(x298 * x299)
    result[1, 3, 2] = numpy.sum(x170 * x295 * x41)
    result[1, 3, 3] = numpy.sum(x301 * x302)
    result[1, 3, 4] = numpy.sum(x280 * x298 * x302)
    result[1, 3, 5] = numpy.sum(x129 * x14 * x295)
    result[1, 3, 6] = numpy.sum(x308 * x310)
    result[1, 3, 7] = numpy.sum(x125 * x301 * x309)
    result[1, 3, 8] = numpy.sum(x129 * x298 * x309)
    result[1, 3, 9] = numpy.sum(x287 * x295 * x309)
    result[1, 4, 0] = numpy.sum(x270 * x296 * x312)
    result[1, 4, 1] = numpy.sum(x141 * x279 * x313)
    result[1, 4, 2] = numpy.sum(x143 * x313 * x314)
    result[1, 4, 3] = numpy.sum(x141 * x302 * x315)
    result[1, 4, 4] = numpy.sum(x14 * x183 * x273)
    result[1, 4, 5] = numpy.sum(x14 * x314 * x316)
    result[1, 4, 6] = numpy.sum(x285 * x309 * x312)
    result[1, 4, 7] = numpy.sum(x143 * x315 * x317)
    result[1, 4, 8] = numpy.sum(x273 * x316 * x317)
    result[1, 4, 9] = numpy.sum(x158 * x270 * x318)
    result[1, 5, 0] = numpy.sum(x234 * x296 * x319)
    result[1, 5, 1] = numpy.sum(x223 * x242 * x41)
    result[1, 5, 2] = numpy.sum(x191 * x290 * x41)
    result[1, 5, 3] = numpy.sum(x14 * x223 * x252)
    result[1, 5, 4] = numpy.sum(x14 * x191 * x289)
    result[1, 5, 5] = numpy.sum(x193 * x234 * x302)
    result[1, 5, 6] = numpy.sum(x263 * x309 * x319)
    result[1, 5, 7] = numpy.sum(x191 * x252 * x317)
    result[1, 5, 8] = numpy.sum(x193 * x242 * x317)
    result[1, 5, 9] = numpy.sum(x234 * x309 * x320)
    result[1, 6, 0] = numpy.sum(x322 * x325)
    result[1, 6, 1] = numpy.sum(x327 * x328)
    result[1, 6, 2] = numpy.sum(x322 * x328 * x83)
    result[1, 6, 3] = numpy.sum(x329 * x331)
    result[1, 6, 4] = numpy.sum(x258 * x327 * x330)
    result[1, 6, 5] = numpy.sum(x260 * x322 * x330)
    result[1, 6, 6] = numpy.sum(x332 * x333)
    result[1, 6, 7] = numpy.sum(x329 * x334 * x83)
    result[1, 6, 8] = numpy.sum(x260 * x327 * x9)
    result[1, 6, 9] = numpy.sum(x139 * x322 * x9)
    result[1, 7, 0] = numpy.sum(x141 * x324 * x335)
    result[1, 7, 1] = numpy.sum(x181 * x298 * x323)
    result[1, 7, 2] = numpy.sum(x143 * x323 * x336)
    result[1, 7, 3] = numpy.sum(x141 * x330 * x337)
    result[1, 7, 4] = numpy.sum(x143 * x298 * x338)
    result[1, 7, 5] = numpy.sum(x152 * x330 * x336)
    result[1, 7, 6] = numpy.sum(x141 * x308 * x334)
    result[1, 7, 7] = numpy.sum(x143 * x337 * x9)
    result[1, 7, 8] = numpy.sum(x152 * x298 * x339)
    result[1, 7, 9] = numpy.sum(x158 * x335 * x9)
    result[1, 8, 0] = numpy.sum(x270 * x319 * x324)
    result[1, 8, 1] = numpy.sum(x223 * x273 * x323)
    result[1, 8, 2] = numpy.sum(x191 * x314 * x323)
    result[1, 8, 3] = numpy.sum(x223 * x277 * x330)
    result[1, 8, 4] = numpy.sum(x191 * x273 * x338)
    result[1, 8, 5] = numpy.sum(x193 * x314 * x330)
    result[1, 8, 6] = numpy.sum(x285 * x319 * x9)
    result[1, 8, 7] = numpy.sum(x191 * x277 * x339)
    result[1, 8, 8] = numpy.sum(x193 * x279 * x9)
    result[1, 8, 9] = numpy.sum(x270 * x320 * x9)
    result[1, 9, 0] = numpy.sum(x234 * x324 * x340)
    result[1, 9, 1] = numpy.sum(x242 * x323 * x341)
    result[1, 9, 2] = numpy.sum(x229 * x234 * x328)
    result[1, 9, 3] = numpy.sum(x252 * x330 * x341)
    result[1, 9, 4] = numpy.sum(x229 * x242 * x342)
    result[1, 9, 5] = numpy.sum(x234 * x330 * x343)
    result[1, 9, 6] = numpy.sum(x263 * x340 * x9)
    result[1, 9, 7] = numpy.sum(x229 * x268 * x9)
    result[1, 9, 8] = numpy.sum(x242 * x343 * x9)
    result[1, 9, 9] = numpy.sum(x233 * x234 * x333)
    result[2, 0, 0] = numpy.sum(x240 * x344)
    result[2, 0, 1] = numpy.sum(x243 * x346 * x75)
    result[2, 0, 2] = numpy.sum(x244 * x348)
    result[2, 0, 3] = numpy.sum(x256 * x344 * x349)
    result[2, 0, 4] = numpy.sum(x256 * x348 * x350)
    result[2, 0, 5] = numpy.sum(x257 * x356)
    result[2, 0, 6] = numpy.sum(x153 * x266 * x344)
    result[2, 0, 7] = numpy.sum(x266 * x348 * x349)
    result[2, 0, 8] = numpy.sum(x266 * x357 * x75)
    result[2, 0, 9] = numpy.sum(x267 * x360)
    result[2, 1, 0] = numpy.sum(x109 * x239 * x346)
    result[2, 1, 1] = numpy.sum(x112 * x288 * x344)
    result[2, 1, 2] = numpy.sum(x109 * x288 * x348)
    result[2, 1, 3] = numpy.sum(x123 * x255 * x361)
    result[2, 1, 4] = numpy.sum(x112 * x255 * x362)
    result[2, 1, 5] = numpy.sum(x184 * x255 * x356)
    result[2, 1, 6] = numpy.sum(x134 * x286 * x344)
    result[2, 1, 7] = numpy.sum(x123 * x291 * x348)
    result[2, 1, 8] = numpy.sum(x112 * x291 * x356)
    result[2, 1, 9] = numpy.sum(x109 * x286 * x360)
    result[2, 2, 0] = numpy.sum(x271 * x364)
    result[2, 2, 1] = numpy.sum(x145 * x364 * x43)
    result[2, 2, 2] = numpy.sum(x274 * x366)
    result[2, 2, 3] = numpy.sum(x144 * x255 * x364)
    result[2, 2, 4] = numpy.sum(x255 * x367 * x368)
    result[2, 2, 5] = numpy.sum(x278 * x371)
    result[2, 2, 6] = numpy.sum(x253 * x364 * x372)
    result[2, 2, 7] = numpy.sum(x144 * x253 * x366)
    result[2, 2, 8] = numpy.sum(x192 * x253 * x371)
    result[2, 2, 9] = numpy.sum(x286 * x377)
    result[2, 3, 0] = numpy.sum(x160 * x296 * x378)
    result[2, 3, 1] = numpy.sum(x164 * x361 * x41)
    result[2, 3, 2] = numpy.sum(x220 * x348 * x41)
    result[2, 3, 3] = numpy.sum(x167 * x302 * x344)
    result[2, 3, 4] = numpy.sum(x14 * x164 * x362)
    result[2, 3, 5] = numpy.sum(x14 * x220 * x356)
    result[2, 3, 6] = numpy.sum(x173 * x309 * x378)
    result[2, 3, 7] = numpy.sum(x167 * x317 * x348)
    result[2, 3, 8] = numpy.sum(x164 * x317 * x356)
    result[2, 3, 9] = numpy.sum(x309 * x360 * x379)
    result[2, 4, 0] = numpy.sum(x109 * x296 * x380)
    result[2, 4, 1] = numpy.sum(x112 * x313 * x381)
    result[2, 4, 2] = numpy.sum(x109 * x313 * x367)
    result[2, 4, 3] = numpy.sum(x14 * x381 * x382)
    result[2, 4, 4] = numpy.sum(x112 * x14 * x383)
    result[2, 4, 5] = numpy.sum(x109 * x302 * x384)
    result[2, 4, 6] = numpy.sum(x134 * x318 * x364)
    result[2, 4, 7] = numpy.sum(x317 * x366 * x382)
    result[2, 4, 8] = numpy.sum(x112 * x317 * x384)
    result[2, 4, 9] = numpy.sum(x109 * x318 * x377)
    result[2, 5, 0] = numpy.sum(x297 * x388)
    result[2, 5, 1] = numpy.sum(x192 * x388 * x41)
    result[2, 5, 2] = numpy.sum(x299 * x389)
    result[2, 5, 3] = numpy.sum(x14 * x144 * x388)
    result[2, 5, 4] = numpy.sum(x302 * x368 * x389)
    result[2, 5, 5] = numpy.sum(x302 * x391)
    result[2, 5, 6] = numpy.sum(x309 * x372 * x388)
    result[2, 5, 7] = numpy.sum(x144 * x309 * x389)
    result[2, 5, 8] = numpy.sum(x145 * x309 * x391)
    result[2, 5, 9] = numpy.sum(x310 * x397)
    result[2, 6, 0] = numpy.sum(x324 * x344 * x398)
    result[2, 6, 1] = numpy.sum(x202 * x328 * x344)
    result[2, 6, 2] = numpy.sum(x323 * x348 * x399)
    result[2, 6, 3] = numpy.sum(x206 * x330 * x378)
    result[2, 6, 4] = numpy.sum(x202 * x342 * x348)
    result[2, 6, 5] = numpy.sum(x330 * x356 * x399)
    result[2, 6, 6] = numpy.sum(x213 * x333 * x344)
    result[2, 6, 7] = numpy.sum(x206 * x348 * x400)
    result[2, 6, 8] = numpy.sum(x202 * x357 * x9)
    result[2, 6, 9] = numpy.sum(x360 * x398 * x9)
    result[2, 7, 0] = numpy.sum(x324 * x364 * x379)
    result[2, 7, 1] = numpy.sum(x164 * x323 * x381)
    result[2, 7, 2] = numpy.sum(x220 * x323 * x366)
    result[2, 7, 3] = numpy.sum(x167 * x330 * x381)
    result[2, 7, 4] = numpy.sum(x164 * x338 * x366)
    result[2, 7, 5] = numpy.sum(x220 * x330 * x371)
    result[2, 7, 6] = numpy.sum(x173 * x364 * x400)
    result[2, 7, 7] = numpy.sum(x167 * x367 * x9)
    result[2, 7, 8] = numpy.sum(x164 * x339 * x371)
    result[2, 7, 9] = numpy.sum(x377 * x379 * x9)
    result[2, 8, 0] = numpy.sum(x109 * x324 * x401)
    result[2, 8, 1] = numpy.sum(x112 * x323 * x402)
    result[2, 8, 2] = numpy.sum(x184 * x323 * x389)
    result[2, 8, 3] = numpy.sum(x123 * x330 * x402)
    result[2, 8, 4] = numpy.sum(x112 * x338 * x389)
    result[2, 8, 5] = numpy.sum(x109 * x330 * x403)
    result[2, 8, 6] = numpy.sum(x134 * x401 * x9)
    result[2, 8, 7] = numpy.sum(x123 * x339 * x389)
    result[2, 8, 8] = numpy.sum(x112 * x403 * x9)
    result[2, 8, 9] = numpy.sum(x109 * x334 * x397)
    result[2, 9, 0] = numpy.sum(x325 * x405)
    result[2, 9, 1] = numpy.sum(x328 * x405 * x75)
    result[2, 9, 2] = numpy.sum(x328 * x407)
    result[2, 9, 3] = numpy.sum(x330 * x349 * x405)
    result[2, 9, 4] = numpy.sum(x330 * x350 * x407)
    result[2, 9, 5] = numpy.sum(x331 * x408)
    result[2, 9, 6] = numpy.sum(x153 * x405 * x9)
    result[2, 9, 7] = numpy.sum(x349 * x407 * x9)
    result[2, 9, 8] = numpy.sum(x334 * x408 * x75)
    result[2, 9, 9] = numpy.sum(x333 * x409)
    result[3, 0, 0] = numpy.sum(x410 * x413)
    result[3, 0, 1] = numpy.sum(x414 * x417)
    result[3, 0, 2] = numpy.sum(x205 * x412 * x414)
    result[3, 0, 3] = numpy.sum(x418 * x421)
    result[3, 0, 4] = numpy.sum(x418 * x422 * x83)
    result[3, 0, 5] = numpy.sum(x260 * x412 * x418)
    result[3, 0, 6] = numpy.sum(x423 * x424)
    result[3, 0, 7] = numpy.sum(x205 * x420 * x424)
    result[3, 0, 8] = numpy.sum(x100 * x416 * x425)
    result[3, 0, 9] = numpy.sum(x108 * x424 * x426)
    result[3, 1, 0] = numpy.sum(x238 * x429)
    result[3, 1, 1] = numpy.sum(x29 * x431)
    result[3, 1, 2] = numpy.sum(x170 * x29 * x428)
    result[3, 1, 3] = numpy.sum(x254 * x435)
    result[3, 1, 4] = numpy.sum(x170 * x254 * x436)
    result[3, 1, 5] = numpy.sum(x129 * x254 * x428)
    result[3, 1, 6] = numpy.sum(x437 * x441)
    result[3, 1, 7] = numpy.sum(x125 * x434 * x437)
    result[3, 1, 8] = numpy.sum(x129 * x430 * x437)
    result[3, 1, 9] = numpy.sum(x287 * x428 * x437)
    result[3, 2, 0] = numpy.sum(x141 * x238 * x442)
    result[3, 2, 1] = numpy.sum(x181 * x29 * x416)
    result[3, 2, 2] = numpy.sum(x143 * x29 * x443)
    result[3, 2, 3] = numpy.sum(x181 * x254 * x420)
    result[3, 2, 4] = numpy.sum(x254 * x416 * x444)
    result[3, 2, 5] = numpy.sum(x152 * x254 * x443)
    result[3, 2, 6] = numpy.sum(x141 * x437 * x445)
    result[3, 2, 7] = numpy.sum(x143 * x420 * x446)
    result[3, 2, 8] = numpy.sum(x152 * x416 * x446)
    result[3, 2, 9] = numpy.sum(x158 * x437 * x442)
    result[3, 3, 0] = numpy.sum(x447 * x449)
    result[3, 3, 1] = numpy.sum(x27 * x452)
    result[3, 3, 2] = numpy.sum(x170 * x27 * x448)
    result[3, 3, 3] = numpy.sum(x453 * x455)
    result[3, 3, 4] = numpy.sum(x125 * x451 * x456)
    result[3, 3, 5] = numpy.sum(x129 * x448 * x453)
    result[3, 3, 6] = numpy.sum(x3 * x459)
    result[3, 3, 7] = numpy.sum(x3 * x460 * x83)
    result[3, 3, 8] = numpy.sum(x129 * x3 * x451)
    result[3, 3, 9] = numpy.sum(x139 * x3 * x461)
    result[3, 4, 0] = numpy.sum(x141 * x447 * x462)
    result[3, 4, 1] = numpy.sum(x181 * x27 * x436)
    result[3, 4, 2] = numpy.sum(x27 * x428 * x444)
    result[3, 4, 3] = numpy.sum(x141 * x434 * x463)
    result[3, 4, 4] = numpy.sum(x183 * x430 * x453)
    result[3, 4, 5] = numpy.sum(x152 * x428 * x463)
    result[3, 4, 6] = numpy.sum(x178 * x3 * x440)
    result[3, 4, 7] = numpy.sum(x3 * x434 * x444)
    result[3, 4, 8] = numpy.sum(x316 * x430 * x464)
    result[3, 4, 9] = numpy.sum(x158 * x3 * x462)
    result[3, 5, 0] = numpy.sum(x319 * x412 * x447)
    result[3, 5, 1] = numpy.sum(x223 * x27 * x416)
    result[3, 5, 2] = numpy.sum(x191 * x27 * x443)
    result[3, 5, 3] = numpy.sum(x223 * x420 * x453)
    result[3, 5, 4] = numpy.sum(x191 * x416 * x463)
    result[3, 5, 5] = numpy.sum(x193 * x443 * x453)
    result[3, 5, 6] = numpy.sum(x189 * x3 * x445)
    result[3, 5, 7] = numpy.sum(x191 * x420 * x464)
    result[3, 5, 8] = numpy.sum(x193 * x416 * x464)
    result[3, 5, 9] = numpy.sum(x3 * x320 * x412)
    result[3, 6, 0] = numpy.sum(x465 * x468)
    result[3, 6, 1] = numpy.sum(x469 * x471)
    result[3, 6, 2] = numpy.sum(x205 * x465 * x469)
    result[3, 6, 3] = numpy.sum(x472 * x8)
    result[3, 6, 4] = numpy.sum(x470 * x8 * x98)
    result[3, 6, 5] = numpy.sum(x100 * x465 * x473)
    result[3, 6, 6] = numpy.sum(
        x72
        * (
            x0 * (x211 * x301 + x211 * x434 + x212 * x434 + 2.0 * x307 + 5.0 * x439)
            + x201 * x458
        )
    )
    result[3, 6, 7] = numpy.sum(x472 * x83)
    result[3, 6, 8] = numpy.sum(x100 * x471)
    result[3, 6, 9] = numpy.sum(x139 * x465)
    result[3, 7, 0] = numpy.sum(x141 * x461 * x468)
    result[3, 7, 1] = numpy.sum(x141 * x451 * x474)
    result[3, 7, 2] = numpy.sum(x143 * x448 * x474)
    result[3, 7, 3] = numpy.sum(x141 * x460 * x8)
    result[3, 7, 4] = numpy.sum(x444 * x451 * x8)
    result[3, 7, 5] = numpy.sum(x152 * x448 * x475)
    result[3, 7, 6] = numpy.sum(x141 * x459)
    result[3, 7, 7] = numpy.sum(x143 * x455)
    result[3, 7, 8] = numpy.sum(x152 * x452)
    result[3, 7, 9] = numpy.sum(x158 * x449)
    result[3, 8, 0] = numpy.sum(x189 * x428 * x477)
    result[3, 8, 1] = numpy.sum(x223 * x430 * x469)
    result[3, 8, 2] = numpy.sum(x191 * x428 * x474)
    result[3, 8, 3] = numpy.sum(x223 * x434 * x8)
    result[3, 8, 4] = numpy.sum(x191 * x436 * x475)
    result[3, 8, 5] = numpy.sum(x193 * x428 * x475)
    result[3, 8, 6] = numpy.sum(x189 * x441)
    result[3, 8, 7] = numpy.sum(x191 * x435)
    result[3, 8, 8] = numpy.sum(x193 * x431)
    result[3, 8, 9] = numpy.sum(x195 * x429)
    result[3, 9, 0] = numpy.sum(x224 * x426 * x478)
    result[3, 9, 1] = numpy.sum(x341 * x416 * x469)
    result[3, 9, 2] = numpy.sum(x229 * x442 * x469)
    result[3, 9, 3] = numpy.sum(x224 * x420 * x473)
    result[3, 9, 4] = numpy.sum(x229 * x422 * x8)
    result[3, 9, 5] = numpy.sum(x230 * x412 * x473)
    result[3, 9, 6] = numpy.sum(x224 * x423)
    result[3, 9, 7] = numpy.sum(x229 * x421)
    result[3, 9, 8] = numpy.sum(x230 * x417)
    result[3, 9, 9] = numpy.sum(x233 * x413)
    result[4, 0, 0] = numpy.sum(x234 * x345 * x410)
    result[4, 0, 1] = numpy.sum(x242 * x378 * x414)
    result[4, 0, 2] = numpy.sum(x348 * x414 * x479)
    result[4, 0, 3] = numpy.sum(x268 * x344 * x418)
    result[4, 0, 4] = numpy.sum(x242 * x311 * x348 * x418)
    result[4, 0, 5] = numpy.sum(x234 * x357 * x418)
    result[4, 0, 6] = numpy.sum(x263 * x344 * x480)
    result[4, 0, 7] = numpy.sum(x252 * x348 * x425)
    result[4, 0, 8] = numpy.sum(x242 * x356 * x425)
    result[4, 0, 9] = numpy.sum(x234 * x360 * x480)
    result[4, 1, 0] = numpy.sum(x238 * x270 * x378)
    result[4, 1, 1] = numpy.sum(x279 * x29 * x344)
    result[4, 1, 2] = numpy.sum(x29 * x314 * x348)
    result[4, 1, 3] = numpy.sum(x254 * x277 * x361)
    result[4, 1, 4] = numpy.sum(x254 * x273 * x362)
    result[4, 1, 5] = numpy.sum(x254 * x314 * x356)
    result[4, 1, 6] = numpy.sum(x285 * x378 * x437)
    result[4, 1, 7] = numpy.sum(x277 * x348 * x446)
    result[4, 1, 8] = numpy.sum(x273 * x356 * x446)
    result[4, 1, 9] = numpy.sum(x270 * x360 * x481)
    result[4, 2, 0] = numpy.sum(x238 * x364 * x479)
    result[4, 2, 1] = numpy.sum(x242 * x29 * x381)
    result[4, 2, 2] = numpy.sum(x234 * x29 * x367)
    result[4, 2, 3] = numpy.sum(x252 * x254 * x381)
    result[4, 2, 4] = numpy.sum(x254 * x289 * x366)
    result[4, 2, 5] = numpy.sum(x254 * x290 * x371)
    result[4, 2, 6] = numpy.sum(x263 * x364 * x481)
    result[4, 2, 7] = numpy.sum(x252 * x366 * x446)
    result[4, 2, 8] = numpy.sum(x242 * x371 * x446)
    result[4, 2, 9] = numpy.sum(x377 * x437 * x479)
    result[4, 3, 0] = numpy.sum(x335 * x344 * x447)
    result[4, 3, 1] = numpy.sum(x27 * x298 * x361)
    result[4, 3, 2] = numpy.sum(x27 * x336 * x348)
    result[4, 3, 3] = numpy.sum(x337 * x344 * x453)
    result[4, 3, 4] = numpy.sum(x298 * x362 * x453)
    result[4, 3, 5] = numpy.sum(x336 * x356 * x453)
    result[4, 3, 6] = numpy.sum(x3 * x308 * x346)
    result[4, 3, 7] = numpy.sum(x3 * x337 * x348)
    result[4, 3, 8] = numpy.sum(x298 * x356 * x464)
    result[4, 3, 9] = numpy.sum(x3 * x335 * x360)
    result[4, 4, 0] = numpy.sum(x270 * x380 * x447)
    result[4, 4, 1] = numpy.sum(x273 * x381 * x482)
    result[4, 4, 2] = numpy.sum(x314 * x366 * x482)
    result[4, 4, 3] = numpy.sum(x315 * x381 * x453)
    result[4, 4, 4] = numpy.sum(x273 * x383 * x453)
    result[4, 4, 5] = numpy.sum(x314 * x384 * x453)
    result[4, 4, 6] = numpy.sum(x285 * x3 * x380)
    result[4, 4, 7] = numpy.sum(x3 * x315 * x367)
    result[4, 4, 8] = numpy.sum(x279 * x3 * x384)
    result[4, 4, 9] = numpy.sum(x270 * x3 * x311 * x377)
    result[4, 5, 0] = numpy.sum(x234 * x401 * x447)
    result[4, 5, 1] = numpy.sum(x242 * x27 * x402)
    result[4, 5, 2] = numpy.sum(x27 * x290 * x389)
    result[4, 5, 3] = numpy.sum(x252 * x402 * x453)
    result[4, 5, 4] = numpy.sum(x289 * x389 * x453)
    result[4, 5, 5] = numpy.sum(x234 * x403 * x453)
    result[4, 5, 6] = numpy.sum(x263 * x3 * x401)
    result[4, 5, 7] = numpy.sum(x252 * x389 * x464)
    result[4, 5, 8] = numpy.sum(x242 * x3 * x403)
    result[4, 5, 9] = numpy.sum(x246 * x3 * x397)
    result[4, 6, 0] = numpy.sum(x322 * x344 * x468)
    result[4, 6, 1] = numpy.sum(x327 * x378 * x469)
    result[4, 6, 2] = numpy.sum(x322 * x348 * x483)
    result[4, 6, 3] = numpy.sum(x329 * x346 * x8)
    result[4, 6, 4] = numpy.sum(x327 * x348 * x484)
    result[4, 6, 5] = numpy.sum(x322 * x357 * x8)
    result[4, 6, 6] = numpy.sum(x332 * x345)
    result[4, 6, 7] = numpy.sum(x329 * x348 * x94)
    result[4, 6, 8] = numpy.sum(x327 * x357)
    result[4, 6, 9] = numpy.sum(x135 * x322 * x360)
    result[4, 7, 0] = numpy.sum(x209 * x295 * x466 * x485)
    result[4, 7, 1] = numpy.sum(x298 * x381 * x469)
    result[4, 7, 2] = numpy.sum(x336 * x366 * x469)
    result[4, 7, 3] = numpy.sum(x337 * x485)
    result[4, 7, 4] = numpy.sum(x298 * x367 * x486)
    result[4, 7, 5] = numpy.sum(x295 * x371 * x475)
    result[4, 7, 6] = numpy.sum(x308 * x364 * x94)
    result[4, 7, 7] = numpy.sum(x301 * x367)
    result[4, 7, 8] = numpy.sum(x168 * x298 * x371)
    result[4, 7, 9] = numpy.sum(x335 * x377)
    result[4, 8, 0] = numpy.sum(x270 * x388 * x477)
    result[4, 8, 1] = numpy.sum(x273 * x402 * x469)
    result[4, 8, 2] = numpy.sum(x314 * x389 * x469)
    result[4, 8, 3] = numpy.sum(x277 * x388 * x475)
    result[4, 8, 4] = numpy.sum(x279 * x389 * x486)
    result[4, 8, 5] = numpy.sum(x314 * x391 * x8)
    result[4, 8, 6] = numpy.sum(x285 * x401)
    result[4, 8, 7] = numpy.sum(x168 * x277 * x389)
    result[4, 8, 8] = numpy.sum(x279 * x391)
    result[4, 8, 9] = numpy.sum(x270 * x397 * x94)
    result[4, 9, 0] = numpy.sum(x234 * x405 * x468)
    result[4, 9, 1] = numpy.sum(x242 * x405 * x483)
    result[4, 9, 2] = numpy.sum(x407 * x469 * x479)
    result[4, 9, 3] = numpy.sum(x268 * x405 * x8)
    result[4, 9, 4] = numpy.sum(x242 * x407 * x484)
    result[4, 9, 5] = numpy.sum(x246 * x408 * x8)
    result[4, 9, 6] = numpy.sum(x135 * x263 * x405)
    result[4, 9, 7] = numpy.sum(x268 * x407)
    result[4, 9, 8] = numpy.sum(x242 * x408 * x94)
    result[4, 9, 9] = numpy.sum(x245 * x409)
    result[5, 0, 0] = numpy.sum(x410 * x489)
    result[5, 0, 1] = numpy.sum(x226 * x414 * x488)
    result[5, 0, 2] = numpy.sum(x414 * x492)
    result[5, 0, 3] = numpy.sum(x349 * x418 * x488)
    result[5, 0, 4] = numpy.sum(x418 * x493 * x75)
    result[5, 0, 5] = numpy.sum(x418 * x496)
    result[5, 0, 6] = numpy.sum(x105 * x424 * x497)
    result[5, 0, 7] = numpy.sum(x425 * x491 * x86)
    result[5, 0, 8] = numpy.sum(x226 * x424 * x495)
    result[5, 0, 9] = numpy.sum(x424 * x498)
    result[5, 1, 0] = numpy.sum(x109 * x238 * x499)
    result[5, 1, 1] = numpy.sum(x112 * x29 * x500)
    result[5, 1, 2] = numpy.sum(x184 * x29 * x491)
    result[5, 1, 3] = numpy.sum(x123 * x254 * x500)
    result[5, 1, 4] = numpy.sum(x254 * x491 * x501)
    result[5, 1, 5] = numpy.sum(x184 * x254 * x495)
    result[5, 1, 6] = numpy.sum(x134 * x481 * x488)
    result[5, 1, 7] = numpy.sum(x123 * x446 * x491)
    result[5, 1, 8] = numpy.sum(x112 * x446 * x495)
    result[5, 1, 9] = numpy.sum(x109 * x437 * x502)
    result[5, 2, 0] = numpy.sum(x238 * x505)
    result[5, 2, 1] = numpy.sum(x192 * x29 * x504)
    result[5, 2, 2] = numpy.sum(x29 * x507)
    result[5, 2, 3] = numpy.sum(x144 * x254 * x504)
    result[5, 2, 4] = numpy.sum(x192 * x254 * x508)
    result[5, 2, 5] = numpy.sum(x254 * x512)
    result[5, 2, 6] = numpy.sum(x372 * x437 * x504)
    result[5, 2, 7] = numpy.sum(x144 * x437 * x506)
    result[5, 2, 8] = numpy.sum(x145 * x437 * x511)
    result[5, 2, 9] = numpy.sum(x437 * x516)
    result[5, 3, 0] = numpy.sum(x379 * x447 * x488)
    result[5, 3, 1] = numpy.sum(x164 * x27 * x500)
    result[5, 3, 2] = numpy.sum(x220 * x27 * x491)
    result[5, 3, 3] = numpy.sum(x167 * x453 * x500)
    result[5, 3, 4] = numpy.sum(x164 * x463 * x491)
    result[5, 3, 5] = numpy.sum(x220 * x453 * x495)
    result[5, 3, 6] = numpy.sum(x173 * x3 * x499)
    result[5, 3, 7] = numpy.sum(x167 * x464 * x491)
    result[5, 3, 8] = numpy.sum(x164 * x464 * x495)
    result[5, 3, 9] = numpy.sum(x160 * x3 * x502)
    result[5, 4, 0] = numpy.sum(x109 * x447 * x517)
    result[5, 4, 1] = numpy.sum(x27 * x501 * x504)
    result[5, 4, 2] = numpy.sum(x184 * x482 * x506)
    result[5, 4, 3] = numpy.sum(x123 * x463 * x504)
    result[5, 4, 4] = numpy.sum(x112 * x182 * x453 * x506)
    result[5, 4, 5] = numpy.sum(x109 * x463 * x511)
    result[5, 4, 6] = numpy.sum(x134 * x3 * x517)
    result[5, 4, 7] = numpy.sum(x382 * x464 * x506)
    result[5, 4, 8] = numpy.sum(x3 * x501 * x511)
    result[5, 4, 9] = numpy.sum(x109 * x3 * x515 * x97)
    result[5, 5, 0] = numpy.sum(x447 * x519)
    result[5, 5, 1] = numpy.sum(x192 * x27 * x518)
    result[5, 5, 2] = numpy.sum(x27 * x522)
    result[5, 5, 3] = numpy.sum(x144 * x453 * x518)
    result[5, 5, 4] = numpy.sum(x145 * x456 * x521)
    result[5, 5, 5] = numpy.sum(x453 * x524)
    result[5, 5, 6] = numpy.sum(x153 * x3 * x525)
    result[5, 5, 7] = numpy.sum(x144 * x3 * x521)
    result[5, 5, 8] = numpy.sum(x3 * x526 * x75)
    result[5, 5, 9] = numpy.sum(x3 * x529)
    result[5, 6, 0] = numpy.sum(x196 * x478 * x497)
    result[5, 6, 1] = numpy.sum(x202 * x483 * x488)
    result[5, 6, 2] = numpy.sum(x399 * x469 * x491)
    result[5, 6, 3] = numpy.sum(x206 * x473 * x488)
    result[5, 6, 4] = numpy.sum(x202 * x493 * x8)
    result[5, 6, 5] = numpy.sum(x196 * x473 * x495)
    result[5, 6, 6] = numpy.sum(x213 * x489)
    result[5, 6, 7] = numpy.sum(x206 * x492)
    result[5, 6, 8] = numpy.sum(x202 * x496)
    result[5, 6, 9] = numpy.sum(x196 * x498)
    result[5, 7, 0] = numpy.sum(x160 * x477 * x504)
    result[5, 7, 1] = numpy.sum(x164 * x474 * x504)
    result[5, 7, 2] = numpy.sum(x220 * x469 * x506)
    result[5, 7, 3] = numpy.sum(x167 * x475 * x504)
    result[5, 7, 4] = numpy.sum(x164 * x475 * x508)
    result[5, 7, 5] = numpy.sum(x220 * x511 * x8)
    result[5, 7, 6] = numpy.sum(x173 * x505)
    result[5, 7, 7] = numpy.sum(x167 * x507)
    result[5, 7, 8] = numpy.sum(x164 * x512)
    result[5, 7, 9] = numpy.sum(x160 * x516)
    result[5, 8, 0] = numpy.sum(x109 * x468 * x525)
    result[5, 8, 1] = numpy.sum(x112 * x474 * x518)
    result[5, 8, 2] = numpy.sum(x109 * x474 * x521)
    result[5, 8, 3] = numpy.sum(x123 * x475 * x518)
    result[5, 8, 4] = numpy.sum(x501 * x521 * x8)
    result[5, 8, 5] = numpy.sum(x109 * x526 * x8)
    result[5, 8, 6] = numpy.sum(x134 * x519)
    result[5, 8, 7] = numpy.sum(x123 * x522)
    result[5, 8, 8] = numpy.sum(x112 * x524)
    result[5, 8, 9] = numpy.sum(x109 * x529)
    result[5, 9, 0] = numpy.sum(x468 * x530)
    result[5, 9, 1] = numpy.sum(x226 * x469 * x530)
    result[5, 9, 2] = numpy.sum(x469 * x532)
    result[5, 9, 3] = numpy.sum(x473 * x530 * x86)
    result[5, 9, 4] = numpy.sum(x531 * x75 * x8 * x97)
    result[5, 9, 5] = numpy.sum(x533 * x8)
    result[5, 9, 6] = numpy.sum(x153 * x530)
    result[5, 9, 7] = numpy.sum(x532 * x86)
    result[5, 9, 8] = numpy.sum(x533 * x75)
    result[5, 9, 9] = numpy.sum(
        x72
        * (
            x0 * (x231 * x391 + x231 * x511 + x232 * x511 + 2.0 * x396 + 5.0 * x514)
            + x228 * x528
        )
    )
    return result


def quadrupole3d_34(ax, da, A, bx, db, B, R):
    """Cartesian 3D (fg) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 10, 15), dtype=float)

    x0 = 0.5 / (ax + bx)
    x1 = (ax + bx) ** (-1.0)
    x2 = -x1 * (ax * A[0] + bx * B[0])
    x3 = -x2 - B[0]
    x4 = ax * bx * x1
    x5 = numpy.exp(-x4 * (A[0] - B[0]) ** 2)
    x6 = 1.772453850905516 * numpy.sqrt(x1)
    x7 = x5 * x6
    x8 = x0 * x7
    x9 = 3.0 * x8
    x10 = -x2 - A[0]
    x11 = x10 * x7
    x12 = x11 * x3
    x13 = -x2 - R[0]
    x14 = x11 * x13
    x15 = x3 * x7
    x16 = x13 * x15
    x17 = x0 * (x12 + x14 + x16 + x9)
    x18 = x13 * x7
    x19 = x0 * (x15 + x18)
    x20 = x16 + x8
    x21 = x10 * x20
    x22 = x19 + x21
    x23 = x22 * x3
    x24 = x17 + x23
    x25 = x24 * x3
    x26 = 2.0 * x25
    x27 = x10 * x24
    x28 = x20 * x3
    x29 = x0 * (x11 + x15)
    x30 = x12 + x8
    x31 = x3 * x30
    x32 = x29 + x31
    x33 = 3.0 * x19
    x34 = 2.0 * x21
    x35 = x33 + x34
    x36 = x0 * (x28 + x32 + x35)
    x37 = x10 * x30
    x38 = 2.0 * x0 * (2.0 * x29 + x31 + x37)
    x39 = 2.0 * x12
    x40 = x3**2 * x7
    x41 = x40 + x9
    x42 = x0 * (x39 + x41)
    x43 = x10 * x32
    x44 = x42 + x43
    x45 = x3 * x44
    x46 = x38 + x45
    x47 = x0 * (x26 + 3.0 * x27 + 5.0 * x36 + x46)
    x48 = x10 * x22
    x49 = 2.0 * x48
    x50 = 4.0 * x17
    x51 = 2.0 * x23 + x50
    x52 = x0 * (x44 + x49 + x51)
    x53 = x27 + x36
    x54 = x3 * x53
    x55 = x52 + x54
    x56 = x10 * x55
    x57 = x13 * x22
    x58 = 2.0 * x57
    x59 = 2.0 * x16
    x60 = x13**2 * x7
    x61 = x60 + x9
    x62 = x0 * (x59 + x61)
    x63 = x13 * x20
    x64 = x19 + x63
    x65 = x3 * x64
    x66 = x62 + x65
    x67 = x0 * (x51 + x58 + x66)
    x68 = x0 * (x11 + x18)
    x69 = x14 + x8
    x70 = x13 * x69
    x71 = x68 + x70
    x72 = x0 * (x35 + x63 + x71)
    x73 = x17 + x57
    x74 = x3 * x73
    x75 = x72 + x74
    x76 = x10 * x75
    x77 = x67 + x76
    x78 = x10 * x77
    x79 = x3 * x77
    x80 = 2.0 * x36
    x81 = x10 * x73
    x82 = x0 * (2.0 * x27 + 4.0 * x72 + 2.0 * x74 + x80 + 2.0 * x81)
    x83 = x3 * x75
    x84 = x0 * (2.0 * x52 + 2.0 * x54 + 5.0 * x67 + 3.0 * x76 + 2.0 * x83)
    x85 = x79 + x82
    x86 = x10 * x85 + x84
    x87 = 2.645751311064591
    x88 = da * db
    x89 = 0.009523809523809524 * x88
    x90 = x87 * x89
    x91 = numpy.exp(-x4 * (A[1] - B[1]) ** 2)
    x92 = numpy.exp(-x4 * (A[2] - B[2]) ** 2)
    x93 = 3.141592653589793 * x1 * x92
    x94 = x91 * x93
    x95 = x90 * x94
    x96 = -x1 * (ax * A[1] + bx * B[1])
    x97 = -x96 - B[1]
    x98 = 0.06666666666666667 * x88
    x99 = x97 * x98
    x100 = x86 * x94
    x101 = -x1 * (ax * A[2] + bx * B[2])
    x102 = -x101 - B[2]
    x103 = x102 * x98
    x104 = x78 + x82
    x105 = x6 * x92
    x106 = 3.872983346207417
    x107 = 0.02222222222222222 * x106
    x108 = x6 * x91
    x109 = x108 * x97**2
    x110 = x0 * x108
    x111 = x109 + x110
    x112 = x111 * x88
    x113 = x107 * x112
    x114 = 2.23606797749979
    x115 = x114 * x94
    x116 = x115 * x99
    x117 = x102**2 * x105
    x118 = x0 * x105
    x119 = x117 + x118
    x120 = x119 * x88
    x121 = x107 * x120
    x122 = 2.0 * x14
    x123 = x0 * (x122 + x61) + x10 * x71
    x124 = x72 + x81
    x125 = x0 * (x123 + x49 + x50 + x58) + x10 * x124
    x126 = 2.0 * x110
    x127 = x97 * (x111 + x126)
    x128 = x127 * x98
    x129 = x102 * x105
    x130 = x129 * x98
    x131 = x114 * x125
    x132 = x108 * x97
    x133 = x132 * x98
    x134 = 2.0 * x118
    x135 = x102 * (x119 + x134)
    x136 = x135 * x98
    x137 = 3.0 * x110
    x138 = x0 * (3.0 * x109 + x137) + x127 * x97
    x139 = x138 * x89
    x140 = x10 * x69
    x141 = 2.0 * x0 * (x140 + 2.0 * x68 + x70) + x10 * x123
    x142 = x141 * x87
    x143 = 3.0 * x118
    x144 = x89 * (x0 * (3.0 * x117 + x143) + x102 * x135)
    x145 = -x96 - A[1]
    x146 = 5.916079783099616
    x147 = x146 * x89
    x148 = x145 * x147
    x149 = x94 * (x3 * x85 + x84)
    x150 = x108 * x145
    x151 = x150 * x97
    x152 = x110 + x151
    x153 = x114 * x152
    x154 = x105 * x98
    x155 = x0 * (x132 + x150)
    x156 = x152 * x97
    x157 = x155 + x156
    x158 = 1.732050807568877
    x159 = x157 * x158
    x160 = 0.1111111111111111 * x88
    x161 = x159 * x160
    x162 = 0.3333333333333333 * x88
    x163 = x152 * x162
    x164 = x150 * x158
    x165 = 0.1111111111111111 * x120
    x166 = x137 + 2.0 * x151
    x167 = x0 * (x109 + x166)
    x168 = x157 * x97
    x169 = x167 + x168
    x170 = x169 * x98
    x171 = x114 * x124
    x172 = x124 * x162
    x173 = x114 * x136
    x174 = x0 * (x127 + 3.0 * x155 + 3.0 * x156) + x169 * x97
    x175 = x105 * x147
    x176 = x114 * x123
    x177 = 0.1111111111111111 * x123
    x178 = x144 * x146
    x179 = -x101 - A[2]
    x180 = x147 * x179
    x181 = x105 * x179
    x182 = x102 * x181
    x183 = x118 + x182
    x184 = x183 * x98
    x185 = x114 * x184
    x186 = 0.1111111111111111 * x112
    x187 = x158 * x186
    x188 = x162 * x183
    x189 = x0 * (x129 + x181)
    x190 = x102 * x183
    x191 = x189 + x190
    x192 = x158 * x191
    x193 = x192 * x88
    x194 = 0.1111111111111111 * x193
    x195 = x114 * x128
    x196 = x143 + 2.0 * x182
    x197 = x0 * (x117 + x196)
    x198 = x102 * x191
    x199 = x197 + x198
    x200 = x199 * x98
    x201 = x139 * x146
    x202 = x89 * (x0 * (x135 + 3.0 * x189 + 3.0 * x190) + x102 * x199)
    x203 = x146 * x202
    x204 = 2.0 * x0 * (2.0 * x19 + x28 + x63) + x3 * x66
    x205 = x67 + x83
    x206 = x0 * (x204 + x26 + 3.0 * x72 + 3.0 * x74 + x80) + x205 * x3
    x207 = x108 * x145**2
    x208 = x110 + x207
    x209 = x147 * x208
    x210 = x145 * x152
    x211 = x155 + x210
    x212 = x114 * x154
    x213 = x208 * x98
    x214 = x114 * x205
    x215 = x158 * x75
    x216 = x145 * x157
    x217 = x167 + x216
    x218 = 0.1111111111111111 * x217
    x219 = x105 * x88
    x220 = x162 * x211
    x221 = 0.1111111111111111 * x208
    x222 = 2.0 * x0 * (2.0 * x155 + x156 + x210)
    x223 = x217 * x97
    x224 = x222 + x223
    x225 = x162 * x73
    x226 = x0 * (5.0 * x167 + 2.0 * x168 + 3.0 * x216)
    x227 = x224 * x97 + x226
    x228 = x146 * x71
    x229 = x228 * x89
    x230 = x114 * x71
    x231 = x158 * x71
    x232 = 10.2469507659596
    x233 = x179 * x232 * x89
    x234 = x152 * x98
    x235 = x106 * x205
    x236 = x157 * x162
    x237 = x162 * x191
    x238 = x106 * x73
    x239 = x232 * x71
    x240 = x181 * x89
    x241 = x106 * x71
    x242 = x105 * x179**2
    x243 = x118 + x242
    x244 = x243 * x89
    x245 = x146 * x244
    x246 = x243 * x98
    x247 = x179 * x183
    x248 = x189 + x247
    x249 = x108 * x98
    x250 = x114 * x249
    x251 = x162 * x248
    x252 = x179 * x191
    x253 = x197 + x252
    x254 = x108 * x160
    x255 = 2.0 * x0 * (2.0 * x189 + x190 + x247)
    x256 = x102 * x253
    x257 = x255 + x256
    x258 = x0 * (5.0 * x197 + 2.0 * x198 + 3.0 * x252)
    x259 = x102 * x257 + x258
    x260 = x145 * (x126 + x208)
    x261 = x0 * (x41 + x59)
    x262 = x19 + x28
    x263 = x262 * x3
    x264 = x0 * (2.0 * x261 + 2.0 * x263 + 3.0 * x62 + 3.0 * x65) + x204 * x3
    x265 = x264 * x90
    x266 = x0 * (x166 + x207) + x145 * x211
    x267 = x266 * x98
    x268 = x260 * x98
    x269 = x145 * x217
    x270 = x222 + x269
    x271 = x107 * x66
    x272 = x114 * x66
    x273 = x145 * x224 + x226
    x274 = x64 * x98
    x275 = x114 * x64
    x276 = 3.0 * x0 * (2.0 * x222 + x223 + x269) + x273 * x97
    x277 = x60 + x8
    x278 = x277 * x87
    x279 = x278 * x89
    x280 = x277 * x98
    x281 = x181 * x98
    x282 = x114 * x204
    x283 = x158 * x218
    x284 = x283 * x88
    x285 = x146 * x277
    x286 = x114 * x277
    x287 = x150 * x98
    x288 = x160 * x253
    x289 = x162 * x253
    x290 = x286 * x98
    x291 = x150 * x89
    x292 = x179 * (x134 + x243)
    x293 = x292 * x98
    x294 = x0 * (x196 + x242) + x179 * x248
    x295 = x294 * x98
    x296 = x179 * x253
    x297 = x255 + x296
    x298 = x297 * x88
    x299 = x179 * x257 + x258
    x300 = 3.0 * x0 * (2.0 * x255 + x256 + x296) + x102 * x299
    x301 = -x96 - R[1]
    x302 = x10 * x53
    x303 = x3 * x32
    x304 = x0 * (2.0 * x303 + 5.0 * x42 + 3.0 * x43)
    x305 = x10 * x46 + x304
    x306 = x47 + x56
    x307 = x95 * (x0 * (3.0 * x302 + x305 + 6.0 * x52 + 3.0 * x54) + x3 * x306)
    x308 = x132 * x301
    x309 = x110 + x308
    x310 = x306 * x94
    x311 = x302 + x52
    x312 = x108 * x301
    x313 = x0 * (x132 + x312)
    x314 = x309 * x97
    x315 = x313 + x314
    x316 = x107 * x315
    x317 = x114 * x309
    x318 = x317 * x98
    x319 = x137 + 2.0 * x308
    x320 = x0 * (x109 + x319)
    x321 = x315 * x97
    x322 = x320 + x321
    x323 = x140 + x68
    x324 = x29 + x37
    x325 = x17 + x48
    x326 = x0 * (2.0 * x19 + x323 + x324 + x34) + x10 * x325
    x327 = x326 * x98
    x328 = x114 * x129
    x329 = x10**2 * x7
    x330 = x329 + x9
    x331 = x0 * (x122 + x330) + x10 * x323
    x332 = 3.0 * x313
    x333 = x0 * (x127 + 3.0 * x314 + x332) + x322 * x97
    x334 = x105 * x90
    x335 = x144 * x87
    x336 = x150 * x301
    x337 = x110 + x336
    x338 = x3 * x55 + x47
    x339 = x145 * x309
    x340 = x313 + x339
    x341 = x337 * x98
    x342 = x0 * (x137 + x151 + x308 + x336)
    x343 = x340 * x97
    x344 = x342 + x343
    x345 = x158 * x344
    x346 = x105 * x160
    x347 = x129 * x162
    x348 = x158 * x337
    x349 = 0.1111111111111111 * x53
    x350 = 2.0 * x339
    x351 = x332 + x350
    x352 = x0 * (x157 + x314 + x351)
    x353 = x344 * x97
    x354 = x352 + x353
    x355 = x162 * x325
    x356 = x119 * x162
    x357 = x0 * (x169 + x322 + 3.0 * x342 + 3.0 * x343) + x354 * x97
    x358 = x323 * x98
    x359 = 0.1111111111111111 * x323
    x360 = x338 * x94
    x361 = x160 * x315
    x362 = x158 * x181
    x363 = x114 * x322
    x364 = x114 * x312
    x365 = x147 * x181
    x366 = x0 * (x150 + x312)
    x367 = x145 * x337
    x368 = x366 + x367
    x369 = x261 + x263
    x370 = x303 + x42
    x371 = x25 + x36
    x372 = x0 * (3.0 * x17 + 3.0 * x23 + x369 + x370) + x3 * x371
    x373 = x145 * x340
    x374 = x342 + x373
    x375 = x368 * x98
    x376 = x145 * x344
    x377 = x352 + x376
    x378 = x158 * x24
    x379 = 0.1111111111111111 * x368
    x380 = 2.0 * x373
    x381 = 4.0 * x342
    x382 = 2.0 * x343 + x381
    x383 = x0 * (x217 + x380 + x382)
    x384 = x377 * x97
    x385 = x383 + x384
    x386 = x162 * x22
    x387 = 2.0 * x353
    x388 = x0 * (x224 + 5.0 * x352 + 3.0 * x376 + x387)
    x389 = x385 * x97 + x388
    x390 = x69 * x98
    x391 = x158 * x69
    x392 = 0.1111111111111111 * x377
    x393 = x232 * x337
    x394 = x106 * x281
    x395 = x106 * x337
    x396 = x162 * x24
    x397 = x158 * x237
    x398 = x232 * x69
    x399 = x106 * x69
    x400 = x364 * x98
    x401 = x147 * x259
    x402 = x137 + 2.0 * x336
    x403 = x0 * (x207 + x402) + x145 * x368
    x404 = x40 + x8
    x405 = 2.0 * x8
    x406 = x3 * (x404 + x405)
    x407 = x0 * (3.0 * x28 + x33 + x406) + x3 * x369
    x408 = x0 * (x211 + 2.0 * x313 + x350 + x368) + x145 * x374
    x409 = x408 * x98
    x410 = x145 * x377
    x411 = x383 + x410
    x412 = x262 * x88
    x413 = x107 * x412
    x414 = x145 * x385
    x415 = x388 + x414
    x416 = x114 * x20
    x417 = x416 * x98
    x418 = x5 * x93
    x419 = x418 * x90
    x420 = x419 * (x0 * (x273 + 6.0 * x383 + 3.0 * x384 + 3.0 * x410) + x415 * x97)
    x421 = x415 * x418
    x422 = x114 * x369
    x423 = x160 * x262
    x424 = x389 * x418
    x425 = x114 * x18
    x426 = x425 * x98
    x427 = x292 * x90
    x428 = x299 * x98
    x429 = 3.141592653589793 * x1 * x5 * x91
    x430 = x429 * x90
    x431 = -x101 - R[2]
    x432 = x129 * x431
    x433 = x118 + x432
    x434 = x105 * x431
    x435 = x114 * x433
    x436 = x0 * (x129 + x434)
    x437 = x102 * x433
    x438 = x436 + x437
    x439 = x438 * x88
    x440 = x107 * x439
    x441 = x114 * x132
    x442 = x143 + 2.0 * x432
    x443 = x0 * (x117 + x442)
    x444 = x102 * x438
    x445 = x443 + x444
    x446 = x139 * x87
    x447 = 3.0 * x436
    x448 = x0 * (x135 + 3.0 * x437 + x447) + x102 * x445
    x449 = x108 * x90
    x450 = x434 * x98
    x451 = x162 * x433
    x452 = x160 * x438
    x453 = x114 * x434
    x454 = x114 * x445
    x455 = x147 * x323
    x456 = x181 * x431
    x457 = x118 + x456
    x458 = x108 * x147
    x459 = x457 * x98
    x460 = x179 * x433
    x461 = x436 + x460
    x462 = x162 * x461
    x463 = x0 * (x143 + x182 + x432 + x456)
    x464 = x102 * x461
    x465 = x463 + x464
    x466 = x158 * x465
    x467 = 2.0 * x460
    x468 = x447 + x467
    x469 = x0 * (x191 + x437 + x468)
    x470 = x102 * x465
    x471 = x469 + x470
    x472 = x0 * (x199 + x445 + 3.0 * x463 + 3.0 * x464) + x102 * x471
    x473 = x114 * x450
    x474 = x218 * x88
    x475 = x147 * x227
    x476 = x457 * x89
    x477 = x232 * x476
    x478 = x106 * x457
    x479 = x106 * x287
    x480 = x0 * (x181 + x434)
    x481 = x179 * x457
    x482 = x480 + x481
    x483 = x482 * x98
    x484 = x179 * x461
    x485 = x463 + x484
    x486 = x162 * x485
    x487 = x179 * x465
    x488 = x469 + x487
    x489 = 2.0 * x484
    x490 = 4.0 * x463
    x491 = 2.0 * x464 + x490
    x492 = x0 * (x253 + x489 + x491)
    x493 = x102 * x488
    x494 = x492 + x493
    x495 = 2.0 * x470
    x496 = x0 * (x257 + 5.0 * x469 + 3.0 * x487 + x495)
    x497 = x102 * x494 + x496
    x498 = x260 * x90
    x499 = x273 * x98
    x500 = x158 * x457
    x501 = x162 * x20
    x502 = x147 * x482
    x503 = x494 * x98
    x504 = x429 * x497
    x505 = x143 + 2.0 * x456
    x506 = x0 * (x242 + x505) + x179 * x482
    x507 = x0 * (x248 + 2.0 * x436 + x467 + x482) + x179 * x485
    x508 = x114 * x507
    x509 = x179 * x488
    x510 = x492 + x509
    x511 = x179 * x494
    x512 = x496 + x511
    x513 = x429 * x512
    x514 = x430 * (x0 * (x299 + 6.0 * x492 + 3.0 * x493 + 3.0 * x509) + x102 * x512)
    x515 = x10 * x44
    x516 = 3.0 * x0 * (2.0 * x38 + x45 + x515) + x3 * x305
    x517 = x108 * x301**2
    x518 = x110 + x517
    x519 = x518 * x87
    x520 = x519 * x89
    x521 = x301 * x309
    x522 = x313 + x521
    x523 = x38 + x515
    x524 = x0 * (x319 + x517)
    x525 = x522 * x97
    x526 = x524 + x525
    x527 = x107 * x526
    x528 = x114 * x522
    x529 = 2.0 * x0 * (2.0 * x313 + x314 + x521) + x526 * x97
    x530 = x0 * (x330 + x39) + x10 * x324
    x531 = x530 * x98
    x532 = x329 + x8
    x533 = x10 * (x405 + x532)
    x534 = x0 * (2.0 * x320 + 2.0 * x321 + 3.0 * x524 + 3.0 * x525) + x529 * x97
    x535 = x533 * x98
    x536 = x301 * x337
    x537 = x366 + x536
    x538 = x3 * x46 + x304
    x539 = x301 * x340
    x540 = x342 + x539
    x541 = x114 * x130
    x542 = x0 * (x351 + x521 + x537)
    x543 = x540 * x97
    x544 = x542 + x543
    x545 = x158 * x544
    x546 = 0.1111111111111111 * x44
    x547 = x158 * x537
    x548 = 2.0 * x539
    x549 = x0 * (x382 + x526 + x548)
    x550 = x544 * x97
    x551 = x549 + x550
    x552 = x162 * x324
    x553 = 2.0 * x352
    x554 = x0 * (x387 + x529 + 3.0 * x542 + 3.0 * x543 + x553) + x551 * x97
    x555 = x147 * x532
    x556 = x532 * x98
    x557 = 0.1111111111111111 * x532
    x558 = x160 * x44
    x559 = x114 * x529
    x560 = x114 * x518
    x561 = x0 * (x402 + x517) + x145 * x537
    x562 = x0 * (3.0 * x29 + 3.0 * x31 + x406) + x3 * x370
    x563 = x370 * x98
    x564 = x145 * x540
    x565 = x542 + x564
    x566 = x114 * x565
    x567 = x145 * x544
    x568 = x549 + x567
    x569 = x158 * x32
    x570 = x160 * x569
    x571 = x162 * x32
    x572 = 0.1111111111111111 * x561
    x573 = x0 * (2.0 * x376 + 4.0 * x542 + 2.0 * x543 + x553 + 2.0 * x564)
    x574 = x568 * x97
    x575 = x573 + x574
    x576 = x114 * x30
    x577 = x162 * x30
    x578 = x0 * (2.0 * x383 + 2.0 * x384 + 5.0 * x549 + 2.0 * x550 + 3.0 * x567)
    x579 = x418 * (x575 * x97 + x578)
    x580 = x10 * x147
    x581 = x10 * x418
    x582 = x103 * x114
    x583 = x11 * x158
    x584 = x232 * x537
    x585 = x106 * x181
    x586 = x106 * x537
    x587 = x30 * x98
    x588 = x106 * x11
    x589 = x560 * x98
    x590 = x11 * x98
    x591 = x0 * (3.0 * x40 + x9) + x3 * x406
    x592 = x591 * x89
    x593 = 2.0 * x0 * (2.0 * x366 + x367 + x536) + x145 * x561
    x594 = x593 * x87
    x595 = x0 * (x380 + x381 + x548 + x561) + x145 * x565
    x596 = x406 * x98
    x597 = x145 * x568
    x598 = x573 + x597
    x599 = x107 * x404
    x600 = x599 * x88
    x601 = x145 * x575 + x578
    x602 = x418 * x601
    x603 = x3 * x98
    x604 = x15 * x98
    x605 = x114 * x604
    x606 = x147 * x591
    x607 = x114 * x596
    x608 = x160 * x404
    x609 = x114 * x603
    x610 = x114 * x15
    x611 = x7 * x98
    x612 = x114 * x611
    x613 = x160 * x7
    x614 = x147 * x7
    x615 = x7 * x90
    x616 = x305 * x98
    x617 = x523 * x88
    x618 = x533 * x90
    x619 = x147 * x538
    x620 = x147 * x368
    x621 = x588 * x98
    x622 = x162 * x465
    x623 = x11 * x89
    x624 = x592 * x87
    x625 = x15 * x162
    x626 = x7 * x88
    x627 = x105 * x431**2
    x628 = x118 + x627
    x629 = x628 * x87
    x630 = x629 * x89
    x631 = x628 * x98
    x632 = x431 * x433
    x633 = x436 + x632
    x634 = x114 * x633
    x635 = x0 * (x442 + x627)
    x636 = x102 * x633
    x637 = x635 + x636
    x638 = x107 * x637
    x639 = 2.0 * x0 * (2.0 * x436 + x437 + x632) + x102 * x637
    x640 = x0 * (2.0 * x443 + 2.0 * x444 + 3.0 * x635 + 3.0 * x636) + x102 * x639
    x641 = x114 * x628
    x642 = x114 * x639
    x643 = x431 * x457
    x644 = x480 + x643
    x645 = x114 * x133
    x646 = x431 * x461
    x647 = x463 + x646
    x648 = x0 * (x468 + x632 + x644)
    x649 = x102 * x647
    x650 = x648 + x649
    x651 = x158 * x650
    x652 = 2.0 * x646
    x653 = x0 * (x491 + x637 + x652)
    x654 = x102 * x650
    x655 = x653 + x654
    x656 = 2.0 * x469
    x657 = x0 * (x495 + x639 + 3.0 * x648 + 3.0 * x649 + x656) + x102 * x655
    x658 = x569 * x88
    x659 = x232 * x644
    x660 = x106 * x644
    x661 = x106 * x150
    x662 = x158 * x647
    x663 = x10 * x429
    x664 = x0 * (x505 + x627) + x179 * x644
    x665 = x179 * x647
    x666 = x648 + x665
    x667 = x114 * x666
    x668 = x179 * x650
    x669 = x653 + x668
    x670 = x0 * (2.0 * x487 + 4.0 * x648 + 2.0 * x649 + x656 + 2.0 * x665)
    x671 = x102 * x669
    x672 = x670 + x671
    x673 = x114 * x99
    x674 = x0 * (2.0 * x492 + 2.0 * x493 + 5.0 * x653 + 2.0 * x654 + 3.0 * x668)
    x675 = x429 * (x102 * x672 + x674)
    x676 = x404 * x88
    x677 = 2.0 * x0 * (2.0 * x480 + x481 + x643) + x179 * x664
    x678 = x677 * x87
    x679 = x0 * (x489 + x490 + x652 + x664) + x179 * x666
    x680 = x179 * x669
    x681 = x670 + x680
    x682 = x179 * x672 + x674
    x683 = x429 * x682

    # 900 item(s)
    result[0, 0, 0] = numpy.sum(
        x95
        * (x0 * (2.0 * x47 + 2.0 * x56 + 3.0 * x78 + 3.0 * x79 + 6.0 * x82) + x3 * x86)
    )
    result[0, 0, 1] = numpy.sum(x100 * x99)
    result[0, 0, 2] = numpy.sum(x100 * x103)
    result[0, 0, 3] = numpy.sum(x104 * x105 * x113)
    result[0, 0, 4] = numpy.sum(x102 * x104 * x116)
    result[0, 0, 5] = numpy.sum(x104 * x108 * x121)
    result[0, 0, 6] = numpy.sum(x105 * x125 * x128)
    result[0, 0, 7] = numpy.sum(x111 * x130 * x131)
    result[0, 0, 8] = numpy.sum(x119 * x131 * x133)
    result[0, 0, 9] = numpy.sum(x108 * x125 * x136)
    result[0, 0, 10] = numpy.sum(x105 * x139 * x142)
    result[0, 0, 11] = numpy.sum(x128 * x129 * x141)
    result[0, 0, 12] = numpy.sum(x111 * x121 * x141)
    result[0, 0, 13] = numpy.sum(x132 * x136 * x141)
    result[0, 0, 14] = numpy.sum(x108 * x142 * x144)
    result[0, 1, 0] = numpy.sum(x148 * x149)
    result[0, 1, 1] = numpy.sum(x153 * x154 * x85)
    result[0, 1, 2] = numpy.sum(x103 * x115 * x145 * x85)
    result[0, 1, 3] = numpy.sum(x105 * x161 * x77)
    result[0, 1, 4] = numpy.sum(x129 * x163 * x77)
    result[0, 1, 5] = numpy.sum(x164 * x165 * x77)
    result[0, 1, 6] = numpy.sum(x105 * x170 * x171)
    result[0, 1, 7] = numpy.sum(x129 * x157 * x172)
    result[0, 1, 8] = numpy.sum(x119 * x152 * x172)
    result[0, 1, 9] = numpy.sum(x124 * x150 * x173)
    result[0, 1, 10] = numpy.sum(x123 * x174 * x175)
    result[0, 1, 11] = numpy.sum(x129 * x170 * x176)
    result[0, 1, 12] = numpy.sum(x120 * x159 * x177)
    result[0, 1, 13] = numpy.sum(x136 * x152 * x176)
    result[0, 1, 14] = numpy.sum(x123 * x150 * x178)
    result[0, 2, 0] = numpy.sum(x149 * x180)
    result[0, 2, 1] = numpy.sum(x116 * x179 * x85)
    result[0, 2, 2] = numpy.sum(x108 * x185 * x85)
    result[0, 2, 3] = numpy.sum(x181 * x187 * x77)
    result[0, 2, 4] = numpy.sum(x132 * x188 * x77)
    result[0, 2, 5] = numpy.sum(x108 * x194 * x77)
    result[0, 2, 6] = numpy.sum(x124 * x181 * x195)
    result[0, 2, 7] = numpy.sum(x111 * x172 * x183)
    result[0, 2, 8] = numpy.sum(x132 * x172 * x191)
    result[0, 2, 9] = numpy.sum(x108 * x171 * x200)
    result[0, 2, 10] = numpy.sum(x123 * x181 * x201)
    result[0, 2, 11] = numpy.sum(x128 * x176 * x183)
    result[0, 2, 12] = numpy.sum(x112 * x177 * x192)
    result[0, 2, 13] = numpy.sum(x132 * x176 * x200)
    result[0, 2, 14] = numpy.sum(x108 * x123 * x203)
    result[0, 3, 0] = numpy.sum(x105 * x206 * x209)
    result[0, 3, 1] = numpy.sum(x205 * x211 * x212)
    result[0, 3, 2] = numpy.sum(x129 * x213 * x214)
    result[0, 3, 3] = numpy.sum(x215 * x218 * x219)
    result[0, 3, 4] = numpy.sum(x129 * x220 * x75)
    result[0, 3, 5] = numpy.sum(x120 * x215 * x221)
    result[0, 3, 6] = numpy.sum(x212 * x224 * x73)
    result[0, 3, 7] = numpy.sum(x129 * x217 * x225)
    result[0, 3, 8] = numpy.sum(x119 * x211 * x225)
    result[0, 3, 9] = numpy.sum(x173 * x208 * x73)
    result[0, 3, 10] = numpy.sum(x105 * x227 * x229)
    result[0, 3, 11] = numpy.sum(x130 * x224 * x230)
    result[0, 3, 12] = numpy.sum(x120 * x218 * x231)
    result[0, 3, 13] = numpy.sum(x173 * x211 * x71)
    result[0, 3, 14] = numpy.sum(x144 * x208 * x228)
    result[0, 4, 0] = numpy.sum(x145 * x206 * x233 * x94)
    result[0, 4, 1] = numpy.sum(x181 * x234 * x235)
    result[0, 4, 2] = numpy.sum(x150 * x184 * x235)
    result[0, 4, 3] = numpy.sum(x181 * x236 * x75)
    result[0, 4, 4] = numpy.sum(x152 * x188 * x215)
    result[0, 4, 5] = numpy.sum(x150 * x237 * x75)
    result[0, 4, 6] = numpy.sum(x170 * x181 * x238)
    result[0, 4, 7] = numpy.sum(x159 * x183 * x225)
    result[0, 4, 8] = numpy.sum(x152 * x192 * x225)
    result[0, 4, 9] = numpy.sum(x150 * x200 * x238)
    result[0, 4, 10] = numpy.sum(x174 * x239 * x240)
    result[0, 4, 11] = numpy.sum(x169 * x184 * x241)
    result[0, 4, 12] = numpy.sum(x157 * x237 * x71)
    result[0, 4, 13] = numpy.sum(x152 * x200 * x241)
    result[0, 4, 14] = numpy.sum(x150 * x202 * x239)
    result[0, 5, 0] = numpy.sum(x108 * x206 * x245)
    result[0, 5, 1] = numpy.sum(x132 * x214 * x246)
    result[0, 5, 2] = numpy.sum(x205 * x248 * x250)
    result[0, 5, 3] = numpy.sum(x186 * x215 * x243)
    result[0, 5, 4] = numpy.sum(x132 * x251 * x75)
    result[0, 5, 5] = numpy.sum(x215 * x253 * x254)
    result[0, 5, 6] = numpy.sum(x195 * x243 * x73)
    result[0, 5, 7] = numpy.sum(x111 * x225 * x248)
    result[0, 5, 8] = numpy.sum(x132 * x225 * x253)
    result[0, 5, 9] = numpy.sum(x250 * x257 * x73)
    result[0, 5, 10] = numpy.sum(x138 * x228 * x244)
    result[0, 5, 11] = numpy.sum(x195 * x248 * x71)
    result[0, 5, 12] = numpy.sum(x186 * x231 * x253)
    result[0, 5, 13] = numpy.sum(x133 * x230 * x257)
    result[0, 5, 14] = numpy.sum(x108 * x229 * x259)
    result[0, 6, 0] = numpy.sum(x105 * x260 * x265)
    result[0, 6, 1] = numpy.sum(x105 * x204 * x267)
    result[0, 6, 2] = numpy.sum(x129 * x204 * x268)
    result[0, 6, 3] = numpy.sum(x219 * x270 * x271)
    result[0, 6, 4] = numpy.sum(x129 * x267 * x272)
    result[0, 6, 5] = numpy.sum(x121 * x260 * x66)
    result[0, 6, 6] = numpy.sum(x105 * x273 * x274)
    result[0, 6, 7] = numpy.sum(x130 * x270 * x275)
    result[0, 6, 8] = numpy.sum(x119 * x267 * x275)
    result[0, 6, 9] = numpy.sum(x136 * x260 * x64)
    result[0, 6, 10] = numpy.sum(x105 * x276 * x279)
    result[0, 6, 11] = numpy.sum(x129 * x273 * x280)
    result[0, 6, 12] = numpy.sum(x121 * x270 * x277)
    result[0, 6, 13] = numpy.sum(x136 * x266 * x277)
    result[0, 6, 14] = numpy.sum(x144 * x260 * x278)
    result[0, 7, 0] = numpy.sum(x181 * x209 * x264)
    result[0, 7, 1] = numpy.sum(x211 * x281 * x282)
    result[0, 7, 2] = numpy.sum(x185 * x204 * x208)
    result[0, 7, 3] = numpy.sum(x181 * x284 * x66)
    result[0, 7, 4] = numpy.sum(x188 * x211 * x66)
    result[0, 7, 5] = numpy.sum(x193 * x221 * x66)
    result[0, 7, 6] = numpy.sum(x224 * x275 * x281)
    result[0, 7, 7] = numpy.sum(x188 * x217 * x64)
    result[0, 7, 8] = numpy.sum(x211 * x237 * x64)
    result[0, 7, 9] = numpy.sum(x200 * x208 * x275)
    result[0, 7, 10] = numpy.sum(x227 * x240 * x285)
    result[0, 7, 11] = numpy.sum(x185 * x224 * x277)
    result[0, 7, 12] = numpy.sum(x193 * x218 * x277)
    result[0, 7, 13] = numpy.sum(x200 * x211 * x286)
    result[0, 7, 14] = numpy.sum(x202 * x208 * x285)
    result[0, 8, 0] = numpy.sum(x150 * x245 * x264)
    result[0, 8, 1] = numpy.sum(x153 * x204 * x246)
    result[0, 8, 2] = numpy.sum(x248 * x282 * x287)
    result[0, 8, 3] = numpy.sum(x161 * x243 * x66)
    result[0, 8, 4] = numpy.sum(x152 * x251 * x66)
    result[0, 8, 5] = numpy.sum(x164 * x288 * x66)
    result[0, 8, 6] = numpy.sum(x169 * x246 * x275)
    result[0, 8, 7] = numpy.sum(x157 * x251 * x64)
    result[0, 8, 8] = numpy.sum(x152 * x289 * x64)
    result[0, 8, 9] = numpy.sum(x257 * x275 * x287)
    result[0, 8, 10] = numpy.sum(x174 * x244 * x285)
    result[0, 8, 11] = numpy.sum(x169 * x248 * x290)
    result[0, 8, 12] = numpy.sum(x161 * x253 * x277)
    result[0, 8, 13] = numpy.sum(x152 * x257 * x290)
    result[0, 8, 14] = numpy.sum(x259 * x285 * x291)
    result[0, 9, 0] = numpy.sum(x108 * x265 * x292)
    result[0, 9, 1] = numpy.sum(x132 * x204 * x293)
    result[0, 9, 2] = numpy.sum(x108 * x204 * x295)
    result[0, 9, 3] = numpy.sum(x113 * x292 * x66)
    result[0, 9, 4] = numpy.sum(x132 * x272 * x295)
    result[0, 9, 5] = numpy.sum(x108 * x271 * x298)
    result[0, 9, 6] = numpy.sum(x128 * x292 * x64)
    result[0, 9, 7] = numpy.sum(x111 * x275 * x295)
    result[0, 9, 8] = numpy.sum(x133 * x275 * x297)
    result[0, 9, 9] = numpy.sum(x108 * x274 * x299)
    result[0, 9, 10] = numpy.sum(x139 * x278 * x292)
    result[0, 9, 11] = numpy.sum(x128 * x277 * x294)
    result[0, 9, 12] = numpy.sum(x113 * x277 * x297)
    result[0, 9, 13] = numpy.sum(x132 * x280 * x299)
    result[0, 9, 14] = numpy.sum(x108 * x279 * x300)
    result[1, 0, 0] = numpy.sum(x301 * x307)
    result[1, 0, 1] = numpy.sum(x154 * x306 * x309)
    result[1, 0, 2] = numpy.sum(x103 * x301 * x310)
    result[1, 0, 3] = numpy.sum(x219 * x311 * x316)
    result[1, 0, 4] = numpy.sum(x129 * x311 * x318)
    result[1, 0, 5] = numpy.sum(x121 * x311 * x312)
    result[1, 0, 6] = numpy.sum(x105 * x322 * x327)
    result[1, 0, 7] = numpy.sum(x315 * x327 * x328)
    result[1, 0, 8] = numpy.sum(x119 * x317 * x327)
    result[1, 0, 9] = numpy.sum(x136 * x312 * x326)
    result[1, 0, 10] = numpy.sum(x331 * x333 * x334)
    result[1, 0, 11] = numpy.sum(x130 * x322 * x331)
    result[1, 0, 12] = numpy.sum(x121 * x315 * x331)
    result[1, 0, 13] = numpy.sum(x136 * x309 * x331)
    result[1, 0, 14] = numpy.sum(x312 * x331 * x335)
    result[1, 1, 0] = numpy.sum(x175 * x337 * x338)
    result[1, 1, 1] = numpy.sum(x212 * x340 * x55)
    result[1, 1, 2] = numpy.sum(x328 * x341 * x55)
    result[1, 1, 3] = numpy.sum(x345 * x346 * x53)
    result[1, 1, 4] = numpy.sum(x340 * x347 * x53)
    result[1, 1, 5] = numpy.sum(x120 * x348 * x349)
    result[1, 1, 6] = numpy.sum(x212 * x325 * x354)
    result[1, 1, 7] = numpy.sum(x129 * x344 * x355)
    result[1, 1, 8] = numpy.sum(x325 * x340 * x356)
    result[1, 1, 9] = numpy.sum(x173 * x325 * x337)
    result[1, 1, 10] = numpy.sum(x175 * x323 * x357)
    result[1, 1, 11] = numpy.sum(x328 * x354 * x358)
    result[1, 1, 12] = numpy.sum(x120 * x345 * x359)
    result[1, 1, 13] = numpy.sum(x173 * x323 * x340)
    result[1, 1, 14] = numpy.sum(x178 * x323 * x337)
    result[1, 2, 0] = numpy.sum(x180 * x301 * x360)
    result[1, 2, 1] = numpy.sum(x181 * x318 * x55)
    result[1, 2, 2] = numpy.sum(x185 * x312 * x55)
    result[1, 2, 3] = numpy.sum(x361 * x362 * x53)
    result[1, 2, 4] = numpy.sum(x188 * x309 * x53)
    result[1, 2, 5] = numpy.sum(x193 * x312 * x349)
    result[1, 2, 6] = numpy.sum(x281 * x325 * x363)
    result[1, 2, 7] = numpy.sum(x188 * x315 * x325)
    result[1, 2, 8] = numpy.sum(x237 * x309 * x325)
    result[1, 2, 9] = numpy.sum(x200 * x325 * x364)
    result[1, 2, 10] = numpy.sum(x323 * x333 * x365)
    result[1, 2, 11] = numpy.sum(x185 * x322 * x323)
    result[1, 2, 12] = numpy.sum(x193 * x315 * x359)
    result[1, 2, 13] = numpy.sum(x200 * x317 * x323)
    result[1, 2, 14] = numpy.sum(x203 * x312 * x323)
    result[1, 3, 0] = numpy.sum(x175 * x368 * x372)
    result[1, 3, 1] = numpy.sum(x212 * x371 * x374)
    result[1, 3, 2] = numpy.sum(x328 * x371 * x375)
    result[1, 3, 3] = numpy.sum(x346 * x377 * x378)
    result[1, 3, 4] = numpy.sum(x24 * x347 * x374)
    result[1, 3, 5] = numpy.sum(x120 * x378 * x379)
    result[1, 3, 6] = numpy.sum(x212 * x22 * x385)
    result[1, 3, 7] = numpy.sum(x129 * x377 * x386)
    result[1, 3, 8] = numpy.sum(x22 * x356 * x374)
    result[1, 3, 9] = numpy.sum(x173 * x22 * x368)
    result[1, 3, 10] = numpy.sum(x175 * x389 * x69)
    result[1, 3, 11] = numpy.sum(x328 * x385 * x390)
    result[1, 3, 12] = numpy.sum(x120 * x391 * x392)
    result[1, 3, 13] = numpy.sum(x173 * x374 * x69)
    result[1, 3, 14] = numpy.sum(x178 * x368 * x69)
    result[1, 4, 0] = numpy.sum(x240 * x372 * x393)
    result[1, 4, 1] = numpy.sum(x340 * x371 * x394)
    result[1, 4, 2] = numpy.sum(x184 * x371 * x395)
    result[1, 4, 3] = numpy.sum(x181 * x344 * x396)
    result[1, 4, 4] = numpy.sum(x188 * x340 * x378)
    result[1, 4, 5] = numpy.sum(x237 * x24 * x337)
    result[1, 4, 6] = numpy.sum(x22 * x354 * x394)
    result[1, 4, 7] = numpy.sum(x188 * x22 * x345)
    result[1, 4, 8] = numpy.sum(x22 * x340 * x397)
    result[1, 4, 9] = numpy.sum(x200 * x22 * x395)
    result[1, 4, 10] = numpy.sum(x240 * x357 * x398)
    result[1, 4, 11] = numpy.sum(x184 * x354 * x399)
    result[1, 4, 12] = numpy.sum(x237 * x344 * x69)
    result[1, 4, 13] = numpy.sum(x200 * x340 * x399)
    result[1, 4, 14] = numpy.sum(x202 * x337 * x398)
    result[1, 5, 0] = numpy.sum(x245 * x312 * x372)
    result[1, 5, 1] = numpy.sum(x246 * x317 * x371)
    result[1, 5, 2] = numpy.sum(x248 * x371 * x400)
    result[1, 5, 3] = numpy.sum(x243 * x361 * x378)
    result[1, 5, 4] = numpy.sum(x24 * x251 * x309)
    result[1, 5, 5] = numpy.sum(x288 * x312 * x378)
    result[1, 5, 6] = numpy.sum(x22 * x246 * x363)
    result[1, 5, 7] = numpy.sum(x22 * x251 * x315)
    result[1, 5, 8] = numpy.sum(x22 * x289 * x309)
    result[1, 5, 9] = numpy.sum(x22 * x257 * x400)
    result[1, 5, 10] = numpy.sum(x245 * x333 * x69)
    result[1, 5, 11] = numpy.sum(x248 * x363 * x390)
    result[1, 5, 12] = numpy.sum(x253 * x361 * x391)
    result[1, 5, 13] = numpy.sum(x257 * x317 * x390)
    result[1, 5, 14] = numpy.sum(x312 * x401 * x69)
    result[1, 6, 0] = numpy.sum(x334 * x403 * x407)
    result[1, 6, 1] = numpy.sum(x105 * x369 * x409)
    result[1, 6, 2] = numpy.sum(x130 * x369 * x403)
    result[1, 6, 3] = numpy.sum(x105 * x411 * x413)
    result[1, 6, 4] = numpy.sum(x262 * x328 * x409)
    result[1, 6, 5] = numpy.sum(x121 * x262 * x403)
    result[1, 6, 6] = numpy.sum(x154 * x20 * x415)
    result[1, 6, 7] = numpy.sum(x129 * x411 * x417)
    result[1, 6, 8] = numpy.sum(x119 * x409 * x416)
    result[1, 6, 9] = numpy.sum(x136 * x20 * x403)
    result[1, 6, 10] = numpy.sum(x13 * x420)
    result[1, 6, 11] = numpy.sum(x103 * x13 * x421)
    result[1, 6, 12] = numpy.sum(x121 * x18 * x411)
    result[1, 6, 13] = numpy.sum(x136 * x18 * x408)
    result[1, 6, 14] = numpy.sum(x18 * x335 * x403)
    result[1, 7, 0] = numpy.sum(x365 * x368 * x407)
    result[1, 7, 1] = numpy.sum(x281 * x374 * x422)
    result[1, 7, 2] = numpy.sum(x185 * x368 * x369)
    result[1, 7, 3] = numpy.sum(x362 * x377 * x423)
    result[1, 7, 4] = numpy.sum(x188 * x262 * x374)
    result[1, 7, 5] = numpy.sum(x193 * x262 * x379)
    result[1, 7, 6] = numpy.sum(x181 * x385 * x417)
    result[1, 7, 7] = numpy.sum(x188 * x20 * x377)
    result[1, 7, 8] = numpy.sum(x20 * x237 * x374)
    result[1, 7, 9] = numpy.sum(x200 * x368 * x416)
    result[1, 7, 10] = numpy.sum(x13 * x180 * x424)
    result[1, 7, 11] = numpy.sum(x18 * x185 * x385)
    result[1, 7, 12] = numpy.sum(x18 * x193 * x392)
    result[1, 7, 13] = numpy.sum(x200 * x374 * x425)
    result[1, 7, 14] = numpy.sum(x18 * x203 * x368)
    result[1, 8, 0] = numpy.sum(x245 * x337 * x407)
    result[1, 8, 1] = numpy.sum(x246 * x340 * x422)
    result[1, 8, 2] = numpy.sum(x248 * x341 * x422)
    result[1, 8, 3] = numpy.sum(x243 * x345 * x423)
    result[1, 8, 4] = numpy.sum(x251 * x262 * x340)
    result[1, 8, 5] = numpy.sum(x253 * x348 * x423)
    result[1, 8, 6] = numpy.sum(x246 * x354 * x416)
    result[1, 8, 7] = numpy.sum(x20 * x251 * x344)
    result[1, 8, 8] = numpy.sum(x20 * x289 * x340)
    result[1, 8, 9] = numpy.sum(x257 * x341 * x416)
    result[1, 8, 10] = numpy.sum(x18 * x245 * x357)
    result[1, 8, 11] = numpy.sum(x248 * x354 * x426)
    result[1, 8, 12] = numpy.sum(x18 * x288 * x345)
    result[1, 8, 13] = numpy.sum(x257 * x340 * x426)
    result[1, 8, 14] = numpy.sum(x18 * x337 * x401)
    result[1, 9, 0] = numpy.sum(x312 * x407 * x427)
    result[1, 9, 1] = numpy.sum(x293 * x309 * x369)
    result[1, 9, 2] = numpy.sum(x295 * x312 * x369)
    result[1, 9, 3] = numpy.sum(x292 * x315 * x413)
    result[1, 9, 4] = numpy.sum(x262 * x295 * x317)
    result[1, 9, 5] = numpy.sum(x297 * x312 * x413)
    result[1, 9, 6] = numpy.sum(x20 * x293 * x322)
    result[1, 9, 7] = numpy.sum(x295 * x315 * x416)
    result[1, 9, 8] = numpy.sum(x20 * x297 * x318)
    result[1, 9, 9] = numpy.sum(x20 * x312 * x428)
    result[1, 9, 10] = numpy.sum(x18 * x333 * x427)
    result[1, 9, 11] = numpy.sum(x18 * x295 * x322)
    result[1, 9, 12] = numpy.sum(x18 * x298 * x316)
    result[1, 9, 13] = numpy.sum(x18 * x309 * x428)
    result[1, 9, 14] = numpy.sum(x13 * x300 * x301 * x430)
    result[2, 0, 0] = numpy.sum(x307 * x431)
    result[2, 0, 1] = numpy.sum(x310 * x431 * x99)
    result[2, 0, 2] = numpy.sum(x249 * x306 * x433)
    result[2, 0, 3] = numpy.sum(x113 * x311 * x434)
    result[2, 0, 4] = numpy.sum(x133 * x311 * x435)
    result[2, 0, 5] = numpy.sum(x108 * x311 * x440)
    result[2, 0, 6] = numpy.sum(x128 * x326 * x434)
    result[2, 0, 7] = numpy.sum(x111 * x327 * x435)
    result[2, 0, 8] = numpy.sum(x327 * x438 * x441)
    result[2, 0, 9] = numpy.sum(x108 * x327 * x445)
    result[2, 0, 10] = numpy.sum(x331 * x434 * x446)
    result[2, 0, 11] = numpy.sum(x128 * x331 * x433)
    result[2, 0, 12] = numpy.sum(x113 * x331 * x438)
    result[2, 0, 13] = numpy.sum(x133 * x331 * x445)
    result[2, 0, 14] = numpy.sum(x331 * x448 * x449)
    result[2, 1, 0] = numpy.sum(x148 * x360 * x431)
    result[2, 1, 1] = numpy.sum(x153 * x450 * x55)
    result[2, 1, 2] = numpy.sum(x287 * x435 * x55)
    result[2, 1, 3] = numpy.sum(x161 * x434 * x53)
    result[2, 1, 4] = numpy.sum(x152 * x451 * x53)
    result[2, 1, 5] = numpy.sum(x164 * x452 * x53)
    result[2, 1, 6] = numpy.sum(x170 * x325 * x453)
    result[2, 1, 7] = numpy.sum(x157 * x325 * x451)
    result[2, 1, 8] = numpy.sum(x152 * x355 * x438)
    result[2, 1, 9] = numpy.sum(x287 * x325 * x454)
    result[2, 1, 10] = numpy.sum(x174 * x434 * x455)
    result[2, 1, 11] = numpy.sum(x169 * x358 * x435)
    result[2, 1, 12] = numpy.sum(x161 * x323 * x438)
    result[2, 1, 13] = numpy.sum(x153 * x358 * x445)
    result[2, 1, 14] = numpy.sum(x150 * x448 * x455)
    result[2, 2, 0] = numpy.sum(x338 * x457 * x458)
    result[2, 2, 1] = numpy.sum(x441 * x459 * x55)
    result[2, 2, 2] = numpy.sum(x250 * x461 * x55)
    result[2, 2, 3] = numpy.sum(x187 * x457 * x53)
    result[2, 2, 4] = numpy.sum(x132 * x462 * x53)
    result[2, 2, 5] = numpy.sum(x254 * x466 * x53)
    result[2, 2, 6] = numpy.sum(x195 * x325 * x457)
    result[2, 2, 7] = numpy.sum(x111 * x355 * x461)
    result[2, 2, 8] = numpy.sum(x132 * x355 * x465)
    result[2, 2, 9] = numpy.sum(x250 * x325 * x471)
    result[2, 2, 10] = numpy.sum(x201 * x323 * x457)
    result[2, 2, 11] = numpy.sum(x195 * x323 * x461)
    result[2, 2, 12] = numpy.sum(x187 * x323 * x465)
    result[2, 2, 13] = numpy.sum(x358 * x441 * x471)
    result[2, 2, 14] = numpy.sum(x323 * x458 * x472)
    result[2, 3, 0] = numpy.sum(x209 * x372 * x434)
    result[2, 3, 1] = numpy.sum(x211 * x371 * x473)
    result[2, 3, 2] = numpy.sum(x213 * x371 * x435)
    result[2, 3, 3] = numpy.sum(x378 * x434 * x474)
    result[2, 3, 4] = numpy.sum(x211 * x24 * x451)
    result[2, 3, 5] = numpy.sum(x221 * x378 * x439)
    result[2, 3, 6] = numpy.sum(x22 * x224 * x473)
    result[2, 3, 7] = numpy.sum(x217 * x22 * x451)
    result[2, 3, 8] = numpy.sum(x211 * x386 * x438)
    result[2, 3, 9] = numpy.sum(x213 * x22 * x454)
    result[2, 3, 10] = numpy.sum(x434 * x475 * x69)
    result[2, 3, 11] = numpy.sum(x224 * x390 * x435)
    result[2, 3, 12] = numpy.sum(x218 * x391 * x439)
    result[2, 3, 13] = numpy.sum(x211 * x390 * x454)
    result[2, 3, 14] = numpy.sum(x209 * x448 * x69)
    result[2, 4, 0] = numpy.sum(x150 * x372 * x477)
    result[2, 4, 1] = numpy.sum(x234 * x371 * x478)
    result[2, 4, 2] = numpy.sum(x371 * x461 * x479)
    result[2, 4, 3] = numpy.sum(x236 * x24 * x457)
    result[2, 4, 4] = numpy.sum(x152 * x378 * x462)
    result[2, 4, 5] = numpy.sum(x150 * x396 * x465)
    result[2, 4, 6] = numpy.sum(x170 * x22 * x478)
    result[2, 4, 7] = numpy.sum(x159 * x386 * x461)
    result[2, 4, 8] = numpy.sum(x152 * x386 * x466)
    result[2, 4, 9] = numpy.sum(x22 * x471 * x479)
    result[2, 4, 10] = numpy.sum(x174 * x398 * x476)
    result[2, 4, 11] = numpy.sum(x170 * x399 * x461)
    result[2, 4, 12] = numpy.sum(x236 * x465 * x69)
    result[2, 4, 13] = numpy.sum(x234 * x399 * x471)
    result[2, 4, 14] = numpy.sum(x291 * x398 * x472)
    result[2, 5, 0] = numpy.sum(x372 * x458 * x482)
    result[2, 5, 1] = numpy.sum(x371 * x441 * x483)
    result[2, 5, 2] = numpy.sum(x250 * x371 * x485)
    result[2, 5, 3] = numpy.sum(x186 * x378 * x482)
    result[2, 5, 4] = numpy.sum(x132 * x24 * x486)
    result[2, 5, 5] = numpy.sum(x254 * x378 * x488)
    result[2, 5, 6] = numpy.sum(x195 * x22 * x482)
    result[2, 5, 7] = numpy.sum(x111 * x386 * x485)
    result[2, 5, 8] = numpy.sum(x132 * x386 * x488)
    result[2, 5, 9] = numpy.sum(x22 * x250 * x494)
    result[2, 5, 10] = numpy.sum(x201 * x482 * x69)
    result[2, 5, 11] = numpy.sum(x195 * x485 * x69)
    result[2, 5, 12] = numpy.sum(x186 * x391 * x488)
    result[2, 5, 13] = numpy.sum(x390 * x441 * x494)
    result[2, 5, 14] = numpy.sum(x458 * x497 * x69)
    result[2, 6, 0] = numpy.sum(x407 * x434 * x498)
    result[2, 6, 1] = numpy.sum(x267 * x369 * x434)
    result[2, 6, 2] = numpy.sum(x268 * x369 * x433)
    result[2, 6, 3] = numpy.sum(x270 * x413 * x434)
    result[2, 6, 4] = numpy.sum(x262 * x267 * x435)
    result[2, 6, 5] = numpy.sum(x260 * x413 * x438)
    result[2, 6, 6] = numpy.sum(x20 * x434 * x499)
    result[2, 6, 7] = numpy.sum(x270 * x417 * x433)
    result[2, 6, 8] = numpy.sum(x267 * x416 * x438)
    result[2, 6, 9] = numpy.sum(x20 * x268 * x445)
    result[2, 6, 10] = numpy.sum(x13 * x276 * x419 * x431)
    result[2, 6, 11] = numpy.sum(x18 * x433 * x499)
    result[2, 6, 12] = numpy.sum(x18 * x270 * x440)
    result[2, 6, 13] = numpy.sum(x18 * x267 * x445)
    result[2, 6, 14] = numpy.sum(x18 * x448 * x498)
    result[2, 7, 0] = numpy.sum(x209 * x407 * x457)
    result[2, 7, 1] = numpy.sum(x211 * x422 * x459)
    result[2, 7, 2] = numpy.sum(x213 * x422 * x461)
    result[2, 7, 3] = numpy.sum(x218 * x412 * x500)
    result[2, 7, 4] = numpy.sum(x211 * x262 * x462)
    result[2, 7, 5] = numpy.sum(x221 * x412 * x466)
    result[2, 7, 6] = numpy.sum(x224 * x417 * x457)
    result[2, 7, 7] = numpy.sum(x20 * x217 * x462)
    result[2, 7, 8] = numpy.sum(x211 * x465 * x501)
    result[2, 7, 9] = numpy.sum(x213 * x416 * x471)
    result[2, 7, 10] = numpy.sum(x18 * x457 * x475)
    result[2, 7, 11] = numpy.sum(x224 * x426 * x461)
    result[2, 7, 12] = numpy.sum(x18 * x466 * x474)
    result[2, 7, 13] = numpy.sum(x211 * x426 * x471)
    result[2, 7, 14] = numpy.sum(x18 * x209 * x472)
    result[2, 8, 0] = numpy.sum(x150 * x407 * x502)
    result[2, 8, 1] = numpy.sum(x153 * x369 * x483)
    result[2, 8, 2] = numpy.sum(x287 * x422 * x485)
    result[2, 8, 3] = numpy.sum(x161 * x262 * x482)
    result[2, 8, 4] = numpy.sum(x152 * x262 * x486)
    result[2, 8, 5] = numpy.sum(x164 * x423 * x488)
    result[2, 8, 6] = numpy.sum(x170 * x416 * x482)
    result[2, 8, 7] = numpy.sum(x20 * x236 * x485)
    result[2, 8, 8] = numpy.sum(x152 * x488 * x501)
    result[2, 8, 9] = numpy.sum(x150 * x417 * x494)
    result[2, 8, 10] = numpy.sum(x174 * x18 * x502)
    result[2, 8, 11] = numpy.sum(x170 * x425 * x485)
    result[2, 8, 12] = numpy.sum(x161 * x18 * x488)
    result[2, 8, 13] = numpy.sum(x153 * x18 * x503)
    result[2, 8, 14] = numpy.sum(x13 * x148 * x504)
    result[2, 9, 0] = numpy.sum(x407 * x449 * x506)
    result[2, 9, 1] = numpy.sum(x133 * x369 * x506)
    result[2, 9, 2] = numpy.sum(x249 * x369 * x507)
    result[2, 9, 3] = numpy.sum(x113 * x262 * x506)
    result[2, 9, 4] = numpy.sum(x133 * x262 * x508)
    result[2, 9, 5] = numpy.sum(x108 * x413 * x510)
    result[2, 9, 6] = numpy.sum(x128 * x20 * x506)
    result[2, 9, 7] = numpy.sum(x111 * x417 * x507)
    result[2, 9, 8] = numpy.sum(x132 * x417 * x510)
    result[2, 9, 9] = numpy.sum(x20 * x249 * x512)
    result[2, 9, 10] = numpy.sum(x18 * x446 * x506)
    result[2, 9, 11] = numpy.sum(x128 * x18 * x507)
    result[2, 9, 12] = numpy.sum(x113 * x18 * x510)
    result[2, 9, 13] = numpy.sum(x13 * x513 * x99)
    result[2, 9, 14] = numpy.sum(x13 * x514)
    result[3, 0, 0] = numpy.sum(x105 * x516 * x520)
    result[3, 0, 1] = numpy.sum(x154 * x305 * x522)
    result[3, 0, 2] = numpy.sum(x130 * x305 * x518)
    result[3, 0, 3] = numpy.sum(x219 * x523 * x527)
    result[3, 0, 4] = numpy.sum(x130 * x523 * x528)
    result[3, 0, 5] = numpy.sum(x121 * x518 * x523)
    result[3, 0, 6] = numpy.sum(x105 * x529 * x531)
    result[3, 0, 7] = numpy.sum(x328 * x526 * x531)
    result[3, 0, 8] = numpy.sum(x119 * x528 * x531)
    result[3, 0, 9] = numpy.sum(x136 * x518 * x530)
    result[3, 0, 10] = numpy.sum(x334 * x533 * x534)
    result[3, 0, 11] = numpy.sum(x129 * x529 * x535)
    result[3, 0, 12] = numpy.sum(x121 * x526 * x533)
    result[3, 0, 13] = numpy.sum(x136 * x522 * x533)
    result[3, 0, 14] = numpy.sum(x144 * x519 * x533)
    result[3, 1, 0] = numpy.sum(x175 * x537 * x538)
    result[3, 1, 1] = numpy.sum(x212 * x46 * x540)
    result[3, 1, 2] = numpy.sum(x46 * x537 * x541)
    result[3, 1, 3] = numpy.sum(x346 * x44 * x545)
    result[3, 1, 4] = numpy.sum(x347 * x44 * x540)
    result[3, 1, 5] = numpy.sum(x120 * x546 * x547)
    result[3, 1, 6] = numpy.sum(x212 * x324 * x551)
    result[3, 1, 7] = numpy.sum(x129 * x544 * x552)
    result[3, 1, 8] = numpy.sum(x324 * x356 * x540)
    result[3, 1, 9] = numpy.sum(x173 * x324 * x537)
    result[3, 1, 10] = numpy.sum(x105 * x554 * x555)
    result[3, 1, 11] = numpy.sum(x328 * x551 * x556)
    result[3, 1, 12] = numpy.sum(x120 * x545 * x557)
    result[3, 1, 13] = numpy.sum(x173 * x532 * x540)
    result[3, 1, 14] = numpy.sum(x178 * x532 * x537)
    result[3, 2, 0] = numpy.sum(x365 * x518 * x538)
    result[3, 2, 1] = numpy.sum(x281 * x46 * x528)
    result[3, 2, 2] = numpy.sum(x185 * x46 * x518)
    result[3, 2, 3] = numpy.sum(x362 * x526 * x558)
    result[3, 2, 4] = numpy.sum(x188 * x44 * x522)
    result[3, 2, 5] = numpy.sum(x193 * x518 * x546)
    result[3, 2, 6] = numpy.sum(x281 * x324 * x559)
    result[3, 2, 7] = numpy.sum(x188 * x324 * x526)
    result[3, 2, 8] = numpy.sum(x237 * x324 * x522)
    result[3, 2, 9] = numpy.sum(x200 * x324 * x560)
    result[3, 2, 10] = numpy.sum(x181 * x534 * x555)
    result[3, 2, 11] = numpy.sum(x185 * x529 * x532)
    result[3, 2, 12] = numpy.sum(x193 * x526 * x557)
    result[3, 2, 13] = numpy.sum(x200 * x528 * x532)
    result[3, 2, 14] = numpy.sum(x203 * x518 * x532)
    result[3, 3, 0] = numpy.sum(x175 * x561 * x562)
    result[3, 3, 1] = numpy.sum(x105 * x563 * x566)
    result[3, 3, 2] = numpy.sum(x328 * x561 * x563)
    result[3, 3, 3] = numpy.sum(x105 * x568 * x570)
    result[3, 3, 4] = numpy.sum(x129 * x565 * x571)
    result[3, 3, 5] = numpy.sum(x120 * x569 * x572)
    result[3, 3, 6] = numpy.sum(x154 * x575 * x576)
    result[3, 3, 7] = numpy.sum(x129 * x568 * x577)
    result[3, 3, 8] = numpy.sum(x30 * x356 * x565)
    result[3, 3, 9] = numpy.sum(x173 * x30 * x561)
    result[3, 3, 10] = numpy.sum(x579 * x580)
    result[3, 3, 11] = numpy.sum(x575 * x581 * x582)
    result[3, 3, 12] = numpy.sum(x165 * x568 * x583)
    result[3, 3, 13] = numpy.sum(x11 * x173 * x565)
    result[3, 3, 14] = numpy.sum(x11 * x178 * x561)
    result[3, 4, 0] = numpy.sum(x240 * x562 * x584)
    result[3, 4, 1] = numpy.sum(x540 * x563 * x585)
    result[3, 4, 2] = numpy.sum(x184 * x370 * x586)
    result[3, 4, 3] = numpy.sum(x181 * x544 * x571)
    result[3, 4, 4] = numpy.sum(x188 * x540 * x569)
    result[3, 4, 5] = numpy.sum(x237 * x32 * x537)
    result[3, 4, 6] = numpy.sum(x551 * x585 * x587)
    result[3, 4, 7] = numpy.sum(x188 * x30 * x545)
    result[3, 4, 8] = numpy.sum(x30 * x397 * x540)
    result[3, 4, 9] = numpy.sum(x200 * x30 * x586)
    result[3, 4, 10] = numpy.sum(x233 * x554 * x581)
    result[3, 4, 11] = numpy.sum(x184 * x551 * x588)
    result[3, 4, 12] = numpy.sum(x11 * x237 * x544)
    result[3, 4, 13] = numpy.sum(x200 * x540 * x588)
    result[3, 4, 14] = numpy.sum(x11 * x202 * x584)
    result[3, 5, 0] = numpy.sum(x245 * x518 * x562)
    result[3, 5, 1] = numpy.sum(x246 * x370 * x528)
    result[3, 5, 2] = numpy.sum(x248 * x370 * x589)
    result[3, 5, 3] = numpy.sum(x243 * x526 * x570)
    result[3, 5, 4] = numpy.sum(x251 * x32 * x522)
    result[3, 5, 5] = numpy.sum(x253 * x518 * x570)
    result[3, 5, 6] = numpy.sum(x246 * x529 * x576)
    result[3, 5, 7] = numpy.sum(x251 * x30 * x526)
    result[3, 5, 8] = numpy.sum(x289 * x30 * x522)
    result[3, 5, 9] = numpy.sum(x257 * x30 * x589)
    result[3, 5, 10] = numpy.sum(x11 * x245 * x534)
    result[3, 5, 11] = numpy.sum(x248 * x559 * x590)
    result[3, 5, 12] = numpy.sum(x288 * x526 * x583)
    result[3, 5, 13] = numpy.sum(x257 * x528 * x590)
    result[3, 5, 14] = numpy.sum(x11 * x401 * x518)
    result[3, 6, 0] = numpy.sum(x105 * x592 * x594)
    result[3, 6, 1] = numpy.sum(x105 * x595 * x596)
    result[3, 6, 2] = numpy.sum(x129 * x593 * x596)
    result[3, 6, 3] = numpy.sum(x105 * x598 * x600)
    result[3, 6, 4] = numpy.sum(x404 * x541 * x595)
    result[3, 6, 5] = numpy.sum(x121 * x404 * x593)
    result[3, 6, 6] = numpy.sum(x602 * x603)
    result[3, 6, 7] = numpy.sum(x3 * x418 * x582 * x598)
    result[3, 6, 8] = numpy.sum(x119 * x595 * x605)
    result[3, 6, 9] = numpy.sum(x136 * x15 * x593)
    result[3, 6, 10] = numpy.sum(
        x419
        * (
            x0 * (2.0 * x388 + 2.0 * x414 + 6.0 * x573 + 3.0 * x574 + 3.0 * x597)
            + x601 * x97
        )
    )
    result[3, 6, 11] = numpy.sum(x103 * x602)
    result[3, 6, 12] = numpy.sum(x121 * x598 * x7)
    result[3, 6, 13] = numpy.sum(x136 * x595 * x7)
    result[3, 6, 14] = numpy.sum(x144 * x594 * x7)
    result[3, 7, 0] = numpy.sum(x181 * x561 * x606)
    result[3, 7, 1] = numpy.sum(x181 * x565 * x607)
    result[3, 7, 2] = numpy.sum(x183 * x561 * x607)
    result[3, 7, 3] = numpy.sum(x362 * x568 * x608)
    result[3, 7, 4] = numpy.sum(x188 * x404 * x565)
    result[3, 7, 5] = numpy.sum(x193 * x404 * x572)
    result[3, 7, 6] = numpy.sum(x179 * x418 * x575 * x609)
    result[3, 7, 7] = numpy.sum(x15 * x188 * x568)
    result[3, 7, 8] = numpy.sum(x15 * x237 * x565)
    result[3, 7, 9] = numpy.sum(x200 * x561 * x610)
    result[3, 7, 10] = numpy.sum(x180 * x579)
    result[3, 7, 11] = numpy.sum(x185 * x575 * x7)
    result[3, 7, 12] = numpy.sum(x194 * x568 * x7)
    result[3, 7, 13] = numpy.sum(x200 * x566 * x7)
    result[3, 7, 14] = numpy.sum(x203 * x561 * x7)
    result[3, 8, 0] = numpy.sum(x245 * x537 * x591)
    result[3, 8, 1] = numpy.sum(x243 * x540 * x607)
    result[3, 8, 2] = numpy.sum(x248 * x537 * x607)
    result[3, 8, 3] = numpy.sum(x243 * x545 * x608)
    result[3, 8, 4] = numpy.sum(x251 * x404 * x540)
    result[3, 8, 5] = numpy.sum(x253 * x547 * x608)
    result[3, 8, 6] = numpy.sum(x246 * x551 * x610)
    result[3, 8, 7] = numpy.sum(x15 * x251 * x544)
    result[3, 8, 8] = numpy.sum(x15 * x289 * x540)
    result[3, 8, 9] = numpy.sum(x257 * x537 * x605)
    result[3, 8, 10] = numpy.sum(x245 * x554 * x7)
    result[3, 8, 11] = numpy.sum(x248 * x551 * x612)
    result[3, 8, 12] = numpy.sum(x253 * x545 * x613)
    result[3, 8, 13] = numpy.sum(x257 * x540 * x612)
    result[3, 8, 14] = numpy.sum(x259 * x537 * x614)
    result[3, 9, 0] = numpy.sum(x292 * x519 * x592)
    result[3, 9, 1] = numpy.sum(x293 * x406 * x522)
    result[3, 9, 2] = numpy.sum(x294 * x518 * x596)
    result[3, 9, 3] = numpy.sum(x292 * x526 * x600)
    result[3, 9, 4] = numpy.sum(x295 * x404 * x528)
    result[3, 9, 5] = numpy.sum(x297 * x518 * x600)
    result[3, 9, 6] = numpy.sum(x15 * x293 * x529)
    result[3, 9, 7] = numpy.sum(x295 * x526 * x610)
    result[3, 9, 8] = numpy.sum(x297 * x528 * x604)
    result[3, 9, 9] = numpy.sum(x15 * x428 * x518)
    result[3, 9, 10] = numpy.sum(x292 * x534 * x615)
    result[3, 9, 11] = numpy.sum(x295 * x529 * x7)
    result[3, 9, 12] = numpy.sum(x298 * x527 * x7)
    result[3, 9, 13] = numpy.sum(x299 * x522 * x611)
    result[3, 9, 14] = numpy.sum(x300 * x520 * x7)
    result[4, 0, 0] = numpy.sum(x301 * x431 * x516 * x95)
    result[4, 0, 1] = numpy.sum(x309 * x434 * x616)
    result[4, 0, 2] = numpy.sum(x312 * x433 * x616)
    result[4, 0, 3] = numpy.sum(x316 * x434 * x617)
    result[4, 0, 4] = numpy.sum(x318 * x433 * x523)
    result[4, 0, 5] = numpy.sum(x312 * x440 * x523)
    result[4, 0, 6] = numpy.sum(x322 * x434 * x531)
    result[4, 0, 7] = numpy.sum(x315 * x435 * x531)
    result[4, 0, 8] = numpy.sum(x318 * x438 * x530)
    result[4, 0, 9] = numpy.sum(x312 * x445 * x531)
    result[4, 0, 10] = numpy.sum(x333 * x434 * x618)
    result[4, 0, 11] = numpy.sum(x322 * x433 * x535)
    result[4, 0, 12] = numpy.sum(x316 * x439 * x533)
    result[4, 0, 13] = numpy.sum(x309 * x445 * x535)
    result[4, 0, 14] = numpy.sum(x312 * x448 * x618)
    result[4, 1, 0] = numpy.sum(x337 * x434 * x619)
    result[4, 1, 1] = numpy.sum(x340 * x46 * x473)
    result[4, 1, 2] = numpy.sum(x341 * x435 * x46)
    result[4, 1, 3] = numpy.sum(x345 * x434 * x558)
    result[4, 1, 4] = numpy.sum(x340 * x44 * x451)
    result[4, 1, 5] = numpy.sum(x348 * x44 * x452)
    result[4, 1, 6] = numpy.sum(x324 * x354 * x473)
    result[4, 1, 7] = numpy.sum(x324 * x344 * x451)
    result[4, 1, 8] = numpy.sum(x340 * x438 * x552)
    result[4, 1, 9] = numpy.sum(x324 * x341 * x454)
    result[4, 1, 10] = numpy.sum(x357 * x434 * x555)
    result[4, 1, 11] = numpy.sum(x354 * x435 * x556)
    result[4, 1, 12] = numpy.sum(x345 * x452 * x532)
    result[4, 1, 13] = numpy.sum(x340 * x454 * x556)
    result[4, 1, 14] = numpy.sum(x337 * x448 * x555)
    result[4, 2, 0] = numpy.sum(x312 * x457 * x619)
    result[4, 2, 1] = numpy.sum(x318 * x457 * x46)
    result[4, 2, 2] = numpy.sum(x400 * x46 * x461)
    result[4, 2, 3] = numpy.sum(x361 * x44 * x500)
    result[4, 2, 4] = numpy.sum(x309 * x44 * x462)
    result[4, 2, 5] = numpy.sum(x312 * x466 * x558)
    result[4, 2, 6] = numpy.sum(x324 * x363 * x459)
    result[4, 2, 7] = numpy.sum(x315 * x324 * x462)
    result[4, 2, 8] = numpy.sum(x309 * x465 * x552)
    result[4, 2, 9] = numpy.sum(x324 * x400 * x471)
    result[4, 2, 10] = numpy.sum(x333 * x457 * x555)
    result[4, 2, 11] = numpy.sum(x363 * x461 * x556)
    result[4, 2, 12] = numpy.sum(x361 * x466 * x532)
    result[4, 2, 13] = numpy.sum(x318 * x471 * x532)
    result[4, 2, 14] = numpy.sum(x312 * x472 * x555)
    result[4, 3, 0] = numpy.sum(x434 * x562 * x620)
    result[4, 3, 1] = numpy.sum(x374 * x453 * x563)
    result[4, 3, 2] = numpy.sum(x370 * x375 * x435)
    result[4, 3, 3] = numpy.sum(x377 * x434 * x570)
    result[4, 3, 4] = numpy.sum(x32 * x374 * x451)
    result[4, 3, 5] = numpy.sum(x368 * x438 * x570)
    result[4, 3, 6] = numpy.sum(x385 * x450 * x576)
    result[4, 3, 7] = numpy.sum(x30 * x377 * x451)
    result[4, 3, 8] = numpy.sum(x374 * x438 * x577)
    result[4, 3, 9] = numpy.sum(x30 * x375 * x454)
    result[4, 3, 10] = numpy.sum(x424 * x431 * x580)
    result[4, 3, 11] = numpy.sum(x385 * x435 * x590)
    result[4, 3, 12] = numpy.sum(x377 * x452 * x583)
    result[4, 3, 13] = numpy.sum(x374 * x454 * x590)
    result[4, 3, 14] = numpy.sum(x11 * x448 * x620)
    result[4, 4, 0] = numpy.sum(x393 * x476 * x562)
    result[4, 4, 1] = numpy.sum(x340 * x478 * x563)
    result[4, 4, 2] = numpy.sum(x395 * x461 * x563)
    result[4, 4, 3] = numpy.sum(x344 * x457 * x571)
    result[4, 4, 4] = numpy.sum(x340 * x462 * x569)
    result[4, 4, 5] = numpy.sum(x337 * x465 * x571)
    result[4, 4, 6] = numpy.sum(x354 * x478 * x587)
    result[4, 4, 7] = numpy.sum(x30 * x345 * x462)
    result[4, 4, 8] = numpy.sum(x340 * x466 * x577)
    result[4, 4, 9] = numpy.sum(x395 * x471 * x587)
    result[4, 4, 10] = numpy.sum(x11 * x357 * x477)
    result[4, 4, 11] = numpy.sum(x354 * x461 * x621)
    result[4, 4, 12] = numpy.sum(x11 * x344 * x622)
    result[4, 4, 13] = numpy.sum(x340 * x471 * x621)
    result[4, 4, 14] = numpy.sum(x393 * x472 * x623)
    result[4, 5, 0] = numpy.sum(x312 * x502 * x562)
    result[4, 5, 1] = numpy.sum(x318 * x370 * x482)
    result[4, 5, 2] = numpy.sum(x364 * x485 * x563)
    result[4, 5, 3] = numpy.sum(x361 * x482 * x569)
    result[4, 5, 4] = numpy.sum(x309 * x32 * x486)
    result[4, 5, 5] = numpy.sum(x312 * x488 * x570)
    result[4, 5, 6] = numpy.sum(x30 * x363 * x483)
    result[4, 5, 7] = numpy.sum(x30 * x315 * x486)
    result[4, 5, 8] = numpy.sum(x309 * x488 * x577)
    result[4, 5, 9] = numpy.sum(x312 * x503 * x576)
    result[4, 5, 10] = numpy.sum(x11 * x333 * x502)
    result[4, 5, 11] = numpy.sum(x363 * x485 * x590)
    result[4, 5, 12] = numpy.sum(x361 * x488 * x583)
    result[4, 5, 13] = numpy.sum(x11 * x318 * x494)
    result[4, 5, 14] = numpy.sum(x301 * x504 * x580)
    result[4, 6, 0] = numpy.sum(x403 * x434 * x624)
    result[4, 6, 1] = numpy.sum(x408 * x434 * x596)
    result[4, 6, 2] = numpy.sum(x403 * x433 * x596)
    result[4, 6, 3] = numpy.sum(x411 * x434 * x600)
    result[4, 6, 4] = numpy.sum(x404 * x409 * x435)
    result[4, 6, 5] = numpy.sum(x403 * x439 * x599)
    result[4, 6, 6] = numpy.sum(x421 * x431 * x603)
    result[4, 6, 7] = numpy.sum(x411 * x435 * x604)
    result[4, 6, 8] = numpy.sum(x409 * x438 * x610)
    result[4, 6, 9] = numpy.sum(x403 * x445 * x604)
    result[4, 6, 10] = numpy.sum(x420 * x431)
    result[4, 6, 11] = numpy.sum(x415 * x433 * x611)
    result[4, 6, 12] = numpy.sum(x411 * x440 * x7)
    result[4, 6, 13] = numpy.sum(x409 * x445 * x7)
    result[4, 6, 14] = numpy.sum(x403 * x448 * x615)
    result[4, 7, 0] = numpy.sum(x368 * x457 * x606)
    result[4, 7, 1] = numpy.sum(x374 * x457 * x607)
    result[4, 7, 2] = numpy.sum(x368 * x461 * x607)
    result[4, 7, 3] = numpy.sum(x377 * x500 * x608)
    result[4, 7, 4] = numpy.sum(x374 * x404 * x462)
    result[4, 7, 5] = numpy.sum(x368 * x466 * x608)
    result[4, 7, 6] = numpy.sum(x385 * x459 * x610)
    result[4, 7, 7] = numpy.sum(x15 * x377 * x462)
    result[4, 7, 8] = numpy.sum(x15 * x374 * x622)
    result[4, 7, 9] = numpy.sum(x375 * x471 * x610)
    result[4, 7, 10] = numpy.sum(x389 * x457 * x614)
    result[4, 7, 11] = numpy.sum(x385 * x461 * x612)
    result[4, 7, 12] = numpy.sum(x377 * x466 * x613)
    result[4, 7, 13] = numpy.sum(x374 * x471 * x612)
    result[4, 7, 14] = numpy.sum(x368 * x472 * x614)
    result[4, 8, 0] = numpy.sum(x337 * x482 * x606)
    result[4, 8, 1] = numpy.sum(x340 * x482 * x607)
    result[4, 8, 2] = numpy.sum(x337 * x485 * x607)
    result[4, 8, 3] = numpy.sum(x345 * x482 * x608)
    result[4, 8, 4] = numpy.sum(x340 * x404 * x486)
    result[4, 8, 5] = numpy.sum(x348 * x488 * x608)
    result[4, 8, 6] = numpy.sum(x354 * x483 * x610)
    result[4, 8, 7] = numpy.sum(x15 * x344 * x486)
    result[4, 8, 8] = numpy.sum(x340 * x488 * x625)
    result[4, 8, 9] = numpy.sum(x341 * x494 * x610)
    result[4, 8, 10] = numpy.sum(x357 * x482 * x614)
    result[4, 8, 11] = numpy.sum(x354 * x485 * x612)
    result[4, 8, 12] = numpy.sum(x345 * x488 * x613)
    result[4, 8, 13] = numpy.sum(x340 * x494 * x612)
    result[4, 8, 14] = numpy.sum(x337 * x497 * x614)
    result[4, 9, 0] = numpy.sum(x312 * x506 * x624)
    result[4, 9, 1] = numpy.sum(x309 * x506 * x596)
    result[4, 9, 2] = numpy.sum(x312 * x507 * x596)
    result[4, 9, 3] = numpy.sum(x315 * x506 * x600)
    result[4, 9, 4] = numpy.sum(x318 * x404 * x507)
    result[4, 9, 5] = numpy.sum(x312 * x510 * x600)
    result[4, 9, 6] = numpy.sum(x322 * x506 * x604)
    result[4, 9, 7] = numpy.sum(x315 * x508 * x604)
    result[4, 9, 8] = numpy.sum(x15 * x318 * x510)
    result[4, 9, 9] = numpy.sum(x301 * x513 * x603)
    result[4, 9, 10] = numpy.sum(x333 * x506 * x615)
    result[4, 9, 11] = numpy.sum(x322 * x507 * x611)
    result[4, 9, 12] = numpy.sum(x316 * x510 * x626)
    result[4, 9, 13] = numpy.sum(x309 * x512 * x611)
    result[4, 9, 14] = numpy.sum(x301 * x514)
    result[5, 0, 0] = numpy.sum(x108 * x516 * x630)
    result[5, 0, 1] = numpy.sum(x132 * x305 * x631)
    result[5, 0, 2] = numpy.sum(x249 * x305 * x633)
    result[5, 0, 3] = numpy.sum(x113 * x523 * x628)
    result[5, 0, 4] = numpy.sum(x133 * x523 * x634)
    result[5, 0, 5] = numpy.sum(x108 * x617 * x638)
    result[5, 0, 6] = numpy.sum(x128 * x530 * x628)
    result[5, 0, 7] = numpy.sum(x111 * x531 * x634)
    result[5, 0, 8] = numpy.sum(x441 * x531 * x637)
    result[5, 0, 9] = numpy.sum(x108 * x531 * x639)
    result[5, 0, 10] = numpy.sum(x139 * x533 * x629)
    result[5, 0, 11] = numpy.sum(x128 * x533 * x633)
    result[5, 0, 12] = numpy.sum(x113 * x533 * x637)
    result[5, 0, 13] = numpy.sum(x132 * x535 * x639)
    result[5, 0, 14] = numpy.sum(x449 * x533 * x640)
    result[5, 1, 0] = numpy.sum(x150 * x619 * x628)
    result[5, 1, 1] = numpy.sum(x153 * x46 * x631)
    result[5, 1, 2] = numpy.sum(x287 * x46 * x634)
    result[5, 1, 3] = numpy.sum(x161 * x44 * x628)
    result[5, 1, 4] = numpy.sum(x163 * x44 * x633)
    result[5, 1, 5] = numpy.sum(x164 * x558 * x637)
    result[5, 1, 6] = numpy.sum(x170 * x324 * x641)
    result[5, 1, 7] = numpy.sum(x236 * x324 * x633)
    result[5, 1, 8] = numpy.sum(x152 * x552 * x637)
    result[5, 1, 9] = numpy.sum(x287 * x324 * x642)
    result[5, 1, 10] = numpy.sum(x174 * x555 * x628)
    result[5, 1, 11] = numpy.sum(x170 * x532 * x634)
    result[5, 1, 12] = numpy.sum(x161 * x532 * x637)
    result[5, 1, 13] = numpy.sum(x153 * x556 * x639)
    result[5, 1, 14] = numpy.sum(x150 * x555 * x640)
    result[5, 2, 0] = numpy.sum(x458 * x538 * x644)
    result[5, 2, 1] = numpy.sum(x46 * x644 * x645)
    result[5, 2, 2] = numpy.sum(x250 * x46 * x647)
    result[5, 2, 3] = numpy.sum(x187 * x44 * x644)
    result[5, 2, 4] = numpy.sum(x132 * x162 * x44 * x647)
    result[5, 2, 5] = numpy.sum(x254 * x44 * x651)
    result[5, 2, 6] = numpy.sum(x195 * x324 * x644)
    result[5, 2, 7] = numpy.sum(x111 * x552 * x647)
    result[5, 2, 8] = numpy.sum(x132 * x552 * x650)
    result[5, 2, 9] = numpy.sum(x250 * x324 * x655)
    result[5, 2, 10] = numpy.sum(x201 * x532 * x644)
    result[5, 2, 11] = numpy.sum(x195 * x532 * x647)
    result[5, 2, 12] = numpy.sum(x187 * x532 * x650)
    result[5, 2, 13] = numpy.sum(x441 * x556 * x655)
    result[5, 2, 14] = numpy.sum(x108 * x555 * x657)
    result[5, 3, 0] = numpy.sum(x209 * x562 * x628)
    result[5, 3, 1] = numpy.sum(x211 * x563 * x641)
    result[5, 3, 2] = numpy.sum(x213 * x370 * x634)
    result[5, 3, 3] = numpy.sum(x218 * x628 * x658)
    result[5, 3, 4] = numpy.sum(x211 * x571 * x633)
    result[5, 3, 5] = numpy.sum(x221 * x637 * x658)
    result[5, 3, 6] = numpy.sum(x224 * x576 * x631)
    result[5, 3, 7] = numpy.sum(x217 * x577 * x633)
    result[5, 3, 8] = numpy.sum(x211 * x577 * x637)
    result[5, 3, 9] = numpy.sum(x213 * x576 * x639)
    result[5, 3, 10] = numpy.sum(x11 * x475 * x628)
    result[5, 3, 11] = numpy.sum(x224 * x590 * x634)
    result[5, 3, 12] = numpy.sum(x11 * x284 * x637)
    result[5, 3, 13] = numpy.sum(x211 * x590 * x642)
    result[5, 3, 14] = numpy.sum(x11 * x209 * x640)
    result[5, 4, 0] = numpy.sum(x291 * x562 * x659)
    result[5, 4, 1] = numpy.sum(x234 * x370 * x660)
    result[5, 4, 2] = numpy.sum(x563 * x647 * x661)
    result[5, 4, 3] = numpy.sum(x236 * x32 * x644)
    result[5, 4, 4] = numpy.sum(x152 * x571 * x662)
    result[5, 4, 5] = numpy.sum(x150 * x571 * x650)
    result[5, 4, 6] = numpy.sum(x170 * x30 * x660)
    result[5, 4, 7] = numpy.sum(x236 * x30 * x662)
    result[5, 4, 8] = numpy.sum(x152 * x577 * x651)
    result[5, 4, 9] = numpy.sum(x587 * x655 * x661)
    result[5, 4, 10] = numpy.sum(x174 * x623 * x659)
    result[5, 4, 11] = numpy.sum(x170 * x588 * x647)
    result[5, 4, 12] = numpy.sum(x11 * x236 * x650)
    result[5, 4, 13] = numpy.sum(x234 * x588 * x655)
    result[5, 4, 14] = numpy.sum(x145 * x232 * x657 * x663 * x89)
    result[5, 5, 0] = numpy.sum(x458 * x562 * x664)
    result[5, 5, 1] = numpy.sum(x441 * x563 * x664)
    result[5, 5, 2] = numpy.sum(x108 * x563 * x667)
    result[5, 5, 3] = numpy.sum(x187 * x32 * x664)
    result[5, 5, 4] = numpy.sum(x132 * x571 * x666)
    result[5, 5, 5] = numpy.sum(x108 * x570 * x669)
    result[5, 5, 6] = numpy.sum(x195 * x30 * x664)
    result[5, 5, 7] = numpy.sum(x111 * x577 * x666)
    result[5, 5, 8] = numpy.sum(x132 * x577 * x669)
    result[5, 5, 9] = numpy.sum(x249 * x576 * x672)
    result[5, 5, 10] = numpy.sum(x11 * x201 * x664)
    result[5, 5, 11] = numpy.sum(x11 * x195 * x666)
    result[5, 5, 12] = numpy.sum(x11 * x187 * x669)
    result[5, 5, 13] = numpy.sum(x663 * x672 * x673)
    result[5, 5, 14] = numpy.sum(x580 * x675)
    result[5, 6, 0] = numpy.sum(x260 * x592 * x629)
    result[5, 6, 1] = numpy.sum(x266 * x596 * x628)
    result[5, 6, 2] = numpy.sum(x268 * x406 * x633)
    result[5, 6, 3] = numpy.sum(x270 * x600 * x628)
    result[5, 6, 4] = numpy.sum(x267 * x404 * x634)
    result[5, 6, 5] = numpy.sum(x260 * x600 * x637)
    result[5, 6, 6] = numpy.sum(x15 * x273 * x631)
    result[5, 6, 7] = numpy.sum(x270 * x604 * x634)
    result[5, 6, 8] = numpy.sum(x267 * x610 * x637)
    result[5, 6, 9] = numpy.sum(x15 * x268 * x639)
    result[5, 6, 10] = numpy.sum(x276 * x630 * x7)
    result[5, 6, 11] = numpy.sum(x273 * x611 * x633)
    result[5, 6, 12] = numpy.sum(x270 * x626 * x638)
    result[5, 6, 13] = numpy.sum(x267 * x639 * x7)
    result[5, 6, 14] = numpy.sum(x260 * x615 * x640)
    result[5, 7, 0] = numpy.sum(x209 * x591 * x644)
    result[5, 7, 1] = numpy.sum(x211 * x607 * x644)
    result[5, 7, 2] = numpy.sum(x208 * x607 * x647)
    result[5, 7, 3] = numpy.sum(x283 * x644 * x676)
    result[5, 7, 4] = numpy.sum(x220 * x404 * x647)
    result[5, 7, 5] = numpy.sum(x221 * x651 * x676)
    result[5, 7, 6] = numpy.sum(x224 * x605 * x644)
    result[5, 7, 7] = numpy.sum(x217 * x625 * x647)
    result[5, 7, 8] = numpy.sum(x15 * x220 * x650)
    result[5, 7, 9] = numpy.sum(x213 * x610 * x655)
    result[5, 7, 10] = numpy.sum(x227 * x614 * x644)
    result[5, 7, 11] = numpy.sum(x224 * x612 * x647)
    result[5, 7, 12] = numpy.sum(x218 * x626 * x651)
    result[5, 7, 13] = numpy.sum(x211 * x612 * x655)
    result[5, 7, 14] = numpy.sum(x209 * x657 * x7)
    result[5, 8, 0] = numpy.sum(x150 * x606 * x664)
    result[5, 8, 1] = numpy.sum(x153 * x596 * x664)
    result[5, 8, 2] = numpy.sum(x150 * x607 * x666)
    result[5, 8, 3] = numpy.sum(x161 * x404 * x664)
    result[5, 8, 4] = numpy.sum(x163 * x404 * x666)
    result[5, 8, 5] = numpy.sum(x164 * x608 * x669)
    result[5, 8, 6] = numpy.sum(x170 * x610 * x664)
    result[5, 8, 7] = numpy.sum(x15 * x236 * x666)
    result[5, 8, 8] = numpy.sum(x15 * x163 * x669)
    result[5, 8, 9] = numpy.sum(x145 * x429 * x609 * x672)
    result[5, 8, 10] = numpy.sum(x174 * x614 * x664)
    result[5, 8, 11] = numpy.sum(x170 * x667 * x7)
    result[5, 8, 12] = numpy.sum(x161 * x669 * x7)
    result[5, 8, 13] = numpy.sum(x153 * x611 * x672)
    result[5, 8, 14] = numpy.sum(x148 * x675)
    result[5, 9, 0] = numpy.sum(x108 * x592 * x678)
    result[5, 9, 1] = numpy.sum(x132 * x596 * x677)
    result[5, 9, 2] = numpy.sum(x108 * x596 * x679)
    result[5, 9, 3] = numpy.sum(x113 * x404 * x677)
    result[5, 9, 4] = numpy.sum(x404 * x645 * x679)
    result[5, 9, 5] = numpy.sum(x108 * x600 * x681)
    result[5, 9, 6] = numpy.sum(x128 * x15 * x677)
    result[5, 9, 7] = numpy.sum(x111 * x605 * x679)
    result[5, 9, 8] = numpy.sum(x3 * x429 * x673 * x681)
    result[5, 9, 9] = numpy.sum(x603 * x683)
    result[5, 9, 10] = numpy.sum(x139 * x678 * x7)
    result[5, 9, 11] = numpy.sum(x128 * x679 * x7)
    result[5, 9, 12] = numpy.sum(x113 * x681 * x7)
    result[5, 9, 13] = numpy.sum(x683 * x99)
    result[5, 9, 14] = numpy.sum(
        x430
        * (
            x0 * (2.0 * x496 + 2.0 * x511 + 6.0 * x670 + 3.0 * x671 + 3.0 * x680)
            + x102 * x682
        )
    )
    return result


def quadrupole3d_40(ax, da, A, bx, db, B, R):
    """Cartesian 3D (gs) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 15, 1), dtype=float)

    x0 = 0.5 / (ax + bx)
    x1 = (ax + bx) ** (-1.0)
    x2 = -x1 * (ax * A[0] + bx * B[0])
    x3 = -x2 - A[0]
    x4 = ax * bx * x1
    x5 = numpy.exp(-x4 * (A[0] - B[0]) ** 2)
    x6 = 1.772453850905516 * numpy.sqrt(x1)
    x7 = x5 * x6
    x8 = x3**2 * x7
    x9 = x0 * x7
    x10 = 3.0 * x9
    x11 = -x2 - R[0]
    x12 = x3 * x7
    x13 = x11 * x12
    x14 = x10 + 2.0 * x13
    x15 = x0 * (x14 + x8)
    x16 = x11 * x7
    x17 = x0 * (x12 + x16)
    x18 = x13 + x9
    x19 = x18 * x3
    x20 = x17 + x19
    x21 = x20 * x3
    x22 = x11**2 * x7
    x23 = x0 * (x14 + x22)
    x24 = x11 * x18
    x25 = x17 + x24
    x26 = x25 * x3
    x27 = x23 + x26
    x28 = 2.0 * x0 * (2.0 * x17 + x19 + x24) + x27 * x3
    x29 = da * db
    x30 = 0.09759000729485332 * x29
    x31 = numpy.exp(-x4 * (A[1] - B[1]) ** 2)
    x32 = numpy.exp(-x4 * (A[2] - B[2]) ** 2)
    x33 = 3.141592653589793 * x1 * x32
    x34 = x31 * x33
    x35 = x30 * x34
    x36 = -x1 * (ax * A[1] + bx * B[1])
    x37 = -x36 - A[1]
    x38 = 0.2581988897471611 * x29
    x39 = x37 * x38
    x40 = x28 * x34
    x41 = -x1 * (ax * A[2] + bx * B[2])
    x42 = -x41 - A[2]
    x43 = x38 * x42
    x44 = x32 * x6
    x45 = x31 * x6
    x46 = x37**2 * x45
    x47 = x0 * x45
    x48 = x46 + x47
    x49 = 0.3333333333333333 * x29
    x50 = x48 * x49
    x51 = 1.732050807568877
    x52 = x42 * x49 * x51
    x53 = x42**2 * x44
    x54 = x0 * x44
    x55 = x53 + x54
    x56 = x49 * x55
    x57 = x37 * (2.0 * x47 + x48)
    x58 = x25 * x38
    x59 = x42 * x44
    x60 = x25 * x51
    x61 = x37 * x45
    x62 = x42 * (2.0 * x54 + x55)
    x63 = 3.0 * x47
    x64 = x0 * (3.0 * x46 + x63) + x37 * x57
    x65 = x22 + x9
    x66 = x30 * x65
    x67 = x38 * x65
    x68 = 3.0 * x54
    x69 = x0 * (3.0 * x53 + x68) + x42 * x62
    x70 = -x36 - R[1]
    x71 = x8 + x9
    x72 = 2.0 * x0 * x12 + x3 * x71
    x73 = x15 + x21
    x74 = x35 * (x0 * (3.0 * x17 + 3.0 * x19 + x72) + x3 * x73)
    x75 = x61 * x70
    x76 = x47 + x75
    x77 = x38 * x44
    x78 = x34 * x73
    x79 = x45 * x70
    x80 = x0 * (x61 + x79)
    x81 = x37 * x76
    x82 = x80 + x81
    x83 = x20 * x49
    x84 = x51 * x76
    x85 = x63 + 2.0 * x75
    x86 = x0 * (x46 + x85)
    x87 = x37 * x82
    x88 = x86 + x87
    x89 = x18 * x51
    x90 = x49 * x82
    x91 = x38 * x62
    x92 = x30 * x5
    x93 = x33 * x92
    x94 = x93 * (x0 * (x57 + 3.0 * x80 + 3.0 * x81) + x37 * x88)
    x95 = x33 * x5
    x96 = x88 * x95
    x97 = 3.141592653589793 * x1 * x31
    x98 = x92 * x97
    x99 = -x41 - R[2]
    x100 = x59 * x99
    x101 = x100 + x54
    x102 = x38 * x45
    x103 = x44 * x99
    x104 = x101 * x51
    x105 = x0 * (x103 + x59)
    x106 = x101 * x42
    x107 = x105 + x106
    x108 = x38 * x57
    x109 = x107 * x49
    x110 = 2.0 * x100 + x68
    x111 = x0 * (x110 + x53)
    x112 = x107 * x42
    x113 = x111 + x112
    x114 = x5 * x97
    x115 = x113 * x114
    x116 = x98 * (x0 * (3.0 * x105 + 3.0 * x106 + x62) + x113 * x42)
    x117 = x0 * (x10 + 3.0 * x8) + x3 * x72
    x118 = x45 * x70**2
    x119 = x118 + x47
    x120 = x119 * x30
    x121 = x70 * x76
    x122 = x121 + x80
    x123 = x38 * x72
    x124 = x0 * (x118 + x85)
    x125 = x122 * x37
    x126 = x124 + x125
    x127 = x49 * x71
    x128 = x122 * x51
    x129 = 2.0 * x0 * (x121 + 2.0 * x80 + x81) + x126 * x37
    x130 = x129 * x95
    x131 = x3 * x38
    x132 = x38 * x7
    x133 = x44 * x99**2
    x134 = x133 + x54
    x135 = x134 * x30
    x136 = x101 * x99
    x137 = x105 + x136
    x138 = x137 * x51
    x139 = x0 * (x110 + x133)
    x140 = x137 * x42
    x141 = x139 + x140
    x142 = 2.0 * x0 * (2.0 * x105 + x106 + x136) + x141 * x42
    x143 = x114 * x142

    # 90 item(s)
    result[0, 0, 0] = numpy.sum(
        x35 * (x0 * (2.0 * x15 + 2.0 * x21 + 3.0 * x23 + 3.0 * x26) + x28 * x3)
    )
    result[0, 1, 0] = numpy.sum(x39 * x40)
    result[0, 2, 0] = numpy.sum(x40 * x43)
    result[0, 3, 0] = numpy.sum(x27 * x44 * x50)
    result[0, 4, 0] = numpy.sum(x27 * x34 * x37 * x52)
    result[0, 5, 0] = numpy.sum(x27 * x45 * x56)
    result[0, 6, 0] = numpy.sum(x44 * x57 * x58)
    result[0, 7, 0] = numpy.sum(x50 * x59 * x60)
    result[0, 8, 0] = numpy.sum(x56 * x60 * x61)
    result[0, 9, 0] = numpy.sum(x45 * x58 * x62)
    result[0, 10, 0] = numpy.sum(x44 * x64 * x66)
    result[0, 11, 0] = numpy.sum(x57 * x59 * x67)
    result[0, 12, 0] = numpy.sum(x48 * x56 * x65)
    result[0, 13, 0] = numpy.sum(x61 * x62 * x67)
    result[0, 14, 0] = numpy.sum(x45 * x66 * x69)
    result[1, 0, 0] = numpy.sum(x70 * x74)
    result[1, 1, 0] = numpy.sum(x73 * x76 * x77)
    result[1, 2, 0] = numpy.sum(x43 * x70 * x78)
    result[1, 3, 0] = numpy.sum(x44 * x82 * x83)
    result[1, 4, 0] = numpy.sum(x59 * x83 * x84)
    result[1, 5, 0] = numpy.sum(x20 * x56 * x79)
    result[1, 6, 0] = numpy.sum(x18 * x77 * x88)
    result[1, 7, 0] = numpy.sum(x59 * x89 * x90)
    result[1, 8, 0] = numpy.sum(x56 * x76 * x89)
    result[1, 9, 0] = numpy.sum(x18 * x79 * x91)
    result[1, 10, 0] = numpy.sum(x11 * x94)
    result[1, 11, 0] = numpy.sum(x11 * x43 * x96)
    result[1, 12, 0] = numpy.sum(x16 * x56 * x82)
    result[1, 13, 0] = numpy.sum(x16 * x76 * x91)
    result[1, 14, 0] = numpy.sum(x11 * x69 * x70 * x98)
    result[2, 0, 0] = numpy.sum(x74 * x99)
    result[2, 1, 0] = numpy.sum(x39 * x78 * x99)
    result[2, 2, 0] = numpy.sum(x101 * x102 * x73)
    result[2, 3, 0] = numpy.sum(x103 * x20 * x50)
    result[2, 4, 0] = numpy.sum(x104 * x61 * x83)
    result[2, 5, 0] = numpy.sum(x107 * x45 * x83)
    result[2, 6, 0] = numpy.sum(x103 * x108 * x18)
    result[2, 7, 0] = numpy.sum(x101 * x50 * x89)
    result[2, 8, 0] = numpy.sum(x109 * x61 * x89)
    result[2, 9, 0] = numpy.sum(x102 * x113 * x18)
    result[2, 10, 0] = numpy.sum(x11 * x64 * x93 * x99)
    result[2, 11, 0] = numpy.sum(x101 * x108 * x16)
    result[2, 12, 0] = numpy.sum(x107 * x16 * x50)
    result[2, 13, 0] = numpy.sum(x11 * x115 * x39)
    result[2, 14, 0] = numpy.sum(x11 * x116)
    result[3, 0, 0] = numpy.sum(x117 * x120 * x44)
    result[3, 1, 0] = numpy.sum(x122 * x72 * x77)
    result[3, 2, 0] = numpy.sum(x119 * x123 * x59)
    result[3, 3, 0] = numpy.sum(x126 * x127 * x44)
    result[3, 4, 0] = numpy.sum(x127 * x128 * x59)
    result[3, 5, 0] = numpy.sum(x119 * x56 * x71)
    result[3, 6, 0] = numpy.sum(x130 * x131)
    result[3, 7, 0] = numpy.sum(x126 * x3 * x52 * x95)
    result[3, 8, 0] = numpy.sum(x12 * x128 * x56)
    result[3, 9, 0] = numpy.sum(x119 * x12 * x91)
    result[3, 10, 0] = numpy.sum(
        x93 * (x0 * (3.0 * x124 + 3.0 * x125 + 2.0 * x86 + 2.0 * x87) + x129 * x37)
    )
    result[3, 11, 0] = numpy.sum(x130 * x43)
    result[3, 12, 0] = numpy.sum(x126 * x56 * x7)
    result[3, 13, 0] = numpy.sum(x122 * x132 * x62)
    result[3, 14, 0] = numpy.sum(x120 * x69 * x7)
    result[4, 0, 0] = numpy.sum(x117 * x35 * x70 * x99)
    result[4, 1, 0] = numpy.sum(x103 * x123 * x76)
    result[4, 2, 0] = numpy.sum(x101 * x123 * x79)
    result[4, 3, 0] = numpy.sum(x103 * x127 * x82)
    result[4, 4, 0] = numpy.sum(x101 * x127 * x84)
    result[4, 5, 0] = numpy.sum(x107 * x127 * x79)
    result[4, 6, 0] = numpy.sum(x131 * x96 * x99)
    result[4, 7, 0] = numpy.sum(x104 * x12 * x90)
    result[4, 8, 0] = numpy.sum(x109 * x12 * x84)
    result[4, 9, 0] = numpy.sum(x115 * x131 * x70)
    result[4, 10, 0] = numpy.sum(x94 * x99)
    result[4, 11, 0] = numpy.sum(x101 * x132 * x88)
    result[4, 12, 0] = numpy.sum(x107 * x7 * x90)
    result[4, 13, 0] = numpy.sum(x113 * x132 * x76)
    result[4, 14, 0] = numpy.sum(x116 * x70)
    result[5, 0, 0] = numpy.sum(x117 * x135 * x45)
    result[5, 1, 0] = numpy.sum(x123 * x134 * x61)
    result[5, 2, 0] = numpy.sum(x102 * x137 * x72)
    result[5, 3, 0] = numpy.sum(x134 * x50 * x71)
    result[5, 4, 0] = numpy.sum(x127 * x138 * x61)
    result[5, 5, 0] = numpy.sum(x127 * x141 * x45)
    result[5, 6, 0] = numpy.sum(x108 * x12 * x134)
    result[5, 7, 0] = numpy.sum(x12 * x138 * x50)
    result[5, 8, 0] = numpy.sum(x114 * x141 * x3 * x37 * x49 * x51)
    result[5, 9, 0] = numpy.sum(x131 * x143)
    result[5, 10, 0] = numpy.sum(x135 * x64 * x7)
    result[5, 11, 0] = numpy.sum(x132 * x137 * x57)
    result[5, 12, 0] = numpy.sum(x141 * x50 * x7)
    result[5, 13, 0] = numpy.sum(x143 * x39)
    result[5, 14, 0] = numpy.sum(
        x98 * (x0 * (2.0 * x111 + 2.0 * x112 + 3.0 * x139 + 3.0 * x140) + x142 * x42)
    )
    return result


def quadrupole3d_41(ax, da, A, bx, db, B, R):
    """Cartesian 3D (gp) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 15, 3), dtype=float)

    x0 = 0.5 / (ax + bx)
    x1 = (ax + bx) ** (-1.0)
    x2 = -x1 * (ax * A[0] + bx * B[0])
    x3 = -x2 - A[0]
    x4 = ax * bx * x1
    x5 = numpy.exp(-x4 * (A[0] - B[0]) ** 2)
    x6 = 1.772453850905516 * numpy.sqrt(x1)
    x7 = x5 * x6
    x8 = x0 * x7
    x9 = -x2 - B[0]
    x10 = -x2 - R[0]
    x11 = x10 * x7
    x12 = x11 * x9
    x13 = x12 + x8
    x14 = x13 * x3
    x15 = 2.0 * x14
    x16 = x7 * x9
    x17 = x0 * (x11 + x16)
    x18 = x3 * x7
    x19 = x0 * (x11 + x18)
    x20 = x10 * x18
    x21 = x20 + x8
    x22 = x21 * x3
    x23 = x19 + x22
    x24 = x0 * (x16 + x18)
    x25 = x18 * x9
    x26 = x25 + x8
    x27 = x26 * x3
    x28 = x24 + x27
    x29 = x0 * (x15 + 2.0 * x17 + x23 + x28)
    x30 = 3.0 * x8
    x31 = x0 * (x12 + x20 + x25 + x30)
    x32 = x14 + x17
    x33 = x3 * x32
    x34 = x31 + x33
    x35 = x3 * x34
    x36 = x10 * x13
    x37 = x10 * x21
    x38 = x19 + x37
    x39 = x0 * (x15 + 3.0 * x17 + x36 + x38)
    x40 = x10 * x32
    x41 = x31 + x40
    x42 = x3 * x41
    x43 = x10**2 * x7
    x44 = 2.0 * x20 + x30
    x45 = x0 * (x43 + x44)
    x46 = x3 * x38
    x47 = x45 + x46
    x48 = 2.0 * x0 * (2.0 * x19 + x22 + x37) + x3 * x47
    x49 = x39 + x42
    x50 = x0 * (4.0 * x31 + 2.0 * x33 + 2.0 * x40 + x47) + x3 * x49
    x51 = da * db
    x52 = 0.09759000729485332
    x53 = x51 * x52
    x54 = numpy.exp(-x4 * (A[1] - B[1]) ** 2)
    x55 = numpy.exp(-x4 * (A[2] - B[2]) ** 2)
    x56 = 3.141592653589793 * x1 * x55
    x57 = x54 * x56
    x58 = x53 * x57
    x59 = -x1 * (ax * A[1] + bx * B[1])
    x60 = -x59 - B[1]
    x61 = x3**2 * x7
    x62 = x0 * (x44 + x61)
    x63 = x23 * x3
    x64 = x58 * (x0 * (3.0 * x45 + 3.0 * x46 + 2.0 * x62 + 2.0 * x63) + x3 * x48)
    x65 = -x1 * (ax * A[2] + bx * B[2])
    x66 = -x65 - B[2]
    x67 = -x59 - A[1]
    x68 = 0.2581988897471611
    x69 = x51 * x68
    x70 = x67 * x69
    x71 = x50 * x57
    x72 = x0 * x6
    x73 = x54 * x72
    x74 = x54 * x6
    x75 = x67 * x74
    x76 = x60 * x75
    x77 = x73 + x76
    x78 = x55 * x6
    x79 = x48 * x69
    x80 = x57 * x79
    x81 = -x65 - A[2]
    x82 = x69 * x81
    x83 = x55 * x72
    x84 = x78 * x81
    x85 = x66 * x84
    x86 = x83 + x85
    x87 = x67**2 * x74
    x88 = x73 + x87
    x89 = 0.3333333333333333 * x51
    x90 = x88 * x89
    x91 = x60 * x74
    x92 = x0 * (x75 + x91)
    x93 = x67 * x77
    x94 = x92 + x93
    x95 = x47 * x89
    x96 = x66 * x78
    x97 = 1.732050807568877
    x98 = x81 * x89 * x97
    x99 = x95 * x97
    x100 = x78 * x81**2
    x101 = x100 + x83
    x102 = x101 * x89
    x103 = x0 * (x84 + x96)
    x104 = x81 * x86
    x105 = x103 + x104
    x106 = x67 * (2.0 * x73 + x88)
    x107 = x106 * x69
    x108 = 3.0 * x73
    x109 = x108 + x87
    x110 = x0 * (x109 + 2.0 * x76) + x67 * x94
    x111 = x38 * x69
    x112 = x41 * x97
    x113 = x38 * x97
    x114 = x113 * x89
    x115 = x81 * (x101 + 2.0 * x83)
    x116 = x115 * x69
    x117 = 3.0 * x83
    x118 = x100 + x117
    x119 = x0 * (x118 + 2.0 * x85) + x105 * x81
    x120 = x0 * (x108 + 3.0 * x87) + x106 * x67
    x121 = x17 + x36
    x122 = x121 * x53
    x123 = x0 * (x106 + 3.0 * x92 + 3.0 * x93) + x110 * x67
    x124 = x43 + x8
    x125 = x124 * x51
    x126 = x125 * x52
    x127 = x125 * x68
    x128 = x0 * (3.0 * x100 + x117) + x115 * x81
    x129 = x0 * (3.0 * x103 + 3.0 * x104 + x115) + x119 * x81
    x130 = -x59 - R[1]
    x131 = x62 + x63
    x132 = x0 * (2.0 * x25 + x30 + x61) + x28 * x3
    x133 = x29 + x35
    x134 = x58 * (x0 * (x131 + x132 + 3.0 * x31 + 3.0 * x33) + x133 * x3)
    x135 = x130 * x91
    x136 = x135 + x73
    x137 = x61 + x8
    x138 = x3 * (x137 + 2.0 * x8)
    x139 = x0 * (x138 + 3.0 * x19 + 3.0 * x22) + x131 * x3
    x140 = x53 * x78
    x141 = x139 * x58
    x142 = x130 * x75
    x143 = x142 + x73
    x144 = x69 * x78
    x145 = x130 * x74
    x146 = x0 * (x145 + x91)
    x147 = x136 * x67
    x148 = x146 + x147
    x149 = x131 * x69
    x150 = x133 * x57
    x151 = x0 * (x145 + x75)
    x152 = x143 * x67
    x153 = x151 + x152
    x154 = x153 * x89
    x155 = x0 * (x108 + x135 + x142 + x76)
    x156 = x148 * x67
    x157 = x155 + x156
    x158 = x23 * x89
    x159 = x143 * x97
    x160 = x159 * x89
    x161 = x84 * x97
    x162 = 2.0 * x142
    x163 = x0 * (x109 + x162)
    x164 = x153 * x67
    x165 = x163 + x164
    x166 = 2.0 * x147
    x167 = x0 * (2.0 * x146 + x153 + x166 + x94)
    x168 = x157 * x67
    x169 = x167 + x168
    x170 = x21 * x69
    x171 = x21 * x97
    x172 = x171 * x89
    x173 = x0 * (x106 + 3.0 * x151 + 3.0 * x152) + x165 * x67
    x174 = x5 * x53
    x175 = x174 * x56
    x176 = x175 * (x0 * (x110 + 3.0 * x155 + 3.0 * x156 + x165) + x169 * x67)
    x177 = x10 * x175
    x178 = x165 * x69
    x179 = x5 * x56
    x180 = x179 * x82
    x181 = x11 * x69
    x182 = x128 * x53
    x183 = 3.141592653589793 * x1 * x54
    x184 = x174 * x183
    x185 = x10 * x184
    x186 = -x65 - R[2]
    x187 = x186 * x96
    x188 = x187 + x83
    x189 = x53 * x74
    x190 = x186 * x78
    x191 = x186 * x84
    x192 = x191 + x83
    x193 = x69 * x74
    x194 = x0 * (x190 + x96)
    x195 = x188 * x81
    x196 = x194 + x195
    x197 = x192 * x97
    x198 = x197 * x89
    x199 = x75 * x97
    x200 = x0 * (x190 + x84)
    x201 = x192 * x81
    x202 = x200 + x201
    x203 = x202 * x89
    x204 = x0 * (x117 + x187 + x191 + x85)
    x205 = x196 * x81
    x206 = x204 + x205
    x207 = 2.0 * x191
    x208 = x0 * (x118 + x207)
    x209 = x202 * x81
    x210 = x208 + x209
    x211 = 2.0 * x195
    x212 = x0 * (x105 + 2.0 * x194 + x202 + x211)
    x213 = x206 * x81
    x214 = x212 + x213
    x215 = x120 * x53
    x216 = x210 * x69
    x217 = x183 * x5
    x218 = x214 * x217
    x219 = x0 * (x115 + 3.0 * x200 + 3.0 * x201) + x210 * x81
    x220 = x184 * (x0 * (x119 + 3.0 * x204 + 3.0 * x205 + x210) + x214 * x81)
    x221 = x130**2 * x74
    x222 = x221 + x73
    x223 = x0 * (x138 + 3.0 * x24 + 3.0 * x27) + x132 * x3
    x224 = x130 * x136
    x225 = x146 + x224
    x226 = x0 * (x30 + 3.0 * x61) + x138 * x3
    x227 = x226 * x53
    x228 = x130 * x143
    x229 = x151 + x228
    x230 = x130 * x148
    x231 = x155 + x230
    x232 = x138 * x69
    x233 = x222 * x69
    x234 = x0 * (x108 + x162 + x221)
    x235 = x229 * x67
    x236 = x234 + x235
    x237 = x236 * x89
    x238 = x0 * (3.0 * x146 + x166 + x224 + x229)
    x239 = x231 * x67
    x240 = x238 + x239
    x241 = x137 * x89
    x242 = x229 * x97
    x243 = x242 * x89
    x244 = 2.0 * x0 * (2.0 * x151 + x152 + x228) + x236 * x67
    x245 = x0 * (4.0 * x155 + 2.0 * x156 + 2.0 * x230 + x236) + x240 * x67
    x246 = x179 * x245
    x247 = x3 * x69
    x248 = x179 * x247
    x249 = x18 * x97
    x250 = x175 * (x0 * (2.0 * x163 + 2.0 * x164 + 3.0 * x234 + 3.0 * x235) + x244 * x67)
    x251 = x69 * x7
    x252 = x53 * x7
    x253 = x132 * x69
    x254 = x186**2 * x78
    x255 = x254 + x83
    x256 = x186 * x188
    x257 = x194 + x256
    x258 = x186 * x192
    x259 = x200 + x258
    x260 = x186 * x196
    x261 = x204 + x260
    x262 = x259 * x97
    x263 = x262 * x89
    x264 = x0 * (x117 + x207 + x254)
    x265 = x259 * x81
    x266 = x264 + x265
    x267 = x266 * x89
    x268 = x0 * (3.0 * x194 + x211 + x256 + x259)
    x269 = x261 * x81
    x270 = x268 + x269
    x271 = 2.0 * x0 * (2.0 * x200 + x201 + x258) + x266 * x81
    x272 = x217 * x271
    x273 = x0 * (4.0 * x204 + 2.0 * x205 + 2.0 * x260 + x266) + x270 * x81
    x274 = x217 * x273
    x275 = x184 * (x0 * (2.0 * x208 + 2.0 * x209 + 3.0 * x264 + 3.0 * x265) + x271 * x81)

    # 270 item(s)
    result[0, 0, 0] = numpy.sum(
        x58 * (x0 * (2.0 * x29 + 2.0 * x35 + 3.0 * x39 + 3.0 * x42 + x48) + x3 * x50)
    )
    result[0, 0, 1] = numpy.sum(x60 * x64)
    result[0, 0, 2] = numpy.sum(x64 * x66)
    result[0, 1, 0] = numpy.sum(x70 * x71)
    result[0, 1, 1] = numpy.sum(x77 * x78 * x79)
    result[0, 1, 2] = numpy.sum(x66 * x67 * x80)
    result[0, 2, 0] = numpy.sum(x71 * x82)
    result[0, 2, 1] = numpy.sum(x60 * x80 * x81)
    result[0, 2, 2] = numpy.sum(x74 * x79 * x86)
    result[0, 3, 0] = numpy.sum(x49 * x78 * x90)
    result[0, 3, 1] = numpy.sum(x78 * x94 * x95)
    result[0, 3, 2] = numpy.sum(x47 * x90 * x96)
    result[0, 4, 0] = numpy.sum(x49 * x57 * x67 * x98)
    result[0, 4, 1] = numpy.sum(x77 * x84 * x99)
    result[0, 4, 2] = numpy.sum(x75 * x86 * x99)
    result[0, 5, 0] = numpy.sum(x102 * x49 * x74)
    result[0, 5, 1] = numpy.sum(x102 * x47 * x91)
    result[0, 5, 2] = numpy.sum(x105 * x74 * x95)
    result[0, 6, 0] = numpy.sum(x107 * x41 * x78)
    result[0, 6, 1] = numpy.sum(x110 * x111 * x78)
    result[0, 6, 2] = numpy.sum(x107 * x38 * x96)
    result[0, 7, 0] = numpy.sum(x112 * x84 * x90)
    result[0, 7, 1] = numpy.sum(x114 * x84 * x94)
    result[0, 7, 2] = numpy.sum(x113 * x86 * x90)
    result[0, 8, 0] = numpy.sum(x102 * x112 * x75)
    result[0, 8, 1] = numpy.sum(x102 * x113 * x77)
    result[0, 8, 2] = numpy.sum(x105 * x114 * x75)
    result[0, 9, 0] = numpy.sum(x116 * x41 * x74)
    result[0, 9, 1] = numpy.sum(x116 * x38 * x91)
    result[0, 9, 2] = numpy.sum(x111 * x119 * x74)
    result[0, 10, 0] = numpy.sum(x120 * x122 * x78)
    result[0, 10, 1] = numpy.sum(x123 * x126 * x78)
    result[0, 10, 2] = numpy.sum(x120 * x126 * x96)
    result[0, 11, 0] = numpy.sum(x107 * x121 * x84)
    result[0, 11, 1] = numpy.sum(x110 * x127 * x84)
    result[0, 11, 2] = numpy.sum(x106 * x127 * x86)
    result[0, 12, 0] = numpy.sum(x102 * x121 * x88)
    result[0, 12, 1] = numpy.sum(x102 * x124 * x94)
    result[0, 12, 2] = numpy.sum(x105 * x124 * x90)
    result[0, 13, 0] = numpy.sum(x116 * x121 * x75)
    result[0, 13, 1] = numpy.sum(x115 * x127 * x77)
    result[0, 13, 2] = numpy.sum(x119 * x127 * x75)
    result[0, 14, 0] = numpy.sum(x122 * x128 * x74)
    result[0, 14, 1] = numpy.sum(x126 * x128 * x91)
    result[0, 14, 2] = numpy.sum(x126 * x129 * x74)
    result[1, 0, 0] = numpy.sum(x130 * x134)
    result[1, 0, 1] = numpy.sum(x136 * x139 * x140)
    result[1, 0, 2] = numpy.sum(x130 * x141 * x66)
    result[1, 1, 0] = numpy.sum(x133 * x143 * x144)
    result[1, 1, 1] = numpy.sum(x131 * x144 * x148)
    result[1, 1, 2] = numpy.sum(x143 * x149 * x96)
    result[1, 2, 0] = numpy.sum(x130 * x150 * x82)
    result[1, 2, 1] = numpy.sum(x136 * x149 * x84)
    result[1, 2, 2] = numpy.sum(x145 * x149 * x86)
    result[1, 3, 0] = numpy.sum(x154 * x34 * x78)
    result[1, 3, 1] = numpy.sum(x157 * x158 * x78)
    result[1, 3, 2] = numpy.sum(x153 * x158 * x96)
    result[1, 4, 0] = numpy.sum(x160 * x34 * x84)
    result[1, 4, 1] = numpy.sum(x148 * x158 * x161)
    result[1, 4, 2] = numpy.sum(x158 * x159 * x86)
    result[1, 5, 0] = numpy.sum(x102 * x145 * x34)
    result[1, 5, 1] = numpy.sum(x102 * x136 * x23)
    result[1, 5, 2] = numpy.sum(x105 * x145 * x158)
    result[1, 6, 0] = numpy.sum(x144 * x165 * x32)
    result[1, 6, 1] = numpy.sum(x144 * x169 * x21)
    result[1, 6, 2] = numpy.sum(x165 * x170 * x96)
    result[1, 7, 0] = numpy.sum(x154 * x161 * x32)
    result[1, 7, 1] = numpy.sum(x157 * x172 * x84)
    result[1, 7, 2] = numpy.sum(x154 * x171 * x86)
    result[1, 8, 0] = numpy.sum(x102 * x159 * x32)
    result[1, 8, 1] = numpy.sum(x102 * x148 * x171)
    result[1, 8, 2] = numpy.sum(x105 * x160 * x21)
    result[1, 9, 0] = numpy.sum(x116 * x145 * x32)
    result[1, 9, 1] = numpy.sum(x116 * x136 * x21)
    result[1, 9, 2] = numpy.sum(x119 * x145 * x170)
    result[1, 10, 0] = numpy.sum(x13 * x140 * x173)
    result[1, 10, 1] = numpy.sum(x10 * x176)
    result[1, 10, 2] = numpy.sum(x173 * x177 * x66)
    result[1, 11, 0] = numpy.sum(x13 * x178 * x84)
    result[1, 11, 1] = numpy.sum(x10 * x169 * x180)
    result[1, 11, 2] = numpy.sum(x11 * x178 * x86)
    result[1, 12, 0] = numpy.sum(x102 * x13 * x153)
    result[1, 12, 1] = numpy.sum(x102 * x11 * x157)
    result[1, 12, 2] = numpy.sum(x105 * x11 * x154)
    result[1, 13, 0] = numpy.sum(x116 * x13 * x143)
    result[1, 13, 1] = numpy.sum(x11 * x116 * x148)
    result[1, 13, 2] = numpy.sum(x119 * x143 * x181)
    result[1, 14, 0] = numpy.sum(x13 * x145 * x182)
    result[1, 14, 1] = numpy.sum(x11 * x136 * x182)
    result[1, 14, 2] = numpy.sum(x129 * x130 * x185)
    result[2, 0, 0] = numpy.sum(x134 * x186)
    result[2, 0, 1] = numpy.sum(x141 * x186 * x60)
    result[2, 0, 2] = numpy.sum(x139 * x188 * x189)
    result[2, 1, 0] = numpy.sum(x150 * x186 * x70)
    result[2, 1, 1] = numpy.sum(x149 * x190 * x77)
    result[2, 1, 2] = numpy.sum(x149 * x188 * x75)
    result[2, 2, 0] = numpy.sum(x133 * x192 * x193)
    result[2, 2, 1] = numpy.sum(x149 * x192 * x91)
    result[2, 2, 2] = numpy.sum(x131 * x193 * x196)
    result[2, 3, 0] = numpy.sum(x190 * x34 * x90)
    result[2, 3, 1] = numpy.sum(x158 * x190 * x94)
    result[2, 3, 2] = numpy.sum(x188 * x23 * x90)
    result[2, 4, 0] = numpy.sum(x198 * x34 * x75)
    result[2, 4, 1] = numpy.sum(x158 * x197 * x77)
    result[2, 4, 2] = numpy.sum(x158 * x196 * x199)
    result[2, 5, 0] = numpy.sum(x203 * x34 * x74)
    result[2, 5, 1] = numpy.sum(x158 * x202 * x91)
    result[2, 5, 2] = numpy.sum(x158 * x206 * x74)
    result[2, 6, 0] = numpy.sum(x107 * x190 * x32)
    result[2, 6, 1] = numpy.sum(x110 * x170 * x190)
    result[2, 6, 2] = numpy.sum(x107 * x188 * x21)
    result[2, 7, 0] = numpy.sum(x197 * x32 * x90)
    result[2, 7, 1] = numpy.sum(x172 * x192 * x94)
    result[2, 7, 2] = numpy.sum(x171 * x196 * x90)
    result[2, 8, 0] = numpy.sum(x199 * x203 * x32)
    result[2, 8, 1] = numpy.sum(x172 * x202 * x77)
    result[2, 8, 2] = numpy.sum(x172 * x206 * x75)
    result[2, 9, 0] = numpy.sum(x193 * x210 * x32)
    result[2, 9, 1] = numpy.sum(x170 * x210 * x91)
    result[2, 9, 2] = numpy.sum(x193 * x21 * x214)
    result[2, 10, 0] = numpy.sum(x13 * x190 * x215)
    result[2, 10, 1] = numpy.sum(x123 * x177 * x186)
    result[2, 10, 2] = numpy.sum(x11 * x188 * x215)
    result[2, 11, 0] = numpy.sum(x107 * x13 * x192)
    result[2, 11, 1] = numpy.sum(x110 * x181 * x192)
    result[2, 11, 2] = numpy.sum(x107 * x11 * x196)
    result[2, 12, 0] = numpy.sum(x13 * x202 * x90)
    result[2, 12, 1] = numpy.sum(x11 * x203 * x94)
    result[2, 12, 2] = numpy.sum(x11 * x206 * x90)
    result[2, 13, 0] = numpy.sum(x13 * x216 * x75)
    result[2, 13, 1] = numpy.sum(x181 * x210 * x77)
    result[2, 13, 2] = numpy.sum(x10 * x218 * x70)
    result[2, 14, 0] = numpy.sum(x13 * x189 * x219)
    result[2, 14, 1] = numpy.sum(x185 * x219 * x60)
    result[2, 14, 2] = numpy.sum(x10 * x220)
    result[3, 0, 0] = numpy.sum(x140 * x222 * x223)
    result[3, 0, 1] = numpy.sum(x140 * x225 * x226)
    result[3, 0, 2] = numpy.sum(x222 * x227 * x96)
    result[3, 1, 0] = numpy.sum(x132 * x144 * x229)
    result[3, 1, 1] = numpy.sum(x231 * x232 * x78)
    result[3, 1, 2] = numpy.sum(x229 * x232 * x96)
    result[3, 2, 0] = numpy.sum(x132 * x233 * x84)
    result[3, 2, 1] = numpy.sum(x225 * x232 * x84)
    result[3, 2, 2] = numpy.sum(x222 * x232 * x86)
    result[3, 3, 0] = numpy.sum(x237 * x28 * x78)
    result[3, 3, 1] = numpy.sum(x240 * x241 * x78)
    result[3, 3, 2] = numpy.sum(x236 * x241 * x96)
    result[3, 4, 0] = numpy.sum(x243 * x28 * x84)
    result[3, 4, 1] = numpy.sum(x161 * x231 * x241)
    result[3, 4, 2] = numpy.sum(x241 * x242 * x86)
    result[3, 5, 0] = numpy.sum(x102 * x222 * x28)
    result[3, 5, 1] = numpy.sum(x102 * x137 * x225)
    result[3, 5, 2] = numpy.sum(x105 * x222 * x241)
    result[3, 6, 0] = numpy.sum(x144 * x244 * x26)
    result[3, 6, 1] = numpy.sum(x246 * x247)
    result[3, 6, 2] = numpy.sum(x244 * x248 * x66)
    result[3, 7, 0] = numpy.sum(x161 * x237 * x26)
    result[3, 7, 1] = numpy.sum(x179 * x240 * x3 * x98)
    result[3, 7, 2] = numpy.sum(x237 * x249 * x86)
    result[3, 8, 0] = numpy.sum(x102 * x242 * x26)
    result[3, 8, 1] = numpy.sum(x102 * x231 * x249)
    result[3, 8, 2] = numpy.sum(x105 * x18 * x243)
    result[3, 9, 0] = numpy.sum(x116 * x222 * x26)
    result[3, 9, 1] = numpy.sum(x116 * x18 * x225)
    result[3, 9, 2] = numpy.sum(x119 * x18 * x233)
    result[3, 10, 0] = numpy.sum(x250 * x9)
    result[3, 10, 1] = numpy.sum(
        x175
        * (x0 * (2.0 * x167 + 2.0 * x168 + 3.0 * x238 + 3.0 * x239 + x244) + x245 * x67)
    )
    result[3, 10, 2] = numpy.sum(x250 * x66)
    result[3, 11, 0] = numpy.sum(x180 * x244 * x9)
    result[3, 11, 1] = numpy.sum(x246 * x82)
    result[3, 11, 2] = numpy.sum(x244 * x251 * x86)
    result[3, 12, 0] = numpy.sum(x102 * x16 * x236)
    result[3, 12, 1] = numpy.sum(x102 * x240 * x7)
    result[3, 12, 2] = numpy.sum(x105 * x237 * x7)
    result[3, 13, 0] = numpy.sum(x116 * x16 * x229)
    result[3, 13, 1] = numpy.sum(x116 * x231 * x7)
    result[3, 13, 2] = numpy.sum(x119 * x229 * x251)
    result[3, 14, 0] = numpy.sum(x16 * x182 * x222)
    result[3, 14, 1] = numpy.sum(x128 * x225 * x252)
    result[3, 14, 2] = numpy.sum(x129 * x222 * x252)
    result[4, 0, 0] = numpy.sum(x130 * x186 * x223 * x58)
    result[4, 0, 1] = numpy.sum(x136 * x190 * x227)
    result[4, 0, 2] = numpy.sum(x145 * x188 * x227)
    result[4, 1, 0] = numpy.sum(x143 * x190 * x253)
    result[4, 1, 1] = numpy.sum(x148 * x190 * x232)
    result[4, 1, 2] = numpy.sum(x143 * x188 * x232)
    result[4, 2, 0] = numpy.sum(x145 * x192 * x253)
    result[4, 2, 1] = numpy.sum(x136 * x192 * x232)
    result[4, 2, 2] = numpy.sum(x145 * x196 * x232)
    result[4, 3, 0] = numpy.sum(x154 * x190 * x28)
    result[4, 3, 1] = numpy.sum(x157 * x190 * x241)
    result[4, 3, 2] = numpy.sum(x153 * x188 * x241)
    result[4, 4, 0] = numpy.sum(x160 * x192 * x28)
    result[4, 4, 1] = numpy.sum(x148 * x197 * x241)
    result[4, 4, 2] = numpy.sum(x159 * x196 * x241)
    result[4, 5, 0] = numpy.sum(x145 * x203 * x28)
    result[4, 5, 1] = numpy.sum(x136 * x202 * x241)
    result[4, 5, 2] = numpy.sum(x145 * x206 * x241)
    result[4, 6, 0] = numpy.sum(x178 * x190 * x26)
    result[4, 6, 1] = numpy.sum(x169 * x186 * x248)
    result[4, 6, 2] = numpy.sum(x178 * x18 * x188)
    result[4, 7, 0] = numpy.sum(x154 * x197 * x26)
    result[4, 7, 1] = numpy.sum(x157 * x18 * x198)
    result[4, 7, 2] = numpy.sum(x154 * x196 * x249)
    result[4, 8, 0] = numpy.sum(x160 * x202 * x26)
    result[4, 8, 1] = numpy.sum(x148 * x203 * x249)
    result[4, 8, 2] = numpy.sum(x160 * x18 * x206)
    result[4, 9, 0] = numpy.sum(x145 * x216 * x26)
    result[4, 9, 1] = numpy.sum(x136 * x18 * x216)
    result[4, 9, 2] = numpy.sum(x130 * x218 * x247)
    result[4, 10, 0] = numpy.sum(x173 * x175 * x186 * x9)
    result[4, 10, 1] = numpy.sum(x176 * x186)
    result[4, 10, 2] = numpy.sum(x173 * x188 * x252)
    result[4, 11, 0] = numpy.sum(x16 * x178 * x192)
    result[4, 11, 1] = numpy.sum(x169 * x192 * x251)
    result[4, 11, 2] = numpy.sum(x165 * x196 * x251)
    result[4, 12, 0] = numpy.sum(x154 * x16 * x202)
    result[4, 12, 1] = numpy.sum(x157 * x203 * x7)
    result[4, 12, 2] = numpy.sum(x154 * x206 * x7)
    result[4, 13, 0] = numpy.sum(x143 * x16 * x216)
    result[4, 13, 1] = numpy.sum(x148 * x210 * x251)
    result[4, 13, 2] = numpy.sum(x143 * x214 * x251)
    result[4, 14, 0] = numpy.sum(x130 * x184 * x219 * x9)
    result[4, 14, 1] = numpy.sum(x136 * x219 * x252)
    result[4, 14, 2] = numpy.sum(x130 * x220)
    result[5, 0, 0] = numpy.sum(x189 * x223 * x255)
    result[5, 0, 1] = numpy.sum(x227 * x255 * x91)
    result[5, 0, 2] = numpy.sum(x189 * x226 * x257)
    result[5, 1, 0] = numpy.sum(x253 * x255 * x75)
    result[5, 1, 1] = numpy.sum(x232 * x255 * x77)
    result[5, 1, 2] = numpy.sum(x232 * x257 * x75)
    result[5, 2, 0] = numpy.sum(x132 * x193 * x259)
    result[5, 2, 1] = numpy.sum(x232 * x259 * x91)
    result[5, 2, 2] = numpy.sum(x232 * x261 * x74)
    result[5, 3, 0] = numpy.sum(x255 * x28 * x90)
    result[5, 3, 1] = numpy.sum(x241 * x255 * x94)
    result[5, 3, 2] = numpy.sum(x137 * x257 * x90)
    result[5, 4, 0] = numpy.sum(x263 * x28 * x75)
    result[5, 4, 1] = numpy.sum(x241 * x262 * x77)
    result[5, 4, 2] = numpy.sum(x199 * x241 * x261)
    result[5, 5, 0] = numpy.sum(x267 * x28 * x74)
    result[5, 5, 1] = numpy.sum(x241 * x266 * x91)
    result[5, 5, 2] = numpy.sum(x241 * x270 * x74)
    result[5, 6, 0] = numpy.sum(x107 * x255 * x26)
    result[5, 6, 1] = numpy.sum(x110 * x18 * x255 * x69)
    result[5, 6, 2] = numpy.sum(x107 * x18 * x257)
    result[5, 7, 0] = numpy.sum(x26 * x262 * x90)
    result[5, 7, 1] = numpy.sum(x18 * x263 * x94)
    result[5, 7, 2] = numpy.sum(x249 * x261 * x90)
    result[5, 8, 0] = numpy.sum(x199 * x26 * x267)
    result[5, 8, 1] = numpy.sum(x249 * x267 * x77)
    result[5, 8, 2] = numpy.sum(x217 * x270 * x3 * x67 * x89 * x97)
    result[5, 9, 0] = numpy.sum(x193 * x26 * x271)
    result[5, 9, 1] = numpy.sum(x247 * x272 * x60)
    result[5, 9, 2] = numpy.sum(x247 * x274)
    result[5, 10, 0] = numpy.sum(x16 * x215 * x255)
    result[5, 10, 1] = numpy.sum(x123 * x252 * x255)
    result[5, 10, 2] = numpy.sum(x120 * x252 * x257)
    result[5, 11, 0] = numpy.sum(x107 * x16 * x259)
    result[5, 11, 1] = numpy.sum(x110 * x251 * x259)
    result[5, 11, 2] = numpy.sum(x107 * x261 * x7)
    result[5, 12, 0] = numpy.sum(x16 * x266 * x90)
    result[5, 12, 1] = numpy.sum(x267 * x7 * x94)
    result[5, 12, 2] = numpy.sum(x270 * x7 * x90)
    result[5, 13, 0] = numpy.sum(x272 * x70 * x9)
    result[5, 13, 1] = numpy.sum(x251 * x271 * x77)
    result[5, 13, 2] = numpy.sum(x274 * x70)
    result[5, 14, 0] = numpy.sum(x275 * x9)
    result[5, 14, 1] = numpy.sum(x275 * x60)
    result[5, 14, 2] = numpy.sum(
        x184
        * (x0 * (2.0 * x212 + 2.0 * x213 + 3.0 * x268 + 3.0 * x269 + x271) + x273 * x81)
    )
    return result


def quadrupole3d_42(ax, da, A, bx, db, B, R):
    """Cartesian 3D (gd) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 15, 6), dtype=float)

    x0 = 0.5 / (ax + bx)
    x1 = (ax + bx) ** (-1.0)
    x2 = -x1 * (ax * A[0] + bx * B[0])
    x3 = -x2 - B[0]
    x4 = ax * bx * x1
    x5 = numpy.exp(-x4 * (A[0] - B[0]) ** 2)
    x6 = 1.772453850905516 * numpy.sqrt(x1)
    x7 = x5 * x6
    x8 = x3 * x7
    x9 = -x2 - R[0]
    x10 = x7 * x9
    x11 = x0 * (x10 + x8)
    x12 = -x2 - A[0]
    x13 = x0 * x7
    x14 = x8 * x9
    x15 = x13 + x14
    x16 = x12 * x15
    x17 = x11 + x16
    x18 = x17 * x3
    x19 = 2.0 * x18
    x20 = x3**2 * x7
    x21 = 3.0 * x13
    x22 = x12 * x7
    x23 = x22 * x3
    x24 = x21 + 2.0 * x23
    x25 = x0 * (x20 + x24)
    x26 = x0 * (x22 + x8)
    x27 = x13 + x23
    x28 = x27 * x3
    x29 = x26 + x28
    x30 = x12 * x29
    x31 = x25 + x30
    x32 = x12 * x17
    x33 = x22 * x9
    x34 = x0 * (x14 + x21 + x23 + x33)
    x35 = 4.0 * x34
    x36 = 2.0 * x32 + x35
    x37 = x0 * (x19 + x31 + x36)
    x38 = x17 * x9
    x39 = 2.0 * x38
    x40 = 2.0 * x33
    x41 = x7 * x9**2
    x42 = x21 + x41
    x43 = x0 * (x40 + x42)
    x44 = x0 * (x10 + x22)
    x45 = x13 + x33
    x46 = x45 * x9
    x47 = x44 + x46
    x48 = x12 * x47
    x49 = x43 + x48
    x50 = x0 * (x36 + x39 + x49)
    x51 = x15 * x3
    x52 = 2.0 * x16
    x53 = 3.0 * x11 + x52
    x54 = x0 * (x29 + x51 + x53)
    x55 = x18 + x34
    x56 = x12 * x55
    x57 = x54 + x56
    x58 = x12 * x57
    x59 = x15 * x9
    x60 = x0 * (x47 + x53 + x59)
    x61 = x34 + x38
    x62 = x12 * x61
    x63 = x60 + x62
    x64 = x12 * x63
    x65 = x11 + x59
    x66 = x0 * (2.0 * x14 + x42) + x3 * x65
    x67 = x0 * (x19 + x35 + x39 + x66)
    x68 = x3 * x61
    x69 = x60 + x68
    x70 = x12 * x69
    x71 = x67 + x70
    x72 = 2.0 * x0 * (x54 + x56 + 2.0 * x60 + x62 + x68) + x12 * x71
    x73 = da * db
    x74 = 5.916079783099616
    x75 = 0.009523809523809524 * x74
    x76 = x73 * x75
    x77 = numpy.exp(-x4 * (A[1] - B[1]) ** 2)
    x78 = numpy.exp(-x4 * (A[2] - B[2]) ** 2)
    x79 = 3.141592653589793 * x1 * x78
    x80 = x77 * x79
    x81 = x76 * x80
    x82 = -x1 * (ax * A[1] + bx * B[1])
    x83 = -x82 - B[1]
    x84 = x73 * x80
    x85 = x83 * x84
    x86 = x12 * x45
    x87 = 2.0 * x0 * (2.0 * x44 + x46 + x86) + x12 * x49
    x88 = x44 + x86
    x89 = x12 * x27
    x90 = x26 + x89
    x91 = x0 * (2.0 * x11 + x52 + x88 + x90)
    x92 = x32 + x34
    x93 = x12 * x92
    x94 = 2.0 * x91 + 2.0 * x93
    x95 = x50 + x64
    x96 = 10.2469507659596
    x97 = 0.009523809523809524 * x96
    x98 = x97 * (x0 * (3.0 * x60 + 3.0 * x62 + x87 + x94) + x12 * x95)
    x99 = -x1 * (ax * A[2] + bx * B[2])
    x100 = -x99 - B[2]
    x101 = x100 * x84
    x102 = x6 * x78
    x103 = x6 * x77
    x104 = x103 * x83**2
    x105 = x0 * x103
    x106 = x73 * (x104 + x105)
    x107 = x12**2 * x7
    x108 = x0 * (x107 + x21 + x40)
    x109 = x12 * x88
    x110 = x0 * (2.0 * x108 + 2.0 * x109 + 3.0 * x43 + 3.0 * x48) + x12 * x87
    x111 = x110 * x75
    x112 = x85 * x97
    x113 = x100**2 * x102
    x114 = x0 * x102
    x115 = x73 * (x113 + x114)
    x116 = -x82 - A[1]
    x117 = 2.23606797749979
    x118 = 0.06666666666666667 * x117
    x119 = x116 * x118
    x120 = x72 * x84
    x121 = x103 * x116
    x122 = x121 * x83
    x123 = x105 + x122
    x124 = 3.872983346207417
    x125 = x123 * x124
    x126 = 0.06666666666666667 * x95
    x127 = x102 * x73
    x128 = 0.06666666666666667 * x124
    x129 = x128 * x95
    x130 = x103 * x83
    x131 = x0 * (x121 + x130)
    x132 = x123 * x83
    x133 = x131 + x132
    x134 = x117 * x133
    x135 = 0.06666666666666667 * x87
    x136 = x100 * x102
    x137 = x136 * x73
    x138 = x117 * x135
    x139 = -x99 - A[2]
    x140 = x118 * x139
    x141 = x102 * x139
    x142 = x100 * x141
    x143 = x114 + x142
    x144 = x124 * x143
    x145 = x144 * x73
    x146 = x0 * (x136 + x141)
    x147 = x100 * x143
    x148 = x146 + x147
    x149 = x103 * x73
    x150 = x103 * x116**2
    x151 = x105 + x150
    x152 = 1.732050807568877
    x153 = 0.1111111111111111 * x152
    x154 = x151 * x153
    x155 = x154 * x73
    x156 = x116 * x123
    x157 = x131 + x156
    x158 = 0.3333333333333333 * x73
    x159 = x102 * x158
    x160 = x151 * x158
    x161 = 3.0 * x105
    x162 = 2.0 * x122 + x161
    x163 = x0 * (x104 + x162)
    x164 = x116 * x133
    x165 = x163 + x164
    x166 = x152 * x49
    x167 = 0.1111111111111111 * x166
    x168 = x157 * x158
    x169 = x139 * x158
    x170 = x123 * x152
    x171 = x141 * x158
    x172 = x143 * x158
    x173 = x152 * x172
    x174 = x148 * x158
    x175 = x102 * x139**2
    x176 = x114 + x175
    x177 = x153 * x176
    x178 = x177 * x73
    x179 = x158 * x176
    x180 = x139 * x143
    x181 = x146 + x180
    x182 = x158 * x181
    x183 = 3.0 * x114
    x184 = 2.0 * x142 + x183
    x185 = x0 * (x113 + x184)
    x186 = x139 * x148
    x187 = x185 + x186
    x188 = x116 * (2.0 * x105 + x151)
    x189 = 0.06666666666666667 * x188
    x190 = x189 * x73
    x191 = x117 * x190
    x192 = x0 * (x150 + x162)
    x193 = x116 * x157
    x194 = x192 + x193
    x195 = x128 * x194
    x196 = x124 * x190
    x197 = 2.0 * x0 * (2.0 * x131 + x132 + x156) + x116 * x165
    x198 = x117 * x47
    x199 = 0.06666666666666667 * x198
    x200 = x141 * x152
    x201 = x143 * x152
    x202 = x158 * x47
    x203 = x152 * x182
    x204 = x139 * (2.0 * x114 + x176)
    x205 = 0.06666666666666667 * x204
    x206 = x205 * x73
    x207 = x117 * x206
    x208 = x124 * x206
    x209 = x0 * (x175 + x184)
    x210 = x139 * x181
    x211 = x209 + x210
    x212 = x128 * x211
    x213 = x212 * x73
    x214 = 2.0 * x0 * (2.0 * x146 + x147 + x180) + x139 * x187
    x215 = (
        0.009523809523809524 * x0 * (3.0 * x150 + x161)
        + 0.009523809523809524 * x116 * x188
    )
    x216 = x66 * x73
    x217 = x216 * x74
    x218 = x0 * (3.0 * x131 + 3.0 * x156 + x188) + x116 * x194
    x219 = 0.009523809523809524 * x127
    x220 = x65 * x96
    x221 = x0 * (3.0 * x163 + 3.0 * x164 + 2.0 * x192 + 2.0 * x193) + x116 * x197
    x222 = x13 + x41
    x223 = x222 * x74
    x224 = x222 * x73
    x225 = x224 * x97
    x226 = x141 * x73
    x227 = 0.06666666666666667 * x224
    x228 = x117 * x197
    x229 = x158 * x65
    x230 = x117 * x121
    x231 = (
        0.009523809523809524 * x0 * (3.0 * x175 + x183)
        + 0.009523809523809524 * x139 * x204
    )
    x232 = x231 * x73
    x233 = x0 * (3.0 * x146 + 3.0 * x180 + x204) + x139 * x211
    x234 = 0.009523809523809524 * x149
    x235 = x0 * (3.0 * x185 + 3.0 * x186 + 2.0 * x209 + 2.0 * x210) + x139 * x214
    x236 = -x82 - R[1]
    x237 = 2.0 * x0 * (2.0 * x26 + x28 + x89) + x12 * x31
    x238 = x37 + x58
    x239 = x81 * (x0 * (x237 + 3.0 * x54 + 3.0 * x56 + x94) + x12 * x238)
    x240 = x108 + x109
    x241 = x0 * (x107 + x24)
    x242 = x12 * x90
    x243 = x241 + x242
    x244 = x91 + x93
    x245 = x0 * (x240 + x243 + 3.0 * x32 + 3.0 * x34) + x12 * x244
    x246 = x130 * x236
    x247 = x105 + x246
    x248 = x247 * x73
    x249 = x248 * x96
    x250 = 0.009523809523809524 * x102
    x251 = x103 * x236
    x252 = x0 * (x130 + x251)
    x253 = x247 * x83
    x254 = x252 + x253
    x255 = x107 + x13
    x256 = 2.0 * x0 * x22 + x12 * x255
    x257 = x0 * (x256 + 3.0 * x44 + 3.0 * x86) + x12 * x240
    x258 = x102 * x76
    x259 = 0.009523809523809524 * x136
    x260 = x115 * x75
    x261 = x121 * x236
    x262 = x105 + x261
    x263 = x117 * x262
    x264 = 0.06666666666666667 * x127
    x265 = x116 * x247
    x266 = x252 + x265
    x267 = x127 * x128
    x268 = x128 * x137
    x269 = x0 * (x122 + x161 + x246 + x261)
    x270 = x266 * x83
    x271 = x269 + x270
    x272 = 0.06666666666666667 * x240
    x273 = x117 * x127
    x274 = x124 * x272
    x275 = x238 * x84
    x276 = x128 * x248
    x277 = 0.06666666666666667 * x251
    x278 = x117 * x272
    x279 = x251 * x73
    x280 = x0 * (x121 + x251)
    x281 = x116 * x262
    x282 = x280 + x281
    x283 = x153 * x282
    x284 = x116 * x266
    x285 = x269 + x284
    x286 = x136 * x158
    x287 = 2.0 * x265
    x288 = 3.0 * x252 + x287
    x289 = x0 * (x133 + x253 + x288)
    x290 = x116 * x271
    x291 = x289 + x290
    x292 = x153 * x88
    x293 = x152 * x266
    x294 = x161 + 2.0 * x261
    x295 = x0 * (x150 + x294)
    x296 = x116 * x282
    x297 = x295 + x296
    x298 = 0.06666666666666667 * x297
    x299 = x0 * (x157 + 2.0 * x252 + x282 + x287)
    x300 = x116 * x285
    x301 = x299 + x300
    x302 = x124 * x298
    x303 = 2.0 * x284
    x304 = 4.0 * x269
    x305 = 2.0 * x270 + x304
    x306 = x0 * (x165 + x303 + x305)
    x307 = x116 * x291
    x308 = x306 + x307
    x309 = x117 * x45
    x310 = x152 * x17
    x311 = x158 * x45
    x312 = x214 * x73
    x313 = x11 + x51
    x314 = x0 * (x188 + 3.0 * x280 + 3.0 * x281) + x116 * x297
    x315 = x0 * (x194 + 3.0 * x269 + 3.0 * x284 + x297) + x116 * x301
    x316 = x15 * x73
    x317 = x316 * x96
    x318 = 2.0 * x299 + 2.0 * x300
    x319 = x5 * x76
    x320 = x319 * x79
    x321 = x320 * (x0 * (x197 + 3.0 * x289 + 3.0 * x290 + x318) + x116 * x308)
    x322 = x73 * x97
    x323 = x100 * x322
    x324 = x5 * x79
    x325 = x315 * x324
    x326 = x117 * x298
    x327 = x128 * x316
    x328 = x140 * x324
    x329 = x73 * x9
    x330 = 0.06666666666666667 * x10
    x331 = x10 * x73
    x332 = x232 * x74
    x333 = 0.009523809523809524 * x233
    x334 = 3.141592653589793 * x1 * x77
    x335 = x319 * x334
    x336 = -x99 - R[2]
    x337 = 0.009523809523809524 * x103
    x338 = x136 * x336
    x339 = x114 + x338
    x340 = x339 * x73
    x341 = x340 * x96
    x342 = x102 * x336
    x343 = x106 * x75
    x344 = 0.009523809523809524 * x341
    x345 = x0 * (x136 + x342)
    x346 = x100 * x339
    x347 = x345 + x346
    x348 = x103 * x76
    x349 = x342 * x73
    x350 = 0.06666666666666667 * x349
    x351 = x128 * x340
    x352 = x230 * x73
    x353 = x141 * x336
    x354 = x114 + x353
    x355 = x117 * x354
    x356 = 0.06666666666666667 * x149
    x357 = x130 * x73
    x358 = x128 * x357
    x359 = x139 * x339
    x360 = x345 + x359
    x361 = x128 * x149
    x362 = x0 * (x142 + x183 + x338 + x353)
    x363 = x100 * x360
    x364 = x362 + x363
    x365 = x117 * x149
    x366 = x158 * x339
    x367 = x158 * x354
    x368 = x158 * x360
    x369 = x152 * x368
    x370 = x158 * x364
    x371 = x0 * (x141 + x342)
    x372 = x139 * x354
    x373 = x371 + x372
    x374 = x153 * x373
    x375 = x158 * x373
    x376 = x139 * x360
    x377 = x362 + x376
    x378 = x158 * x377
    x379 = 2.0 * x359
    x380 = 3.0 * x345 + x379
    x381 = x0 * (x148 + x346 + x380)
    x382 = x139 * x364
    x383 = x381 + x382
    x384 = x152 * x360
    x385 = x183 + 2.0 * x353
    x386 = x0 * (x175 + x385)
    x387 = x139 * x373
    x388 = x386 + x387
    x389 = 0.06666666666666667 * x388
    x390 = x0 * (x181 + 2.0 * x345 + x373 + x379)
    x391 = x139 * x377
    x392 = x390 + x391
    x393 = 2.0 * x376
    x394 = 4.0 * x362
    x395 = 2.0 * x363 + x394
    x396 = x0 * (x187 + x393 + x395)
    x397 = x139 * x383
    x398 = x396 + x397
    x399 = x215 * x74
    x400 = 0.009523809523809524 * x317
    x401 = x330 * x73
    x402 = x15 * x158
    x403 = x334 * x5
    x404 = x119 * x403
    x405 = x0 * (x204 + 3.0 * x371 + 3.0 * x372) + x139 * x388
    x406 = x0 * (x211 + 3.0 * x362 + 3.0 * x376 + x388) + x139 * x392
    x407 = x322 * x83
    x408 = x403 * x406
    x409 = 2.0 * x390 + 2.0 * x391
    x410 = x335 * (x0 * (x214 + 3.0 * x381 + 3.0 * x382 + x409) + x139 * x398)
    x411 = x0 * (2.0 * x241 + 2.0 * x242 + 3.0 * x25 + 3.0 * x30) + x12 * x237
    x412 = x103 * x236**2
    x413 = x105 + x412
    x414 = x413 * x74
    x415 = x0 * (x256 + 3.0 * x26 + 3.0 * x89) + x12 * x243
    x416 = x236 * x247
    x417 = x252 + x416
    x418 = x417 * x96
    x419 = x413 * x73
    x420 = x419 * x97
    x421 = (
        0.009523809523809524 * x0 * (3.0 * x107 + x21) + 0.009523809523809524 * x12 * x256
    )
    x422 = x0 * (x161 + 2.0 * x246 + x412) + x417 * x83
    x423 = x422 * x74
    x424 = x236 * x262
    x425 = x280 + x424
    x426 = x117 * x425
    x427 = x236 * x266
    x428 = x269 + x427
    x429 = x0 * (x288 + x416 + x425)
    x430 = x428 * x83
    x431 = x429 + x430
    x432 = 0.06666666666666667 * x256
    x433 = x117 * x432
    x434 = x118 * x419
    x435 = x128 * x243
    x436 = 0.06666666666666667 * x243
    x437 = x0 * (x294 + x412)
    x438 = x116 * x425
    x439 = x437 + x438
    x440 = x153 * x439
    x441 = x116 * x428
    x442 = x429 + x441
    x443 = x158 * x90
    x444 = 2.0 * x427
    x445 = x0 * (x305 + x422 + x444)
    x446 = x116 * x431
    x447 = x445 + x446
    x448 = x153 * x255
    x449 = x448 * x73
    x450 = x158 * x255
    x451 = 2.0 * x0 * (2.0 * x280 + x281 + x424) + x116 * x439
    x452 = x117 * x451
    x453 = x0 * (x303 + x304 + x439 + x444)
    x454 = x116 * x442
    x455 = x453 + x454
    x456 = 2.0 * x0 * (x289 + x290 + 2.0 * x429 + x430 + x441) + x116 * x447
    x457 = x456 * x73
    x458 = x12 * x5
    x459 = x458 * x79
    x460 = x118 * x459
    x461 = x128 * x73
    x462 = x455 * x461
    x463 = 0.06666666666666667 * x452
    x464 = x152 * x27
    x465 = x158 * x464
    x466 = x158 * x22
    x467 = x13 + x20
    x468 = x467 * x73
    x469 = x0 * (2.0 * x295 + 2.0 * x296 + 3.0 * x437 + 3.0 * x438) + x116 * x451
    x470 = x469 * x75
    x471 = x3 * x322
    x472 = x324 * (x0 * (x318 + 3.0 * x429 + 3.0 * x441 + x451) + x116 * x455)
    x473 = 0.06666666666666667 * x8
    x474 = 0.06666666666666667 * x7
    x475 = x474 * x73
    x476 = x7 * x73
    x477 = 0.009523809523809524 * x476
    x478 = 0.009523809523809524 * x249
    x479 = x421 * x74
    x480 = x432 * x73
    x481 = x128 * x27
    x482 = x22 * x73
    x483 = x117 * x389
    x484 = x334 * x458
    x485 = x118 * x484
    x486 = x468 * x75
    x487 = 0.009523809523809524 * x7
    x488 = x7 * x76
    x489 = x73 * x8
    x490 = x128 * x489
    x491 = x128 * x476
    x492 = x117 * x476
    x493 = x102 * x336**2
    x494 = x114 + x493
    x495 = x494 * x74
    x496 = x494 * x73
    x497 = x496 * x97
    x498 = x336 * x339
    x499 = x345 + x498
    x500 = x499 * x96
    x501 = x0 * (x183 + 2.0 * x338 + x493) + x100 * x499
    x502 = x501 * x74
    x503 = x118 * x121
    x504 = x121 * x73
    x505 = x336 * x354
    x506 = x371 + x505
    x507 = x117 * x356
    x508 = x336 * x360
    x509 = x362 + x508
    x510 = x0 * (x380 + x498 + x506)
    x511 = x100 * x509
    x512 = x510 + x511
    x513 = x152 * x509
    x514 = x0 * (x385 + x493)
    x515 = x139 * x506
    x516 = x514 + x515
    x517 = x153 * x516
    x518 = x139 * x509
    x519 = x510 + x518
    x520 = 2.0 * x508
    x521 = x0 * (x395 + x501 + x520)
    x522 = x139 * x512
    x523 = x521 + x522
    x524 = x158 * x516
    x525 = 2.0 * x0 * (2.0 * x371 + x372 + x505) + x139 * x516
    x526 = x0 * (x393 + x394 + x516 + x520)
    x527 = x139 * x519
    x528 = x526 + x527
    x529 = x461 * x528
    x530 = 2.0 * x0 * (x381 + x382 + 2.0 * x510 + x511 + x518) + x139 * x523
    x531 = x530 * x73
    x532 = x0 * (2.0 * x386 + 2.0 * x387 + 3.0 * x514 + 3.0 * x515) + x139 * x525
    x533 = x532 * x75
    x534 = x403 * (x0 * (x409 + 3.0 * x510 + 3.0 * x518 + x525) + x139 * x528)

    # 540 item(s)
    result[0, 0, 0] = numpy.sum(
        x81
        * (
            x0 * (2.0 * x37 + 2.0 * x50 + 2.0 * x58 + 2.0 * x64 + 3.0 * x67 + 3.0 * x70)
            + x12 * x72
        )
    )
    result[0, 0, 1] = numpy.sum(x85 * x98)
    result[0, 0, 2] = numpy.sum(x101 * x98)
    result[0, 0, 3] = numpy.sum(x102 * x106 * x111)
    result[0, 0, 4] = numpy.sum(x100 * x110 * x112)
    result[0, 0, 5] = numpy.sum(x103 * x111 * x115)
    result[0, 1, 0] = numpy.sum(x119 * x120)
    result[0, 1, 1] = numpy.sum(x125 * x126 * x127)
    result[0, 1, 2] = numpy.sum(x101 * x116 * x129)
    result[0, 1, 3] = numpy.sum(x127 * x134 * x135)
    result[0, 1, 4] = numpy.sum(x125 * x135 * x137)
    result[0, 1, 5] = numpy.sum(x115 * x121 * x138)
    result[0, 2, 0] = numpy.sum(x120 * x140)
    result[0, 2, 1] = numpy.sum(x129 * x139 * x85)
    result[0, 2, 2] = numpy.sum(x103 * x126 * x145)
    result[0, 2, 3] = numpy.sum(x106 * x138 * x141)
    result[0, 2, 4] = numpy.sum(x130 * x135 * x145)
    result[0, 2, 5] = numpy.sum(x138 * x148 * x149)
    result[0, 3, 0] = numpy.sum(x102 * x155 * x71)
    result[0, 3, 1] = numpy.sum(x157 * x159 * x63)
    result[0, 3, 2] = numpy.sum(x136 * x160 * x63)
    result[0, 3, 3] = numpy.sum(x127 * x165 * x167)
    result[0, 3, 4] = numpy.sum(x136 * x168 * x49)
    result[0, 3, 5] = numpy.sum(x115 * x151 * x167)
    result[0, 4, 0] = numpy.sum(x116 * x169 * x71 * x80)
    result[0, 4, 1] = numpy.sum(x170 * x171 * x63)
    result[0, 4, 2] = numpy.sum(x121 * x173 * x63)
    result[0, 4, 3] = numpy.sum(x133 * x171 * x49)
    result[0, 4, 4] = numpy.sum(x123 * x166 * x172)
    result[0, 4, 5] = numpy.sum(x121 * x174 * x49)
    result[0, 5, 0] = numpy.sum(x103 * x178 * x71)
    result[0, 5, 1] = numpy.sum(x130 * x179 * x63)
    result[0, 5, 2] = numpy.sum(x103 * x182 * x63)
    result[0, 5, 3] = numpy.sum(x106 * x167 * x176)
    result[0, 5, 4] = numpy.sum(x130 * x182 * x49)
    result[0, 5, 5] = numpy.sum(x149 * x167 * x187)
    result[0, 6, 0] = numpy.sum(x102 * x191 * x69)
    result[0, 6, 1] = numpy.sum(x127 * x195 * x61)
    result[0, 6, 2] = numpy.sum(x136 * x196 * x61)
    result[0, 6, 3] = numpy.sum(x127 * x197 * x199)
    result[0, 6, 4] = numpy.sum(x137 * x195 * x47)
    result[0, 6, 5] = numpy.sum(x115 * x189 * x198)
    result[0, 7, 0] = numpy.sum(x141 * x160 * x69)
    result[0, 7, 1] = numpy.sum(x168 * x200 * x61)
    result[0, 7, 2] = numpy.sum(x160 * x201 * x61)
    result[0, 7, 3] = numpy.sum(x141 * x165 * x202)
    result[0, 7, 4] = numpy.sum(x157 * x201 * x202)
    result[0, 7, 5] = numpy.sum(x148 * x151 * x202)
    result[0, 8, 0] = numpy.sum(x121 * x179 * x69)
    result[0, 8, 1] = numpy.sum(x170 * x179 * x61)
    result[0, 8, 2] = numpy.sum(x121 * x203 * x61)
    result[0, 8, 3] = numpy.sum(x133 * x176 * x202)
    result[0, 8, 4] = numpy.sum(x170 * x181 * x202)
    result[0, 8, 5] = numpy.sum(x121 * x187 * x202)
    result[0, 9, 0] = numpy.sum(x103 * x207 * x69)
    result[0, 9, 1] = numpy.sum(x130 * x208 * x61)
    result[0, 9, 2] = numpy.sum(x149 * x212 * x61)
    result[0, 9, 3] = numpy.sum(x106 * x198 * x205)
    result[0, 9, 4] = numpy.sum(x130 * x213 * x47)
    result[0, 9, 5] = numpy.sum(x149 * x199 * x214)
    result[0, 10, 0] = numpy.sum(x102 * x215 * x217)
    result[0, 10, 1] = numpy.sum(x218 * x219 * x220)
    result[0, 10, 2] = numpy.sum(x137 * x215 * x220)
    result[0, 10, 3] = numpy.sum(x219 * x221 * x223)
    result[0, 10, 4] = numpy.sum(x136 * x218 * x225)
    result[0, 10, 5] = numpy.sum(x115 * x215 * x223)
    result[0, 11, 0] = numpy.sum(x141 * x191 * x66)
    result[0, 11, 1] = numpy.sum(x195 * x226 * x65)
    result[0, 11, 2] = numpy.sum(x144 * x190 * x65)
    result[0, 11, 3] = numpy.sum(x141 * x227 * x228)
    result[0, 11, 4] = numpy.sum(x144 * x194 * x227)
    result[0, 11, 5] = numpy.sum(x148 * x191 * x222)
    result[0, 12, 0] = numpy.sum(x151 * x177 * x216)
    result[0, 12, 1] = numpy.sum(x157 * x176 * x229)
    result[0, 12, 2] = numpy.sum(x151 * x181 * x229)
    result[0, 12, 3] = numpy.sum(x165 * x177 * x224)
    result[0, 12, 4] = numpy.sum(x157 * x182 * x222)
    result[0, 12, 5] = numpy.sum(x154 * x187 * x224)
    result[0, 13, 0] = numpy.sum(x205 * x216 * x230)
    result[0, 13, 1] = numpy.sum(x125 * x206 * x65)
    result[0, 13, 2] = numpy.sum(x121 * x213 * x65)
    result[0, 13, 3] = numpy.sum(x134 * x205 * x224)
    result[0, 13, 4] = numpy.sum(x125 * x211 * x227)
    result[0, 13, 5] = numpy.sum(x214 * x227 * x230)
    result[0, 14, 0] = numpy.sum(x103 * x217 * x231)
    result[0, 14, 1] = numpy.sum(x130 * x220 * x232)
    result[0, 14, 2] = numpy.sum(x220 * x233 * x234)
    result[0, 14, 3] = numpy.sum(x106 * x223 * x231)
    result[0, 14, 4] = numpy.sum(x130 * x225 * x233)
    result[0, 14, 5] = numpy.sum(x223 * x234 * x235)
    result[1, 0, 0] = numpy.sum(x236 * x239)
    result[1, 0, 1] = numpy.sum(x245 * x249 * x250)
    result[1, 0, 2] = numpy.sum(x101 * x236 * x245 * x97)
    result[1, 0, 3] = numpy.sum(x254 * x257 * x258)
    result[1, 0, 4] = numpy.sum(x249 * x257 * x259)
    result[1, 0, 5] = numpy.sum(x251 * x257 * x260)
    result[1, 1, 0] = numpy.sum(x238 * x263 * x264)
    result[1, 1, 1] = numpy.sum(x244 * x266 * x267)
    result[1, 1, 2] = numpy.sum(x244 * x262 * x268)
    result[1, 1, 3] = numpy.sum(x271 * x272 * x273)
    result[1, 1, 4] = numpy.sum(x137 * x266 * x274)
    result[1, 1, 5] = numpy.sum(x115 * x263 * x272)
    result[1, 2, 0] = numpy.sum(x140 * x236 * x275)
    result[1, 2, 1] = numpy.sum(x141 * x244 * x276)
    result[1, 2, 2] = numpy.sum(x145 * x244 * x277)
    result[1, 2, 3] = numpy.sum(x226 * x254 * x278)
    result[1, 2, 4] = numpy.sum(x144 * x248 * x272)
    result[1, 2, 5] = numpy.sum(x148 * x278 * x279)
    result[1, 3, 0] = numpy.sum(x127 * x283 * x57)
    result[1, 3, 1] = numpy.sum(x159 * x285 * x92)
    result[1, 3, 2] = numpy.sum(x282 * x286 * x92)
    result[1, 3, 3] = numpy.sum(x127 * x291 * x292)
    result[1, 3, 4] = numpy.sum(x285 * x286 * x88)
    result[1, 3, 5] = numpy.sum(x115 * x282 * x292)
    result[1, 4, 0] = numpy.sum(x171 * x262 * x57)
    result[1, 4, 1] = numpy.sum(x171 * x293 * x92)
    result[1, 4, 2] = numpy.sum(x173 * x262 * x92)
    result[1, 4, 3] = numpy.sum(x171 * x271 * x88)
    result[1, 4, 4] = numpy.sum(x173 * x266 * x88)
    result[1, 4, 5] = numpy.sum(x174 * x262 * x88)
    result[1, 5, 0] = numpy.sum(x178 * x251 * x57)
    result[1, 5, 1] = numpy.sum(x179 * x247 * x92)
    result[1, 5, 2] = numpy.sum(x182 * x251 * x92)
    result[1, 5, 3] = numpy.sum(x178 * x254 * x88)
    result[1, 5, 4] = numpy.sum(x182 * x247 * x88)
    result[1, 5, 5] = numpy.sum(x187 * x279 * x292)
    result[1, 6, 0] = numpy.sum(x273 * x298 * x55)
    result[1, 6, 1] = numpy.sum(x17 * x267 * x301)
    result[1, 6, 2] = numpy.sum(x137 * x17 * x302)
    result[1, 6, 3] = numpy.sum(x264 * x308 * x309)
    result[1, 6, 4] = numpy.sum(x268 * x301 * x45)
    result[1, 6, 5] = numpy.sum(x115 * x298 * x309)
    result[1, 7, 0] = numpy.sum(x171 * x282 * x55)
    result[1, 7, 1] = numpy.sum(x171 * x285 * x310)
    result[1, 7, 2] = numpy.sum(x17 * x173 * x282)
    result[1, 7, 3] = numpy.sum(x141 * x291 * x311)
    result[1, 7, 4] = numpy.sum(x201 * x285 * x311)
    result[1, 7, 5] = numpy.sum(x174 * x282 * x45)
    result[1, 8, 0] = numpy.sum(x179 * x262 * x55)
    result[1, 8, 1] = numpy.sum(x179 * x266 * x310)
    result[1, 8, 2] = numpy.sum(x182 * x262 * x310)
    result[1, 8, 3] = numpy.sum(x179 * x271 * x45)
    result[1, 8, 4] = numpy.sum(x203 * x266 * x45)
    result[1, 8, 5] = numpy.sum(x187 * x262 * x311)
    result[1, 9, 0] = numpy.sum(x207 * x251 * x55)
    result[1, 9, 1] = numpy.sum(x17 * x208 * x247)
    result[1, 9, 2] = numpy.sum(x17 * x213 * x251)
    result[1, 9, 3] = numpy.sum(x206 * x254 * x309)
    result[1, 9, 4] = numpy.sum(x212 * x248 * x45)
    result[1, 9, 5] = numpy.sum(x277 * x309 * x312)
    result[1, 10, 0] = numpy.sum(x258 * x313 * x314)
    result[1, 10, 1] = numpy.sum(x250 * x315 * x317)
    result[1, 10, 2] = numpy.sum(x259 * x314 * x317)
    result[1, 10, 3] = numpy.sum(x321 * x9)
    result[1, 10, 4] = numpy.sum(x323 * x325 * x9)
    result[1, 10, 5] = numpy.sum(x10 * x260 * x314)
    result[1, 11, 0] = numpy.sum(x226 * x313 * x326)
    result[1, 11, 1] = numpy.sum(x141 * x301 * x327)
    result[1, 11, 2] = numpy.sum(x144 * x298 * x316)
    result[1, 11, 3] = numpy.sum(x308 * x328 * x329)
    result[1, 11, 4] = numpy.sum(x145 * x301 * x330)
    result[1, 11, 5] = numpy.sum(x148 * x326 * x331)
    result[1, 12, 0] = numpy.sum(x178 * x282 * x313)
    result[1, 12, 1] = numpy.sum(x15 * x179 * x285)
    result[1, 12, 2] = numpy.sum(x15 * x182 * x282)
    result[1, 12, 3] = numpy.sum(x10 * x178 * x291)
    result[1, 12, 4] = numpy.sum(x10 * x182 * x285)
    result[1, 12, 5] = numpy.sum(x187 * x283 * x331)
    result[1, 13, 0] = numpy.sum(x206 * x263 * x313)
    result[1, 13, 1] = numpy.sum(x15 * x208 * x266)
    result[1, 13, 2] = numpy.sum(x212 * x262 * x316)
    result[1, 13, 3] = numpy.sum(x10 * x207 * x271)
    result[1, 13, 4] = numpy.sum(x10 * x213 * x266)
    result[1, 13, 5] = numpy.sum(x263 * x312 * x330)
    result[1, 14, 0] = numpy.sum(x251 * x313 * x332)
    result[1, 14, 1] = numpy.sum(x15 * x231 * x249)
    result[1, 14, 2] = numpy.sum(x251 * x317 * x333)
    result[1, 14, 3] = numpy.sum(x10 * x254 * x332)
    result[1, 14, 4] = numpy.sum(x10 * x249 * x333)
    result[1, 14, 5] = numpy.sum(x235 * x236 * x335 * x9)
    result[2, 0, 0] = numpy.sum(x239 * x336)
    result[2, 0, 1] = numpy.sum(x112 * x245 * x336)
    result[2, 0, 2] = numpy.sum(x245 * x337 * x341)
    result[2, 0, 3] = numpy.sum(x257 * x342 * x343)
    result[2, 0, 4] = numpy.sum(x130 * x257 * x344)
    result[2, 0, 5] = numpy.sum(x257 * x347 * x348)
    result[2, 1, 0] = numpy.sum(x119 * x275 * x336)
    result[2, 1, 1] = numpy.sum(x125 * x244 * x350)
    result[2, 1, 2] = numpy.sum(x121 * x244 * x351)
    result[2, 1, 3] = numpy.sum(x134 * x272 * x349)
    result[2, 1, 4] = numpy.sum(x125 * x272 * x340)
    result[2, 1, 5] = numpy.sum(x272 * x347 * x352)
    result[2, 2, 0] = numpy.sum(x238 * x355 * x356)
    result[2, 2, 1] = numpy.sum(x244 * x354 * x358)
    result[2, 2, 2] = numpy.sum(x244 * x360 * x361)
    result[2, 2, 3] = numpy.sum(x106 * x272 * x355)
    result[2, 2, 4] = numpy.sum(x274 * x357 * x360)
    result[2, 2, 5] = numpy.sum(x272 * x364 * x365)
    result[2, 3, 0] = numpy.sum(x155 * x342 * x57)
    result[2, 3, 1] = numpy.sum(x168 * x342 * x92)
    result[2, 3, 2] = numpy.sum(x151 * x366 * x92)
    result[2, 3, 3] = numpy.sum(x165 * x292 * x349)
    result[2, 3, 4] = numpy.sum(x157 * x366 * x88)
    result[2, 3, 5] = numpy.sum(x155 * x347 * x88)
    result[2, 4, 0] = numpy.sum(x121 * x367 * x57)
    result[2, 4, 1] = numpy.sum(x170 * x367 * x92)
    result[2, 4, 2] = numpy.sum(x121 * x369 * x92)
    result[2, 4, 3] = numpy.sum(x133 * x367 * x88)
    result[2, 4, 4] = numpy.sum(x170 * x368 * x88)
    result[2, 4, 5] = numpy.sum(x121 * x370 * x88)
    result[2, 5, 0] = numpy.sum(x149 * x374 * x57)
    result[2, 5, 1] = numpy.sum(x130 * x375 * x92)
    result[2, 5, 2] = numpy.sum(x103 * x378 * x92)
    result[2, 5, 3] = numpy.sum(x106 * x292 * x373)
    result[2, 5, 4] = numpy.sum(x130 * x378 * x88)
    result[2, 5, 5] = numpy.sum(x149 * x292 * x383)
    result[2, 6, 0] = numpy.sum(x191 * x342 * x55)
    result[2, 6, 1] = numpy.sum(x17 * x195 * x349)
    result[2, 6, 2] = numpy.sum(x17 * x196 * x339)
    result[2, 6, 3] = numpy.sum(x197 * x309 * x350)
    result[2, 6, 4] = numpy.sum(x195 * x340 * x45)
    result[2, 6, 5] = numpy.sum(x191 * x347 * x45)
    result[2, 7, 0] = numpy.sum(x151 * x367 * x55)
    result[2, 7, 1] = numpy.sum(x157 * x310 * x367)
    result[2, 7, 2] = numpy.sum(x160 * x310 * x360)
    result[2, 7, 3] = numpy.sum(x165 * x311 * x354)
    result[2, 7, 4] = numpy.sum(x157 * x311 * x384)
    result[2, 7, 5] = numpy.sum(x151 * x311 * x364)
    result[2, 8, 0] = numpy.sum(x121 * x375 * x55)
    result[2, 8, 1] = numpy.sum(x17 * x170 * x375)
    result[2, 8, 2] = numpy.sum(x121 * x310 * x378)
    result[2, 8, 3] = numpy.sum(x133 * x311 * x373)
    result[2, 8, 4] = numpy.sum(x170 * x311 * x377)
    result[2, 8, 5] = numpy.sum(x121 * x311 * x383)
    result[2, 9, 0] = numpy.sum(x365 * x389 * x55)
    result[2, 9, 1] = numpy.sum(x17 * x358 * x388)
    result[2, 9, 2] = numpy.sum(x17 * x361 * x392)
    result[2, 9, 3] = numpy.sum(x106 * x309 * x389)
    result[2, 9, 4] = numpy.sum(x358 * x392 * x45)
    result[2, 9, 5] = numpy.sum(x309 * x356 * x398)
    result[2, 10, 0] = numpy.sum(x313 * x349 * x399)
    result[2, 10, 1] = numpy.sum(x218 * x342 * x400)
    result[2, 10, 2] = numpy.sum(x215 * x317 * x339)
    result[2, 10, 3] = numpy.sum(x221 * x320 * x336 * x9)
    result[2, 10, 4] = numpy.sum(x10 * x218 * x344)
    result[2, 10, 5] = numpy.sum(x331 * x347 * x399)
    result[2, 11, 0] = numpy.sum(x191 * x313 * x354)
    result[2, 11, 1] = numpy.sum(x195 * x316 * x354)
    result[2, 11, 2] = numpy.sum(x15 * x196 * x360)
    result[2, 11, 3] = numpy.sum(x197 * x355 * x401)
    result[2, 11, 4] = numpy.sum(x195 * x331 * x360)
    result[2, 11, 5] = numpy.sum(x10 * x191 * x364)
    result[2, 12, 0] = numpy.sum(x155 * x313 * x373)
    result[2, 12, 1] = numpy.sum(x157 * x373 * x402)
    result[2, 12, 2] = numpy.sum(x151 * x377 * x402)
    result[2, 12, 3] = numpy.sum(x165 * x331 * x374)
    result[2, 12, 4] = numpy.sum(x10 * x157 * x378)
    result[2, 12, 5] = numpy.sum(x10 * x155 * x383)
    result[2, 13, 0] = numpy.sum(x313 * x352 * x389)
    result[2, 13, 1] = numpy.sum(x125 * x316 * x389)
    result[2, 13, 2] = numpy.sum(x121 * x327 * x392)
    result[2, 13, 3] = numpy.sum(x134 * x331 * x389)
    result[2, 13, 4] = numpy.sum(x125 * x392 * x401)
    result[2, 13, 5] = numpy.sum(x329 * x398 * x404)
    result[2, 14, 0] = numpy.sum(x313 * x348 * x405)
    result[2, 14, 1] = numpy.sum(x130 * x400 * x405)
    result[2, 14, 2] = numpy.sum(x317 * x337 * x406)
    result[2, 14, 3] = numpy.sum(x10 * x343 * x405)
    result[2, 14, 4] = numpy.sum(x407 * x408 * x9)
    result[2, 14, 5] = numpy.sum(x410 * x9)
    result[3, 0, 0] = numpy.sum(x219 * x411 * x414)
    result[3, 0, 1] = numpy.sum(x219 * x415 * x418)
    result[3, 0, 2] = numpy.sum(x136 * x415 * x420)
    result[3, 0, 3] = numpy.sum(x127 * x421 * x423)
    result[3, 0, 4] = numpy.sum(x137 * x418 * x421)
    result[3, 0, 5] = numpy.sum(x115 * x414 * x421)
    result[3, 1, 0] = numpy.sum(x237 * x264 * x426)
    result[3, 1, 1] = numpy.sum(x243 * x267 * x428)
    result[3, 1, 2] = numpy.sum(x243 * x268 * x425)
    result[3, 1, 3] = numpy.sum(x127 * x431 * x433)
    result[3, 1, 4] = numpy.sum(x256 * x268 * x428)
    result[3, 1, 5] = numpy.sum(x115 * x426 * x432)
    result[3, 2, 0] = numpy.sum(x141 * x237 * x434)
    result[3, 2, 1] = numpy.sum(x226 * x417 * x435)
    result[3, 2, 2] = numpy.sum(x145 * x413 * x436)
    result[3, 2, 3] = numpy.sum(x226 * x422 * x433)
    result[3, 2, 4] = numpy.sum(x145 * x417 * x432)
    result[3, 2, 5] = numpy.sum(x148 * x419 * x433)
    result[3, 3, 0] = numpy.sum(x127 * x31 * x440)
    result[3, 3, 1] = numpy.sum(x102 * x442 * x443)
    result[3, 3, 2] = numpy.sum(x136 * x439 * x443)
    result[3, 3, 3] = numpy.sum(x102 * x447 * x449)
    result[3, 3, 4] = numpy.sum(x136 * x442 * x450)
    result[3, 3, 5] = numpy.sum(x115 * x439 * x448)
    result[3, 4, 0] = numpy.sum(x171 * x31 * x425)
    result[3, 4, 1] = numpy.sum(x200 * x428 * x443)
    result[3, 4, 2] = numpy.sum(x201 * x425 * x443)
    result[3, 4, 3] = numpy.sum(x141 * x431 * x450)
    result[3, 4, 4] = numpy.sum(x201 * x428 * x450)
    result[3, 4, 5] = numpy.sum(x174 * x255 * x425)
    result[3, 5, 0] = numpy.sum(x178 * x31 * x413)
    result[3, 5, 1] = numpy.sum(x179 * x417 * x90)
    result[3, 5, 2] = numpy.sum(x182 * x413 * x90)
    result[3, 5, 3] = numpy.sum(x178 * x255 * x422)
    result[3, 5, 4] = numpy.sum(x182 * x255 * x417)
    result[3, 5, 5] = numpy.sum(x187 * x419 * x448)
    result[3, 6, 0] = numpy.sum(x264 * x29 * x452)
    result[3, 6, 1] = numpy.sum(x267 * x27 * x455)
    result[3, 6, 2] = numpy.sum(x268 * x27 * x451)
    result[3, 6, 3] = numpy.sum(x457 * x460)
    result[3, 6, 4] = numpy.sum(x100 * x459 * x462)
    result[3, 6, 5] = numpy.sum(x115 * x22 * x463)
    result[3, 7, 0] = numpy.sum(x171 * x29 * x439)
    result[3, 7, 1] = numpy.sum(x141 * x442 * x465)
    result[3, 7, 2] = numpy.sum(x173 * x27 * x439)
    result[3, 7, 3] = numpy.sum(x169 * x447 * x459)
    result[3, 7, 4] = numpy.sum(x173 * x22 * x442)
    result[3, 7, 5] = numpy.sum(x174 * x22 * x439)
    result[3, 8, 0] = numpy.sum(x179 * x29 * x425)
    result[3, 8, 1] = numpy.sum(x179 * x428 * x464)
    result[3, 8, 2] = numpy.sum(x203 * x27 * x425)
    result[3, 8, 3] = numpy.sum(x179 * x22 * x431)
    result[3, 8, 4] = numpy.sum(x203 * x22 * x428)
    result[3, 8, 5] = numpy.sum(x187 * x425 * x466)
    result[3, 9, 0] = numpy.sum(x207 * x29 * x413)
    result[3, 9, 1] = numpy.sum(x208 * x27 * x417)
    result[3, 9, 2] = numpy.sum(x212 * x27 * x419)
    result[3, 9, 3] = numpy.sum(x207 * x22 * x422)
    result[3, 9, 4] = numpy.sum(x213 * x22 * x417)
    result[3, 9, 5] = numpy.sum(x214 * x22 * x434)
    result[3, 10, 0] = numpy.sum(x102 * x468 * x470)
    result[3, 10, 1] = numpy.sum(x471 * x472)
    result[3, 10, 2] = numpy.sum(x100 * x324 * x469 * x471)
    result[3, 10, 3] = numpy.sum(
        x320
        * (
            x0
            * (
                2.0 * x306
                + 2.0 * x307
                + 3.0 * x445
                + 3.0 * x446
                + 2.0 * x453
                + 2.0 * x454
            )
            + x116 * x456
        )
    )
    result[3, 10, 4] = numpy.sum(x323 * x472)
    result[3, 10, 5] = numpy.sum(x115 * x470 * x7)
    result[3, 11, 0] = numpy.sum(x141 * x463 * x468)
    result[3, 11, 1] = numpy.sum(x139 * x3 * x324 * x462)
    result[3, 11, 2] = numpy.sum(x145 * x451 * x473)
    result[3, 11, 3] = numpy.sum(x328 * x457)
    result[3, 11, 4] = numpy.sum(x145 * x455 * x474)
    result[3, 11, 5] = numpy.sum(x148 * x452 * x475)
    result[3, 12, 0] = numpy.sum(x178 * x439 * x467)
    result[3, 12, 1] = numpy.sum(x179 * x442 * x8)
    result[3, 12, 2] = numpy.sum(x182 * x439 * x8)
    result[3, 12, 3] = numpy.sum(x178 * x447 * x7)
    result[3, 12, 4] = numpy.sum(x182 * x442 * x7)
    result[3, 12, 5] = numpy.sum(x187 * x440 * x476)
    result[3, 13, 0] = numpy.sum(x206 * x426 * x467)
    result[3, 13, 1] = numpy.sum(x208 * x428 * x8)
    result[3, 13, 2] = numpy.sum(x213 * x425 * x8)
    result[3, 13, 3] = numpy.sum(x207 * x431 * x7)
    result[3, 13, 4] = numpy.sum(x212 * x428 * x476)
    result[3, 13, 5] = numpy.sum(x214 * x426 * x475)
    result[3, 14, 0] = numpy.sum(x231 * x414 * x468)
    result[3, 14, 1] = numpy.sum(x232 * x418 * x8)
    result[3, 14, 2] = numpy.sum(x233 * x420 * x8)
    result[3, 14, 3] = numpy.sum(x231 * x423 * x476)
    result[3, 14, 4] = numpy.sum(x233 * x418 * x477)
    result[3, 14, 5] = numpy.sum(x235 * x414 * x477)
    result[4, 0, 0] = numpy.sum(x236 * x336 * x411 * x81)
    result[4, 0, 1] = numpy.sum(x342 * x415 * x478)
    result[4, 0, 2] = numpy.sum(x251 * x344 * x415)
    result[4, 0, 3] = numpy.sum(x254 * x349 * x479)
    result[4, 0, 4] = numpy.sum(x249 * x339 * x421)
    result[4, 0, 5] = numpy.sum(x279 * x347 * x479)
    result[4, 1, 0] = numpy.sum(x237 * x263 * x350)
    result[4, 1, 1] = numpy.sum(x266 * x349 * x435)
    result[4, 1, 2] = numpy.sum(x243 * x262 * x351)
    result[4, 1, 3] = numpy.sum(x271 * x349 * x433)
    result[4, 1, 4] = numpy.sum(x256 * x266 * x351)
    result[4, 1, 5] = numpy.sum(x263 * x347 * x480)
    result[4, 2, 0] = numpy.sum(x237 * x277 * x355 * x73)
    result[4, 2, 1] = numpy.sum(x243 * x276 * x354)
    result[4, 2, 2] = numpy.sum(x279 * x360 * x435)
    result[4, 2, 3] = numpy.sum(x254 * x355 * x480)
    result[4, 2, 4] = numpy.sum(x256 * x276 * x360)
    result[4, 2, 5] = numpy.sum(x279 * x364 * x433)
    result[4, 3, 0] = numpy.sum(x283 * x31 * x349)
    result[4, 3, 1] = numpy.sum(x285 * x342 * x443)
    result[4, 3, 2] = numpy.sum(x282 * x366 * x90)
    result[4, 3, 3] = numpy.sum(x291 * x342 * x449)
    result[4, 3, 4] = numpy.sum(x255 * x285 * x366)
    result[4, 3, 5] = numpy.sum(x282 * x347 * x449)
    result[4, 4, 0] = numpy.sum(x262 * x31 * x367)
    result[4, 4, 1] = numpy.sum(x293 * x367 * x90)
    result[4, 4, 2] = numpy.sum(x262 * x384 * x443)
    result[4, 4, 3] = numpy.sum(x255 * x271 * x367)
    result[4, 4, 4] = numpy.sum(x266 * x384 * x450)
    result[4, 4, 5] = numpy.sum(x262 * x364 * x450)
    result[4, 5, 0] = numpy.sum(x279 * x31 * x374)
    result[4, 5, 1] = numpy.sum(x247 * x375 * x90)
    result[4, 5, 2] = numpy.sum(x251 * x377 * x443)
    result[4, 5, 3] = numpy.sum(x254 * x373 * x449)
    result[4, 5, 4] = numpy.sum(x247 * x377 * x450)
    result[4, 5, 5] = numpy.sum(x251 * x383 * x449)
    result[4, 6, 0] = numpy.sum(x29 * x326 * x349)
    result[4, 6, 1] = numpy.sum(x301 * x349 * x481)
    result[4, 6, 2] = numpy.sum(x27 * x302 * x340)
    result[4, 6, 3] = numpy.sum(x308 * x336 * x460 * x73)
    result[4, 6, 4] = numpy.sum(x22 * x301 * x351)
    result[4, 6, 5] = numpy.sum(x326 * x347 * x482)
    result[4, 7, 0] = numpy.sum(x282 * x29 * x367)
    result[4, 7, 1] = numpy.sum(x285 * x367 * x464)
    result[4, 7, 2] = numpy.sum(x282 * x368 * x464)
    result[4, 7, 3] = numpy.sum(x22 * x291 * x367)
    result[4, 7, 4] = numpy.sum(x22 * x285 * x369)
    result[4, 7, 5] = numpy.sum(x22 * x282 * x370)
    result[4, 8, 0] = numpy.sum(x262 * x29 * x375)
    result[4, 8, 1] = numpy.sum(x266 * x375 * x464)
    result[4, 8, 2] = numpy.sum(x262 * x377 * x465)
    result[4, 8, 3] = numpy.sum(x22 * x271 * x375)
    result[4, 8, 4] = numpy.sum(x22 * x293 * x378)
    result[4, 8, 5] = numpy.sum(x262 * x383 * x466)
    result[4, 9, 0] = numpy.sum(x279 * x29 * x483)
    result[4, 9, 1] = numpy.sum(x27 * x276 * x388)
    result[4, 9, 2] = numpy.sum(x279 * x392 * x481)
    result[4, 9, 3] = numpy.sum(x254 * x482 * x483)
    result[4, 9, 4] = numpy.sum(x22 * x276 * x392)
    result[4, 9, 5] = numpy.sum(x236 * x398 * x485 * x73)
    result[4, 10, 0] = numpy.sum(x314 * x342 * x486)
    result[4, 10, 1] = numpy.sum(x325 * x336 * x471)
    result[4, 10, 2] = numpy.sum(x314 * x344 * x8)
    result[4, 10, 3] = numpy.sum(x321 * x336)
    result[4, 10, 4] = numpy.sum(x315 * x341 * x487)
    result[4, 10, 5] = numpy.sum(x314 * x347 * x488)
    result[4, 11, 0] = numpy.sum(x298 * x355 * x468)
    result[4, 11, 1] = numpy.sum(x301 * x354 * x490)
    result[4, 11, 2] = numpy.sum(x302 * x360 * x489)
    result[4, 11, 3] = numpy.sum(x308 * x355 * x475)
    result[4, 11, 4] = numpy.sum(x301 * x360 * x491)
    result[4, 11, 5] = numpy.sum(x298 * x364 * x492)
    result[4, 12, 0] = numpy.sum(x283 * x373 * x468)
    result[4, 12, 1] = numpy.sum(x285 * x375 * x8)
    result[4, 12, 2] = numpy.sum(x282 * x378 * x8)
    result[4, 12, 3] = numpy.sum(x291 * x374 * x476)
    result[4, 12, 4] = numpy.sum(x285 * x378 * x7)
    result[4, 12, 5] = numpy.sum(x283 * x383 * x476)
    result[4, 13, 0] = numpy.sum(x263 * x389 * x468)
    result[4, 13, 1] = numpy.sum(x266 * x388 * x490)
    result[4, 13, 2] = numpy.sum(x262 * x392 * x490)
    result[4, 13, 3] = numpy.sum(x271 * x389 * x492)
    result[4, 13, 4] = numpy.sum(x266 * x392 * x491)
    result[4, 13, 5] = numpy.sum(x263 * x398 * x475)
    result[4, 14, 0] = numpy.sum(x251 * x405 * x486)
    result[4, 14, 1] = numpy.sum(x405 * x478 * x8)
    result[4, 14, 2] = numpy.sum(x236 * x408 * x471)
    result[4, 14, 3] = numpy.sum(x254 * x405 * x488)
    result[4, 14, 4] = numpy.sum(x249 * x406 * x487)
    result[4, 14, 5] = numpy.sum(x236 * x410)
    result[5, 0, 0] = numpy.sum(x234 * x411 * x495)
    result[5, 0, 1] = numpy.sum(x130 * x415 * x497)
    result[5, 0, 2] = numpy.sum(x234 * x415 * x500)
    result[5, 0, 3] = numpy.sum(x106 * x421 * x495)
    result[5, 0, 4] = numpy.sum(x357 * x421 * x500)
    result[5, 0, 5] = numpy.sum(x149 * x421 * x502)
    result[5, 1, 0] = numpy.sum(x237 * x496 * x503)
    result[5, 1, 1] = numpy.sum(x125 * x436 * x496)
    result[5, 1, 2] = numpy.sum(x435 * x499 * x504)
    result[5, 1, 3] = numpy.sum(x134 * x480 * x494)
    result[5, 1, 4] = numpy.sum(x125 * x480 * x499)
    result[5, 1, 5] = numpy.sum(x433 * x501 * x504)
    result[5, 2, 0] = numpy.sum(x237 * x506 * x507)
    result[5, 2, 1] = numpy.sum(x243 * x358 * x506)
    result[5, 2, 2] = numpy.sum(x243 * x361 * x509)
    result[5, 2, 3] = numpy.sum(x106 * x433 * x506)
    result[5, 2, 4] = numpy.sum(x256 * x358 * x509)
    result[5, 2, 5] = numpy.sum(x149 * x433 * x512)
    result[5, 3, 0] = numpy.sum(x155 * x31 * x494)
    result[5, 3, 1] = numpy.sum(x157 * x443 * x494)
    result[5, 3, 2] = numpy.sum(x151 * x443 * x499)
    result[5, 3, 3] = numpy.sum(x165 * x449 * x494)
    result[5, 3, 4] = numpy.sum(x157 * x450 * x499)
    result[5, 3, 5] = numpy.sum(x155 * x255 * x501)
    result[5, 4, 0] = numpy.sum(x121 * x158 * x31 * x506)
    result[5, 4, 1] = numpy.sum(x170 * x443 * x506)
    result[5, 4, 2] = numpy.sum(x121 * x443 * x513)
    result[5, 4, 3] = numpy.sum(x133 * x450 * x506)
    result[5, 4, 4] = numpy.sum(x170 * x450 * x509)
    result[5, 4, 5] = numpy.sum(x121 * x450 * x512)
    result[5, 5, 0] = numpy.sum(x149 * x31 * x517)
    result[5, 5, 1] = numpy.sum(x130 * x443 * x516)
    result[5, 5, 2] = numpy.sum(x103 * x443 * x519)
    result[5, 5, 3] = numpy.sum(x106 * x448 * x516)
    result[5, 5, 4] = numpy.sum(x130 * x450 * x519)
    result[5, 5, 5] = numpy.sum(x103 * x449 * x523)
    result[5, 6, 0] = numpy.sum(x191 * x29 * x494)
    result[5, 6, 1] = numpy.sum(x195 * x27 * x496)
    result[5, 6, 2] = numpy.sum(x196 * x27 * x499)
    result[5, 6, 3] = numpy.sum(0.06666666666666667 * x22 * x228 * x496)
    result[5, 6, 4] = numpy.sum(x195 * x482 * x499)
    result[5, 6, 5] = numpy.sum(x191 * x22 * x501)
    result[5, 7, 0] = numpy.sum(x160 * x29 * x506)
    result[5, 7, 1] = numpy.sum(x157 * x465 * x506)
    result[5, 7, 2] = numpy.sum(x160 * x464 * x509)
    result[5, 7, 3] = numpy.sum(x165 * x466 * x506)
    result[5, 7, 4] = numpy.sum(x168 * x22 * x513)
    result[5, 7, 5] = numpy.sum(x160 * x22 * x512)
    result[5, 8, 0] = numpy.sum(x121 * x29 * x524)
    result[5, 8, 1] = numpy.sum(x170 * x27 * x524)
    result[5, 8, 2] = numpy.sum(x121 * x465 * x519)
    result[5, 8, 3] = numpy.sum(x133 * x22 * x524)
    result[5, 8, 4] = numpy.sum(x170 * x466 * x519)
    result[5, 8, 5] = numpy.sum(x116 * x158 * x484 * x523)
    result[5, 9, 0] = numpy.sum(x29 * x507 * x525)
    result[5, 9, 1] = numpy.sum(x27 * x358 * x525)
    result[5, 9, 2] = numpy.sum(x27 * x361 * x528)
    result[5, 9, 3] = numpy.sum(x106 * x118 * x22 * x525)
    result[5, 9, 4] = numpy.sum(x484 * x529 * x83)
    result[5, 9, 5] = numpy.sum(x485 * x531)
    result[5, 10, 0] = numpy.sum(x215 * x468 * x495)
    result[5, 10, 1] = numpy.sum(x218 * x497 * x8)
    result[5, 10, 2] = numpy.sum(x215 * x489 * x500)
    result[5, 10, 3] = numpy.sum(x221 * x477 * x495)
    result[5, 10, 4] = numpy.sum(x218 * x477 * x500)
    result[5, 10, 5] = numpy.sum(x215 * x476 * x502)
    result[5, 11, 0] = numpy.sum(x191 * x467 * x506)
    result[5, 11, 1] = numpy.sum(x195 * x489 * x506)
    result[5, 11, 2] = numpy.sum(x196 * x509 * x8)
    result[5, 11, 3] = numpy.sum(x228 * x475 * x506)
    result[5, 11, 4] = numpy.sum(x195 * x476 * x509)
    result[5, 11, 5] = numpy.sum(x191 * x512 * x7)
    result[5, 12, 0] = numpy.sum(x155 * x467 * x516)
    result[5, 12, 1] = numpy.sum(x157 * x524 * x8)
    result[5, 12, 2] = numpy.sum(x160 * x519 * x8)
    result[5, 12, 3] = numpy.sum(x165 * x476 * x517)
    result[5, 12, 4] = numpy.sum(x168 * x519 * x7)
    result[5, 12, 5] = numpy.sum(x155 * x523 * x7)
    result[5, 13, 0] = numpy.sum(x468 * x503 * x525)
    result[5, 13, 1] = numpy.sum(x125 * x473 * x525 * x73)
    result[5, 13, 2] = numpy.sum(x116 * x3 * x403 * x529)
    result[5, 13, 3] = numpy.sum(x134 * x475 * x525)
    result[5, 13, 4] = numpy.sum(x125 * x475 * x528)
    result[5, 13, 5] = numpy.sum(x404 * x531)
    result[5, 14, 0] = numpy.sum(x103 * x468 * x533)
    result[5, 14, 1] = numpy.sum(x403 * x471 * x532 * x83)
    result[5, 14, 2] = numpy.sum(x471 * x534)
    result[5, 14, 3] = numpy.sum(x106 * x533 * x7)
    result[5, 14, 4] = numpy.sum(x407 * x534)
    result[5, 14, 5] = numpy.sum(
        x335
        * (
            x0
            * (
                2.0 * x396
                + 2.0 * x397
                + 3.0 * x521
                + 3.0 * x522
                + 2.0 * x526
                + 2.0 * x527
            )
            + x139 * x530
        )
    )
    return result


def quadrupole3d_43(ax, da, A, bx, db, B, R):
    """Cartesian 3D (gf) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 15, 10), dtype=float)

    x0 = 0.5 / (ax + bx)
    x1 = (ax + bx) ** (-1.0)
    x2 = -x1 * (ax * A[0] + bx * B[0])
    x3 = -x2 - B[0]
    x4 = ax * bx * x1
    x5 = numpy.exp(-x4 * (A[0] - B[0]) ** 2)
    x6 = 1.772453850905516 * numpy.sqrt(x1)
    x7 = x5 * x6
    x8 = x0 * x7
    x9 = 3.0 * x8
    x10 = -x2 - A[0]
    x11 = x10 * x7
    x12 = x11 * x3
    x13 = -x2 - R[0]
    x14 = x11 * x13
    x15 = x3 * x7
    x16 = x13 * x15
    x17 = x0 * (x12 + x14 + x16 + x9)
    x18 = x13 * x7
    x19 = x0 * (x15 + x18)
    x20 = x16 + x8
    x21 = x10 * x20
    x22 = x19 + x21
    x23 = x22 * x3
    x24 = x17 + x23
    x25 = x24 * x3
    x26 = x10 * x24
    x27 = 3.0 * x26
    x28 = x20 * x3
    x29 = x0 * (x11 + x15)
    x30 = x12 + x8
    x31 = x3 * x30
    x32 = x29 + x31
    x33 = 2.0 * x21
    x34 = 3.0 * x19 + x33
    x35 = x0 * (x28 + x32 + x34)
    x36 = x10 * x30
    x37 = 2.0 * x0 * (2.0 * x29 + x31 + x36)
    x38 = x3**2 * x7
    x39 = 2.0 * x12 + x9
    x40 = x0 * (x38 + x39)
    x41 = x10 * x32
    x42 = x40 + x41
    x43 = x3 * x42
    x44 = x37 + x43
    x45 = x0 * (2.0 * x25 + x27 + 5.0 * x35 + x44)
    x46 = x10 * x22
    x47 = 2.0 * x46
    x48 = 4.0 * x17
    x49 = 2.0 * x23 + x48
    x50 = x0 * (x42 + x47 + x49)
    x51 = x26 + x35
    x52 = x3 * x51
    x53 = x50 + x52
    x54 = x10 * x53
    x55 = x13 * x22
    x56 = 2.0 * x55
    x57 = 2.0 * x16
    x58 = x13**2 * x7
    x59 = x58 + x9
    x60 = x13 * x20
    x61 = x19 + x60
    x62 = x0 * (x57 + x59) + x3 * x61
    x63 = x0 * (x49 + x56 + x62)
    x64 = x0 * (x11 + x18)
    x65 = x14 + x8
    x66 = x13 * x65
    x67 = x64 + x66
    x68 = x0 * (x34 + x60 + x67)
    x69 = x17 + x55
    x70 = x3 * x69
    x71 = x68 + x70
    x72 = x10 * x71
    x73 = x63 + x72
    x74 = x10 * x73
    x75 = x3 * x73
    x76 = x10 * x69
    x77 = 2.0 * x0 * (x26 + x35 + 2.0 * x68 + x70 + x76)
    x78 = x3 * x71
    x79 = 2.0 * x50 + 3.0 * x72
    x80 = x75 + x77
    x81 = x0 * (2.0 * x52 + 5.0 * x63 + 2.0 * x78 + x79) + x10 * x80
    x82 = 2.645751311064591
    x83 = da * db
    x84 = 0.009523809523809524 * x83
    x85 = x82 * x84
    x86 = numpy.exp(-x4 * (A[1] - B[1]) ** 2)
    x87 = numpy.exp(-x4 * (A[2] - B[2]) ** 2)
    x88 = 3.141592653589793 * x1 * x87
    x89 = x86 * x88
    x90 = x85 * x89
    x91 = -x1 * (ax * A[1] + bx * B[1])
    x92 = -x91 - B[1]
    x93 = 5.916079783099616
    x94 = x84 * x93
    x95 = x92 * x94
    x96 = 2.0 * x14
    x97 = x0 * (x59 + x96)
    x98 = x10 * x67
    x99 = x97 + x98
    x100 = x0 * (x47 + x48 + x56 + x99)
    x101 = x10 * x51
    x102 = x68 + x76
    x103 = x10 * x102
    x104 = x74 + x77
    x105 = x89 * (
        x0 * (2.0 * x100 + 2.0 * x101 + 2.0 * x103 + 3.0 * x63 + x79) + x10 * x104
    )
    x106 = -x1 * (ax * A[2] + bx * B[2])
    x107 = -x106 - B[2]
    x108 = x107 * x94
    x109 = x10 * x65
    x110 = 2.0 * x0 * (x109 + 2.0 * x64 + x66) + x10 * x99
    x111 = x109 + x64
    x112 = x29 + x36
    x113 = x0 * (x111 + x112 + 2.0 * x19 + x33)
    x114 = x17 + x46
    x115 = x10 * x114
    x116 = 2.0 * x113 + 2.0 * x115
    x117 = x100 + x103
    x118 = x0 * (x110 + x116 + 3.0 * x68 + 3.0 * x76) + x10 * x117
    x119 = x6 * x87
    x120 = x6 * x86
    x121 = x120 * x92**2
    x122 = x0 * x120
    x123 = x121 + x122
    x124 = x123 * x93
    x125 = x124 * x84
    x126 = x107 * x89
    x127 = 10.2469507659596
    x128 = x127 * x84
    x129 = x128 * x92
    x130 = x107**2 * x119
    x131 = x0 * x119
    x132 = x130 + x131
    x133 = x132 * x93
    x134 = x133 * x84
    x135 = 2.0 * x122
    x136 = x92 * (x123 + x135)
    x137 = x136 * x82
    x138 = x10**2 * x7
    x139 = x0 * (x138 + x9 + x96)
    x140 = x10 * x111
    x141 = x0 * (2.0 * x139 + 2.0 * x140 + 3.0 * x97 + 3.0 * x98) + x10 * x110
    x142 = x141 * x84
    x143 = x107 * x119
    x144 = x120 * x92
    x145 = 2.0 * x131
    x146 = x107 * (x132 + x145)
    x147 = x146 * x82
    x148 = -x91 - A[1]
    x149 = 0.06666666666666667 * x83
    x150 = x148 * x149
    x151 = x81 * x89
    x152 = 2.23606797749979
    x153 = x120 * x148
    x154 = x153 * x92
    x155 = x122 + x154
    x156 = x149 * x155
    x157 = x152 * x156
    x158 = x149 * x152
    x159 = x104 * x158
    x160 = x0 * (x144 + x153)
    x161 = x155 * x92
    x162 = x160 + x161
    x163 = x117 * x158
    x164 = 3.872983346207417
    x165 = x117 * x164
    x166 = x132 * x152
    x167 = x149 * x166
    x168 = 3.0 * x122
    x169 = 2.0 * x154 + x168
    x170 = x0 * (x121 + x169)
    x171 = x162 * x92
    x172 = x170 + x171
    x173 = x110 * x149
    x174 = x110 * x158
    x175 = x146 * x149
    x176 = -x106 - A[2]
    x177 = x149 * x176
    x178 = x176 * x89
    x179 = x119 * x176
    x180 = x107 * x179
    x181 = x131 + x180
    x182 = x149 * x181
    x183 = x152 * x182
    x184 = x123 * x152
    x185 = x149 * x184
    x186 = x0 * (x143 + x179)
    x187 = x107 * x181
    x188 = x186 + x187
    x189 = x136 * x149
    x190 = 3.0 * x131
    x191 = 2.0 * x180 + x190
    x192 = x0 * (x130 + x191)
    x193 = x107 * x188
    x194 = x192 + x193
    x195 = x120 * x148**2
    x196 = x122 + x195
    x197 = x196 * x83
    x198 = 0.02222222222222222 * x164
    x199 = x197 * x198
    x200 = x148 * x155
    x201 = x160 + x200
    x202 = 1.732050807568877
    x203 = x201 * x202
    x204 = 0.1111111111111111 * x203
    x205 = x119 * x83
    x206 = 0.1111111111111111 * x197
    x207 = x143 * x202
    x208 = x102 * x202
    x209 = x148 * x162
    x210 = x170 + x209
    x211 = 0.1111111111111111 * x210
    x212 = 0.3333333333333333 * x83
    x213 = x102 * x212
    x214 = 0.1111111111111111 * x132
    x215 = 2.0 * x0 * (2.0 * x160 + x161 + x200)
    x216 = x210 * x92
    x217 = x215 + x216
    x218 = x83 * x99
    x219 = x198 * x218
    x220 = x148 * x158
    x221 = x179 * x212
    x222 = x181 * x212
    x223 = x158 * x179
    x224 = x212 * x99
    x225 = x158 * x194
    x226 = x119 * x176**2
    x227 = x131 + x226
    x228 = x227 * x83
    x229 = x198 * x228
    x230 = 0.1111111111111111 * x228
    x231 = x144 * x202
    x232 = x176 * x181
    x233 = x186 + x232
    x234 = x202 * x233
    x235 = 0.1111111111111111 * x234
    x236 = x120 * x83
    x237 = 0.1111111111111111 * x123
    x238 = x212 * x233
    x239 = x176 * x188
    x240 = x192 + x239
    x241 = 0.1111111111111111 * x240
    x242 = x136 * x198
    x243 = 2.0 * x0 * (2.0 * x186 + x187 + x232)
    x244 = x107 * x240
    x245 = x243 + x244
    x246 = x63 + x78
    x247 = x148 * (x135 + x196)
    x248 = x149 * x247
    x249 = x0 * (x169 + x195)
    x250 = x148 * x201
    x251 = x249 + x250
    x252 = x119 * x158
    x253 = x152 * x248
    x254 = x148 * x210
    x255 = x215 + x254
    x256 = x149 * x251
    x257 = x164 * x69
    x258 = 3.0 * x209
    x259 = x0 * (5.0 * x170 + 2.0 * x171 + x258) + x148 * x217
    x260 = x149 * x67
    x261 = x152 * x260
    x262 = x196 * x212
    x263 = x212 * x67
    x264 = x158 * x227
    x265 = x212 * x227
    x266 = x202 * x238
    x267 = x153 * x212
    x268 = x176 * (x145 + x227)
    x269 = x149 * x268
    x270 = x152 * x269
    x271 = x0 * (x191 + x226)
    x272 = x176 * x233
    x273 = x271 + x272
    x274 = x120 * x158
    x275 = x149 * x273
    x276 = x176 * x240
    x277 = x243 + x276
    x278 = 3.0 * x239
    x279 = x0 * (5.0 * x192 + 2.0 * x193 + x278) + x176 * x245
    x280 = 2.0 * x0 * (2.0 * x19 + x28 + x60) + x3 * x62
    x281 = x84 * (x0 * (x168 + 3.0 * x195) + x148 * x247)
    x282 = x119 * x82
    x283 = x0 * (3.0 * x160 + 3.0 * x200 + x247) + x148 * x251
    x284 = x119 * x94
    x285 = x281 * x93
    x286 = x0 * (3.0 * x170 + 2.0 * x249 + 2.0 * x250 + x258) + x148 * x255
    x287 = x143 * x84
    x288 = x127 * x61
    x289 = 3.0 * x0 * (2.0 * x215 + x216 + x254) + x148 * x259
    x290 = x58 + x8
    x291 = x290 * x84
    x292 = x291 * x93
    x293 = x164 * x61
    x294 = x149 * x290
    x295 = x152 * x294
    x296 = 0.1111111111111111 * x62
    x297 = x202 * x61
    x298 = x290 * x83
    x299 = x153 * x158
    x300 = x0 * (x190 + 3.0 * x226) + x176 * x268
    x301 = x300 * x84
    x302 = x120 * x82
    x303 = x301 * x93
    x304 = x0 * (3.0 * x186 + 3.0 * x232 + x268) + x176 * x273
    x305 = x120 * x94
    x306 = x304 * x84
    x307 = x0 * (3.0 * x192 + 2.0 * x271 + 2.0 * x272 + x278) + x176 * x277
    x308 = 3.0 * x0 * (2.0 * x243 + x244 + x276) + x176 * x279
    x309 = -x91 - R[1]
    x310 = x3 * x32
    x311 = 3.0 * x41
    x312 = x0 * (2.0 * x310 + x311 + 5.0 * x40) + x10 * x44
    x313 = x45 + x54
    x314 = x90 * (x0 * (3.0 * x101 + x312 + 6.0 * x50 + 3.0 * x52) + x10 * x313)
    x315 = x10 * x42
    x316 = x315 + x37
    x317 = x101 + x50
    x318 = x0 * (x116 + x27 + x316 + 3.0 * x35) + x10 * x317
    x319 = x144 * x309
    x320 = x122 + x319
    x321 = x320 * x84
    x322 = x119 * x93
    x323 = x318 * x89
    x324 = x120 * x309
    x325 = x0 * (x144 + x324)
    x326 = x320 * x92
    x327 = x325 + x326
    x328 = x139 + x140
    x329 = x0 * (x138 + x39)
    x330 = x10 * x112
    x331 = x329 + x330
    x332 = x113 + x115
    x333 = x0 * (3.0 * x17 + x328 + x331 + 3.0 * x46) + x10 * x332
    x334 = x127 * x320
    x335 = x168 + 2.0 * x319
    x336 = x0 * (x121 + x335) + x327 * x92
    x337 = x138 + x8
    x338 = 2.0 * x8
    x339 = x10 * (x337 + x338)
    x340 = x0 * (3.0 * x109 + x339 + 3.0 * x64) + x10 * x328
    x341 = x119 * x85
    x342 = x143 * x94
    x343 = x147 * x84
    x344 = x153 * x309
    x345 = x122 + x344
    x346 = x119 * x149
    x347 = x148 * x320
    x348 = x325 + x347
    x349 = x158 * x345
    x350 = x0 * (x154 + x168 + x319 + x344)
    x351 = x348 * x92
    x352 = x350 + x351
    x353 = x149 * x164
    x354 = x143 * x353
    x355 = 2.0 * x347
    x356 = 3.0 * x325 + x355
    x357 = x0 * (x162 + x326 + x356)
    x358 = x352 * x92
    x359 = x357 + x358
    x360 = x152 * x328
    x361 = x149 * x360
    x362 = x313 * x89
    x363 = x149 * x320
    x364 = x152 * x363
    x365 = x164 * x320
    x366 = x158 * x324
    x367 = x149 * x328
    x368 = x0 * (x153 + x324)
    x369 = x148 * x345
    x370 = x368 + x369
    x371 = x370 * x83
    x372 = x198 * x371
    x373 = x148 * x348
    x374 = x350 + x373
    x375 = x202 * x374
    x376 = 0.1111111111111111 * x205
    x377 = 0.1111111111111111 * x371
    x378 = x148 * x352
    x379 = x357 + x378
    x380 = x114 * x202
    x381 = x114 * x212
    x382 = 2.0 * x373
    x383 = 4.0 * x350
    x384 = 2.0 * x351 + x383
    x385 = x0 * (x210 + x382 + x384)
    x386 = x379 * x92
    x387 = x385 + x386
    x388 = x111 * x83
    x389 = x198 * x388
    x390 = 0.1111111111111111 * x388
    x391 = x146 * x198
    x392 = x212 * x345
    x393 = x202 * x381
    x394 = x111 * x212
    x395 = x202 * x320
    x396 = x324 * x83
    x397 = x168 + 2.0 * x344
    x398 = x0 * (x195 + x397)
    x399 = x148 * x370
    x400 = x398 + x399
    x401 = x25 + x35
    x402 = x0 * (x201 + 2.0 * x325 + x355 + x370)
    x403 = x148 * x374
    x404 = x402 + x403
    x405 = x158 * x400
    x406 = x148 * x379
    x407 = x385 + x406
    x408 = 3.0 * x378
    x409 = x0 * (x217 + 5.0 * x357 + 2.0 * x358 + x408)
    x410 = x148 * x387
    x411 = x409 + x410
    x412 = x158 * x65
    x413 = x212 * x370
    x414 = x212 * x65
    x415 = x149 * x279
    x416 = x19 + x28
    x417 = x0 * (x38 + x57 + x9) + x3 * x416
    x418 = x0 * (x247 + 3.0 * x368 + 3.0 * x369) + x148 * x400
    x419 = x0 * (x251 + 3.0 * x350 + 3.0 * x373 + x400) + x148 * x404
    x420 = 2.0 * x402 + 2.0 * x403
    x421 = x0 * (x255 + 3.0 * x357 + x408 + x420) + x148 * x407
    x422 = x20 * x84
    x423 = x127 * x422
    x424 = x5 * x88
    x425 = x424 * x85
    x426 = x425 * (x0 * (x259 + 6.0 * x385 + 3.0 * x386 + 3.0 * x406) + x148 * x411)
    x427 = x421 * x424
    x428 = x149 * x400
    x429 = x158 * x416
    x430 = x152 * x416
    x431 = x158 * x20
    x432 = x164 * x20
    x433 = x411 * x424
    x434 = x158 * x18
    x435 = x20 * x202
    x436 = x18 * x83
    x437 = x301 * x82
    x438 = x304 * x94
    x439 = x307 * x93
    x440 = 3.141592653589793 * x1 * x5 * x86
    x441 = x440 * x85
    x442 = -x106 - R[2]
    x443 = x143 * x442
    x444 = x131 + x443
    x445 = x444 * x84
    x446 = x120 * x93
    x447 = x119 * x442
    x448 = x127 * x445
    x449 = x0 * (x143 + x447)
    x450 = x107 * x444
    x451 = x449 + x450
    x452 = x137 * x84
    x453 = x451 * x94
    x454 = x190 + 2.0 * x443
    x455 = x0 * (x130 + x454) + x107 * x451
    x456 = x120 * x85
    x457 = x158 * x447
    x458 = x164 * x444
    x459 = x179 * x442
    x460 = x131 + x459
    x461 = x120 * x149
    x462 = x158 * x460
    x463 = x176 * x444
    x464 = x449 + x463
    x465 = x144 * x353
    x466 = x0 * (x180 + x190 + x443 + x459)
    x467 = x107 * x464
    x468 = x466 + x467
    x469 = 2.0 * x463
    x470 = 3.0 * x449 + x469
    x471 = x0 * (x188 + x450 + x470)
    x472 = x107 * x468
    x473 = x471 + x472
    x474 = x447 * x83
    x475 = x202 * x444
    x476 = x212 * x460
    x477 = x212 * x464
    x478 = x0 * (x179 + x447)
    x479 = x176 * x460
    x480 = x478 + x479
    x481 = x480 * x83
    x482 = x198 * x481
    x483 = 0.1111111111111111 * x481
    x484 = x176 * x464
    x485 = x466 + x484
    x486 = x202 * x485
    x487 = 0.1111111111111111 * x236
    x488 = x176 * x468
    x489 = x471 + x488
    x490 = 2.0 * x484
    x491 = 4.0 * x466
    x492 = 2.0 * x467 + x491
    x493 = x0 * (x240 + x490 + x492)
    x494 = x107 * x489
    x495 = x493 + x494
    x496 = x149 * x259
    x497 = x212 * x480
    x498 = x212 * x485
    x499 = x202 * x498
    x500 = x190 + 2.0 * x459
    x501 = x0 * (x226 + x500)
    x502 = x176 * x480
    x503 = x501 + x502
    x504 = x158 * x503
    x505 = x0 * (x233 + 2.0 * x449 + x469 + x480)
    x506 = x176 * x485
    x507 = x505 + x506
    x508 = x176 * x489
    x509 = x493 + x508
    x510 = 3.0 * x488
    x511 = x0 * (x245 + 5.0 * x471 + 2.0 * x472 + x510)
    x512 = x176 * x495
    x513 = x511 + x512
    x514 = x281 * x82
    x515 = x416 * x94
    x516 = x286 * x93
    x517 = x149 * x503
    x518 = x440 * x513
    x519 = x0 * (x268 + 3.0 * x478 + 3.0 * x479) + x176 * x503
    x520 = x0 * (x273 + 3.0 * x466 + 3.0 * x484 + x503) + x176 * x507
    x521 = 2.0 * x505 + 2.0 * x506
    x522 = x0 * (x277 + 3.0 * x471 + x510 + x521) + x176 * x509
    x523 = x440 * x522
    x524 = x441 * (x0 * (x279 + 6.0 * x493 + 3.0 * x494 + 3.0 * x508) + x176 * x513)
    x525 = 3.0 * x0 * (x315 + 2.0 * x37 + x43) + x10 * x312
    x526 = x120 * x309**2
    x527 = x122 + x526
    x528 = x527 * x84
    x529 = x309 * x320
    x530 = x325 + x529
    x531 = x0 * (x311 + 2.0 * x329 + 2.0 * x330 + 3.0 * x40) + x10 * x316
    x532 = x531 * x93
    x533 = x0 * (x335 + x526) + x530 * x92
    x534 = x0 * (3.0 * x29 + x339 + 3.0 * x36) + x10 * x331
    x535 = x127 * x530
    x536 = x0 * (3.0 * x138 + x9) + x10 * x339
    x537 = 2.0 * x0 * (2.0 * x325 + x326 + x529) + x533 * x92
    x538 = x536 * x93
    x539 = x309 * x345
    x540 = x368 + x539
    x541 = x309 * x348
    x542 = x350 + x541
    x543 = x143 * x158
    x544 = x0 * (x356 + x529 + x540)
    x545 = x542 * x92
    x546 = x544 + x545
    x547 = x149 * x331
    x548 = x143 * x164
    x549 = 2.0 * x541
    x550 = x0 * (x384 + x533 + x549)
    x551 = x546 * x92
    x552 = x550 + x551
    x553 = x149 * x339
    x554 = x152 * x553
    x555 = x149 * x312
    x556 = x164 * x530
    x557 = x158 * x527
    x558 = x0 * (x397 + x526)
    x559 = x148 * x540
    x560 = x558 + x559
    x561 = x560 * x83
    x562 = x198 * x561
    x563 = x148 * x542
    x564 = x544 + x563
    x565 = x202 * x564
    x566 = 0.1111111111111111 * x42
    x567 = x148 * x546
    x568 = x550 + x567
    x569 = x112 * x202
    x570 = x112 * x212
    x571 = 2.0 * x0 * (x357 + x378 + 2.0 * x544 + x545 + x563)
    x572 = x568 * x92
    x573 = x571 + x572
    x574 = x337 * x83
    x575 = x198 * x574
    x576 = 0.1111111111111111 * x574
    x577 = x212 * x540
    x578 = x202 * x570
    x579 = x212 * x337
    x580 = x202 * x530
    x581 = x527 * x83
    x582 = 2.0 * x0 * (2.0 * x368 + x369 + x539) + x148 * x560
    x583 = x310 + x40
    x584 = x0 * (x382 + x383 + x549 + x560)
    x585 = x148 * x564
    x586 = x584 + x585
    x587 = x148 * x568
    x588 = x571 + x587
    x589 = x158 * x30
    x590 = x149 * x30
    x591 = 2.0 * x385 + 3.0 * x567
    x592 = x0 * (2.0 * x386 + 5.0 * x550 + 2.0 * x551 + x591) + x148 * x573
    x593 = x424 * x592
    x594 = x10 * x149
    x595 = x107 * x424
    x596 = x10 * x158
    x597 = x212 * x560
    x598 = x212 * x30
    x599 = x176 * x424
    x600 = x11 * x212
    x601 = x11 * x158
    x602 = x0 * (2.0 * x398 + 2.0 * x399 + 3.0 * x558 + 3.0 * x559) + x148 * x582
    x603 = x38 + x8
    x604 = x3 * (x338 + x603)
    x605 = x604 * x82
    x606 = x119 * x84
    x607 = x0 * (x420 + 3.0 * x544 + 3.0 * x563 + x582) + x148 * x586
    x608 = x603 * x93
    x609 = x424 * (
        x0 * (2.0 * x406 + 3.0 * x550 + 2.0 * x584 + 2.0 * x585 + x591) + x148 * x588
    )
    x610 = x3 * x94
    x611 = x7 * x84
    x612 = x149 * x604
    x613 = x158 * x603
    x614 = x15 * x164
    x615 = x15 * x158
    x616 = x158 * x7
    x617 = x149 * x7
    x618 = x15 * x202
    x619 = x7 * x83
    x620 = x7 * x82
    x621 = x7 * x94
    x622 = x327 * x94
    x623 = x536 * x85
    x624 = 0.1111111111111111 * x569
    x625 = x198 * x337
    x626 = x447 * x84
    x627 = x7 * x93
    x628 = x7 * x85
    x629 = x15 * x353
    x630 = 0.1111111111111111 * x619
    x631 = x324 * x84
    x632 = x15 * x84
    x633 = x119 * x442**2
    x634 = x131 + x633
    x635 = x634 * x84
    x636 = x442 * x444
    x637 = x449 + x636
    x638 = x144 * x84
    x639 = x127 * x637
    x640 = x0 * (x454 + x633) + x107 * x637
    x641 = 2.0 * x0 * (2.0 * x449 + x450 + x636) + x107 * x640
    x642 = x158 * x634
    x643 = x164 * x637
    x644 = x442 * x460
    x645 = x478 + x644
    x646 = x158 * x645
    x647 = x442 * x464
    x648 = x466 + x647
    x649 = x144 * x164
    x650 = x0 * (x470 + x636 + x645)
    x651 = x107 * x648
    x652 = x650 + x651
    x653 = 2.0 * x647
    x654 = x0 * (x492 + x640 + x653)
    x655 = x107 * x652
    x656 = x654 + x655
    x657 = x634 * x83
    x658 = x202 * x637
    x659 = x212 * x645
    x660 = x0 * (x500 + x633)
    x661 = x176 * x645
    x662 = x660 + x661
    x663 = x662 * x83
    x664 = x198 * x663
    x665 = x176 * x648
    x666 = x650 + x665
    x667 = x202 * x666
    x668 = x176 * x652
    x669 = x654 + x668
    x670 = 2.0 * x0 * (x471 + x488 + 2.0 * x650 + x651 + x665)
    x671 = x107 * x669
    x672 = x670 + x671
    x673 = x212 * x662
    x674 = x220 * x440
    x675 = 2.0 * x0 * (2.0 * x478 + x479 + x644) + x176 * x662
    x676 = x0 * (x490 + x491 + x653 + x662)
    x677 = x176 * x666
    x678 = x676 + x677
    x679 = x176 * x669
    x680 = x670 + x679
    x681 = 2.0 * x493 + 3.0 * x668
    x682 = x0 * (2.0 * x494 + 5.0 * x654 + 2.0 * x655 + x681) + x176 * x672
    x683 = x440 * x682
    x684 = x0 * (2.0 * x501 + 2.0 * x502 + 3.0 * x660 + 3.0 * x661) + x176 * x675
    x685 = x120 * x84
    x686 = x0 * (x521 + 3.0 * x650 + 3.0 * x665 + x675) + x176 * x678
    x687 = x440 * (
        x0 * (2.0 * x508 + 3.0 * x654 + 2.0 * x676 + 2.0 * x677 + x681) + x176 * x680
    )

    # 900 item(s)
    result[0, 0, 0] = numpy.sum(
        x90
        * (x0 * (2.0 * x45 + 2.0 * x54 + 3.0 * x74 + 3.0 * x75 + 6.0 * x77) + x10 * x81)
    )
    result[0, 0, 1] = numpy.sum(x105 * x95)
    result[0, 0, 2] = numpy.sum(x105 * x108)
    result[0, 0, 3] = numpy.sum(x118 * x119 * x125)
    result[0, 0, 4] = numpy.sum(x118 * x126 * x129)
    result[0, 0, 5] = numpy.sum(x118 * x120 * x134)
    result[0, 0, 6] = numpy.sum(x119 * x137 * x142)
    result[0, 0, 7] = numpy.sum(x125 * x141 * x143)
    result[0, 0, 8] = numpy.sum(x134 * x141 * x144)
    result[0, 0, 9] = numpy.sum(x120 * x142 * x147)
    result[0, 1, 0] = numpy.sum(x150 * x151)
    result[0, 1, 1] = numpy.sum(x104 * x119 * x157)
    result[0, 1, 2] = numpy.sum(x126 * x148 * x159)
    result[0, 1, 3] = numpy.sum(x119 * x162 * x163)
    result[0, 1, 4] = numpy.sum(x143 * x156 * x165)
    result[0, 1, 5] = numpy.sum(x117 * x153 * x167)
    result[0, 1, 6] = numpy.sum(x119 * x172 * x173)
    result[0, 1, 7] = numpy.sum(x143 * x162 * x174)
    result[0, 1, 8] = numpy.sum(x110 * x156 * x166)
    result[0, 1, 9] = numpy.sum(x110 * x153 * x175)
    result[0, 2, 0] = numpy.sum(x151 * x177)
    result[0, 2, 1] = numpy.sum(x159 * x178 * x92)
    result[0, 2, 2] = numpy.sum(x104 * x120 * x183)
    result[0, 2, 3] = numpy.sum(x117 * x179 * x185)
    result[0, 2, 4] = numpy.sum(x144 * x165 * x182)
    result[0, 2, 5] = numpy.sum(x120 * x163 * x188)
    result[0, 2, 6] = numpy.sum(x110 * x179 * x189)
    result[0, 2, 7] = numpy.sum(x110 * x182 * x184)
    result[0, 2, 8] = numpy.sum(x144 * x174 * x188)
    result[0, 2, 9] = numpy.sum(x120 * x173 * x194)
    result[0, 3, 0] = numpy.sum(x119 * x199 * x80)
    result[0, 3, 1] = numpy.sum(x204 * x205 * x73)
    result[0, 3, 2] = numpy.sum(x206 * x207 * x73)
    result[0, 3, 3] = numpy.sum(x205 * x208 * x211)
    result[0, 3, 4] = numpy.sum(x143 * x201 * x213)
    result[0, 3, 5] = numpy.sum(x197 * x208 * x214)
    result[0, 3, 6] = numpy.sum(x119 * x217 * x219)
    result[0, 3, 7] = numpy.sum(x207 * x211 * x218)
    result[0, 3, 8] = numpy.sum(x203 * x214 * x218)
    result[0, 3, 9] = numpy.sum(x146 * x199 * x99)
    result[0, 4, 0] = numpy.sum(x178 * x220 * x80)
    result[0, 4, 1] = numpy.sum(x155 * x221 * x73)
    result[0, 4, 2] = numpy.sum(x153 * x222 * x73)
    result[0, 4, 3] = numpy.sum(x162 * x179 * x213)
    result[0, 4, 4] = numpy.sum(x155 * x208 * x222)
    result[0, 4, 5] = numpy.sum(x153 * x188 * x213)
    result[0, 4, 6] = numpy.sum(x172 * x223 * x99)
    result[0, 4, 7] = numpy.sum(x162 * x181 * x224)
    result[0, 4, 8] = numpy.sum(x155 * x188 * x224)
    result[0, 4, 9] = numpy.sum(x153 * x225 * x99)
    result[0, 5, 0] = numpy.sum(x120 * x229 * x80)
    result[0, 5, 1] = numpy.sum(x230 * x231 * x73)
    result[0, 5, 2] = numpy.sum(x235 * x236 * x73)
    result[0, 5, 3] = numpy.sum(x208 * x228 * x237)
    result[0, 5, 4] = numpy.sum(x102 * x144 * x238)
    result[0, 5, 5] = numpy.sum(x208 * x236 * x241)
    result[0, 5, 6] = numpy.sum(x218 * x227 * x242)
    result[0, 5, 7] = numpy.sum(x218 * x234 * x237)
    result[0, 5, 8] = numpy.sum(x218 * x231 * x241)
    result[0, 5, 9] = numpy.sum(x120 * x219 * x245)
    result[0, 6, 0] = numpy.sum(x119 * x246 * x248)
    result[0, 6, 1] = numpy.sum(x251 * x252 * x71)
    result[0, 6, 2] = numpy.sum(x143 * x253 * x71)
    result[0, 6, 3] = numpy.sum(x252 * x255 * x69)
    result[0, 6, 4] = numpy.sum(x143 * x256 * x257)
    result[0, 6, 5] = numpy.sum(x166 * x248 * x69)
    result[0, 6, 6] = numpy.sum(x119 * x259 * x260)
    result[0, 6, 7] = numpy.sum(x143 * x255 * x261)
    result[0, 6, 8] = numpy.sum(x166 * x251 * x260)
    result[0, 6, 9] = numpy.sum(x146 * x247 * x260)
    result[0, 7, 0] = numpy.sum(x196 * x223 * x246)
    result[0, 7, 1] = numpy.sum(x201 * x221 * x71)
    result[0, 7, 2] = numpy.sum(x181 * x262 * x71)
    result[0, 7, 3] = numpy.sum(x210 * x221 * x69)
    result[0, 7, 4] = numpy.sum(x203 * x222 * x69)
    result[0, 7, 5] = numpy.sum(x188 * x262 * x69)
    result[0, 7, 6] = numpy.sum(x179 * x217 * x261)
    result[0, 7, 7] = numpy.sum(x181 * x210 * x263)
    result[0, 7, 8] = numpy.sum(x188 * x201 * x263)
    result[0, 7, 9] = numpy.sum(x194 * x196 * x261)
    result[0, 8, 0] = numpy.sum(x153 * x246 * x264)
    result[0, 8, 1] = numpy.sum(x155 * x265 * x71)
    result[0, 8, 2] = numpy.sum(x153 * x238 * x71)
    result[0, 8, 3] = numpy.sum(x162 * x265 * x69)
    result[0, 8, 4] = numpy.sum(x155 * x266 * x69)
    result[0, 8, 5] = numpy.sum(x240 * x267 * x69)
    result[0, 8, 6] = numpy.sum(x172 * x227 * x261)
    result[0, 8, 7] = numpy.sum(x162 * x233 * x263)
    result[0, 8, 8] = numpy.sum(x155 * x240 * x263)
    result[0, 8, 9] = numpy.sum(x153 * x245 * x261)
    result[0, 9, 0] = numpy.sum(x120 * x246 * x269)
    result[0, 9, 1] = numpy.sum(x144 * x270 * x71)
    result[0, 9, 2] = numpy.sum(x273 * x274 * x71)
    result[0, 9, 3] = numpy.sum(x184 * x269 * x69)
    result[0, 9, 4] = numpy.sum(x144 * x257 * x275)
    result[0, 9, 5] = numpy.sum(x274 * x277 * x69)
    result[0, 9, 6] = numpy.sum(x136 * x260 * x268)
    result[0, 9, 7] = numpy.sum(x184 * x260 * x273)
    result[0, 9, 8] = numpy.sum(x144 * x261 * x277)
    result[0, 9, 9] = numpy.sum(x120 * x260 * x279)
    result[0, 10, 0] = numpy.sum(x280 * x281 * x282)
    result[0, 10, 1] = numpy.sum(x283 * x284 * x62)
    result[0, 10, 2] = numpy.sum(x143 * x285 * x62)
    result[0, 10, 3] = numpy.sum(x284 * x286 * x61)
    result[0, 10, 4] = numpy.sum(x283 * x287 * x288)
    result[0, 10, 5] = numpy.sum(x133 * x281 * x61)
    result[0, 10, 6] = numpy.sum(x282 * x289 * x291)
    result[0, 10, 7] = numpy.sum(x143 * x286 * x292)
    result[0, 10, 8] = numpy.sum(x133 * x283 * x291)
    result[0, 10, 9] = numpy.sum(x147 * x281 * x290)
    result[0, 11, 0] = numpy.sum(x179 * x248 * x280)
    result[0, 11, 1] = numpy.sum(x223 * x251 * x62)
    result[0, 11, 2] = numpy.sum(x181 * x253 * x62)
    result[0, 11, 3] = numpy.sum(x223 * x255 * x61)
    result[0, 11, 4] = numpy.sum(x182 * x251 * x293)
    result[0, 11, 5] = numpy.sum(x188 * x253 * x61)
    result[0, 11, 6] = numpy.sum(x179 * x259 * x294)
    result[0, 11, 7] = numpy.sum(x181 * x255 * x295)
    result[0, 11, 8] = numpy.sum(x188 * x251 * x295)
    result[0, 11, 9] = numpy.sum(x194 * x247 * x294)
    result[0, 12, 0] = numpy.sum(x199 * x227 * x280)
    result[0, 12, 1] = numpy.sum(x203 * x228 * x296)
    result[0, 12, 2] = numpy.sum(x197 * x234 * x296)
    result[0, 12, 3] = numpy.sum(x211 * x228 * x297)
    result[0, 12, 4] = numpy.sum(x201 * x238 * x61)
    result[0, 12, 5] = numpy.sum(x197 * x241 * x297)
    result[0, 12, 6] = numpy.sum(x217 * x229 * x290)
    result[0, 12, 7] = numpy.sum(x211 * x234 * x298)
    result[0, 12, 8] = numpy.sum(x203 * x241 * x298)
    result[0, 12, 9] = numpy.sum(x199 * x245 * x290)
    result[0, 13, 0] = numpy.sum(x153 * x269 * x280)
    result[0, 13, 1] = numpy.sum(x155 * x270 * x62)
    result[0, 13, 2] = numpy.sum(x273 * x299 * x62)
    result[0, 13, 3] = numpy.sum(x162 * x270 * x61)
    result[0, 13, 4] = numpy.sum(x156 * x273 * x293)
    result[0, 13, 5] = numpy.sum(x277 * x299 * x61)
    result[0, 13, 6] = numpy.sum(x172 * x268 * x294)
    result[0, 13, 7] = numpy.sum(x162 * x273 * x295)
    result[0, 13, 8] = numpy.sum(x155 * x277 * x295)
    result[0, 13, 9] = numpy.sum(x153 * x279 * x294)
    result[0, 14, 0] = numpy.sum(x280 * x301 * x302)
    result[0, 14, 1] = numpy.sum(x144 * x303 * x62)
    result[0, 14, 2] = numpy.sum(x304 * x305 * x62)
    result[0, 14, 3] = numpy.sum(x124 * x301 * x61)
    result[0, 14, 4] = numpy.sum(x144 * x288 * x306)
    result[0, 14, 5] = numpy.sum(x305 * x307 * x61)
    result[0, 14, 6] = numpy.sum(x137 * x291 * x300)
    result[0, 14, 7] = numpy.sum(x124 * x291 * x304)
    result[0, 14, 8] = numpy.sum(x144 * x292 * x307)
    result[0, 14, 9] = numpy.sum(x291 * x302 * x308)
    result[1, 0, 0] = numpy.sum(x309 * x314)
    result[1, 0, 1] = numpy.sum(x318 * x321 * x322)
    result[1, 0, 2] = numpy.sum(x108 * x309 * x323)
    result[1, 0, 3] = numpy.sum(x284 * x327 * x333)
    result[1, 0, 4] = numpy.sum(x287 * x333 * x334)
    result[1, 0, 5] = numpy.sum(x134 * x324 * x333)
    result[1, 0, 6] = numpy.sum(x336 * x340 * x341)
    result[1, 0, 7] = numpy.sum(x327 * x340 * x342)
    result[1, 0, 8] = numpy.sum(x134 * x320 * x340)
    result[1, 0, 9] = numpy.sum(x324 * x340 * x343)
    result[1, 1, 0] = numpy.sum(x313 * x345 * x346)
    result[1, 1, 1] = numpy.sum(x252 * x317 * x348)
    result[1, 1, 2] = numpy.sum(x143 * x317 * x349)
    result[1, 1, 3] = numpy.sum(x252 * x332 * x352)
    result[1, 1, 4] = numpy.sum(x332 * x348 * x354)
    result[1, 1, 5] = numpy.sum(x167 * x332 * x345)
    result[1, 1, 6] = numpy.sum(x328 * x346 * x359)
    result[1, 1, 7] = numpy.sum(x143 * x352 * x361)
    result[1, 1, 8] = numpy.sum(x167 * x328 * x348)
    result[1, 1, 9] = numpy.sum(x175 * x328 * x345)
    result[1, 2, 0] = numpy.sum(x177 * x309 * x362)
    result[1, 2, 1] = numpy.sum(x179 * x317 * x364)
    result[1, 2, 2] = numpy.sum(x183 * x317 * x324)
    result[1, 2, 3] = numpy.sum(x223 * x327 * x332)
    result[1, 2, 4] = numpy.sum(x182 * x332 * x365)
    result[1, 2, 5] = numpy.sum(x188 * x332 * x366)
    result[1, 2, 6] = numpy.sum(x179 * x336 * x367)
    result[1, 2, 7] = numpy.sum(x182 * x327 * x360)
    result[1, 2, 8] = numpy.sum(x188 * x360 * x363)
    result[1, 2, 9] = numpy.sum(x194 * x324 * x367)
    result[1, 3, 0] = numpy.sum(x119 * x372 * x53)
    result[1, 3, 1] = numpy.sum(x375 * x376 * x51)
    result[1, 3, 2] = numpy.sum(x207 * x377 * x51)
    result[1, 3, 3] = numpy.sum(x376 * x379 * x380)
    result[1, 3, 4] = numpy.sum(x143 * x374 * x381)
    result[1, 3, 5] = numpy.sum(x214 * x371 * x380)
    result[1, 3, 6] = numpy.sum(x119 * x387 * x389)
    result[1, 3, 7] = numpy.sum(x207 * x379 * x390)
    result[1, 3, 8] = numpy.sum(x214 * x375 * x388)
    result[1, 3, 9] = numpy.sum(x111 * x371 * x391)
    result[1, 4, 0] = numpy.sum(x179 * x349 * x53)
    result[1, 4, 1] = numpy.sum(x221 * x348 * x51)
    result[1, 4, 2] = numpy.sum(x181 * x392 * x51)
    result[1, 4, 3] = numpy.sum(x179 * x352 * x381)
    result[1, 4, 4] = numpy.sum(x181 * x348 * x393)
    result[1, 4, 5] = numpy.sum(x114 * x188 * x392)
    result[1, 4, 6] = numpy.sum(x111 * x223 * x359)
    result[1, 4, 7] = numpy.sum(x181 * x352 * x394)
    result[1, 4, 8] = numpy.sum(x188 * x348 * x394)
    result[1, 4, 9] = numpy.sum(x111 * x225 * x345)
    result[1, 5, 0] = numpy.sum(x229 * x324 * x53)
    result[1, 5, 1] = numpy.sum(x230 * x395 * x51)
    result[1, 5, 2] = numpy.sum(x235 * x396 * x51)
    result[1, 5, 3] = numpy.sum(x230 * x327 * x380)
    result[1, 5, 4] = numpy.sum(x114 * x238 * x320)
    result[1, 5, 5] = numpy.sum(x241 * x380 * x396)
    result[1, 5, 6] = numpy.sum(x111 * x229 * x336)
    result[1, 5, 7] = numpy.sum(x235 * x327 * x388)
    result[1, 5, 8] = numpy.sum(x241 * x388 * x395)
    result[1, 5, 9] = numpy.sum(x245 * x324 * x389)
    result[1, 6, 0] = numpy.sum(x346 * x400 * x401)
    result[1, 6, 1] = numpy.sum(x24 * x252 * x404)
    result[1, 6, 2] = numpy.sum(x143 * x24 * x405)
    result[1, 6, 3] = numpy.sum(x22 * x252 * x407)
    result[1, 6, 4] = numpy.sum(x22 * x354 * x404)
    result[1, 6, 5] = numpy.sum(x167 * x22 * x400)
    result[1, 6, 6] = numpy.sum(x346 * x411 * x65)
    result[1, 6, 7] = numpy.sum(x143 * x407 * x412)
    result[1, 6, 8] = numpy.sum(x167 * x404 * x65)
    result[1, 6, 9] = numpy.sum(x175 * x400 * x65)
    result[1, 7, 0] = numpy.sum(x223 * x370 * x401)
    result[1, 7, 1] = numpy.sum(x221 * x24 * x374)
    result[1, 7, 2] = numpy.sum(x181 * x24 * x413)
    result[1, 7, 3] = numpy.sum(x22 * x221 * x379)
    result[1, 7, 4] = numpy.sum(x22 * x222 * x375)
    result[1, 7, 5] = numpy.sum(x188 * x22 * x413)
    result[1, 7, 6] = numpy.sum(x179 * x387 * x412)
    result[1, 7, 7] = numpy.sum(x181 * x379 * x414)
    result[1, 7, 8] = numpy.sum(x188 * x374 * x414)
    result[1, 7, 9] = numpy.sum(x225 * x370 * x65)
    result[1, 8, 0] = numpy.sum(x264 * x345 * x401)
    result[1, 8, 1] = numpy.sum(x24 * x265 * x348)
    result[1, 8, 2] = numpy.sum(x238 * x24 * x345)
    result[1, 8, 3] = numpy.sum(x22 * x265 * x352)
    result[1, 8, 4] = numpy.sum(x22 * x266 * x348)
    result[1, 8, 5] = numpy.sum(x22 * x240 * x392)
    result[1, 8, 6] = numpy.sum(x264 * x359 * x65)
    result[1, 8, 7] = numpy.sum(x238 * x352 * x65)
    result[1, 8, 8] = numpy.sum(x240 * x348 * x414)
    result[1, 8, 9] = numpy.sum(x245 * x345 * x412)
    result[1, 9, 0] = numpy.sum(x269 * x324 * x401)
    result[1, 9, 1] = numpy.sum(x24 * x270 * x320)
    result[1, 9, 2] = numpy.sum(x24 * x273 * x366)
    result[1, 9, 3] = numpy.sum(x22 * x270 * x327)
    result[1, 9, 4] = numpy.sum(x22 * x275 * x365)
    result[1, 9, 5] = numpy.sum(x22 * x277 * x366)
    result[1, 9, 6] = numpy.sum(x269 * x336 * x65)
    result[1, 9, 7] = numpy.sum(x273 * x327 * x412)
    result[1, 9, 8] = numpy.sum(x277 * x364 * x65)
    result[1, 9, 9] = numpy.sum(x324 * x415 * x65)
    result[1, 10, 0] = numpy.sum(x341 * x417 * x418)
    result[1, 10, 1] = numpy.sum(x284 * x416 * x419)
    result[1, 10, 2] = numpy.sum(x342 * x416 * x418)
    result[1, 10, 3] = numpy.sum(x322 * x421 * x422)
    result[1, 10, 4] = numpy.sum(x143 * x419 * x423)
    result[1, 10, 5] = numpy.sum(x134 * x20 * x418)
    result[1, 10, 6] = numpy.sum(x13 * x426)
    result[1, 10, 7] = numpy.sum(x108 * x13 * x427)
    result[1, 10, 8] = numpy.sum(x134 * x18 * x419)
    result[1, 10, 9] = numpy.sum(x18 * x343 * x418)
    result[1, 11, 0] = numpy.sum(x179 * x417 * x428)
    result[1, 11, 1] = numpy.sum(x179 * x404 * x429)
    result[1, 11, 2] = numpy.sum(x182 * x400 * x430)
    result[1, 11, 3] = numpy.sum(x179 * x407 * x431)
    result[1, 11, 4] = numpy.sum(x182 * x404 * x432)
    result[1, 11, 5] = numpy.sum(x188 * x400 * x431)
    result[1, 11, 6] = numpy.sum(x13 * x177 * x433)
    result[1, 11, 7] = numpy.sum(x18 * x183 * x407)
    result[1, 11, 8] = numpy.sum(x188 * x404 * x434)
    result[1, 11, 9] = numpy.sum(x18 * x194 * x428)
    result[1, 12, 0] = numpy.sum(x229 * x370 * x417)
    result[1, 12, 1] = numpy.sum(x230 * x375 * x416)
    result[1, 12, 2] = numpy.sum(x235 * x371 * x416)
    result[1, 12, 3] = numpy.sum(x230 * x379 * x435)
    result[1, 12, 4] = numpy.sum(x20 * x238 * x374)
    result[1, 12, 5] = numpy.sum(x241 * x371 * x435)
    result[1, 12, 6] = numpy.sum(x18 * x229 * x387)
    result[1, 12, 7] = numpy.sum(x235 * x379 * x436)
    result[1, 12, 8] = numpy.sum(x241 * x375 * x436)
    result[1, 12, 9] = numpy.sum(x18 * x245 * x372)
    result[1, 13, 0] = numpy.sum(x269 * x345 * x417)
    result[1, 13, 1] = numpy.sum(x270 * x348 * x416)
    result[1, 13, 2] = numpy.sum(x273 * x345 * x429)
    result[1, 13, 3] = numpy.sum(x20 * x270 * x352)
    result[1, 13, 4] = numpy.sum(x275 * x348 * x432)
    result[1, 13, 5] = numpy.sum(x277 * x345 * x431)
    result[1, 13, 6] = numpy.sum(x18 * x269 * x359)
    result[1, 13, 7] = numpy.sum(x273 * x352 * x434)
    result[1, 13, 8] = numpy.sum(x277 * x348 * x434)
    result[1, 13, 9] = numpy.sum(x18 * x345 * x415)
    result[1, 14, 0] = numpy.sum(x324 * x417 * x437)
    result[1, 14, 1] = numpy.sum(x303 * x320 * x416)
    result[1, 14, 2] = numpy.sum(x324 * x416 * x438)
    result[1, 14, 3] = numpy.sum(x20 * x303 * x327)
    result[1, 14, 4] = numpy.sum(x304 * x320 * x423)
    result[1, 14, 5] = numpy.sum(x324 * x422 * x439)
    result[1, 14, 6] = numpy.sum(x18 * x336 * x437)
    result[1, 14, 7] = numpy.sum(x18 * x327 * x438)
    result[1, 14, 8] = numpy.sum(x18 * x321 * x439)
    result[1, 14, 9] = numpy.sum(x13 * x308 * x309 * x441)
    result[2, 0, 0] = numpy.sum(x314 * x442)
    result[2, 0, 1] = numpy.sum(x323 * x442 * x95)
    result[2, 0, 2] = numpy.sum(x318 * x445 * x446)
    result[2, 0, 3] = numpy.sum(x125 * x333 * x447)
    result[2, 0, 4] = numpy.sum(x144 * x333 * x448)
    result[2, 0, 5] = numpy.sum(x305 * x333 * x451)
    result[2, 0, 6] = numpy.sum(x340 * x447 * x452)
    result[2, 0, 7] = numpy.sum(x124 * x340 * x445)
    result[2, 0, 8] = numpy.sum(x144 * x340 * x453)
    result[2, 0, 9] = numpy.sum(x340 * x455 * x456)
    result[2, 1, 0] = numpy.sum(x150 * x362 * x442)
    result[2, 1, 1] = numpy.sum(x157 * x317 * x447)
    result[2, 1, 2] = numpy.sum(x299 * x317 * x444)
    result[2, 1, 3] = numpy.sum(x162 * x332 * x457)
    result[2, 1, 4] = numpy.sum(x156 * x332 * x458)
    result[2, 1, 5] = numpy.sum(x299 * x332 * x451)
    result[2, 1, 6] = numpy.sum(x172 * x367 * x447)
    result[2, 1, 7] = numpy.sum(x162 * x361 * x444)
    result[2, 1, 8] = numpy.sum(x156 * x360 * x451)
    result[2, 1, 9] = numpy.sum(x153 * x367 * x455)
    result[2, 2, 0] = numpy.sum(x313 * x460 * x461)
    result[2, 2, 1] = numpy.sum(x144 * x317 * x462)
    result[2, 2, 2] = numpy.sum(x274 * x317 * x464)
    result[2, 2, 3] = numpy.sum(x185 * x332 * x460)
    result[2, 2, 4] = numpy.sum(x332 * x464 * x465)
    result[2, 2, 5] = numpy.sum(x274 * x332 * x468)
    result[2, 2, 6] = numpy.sum(x189 * x328 * x460)
    result[2, 2, 7] = numpy.sum(x185 * x328 * x464)
    result[2, 2, 8] = numpy.sum(x144 * x361 * x468)
    result[2, 2, 9] = numpy.sum(x328 * x461 * x473)
    result[2, 3, 0] = numpy.sum(x199 * x447 * x53)
    result[2, 3, 1] = numpy.sum(x204 * x474 * x51)
    result[2, 3, 2] = numpy.sum(x206 * x475 * x51)
    result[2, 3, 3] = numpy.sum(x211 * x380 * x474)
    result[2, 3, 4] = numpy.sum(x201 * x381 * x444)
    result[2, 3, 5] = numpy.sum(x206 * x380 * x451)
    result[2, 3, 6] = numpy.sum(x217 * x389 * x447)
    result[2, 3, 7] = numpy.sum(x211 * x388 * x475)
    result[2, 3, 8] = numpy.sum(x204 * x388 * x451)
    result[2, 3, 9] = numpy.sum(x111 * x199 * x455)
    result[2, 4, 0] = numpy.sum(x153 * x462 * x53)
    result[2, 4, 1] = numpy.sum(x155 * x476 * x51)
    result[2, 4, 2] = numpy.sum(x153 * x477 * x51)
    result[2, 4, 3] = numpy.sum(x162 * x381 * x460)
    result[2, 4, 4] = numpy.sum(x155 * x393 * x464)
    result[2, 4, 5] = numpy.sum(x153 * x381 * x468)
    result[2, 4, 6] = numpy.sum(x111 * x172 * x462)
    result[2, 4, 7] = numpy.sum(x162 * x394 * x464)
    result[2, 4, 8] = numpy.sum(x155 * x394 * x468)
    result[2, 4, 9] = numpy.sum(x111 * x299 * x473)
    result[2, 5, 0] = numpy.sum(x120 * x482 * x53)
    result[2, 5, 1] = numpy.sum(x231 * x483 * x51)
    result[2, 5, 2] = numpy.sum(x486 * x487 * x51)
    result[2, 5, 3] = numpy.sum(x237 * x380 * x481)
    result[2, 5, 4] = numpy.sum(x144 * x381 * x485)
    result[2, 5, 5] = numpy.sum(x380 * x487 * x489)
    result[2, 5, 6] = numpy.sum(x242 * x388 * x480)
    result[2, 5, 7] = numpy.sum(x237 * x388 * x486)
    result[2, 5, 8] = numpy.sum(x231 * x390 * x489)
    result[2, 5, 9] = numpy.sum(x120 * x389 * x495)
    result[2, 6, 0] = numpy.sum(x248 * x401 * x447)
    result[2, 6, 1] = numpy.sum(x24 * x251 * x457)
    result[2, 6, 2] = numpy.sum(x24 * x253 * x444)
    result[2, 6, 3] = numpy.sum(x22 * x255 * x457)
    result[2, 6, 4] = numpy.sum(x22 * x256 * x458)
    result[2, 6, 5] = numpy.sum(x22 * x253 * x451)
    result[2, 6, 6] = numpy.sum(x447 * x496 * x65)
    result[2, 6, 7] = numpy.sum(x255 * x412 * x444)
    result[2, 6, 8] = numpy.sum(x251 * x412 * x451)
    result[2, 6, 9] = numpy.sum(x248 * x455 * x65)
    result[2, 7, 0] = numpy.sum(x196 * x401 * x462)
    result[2, 7, 1] = numpy.sum(x201 * x24 * x476)
    result[2, 7, 2] = numpy.sum(x24 * x262 * x464)
    result[2, 7, 3] = numpy.sum(x210 * x22 * x476)
    result[2, 7, 4] = numpy.sum(x203 * x22 * x477)
    result[2, 7, 5] = numpy.sum(x22 * x262 * x468)
    result[2, 7, 6] = numpy.sum(x217 * x412 * x460)
    result[2, 7, 7] = numpy.sum(x210 * x414 * x464)
    result[2, 7, 8] = numpy.sum(x201 * x414 * x468)
    result[2, 7, 9] = numpy.sum(x196 * x412 * x473)
    result[2, 8, 0] = numpy.sum(x299 * x401 * x480)
    result[2, 8, 1] = numpy.sum(x155 * x24 * x497)
    result[2, 8, 2] = numpy.sum(x153 * x24 * x498)
    result[2, 8, 3] = numpy.sum(x162 * x22 * x497)
    result[2, 8, 4] = numpy.sum(x155 * x22 * x499)
    result[2, 8, 5] = numpy.sum(x22 * x267 * x489)
    result[2, 8, 6] = numpy.sum(x172 * x412 * x480)
    result[2, 8, 7] = numpy.sum(x162 * x414 * x485)
    result[2, 8, 8] = numpy.sum(x155 * x414 * x489)
    result[2, 8, 9] = numpy.sum(x153 * x412 * x495)
    result[2, 9, 0] = numpy.sum(x401 * x461 * x503)
    result[2, 9, 1] = numpy.sum(x144 * x24 * x504)
    result[2, 9, 2] = numpy.sum(x24 * x274 * x507)
    result[2, 9, 3] = numpy.sum(x185 * x22 * x503)
    result[2, 9, 4] = numpy.sum(x22 * x465 * x507)
    result[2, 9, 5] = numpy.sum(x22 * x274 * x509)
    result[2, 9, 6] = numpy.sum(x189 * x503 * x65)
    result[2, 9, 7] = numpy.sum(x185 * x507 * x65)
    result[2, 9, 8] = numpy.sum(x144 * x412 * x509)
    result[2, 9, 9] = numpy.sum(x461 * x513 * x65)
    result[2, 10, 0] = numpy.sum(x417 * x447 * x514)
    result[2, 10, 1] = numpy.sum(x283 * x447 * x515)
    result[2, 10, 2] = numpy.sum(x285 * x416 * x444)
    result[2, 10, 3] = numpy.sum(x422 * x447 * x516)
    result[2, 10, 4] = numpy.sum(x283 * x423 * x444)
    result[2, 10, 5] = numpy.sum(x20 * x285 * x451)
    result[2, 10, 6] = numpy.sum(x13 * x289 * x425 * x442)
    result[2, 10, 7] = numpy.sum(x18 * x445 * x516)
    result[2, 10, 8] = numpy.sum(x18 * x283 * x453)
    result[2, 10, 9] = numpy.sum(x18 * x455 * x514)
    result[2, 11, 0] = numpy.sum(x248 * x417 * x460)
    result[2, 11, 1] = numpy.sum(x251 * x429 * x460)
    result[2, 11, 2] = numpy.sum(x253 * x416 * x464)
    result[2, 11, 3] = numpy.sum(x255 * x431 * x460)
    result[2, 11, 4] = numpy.sum(x256 * x432 * x464)
    result[2, 11, 5] = numpy.sum(x20 * x253 * x468)
    result[2, 11, 6] = numpy.sum(x18 * x460 * x496)
    result[2, 11, 7] = numpy.sum(x255 * x434 * x464)
    result[2, 11, 8] = numpy.sum(x251 * x434 * x468)
    result[2, 11, 9] = numpy.sum(x18 * x248 * x473)
    result[2, 12, 0] = numpy.sum(x199 * x417 * x480)
    result[2, 12, 1] = numpy.sum(x204 * x416 * x481)
    result[2, 12, 2] = numpy.sum(x206 * x416 * x486)
    result[2, 12, 3] = numpy.sum(x211 * x435 * x481)
    result[2, 12, 4] = numpy.sum(x20 * x201 * x498)
    result[2, 12, 5] = numpy.sum(x206 * x435 * x489)
    result[2, 12, 6] = numpy.sum(x18 * x217 * x482)
    result[2, 12, 7] = numpy.sum(x211 * x436 * x486)
    result[2, 12, 8] = numpy.sum(x204 * x436 * x489)
    result[2, 12, 9] = numpy.sum(x18 * x199 * x495)
    result[2, 13, 0] = numpy.sum(x153 * x417 * x517)
    result[2, 13, 1] = numpy.sum(x156 * x430 * x503)
    result[2, 13, 2] = numpy.sum(x153 * x429 * x507)
    result[2, 13, 3] = numpy.sum(x162 * x431 * x503)
    result[2, 13, 4] = numpy.sum(x156 * x432 * x507)
    result[2, 13, 5] = numpy.sum(x153 * x431 * x509)
    result[2, 13, 6] = numpy.sum(x172 * x18 * x517)
    result[2, 13, 7] = numpy.sum(x162 * x434 * x507)
    result[2, 13, 8] = numpy.sum(x157 * x18 * x509)
    result[2, 13, 9] = numpy.sum(x13 * x150 * x518)
    result[2, 14, 0] = numpy.sum(x417 * x456 * x519)
    result[2, 14, 1] = numpy.sum(x144 * x515 * x519)
    result[2, 14, 2] = numpy.sum(x305 * x416 * x520)
    result[2, 14, 3] = numpy.sum(x124 * x422 * x519)
    result[2, 14, 4] = numpy.sum(x144 * x423 * x520)
    result[2, 14, 5] = numpy.sum(x422 * x446 * x522)
    result[2, 14, 6] = numpy.sum(x18 * x452 * x519)
    result[2, 14, 7] = numpy.sum(x125 * x18 * x520)
    result[2, 14, 8] = numpy.sum(x13 * x523 * x95)
    result[2, 14, 9] = numpy.sum(x13 * x524)
    result[3, 0, 0] = numpy.sum(x282 * x525 * x528)
    result[3, 0, 1] = numpy.sum(x284 * x530 * x531)
    result[3, 0, 2] = numpy.sum(x143 * x528 * x532)
    result[3, 0, 3] = numpy.sum(x284 * x533 * x534)
    result[3, 0, 4] = numpy.sum(x287 * x534 * x535)
    result[3, 0, 5] = numpy.sum(x134 * x527 * x534)
    result[3, 0, 6] = numpy.sum(x341 * x536 * x537)
    result[3, 0, 7] = numpy.sum(x287 * x533 * x538)
    result[3, 0, 8] = numpy.sum(x134 * x530 * x536)
    result[3, 0, 9] = numpy.sum(x147 * x528 * x536)
    result[3, 1, 0] = numpy.sum(x312 * x346 * x540)
    result[3, 1, 1] = numpy.sum(x252 * x316 * x542)
    result[3, 1, 2] = numpy.sum(x316 * x540 * x543)
    result[3, 1, 3] = numpy.sum(x252 * x331 * x546)
    result[3, 1, 4] = numpy.sum(x542 * x547 * x548)
    result[3, 1, 5] = numpy.sum(x167 * x331 * x540)
    result[3, 1, 6] = numpy.sum(x119 * x552 * x553)
    result[3, 1, 7] = numpy.sum(x143 * x546 * x554)
    result[3, 1, 8] = numpy.sum(x166 * x542 * x553)
    result[3, 1, 9] = numpy.sum(x175 * x339 * x540)
    result[3, 2, 0] = numpy.sum(x179 * x527 * x555)
    result[3, 2, 1] = numpy.sum(x223 * x316 * x530)
    result[3, 2, 2] = numpy.sum(x183 * x316 * x527)
    result[3, 2, 3] = numpy.sum(x223 * x331 * x533)
    result[3, 2, 4] = numpy.sum(x182 * x331 * x556)
    result[3, 2, 5] = numpy.sum(x188 * x331 * x557)
    result[3, 2, 6] = numpy.sum(x179 * x537 * x553)
    result[3, 2, 7] = numpy.sum(x181 * x533 * x554)
    result[3, 2, 8] = numpy.sum(x188 * x530 * x554)
    result[3, 2, 9] = numpy.sum(x194 * x527 * x553)
    result[3, 3, 0] = numpy.sum(x119 * x44 * x562)
    result[3, 3, 1] = numpy.sum(x376 * x42 * x565)
    result[3, 3, 2] = numpy.sum(x207 * x561 * x566)
    result[3, 3, 3] = numpy.sum(x376 * x568 * x569)
    result[3, 3, 4] = numpy.sum(x143 * x564 * x570)
    result[3, 3, 5] = numpy.sum(x214 * x561 * x569)
    result[3, 3, 6] = numpy.sum(x119 * x573 * x575)
    result[3, 3, 7] = numpy.sum(x207 * x568 * x576)
    result[3, 3, 8] = numpy.sum(x214 * x565 * x574)
    result[3, 3, 9] = numpy.sum(x337 * x391 * x561)
    result[3, 4, 0] = numpy.sum(x223 * x44 * x540)
    result[3, 4, 1] = numpy.sum(x221 * x42 * x542)
    result[3, 4, 2] = numpy.sum(x181 * x42 * x577)
    result[3, 4, 3] = numpy.sum(x179 * x546 * x570)
    result[3, 4, 4] = numpy.sum(x181 * x542 * x578)
    result[3, 4, 5] = numpy.sum(x112 * x188 * x577)
    result[3, 4, 6] = numpy.sum(x223 * x337 * x552)
    result[3, 4, 7] = numpy.sum(x181 * x546 * x579)
    result[3, 4, 8] = numpy.sum(x188 * x542 * x579)
    result[3, 4, 9] = numpy.sum(x225 * x337 * x540)
    result[3, 5, 0] = numpy.sum(x229 * x44 * x527)
    result[3, 5, 1] = numpy.sum(x230 * x42 * x580)
    result[3, 5, 2] = numpy.sum(x235 * x42 * x581)
    result[3, 5, 3] = numpy.sum(x230 * x533 * x569)
    result[3, 5, 4] = numpy.sum(x112 * x238 * x530)
    result[3, 5, 5] = numpy.sum(x241 * x569 * x581)
    result[3, 5, 6] = numpy.sum(x229 * x337 * x537)
    result[3, 5, 7] = numpy.sum(x235 * x533 * x574)
    result[3, 5, 8] = numpy.sum(x241 * x574 * x580)
    result[3, 5, 9] = numpy.sum(x245 * x527 * x575)
    result[3, 6, 0] = numpy.sum(x346 * x582 * x583)
    result[3, 6, 1] = numpy.sum(x252 * x32 * x586)
    result[3, 6, 2] = numpy.sum(x32 * x543 * x582)
    result[3, 6, 3] = numpy.sum(x119 * x588 * x589)
    result[3, 6, 4] = numpy.sum(x548 * x586 * x590)
    result[3, 6, 5] = numpy.sum(x167 * x30 * x582)
    result[3, 6, 6] = numpy.sum(x593 * x594)
    result[3, 6, 7] = numpy.sum(x588 * x595 * x596)
    result[3, 6, 8] = numpy.sum(x11 * x167 * x586)
    result[3, 6, 9] = numpy.sum(x11 * x175 * x582)
    result[3, 7, 0] = numpy.sum(x223 * x560 * x583)
    result[3, 7, 1] = numpy.sum(x221 * x32 * x564)
    result[3, 7, 2] = numpy.sum(x181 * x32 * x597)
    result[3, 7, 3] = numpy.sum(x179 * x568 * x598)
    result[3, 7, 4] = numpy.sum(x222 * x30 * x565)
    result[3, 7, 5] = numpy.sum(x188 * x30 * x597)
    result[3, 7, 6] = numpy.sum(x573 * x596 * x599)
    result[3, 7, 7] = numpy.sum(x11 * x222 * x568)
    result[3, 7, 8] = numpy.sum(x188 * x564 * x600)
    result[3, 7, 9] = numpy.sum(x11 * x225 * x560)
    result[3, 8, 0] = numpy.sum(x264 * x540 * x583)
    result[3, 8, 1] = numpy.sum(x265 * x32 * x542)
    result[3, 8, 2] = numpy.sum(x238 * x32 * x540)
    result[3, 8, 3] = numpy.sum(x265 * x30 * x546)
    result[3, 8, 4] = numpy.sum(x266 * x30 * x542)
    result[3, 8, 5] = numpy.sum(x240 * x30 * x577)
    result[3, 8, 6] = numpy.sum(x11 * x264 * x552)
    result[3, 8, 7] = numpy.sum(x11 * x238 * x546)
    result[3, 8, 8] = numpy.sum(x240 * x542 * x600)
    result[3, 8, 9] = numpy.sum(x245 * x540 * x601)
    result[3, 9, 0] = numpy.sum(x269 * x527 * x583)
    result[3, 9, 1] = numpy.sum(x270 * x32 * x530)
    result[3, 9, 2] = numpy.sum(x273 * x32 * x557)
    result[3, 9, 3] = numpy.sum(x270 * x30 * x533)
    result[3, 9, 4] = numpy.sum(x275 * x30 * x556)
    result[3, 9, 5] = numpy.sum(x277 * x30 * x557)
    result[3, 9, 6] = numpy.sum(x11 * x269 * x537)
    result[3, 9, 7] = numpy.sum(x273 * x533 * x601)
    result[3, 9, 8] = numpy.sum(x277 * x530 * x601)
    result[3, 9, 9] = numpy.sum(x11 * x415 * x527)
    result[3, 10, 0] = numpy.sum(x602 * x605 * x606)
    result[3, 10, 1] = numpy.sum(x606 * x607 * x608)
    result[3, 10, 2] = numpy.sum(x287 * x602 * x608)
    result[3, 10, 3] = numpy.sum(x609 * x610)
    result[3, 10, 4] = numpy.sum(x128 * x3 * x595 * x607)
    result[3, 10, 5] = numpy.sum(x134 * x15 * x602)
    result[3, 10, 6] = numpy.sum(
        x425
        * (
            x0 * (2.0 * x409 + 2.0 * x410 + 6.0 * x571 + 3.0 * x572 + 3.0 * x587)
            + x148 * x592
        )
    )
    result[3, 10, 7] = numpy.sum(x108 * x609)
    result[3, 10, 8] = numpy.sum(x134 * x607 * x7)
    result[3, 10, 9] = numpy.sum(x147 * x602 * x611)
    result[3, 11, 0] = numpy.sum(x179 * x582 * x612)
    result[3, 11, 1] = numpy.sum(x179 * x586 * x613)
    result[3, 11, 2] = numpy.sum(x183 * x582 * x603)
    result[3, 11, 3] = numpy.sum(x158 * x3 * x588 * x599)
    result[3, 11, 4] = numpy.sum(x182 * x586 * x614)
    result[3, 11, 5] = numpy.sum(x188 * x582 * x615)
    result[3, 11, 6] = numpy.sum(x177 * x593)
    result[3, 11, 7] = numpy.sum(x183 * x588 * x7)
    result[3, 11, 8] = numpy.sum(x188 * x586 * x616)
    result[3, 11, 9] = numpy.sum(x194 * x582 * x617)
    result[3, 12, 0] = numpy.sum(x229 * x560 * x604)
    result[3, 12, 1] = numpy.sum(x230 * x565 * x603)
    result[3, 12, 2] = numpy.sum(x235 * x561 * x603)
    result[3, 12, 3] = numpy.sum(x230 * x568 * x618)
    result[3, 12, 4] = numpy.sum(x15 * x238 * x564)
    result[3, 12, 5] = numpy.sum(x241 * x561 * x618)
    result[3, 12, 6] = numpy.sum(x229 * x573 * x7)
    result[3, 12, 7] = numpy.sum(x235 * x568 * x619)
    result[3, 12, 8] = numpy.sum(x241 * x565 * x619)
    result[3, 12, 9] = numpy.sum(x245 * x562 * x7)
    result[3, 13, 0] = numpy.sum(x269 * x540 * x604)
    result[3, 13, 1] = numpy.sum(x270 * x542 * x603)
    result[3, 13, 2] = numpy.sum(x273 * x540 * x613)
    result[3, 13, 3] = numpy.sum(x15 * x270 * x546)
    result[3, 13, 4] = numpy.sum(x275 * x542 * x614)
    result[3, 13, 5] = numpy.sum(x277 * x540 * x615)
    result[3, 13, 6] = numpy.sum(x269 * x552 * x7)
    result[3, 13, 7] = numpy.sum(x273 * x546 * x616)
    result[3, 13, 8] = numpy.sum(x277 * x542 * x616)
    result[3, 13, 9] = numpy.sum(x279 * x540 * x617)
    result[3, 14, 0] = numpy.sum(x301 * x527 * x605)
    result[3, 14, 1] = numpy.sum(x303 * x530 * x603)
    result[3, 14, 2] = numpy.sum(x304 * x528 * x608)
    result[3, 14, 3] = numpy.sum(x15 * x303 * x533)
    result[3, 14, 4] = numpy.sum(x15 * x306 * x535)
    result[3, 14, 5] = numpy.sum(x15 * x439 * x528)
    result[3, 14, 6] = numpy.sum(x301 * x537 * x620)
    result[3, 14, 7] = numpy.sum(x304 * x533 * x621)
    result[3, 14, 8] = numpy.sum(x307 * x530 * x621)
    result[3, 14, 9] = numpy.sum(x308 * x528 * x620)
    result[4, 0, 0] = numpy.sum(x309 * x442 * x525 * x90)
    result[4, 0, 1] = numpy.sum(x321 * x447 * x532)
    result[4, 0, 2] = numpy.sum(x324 * x445 * x532)
    result[4, 0, 3] = numpy.sum(x447 * x534 * x622)
    result[4, 0, 4] = numpy.sum(x334 * x445 * x534)
    result[4, 0, 5] = numpy.sum(x324 * x453 * x534)
    result[4, 0, 6] = numpy.sum(x336 * x447 * x623)
    result[4, 0, 7] = numpy.sum(x327 * x445 * x538)
    result[4, 0, 8] = numpy.sum(x321 * x451 * x538)
    result[4, 0, 9] = numpy.sum(x324 * x455 * x623)
    result[4, 1, 0] = numpy.sum(x345 * x447 * x555)
    result[4, 1, 1] = numpy.sum(x316 * x348 * x457)
    result[4, 1, 2] = numpy.sum(x316 * x349 * x444)
    result[4, 1, 3] = numpy.sum(x331 * x352 * x457)
    result[4, 1, 4] = numpy.sum(x348 * x458 * x547)
    result[4, 1, 5] = numpy.sum(x331 * x349 * x451)
    result[4, 1, 6] = numpy.sum(x359 * x447 * x553)
    result[4, 1, 7] = numpy.sum(x352 * x444 * x554)
    result[4, 1, 8] = numpy.sum(x348 * x451 * x554)
    result[4, 1, 9] = numpy.sum(x345 * x455 * x553)
    result[4, 2, 0] = numpy.sum(x324 * x460 * x555)
    result[4, 2, 1] = numpy.sum(x316 * x364 * x460)
    result[4, 2, 2] = numpy.sum(x316 * x366 * x464)
    result[4, 2, 3] = numpy.sum(x327 * x331 * x462)
    result[4, 2, 4] = numpy.sum(x365 * x464 * x547)
    result[4, 2, 5] = numpy.sum(x331 * x366 * x468)
    result[4, 2, 6] = numpy.sum(x336 * x460 * x553)
    result[4, 2, 7] = numpy.sum(x327 * x464 * x554)
    result[4, 2, 8] = numpy.sum(x320 * x468 * x554)
    result[4, 2, 9] = numpy.sum(x324 * x473 * x553)
    result[4, 3, 0] = numpy.sum(x372 * x44 * x447)
    result[4, 3, 1] = numpy.sum(x375 * x474 * x566)
    result[4, 3, 2] = numpy.sum(x377 * x42 * x475)
    result[4, 3, 3] = numpy.sum(x379 * x474 * x624)
    result[4, 3, 4] = numpy.sum(x374 * x444 * x570)
    result[4, 3, 5] = numpy.sum(x377 * x451 * x569)
    result[4, 3, 6] = numpy.sum(x387 * x447 * x575)
    result[4, 3, 7] = numpy.sum(x379 * x475 * x576)
    result[4, 3, 8] = numpy.sum(x375 * x451 * x576)
    result[4, 3, 9] = numpy.sum(x371 * x455 * x625)
    result[4, 4, 0] = numpy.sum(x345 * x44 * x462)
    result[4, 4, 1] = numpy.sum(x348 * x42 * x476)
    result[4, 4, 2] = numpy.sum(x392 * x42 * x464)
    result[4, 4, 3] = numpy.sum(x112 * x352 * x476)
    result[4, 4, 4] = numpy.sum(x348 * x464 * x578)
    result[4, 4, 5] = numpy.sum(x112 * x392 * x468)
    result[4, 4, 6] = numpy.sum(x337 * x359 * x462)
    result[4, 4, 7] = numpy.sum(x352 * x464 * x579)
    result[4, 4, 8] = numpy.sum(x348 * x468 * x579)
    result[4, 4, 9] = numpy.sum(x337 * x349 * x473)
    result[4, 5, 0] = numpy.sum(x324 * x44 * x482)
    result[4, 5, 1] = numpy.sum(x395 * x42 * x483)
    result[4, 5, 2] = numpy.sum(x396 * x486 * x566)
    result[4, 5, 3] = numpy.sum(x327 * x483 * x569)
    result[4, 5, 4] = numpy.sum(x112 * x320 * x498)
    result[4, 5, 5] = numpy.sum(x396 * x489 * x624)
    result[4, 5, 6] = numpy.sum(x336 * x481 * x625)
    result[4, 5, 7] = numpy.sum(x327 * x486 * x576)
    result[4, 5, 8] = numpy.sum(x395 * x489 * x576)
    result[4, 5, 9] = numpy.sum(x324 * x495 * x575)
    result[4, 6, 0] = numpy.sum(x428 * x447 * x583)
    result[4, 6, 1] = numpy.sum(x32 * x404 * x457)
    result[4, 6, 2] = numpy.sum(x32 * x405 * x444)
    result[4, 6, 3] = numpy.sum(x407 * x447 * x589)
    result[4, 6, 4] = numpy.sum(x404 * x458 * x590)
    result[4, 6, 5] = numpy.sum(x30 * x405 * x451)
    result[4, 6, 6] = numpy.sum(x433 * x442 * x594)
    result[4, 6, 7] = numpy.sum(x407 * x444 * x601)
    result[4, 6, 8] = numpy.sum(x404 * x451 * x601)
    result[4, 6, 9] = numpy.sum(x11 * x428 * x455)
    result[4, 7, 0] = numpy.sum(x370 * x462 * x583)
    result[4, 7, 1] = numpy.sum(x32 * x374 * x476)
    result[4, 7, 2] = numpy.sum(x32 * x413 * x464)
    result[4, 7, 3] = numpy.sum(x30 * x379 * x476)
    result[4, 7, 4] = numpy.sum(x30 * x375 * x477)
    result[4, 7, 5] = numpy.sum(x30 * x413 * x468)
    result[4, 7, 6] = numpy.sum(x11 * x387 * x462)
    result[4, 7, 7] = numpy.sum(x11 * x379 * x477)
    result[4, 7, 8] = numpy.sum(x374 * x468 * x600)
    result[4, 7, 9] = numpy.sum(x370 * x473 * x601)
    result[4, 8, 0] = numpy.sum(x349 * x480 * x583)
    result[4, 8, 1] = numpy.sum(x32 * x348 * x497)
    result[4, 8, 2] = numpy.sum(x32 * x392 * x485)
    result[4, 8, 3] = numpy.sum(x30 * x352 * x497)
    result[4, 8, 4] = numpy.sum(x30 * x348 * x499)
    result[4, 8, 5] = numpy.sum(x30 * x392 * x489)
    result[4, 8, 6] = numpy.sum(x359 * x480 * x601)
    result[4, 8, 7] = numpy.sum(x11 * x352 * x498)
    result[4, 8, 8] = numpy.sum(x348 * x489 * x600)
    result[4, 8, 9] = numpy.sum(x11 * x349 * x495)
    result[4, 9, 0] = numpy.sum(x324 * x517 * x583)
    result[4, 9, 1] = numpy.sum(x32 * x364 * x503)
    result[4, 9, 2] = numpy.sum(x32 * x366 * x507)
    result[4, 9, 3] = numpy.sum(x327 * x503 * x589)
    result[4, 9, 4] = numpy.sum(x365 * x507 * x590)
    result[4, 9, 5] = numpy.sum(x324 * x509 * x589)
    result[4, 9, 6] = numpy.sum(x11 * x336 * x517)
    result[4, 9, 7] = numpy.sum(x327 * x507 * x601)
    result[4, 9, 8] = numpy.sum(x11 * x364 * x509)
    result[4, 9, 9] = numpy.sum(x309 * x518 * x594)
    result[4, 10, 0] = numpy.sum(x418 * x605 * x626)
    result[4, 10, 1] = numpy.sum(x419 * x608 * x626)
    result[4, 10, 2] = numpy.sum(x418 * x445 * x608)
    result[4, 10, 3] = numpy.sum(x427 * x442 * x610)
    result[4, 10, 4] = numpy.sum(x15 * x419 * x448)
    result[4, 10, 5] = numpy.sum(x15 * x418 * x453)
    result[4, 10, 6] = numpy.sum(x426 * x442)
    result[4, 10, 7] = numpy.sum(x421 * x445 * x627)
    result[4, 10, 8] = numpy.sum(x419 * x451 * x621)
    result[4, 10, 9] = numpy.sum(x418 * x455 * x628)
    result[4, 11, 0] = numpy.sum(x400 * x460 * x612)
    result[4, 11, 1] = numpy.sum(x404 * x462 * x603)
    result[4, 11, 2] = numpy.sum(x400 * x464 * x613)
    result[4, 11, 3] = numpy.sum(x15 * x407 * x462)
    result[4, 11, 4] = numpy.sum(x404 * x464 * x629)
    result[4, 11, 5] = numpy.sum(x15 * x405 * x468)
    result[4, 11, 6] = numpy.sum(x411 * x460 * x617)
    result[4, 11, 7] = numpy.sum(x407 * x464 * x616)
    result[4, 11, 8] = numpy.sum(x404 * x468 * x616)
    result[4, 11, 9] = numpy.sum(x400 * x473 * x617)
    result[4, 12, 0] = numpy.sum(x372 * x480 * x604)
    result[4, 12, 1] = numpy.sum(x375 * x483 * x603)
    result[4, 12, 2] = numpy.sum(x377 * x486 * x603)
    result[4, 12, 3] = numpy.sum(x379 * x483 * x618)
    result[4, 12, 4] = numpy.sum(x15 * x374 * x498)
    result[4, 12, 5] = numpy.sum(x377 * x489 * x618)
    result[4, 12, 6] = numpy.sum(x387 * x482 * x7)
    result[4, 12, 7] = numpy.sum(x379 * x486 * x630)
    result[4, 12, 8] = numpy.sum(x375 * x489 * x630)
    result[4, 12, 9] = numpy.sum(x372 * x495 * x7)
    result[4, 13, 0] = numpy.sum(x345 * x503 * x612)
    result[4, 13, 1] = numpy.sum(x348 * x503 * x613)
    result[4, 13, 2] = numpy.sum(x345 * x507 * x613)
    result[4, 13, 3] = numpy.sum(x15 * x352 * x504)
    result[4, 13, 4] = numpy.sum(x348 * x507 * x629)
    result[4, 13, 5] = numpy.sum(x15 * x349 * x509)
    result[4, 13, 6] = numpy.sum(x359 * x503 * x617)
    result[4, 13, 7] = numpy.sum(x352 * x507 * x616)
    result[4, 13, 8] = numpy.sum(x348 * x509 * x616)
    result[4, 13, 9] = numpy.sum(x345 * x513 * x617)
    result[4, 14, 0] = numpy.sum(x519 * x605 * x631)
    result[4, 14, 1] = numpy.sum(x321 * x519 * x608)
    result[4, 14, 2] = numpy.sum(x520 * x608 * x631)
    result[4, 14, 3] = numpy.sum(x15 * x519 * x622)
    result[4, 14, 4] = numpy.sum(x334 * x520 * x632)
    result[4, 14, 5] = numpy.sum(x309 * x523 * x610)
    result[4, 14, 6] = numpy.sum(x336 * x519 * x628)
    result[4, 14, 7] = numpy.sum(x327 * x520 * x621)
    result[4, 14, 8] = numpy.sum(x321 * x522 * x627)
    result[4, 14, 9] = numpy.sum(x309 * x524)
    result[5, 0, 0] = numpy.sum(x302 * x525 * x635)
    result[5, 0, 1] = numpy.sum(x144 * x532 * x635)
    result[5, 0, 2] = numpy.sum(x305 * x531 * x637)
    result[5, 0, 3] = numpy.sum(x124 * x534 * x635)
    result[5, 0, 4] = numpy.sum(x534 * x638 * x639)
    result[5, 0, 5] = numpy.sum(x305 * x534 * x640)
    result[5, 0, 6] = numpy.sum(x137 * x536 * x635)
    result[5, 0, 7] = numpy.sum(x125 * x536 * x637)
    result[5, 0, 8] = numpy.sum(x538 * x638 * x640)
    result[5, 0, 9] = numpy.sum(x456 * x536 * x641)
    result[5, 1, 0] = numpy.sum(x153 * x555 * x634)
    result[5, 1, 1] = numpy.sum(x157 * x316 * x634)
    result[5, 1, 2] = numpy.sum(x299 * x316 * x637)
    result[5, 1, 3] = numpy.sum(x162 * x331 * x642)
    result[5, 1, 4] = numpy.sum(x156 * x331 * x643)
    result[5, 1, 5] = numpy.sum(x299 * x331 * x640)
    result[5, 1, 6] = numpy.sum(x172 * x553 * x634)
    result[5, 1, 7] = numpy.sum(x162 * x554 * x637)
    result[5, 1, 8] = numpy.sum(x155 * x554 * x640)
    result[5, 1, 9] = numpy.sum(x153 * x553 * x641)
    result[5, 2, 0] = numpy.sum(x312 * x461 * x645)
    result[5, 2, 1] = numpy.sum(x144 * x316 * x646)
    result[5, 2, 2] = numpy.sum(x274 * x316 * x648)
    result[5, 2, 3] = numpy.sum(x185 * x331 * x645)
    result[5, 2, 4] = numpy.sum(x547 * x648 * x649)
    result[5, 2, 5] = numpy.sum(x274 * x331 * x652)
    result[5, 2, 6] = numpy.sum(x189 * x339 * x645)
    result[5, 2, 7] = numpy.sum(x184 * x553 * x648)
    result[5, 2, 8] = numpy.sum(x144 * x554 * x652)
    result[5, 2, 9] = numpy.sum(x120 * x553 * x656)
    result[5, 3, 0] = numpy.sum(x199 * x44 * x634)
    result[5, 3, 1] = numpy.sum(x204 * x42 * x657)
    result[5, 3, 2] = numpy.sum(x206 * x42 * x658)
    result[5, 3, 3] = numpy.sum(x211 * x569 * x657)
    result[5, 3, 4] = numpy.sum(x201 * x570 * x637)
    result[5, 3, 5] = numpy.sum(x206 * x569 * x640)
    result[5, 3, 6] = numpy.sum(x217 * x575 * x634)
    result[5, 3, 7] = numpy.sum(x211 * x574 * x658)
    result[5, 3, 8] = numpy.sum(x204 * x574 * x640)
    result[5, 3, 9] = numpy.sum(x199 * x337 * x641)
    result[5, 4, 0] = numpy.sum(x153 * x44 * x646)
    result[5, 4, 1] = numpy.sum(x155 * x42 * x659)
    result[5, 4, 2] = numpy.sum(x267 * x42 * x648)
    result[5, 4, 3] = numpy.sum(x162 * x570 * x645)
    result[5, 4, 4] = numpy.sum(x155 * x578 * x648)
    result[5, 4, 5] = numpy.sum(x153 * x570 * x652)
    result[5, 4, 6] = numpy.sum(x172 * x337 * x646)
    result[5, 4, 7] = numpy.sum(x162 * x579 * x648)
    result[5, 4, 8] = numpy.sum(x155 * x579 * x652)
    result[5, 4, 9] = numpy.sum(x299 * x337 * x656)
    result[5, 5, 0] = numpy.sum(x120 * x44 * x664)
    result[5, 5, 1] = numpy.sum(x231 * x566 * x663)
    result[5, 5, 2] = numpy.sum(x42 * x487 * x667)
    result[5, 5, 3] = numpy.sum(x237 * x569 * x663)
    result[5, 5, 4] = numpy.sum(x144 * x570 * x666)
    result[5, 5, 5] = numpy.sum(x487 * x569 * x669)
    result[5, 5, 6] = numpy.sum(x242 * x574 * x662)
    result[5, 5, 7] = numpy.sum(x237 * x574 * x667)
    result[5, 5, 8] = numpy.sum(x231 * x576 * x669)
    result[5, 5, 9] = numpy.sum(x120 * x575 * x672)
    result[5, 6, 0] = numpy.sum(x248 * x583 * x634)
    result[5, 6, 1] = numpy.sum(x251 * x32 * x642)
    result[5, 6, 2] = numpy.sum(x253 * x32 * x637)
    result[5, 6, 3] = numpy.sum(x255 * x589 * x634)
    result[5, 6, 4] = numpy.sum(x256 * x30 * x643)
    result[5, 6, 5] = numpy.sum(x253 * x30 * x640)
    result[5, 6, 6] = numpy.sum(x11 * x496 * x634)
    result[5, 6, 7] = numpy.sum(x255 * x601 * x637)
    result[5, 6, 8] = numpy.sum(x251 * x601 * x640)
    result[5, 6, 9] = numpy.sum(x11 * x248 * x641)
    result[5, 7, 0] = numpy.sum(x196 * x583 * x646)
    result[5, 7, 1] = numpy.sum(x201 * x32 * x659)
    result[5, 7, 2] = numpy.sum(x262 * x32 * x648)
    result[5, 7, 3] = numpy.sum(x210 * x30 * x659)
    result[5, 7, 4] = numpy.sum(x203 * x598 * x648)
    result[5, 7, 5] = numpy.sum(x262 * x30 * x652)
    result[5, 7, 6] = numpy.sum(x11 * x217 * x646)
    result[5, 7, 7] = numpy.sum(x210 * x600 * x648)
    result[5, 7, 8] = numpy.sum(x201 * x600 * x652)
    result[5, 7, 9] = numpy.sum(x196 * x601 * x656)
    result[5, 8, 0] = numpy.sum(x299 * x583 * x662)
    result[5, 8, 1] = numpy.sum(x155 * x32 * x673)
    result[5, 8, 2] = numpy.sum(x267 * x32 * x666)
    result[5, 8, 3] = numpy.sum(x162 * x30 * x673)
    result[5, 8, 4] = numpy.sum(x155 * x598 * x667)
    result[5, 8, 5] = numpy.sum(x153 * x598 * x669)
    result[5, 8, 6] = numpy.sum(x172 * x601 * x662)
    result[5, 8, 7] = numpy.sum(x162 * x600 * x666)
    result[5, 8, 8] = numpy.sum(x155 * x600 * x669)
    result[5, 8, 9] = numpy.sum(x10 * x672 * x674)
    result[5, 9, 0] = numpy.sum(x461 * x583 * x675)
    result[5, 9, 1] = numpy.sum(x144 * x158 * x32 * x675)
    result[5, 9, 2] = numpy.sum(x274 * x32 * x678)
    result[5, 9, 3] = numpy.sum(x185 * x30 * x675)
    result[5, 9, 4] = numpy.sum(x590 * x649 * x678)
    result[5, 9, 5] = numpy.sum(x120 * x589 * x680)
    result[5, 9, 6] = numpy.sum(x11 * x189 * x675)
    result[5, 9, 7] = numpy.sum(x11 * x185 * x678)
    result[5, 9, 8] = numpy.sum(x440 * x596 * x680 * x92)
    result[5, 9, 9] = numpy.sum(x594 * x683)
    result[5, 10, 0] = numpy.sum(x281 * x605 * x634)
    result[5, 10, 1] = numpy.sum(x283 * x608 * x635)
    result[5, 10, 2] = numpy.sum(x285 * x603 * x637)
    result[5, 10, 3] = numpy.sum(x15 * x516 * x635)
    result[5, 10, 4] = numpy.sum(x283 * x632 * x639)
    result[5, 10, 5] = numpy.sum(x15 * x285 * x640)
    result[5, 10, 6] = numpy.sum(x289 * x620 * x635)
    result[5, 10, 7] = numpy.sum(x286 * x621 * x637)
    result[5, 10, 8] = numpy.sum(x283 * x621 * x640)
    result[5, 10, 9] = numpy.sum(x281 * x620 * x641)
    result[5, 11, 0] = numpy.sum(x248 * x604 * x645)
    result[5, 11, 1] = numpy.sum(x251 * x613 * x645)
    result[5, 11, 2] = numpy.sum(x253 * x603 * x648)
    result[5, 11, 3] = numpy.sum(x15 * x255 * x646)
    result[5, 11, 4] = numpy.sum(x256 * x614 * x648)
    result[5, 11, 5] = numpy.sum(x15 * x253 * x652)
    result[5, 11, 6] = numpy.sum(x259 * x617 * x645)
    result[5, 11, 7] = numpy.sum(x255 * x616 * x648)
    result[5, 11, 8] = numpy.sum(x251 * x616 * x652)
    result[5, 11, 9] = numpy.sum(x248 * x656 * x7)
    result[5, 12, 0] = numpy.sum(x199 * x604 * x662)
    result[5, 12, 1] = numpy.sum(x204 * x603 * x663)
    result[5, 12, 2] = numpy.sum(x206 * x603 * x667)
    result[5, 12, 3] = numpy.sum(x211 * x618 * x663)
    result[5, 12, 4] = numpy.sum(x15 * x201 * x212 * x666)
    result[5, 12, 5] = numpy.sum(x206 * x618 * x669)
    result[5, 12, 6] = numpy.sum(x217 * x664 * x7)
    result[5, 12, 7] = numpy.sum(x211 * x619 * x667)
    result[5, 12, 8] = numpy.sum(x204 * x619 * x669)
    result[5, 12, 9] = numpy.sum(x199 * x672 * x7)
    result[5, 13, 0] = numpy.sum(x153 * x612 * x675)
    result[5, 13, 1] = numpy.sum(x157 * x603 * x675)
    result[5, 13, 2] = numpy.sum(x153 * x613 * x678)
    result[5, 13, 3] = numpy.sum(x162 * x615 * x675)
    result[5, 13, 4] = numpy.sum(x156 * x614 * x678)
    result[5, 13, 5] = numpy.sum(x3 * x674 * x680)
    result[5, 13, 6] = numpy.sum(x172 * x617 * x675)
    result[5, 13, 7] = numpy.sum(x162 * x616 * x678)
    result[5, 13, 8] = numpy.sum(x157 * x680 * x7)
    result[5, 13, 9] = numpy.sum(x150 * x683)
    result[5, 14, 0] = numpy.sum(x605 * x684 * x685)
    result[5, 14, 1] = numpy.sum(x608 * x638 * x684)
    result[5, 14, 2] = numpy.sum(x608 * x685 * x686)
    result[5, 14, 3] = numpy.sum(x125 * x15 * x684)
    result[5, 14, 4] = numpy.sum(x129 * x3 * x440 * x686)
    result[5, 14, 5] = numpy.sum(x610 * x687)
    result[5, 14, 6] = numpy.sum(x137 * x611 * x684)
    result[5, 14, 7] = numpy.sum(x125 * x686 * x7)
    result[5, 14, 8] = numpy.sum(x687 * x95)
    result[5, 14, 9] = numpy.sum(
        x441
        * (
            x0 * (2.0 * x511 + 2.0 * x512 + 6.0 * x670 + 3.0 * x671 + 3.0 * x679)
            + x176 * x682
        )
    )
    return result


def quadrupole3d_44(ax, da, A, bx, db, B, R):
    """Cartesian 3D (gg) quadrupole moment integrals.
    The origin is at R.

    Generated code; DO NOT modify by hand!"""

    result = numpy.zeros((6, 15, 15), dtype=float)

    x0 = 0.5 / (ax + bx)
    x1 = (ax + bx) ** (-1.0)
    x2 = -x1 * (ax * A[0] + bx * B[0])
    x3 = -x2 - A[0]
    x4 = -x2 - B[0]
    x5 = ax * bx * x1
    x6 = numpy.exp(-x5 * (A[0] - B[0]) ** 2)
    x7 = 1.772453850905516 * numpy.sqrt(x1)
    x8 = x6 * x7
    x9 = x0 * x8
    x10 = -x2 - R[0]
    x11 = x4 * x8
    x12 = x10 * x11
    x13 = x12 + x9
    x14 = x13 * x4
    x15 = x3 * x6
    x16 = x15 * x7
    x17 = x0 * (x11 + x16)
    x18 = x16 * x4
    x19 = x18 + x9
    x20 = x19 * x4
    x21 = x17 + x20
    x22 = x10 * x8
    x23 = x0 * (x11 + x22)
    x24 = 3.0 * x23
    x25 = x13 * x3
    x26 = 2.0 * x25
    x27 = x24 + x26
    x28 = x0 * (x14 + x21 + x27)
    x29 = 3.0 * x9
    x30 = x10 * x16
    x31 = x0 * (x12 + x18 + x29 + x30)
    x32 = x23 + x25
    x33 = x32 * x4
    x34 = x31 + x33
    x35 = x3 * x34
    x36 = x28 + x35
    x37 = x3 * x36
    x38 = x36 * x4
    x39 = x3 * x32
    x40 = 2.0 * x39
    x41 = 2.0 * x18
    x42 = x4**2 * x8
    x43 = x29 + x42
    x44 = x0 * (x41 + x43)
    x45 = x21 * x3
    x46 = x44 + x45
    x47 = 4.0 * x31
    x48 = 2.0 * x33 + x47
    x49 = x0 * (x40 + x46 + x48)
    x50 = x21 * x4
    x51 = 3.0 * x45
    x52 = x0 * (5.0 * x44 + 2.0 * x50 + x51)
    x53 = x19 * x3
    x54 = 2.0 * x0 * (2.0 * x17 + x20 + x53)
    x55 = x4 * x46
    x56 = x54 + x55
    x57 = x3 * x56
    x58 = x52 + x57
    x59 = x0 * (3.0 * x37 + 3.0 * x38 + 6.0 * x49 + x58)
    x60 = x34 * x4
    x61 = 2.0 * x60
    x62 = 3.0 * x35
    x63 = x0 * (5.0 * x28 + x56 + x61 + x62)
    x64 = x38 + x49
    x65 = x3 * x64
    x66 = x63 + x65
    x67 = x4 * x66
    x68 = 2.0 * x28
    x69 = x10 * x32
    x70 = x31 + x69
    x71 = x3 * x70
    x72 = x4 * x70
    x73 = x10 * x13
    x74 = x0 * (x16 + x22)
    x75 = x30 + x9
    x76 = x10 * x75
    x77 = x74 + x76
    x78 = x0 * (x27 + x73 + x77)
    x79 = x0 * (2.0 * x35 + x68 + 2.0 * x71 + 2.0 * x72 + 4.0 * x78)
    x80 = 2.0 * x69
    x81 = 2.0 * x12
    x82 = x10**2 * x8
    x83 = x29 + x82
    x84 = x0 * (x81 + x83)
    x85 = x23 + x73
    x86 = x4 * x85
    x87 = x84 + x86
    x88 = x0 * (x48 + x80 + x87)
    x89 = x72 + x78
    x90 = x3 * x89
    x91 = x88 + x90
    x92 = x4 * x91
    x93 = x79 + x92
    x94 = x4 * x93
    x95 = x3 * x93
    x96 = x4 * x89
    x97 = 2.0 * x49 + 3.0 * x90
    x98 = x0 * (2.0 * x38 + 5.0 * x88 + 2.0 * x96 + x97)
    x99 = x3 * x91
    x100 = x0 * (2.0 * x63 + 2.0 * x65 + 6.0 * x79 + 3.0 * x92 + 3.0 * x99)
    x101 = x95 + x98
    x102 = x100 + x101 * x4
    x103 = numpy.exp(-x5 * (A[1] - B[1]) ** 2)
    x104 = da * db
    x105 = 0.009523809523809524 * x104
    x106 = numpy.exp(-x5 * (A[2] - B[2]) ** 2)
    x107 = 3.141592653589793 * x1 * x106
    x108 = x105 * x107
    x109 = x103 * x108
    x110 = -x1 * (ax * A[1] + bx * B[1])
    x111 = -x110 - B[1]
    x112 = 2.645751311064591
    x113 = x109 * x112
    x114 = x113 * (x100 + x101 * x3)
    x115 = -x1 * (ax * A[2] + bx * B[2])
    x116 = -x115 - B[2]
    x117 = 2.0 * x30
    x118 = x0 * (x117 + x83)
    x119 = x3 * x77
    x120 = x118 + x119
    x121 = x0 * (x120 + x40 + x47 + x80)
    x122 = x71 + x78
    x123 = x122 * x3
    x124 = x79 + x99
    x125 = x0 * (2.0 * x121 + 2.0 * x123 + 2.0 * x37 + 3.0 * x88 + x97) + x124 * x3
    x126 = x106 * x7
    x127 = 0.03253000243161777
    x128 = x103 * x7
    x129 = x111**2 * x128
    x130 = x0 * x128
    x131 = x129 + x130
    x132 = x104 * x131
    x133 = x127 * x132
    x134 = 5.916079783099616
    x135 = x109 * x134
    x136 = x116**2 * x126
    x137 = x0 * x126
    x138 = x136 + x137
    x139 = x104 * x138
    x140 = x127 * x139
    x141 = 3.0 * x78
    x142 = x3 * x75
    x143 = 2.0 * x0 * (x142 + 2.0 * x74 + x76) + x120 * x3
    x144 = x142 + x74
    x145 = x17 + x53
    x146 = x0 * (x144 + x145 + 2.0 * x23 + x26)
    x147 = x31 + x39
    x148 = x147 * x3
    x149 = 2.0 * x146 + 2.0 * x148
    x150 = x121 + x123
    x151 = x0 * (x141 + x143 + x149 + 3.0 * x71) + x150 * x3
    x152 = 2.0 * x130
    x153 = x111 * (x131 + x152)
    x154 = x112 * x153
    x155 = x105 * x126
    x156 = x116 * x126
    x157 = x131 * x134
    x158 = x105 * x151
    x159 = x111 * x128
    x160 = x134 * x138
    x161 = 2.0 * x137
    x162 = x116 * (x138 + x161)
    x163 = x112 * x162
    x164 = x105 * x128
    x165 = 3.0 * x130
    x166 = x0 * (3.0 * x129 + x165) + x111 * x153
    x167 = x3**2 * x8
    x168 = x167 + x29
    x169 = x0 * (x117 + x168)
    x170 = x144 * x3
    x171 = x0 * (3.0 * x118 + 3.0 * x119 + 2.0 * x169 + 2.0 * x170) + x143 * x3
    x172 = x105 * x171
    x173 = 3.0 * x137
    x174 = x0 * (3.0 * x136 + x173) + x116 * x162
    x175 = -x110 - A[1]
    x176 = x102 * x113
    x177 = x128 * x175
    x178 = x111 * x177
    x179 = x130 + x178
    x180 = x104 * x179
    x181 = 0.06666666666666667 * x180
    x182 = 0.06666666666666667 * x104
    x183 = x107 * x182
    x184 = x116 * x183
    x185 = x101 * x103
    x186 = x0 * (x159 + x177)
    x187 = x111 * x179
    x188 = x186 + x187
    x189 = 0.08606629658238704
    x190 = x104 * x189
    x191 = x188 * x190
    x192 = 2.23606797749979
    x193 = x181 * x192
    x194 = x139 * x189
    x195 = x165 + 2.0 * x178
    x196 = x0 * (x129 + x195)
    x197 = x111 * x188
    x198 = x196 + x197
    x199 = x126 * x182
    x200 = x150 * x192
    x201 = x182 * x200
    x202 = x162 * x182
    x203 = 3.0 * x186
    x204 = x0 * (x153 + 3.0 * x187 + x203) + x111 * x198
    x205 = x112 * x143
    x206 = 0.06666666666666667 * x143
    x207 = x104 * x206
    x208 = x143 * x189
    x209 = x112 * x174
    x210 = x105 * x143
    x211 = -x115 - A[2]
    x212 = x183 * x211
    x213 = x126 * x211
    x214 = x116 * x213
    x215 = x137 + x214
    x216 = x104 * x215
    x217 = 0.06666666666666667 * x216
    x218 = x132 * x189
    x219 = x192 * x217
    x220 = x0 * (x156 + x213)
    x221 = x116 * x215
    x222 = x220 + x221
    x223 = x190 * x222
    x224 = x153 * x182
    x225 = x173 + 2.0 * x214
    x226 = x0 * (x136 + x225)
    x227 = x116 * x222
    x228 = x226 + x227
    x229 = x182 * x228
    x230 = x112 * x166
    x231 = 3.0 * x220
    x232 = x0 * (x162 + 3.0 * x221 + x231) + x116 * x228
    x233 = x94 + x98
    x234 = x128 * x175**2
    x235 = x130 + x234
    x236 = x104 * x235
    x237 = x127 * x236
    x238 = x175 * x179
    x239 = x186 + x238
    x240 = x126 * x190
    x241 = x189 * x236
    x242 = x175 * x188
    x243 = x196 + x242
    x244 = 0.1111111111111111 * x104
    x245 = x243 * x244
    x246 = 1.732050807568877
    x247 = x239 * x246
    x248 = x244 * x247
    x249 = x138 * x244
    x250 = 2.0 * x0 * (2.0 * x186 + x187 + x238)
    x251 = x111 * x243
    x252 = x250 + x251
    x253 = x122 * x246
    x254 = x162 * x189
    x255 = 3.0 * x242
    x256 = x0 * (5.0 * x196 + 2.0 * x197 + x255)
    x257 = x111 * x252
    x258 = x256 + x257
    x259 = x104 * x120
    x260 = x127 * x259
    x261 = x189 * x259
    x262 = x188 * x246
    x263 = x244 * x262
    x264 = 0.3333333333333333 * x180
    x265 = x222 * x246
    x266 = x244 * x265
    x267 = x192 * x198
    x268 = x182 * x267
    x269 = 0.3333333333333333 * x216
    x270 = x192 * x229
    x271 = x105 * x120
    x272 = x134 * x213
    x273 = x120 * x192
    x274 = x134 * x232
    x275 = x126 * x211**2
    x276 = x137 + x275
    x277 = x104 * x276
    x278 = x127 * x277
    x279 = x189 * x277
    x280 = x211 * x215
    x281 = x220 + x280
    x282 = x128 * x190
    x283 = x131 * x244
    x284 = x246 * x281
    x285 = x244 * x284
    x286 = x211 * x222
    x287 = x226 + x286
    x288 = x244 * x287
    x289 = x153 * x189
    x290 = 2.0 * x0 * (2.0 * x220 + x221 + x280)
    x291 = x116 * x287
    x292 = x290 + x291
    x293 = x127 * x166
    x294 = 3.0 * x286
    x295 = x0 * (5.0 * x226 + 2.0 * x227 + x294)
    x296 = x116 * x292
    x297 = x295 + x296
    x298 = 2.0 * x0 * (x14 + 2.0 * x23 + x73) + x4 * x87
    x299 = x88 + x96
    x300 = x0 * (x141 + x298 + x61 + x68 + 3.0 * x72) + x299 * x4
    x301 = x175 * (x152 + x235)
    x302 = x112 * x301
    x303 = x0 * (x195 + x234)
    x304 = x175 * x239
    x305 = x303 + x304
    x306 = 0.06666666666666667 * x301
    x307 = x104 * x306
    x308 = x175 * x243
    x309 = x250 + x308
    x310 = x192 * x305
    x311 = x182 * x310
    x312 = x175 * x252
    x313 = x256 + x312
    x314 = x104 * x70
    x315 = 0.06666666666666667 * x314
    x316 = x192 * x315
    x317 = 3.0 * x0 * (2.0 * x250 + x251 + x308)
    x318 = x111 * x313 + x317
    x319 = x105 * x77
    x320 = x112 * x319
    x321 = x182 * x77
    x322 = x105 * x300
    x323 = x182 * x192
    x324 = x299 * x323
    x325 = x192 * x235
    x326 = x245 * x246
    x327 = 0.3333333333333333 * x314
    x328 = x134 * x319
    x329 = x192 * x77
    x330 = x192 * x321
    x331 = x134 * x276
    x332 = x192 * x276
    x333 = x244 * x276
    x334 = x246 * x288
    x335 = x211 * (x161 + x276)
    x336 = x112 * x335
    x337 = x182 * x335
    x338 = x0 * (x225 + x275)
    x339 = x211 * x281
    x340 = x338 + x339
    x341 = x182 * x340
    x342 = x192 * x341
    x343 = x211 * x287
    x344 = x290 + x343
    x345 = x190 * x344
    x346 = 0.06666666666666667 * x335
    x347 = x192 * x340
    x348 = x211 * x292
    x349 = x295 + x348
    x350 = 3.0 * x0 * (2.0 * x290 + x291 + x343)
    x351 = x116 * x349 + x350
    x352 = x0 * (x165 + 3.0 * x234) + x175 * x301
    x353 = x0 * (x43 + x81)
    x354 = x14 + x23
    x355 = x354 * x4
    x356 = x0 * (2.0 * x353 + 2.0 * x355 + 3.0 * x84 + 3.0 * x86) + x298 * x4
    x357 = x105 * x356
    x358 = x0 * (x203 + 3.0 * x238 + x301) + x175 * x305
    x359 = x112 * x298
    x360 = x112 * x352
    x361 = x105 * x156
    x362 = x126 * x127
    x363 = x0 * (3.0 * x196 + x255 + 2.0 * x303 + 2.0 * x304) + x175 * x309
    x364 = x104 * x363
    x365 = x134 * x358
    x366 = x175 * x313 + x317
    x367 = x105 * x85
    x368 = x112 * x367
    x369 = x134 * x367
    x370 = x0 * (7.0 * x256 + 3.0 * x257 + 4.0 * x312) + x175 * x318
    x371 = x82 + x9
    x372 = x105 * x371
    x373 = x112 * x372
    x374 = x182 * x213
    x375 = x190 * x309
    x376 = x192 * x85
    x377 = 0.06666666666666667 * x371
    x378 = x182 * x371
    x379 = x189 * x298
    x380 = x244 * x87
    x381 = x189 * x85
    x382 = x127 * x371
    x383 = x190 * x371
    x384 = x190 * x335
    x385 = x177 * x182
    x386 = x0 * (x173 + 3.0 * x275) + x211 * x335
    x387 = x112 * x386
    x388 = x105 * x159
    x389 = x0 * (x231 + 3.0 * x280 + x335) + x211 * x340
    x390 = x127 * x386
    x391 = x134 * x389
    x392 = x104 * x128
    x393 = x0 * (3.0 * x226 + x294 + 2.0 * x338 + 2.0 * x339) + x211 * x344
    x394 = x127 * x393
    x395 = x211 * x349 + x350
    x396 = x0 * (7.0 * x295 + 3.0 * x296 + 4.0 * x348) + x211 * x351
    x397 = -x110 - R[1]
    x398 = x4 * x64
    x399 = x3 * x46
    x400 = 3.0 * x0 * (x399 + 2.0 * x54 + x55)
    x401 = x4 * x58 + x400
    x402 = x59 + x67
    x403 = x109 * (x0 * (3.0 * x398 + x401 + 7.0 * x63 + 4.0 * x65) + x3 * x402)
    x404 = x3 * x66 + x59
    x405 = x159 * x397
    x406 = x130 + x405
    x407 = x105 * x406
    x408 = x112 * x126
    x409 = x113 * x397
    x410 = x399 + x54
    x411 = x37 + x49
    x412 = x0 * (x149 + 3.0 * x28 + x410 + x62) + x3 * x411
    x413 = x128 * x397
    x414 = x0 * (x159 + x413)
    x415 = x111 * x406
    x416 = x414 + x415
    x417 = x104 * x416
    x418 = x134 * x407
    x419 = x165 + 2.0 * x405
    x420 = x0 * (x129 + x419)
    x421 = x111 * x416
    x422 = x420 + x421
    x423 = 3.0 * x31
    x424 = x169 + x170
    x425 = x0 * (x168 + x41)
    x426 = x145 * x3
    x427 = x425 + x426
    x428 = x146 + x148
    x429 = x0 * (3.0 * x39 + x423 + x424 + x427) + x3 * x428
    x430 = x112 * x155
    x431 = x134 * x361
    x432 = x105 * x413
    x433 = x167 + x9
    x434 = 2.0 * x0
    x435 = x16 * x434 + x3 * x433
    x436 = x0 * (3.0 * x142 + x435 + 3.0 * x74) + x3 * x424
    x437 = 3.0 * x414
    x438 = x0 * (x153 + 3.0 * x415 + x437) + x111 * x422
    x439 = x112 * x361
    x440 = x105 * x174
    x441 = x177 * x397
    x442 = x130 + x441
    x443 = x105 * x442
    x444 = x175 * x406
    x445 = x414 + x444
    x446 = x182 * x445
    x447 = x182 * x442
    x448 = x0 * (x165 + x178 + x405 + x441)
    x449 = x111 * x445
    x450 = x448 + x449
    x451 = x192 * x446
    x452 = 2.0 * x444
    x453 = x437 + x452
    x454 = x0 * (x188 + x415 + x453)
    x455 = x111 * x450
    x456 = x454 + x455
    x457 = x192 * x428
    x458 = x182 * x457
    x459 = 3.0 * x448
    x460 = x0 * (x198 + x422 + 3.0 * x449 + x459) + x111 * x456
    x461 = x182 * x424
    x462 = x182 * x406
    x463 = x190 * x416
    x464 = x112 * x424
    x465 = x105 * x213
    x466 = x398 + x63
    x467 = x0 * (x177 + x413)
    x468 = x175 * x442
    x469 = x467 + x468
    x470 = x127 * x469
    x471 = x104 * x126
    x472 = x175 * x445
    x473 = x448 + x472
    x474 = x190 * x469
    x475 = x175 * x450
    x476 = x454 + x475
    x477 = x244 * x36
    x478 = x246 * x473
    x479 = 2.0 * x472
    x480 = 4.0 * x448
    x481 = 2.0 * x449 + x480
    x482 = x0 * (x243 + x479 + x481)
    x483 = x111 * x476
    x484 = x482 + x483
    x485 = x147 * x246
    x486 = x244 * x476
    x487 = x104 * x254
    x488 = 2.0 * x455
    x489 = 3.0 * x475
    x490 = x0 * (x252 + 5.0 * x454 + x488 + x489)
    x491 = x111 * x484
    x492 = x490 + x491
    x493 = x127 * x144
    x494 = x144 * x190
    x495 = x104 * x174
    x496 = x134 * x443
    x497 = x246 * x450
    x498 = x213 * x323
    x499 = 0.3333333333333333 * x104
    x500 = x222 * x499
    x501 = x105 * x272
    x502 = x190 * x413
    x503 = x244 * x416
    x504 = x104 * x297
    x505 = x165 + 2.0 * x441
    x506 = x0 * (x234 + x505)
    x507 = x175 * x469
    x508 = x506 + x507
    x509 = x353 + x355
    x510 = x44 + x50
    x511 = x28 + x60
    x512 = x0 * (3.0 * x33 + x423 + x509 + x510) + x4 * x511
    x513 = x0 * (x239 + 2.0 * x414 + x452 + x469)
    x514 = x175 * x473
    x515 = x513 + x514
    x516 = x182 * x508
    x517 = x175 * x476
    x518 = x482 + x517
    x519 = x192 * x515
    x520 = x175 * x484
    x521 = x490 + x520
    x522 = x182 * x32
    x523 = x192 * x522
    x524 = x0 * (x313 + 6.0 * x482 + 3.0 * x483 + 3.0 * x517)
    x525 = x111 * x521
    x526 = x524 + x525
    x527 = x105 * x75
    x528 = x182 * x75
    x529 = x246 * x34
    x530 = x134 * x527
    x531 = x192 * x528
    x532 = x192 * x447
    x533 = x281 * x499
    x534 = x32 * x499
    x535 = x192 * x32
    x536 = x112 * x351
    x537 = x0 * (x301 + 3.0 * x467 + 3.0 * x468) + x175 * x508
    x538 = x42 + x9
    x539 = x11 * x434 + x4 * x538
    x540 = x0 * (3.0 * x14 + x24 + x539) + x4 * x509
    x541 = x0 * (x305 + x459 + 3.0 * x472 + x508) + x175 * x515
    x542 = 2.0 * x513 + 2.0 * x514
    x543 = x0 * (x309 + 3.0 * x454 + x489 + x542) + x175 * x518
    x544 = x104 * x354
    x545 = x175 * x521 + x524
    x546 = x105 * x13
    x547 = x134 * x546
    x548 = x108 * x6
    x549 = x548 * (x0 * (x318 + 7.0 * x490 + 3.0 * x491 + 4.0 * x520) + x175 * x526)
    x550 = x10 * x548
    x551 = x112 * x116
    x552 = x105 * x22
    x553 = x112 * x508
    x554 = x190 * x354
    x555 = x13 * x182
    x556 = x112 * x526
    x557 = x244 * x354
    x558 = x190 * x22
    x559 = x192 * x555
    x560 = x105 * x386
    x561 = x112 * x389
    x562 = x112 * x395
    x563 = 3.141592653589793 * x1 * x103
    x564 = x105 * x563
    x565 = x564 * x6
    x566 = x10 * x565
    x567 = -x115 - R[2]
    x568 = x113 * x567
    x569 = x156 * x567
    x570 = x137 + x569
    x571 = x105 * x570
    x572 = x112 * x128
    x573 = x126 * x567
    x574 = x134 * x571
    x575 = x0 * (x156 + x573)
    x576 = x116 * x570
    x577 = x575 + x576
    x578 = x127 * x577
    x579 = x105 * x573
    x580 = x134 * x388
    x581 = x173 + 2.0 * x569
    x582 = x0 * (x136 + x581)
    x583 = x116 * x577
    x584 = x582 + x583
    x585 = x112 * x164
    x586 = x112 * x584
    x587 = 3.0 * x575
    x588 = x0 * (x162 + 3.0 * x576 + x587) + x116 * x584
    x589 = x182 * x570
    x590 = x190 * x577
    x591 = x182 * x573
    x592 = x105 * x177
    x593 = x213 * x567
    x594 = x137 + x593
    x595 = x105 * x594
    x596 = x182 * x594
    x597 = x211 * x570
    x598 = x575 + x597
    x599 = x182 * x598
    x600 = x192 * x599
    x601 = x0 * (x173 + x214 + x569 + x593)
    x602 = x116 * x598
    x603 = x601 + x602
    x604 = 2.0 * x597
    x605 = x587 + x604
    x606 = x0 * (x222 + x576 + x605)
    x607 = x116 * x603
    x608 = x606 + x607
    x609 = x128 * x182
    x610 = 3.0 * x601
    x611 = x0 * (x228 + x584 + 3.0 * x602 + x610) + x116 * x608
    x612 = x190 * x573
    x613 = x244 * x577
    x614 = x104 * x573
    x615 = x134 * x595
    x616 = x246 * x477
    x617 = x499 * x598
    x618 = x177 * x323
    x619 = x244 * x603
    x620 = x134 * x592
    x621 = x0 * (x213 + x573)
    x622 = x211 * x594
    x623 = x621 + x622
    x624 = x104 * x623
    x625 = x127 * x128
    x626 = x190 * x623
    x627 = x211 * x598
    x628 = x601 + x627
    x629 = x211 * x603
    x630 = x606 + x629
    x631 = x244 * x630
    x632 = 2.0 * x627
    x633 = 4.0 * x601
    x634 = 2.0 * x602 + x633
    x635 = x0 * (x287 + x632 + x634)
    x636 = x116 * x630
    x637 = x635 + x636
    x638 = x104 * x289
    x639 = 2.0 * x607
    x640 = 3.0 * x629
    x641 = x0 * (x292 + 5.0 * x606 + x639 + x640)
    x642 = x116 * x637
    x643 = x641 + x642
    x644 = x112 * x573
    x645 = x192 * x596
    x646 = x173 + 2.0 * x593
    x647 = x0 * (x275 + x646)
    x648 = x211 * x623
    x649 = x647 + x648
    x650 = x182 * x649
    x651 = x0 * (x281 + 2.0 * x575 + x604 + x623)
    x652 = x211 * x628
    x653 = x651 + x652
    x654 = x323 * x653
    x655 = x211 * x630
    x656 = x635 + x655
    x657 = x211 * x637
    x658 = x641 + x657
    x659 = x0 * (x349 + 6.0 * x635 + 3.0 * x636 + 3.0 * x655)
    x660 = x116 * x658
    x661 = x659 + x660
    x662 = x112 * x509
    x663 = x112 * x22
    x664 = x182 * x22
    x665 = x127 * x624
    x666 = x112 * x649
    x667 = x112 * x661
    x668 = x0 * (x335 + 3.0 * x621 + 3.0 * x622) + x211 * x649
    x669 = x0 * (x340 + x610 + 3.0 * x627 + x649) + x211 * x653
    x670 = x127 * x668
    x671 = 2.0 * x651 + 2.0 * x652
    x672 = x0 * (x344 + 3.0 * x606 + x640 + x671) + x211 * x656
    x673 = x211 * x658 + x659
    x674 = x111 * x112
    x675 = x565 * (x0 * (x351 + 7.0 * x641 + 3.0 * x642 + 4.0 * x657) + x211 * x661)
    x676 = x4 * x56
    x677 = x0 * (7.0 * x52 + 4.0 * x57 + 3.0 * x676) + x3 * x401
    x678 = x128 * x397**2
    x679 = x130 + x678
    x680 = x105 * x679
    x681 = x3 * x58 + x400
    x682 = x397 * x406
    x683 = x414 + x682
    x684 = x105 * x683
    x685 = x112 * x680
    x686 = x0 * (x419 + x678)
    x687 = x111 * x683
    x688 = x686 + x687
    x689 = x0 * (2.0 * x425 + 2.0 * x426 + 3.0 * x44 + x51) + x3 * x410
    x690 = x104 * x362
    x691 = x134 * x684
    x692 = 2.0 * x0 * (2.0 * x414 + x415 + x682) + x111 * x688
    x693 = 3.0 * x17
    x694 = x0 * (x435 + 3.0 * x53 + x693) + x3 * x427
    x695 = x0 * (3.0 * x167 + x29) + x3 * x435
    x696 = x0 * (2.0 * x420 + 2.0 * x421 + 3.0 * x686 + 3.0 * x687) + x111 * x692
    x697 = x112 * x695
    x698 = x397 * x442
    x699 = x467 + x698
    x700 = x105 * x699
    x701 = x397 * x445
    x702 = x448 + x701
    x703 = x182 * x702
    x704 = x182 * x699
    x705 = x0 * (x453 + x682 + x699)
    x706 = x111 * x702
    x707 = x705 + x706
    x708 = x192 * x703
    x709 = 2.0 * x701
    x710 = x0 * (x481 + x688 + x709)
    x711 = x111 * x707
    x712 = x710 + x711
    x713 = x323 * x427
    x714 = x138 * x192
    x715 = 2.0 * x454
    x716 = 3.0 * x705
    x717 = x0 * (x488 + x692 + 3.0 * x706 + x715 + x716) + x111 * x712
    x718 = x112 * x435
    x719 = x182 * x435
    x720 = x190 * x410
    x721 = x0 * (x505 + x678)
    x722 = x175 * x699
    x723 = x721 + x722
    x724 = x52 + x676
    x725 = x175 * x702
    x726 = x705 + x725
    x727 = x190 * x723
    x728 = x175 * x707
    x729 = x710 + x728
    x730 = x244 * x46
    x731 = x246 * x730
    x732 = x0 * (2.0 * x475 + 4.0 * x705 + 2.0 * x706 + x715 + 2.0 * x725)
    x733 = x111 * x729
    x734 = x732 + x733
    x735 = x145 * x246
    x736 = x244 * x735
    x737 = 2.0 * x482 + 3.0 * x728
    x738 = x0 * (2.0 * x483 + 5.0 * x710 + 2.0 * x711 + x737)
    x739 = x111 * x734
    x740 = x738 + x739
    x741 = x127 * x433
    x742 = x104 * x741
    x743 = x190 * x433
    x744 = x190 * x679
    x745 = x104 * x679
    x746 = 2.0 * x0 * (2.0 * x467 + x468 + x698) + x175 * x723
    x747 = x0 * (3.0 * x20 + x539 + x693) + x4 * x510
    x748 = x0 * (x479 + x480 + x709 + x723)
    x749 = x175 * x726
    x750 = x748 + x749
    x751 = x182 * x510
    x752 = x175 * x729
    x753 = x732 + x752
    x754 = x190 * x21
    x755 = x323 * x750
    x756 = x175 * x734
    x757 = x738 + x756
    x758 = x182 * x19
    x759 = x192 * x758
    x760 = x0 * (2.0 * x490 + 2.0 * x520 + 6.0 * x732 + 3.0 * x733 + 3.0 * x752)
    x761 = x111 * x757 + x760
    x762 = x112 * x761
    x763 = x108 * x15
    x764 = x105 * x16
    x765 = x323 * x510
    x766 = x21 * x246
    x767 = x244 * x766
    x768 = x19 * x499
    x769 = x16 * x323
    x770 = x16 * x182
    x771 = x0 * (x29 + 3.0 * x42) + x4 * x539
    x772 = x0 * (2.0 * x506 + 2.0 * x507 + 3.0 * x721 + 3.0 * x722) + x175 * x746
    x773 = x0 * (x542 + x716 + 3.0 * x725 + x746) + x175 * x750
    x774 = x112 * x539
    x775 = x0 * (2.0 * x517 + 3.0 * x710 + x737 + 2.0 * x748 + 2.0 * x749) + x175 * x753
    x776 = x104 * x538
    x777 = x127 * x776
    x778 = x112 * x4
    x779 = x548 * (x175 * x757 + x760)
    x780 = x134 * x4
    x781 = x105 * x11
    x782 = x105 * x8
    x783 = x112 * x771
    x784 = x182 * x539
    x785 = x190 * x538
    x786 = x4 * x6
    x787 = x112 * x782
    x788 = x190 * x539
    x789 = x244 * x538
    x790 = x190 * x8
    x791 = x104 * x8
    x792 = x112 * x8
    x793 = x127 * x538
    x794 = x112 * x413
    x795 = x127 * x689
    x796 = x112 * x422
    x797 = x192 * x462
    x798 = x445 * x499
    x799 = x134 * x764
    x800 = x15 * x564
    x801 = x134 * x781
    x802 = x11 * x182
    x803 = x182 * x8
    x804 = x127 * x8
    x805 = x126 * x567**2
    x806 = x137 + x805
    x807 = x105 * x806
    x808 = x112 * x807
    x809 = x567 * x570
    x810 = x575 + x809
    x811 = x105 * x810
    x812 = x134 * x811
    x813 = x0 * (x581 + x805)
    x814 = x116 * x810
    x815 = x813 + x814
    x816 = 2.0 * x0 * (2.0 * x575 + x576 + x809) + x116 * x815
    x817 = x0 * (2.0 * x582 + 2.0 * x583 + 3.0 * x813 + 3.0 * x814) + x116 * x816
    x818 = x182 * x806
    x819 = x567 * x594
    x820 = x621 + x819
    x821 = x105 * x820
    x822 = x182 * x820
    x823 = x567 * x598
    x824 = x601 + x823
    x825 = x159 * x323
    x826 = x0 * (x605 + x809 + x820)
    x827 = x116 * x824
    x828 = x826 + x827
    x829 = 2.0 * x823
    x830 = x0 * (x634 + x815 + x829)
    x831 = x116 * x828
    x832 = x830 + x831
    x833 = 2.0 * x606
    x834 = 3.0 * x826
    x835 = x0 * (x639 + x816 + 3.0 * x827 + x833 + x834) + x116 * x832
    x836 = x190 * x806
    x837 = x134 * x821
    x838 = x499 * x824
    x839 = x0 * (x646 + x805)
    x840 = x211 * x820
    x841 = x839 + x840
    x842 = x104 * x841
    x843 = x190 * x841
    x844 = x211 * x824
    x845 = x826 + x844
    x846 = x211 * x828
    x847 = x830 + x846
    x848 = x0 * (2.0 * x629 + 4.0 * x826 + 2.0 * x827 + x833 + 2.0 * x844)
    x849 = x116 * x847
    x850 = x848 + x849
    x851 = 2.0 * x635 + 3.0 * x846
    x852 = x0 * (2.0 * x636 + 5.0 * x830 + 2.0 * x831 + x851)
    x853 = x116 * x850
    x854 = x852 + x853
    x855 = 2.0 * x0 * (2.0 * x621 + x622 + x819) + x211 * x841
    x856 = x0 * (x632 + x633 + x829 + x841)
    x857 = x211 * x845
    x858 = x856 + x857
    x859 = x211 * x847
    x860 = x848 + x859
    x861 = x211 * x850
    x862 = x852 + x861
    x863 = x182 * x563 * x862
    x864 = x0 * (2.0 * x641 + 2.0 * x657 + 6.0 * x848 + 3.0 * x849 + 3.0 * x859)
    x865 = x116 * x862 + x864
    x866 = x112 * x865
    x867 = x11 * x323
    x868 = x0 * (2.0 * x647 + 2.0 * x648 + 3.0 * x839 + 3.0 * x840) + x211 * x855
    x869 = x0 * (x671 + x834 + 3.0 * x844 + x855) + x211 * x858
    x870 = x0 * (2.0 * x655 + 3.0 * x830 + x851 + 2.0 * x856 + 2.0 * x857) + x211 * x860
    x871 = x565 * (x211 * x862 + x864)

    # 1350 item(s)
    result[0, 0, 0] = numpy.sum(
        x109
        * (x0 * (2.0 * x59 + 2.0 * x67 + 3.0 * x94 + 4.0 * x95 + 7.0 * x98) + x102 * x3)
    )
    result[0, 0, 1] = numpy.sum(x111 * x114)
    result[0, 0, 2] = numpy.sum(x114 * x116)
    result[0, 0, 3] = numpy.sum(x125 * x126 * x133)
    result[0, 0, 4] = numpy.sum(x111 * x116 * x125 * x135)
    result[0, 0, 5] = numpy.sum(x125 * x128 * x140)
    result[0, 0, 6] = numpy.sum(x151 * x154 * x155)
    result[0, 0, 7] = numpy.sum(x156 * x157 * x158)
    result[0, 0, 8] = numpy.sum(x158 * x159 * x160)
    result[0, 0, 9] = numpy.sum(x151 * x163 * x164)
    result[0, 0, 10] = numpy.sum(x126 * x166 * x172)
    result[0, 0, 11] = numpy.sum(x154 * x156 * x172)
    result[0, 0, 12] = numpy.sum(x131 * x140 * x171)
    result[0, 0, 13] = numpy.sum(x159 * x163 * x172)
    result[0, 0, 14] = numpy.sum(x128 * x172 * x174)
    result[0, 1, 0] = numpy.sum(x175 * x176)
    result[0, 1, 1] = numpy.sum(x101 * x126 * x181)
    result[0, 1, 2] = numpy.sum(x175 * x184 * x185)
    result[0, 1, 3] = numpy.sum(x124 * x126 * x191)
    result[0, 1, 4] = numpy.sum(x124 * x156 * x193)
    result[0, 1, 5] = numpy.sum(x124 * x177 * x194)
    result[0, 1, 6] = numpy.sum(x150 * x198 * x199)
    result[0, 1, 7] = numpy.sum(x156 * x188 * x201)
    result[0, 1, 8] = numpy.sum(x138 * x181 * x200)
    result[0, 1, 9] = numpy.sum(x150 * x177 * x202)
    result[0, 1, 10] = numpy.sum(x155 * x204 * x205)
    result[0, 1, 11] = numpy.sum(x156 * x198 * x207)
    result[0, 1, 12] = numpy.sum(x139 * x188 * x208)
    result[0, 1, 13] = numpy.sum(x162 * x180 * x206)
    result[0, 1, 14] = numpy.sum(x177 * x209 * x210)
    result[0, 2, 0] = numpy.sum(x176 * x211)
    result[0, 2, 1] = numpy.sum(x111 * x185 * x212)
    result[0, 2, 2] = numpy.sum(x101 * x128 * x217)
    result[0, 2, 3] = numpy.sum(x124 * x213 * x218)
    result[0, 2, 4] = numpy.sum(x124 * x159 * x219)
    result[0, 2, 5] = numpy.sum(x124 * x128 * x223)
    result[0, 2, 6] = numpy.sum(x150 * x213 * x224)
    result[0, 2, 7] = numpy.sum(x131 * x200 * x217)
    result[0, 2, 8] = numpy.sum(x159 * x201 * x222)
    result[0, 2, 9] = numpy.sum(x128 * x150 * x229)
    result[0, 2, 10] = numpy.sum(x210 * x213 * x230)
    result[0, 2, 11] = numpy.sum(x153 * x206 * x216)
    result[0, 2, 12] = numpy.sum(x132 * x208 * x222)
    result[0, 2, 13] = numpy.sum(x159 * x207 * x228)
    result[0, 2, 14] = numpy.sum(x164 * x205 * x232)
    result[0, 3, 0] = numpy.sum(x126 * x233 * x237)
    result[0, 3, 1] = numpy.sum(x239 * x240 * x93)
    result[0, 3, 2] = numpy.sum(x156 * x241 * x93)
    result[0, 3, 3] = numpy.sum(x126 * x245 * x91)
    result[0, 3, 4] = numpy.sum(x156 * x248 * x91)
    result[0, 3, 5] = numpy.sum(x235 * x249 * x91)
    result[0, 3, 6] = numpy.sum(x122 * x240 * x252)
    result[0, 3, 7] = numpy.sum(x156 * x245 * x253)
    result[0, 3, 8] = numpy.sum(x239 * x249 * x253)
    result[0, 3, 9] = numpy.sum(x122 * x236 * x254)
    result[0, 3, 10] = numpy.sum(x126 * x258 * x260)
    result[0, 3, 11] = numpy.sum(x156 * x252 * x261)
    result[0, 3, 12] = numpy.sum(x120 * x243 * x249)
    result[0, 3, 13] = numpy.sum(x239 * x254 * x259)
    result[0, 3, 14] = numpy.sum(x120 * x174 * x237)
    result[0, 4, 0] = numpy.sum(x135 * x175 * x211 * x233)
    result[0, 4, 1] = numpy.sum(x193 * x213 * x93)
    result[0, 4, 2] = numpy.sum(x177 * x219 * x93)
    result[0, 4, 3] = numpy.sum(x213 * x263 * x91)
    result[0, 4, 4] = numpy.sum(x215 * x264 * x91)
    result[0, 4, 5] = numpy.sum(x177 * x266 * x91)
    result[0, 4, 6] = numpy.sum(x122 * x213 * x268)
    result[0, 4, 7] = numpy.sum(x122 * x188 * x269)
    result[0, 4, 8] = numpy.sum(x122 * x222 * x264)
    result[0, 4, 9] = numpy.sum(x122 * x177 * x270)
    result[0, 4, 10] = numpy.sum(x204 * x271 * x272)
    result[0, 4, 11] = numpy.sum(x198 * x217 * x273)
    result[0, 4, 12] = numpy.sum(x120 * x188 * x266)
    result[0, 4, 13] = numpy.sum(x181 * x228 * x273)
    result[0, 4, 14] = numpy.sum(x177 * x271 * x274)
    result[0, 5, 0] = numpy.sum(x128 * x233 * x278)
    result[0, 5, 1] = numpy.sum(x159 * x279 * x93)
    result[0, 5, 2] = numpy.sum(x281 * x282 * x93)
    result[0, 5, 3] = numpy.sum(x276 * x283 * x91)
    result[0, 5, 4] = numpy.sum(x159 * x285 * x91)
    result[0, 5, 5] = numpy.sum(x128 * x288 * x91)
    result[0, 5, 6] = numpy.sum(x122 * x277 * x289)
    result[0, 5, 7] = numpy.sum(x253 * x281 * x283)
    result[0, 5, 8] = numpy.sum(x159 * x253 * x288)
    result[0, 5, 9] = numpy.sum(x122 * x282 * x292)
    result[0, 5, 10] = numpy.sum(x259 * x276 * x293)
    result[0, 5, 11] = numpy.sum(x259 * x281 * x289)
    result[0, 5, 12] = numpy.sum(x120 * x283 * x287)
    result[0, 5, 13] = numpy.sum(x159 * x261 * x292)
    result[0, 5, 14] = numpy.sum(x128 * x260 * x297)
    result[0, 6, 0] = numpy.sum(x155 * x300 * x302)
    result[0, 6, 1] = numpy.sum(x199 * x299 * x305)
    result[0, 6, 2] = numpy.sum(x156 * x299 * x307)
    result[0, 6, 3] = numpy.sum(x240 * x309 * x89)
    result[0, 6, 4] = numpy.sum(x156 * x311 * x89)
    result[0, 6, 5] = numpy.sum(x194 * x301 * x89)
    result[0, 6, 6] = numpy.sum(x126 * x313 * x315)
    result[0, 6, 7] = numpy.sum(x156 * x309 * x316)
    result[0, 6, 8] = numpy.sum(x138 * x310 * x315)
    result[0, 6, 9] = numpy.sum(x162 * x306 * x314)
    result[0, 6, 10] = numpy.sum(x126 * x318 * x320)
    result[0, 6, 11] = numpy.sum(x156 * x313 * x321)
    result[0, 6, 12] = numpy.sum(x194 * x309 * x77)
    result[0, 6, 13] = numpy.sum(x162 * x305 * x321)
    result[0, 6, 14] = numpy.sum(x174 * x302 * x319)
    result[0, 7, 0] = numpy.sum(x235 * x272 * x322)
    result[0, 7, 1] = numpy.sum(x213 * x239 * x324)
    result[0, 7, 2] = numpy.sum(x217 * x299 * x325)
    result[0, 7, 3] = numpy.sum(x213 * x326 * x89)
    result[0, 7, 4] = numpy.sum(x239 * x269 * x89)
    result[0, 7, 5] = numpy.sum(x235 * x266 * x89)
    result[0, 7, 6] = numpy.sum(x213 * x252 * x316)
    result[0, 7, 7] = numpy.sum(x243 * x269 * x70)
    result[0, 7, 8] = numpy.sum(x222 * x239 * x327)
    result[0, 7, 9] = numpy.sum(x228 * x315 * x325)
    result[0, 7, 10] = numpy.sum(x213 * x258 * x328)
    result[0, 7, 11] = numpy.sum(x217 * x252 * x329)
    result[0, 7, 12] = numpy.sum(x245 * x265 * x77)
    result[0, 7, 13] = numpy.sum(x228 * x239 * x330)
    result[0, 7, 14] = numpy.sum(x232 * x235 * x328)
    result[0, 8, 0] = numpy.sum(x177 * x322 * x331)
    result[0, 8, 1] = numpy.sum(x181 * x299 * x332)
    result[0, 8, 2] = numpy.sum(x177 * x281 * x324)
    result[0, 8, 3] = numpy.sum(x262 * x333 * x89)
    result[0, 8, 4] = numpy.sum(x264 * x281 * x89)
    result[0, 8, 5] = numpy.sum(x177 * x334 * x89)
    result[0, 8, 6] = numpy.sum(x198 * x315 * x332)
    result[0, 8, 7] = numpy.sum(x188 * x281 * x327)
    result[0, 8, 8] = numpy.sum(x264 * x287 * x70)
    result[0, 8, 9] = numpy.sum(x177 * x292 * x316)
    result[0, 8, 10] = numpy.sum(x204 * x276 * x328)
    result[0, 8, 11] = numpy.sum(x198 * x281 * x330)
    result[0, 8, 12] = numpy.sum(x262 * x288 * x77)
    result[0, 8, 13] = numpy.sum(x181 * x292 * x329)
    result[0, 8, 14] = numpy.sum(x177 * x297 * x328)
    result[0, 9, 0] = numpy.sum(x164 * x300 * x336)
    result[0, 9, 1] = numpy.sum(x159 * x299 * x337)
    result[0, 9, 2] = numpy.sum(x128 * x299 * x341)
    result[0, 9, 3] = numpy.sum(x218 * x335 * x89)
    result[0, 9, 4] = numpy.sum(x159 * x342 * x89)
    result[0, 9, 5] = numpy.sum(x128 * x345 * x89)
    result[0, 9, 6] = numpy.sum(x153 * x314 * x346)
    result[0, 9, 7] = numpy.sum(x131 * x315 * x347)
    result[0, 9, 8] = numpy.sum(x159 * x316 * x344)
    result[0, 9, 9] = numpy.sum(x128 * x315 * x349)
    result[0, 9, 10] = numpy.sum(x166 * x319 * x336)
    result[0, 9, 11] = numpy.sum(x153 * x321 * x340)
    result[0, 9, 12] = numpy.sum(x218 * x344 * x77)
    result[0, 9, 13] = numpy.sum(x159 * x321 * x349)
    result[0, 9, 14] = numpy.sum(x128 * x320 * x351)
    result[0, 10, 0] = numpy.sum(x126 * x352 * x357)
    result[0, 10, 1] = numpy.sum(x155 * x358 * x359)
    result[0, 10, 2] = numpy.sum(x298 * x360 * x361)
    result[0, 10, 3] = numpy.sum(x362 * x364 * x87)
    result[0, 10, 4] = numpy.sum(x361 * x365 * x87)
    result[0, 10, 5] = numpy.sum(x140 * x352 * x87)
    result[0, 10, 6] = numpy.sum(x126 * x366 * x368)
    result[0, 10, 7] = numpy.sum(x156 * x363 * x369)
    result[0, 10, 8] = numpy.sum(x138 * x358 * x369)
    result[0, 10, 9] = numpy.sum(x163 * x352 * x367)
    result[0, 10, 10] = numpy.sum(x126 * x370 * x372)
    result[0, 10, 11] = numpy.sum(x156 * x366 * x373)
    result[0, 10, 12] = numpy.sum(x140 * x363 * x371)
    result[0, 10, 13] = numpy.sum(x163 * x358 * x372)
    result[0, 10, 14] = numpy.sum(x174 * x352 * x372)
    result[0, 11, 0] = numpy.sum(x213 * x302 * x357)
    result[0, 11, 1] = numpy.sum(x298 * x305 * x374)
    result[0, 11, 2] = numpy.sum(x216 * x298 * x306)
    result[0, 11, 3] = numpy.sum(x213 * x375 * x87)
    result[0, 11, 4] = numpy.sum(x217 * x310 * x87)
    result[0, 11, 5] = numpy.sum(x223 * x301 * x87)
    result[0, 11, 6] = numpy.sum(x313 * x374 * x85)
    result[0, 11, 7] = numpy.sum(x217 * x309 * x376)
    result[0, 11, 8] = numpy.sum(x222 * x311 * x85)
    result[0, 11, 9] = numpy.sum(x228 * x307 * x85)
    result[0, 11, 10] = numpy.sum(x213 * x318 * x373)
    result[0, 11, 11] = numpy.sum(x216 * x313 * x377)
    result[0, 11, 12] = numpy.sum(x223 * x309 * x371)
    result[0, 11, 13] = numpy.sum(x228 * x305 * x378)
    result[0, 11, 14] = numpy.sum(x232 * x302 * x372)
    result[0, 12, 0] = numpy.sum(x237 * x276 * x356)
    result[0, 12, 1] = numpy.sum(x239 * x277 * x379)
    result[0, 12, 2] = numpy.sum(x236 * x281 * x379)
    result[0, 12, 3] = numpy.sum(x243 * x276 * x380)
    result[0, 12, 4] = numpy.sum(x239 * x284 * x380)
    result[0, 12, 5] = numpy.sum(x235 * x287 * x380)
    result[0, 12, 6] = numpy.sum(x252 * x277 * x381)
    result[0, 12, 7] = numpy.sum(x245 * x284 * x85)
    result[0, 12, 8] = numpy.sum(x239 * x334 * x85)
    result[0, 12, 9] = numpy.sum(x236 * x292 * x381)
    result[0, 12, 10] = numpy.sum(x258 * x277 * x382)
    result[0, 12, 11] = numpy.sum(x252 * x281 * x383)
    result[0, 12, 12] = numpy.sum(x243 * x288 * x371)
    result[0, 12, 13] = numpy.sum(x239 * x292 * x383)
    result[0, 12, 14] = numpy.sum(x237 * x297 * x371)
    result[0, 13, 0] = numpy.sum(x177 * x336 * x357)
    result[0, 13, 1] = numpy.sum(x180 * x298 * x346)
    result[0, 13, 2] = numpy.sum(x177 * x298 * x341)
    result[0, 13, 3] = numpy.sum(x188 * x384 * x87)
    result[0, 13, 4] = numpy.sum(x181 * x347 * x87)
    result[0, 13, 5] = numpy.sum(x177 * x345 * x87)
    result[0, 13, 6] = numpy.sum(x198 * x337 * x85)
    result[0, 13, 7] = numpy.sum(x188 * x341 * x376)
    result[0, 13, 8] = numpy.sum(x181 * x344 * x376)
    result[0, 13, 9] = numpy.sum(x349 * x385 * x85)
    result[0, 13, 10] = numpy.sum(x204 * x336 * x372)
    result[0, 13, 11] = numpy.sum(x198 * x340 * x378)
    result[0, 13, 12] = numpy.sum(x188 * x344 * x383)
    result[0, 13, 13] = numpy.sum(x180 * x349 * x377)
    result[0, 13, 14] = numpy.sum(x177 * x351 * x373)
    result[0, 14, 0] = numpy.sum(x128 * x357 * x386)
    result[0, 14, 1] = numpy.sum(x298 * x387 * x388)
    result[0, 14, 2] = numpy.sum(x164 * x359 * x389)
    result[0, 14, 3] = numpy.sum(x132 * x390 * x87)
    result[0, 14, 4] = numpy.sum(x388 * x391 * x87)
    result[0, 14, 5] = numpy.sum(x392 * x394 * x87)
    result[0, 14, 6] = numpy.sum(x154 * x367 * x386)
    result[0, 14, 7] = numpy.sum(x131 * x369 * x389)
    result[0, 14, 8] = numpy.sum(x159 * x369 * x393)
    result[0, 14, 9] = numpy.sum(x128 * x368 * x395)
    result[0, 14, 10] = numpy.sum(x166 * x372 * x386)
    result[0, 14, 11] = numpy.sum(x154 * x372 * x389)
    result[0, 14, 12] = numpy.sum(x132 * x382 * x393)
    result[0, 14, 13] = numpy.sum(x159 * x373 * x395)
    result[0, 14, 14] = numpy.sum(x128 * x372 * x396)
    result[1, 0, 0] = numpy.sum(x397 * x403)
    result[1, 0, 1] = numpy.sum(x404 * x407 * x408)
    result[1, 0, 2] = numpy.sum(x116 * x404 * x409)
    result[1, 0, 3] = numpy.sum(x362 * x412 * x417)
    result[1, 0, 4] = numpy.sum(x156 * x412 * x418)
    result[1, 0, 5] = numpy.sum(x140 * x412 * x413)
    result[1, 0, 6] = numpy.sum(x422 * x429 * x430)
    result[1, 0, 7] = numpy.sum(x416 * x429 * x431)
    result[1, 0, 8] = numpy.sum(x160 * x407 * x429)
    result[1, 0, 9] = numpy.sum(x163 * x429 * x432)
    result[1, 0, 10] = numpy.sum(x155 * x436 * x438)
    result[1, 0, 11] = numpy.sum(x422 * x436 * x439)
    result[1, 0, 12] = numpy.sum(x140 * x416 * x436)
    result[1, 0, 13] = numpy.sum(x163 * x407 * x436)
    result[1, 0, 14] = numpy.sum(x413 * x436 * x440)
    result[1, 1, 0] = numpy.sum(x402 * x408 * x443)
    result[1, 1, 1] = numpy.sum(x126 * x446 * x66)
    result[1, 1, 2] = numpy.sum(x156 * x447 * x66)
    result[1, 1, 3] = numpy.sum(x240 * x411 * x450)
    result[1, 1, 4] = numpy.sum(x156 * x411 * x451)
    result[1, 1, 5] = numpy.sum(x194 * x411 * x442)
    result[1, 1, 6] = numpy.sum(x199 * x428 * x456)
    result[1, 1, 7] = numpy.sum(x156 * x450 * x458)
    result[1, 1, 8] = numpy.sum(x138 * x446 * x457)
    result[1, 1, 9] = numpy.sum(x202 * x428 * x442)
    result[1, 1, 10] = numpy.sum(x424 * x430 * x460)
    result[1, 1, 11] = numpy.sum(x156 * x456 * x461)
    result[1, 1, 12] = numpy.sum(x194 * x424 * x450)
    result[1, 1, 13] = numpy.sum(x202 * x424 * x445)
    result[1, 1, 14] = numpy.sum(x209 * x424 * x443)
    result[1, 2, 0] = numpy.sum(x211 * x402 * x409)
    result[1, 2, 1] = numpy.sum(x213 * x462 * x66)
    result[1, 2, 2] = numpy.sum(x217 * x413 * x66)
    result[1, 2, 3] = numpy.sum(x213 * x411 * x463)
    result[1, 2, 4] = numpy.sum(x219 * x406 * x411)
    result[1, 2, 5] = numpy.sum(x223 * x411 * x413)
    result[1, 2, 6] = numpy.sum(x374 * x422 * x428)
    result[1, 2, 7] = numpy.sum(x217 * x416 * x457)
    result[1, 2, 8] = numpy.sum(x222 * x457 * x462)
    result[1, 2, 9] = numpy.sum(x229 * x413 * x428)
    result[1, 2, 10] = numpy.sum(x438 * x464 * x465)
    result[1, 2, 11] = numpy.sum(x217 * x422 * x424)
    result[1, 2, 12] = numpy.sum(x223 * x416 * x424)
    result[1, 2, 13] = numpy.sum(x228 * x424 * x462)
    result[1, 2, 14] = numpy.sum(x232 * x432 * x464)
    result[1, 3, 0] = numpy.sum(x466 * x470 * x471)
    result[1, 3, 1] = numpy.sum(x240 * x473 * x64)
    result[1, 3, 2] = numpy.sum(x156 * x474 * x64)
    result[1, 3, 3] = numpy.sum(x126 * x476 * x477)
    result[1, 3, 4] = numpy.sum(x156 * x477 * x478)
    result[1, 3, 5] = numpy.sum(x249 * x36 * x469)
    result[1, 3, 6] = numpy.sum(x147 * x240 * x484)
    result[1, 3, 7] = numpy.sum(x156 * x485 * x486)
    result[1, 3, 8] = numpy.sum(x249 * x473 * x485)
    result[1, 3, 9] = numpy.sum(x147 * x469 * x487)
    result[1, 3, 10] = numpy.sum(x471 * x492 * x493)
    result[1, 3, 11] = numpy.sum(x156 * x484 * x494)
    result[1, 3, 12] = numpy.sum(x144 * x249 * x476)
    result[1, 3, 13] = numpy.sum(x144 * x473 * x487)
    result[1, 3, 14] = numpy.sum(x469 * x493 * x495)
    result[1, 4, 0] = numpy.sum(x213 * x466 * x496)
    result[1, 4, 1] = numpy.sum(x213 * x451 * x64)
    result[1, 4, 2] = numpy.sum(x219 * x442 * x64)
    result[1, 4, 3] = numpy.sum(x213 * x477 * x497)
    result[1, 4, 4] = numpy.sum(x269 * x36 * x445)
    result[1, 4, 5] = numpy.sum(x266 * x36 * x442)
    result[1, 4, 6] = numpy.sum(x147 * x456 * x498)
    result[1, 4, 7] = numpy.sum(x147 * x269 * x450)
    result[1, 4, 8] = numpy.sum(x147 * x445 * x500)
    result[1, 4, 9] = numpy.sum(x147 * x270 * x442)
    result[1, 4, 10] = numpy.sum(x144 * x460 * x501)
    result[1, 4, 11] = numpy.sum(x144 * x219 * x456)
    result[1, 4, 12] = numpy.sum(x144 * x266 * x450)
    result[1, 4, 13] = numpy.sum(x144 * x270 * x445)
    result[1, 4, 14] = numpy.sum(x144 * x274 * x443)
    result[1, 5, 0] = numpy.sum(x278 * x413 * x466)
    result[1, 5, 1] = numpy.sum(x279 * x406 * x64)
    result[1, 5, 2] = numpy.sum(x281 * x502 * x64)
    result[1, 5, 3] = numpy.sum(x333 * x36 * x416)
    result[1, 5, 4] = numpy.sum(x285 * x36 * x406)
    result[1, 5, 5] = numpy.sum(x288 * x36 * x413)
    result[1, 5, 6] = numpy.sum(x147 * x279 * x422)
    result[1, 5, 7] = numpy.sum(x147 * x284 * x503)
    result[1, 5, 8] = numpy.sum(x147 * x334 * x406)
    result[1, 5, 9] = numpy.sum(x147 * x292 * x502)
    result[1, 5, 10] = numpy.sum(x277 * x438 * x493)
    result[1, 5, 11] = numpy.sum(x281 * x422 * x494)
    result[1, 5, 12] = numpy.sum(x144 * x288 * x416)
    result[1, 5, 13] = numpy.sum(x292 * x406 * x494)
    result[1, 5, 14] = numpy.sum(x413 * x493 * x504)
    result[1, 6, 0] = numpy.sum(x430 * x508 * x512)
    result[1, 6, 1] = numpy.sum(x199 * x511 * x515)
    result[1, 6, 2] = numpy.sum(x156 * x511 * x516)
    result[1, 6, 3] = numpy.sum(x240 * x34 * x518)
    result[1, 6, 4] = numpy.sum(x156 * x182 * x34 * x519)
    result[1, 6, 5] = numpy.sum(x194 * x34 * x508)
    result[1, 6, 6] = numpy.sum(x126 * x521 * x522)
    result[1, 6, 7] = numpy.sum(x156 * x518 * x523)
    result[1, 6, 8] = numpy.sum(x138 * x519 * x522)
    result[1, 6, 9] = numpy.sum(x202 * x32 * x508)
    result[1, 6, 10] = numpy.sum(x408 * x526 * x527)
    result[1, 6, 11] = numpy.sum(x156 * x521 * x528)
    result[1, 6, 12] = numpy.sum(x194 * x518 * x75)
    result[1, 6, 13] = numpy.sum(x202 * x515 * x75)
    result[1, 6, 14] = numpy.sum(x209 * x508 * x527)
    result[1, 7, 0] = numpy.sum(x469 * x501 * x512)
    result[1, 7, 1] = numpy.sum(x473 * x498 * x511)
    result[1, 7, 2] = numpy.sum(x219 * x469 * x511)
    result[1, 7, 3] = numpy.sum(x213 * x486 * x529)
    result[1, 7, 4] = numpy.sum(x269 * x34 * x473)
    result[1, 7, 5] = numpy.sum(x266 * x34 * x469)
    result[1, 7, 6] = numpy.sum(x213 * x484 * x523)
    result[1, 7, 7] = numpy.sum(x269 * x32 * x476)
    result[1, 7, 8] = numpy.sum(x32 * x473 * x500)
    result[1, 7, 9] = numpy.sum(x270 * x32 * x469)
    result[1, 7, 10] = numpy.sum(x213 * x492 * x530)
    result[1, 7, 11] = numpy.sum(x219 * x484 * x75)
    result[1, 7, 12] = numpy.sum(x266 * x476 * x75)
    result[1, 7, 13] = numpy.sum(x228 * x473 * x531)
    result[1, 7, 14] = numpy.sum(x274 * x469 * x527)
    result[1, 8, 0] = numpy.sum(x331 * x443 * x512)
    result[1, 8, 1] = numpy.sum(x332 * x446 * x511)
    result[1, 8, 2] = numpy.sum(x281 * x511 * x532)
    result[1, 8, 3] = numpy.sum(x333 * x450 * x529)
    result[1, 8, 4] = numpy.sum(x34 * x445 * x533)
    result[1, 8, 5] = numpy.sum(x334 * x34 * x442)
    result[1, 8, 6] = numpy.sum(x332 * x456 * x522)
    result[1, 8, 7] = numpy.sum(x32 * x450 * x533)
    result[1, 8, 8] = numpy.sum(x287 * x445 * x534)
    result[1, 8, 9] = numpy.sum(x292 * x32 * x532)
    result[1, 8, 10] = numpy.sum(x331 * x460 * x527)
    result[1, 8, 11] = numpy.sum(x281 * x456 * x531)
    result[1, 8, 12] = numpy.sum(x334 * x450 * x75)
    result[1, 8, 13] = numpy.sum(x292 * x445 * x531)
    result[1, 8, 14] = numpy.sum(x297 * x496 * x75)
    result[1, 9, 0] = numpy.sum(x336 * x432 * x512)
    result[1, 9, 1] = numpy.sum(x337 * x406 * x511)
    result[1, 9, 2] = numpy.sum(x341 * x413 * x511)
    result[1, 9, 3] = numpy.sum(x34 * x384 * x416)
    result[1, 9, 4] = numpy.sum(x34 * x347 * x462)
    result[1, 9, 5] = numpy.sum(x34 * x345 * x413)
    result[1, 9, 6] = numpy.sum(x32 * x337 * x422)
    result[1, 9, 7] = numpy.sum(x341 * x416 * x535)
    result[1, 9, 8] = numpy.sum(x344 * x462 * x535)
    result[1, 9, 9] = numpy.sum(x349 * x413 * x522)
    result[1, 9, 10] = numpy.sum(x336 * x438 * x527)
    result[1, 9, 11] = numpy.sum(x341 * x422 * x75)
    result[1, 9, 12] = numpy.sum(x345 * x416 * x75)
    result[1, 9, 13] = numpy.sum(x349 * x462 * x75)
    result[1, 9, 14] = numpy.sum(x413 * x527 * x536)
    result[1, 10, 0] = numpy.sum(x155 * x537 * x540)
    result[1, 10, 1] = numpy.sum(x430 * x509 * x541)
    result[1, 10, 2] = numpy.sum(x439 * x509 * x537)
    result[1, 10, 3] = numpy.sum(x362 * x543 * x544)
    result[1, 10, 4] = numpy.sum(x354 * x431 * x541)
    result[1, 10, 5] = numpy.sum(x140 * x354 * x537)
    result[1, 10, 6] = numpy.sum(x408 * x545 * x546)
    result[1, 10, 7] = numpy.sum(x156 * x543 * x547)
    result[1, 10, 8] = numpy.sum(x160 * x541 * x546)
    result[1, 10, 9] = numpy.sum(x163 * x537 * x546)
    result[1, 10, 10] = numpy.sum(x10 * x549)
    result[1, 10, 11] = numpy.sum(x545 * x550 * x551)
    result[1, 10, 12] = numpy.sum(x140 * x22 * x543)
    result[1, 10, 13] = numpy.sum(x163 * x541 * x552)
    result[1, 10, 14] = numpy.sum(x22 * x440 * x537)
    result[1, 11, 0] = numpy.sum(x465 * x540 * x553)
    result[1, 11, 1] = numpy.sum(x374 * x509 * x515)
    result[1, 11, 2] = numpy.sum(x217 * x508 * x509)
    result[1, 11, 3] = numpy.sum(x213 * x518 * x554)
    result[1, 11, 4] = numpy.sum(x219 * x354 * x515)
    result[1, 11, 5] = numpy.sum(x223 * x354 * x508)
    result[1, 11, 6] = numpy.sum(x213 * x521 * x555)
    result[1, 11, 7] = numpy.sum(x13 * x219 * x518)
    result[1, 11, 8] = numpy.sum(x222 * x519 * x555)
    result[1, 11, 9] = numpy.sum(x228 * x508 * x555)
    result[1, 11, 10] = numpy.sum(x211 * x550 * x556)
    result[1, 11, 11] = numpy.sum(x217 * x22 * x521)
    result[1, 11, 12] = numpy.sum(x22 * x223 * x518)
    result[1, 11, 13] = numpy.sum(x22 * x229 * x515)
    result[1, 11, 14] = numpy.sum(x232 * x552 * x553)
    result[1, 12, 0] = numpy.sum(x278 * x469 * x540)
    result[1, 12, 1] = numpy.sum(x279 * x473 * x509)
    result[1, 12, 2] = numpy.sum(x281 * x474 * x509)
    result[1, 12, 3] = numpy.sum(x333 * x354 * x476)
    result[1, 12, 4] = numpy.sum(x284 * x473 * x557)
    result[1, 12, 5] = numpy.sum(x288 * x354 * x469)
    result[1, 12, 6] = numpy.sum(x13 * x279 * x484)
    result[1, 12, 7] = numpy.sum(x13 * x285 * x476)
    result[1, 12, 8] = numpy.sum(x13 * x334 * x473)
    result[1, 12, 9] = numpy.sum(x13 * x292 * x474)
    result[1, 12, 10] = numpy.sum(x22 * x278 * x492)
    result[1, 12, 11] = numpy.sum(x281 * x484 * x558)
    result[1, 12, 12] = numpy.sum(x22 * x288 * x476)
    result[1, 12, 13] = numpy.sum(x292 * x473 * x558)
    result[1, 12, 14] = numpy.sum(x22 * x470 * x504)
    result[1, 13, 0] = numpy.sum(x336 * x443 * x540)
    result[1, 13, 1] = numpy.sum(x337 * x445 * x509)
    result[1, 13, 2] = numpy.sum(x341 * x442 * x509)
    result[1, 13, 3] = numpy.sum(x354 * x384 * x450)
    result[1, 13, 4] = numpy.sum(x342 * x354 * x445)
    result[1, 13, 5] = numpy.sum(x345 * x354 * x442)
    result[1, 13, 6] = numpy.sum(x13 * x337 * x456)
    result[1, 13, 7] = numpy.sum(x13 * x342 * x450)
    result[1, 13, 8] = numpy.sum(x344 * x445 * x559)
    result[1, 13, 9] = numpy.sum(x349 * x442 * x555)
    result[1, 13, 10] = numpy.sum(x336 * x460 * x552)
    result[1, 13, 11] = numpy.sum(x22 * x341 * x456)
    result[1, 13, 12] = numpy.sum(x22 * x345 * x450)
    result[1, 13, 13] = numpy.sum(x22 * x349 * x446)
    result[1, 13, 14] = numpy.sum(x22 * x443 * x536)
    result[1, 14, 0] = numpy.sum(x413 * x540 * x560)
    result[1, 14, 1] = numpy.sum(x387 * x407 * x509)
    result[1, 14, 2] = numpy.sum(x432 * x509 * x561)
    result[1, 14, 3] = numpy.sum(x390 * x416 * x544)
    result[1, 14, 4] = numpy.sum(x354 * x391 * x407)
    result[1, 14, 5] = numpy.sum(x394 * x413 * x544)
    result[1, 14, 6] = numpy.sum(x387 * x422 * x546)
    result[1, 14, 7] = numpy.sum(x391 * x416 * x546)
    result[1, 14, 8] = numpy.sum(x13 * x393 * x418)
    result[1, 14, 9] = numpy.sum(x413 * x546 * x562)
    result[1, 14, 10] = numpy.sum(x22 * x438 * x560)
    result[1, 14, 11] = numpy.sum(x422 * x552 * x561)
    result[1, 14, 12] = numpy.sum(x22 * x394 * x417)
    result[1, 14, 13] = numpy.sum(x22 * x407 * x562)
    result[1, 14, 14] = numpy.sum(x396 * x397 * x566)
    result[2, 0, 0] = numpy.sum(x403 * x567)
    result[2, 0, 1] = numpy.sum(x111 * x404 * x568)
    result[2, 0, 2] = numpy.sum(x404 * x571 * x572)
    result[2, 0, 3] = numpy.sum(x133 * x412 * x573)
    result[2, 0, 4] = numpy.sum(x159 * x412 * x574)
    result[2, 0, 5] = numpy.sum(x392 * x412 * x578)
    result[2, 0, 6] = numpy.sum(x154 * x429 * x579)
    result[2, 0, 7] = numpy.sum(x157 * x429 * x571)
    result[2, 0, 8] = numpy.sum(x429 * x577 * x580)
    result[2, 0, 9] = numpy.sum(x429 * x584 * x585)
    result[2, 0, 10] = numpy.sum(x166 * x436 * x579)
    result[2, 0, 11] = numpy.sum(x154 * x436 * x571)
    result[2, 0, 12] = numpy.sum(x132 * x436 * x578)
    result[2, 0, 13] = numpy.sum(x388 * x436 * x586)
    result[2, 0, 14] = numpy.sum(x164 * x436 * x588)
    result[2, 1, 0] = numpy.sum(x175 * x402 * x568)
    result[2, 1, 1] = numpy.sum(x181 * x573 * x66)
    result[2, 1, 2] = numpy.sum(x177 * x589 * x66)
    result[2, 1, 3] = numpy.sum(x191 * x411 * x573)
    result[2, 1, 4] = numpy.sum(x193 * x411 * x570)
    result[2, 1, 5] = numpy.sum(x177 * x411 * x590)
    result[2, 1, 6] = numpy.sum(x198 * x428 * x591)
    result[2, 1, 7] = numpy.sum(x188 * x458 * x570)
    result[2, 1, 8] = numpy.sum(x181 * x457 * x577)
    result[2, 1, 9] = numpy.sum(x385 * x428 * x584)
    result[2, 1, 10] = numpy.sum(x204 * x464 * x579)
    result[2, 1, 11] = numpy.sum(x198 * x461 * x570)
    result[2, 1, 12] = numpy.sum(x188 * x424 * x590)
    result[2, 1, 13] = numpy.sum(x181 * x424 * x584)
    result[2, 1, 14] = numpy.sum(x464 * x588 * x592)
    result[2, 2, 0] = numpy.sum(x402 * x572 * x595)
    result[2, 2, 1] = numpy.sum(x159 * x596 * x66)
    result[2, 2, 2] = numpy.sum(x128 * x599 * x66)
    result[2, 2, 3] = numpy.sum(x218 * x411 * x594)
    result[2, 2, 4] = numpy.sum(x159 * x411 * x600)
    result[2, 2, 5] = numpy.sum(x282 * x411 * x603)
    result[2, 2, 6] = numpy.sum(x224 * x428 * x594)
    result[2, 2, 7] = numpy.sum(x131 * x458 * x598)
    result[2, 2, 8] = numpy.sum(x159 * x458 * x603)
    result[2, 2, 9] = numpy.sum(x428 * x608 * x609)
    result[2, 2, 10] = numpy.sum(x230 * x424 * x595)
    result[2, 2, 11] = numpy.sum(x153 * x461 * x598)
    result[2, 2, 12] = numpy.sum(x218 * x424 * x603)
    result[2, 2, 13] = numpy.sum(x159 * x461 * x608)
    result[2, 2, 14] = numpy.sum(x424 * x585 * x611)
    result[2, 3, 0] = numpy.sum(x237 * x466 * x573)
    result[2, 3, 1] = numpy.sum(x239 * x612 * x64)
    result[2, 3, 2] = numpy.sum(x241 * x570 * x64)
    result[2, 3, 3] = numpy.sum(x245 * x36 * x573)
    result[2, 3, 4] = numpy.sum(x247 * x477 * x570)
    result[2, 3, 5] = numpy.sum(x235 * x36 * x613)
    result[2, 3, 6] = numpy.sum(x147 * x252 * x612)
    result[2, 3, 7] = numpy.sum(x245 * x485 * x570)
    result[2, 3, 8] = numpy.sum(x239 * x485 * x613)
    result[2, 3, 9] = numpy.sum(x147 * x241 * x584)
    result[2, 3, 10] = numpy.sum(x258 * x493 * x614)
    result[2, 3, 11] = numpy.sum(x252 * x494 * x570)
    result[2, 3, 12] = numpy.sum(x144 * x243 * x613)
    result[2, 3, 13] = numpy.sum(x239 * x494 * x584)
    result[2, 3, 14] = numpy.sum(x144 * x237 * x588)
    result[2, 4, 0] = numpy.sum(x177 * x466 * x615)
    result[2, 4, 1] = numpy.sum(x193 * x594 * x64)
    result[2, 4, 2] = numpy.sum(x177 * x600 * x64)
    result[2, 4, 3] = numpy.sum(x262 * x477 * x594)
    result[2, 4, 4] = numpy.sum(x264 * x36 * x598)
    result[2, 4, 5] = numpy.sum(x177 * x603 * x616)
    result[2, 4, 6] = numpy.sum(x147 * x267 * x596)
    result[2, 4, 7] = numpy.sum(x147 * x188 * x617)
    result[2, 4, 8] = numpy.sum(x147 * x264 * x603)
    result[2, 4, 9] = numpy.sum(x147 * x608 * x618)
    result[2, 4, 10] = numpy.sum(x144 * x204 * x615)
    result[2, 4, 11] = numpy.sum(x144 * x267 * x599)
    result[2, 4, 12] = numpy.sum(x144 * x262 * x619)
    result[2, 4, 13] = numpy.sum(x144 * x193 * x608)
    result[2, 4, 14] = numpy.sum(x144 * x611 * x620)
    result[2, 5, 0] = numpy.sum(x466 * x624 * x625)
    result[2, 5, 1] = numpy.sum(x159 * x626 * x64)
    result[2, 5, 2] = numpy.sum(x282 * x628 * x64)
    result[2, 5, 3] = numpy.sum(x283 * x36 * x623)
    result[2, 5, 4] = numpy.sum(x159 * x616 * x628)
    result[2, 5, 5] = numpy.sum(x128 * x477 * x630)
    result[2, 5, 6] = numpy.sum(x147 * x289 * x624)
    result[2, 5, 7] = numpy.sum(x283 * x485 * x628)
    result[2, 5, 8] = numpy.sum(x159 * x485 * x631)
    result[2, 5, 9] = numpy.sum(x147 * x282 * x637)
    result[2, 5, 10] = numpy.sum(x144 * x293 * x624)
    result[2, 5, 11] = numpy.sum(x144 * x628 * x638)
    result[2, 5, 12] = numpy.sum(x144 * x283 * x630)
    result[2, 5, 13] = numpy.sum(x159 * x494 * x637)
    result[2, 5, 14] = numpy.sum(x392 * x493 * x643)
    result[2, 6, 0] = numpy.sum(x302 * x512 * x579)
    result[2, 6, 1] = numpy.sum(x305 * x511 * x591)
    result[2, 6, 2] = numpy.sum(x307 * x511 * x570)
    result[2, 6, 3] = numpy.sum(x34 * x375 * x573)
    result[2, 6, 4] = numpy.sum(x311 * x34 * x570)
    result[2, 6, 5] = numpy.sum(x301 * x34 * x590)
    result[2, 6, 6] = numpy.sum(x313 * x522 * x573)
    result[2, 6, 7] = numpy.sum(x309 * x523 * x570)
    result[2, 6, 8] = numpy.sum(x311 * x32 * x577)
    result[2, 6, 9] = numpy.sum(x307 * x32 * x584)
    result[2, 6, 10] = numpy.sum(x318 * x527 * x644)
    result[2, 6, 11] = numpy.sum(x313 * x528 * x570)
    result[2, 6, 12] = numpy.sum(x309 * x590 * x75)
    result[2, 6, 13] = numpy.sum(x305 * x528 * x584)
    result[2, 6, 14] = numpy.sum(x302 * x527 * x588)
    result[2, 7, 0] = numpy.sum(x235 * x512 * x615)
    result[2, 7, 1] = numpy.sum(x239 * x511 * x645)
    result[2, 7, 2] = numpy.sum(x325 * x511 * x599)
    result[2, 7, 3] = numpy.sum(x245 * x529 * x594)
    result[2, 7, 4] = numpy.sum(x239 * x34 * x617)
    result[2, 7, 5] = numpy.sum(x235 * x529 * x619)
    result[2, 7, 6] = numpy.sum(x252 * x535 * x596)
    result[2, 7, 7] = numpy.sum(x243 * x534 * x598)
    result[2, 7, 8] = numpy.sum(x239 * x534 * x603)
    result[2, 7, 9] = numpy.sum(x325 * x522 * x608)
    result[2, 7, 10] = numpy.sum(x258 * x530 * x594)
    result[2, 7, 11] = numpy.sum(x252 * x531 * x598)
    result[2, 7, 12] = numpy.sum(x326 * x603 * x75)
    result[2, 7, 13] = numpy.sum(x239 * x531 * x608)
    result[2, 7, 14] = numpy.sum(x235 * x530 * x611)
    result[2, 8, 0] = numpy.sum(x512 * x620 * x623)
    result[2, 8, 1] = numpy.sum(x193 * x511 * x623)
    result[2, 8, 2] = numpy.sum(x511 * x618 * x628)
    result[2, 8, 3] = numpy.sum(x263 * x34 * x623)
    result[2, 8, 4] = numpy.sum(x264 * x34 * x628)
    result[2, 8, 5] = numpy.sum(x177 * x529 * x631)
    result[2, 8, 6] = numpy.sum(x267 * x522 * x623)
    result[2, 8, 7] = numpy.sum(x188 * x534 * x628)
    result[2, 8, 8] = numpy.sum(x264 * x32 * x630)
    result[2, 8, 9] = numpy.sum(x177 * x523 * x637)
    result[2, 8, 10] = numpy.sum(x204 * x530 * x623)
    result[2, 8, 11] = numpy.sum(x198 * x531 * x628)
    result[2, 8, 12] = numpy.sum(x263 * x630 * x75)
    result[2, 8, 13] = numpy.sum(x193 * x637 * x75)
    result[2, 8, 14] = numpy.sum(x177 * x530 * x643)
    result[2, 9, 0] = numpy.sum(x512 * x585 * x649)
    result[2, 9, 1] = numpy.sum(x159 * x511 * x650)
    result[2, 9, 2] = numpy.sum(x511 * x609 * x653)
    result[2, 9, 3] = numpy.sum(x218 * x34 * x649)
    result[2, 9, 4] = numpy.sum(x159 * x34 * x654)
    result[2, 9, 5] = numpy.sum(x282 * x34 * x656)
    result[2, 9, 6] = numpy.sum(x224 * x32 * x649)
    result[2, 9, 7] = numpy.sum(x131 * x523 * x653)
    result[2, 9, 8] = numpy.sum(x159 * x523 * x656)
    result[2, 9, 9] = numpy.sum(x128 * x522 * x658)
    result[2, 9, 10] = numpy.sum(x230 * x527 * x649)
    result[2, 9, 11] = numpy.sum(x224 * x653 * x75)
    result[2, 9, 12] = numpy.sum(x218 * x656 * x75)
    result[2, 9, 13] = numpy.sum(x159 * x528 * x658)
    result[2, 9, 14] = numpy.sum(x527 * x572 * x661)
    result[2, 10, 0] = numpy.sum(x352 * x540 * x579)
    result[2, 10, 1] = numpy.sum(x358 * x579 * x662)
    result[2, 10, 2] = numpy.sum(x360 * x509 * x571)
    result[2, 10, 3] = numpy.sum(x127 * x363 * x544 * x573)
    result[2, 10, 4] = numpy.sum(x354 * x365 * x571)
    result[2, 10, 5] = numpy.sum(x352 * x544 * x578)
    result[2, 10, 6] = numpy.sum(x366 * x546 * x644)
    result[2, 10, 7] = numpy.sum(x363 * x547 * x570)
    result[2, 10, 8] = numpy.sum(x365 * x546 * x577)
    result[2, 10, 9] = numpy.sum(x360 * x546 * x584)
    result[2, 10, 10] = numpy.sum(x370 * x550 * x567)
    result[2, 10, 11] = numpy.sum(x366 * x571 * x663)
    result[2, 10, 12] = numpy.sum(x22 * x364 * x578)
    result[2, 10, 13] = numpy.sum(x358 * x552 * x586)
    result[2, 10, 14] = numpy.sum(x352 * x552 * x588)
    result[2, 11, 0] = numpy.sum(x302 * x540 * x595)
    result[2, 11, 1] = numpy.sum(x305 * x509 * x596)
    result[2, 11, 2] = numpy.sum(x307 * x509 * x598)
    result[2, 11, 3] = numpy.sum(x309 * x554 * x594)
    result[2, 11, 4] = numpy.sum(x311 * x354 * x598)
    result[2, 11, 5] = numpy.sum(x301 * x554 * x603)
    result[2, 11, 6] = numpy.sum(x313 * x555 * x594)
    result[2, 11, 7] = numpy.sum(x309 * x559 * x598)
    result[2, 11, 8] = numpy.sum(x310 * x555 * x603)
    result[2, 11, 9] = numpy.sum(x13 * x307 * x608)
    result[2, 11, 10] = numpy.sum(x318 * x595 * x663)
    result[2, 11, 11] = numpy.sum(x22 * x313 * x599)
    result[2, 11, 12] = numpy.sum(x22 * x375 * x603)
    result[2, 11, 13] = numpy.sum(x305 * x608 * x664)
    result[2, 11, 14] = numpy.sum(x302 * x552 * x611)
    result[2, 12, 0] = numpy.sum(x237 * x540 * x623)
    result[2, 12, 1] = numpy.sum(x239 * x509 * x626)
    result[2, 12, 2] = numpy.sum(x241 * x509 * x628)
    result[2, 12, 3] = numpy.sum(x243 * x557 * x623)
    result[2, 12, 4] = numpy.sum(x247 * x557 * x628)
    result[2, 12, 5] = numpy.sum(x235 * x557 * x630)
    result[2, 12, 6] = numpy.sum(x13 * x252 * x626)
    result[2, 12, 7] = numpy.sum(x13 * x326 * x628)
    result[2, 12, 8] = numpy.sum(x13 * x247 * x631)
    result[2, 12, 9] = numpy.sum(x13 * x241 * x637)
    result[2, 12, 10] = numpy.sum(x22 * x258 * x665)
    result[2, 12, 11] = numpy.sum(x252 * x558 * x628)
    result[2, 12, 12] = numpy.sum(x22 * x245 * x630)
    result[2, 12, 13] = numpy.sum(x239 * x558 * x637)
    result[2, 12, 14] = numpy.sum(x22 * x237 * x643)
    result[2, 13, 0] = numpy.sum(x540 * x592 * x666)
    result[2, 13, 1] = numpy.sum(x181 * x509 * x649)
    result[2, 13, 2] = numpy.sum(x385 * x509 * x653)
    result[2, 13, 3] = numpy.sum(x188 * x554 * x649)
    result[2, 13, 4] = numpy.sum(x193 * x354 * x653)
    result[2, 13, 5] = numpy.sum(x177 * x554 * x656)
    result[2, 13, 6] = numpy.sum(x198 * x555 * x649)
    result[2, 13, 7] = numpy.sum(x188 * x559 * x653)
    result[2, 13, 8] = numpy.sum(x13 * x193 * x656)
    result[2, 13, 9] = numpy.sum(x177 * x555 * x658)
    result[2, 13, 10] = numpy.sum(x204 * x552 * x666)
    result[2, 13, 11] = numpy.sum(x198 * x653 * x664)
    result[2, 13, 12] = numpy.sum(x191 * x22 * x656)
    result[2, 13, 13] = numpy.sum(x181 * x22 * x658)
    result[2, 13, 14] = numpy.sum(x175 * x566 * x667)
    result[2, 14, 0] = numpy.sum(x164 * x540 * x668)
    result[2, 14, 1] = numpy.sum(x388 * x662 * x668)
    result[2, 14, 2] = numpy.sum(x509 * x585 * x669)
    result[2, 14, 3] = numpy.sum(x132 * x354 * x670)
    result[2, 14, 4] = numpy.sum(x354 * x580 * x669)
    result[2, 14, 5] = numpy.sum(x544 * x625 * x672)
    result[2, 14, 6] = numpy.sum(x154 * x546 * x668)
    result[2, 14, 7] = numpy.sum(x157 * x546 * x669)
    result[2, 14, 8] = numpy.sum(x159 * x547 * x672)
    result[2, 14, 9] = numpy.sum(x546 * x572 * x673)
    result[2, 14, 10] = numpy.sum(x166 * x552 * x668)
    result[2, 14, 11] = numpy.sum(x154 * x552 * x669)
    result[2, 14, 12] = numpy.sum(x133 * x22 * x672)
    result[2, 14, 13] = numpy.sum(x566 * x673 * x674)
    result[2, 14, 14] = numpy.sum(x10 * x675)
    result[3, 0, 0] = numpy.sum(x126 * x677 * x680)
    result[3, 0, 1] = numpy.sum(x408 * x681 * x684)
    result[3, 0, 2] = numpy.sum(x156 * x681 * x685)
    result[3, 0, 3] = numpy.sum(x688 * x689 * x690)
    result[3, 0, 4] = numpy.sum(x156 * x689 * x691)
    result[3, 0, 5] = numpy.sum(x140 * x679 * x689)
    result[3, 0, 6] = numpy.sum(x430 * x692 * x694)
    result[3, 0, 7] = numpy.sum(x431 * x688 * x694)
    result[3, 0, 8] = numpy.sum(x160 * x684 * x694)
    result[3, 0, 9] = numpy.sum(x163 * x680 * x694)
    result[3, 0, 10] = numpy.sum(x155 * x695 * x696)
    result[3, 0, 11] = numpy.sum(x361 * x692 * x697)
    result[3, 0, 12] = numpy.sum(x140 * x688 * x695)
    result[3, 0, 13] = numpy.sum(x163 * x684 * x695)
    result[3, 0, 14] = numpy.sum(x174 * x680 * x695)
    result[3, 1, 0] = numpy.sum(x401 * x408 * x700)
    result[3, 1, 1] = numpy.sum(x126 * x58 * x703)
    result[3, 1, 2] = numpy.sum(x156 * x58 * x704)
    result[3, 1, 3] = numpy.sum(x240 * x410 * x707)
    result[3, 1, 4] = numpy.sum(x156 * x410 * x708)
    result[3, 1, 5] = numpy.sum(x194 * x410 * x699)
    result[3, 1, 6] = numpy.sum(x199 * x427 * x712)
    result[3, 1, 7] = numpy.sum(x156 * x707 * x713)
    result[3, 1, 8] = numpy.sum(x427 * x703 * x714)
    result[3, 1, 9] = numpy.sum(x202 * x427 * x699)
    result[3, 1, 10] = numpy.sum(x155 * x717 * x718)
    result[3, 1, 11] = numpy.sum(x156 * x712 * x719)
    result[3, 1, 12] = numpy.sum(x194 * x435 * x707)
    result[3, 1, 13] = numpy.sum(x202 * x435 * x702)
    result[3, 1, 14] = numpy.sum(x209 * x435 * x700)
    result[3, 2, 0] = numpy.sum(x213 * x401 * x685)
    result[3, 2, 1] = numpy.sum(x374 * x58 * x683)
    result[3, 2, 2] = numpy.sum(x217 * x58 * x679)
    result[3, 2, 3] = numpy.sum(x213 * x688 * x720)
    result[3, 2, 4] = numpy.sum(x219 * x410 * x683)
    result[3, 2, 5] = numpy.sum(x223 * x410 * x679)
    result[3, 2, 6] = numpy.sum(x374 * x427 * x692)
    result[3, 2, 7] = numpy.sum(x219 * x427 * x688)
    result[3, 2, 8] = numpy.sum(x222 * x683 * x713)
    result[3, 2, 9] = numpy.sum(x229 * x427 * x679)
    result[3, 2, 10] = numpy.sum(x465 * x696 * x718)
    result[3, 2, 11] = numpy.sum(x217 * x435 * x692)
    result[3, 2, 12] = numpy.sum(x223 * x435 * x688)
    result[3, 2, 13] = numpy.sum(x229 * x435 * x683)
    result[3, 2, 14] = numpy.sum(x232 * x435 * x685)
    result[3, 3, 0] = numpy.sum(x690 * x723 * x724)
    result[3, 3, 1] = numpy.sum(x240 * x56 * x726)
    result[3, 3, 2] = numpy.sum(x156 * x56 * x727)
    result[3, 3, 3] = numpy.sum(x126 * x729 * x730)
    result[3, 3, 4] = numpy.sum(x156 * x726 * x731)
    result[3, 3, 5] = numpy.sum(x249 * x46 * x723)
    result[3, 3, 6] = numpy.sum(x145 * x240 * x734)
    result[3, 3, 7] = numpy.sum(x156 * x729 * x736)
    result[3, 3, 8] = numpy.sum(x249 * x726 * x735)
    result[3, 3, 9] = numpy.sum(x145 * x487 * x723)
    result[3, 3, 10] = numpy.sum(x126 * x740 * x742)
    result[3, 3, 11] = numpy.sum(x156 * x734 * x743)
    result[3, 3, 12] = numpy.sum(x249 * x433 * x729)
    result[3, 3, 13] = numpy.sum(x433 * x487 * x726)
    result[3, 3, 14] = numpy.sum(x495 * x723 * x741)
    result[3, 4, 0] = numpy.sum(x272 * x700 * x724)
    result[3, 4, 1] = numpy.sum(x213 * x56 * x708)
    result[3, 4, 2] = numpy.sum(x219 * x56 * x699)
    result[3, 4, 3] = numpy.sum(x213 * x707 * x731)
    result[3, 4, 4] = numpy.sum(x269 * x46 * x702)
    result[3, 4, 5] = numpy.sum(x265 * x699 * x730)
    result[3, 4, 6] = numpy.sum(x145 * x498 * x712)
    result[3, 4, 7] = numpy.sum(x145 * x269 * x707)
    result[3, 4, 8] = numpy.sum(x145 * x500 * x702)
    result[3, 4, 9] = numpy.sum(x145 * x270 * x699)
    result[3, 4, 10] = numpy.sum(x433 * x501 * x717)
    result[3, 4, 11] = numpy.sum(x219 * x433 * x712)
    result[3, 4, 12] = numpy.sum(x266 * x433 * x707)
    result[3, 4, 13] = numpy.sum(x270 * x433 * x702)
    result[3, 4, 14] = numpy.sum(x274 * x433 * x700)
    result[3, 5, 0] = numpy.sum(x278 * x679 * x724)
    result[3, 5, 1] = numpy.sum(x279 * x56 * x683)
    result[3, 5, 2] = numpy.sum(x281 * x56 * x744)
    result[3, 5, 3] = numpy.sum(x333 * x46 * x688)
    result[3, 5, 4] = numpy.sum(x284 * x683 * x730)
    result[3, 5, 5] = numpy.sum(x288 * x46 * x679)
    result[3, 5, 6] = numpy.sum(x145 * x279 * x692)
    result[3, 5, 7] = numpy.sum(x145 * x285 * x688)
    result[3, 5, 8] = numpy.sum(x145 * x334 * x683)
    result[3, 5, 9] = numpy.sum(x145 * x292 * x744)
    result[3, 5, 10] = numpy.sum(x278 * x433 * x696)
    result[3, 5, 11] = numpy.sum(x281 * x692 * x743)
    result[3, 5, 12] = numpy.sum(x288 * x433 * x688)
    result[3, 5, 13] = numpy.sum(x292 * x683 * x743)
    result[3, 5, 14] = numpy.sum(x297 * x741 * x745)
    result[3, 6, 0] = numpy.sum(x430 * x746 * x747)
    result[3, 6, 1] = numpy.sum(x126 * x750 * x751)
    result[3, 6, 2] = numpy.sum(x156 * x746 * x751)
    result[3, 6, 3] = numpy.sum(x126 * x753 * x754)
    result[3, 6, 4] = numpy.sum(x156 * x21 * x755)
    result[3, 6, 5] = numpy.sum(x194 * x21 * x746)
    result[3, 6, 6] = numpy.sum(x126 * x757 * x758)
    result[3, 6, 7] = numpy.sum(x156 * x753 * x759)
    result[3, 6, 8] = numpy.sum(x714 * x750 * x758)
    result[3, 6, 9] = numpy.sum(x19 * x202 * x746)
    result[3, 6, 10] = numpy.sum(x762 * x763)
    result[3, 6, 11] = numpy.sum(x15 * x184 * x757)
    result[3, 6, 12] = numpy.sum(x16 * x194 * x753)
    result[3, 6, 13] = numpy.sum(x16 * x202 * x750)
    result[3, 6, 14] = numpy.sum(x209 * x746 * x764)
    result[3, 7, 0] = numpy.sum(x501 * x723 * x747)
    result[3, 7, 1] = numpy.sum(x213 * x726 * x765)
    result[3, 7, 2] = numpy.sum(x219 * x510 * x723)
    result[3, 7, 3] = numpy.sum(x213 * x729 * x767)
    result[3, 7, 4] = numpy.sum(x21 * x269 * x726)
    result[3, 7, 5] = numpy.sum(x21 * x266 * x723)
    result[3, 7, 6] = numpy.sum(x213 * x734 * x759)
    result[3, 7, 7] = numpy.sum(x19 * x269 * x729)
    result[3, 7, 8] = numpy.sum(x19 * x500 * x726)
    result[3, 7, 9] = numpy.sum(x19 * x270 * x723)
    result[3, 7, 10] = numpy.sum(x134 * x211 * x740 * x763)
    result[3, 7, 11] = numpy.sum(x16 * x219 * x734)
    result[3, 7, 12] = numpy.sum(x16 * x266 * x729)
    result[3, 7, 13] = numpy.sum(x16 * x270 * x726)
    result[3, 7, 14] = numpy.sum(x274 * x723 * x764)
    result[3, 8, 0] = numpy.sum(x331 * x700 * x747)
    result[3, 8, 1] = numpy.sum(x332 * x510 * x703)
    result[3, 8, 2] = numpy.sum(x281 * x699 * x765)
    result[3, 8, 3] = numpy.sum(x333 * x707 * x766)
    result[3, 8, 4] = numpy.sum(x21 * x533 * x702)
    result[3, 8, 5] = numpy.sum(x21 * x334 * x699)
    result[3, 8, 6] = numpy.sum(x332 * x712 * x758)
    result[3, 8, 7] = numpy.sum(x19 * x533 * x707)
    result[3, 8, 8] = numpy.sum(x287 * x702 * x768)
    result[3, 8, 9] = numpy.sum(x292 * x699 * x759)
    result[3, 8, 10] = numpy.sum(x331 * x717 * x764)
    result[3, 8, 11] = numpy.sum(x281 * x712 * x769)
    result[3, 8, 12] = numpy.sum(x16 * x334 * x707)
    result[3, 8, 13] = numpy.sum(x16 * x292 * x708)
    result[3, 8, 14] = numpy.sum(x134 * x16 * x297 * x700)
    result[3, 9, 0] = numpy.sum(x336 * x680 * x747)
    result[3, 9, 1] = numpy.sum(x337 * x510 * x683)
    result[3, 9, 2] = numpy.sum(x341 * x510 * x679)
    result[3, 9, 3] = numpy.sum(x21 * x384 * x688)
    result[3, 9, 4] = numpy.sum(x21 * x342 * x683)
    result[3, 9, 5] = numpy.sum(x21 * x345 * x679)
    result[3, 9, 6] = numpy.sum(x19 * x337 * x692)
    result[3, 9, 7] = numpy.sum(x19 * x342 * x688)
    result[3, 9, 8] = numpy.sum(x344 * x683 * x759)
    result[3, 9, 9] = numpy.sum(x349 * x679 * x758)
    result[3, 9, 10] = numpy.sum(x336 * x696 * x764)
    result[3, 9, 11] = numpy.sum(x16 * x341 * x692)
    result[3, 9, 12] = numpy.sum(x16 * x345 * x688)
    result[3, 9, 13] = numpy.sum(x349 * x683 * x770)
    result[3, 9, 14] = numpy.sum(x16 * x351 * x685)
    result[3, 10, 0] = numpy.sum(x155 * x771 * x772)
    result[3, 10, 1] = numpy.sum(x155 * x773 * x774)
    result[3, 10, 2] = numpy.sum(x361 * x772 * x774)
    result[3, 10, 3] = numpy.sum(x126 * x775 * x777)
    result[3, 10, 4] = numpy.sum(x431 * x538 * x773)
    result[3, 10, 5] = numpy.sum(x140 * x538 * x772)
    result[3, 10, 6] = numpy.sum(x778 * x779)
    result[3, 10, 7] = numpy.sum(x116 * x548 * x775 * x780)
    result[3, 10, 8] = numpy.sum(x160 * x773 * x781)
    result[3, 10, 9] = numpy.sum(x163 * x772 * x781)
    result[3, 10, 10] = numpy.sum(
        x548
        * (
            x0 * (2.0 * x524 + 2.0 * x525 + 7.0 * x738 + 3.0 * x739 + 4.0 * x756)
            + x175 * x761
        )
    )
    result[3, 10, 11] = numpy.sum(x551 * x779)
    result[3, 10, 12] = numpy.sum(x140 * x775 * x8)
    result[3, 10, 13] = numpy.sum(x163 * x773 * x782)
    result[3, 10, 14] = numpy.sum(x174 * x772 * x782)
    result[3, 11, 0] = numpy.sum(x465 * x746 * x783)
    result[3, 11, 1] = numpy.sum(x213 * x750 * x784)
    result[3, 11, 2] = numpy.sum(x217 * x539 * x746)
    result[3, 11, 3] = numpy.sum(x213 * x753 * x785)
    result[3, 11, 4] = numpy.sum(x219 * x538 * x750)
    result[3, 11, 5] = numpy.sum(x223 * x538 * x746)
    result[3, 11, 6] = numpy.sum(x212 * x757 * x786)
    result[3, 11, 7] = numpy.sum(x11 * x219 * x753)
    result[3, 11, 8] = numpy.sum(x11 * x222 * x755)
    result[3, 11, 9] = numpy.sum(x11 * x229 * x746)
    result[3, 11, 10] = numpy.sum(x211 * x548 * x762)
    result[3, 11, 11] = numpy.sum(x217 * x757 * x8)
    result[3, 11, 12] = numpy.sum(x223 * x753 * x8)
    result[3, 11, 13] = numpy.sum(x229 * x750 * x8)
    result[3, 11, 14] = numpy.sum(x232 * x746 * x787)
    result[3, 12, 0] = numpy.sum(x278 * x723 * x771)
    result[3, 12, 1] = numpy.sum(x279 * x539 * x726)
    result[3, 12, 2] = numpy.sum(x281 * x723 * x788)
    result[3, 12, 3] = numpy.sum(x333 * x538 * x729)
    result[3, 12, 4] = numpy.sum(x284 * x726 * x789)
    result[3, 12, 5] = numpy.sum(x288 * x538 * x723)
    result[3, 12, 6] = numpy.sum(x11 * x279 * x734)
    result[3, 12, 7] = numpy.sum(x11 * x285 * x729)
    result[3, 12, 8] = numpy.sum(x11 * x334 * x726)
    result[3, 12, 9] = numpy.sum(x11 * x292 * x727)
    result[3, 12, 10] = numpy.sum(x278 * x740 * x8)
    result[3, 12, 11] = numpy.sum(x281 * x734 * x790)
    result[3, 12, 12] = numpy.sum(x288 * x729 * x8)
    result[3, 12, 13] = numpy.sum(x292 * x726 * x790)
    result[3, 12, 14] = numpy.sum(x127 * x297 * x723 * x791)
    result[3, 13, 0] = numpy.sum(x336 * x700 * x771)
    result[3, 13, 1] = numpy.sum(x337 * x539 * x702)
    result[3, 13, 2] = numpy.sum(x341 * x539 * x699)
    result[3, 13, 3] = numpy.sum(x384 * x538 * x707)
    result[3, 13, 4] = numpy.sum(x342 * x538 * x702)
    result[3, 13, 5] = numpy.sum(x345 * x538 * x699)
    result[3, 13, 6] = numpy.sum(x11 * x337 * x712)
    result[3, 13, 7] = numpy.sum(x11 * x342 * x707)
    result[3, 13, 8] = numpy.sum(x11 * x344 * x708)
    result[3, 13, 9] = numpy.sum(x11 * x349 * x704)
    result[3, 13, 10] = numpy.sum(x336 * x717 * x782)
    result[3, 13, 11] = numpy.sum(x341 * x712 * x8)
    result[3, 13, 12] = numpy.sum(x345 * x707 * x8)
    result[3, 13, 13] = numpy.sum(x349 * x703 * x8)
    result[3, 13, 14] = numpy.sum(x351 * x700 * x792)
    result[3, 14, 0] = numpy.sum(x386 * x680 * x771)
    result[3, 14, 1] = numpy.sum(x387 * x539 * x684)
    result[3, 14, 2] = numpy.sum(x389 * x539 * x685)
    result[3, 14, 3] = numpy.sum(x390 * x688 * x776)
    result[3, 14, 4] = numpy.sum(x391 * x538 * x684)
    result[3, 14, 5] = numpy.sum(x393 * x745 * x793)
    result[3, 14, 6] = numpy.sum(x387 * x692 * x781)
    result[3, 14, 7] = numpy.sum(x391 * x688 * x781)
    result[3, 14, 8] = numpy.sum(x11 * x393 * x691)
    result[3, 14, 9] = numpy.sum(x11 * x395 * x685)
    result[3, 14, 10] = numpy.sum(x386 * x696 * x782)
    result[3, 14, 11] = numpy.sum(x389 * x692 * x787)
    result[3, 14, 12] = numpy.sum(x394 * x688 * x791)
    result[3, 14, 13] = numpy.sum(x395 * x684 * x792)
    result[3, 14, 14] = numpy.sum(x396 * x680 * x8)
    result[4, 0, 0] = numpy.sum(x109 * x397 * x567 * x677)
    result[4, 0, 1] = numpy.sum(x407 * x644 * x681)
    result[4, 0, 2] = numpy.sum(x571 * x681 * x794)
    result[4, 0, 3] = numpy.sum(x417 * x573 * x795)
    result[4, 0, 4] = numpy.sum(x418 * x570 * x689)
    result[4, 0, 5] = numpy.sum(x104 * x413 * x578 * x689)
    result[4, 0, 6] = numpy.sum(x579 * x694 * x796)
    result[4, 0, 7] = numpy.sum(x416 * x574 * x694)
    result[4, 0, 8] = numpy.sum(x418 * x577 * x694)
    result[4, 0, 9] = numpy.sum(x432 * x586 * x694)
    result[4, 0, 10] = numpy.sum(x438 * x579 * x695)
    result[4, 0, 11] = numpy.sum(x422 * x571 * x697)
    result[4, 0, 12] = numpy.sum(x417 * x578 * x695)
    result[4, 0, 13] = numpy.sum(x407 * x584 * x697)
    result[4, 0, 14] = numpy.sum(x432 * x588 * x695)
    result[4, 1, 0] = numpy.sum(x401 * x443 * x644)
    result[4, 1, 1] = numpy.sum(x446 * x573 * x58)
    result[4, 1, 2] = numpy.sum(x447 * x570 * x58)
    result[4, 1, 3] = numpy.sum(x410 * x450 * x612)
    result[4, 1, 4] = numpy.sum(x410 * x451 * x570)
    result[4, 1, 5] = numpy.sum(x410 * x442 * x590)
    result[4, 1, 6] = numpy.sum(x427 * x456 * x591)
    result[4, 1, 7] = numpy.sum(x450 * x570 * x713)
    result[4, 1, 8] = numpy.sum(x427 * x451 * x577)
    result[4, 1, 9] = numpy.sum(x427 * x447 * x584)
    result[4, 1, 10] = numpy.sum(x460 * x579 * x718)
    result[4, 1, 11] = numpy.sum(x456 * x570 * x719)
    result[4, 1, 12] = numpy.sum(x435 * x450 * x590)
    result[4, 1, 13] = numpy.sum(x445 * x584 * x719)
    result[4, 1, 14] = numpy.sum(x443 * x588 * x718)
    result[4, 2, 0] = numpy.sum(x401 * x595 * x794)
    result[4, 2, 1] = numpy.sum(x462 * x58 * x594)
    result[4, 2, 2] = numpy.sum(x413 * x58 * x599)
    result[4, 2, 3] = numpy.sum(x410 * x463 * x594)
    result[4, 2, 4] = numpy.sum(x410 * x598 * x797)
    result[4, 2, 5] = numpy.sum(x410 * x502 * x603)
    result[4, 2, 6] = numpy.sum(x422 * x427 * x596)
    result[4, 2, 7] = numpy.sum(x416 * x427 * x600)
    result[4, 2, 8] = numpy.sum(x427 * x603 * x797)
    result[4, 2, 9] = numpy.sum(x182 * x413 * x427 * x608)
    result[4, 2, 10] = numpy.sum(x438 * x595 * x718)
    result[4, 2, 11] = numpy.sum(x422 * x598 * x719)
    result[4, 2, 12] = numpy.sum(x435 * x463 * x603)
    result[4, 2, 13] = numpy.sum(x435 * x462 * x608)
    result[4, 2, 14] = numpy.sum(x432 * x611 * x718)
    result[4, 3, 0] = numpy.sum(x470 * x614 * x724)
    result[4, 3, 1] = numpy.sum(x473 * x56 * x612)
    result[4, 3, 2] = numpy.sum(x474 * x56 * x570)
    result[4, 3, 3] = numpy.sum(x476 * x573 * x730)
    result[4, 3, 4] = numpy.sum(x473 * x570 * x731)
    result[4, 3, 5] = numpy.sum(x46 * x469 * x613)
    result[4, 3, 6] = numpy.sum(x145 * x484 * x612)
    result[4, 3, 7] = numpy.sum(x486 * x570 * x735)
    result[4, 3, 8] = numpy.sum(x473 * x613 * x735)
    result[4, 3, 9] = numpy.sum(x145 * x474 * x584)
    result[4, 3, 10] = numpy.sum(x492 * x573 * x742)
    result[4, 3, 11] = numpy.sum(x484 * x570 * x743)
    result[4, 3, 12] = numpy.sum(x433 * x476 * x613)
    result[4, 3, 13] = numpy.sum(x473 * x584 * x743)
    result[4, 3, 14] = numpy.sum(x469 * x588 * x742)
    result[4, 4, 0] = numpy.sum(x496 * x594 * x724)
    result[4, 4, 1] = numpy.sum(x445 * x56 * x645)
    result[4, 4, 2] = numpy.sum(x532 * x56 * x598)
    result[4, 4, 3] = numpy.sum(x450 * x594 * x731)
    result[4, 4, 4] = numpy.sum(x445 * x46 * x617)
    result[4, 4, 5] = numpy.sum(x442 * x603 * x731)
    result[4, 4, 6] = numpy.sum(x145 * x456 * x645)
    result[4, 4, 7] = numpy.sum(x145 * x450 * x617)
    result[4, 4, 8] = numpy.sum(x145 * x603 * x798)
    result[4, 4, 9] = numpy.sum(x145 * x532 * x608)
    result[4, 4, 10] = numpy.sum(x433 * x460 * x615)
    result[4, 4, 11] = numpy.sum(x433 * x456 * x600)
    result[4, 4, 12] = numpy.sum(x433 * x497 * x619)
    result[4, 4, 13] = numpy.sum(x433 * x451 * x608)
    result[4, 4, 14] = numpy.sum(x433 * x496 * x611)
    result[4, 5, 0] = numpy.sum(x413 * x665 * x724)
    result[4, 5, 1] = numpy.sum(x406 * x56 * x626)
    result[4, 5, 2] = numpy.sum(x502 * x56 * x628)
    result[4, 5, 3] = numpy.sum(x46 * x503 * x623)
    result[4, 5, 4] = numpy.sum(x406 * x628 * x731)
    result[4, 5, 5] = numpy.sum(x413 * x630 * x730)
    result[4, 5, 6] = numpy.sum(x145 * x422 * x626)
    result[4, 5, 7] = numpy.sum(x503 * x628 * x735)
    result[4, 5, 8] = numpy.sum(x406 * x631 * x735)
    result[4, 5, 9] = numpy.sum(x145 * x502 * x637)
    result[4, 5, 10] = numpy.sum(x438 * x624 * x741)
    result[4, 5, 11] = numpy.sum(x422 * x628 * x743)
    result[4, 5, 12] = numpy.sum(x433 * x503 * x630)
    result[4, 5, 13] = numpy.sum(x406 * x637 * x743)
    result[4, 5, 14] = numpy.sum(x413 * x643 * x742)
    result[4, 6, 0] = numpy.sum(x553 * x579 * x747)
    result[4, 6, 1] = numpy.sum(x515 * x573 * x751)
    result[4, 6, 2] = numpy.sum(x510 * x516 * x570)
    result[4, 6, 3] = numpy.sum(x518 * x573 * x754)
    result[4, 6, 4] = numpy.sum(x21 * x519 * x589)
    result[4, 6, 5] = numpy.sum(x21 * x508 * x590)
    result[4, 6, 6] = numpy.sum(x521 * x573 * x758)
    result[4, 6, 7] = numpy.sum(x518 * x570 * x759)
    result[4, 6, 8] = numpy.sum(x519 * x577 * x758)
    result[4, 6, 9] = numpy.sum(x508 * x584 * x758)
    result[4, 6, 10] = numpy.sum(x556 * x567 * x763)
    result[4, 6, 11] = numpy.sum(x16 * x521 * x589)
    result[4, 6, 12] = numpy.sum(x16 * x518 * x590)
    result[4, 6, 13] = numpy.sum(x515 * x584 * x770)
    result[4, 6, 14] = numpy.sum(x553 * x588 * x764)
    result[4, 7, 0] = numpy.sum(x469 * x615 * x747)
    result[4, 7, 1] = numpy.sum(x473 * x510 * x645)
    result[4, 7, 2] = numpy.sum(x469 * x510 * x600)
    result[4, 7, 3] = numpy.sum(x486 * x594 * x766)
    result[4, 7, 4] = numpy.sum(x21 * x473 * x617)
    result[4, 7, 5] = numpy.sum(x469 * x619 * x766)
    result[4, 7, 6] = numpy.sum(x19 * x484 * x645)
    result[4, 7, 7] = numpy.sum(x19 * x476 * x617)
    result[4, 7, 8] = numpy.sum(x473 * x603 * x768)
    result[4, 7, 9] = numpy.sum(x469 * x608 * x759)
    result[4, 7, 10] = numpy.sum(x16 * x492 * x615)
    result[4, 7, 11] = numpy.sum(x16 * x484 * x600)
    result[4, 7, 12] = numpy.sum(x16 * x246 * x476 * x619)
    result[4, 7, 13] = numpy.sum(x473 * x608 * x769)
    result[4, 7, 14] = numpy.sum(x469 * x611 * x799)
    result[4, 8, 0] = numpy.sum(x496 * x623 * x747)
    result[4, 8, 1] = numpy.sum(x451 * x510 * x623)
    result[4, 8, 2] = numpy.sum(x510 * x532 * x628)
    result[4, 8, 3] = numpy.sum(x450 * x623 * x767)
    result[4, 8, 4] = numpy.sum(x21 * x628 * x798)
    result[4, 8, 5] = numpy.sum(x442 * x631 * x766)
    result[4, 8, 6] = numpy.sum(x456 * x623 * x759)
    result[4, 8, 7] = numpy.sum(x450 * x628 * x768)
    result[4, 8, 8] = numpy.sum(x445 * x630 * x768)
    result[4, 8, 9] = numpy.sum(x442 * x637 * x759)
    result[4, 8, 10] = numpy.sum(x460 * x623 * x799)
    result[4, 8, 11] = numpy.sum(x456 * x628 * x769)
    result[4, 8, 12] = numpy.sum(x16 * x497 * x631)
    result[4, 8, 13] = numpy.sum(x16 * x451 * x637)
    result[4, 8, 14] = numpy.sum(x16 * x496 * x643)
    result[4, 9, 0] = numpy.sum(x432 * x666 * x747)
    result[4, 9, 1] = numpy.sum(x462 * x510 * x649)
    result[4, 9, 2] = numpy.sum(x413 * x653 * x751)
    result[4, 9, 3] = numpy.sum(x21 * x463 * x649)
    result[4, 9, 4] = numpy.sum(x21 * x653 * x797)
    result[4, 9, 5] = numpy.sum(x413 * x656 * x754)
    result[4, 9, 6] = numpy.sum(x422 * x649 * x758)
    result[4, 9, 7] = numpy.sum(x416 * x653 * x759)
    result[4, 9, 8] = numpy.sum(x19 * x656 * x797)
    result[4, 9, 9] = numpy.sum(x413 * x658 * x758)
    result[4, 9, 10] = numpy.sum(x438 * x666 * x764)
    result[4, 9, 11] = numpy.sum(x422 * x653 * x770)
    result[4, 9, 12] = numpy.sum(x16 * x463 * x656)
    result[4, 9, 13] = numpy.sum(x16 * x462 * x658)
    result[4, 9, 14] = numpy.sum(x397 * x667 * x800)
    result[4, 10, 0] = numpy.sum(x537 * x579 * x771)
    result[4, 10, 1] = numpy.sum(x541 * x579 * x774)
    result[4, 10, 2] = numpy.sum(x537 * x571 * x774)
    result[4, 10, 3] = numpy.sum(x543 * x573 * x777)
    result[4, 10, 4] = numpy.sum(x538 * x541 * x574)
    result[4, 10, 5] = numpy.sum(x537 * x578 * x776)
    result[4, 10, 6] = numpy.sum(x545 * x548 * x567 * x778)
    result[4, 10, 7] = numpy.sum(x11 * x543 * x574)
    result[4, 10, 8] = numpy.sum(x541 * x577 * x801)
    result[4, 10, 9] = numpy.sum(x537 * x586 * x781)
    result[4, 10, 10] = numpy.sum(x549 * x567)
    result[4, 10, 11] = numpy.sum(x545 * x571 * x792)
    result[4, 10, 12] = numpy.sum(x543 * x578 * x791)
    result[4, 10, 13] = numpy.sum(x541 * x584 * x787)
    result[4, 10, 14] = numpy.sum(x537 * x588 * x782)
    result[4, 11, 0] = numpy.sum(x508 * x595 * x783)
    result[4, 11, 1] = numpy.sum(x515 * x539 * x596)
    result[4, 11, 2] = numpy.sum(x516 * x539 * x598)
    result[4, 11, 3] = numpy.sum(x518 * x594 * x785)
    result[4, 11, 4] = numpy.sum(x519 * x538 * x599)
    result[4, 11, 5] = numpy.sum(x508 * x603 * x785)
    result[4, 11, 6] = numpy.sum(x11 * x521 * x596)
    result[4, 11, 7] = numpy.sum(x11 * x518 * x600)
    result[4, 11, 8] = numpy.sum(x519 * x603 * x802)
    result[4, 11, 9] = numpy.sum(x11 * x516 * x608)
    result[4, 11, 10] = numpy.sum(x526 * x595 * x792)
    result[4, 11, 11] = numpy.sum(x521 * x599 * x8)
    result[4, 11, 12] = numpy.sum(x518 * x603 * x790)
    result[4, 11, 13] = numpy.sum(x515 * x608 * x803)
    result[4, 11, 14] = numpy.sum(x508 * x611 * x787)
    result[4, 12, 0] = numpy.sum(x470 * x624 * x771)
    result[4, 12, 1] = numpy.sum(x473 * x539 * x626)
    result[4, 12, 2] = numpy.sum(x474 * x539 * x628)
    result[4, 12, 3] = numpy.sum(x476 * x623 * x789)
    result[4, 12, 4] = numpy.sum(x478 * x628 * x789)
    result[4, 12, 5] = numpy.sum(x469 * x630 * x789)
    result[4, 12, 6] = numpy.sum(x11 * x484 * x626)
    result[4, 12, 7] = numpy.sum(x11 * x246 * x486 * x628)
    result[4, 12, 8] = numpy.sum(x11 * x478 * x631)
    result[4, 12, 9] = numpy.sum(x11 * x474 * x637)
    result[4, 12, 10] = numpy.sum(x492 * x624 * x804)
    result[4, 12, 11] = numpy.sum(x484 * x628 * x790)
    result[4, 12, 12] = numpy.sum(x476 * x631 * x8)
    result[4, 12, 13] = numpy.sum(x473 * x637 * x790)
    result[4, 12, 14] = numpy.sum(x470 * x643 * x791)
    result[4, 13, 0] = numpy.sum(x443 * x649 * x783)
    result[4, 13, 1] = numpy.sum(x445 * x649 * x784)
    result[4, 13, 2] = numpy.sum(x447 * x539 * x653)
    result[4, 13, 3] = numpy.sum(x450 * x649 * x785)
    result[4, 13, 4] = numpy.sum(x451 * x538 * x653)
    result[4, 13, 5] = numpy.sum(x442 * x656 * x785)
    result[4, 13, 6] = numpy.sum(x11 * x456 * x650)
    result[4, 13, 7] = numpy.sum(x11 * x450 * x654)
    result[4, 13, 8] = numpy.sum(x11 * x451 * x656)
    result[4, 13, 9] = numpy.sum(x11 * x447 * x658)
    result[4, 13, 10] = numpy.sum(x460 * x649 * x787)
    result[4, 13, 11] = numpy.sum(x456 * x653 * x803)
    result[4, 13, 12] = numpy.sum(x450 * x656 * x790)
    result[4, 13, 13] = numpy.sum(x446 * x658 * x8)
    result[4, 13, 14] = numpy.sum(x443 * x661 * x792)
    result[4, 14, 0] = numpy.sum(x432 * x668 * x771)
    result[4, 14, 1] = numpy.sum(x407 * x668 * x774)
    result[4, 14, 2] = numpy.sum(x432 * x669 * x774)
    result[4, 14, 3] = numpy.sum(x416 * x670 * x776)
    result[4, 14, 4] = numpy.sum(x418 * x538 * x669)
    result[4, 14, 5] = numpy.sum(x413 * x672 * x777)
    result[4, 14, 6] = numpy.sum(x668 * x781 * x796)
    result[4, 14, 7] = numpy.sum(x416 * x669 * x801)
    result[4, 14, 8] = numpy.sum(x11 * x418 * x672)
    result[4, 14, 9] = numpy.sum(x397 * x565 * x673 * x778)
    result[4, 14, 10] = numpy.sum(x438 * x668 * x782)
    result[4, 14, 11] = numpy.sum(x422 * x669 * x787)
    result[4, 14, 12] = numpy.sum(x417 * x672 * x804)
    result[4, 14, 13] = numpy.sum(x407 * x673 * x792)
    result[4, 14, 14] = numpy.sum(x397 * x675)
    result[5, 0, 0] = numpy.sum(x128 * x677 * x807)
    result[5, 0, 1] = numpy.sum(x159 * x681 * x808)
    result[5, 0, 2] = numpy.sum(x572 * x681 * x811)
    result[5, 0, 3] = numpy.sum(x133 * x689 * x806)
    result[5, 0, 4] = numpy.sum(x159 * x689 * x812)
    result[5, 0, 5] = numpy.sum(x392 * x795 * x815)
    result[5, 0, 6] = numpy.sum(x154 * x694 * x807)
    result[5, 0, 7] = numpy.sum(x157 * x694 * x811)
    result[5, 0, 8] = numpy.sum(x580 * x694 * x815)
    result[5, 0, 9] = numpy.sum(x585 * x694 * x816)
    result[5, 0, 10] = numpy.sum(x166 * x695 * x807)
    result[5, 0, 11] = numpy.sum(x154 * x695 * x811)
    result[5, 0, 12] = numpy.sum(x133 * x695 * x815)
    result[5, 0, 13] = numpy.sum(x388 * x697 * x816)
    result[5, 0, 14] = numpy.sum(x164 * x695 * x817)
    result[5, 1, 0] = numpy.sum(x177 * x401 * x808)
    result[5, 1, 1] = numpy.sum(x181 * x58 * x806)
    result[5, 1, 2] = numpy.sum(x385 * x58 * x810)
    result[5, 1, 3] = numpy.sum(x191 * x410 * x806)
    result[5, 1, 4] = numpy.sum(x193 * x410 * x810)
    result[5, 1, 5] = numpy.sum(x177 * x720 * x815)
    result[5, 1, 6] = numpy.sum(x198 * x427 * x818)
    result[5, 1, 7] = numpy.sum(x188 * x713 * x810)
    result[5, 1, 8] = numpy.sum(x193 * x427 * x815)
    result[5, 1, 9] = numpy.sum(x385 * x427 * x816)
    result[5, 1, 10] = numpy.sum(x204 * x718 * x807)
    result[5, 1, 11] = numpy.sum(x198 * x719 * x810)
    result[5, 1, 12] = numpy.sum(x191 * x435 * x815)
    result[5, 1, 13] = numpy.sum(x181 * x435 * x816)
    result[5, 1, 14] = numpy.sum(x592 * x718 * x817)
    result[5, 2, 0] = numpy.sum(x401 * x572 * x821)
    result[5, 2, 1] = numpy.sum(x159 * x58 * x822)
    result[5, 2, 2] = numpy.sum(x58 * x609 * x824)
    result[5, 2, 3] = numpy.sum(x218 * x410 * x820)
    result[5, 2, 4] = numpy.sum(x410 * x824 * x825)
    result[5, 2, 5] = numpy.sum(x282 * x410 * x828)
    result[5, 2, 6] = numpy.sum(x224 * x427 * x820)
    result[5, 2, 7] = numpy.sum(x131 * x713 * x824)
    result[5, 2, 8] = numpy.sum(x159 * x713 * x828)
    result[5, 2, 9] = numpy.sum(x427 * x609 * x832)
    result[5, 2, 10] = numpy.sum(x230 * x435 * x821)
    result[5, 2, 11] = numpy.sum(x224 * x435 * x824)
    result[5, 2, 12] = numpy.sum(x218 * x435 * x828)
    result[5, 2, 13] = numpy.sum(x159 * x719 * x832)
    result[5, 2, 14] = numpy.sum(x164 * x718 * x835)
    result[5, 3, 0] = numpy.sum(x237 * x724 * x806)
    result[5, 3, 1] = numpy.sum(x239 * x56 * x836)
    result[5, 3, 2] = numpy.sum(x241 * x56 * x810)
    result[5, 3, 3] = numpy.sum(x243 * x730 * x806)
    result[5, 3, 4] = numpy.sum(x247 * x730 * x810)
    result[5, 3, 5] = numpy.sum(x235 * x730 * x815)
    result[5, 3, 6] = numpy.sum(x145 * x252 * x836)
    result[5, 3, 7] = numpy.sum(x145 * x326 * x810)
    result[5, 3, 8] = numpy.sum(x145 * x248 * x815)
    result[5, 3, 9] = numpy.sum(x145 * x241 * x816)
    result[5, 3, 10] = numpy.sum(x258 * x742 * x806)
    result[5, 3, 11] = numpy.sum(x252 * x743 * x810)
    result[5, 3, 12] = numpy.sum(x245 * x433 * x815)
    result[5, 3, 13] = numpy.sum(x239 * x743 * x816)
    result[5, 3, 14] = numpy.sum(x237 * x433 * x817)
    result[5, 4, 0] = numpy.sum(x177 * x724 * x837)
    result[5, 4, 1] = numpy.sum(x193 * x56 * x820)
    result[5, 4, 2] = numpy.sum(x56 * x618 * x824)
    result[5, 4, 3] = numpy.sum(x262 * x730 * x820)
    result[5, 4, 4] = numpy.sum(x264 * x46 * x824)
    result[5, 4, 5] = numpy.sum(x177 * x731 * x828)
    result[5, 4, 6] = numpy.sum(x145 * x268 * x820)
    result[5, 4, 7] = numpy.sum(x145 * x188 * x838)
    result[5, 4, 8] = numpy.sum(x145 * x264 * x828)
    result[5, 4, 9] = numpy.sum(x145 * x618 * x832)
    result[5, 4, 10] = numpy.sum(x204 * x433 * x837)
    result[5, 4, 11] = numpy.sum(x268 * x433 * x824)
    result[5, 4, 12] = numpy.sum(x263 * x433 * x828)
    result[5, 4, 13] = numpy.sum(x193 * x433 * x832)
    result[5, 4, 14] = numpy.sum(x433 * x620 * x835)
    result[5, 5, 0] = numpy.sum(x625 * x724 * x842)
    result[5, 5, 1] = numpy.sum(x159 * x56 * x843)
    result[5, 5, 2] = numpy.sum(x282 * x56 * x845)
    result[5, 5, 3] = numpy.sum(x283 * x46 * x841)
    result[5, 5, 4] = numpy.sum(x159 * x731 * x845)
    result[5, 5, 5] = numpy.sum(x128 * x730 * x847)
    result[5, 5, 6] = numpy.sum(x145 * x638 * x841)
    result[5, 5, 7] = numpy.sum(x283 * x735 * x845)
    result[5, 5, 8] = numpy.sum(x159 * x736 * x847)
    result[5, 5, 9] = numpy.sum(x145 * x282 * x850)
    result[5, 5, 10] = numpy.sum(x293 * x433 * x842)
    result[5, 5, 11] = numpy.sum(x433 * x638 * x845)
    result[5, 5, 12] = numpy.sum(x283 * x433 * x847)
    result[5, 5, 13] = numpy.sum(x159 * x743 * x850)
    result[5, 5, 14] = numpy.sum(x128 * x742 * x854)
    result[5, 6, 0] = numpy.sum(x302 * x747 * x807)
    result[5, 6, 1] = numpy.sum(x305 * x510 * x818)
    result[5, 6, 2] = numpy.sum(x307 * x510 * x810)
    result[5, 6, 3] = numpy.sum(x21 * x309 * x836)
    result[5, 6, 4] = numpy.sum(x21 * x311 * x810)
    result[5, 6, 5] = numpy.sum(x301 * x754 * x815)
    result[5, 6, 6] = numpy.sum(x313 * x758 * x806)
    result[5, 6, 7] = numpy.sum(x309 * x759 * x810)
    result[5, 6, 8] = numpy.sum(x310 * x758 * x815)
    result[5, 6, 9] = numpy.sum(x19 * x307 * x816)
    result[5, 6, 10] = numpy.sum(x16 * x318 * x808)
    result[5, 6, 11] = numpy.sum(x313 * x770 * x810)
    result[5, 6, 12] = numpy.sum(x16 * x375 * x815)
    result[5, 6, 13] = numpy.sum(x305 * x770 * x816)
    result[5, 6, 14] = numpy.sum(x302 * x764 * x817)
    result[5, 7, 0] = numpy.sum(x235 * x747 * x837)
    result[5, 7, 1] = numpy.sum(x239 * x765 * x820)
    result[5, 7, 2] = numpy.sum(x325 * x751 * x824)
    result[5, 7, 3] = numpy.sum(x21 * x326 * x820)
    result[5, 7, 4] = numpy.sum(x21 * x239 * x838)
    result[5, 7, 5] = numpy.sum(x235 * x767 * x828)
    result[5, 7, 6] = numpy.sum(x252 * x759 * x820)
    result[5, 7, 7] = numpy.sum(x243 * x768 * x824)
    result[5, 7, 8] = numpy.sum(x239 * x768 * x828)
    result[5, 7, 9] = numpy.sum(x325 * x758 * x832)
    result[5, 7, 10] = numpy.sum(x16 * x258 * x837)
    result[5, 7, 11] = numpy.sum(x252 * x769 * x824)
    result[5, 7, 12] = numpy.sum(x16 * x326 * x828)
    result[5, 7, 13] = numpy.sum(x239 * x769 * x832)
    result[5, 7, 14] = numpy.sum(x235 * x799 * x835)
    result[5, 8, 0] = numpy.sum(x620 * x747 * x841)
    result[5, 8, 1] = numpy.sum(x193 * x510 * x841)
    result[5, 8, 2] = numpy.sum(x177 * x765 * x845)
    result[5, 8, 3] = numpy.sum(x21 * x263 * x841)
    result[5, 8, 4] = numpy.sum(x21 * x264 * x845)
    result[5, 8, 5] = numpy.sum(x177 * x767 * x847)
    result[5, 8, 6] = numpy.sum(x267 * x758 * x841)
    result[5, 8, 7] = numpy.sum(x188 * x768 * x845)
    result[5, 8, 8] = numpy.sum(x19 * x264 * x847)
    result[5, 8, 9] = numpy.sum(x177 * x759 * x850)
    result[5, 8, 10] = numpy.sum(x204 * x799 * x841)
    result[5, 8, 11] = numpy.sum(x16 * x268 * x845)
    result[5, 8, 12] = numpy.sum(x16 * x263 * x847)
    result[5, 8, 13] = numpy.sum(x16 * x193 * x850)
    result[5, 8, 14] = numpy.sum(x134 * x175 * x800 * x854)
    result[5, 9, 0] = numpy.sum(x585 * x747 * x855)
    result[5, 9, 1] = numpy.sum(x159 * x751 * x855)
    result[5, 9, 2] = numpy.sum(x128 * x751 * x858)
    result[5, 9, 3] = numpy.sum(x21 * x218 * x855)
    result[5, 9, 4] = numpy.sum(x21 * x825 * x858)
    result[5, 9, 5] = numpy.sum(x128 * x754 * x860)
    result[5, 9, 6] = numpy.sum(x19 * x224 * x855)
    result[5, 9, 7] = numpy.sum(x131 * x759 * x858)
    result[5, 9, 8] = numpy.sum(x159 * x759 * x860)
    result[5, 9, 9] = numpy.sum(x128 * x758 * x862)
    result[5, 9, 10] = numpy.sum(x230 * x764 * x855)
    result[5, 9, 11] = numpy.sum(x16 * x224 * x858)
    result[5, 9, 12] = numpy.sum(x16 * x218 * x860)
    result[5, 9, 13] = numpy.sum(x111 * x15 * x863)
    result[5, 9, 14] = numpy.sum(x800 * x866)
    result[5, 10, 0] = numpy.sum(x352 * x771 * x807)
    result[5, 10, 1] = numpy.sum(x358 * x774 * x807)
    result[5, 10, 2] = numpy.sum(x360 * x539 * x811)
    result[5, 10, 3] = numpy.sum(x363 * x777 * x806)
    result[5, 10, 4] = numpy.sum(x365 * x538 * x811)
    result[5, 10, 5] = numpy.sum(x352 * x777 * x815)
    result[5, 10, 6] = numpy.sum(x11 * x366 * x808)
    result[5, 10, 7] = numpy.sum(x11 * x363 * x812)
    result[5, 10, 8] = numpy.sum(x365 * x781 * x815)
    result[5, 10, 9] = numpy.sum(x360 * x781 * x816)
    result[5, 10, 10] = numpy.sum(x370 * x8 * x807)
    result[5, 10, 11] = numpy.sum(x366 * x792 * x811)
    result[5, 10, 12] = numpy.sum(x364 * x804 * x815)
    result[5, 10, 13] = numpy.sum(x358 * x787 * x816)
    result[5, 10, 14] = numpy.sum(x352 * x782 * x817)
    result[5, 11, 0] = numpy.sum(x302 * x771 * x821)
    result[5, 11, 1] = numpy.sum(x305 * x784 * x820)
    result[5, 11, 2] = numpy.sum(x307 * x539 * x824)
    result[5, 11, 3] = numpy.sum(x309 * x785 * x820)
    result[5, 11, 4] = numpy.sum(x311 * x538 * x824)
    result[5, 11, 5] = numpy.sum(x301 * x785 * x828)
    result[5, 11, 6] = numpy.sum(x11 * x313 * x822)
    result[5, 11, 7] = numpy.sum(x309 * x824 * x867)
    result[5, 11, 8] = numpy.sum(x11 * x311 * x828)
    result[5, 11, 9] = numpy.sum(x11 * x307 * x832)
    result[5, 11, 10] = numpy.sum(x318 * x792 * x821)
    result[5, 11, 11] = numpy.sum(x313 * x803 * x824)
    result[5, 11, 12] = numpy.sum(x309 * x790 * x828)
    result[5, 11, 13] = numpy.sum(x305 * x803 * x832)
    result[5, 11, 14] = numpy.sum(x302 * x782 * x835)
    result[5, 12, 0] = numpy.sum(x237 * x771 * x841)
    result[5, 12, 1] = numpy.sum(x239 * x788 * x841)
    result[5, 12, 2] = numpy.sum(x241 * x539 * x845)
    result[5, 12, 3] = numpy.sum(x243 * x789 * x841)
    result[5, 12, 4] = numpy.sum(x247 * x789 * x845)
    result[5, 12, 5] = numpy.sum(x235 * x789 * x847)
    result[5, 12, 6] = numpy.sum(x11 * x252 * x843)
    result[5, 12, 7] = numpy.sum(x11 * x326 * x845)
    result[5, 12, 8] = numpy.sum(x11 * x248 * x847)
    result[5, 12, 9] = numpy.sum(x11 * x241 * x850)
    result[5, 12, 10] = numpy.sum(x258 * x804 * x842)
    result[5, 12, 11] = numpy.sum(x252 * x790 * x845)
    result[5, 12, 12] = numpy.sum(x245 * x8 * x847)
    result[5, 12, 13] = numpy.sum(x239 * x790 * x850)
    result[5, 12, 14] = numpy.sum(x237 * x8 * x854)
    result[5, 13, 0] = numpy.sum(x592 * x783 * x855)
    result[5, 13, 1] = numpy.sum(x181 * x539 * x855)
    result[5, 13, 2] = numpy.sum(x177 * x784 * x858)
    result[5, 13, 3] = numpy.sum(x188 * x785 * x855)
    result[5, 13, 4] = numpy.sum(x193 * x538 * x858)
    result[5, 13, 5] = numpy.sum(x177 * x785 * x860)
    result[5, 13, 6] = numpy.sum(x198 * x802 * x855)
    result[5, 13, 7] = numpy.sum(x188 * x858 * x867)
    result[5, 13, 8] = numpy.sum(x11 * x193 * x860)
    result[5, 13, 9] = numpy.sum(x175 * x786 * x863)
    result[5, 13, 10] = numpy.sum(x204 * x787 * x855)
    result[5, 13, 11] = numpy.sum(x198 * x803 * x858)
    result[5, 13, 12] = numpy.sum(x191 * x8 * x860)
    result[5, 13, 13] = numpy.sum(x181 * x8 * x862)
    result[5, 13, 14] = numpy.sum(x175 * x565 * x866)
    result[5, 14, 0] = numpy.sum(x164 * x771 * x868)
    result[5, 14, 1] = numpy.sum(x388 * x774 * x868)
    result[5, 14, 2] = numpy.sum(x164 * x774 * x869)
    result[5, 14, 3] = numpy.sum(x132 * x793 * x868)
    result[5, 14, 4] = numpy.sum(x538 * x580 * x869)
    result[5, 14, 5] = numpy.sum(x128 * x777 * x870)
    result[5, 14, 6] = numpy.sum(x154 * x781 * x868)
    result[5, 14, 7] = numpy.sum(x157 * x781 * x869)
    result[5, 14, 8] = numpy.sum(x111 * x565 * x780 * x870)
    result[5, 14, 9] = numpy.sum(x778 * x871)
    result[5, 14, 10] = numpy.sum(x166 * x782 * x868)
    result[5, 14, 11] = numpy.sum(x154 * x782 * x869)
    result[5, 14, 12] = numpy.sum(x133 * x8 * x870)
    result[5, 14, 13] = numpy.sum(x674 * x871)
    result[5, 14, 14] = numpy.sum(
        x565
        * (
            x0 * (2.0 * x659 + 2.0 * x660 + 7.0 * x852 + 3.0 * x853 + 4.0 * x861)
            + x211 * x865
        )
    )
    return result


quadrupole3d = {
    (0, 0): quadrupole3d_00,
    (0, 1): quadrupole3d_01,
    (0, 2): quadrupole3d_02,
    (0, 3): quadrupole3d_03,
    (0, 4): quadrupole3d_04,
    (1, 0): quadrupole3d_10,
    (1, 1): quadrupole3d_11,
    (1, 2): quadrupole3d_12,
    (1, 3): quadrupole3d_13,
    (1, 4): quadrupole3d_14,
    (2, 0): quadrupole3d_20,
    (2, 1): quadrupole3d_21,
    (2, 2): quadrupole3d_22,
    (2, 3): quadrupole3d_23,
    (2, 4): quadrupole3d_24,
    (3, 0): quadrupole3d_30,
    (3, 1): quadrupole3d_31,
    (3, 2): quadrupole3d_32,
    (3, 3): quadrupole3d_33,
    (3, 4): quadrupole3d_34,
    (4, 0): quadrupole3d_40,
    (4, 1): quadrupole3d_41,
    (4, 2): quadrupole3d_42,
    (4, 3): quadrupole3d_43,
    (4, 4): quadrupole3d_44,
}
